;****I* idl/idl.angles
; NAME
; angles
; 
; PURPOSE
; Calculate view zenith and azimuth angles for solar and satellite
; views for certain locations on the Earth.
; 
; USAGE
; See procedure definition below.
; 
; INPUT
; lat - latitude (deg)
; lon - longitude (deg) (same dimension as lat)
; time - time of day (result from ftime ... decimal hour)
; jday - day of year
; 
; OUTPUT
; sza - Solar zenith angle (deg)
; az_not - Solar azimuth angle (deg_east_of_north)
; vza - satellite view zenith angle (deg)
; azimuth - Satellite view azimuth angle (deg_east_of_north)
; 
; KEYWORDS
;   GOESE - GOES-East position (default) (-75 deg East)
;   GOESW - GOES-West position          (-135 deg East)
;   INGLON - input Geo longitude (deg)  (degree_east)
;   SUNONLY - only perform the solar angle calculations
;   SATONLY - only perform the satellite angle calculations
;   
; EXAMPLE
; USES
; Nothing
; 
; USED BY
; * b1u_2_grisat.pro
; * b1u_coastline_navigation.pro
; * b1u_imc.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Tue Jun 26 09:51:47 EDT 2007
; 
; HISTORY
; Jun2010 - Verified header info.
; 
; SOURCE

pro angles,lat,lon,time,jday,sza,az_not,vza,azimuth,$
           GOESE=GOESE, $
           GOESW=GOESW, $
           inglon=inglon,$
           sunonly=sunonly, $
           satonly=satonly

if (keyword_set(GOESW)) then glon = -135. else glon = -75.
if (keyword_set(inglon)) then glon = inglon
pi = !pi
rad = !pi / 180.
lat1=lat*rad
lon1=lon*rad

if keyword_set(satonly) then goto,satellite

;print,glon,jday,time
; SZA
h_ang = -1. * !pi * (time - 12.) / 12.
declin = -23.45 * COS(FLOAT(jday)*2.*PI/365. + 10.*2.*PI/365.)
mu_not = sin(lat*rad)*sin(declin*rad) + $
  cos(lat*rad)*cos(declin*rad)*cos(h_ang-lon*rad)
sza = acos(mu_not) * 180. / !pi


lat2=declin*rad
lon2 = h_ang

;az_not=atan2(sin(lon1-lon2)*cos(lat2), $
;             cos(lat1)*sin(lat2)-sin(lat1)*cos(lat2)*cos(lon1-lon2)) mod 2*pi
;az_not = az_not/rad
;AZI_NOT
beta2 = acos(cos((lat-declin)*rad)*cos(H_ang-lon*rad))
az_not= sin((H_ang-lon*rad))/sin(beta2)
az_not = az_not > (-1.) < 1.
az_not = asin(az_not)
az_not = az_not / rad
h = (lon*rad-h_ang) mod (2.*!pi)
ns = lat - declin
rep = where(h lt 0,nrep)
if (nrep gt 0) then h(rep) = h(rep) + 2.*!pi
;if (h lt 0) then h = h + 2.*!pi
az_not = 180. - az_not
;points north of sun
part1 = where(ns gt 0,npart1,complement=part2,ncomp=npart2)
if (part1(0) ge 0) then begin
    rep = where(h(part1) ge (pi/2.) and h(part1) le (3.*pi/2.),nrep)
    if (nrep gt 0) then az_not(part1(rep)) = 180. - az_not(part1(rep))
    rep = where(az_not(part1) le 0.,nrep)
    if (nrep gt 0) then az_not(part1(rep)) = az_not(part1(rep)) + 360.
endif
;points south of sun
if (npart2 gt 0) then begin
  rep = where(h(part2) le (pi/2.) or h(part2) ge (3.*pi/2.),nrep)
  if (nrep gt 0) then az_not(part2(rep)) = 180. - az_not(part2(rep))
endif

;Quick fix, since current range of az_not is (-90,270)
below = where(az_not lt 0.,nb)
if (nb gt 0) then az_not(below) = 360. + az_not(below)

;if (ns gt 0.) then begin
;    if (h ge pi/2. and h le 3.*pi/2.) then az_not = 180. - az_not
;    if (az_not le 0) then az_not = az_not + 360.
;endif else begin
;    if (h le pi/2. or h ge 3.*pi/2.) then az_not = 180. - az_not
;    if (az_not le 0) then az_not = az_not + 360.
;endelse

if (keyword_set(sunonly)) then return

satellite:
lat2=0.
lon2 = glon*rad

;coslat2 = cos(lat2)
;azimuth1=atan2(sin(lon1-lon2)*coslat2, $
;              cos(lat1)*sin(lat2)-sin(lat1)*coslat2*cos(lon1-lon2)) mod 2*pi

;azimuth1=azimuth1/rad
; AZIMUTH
beta = ACOS(COS(LAT*RAD) * COS((GLON - LON)*RAD))
AZIMUTH = SIN((GLON-LON)*RAD) / SIN(BETA)
azimuth = (azimuth > (-1.)) < 1.
azimuth = asin(azimuth)
;azimuth = pi - azimuth
azimuth = azimuth * (180. / !pi)
below = where(lat lt 0. and azimuth lt 0,nb)
if (nb gt 0) then azimuth(below) = 360.+ azimuth(below)
above = where(lat gt 0.,na)
if (na gt 0) then azimuth(above) = 180. - azimuth(above)

; VZA
vza = asin((42164.*sin(beta) / $
              sqrt(1.8084e9 - 5.3725e8*cos(beta))) < 1) / rad

;vza = theta

;print,sza,az_not
;print,vza,azimuth

end

;***
