;****** isccpb1u/isccp_b1u_production
; NAME
;  isccp_b1u_production
;  
; PURPOSE
; To document the production of ISCCP B1U files.
; 
; NOTES
; The following are the steps used in producing ISCCP B1U files.
; 
; USES
; * idl.b1_2_b1u
; 
; USED BY
; ISCCP B1U files are used by many of the processes I run. These include:
; * idl.b1u_2_grisat
; 
; INPUT
; OUTPUT
; RESULT
; EXAMPLE
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Tue Apr  7 11:03:54 EDT 2009
; 
; HISTORY
; 07April2009 - Added this summary documentation header.
; 
;***
;****I* isccpb1u/idl.b1_2_b1u
; NAME
;  b1_2_b1u
;  
; PURPOSE
;    This routine converts the ISCCP B1 files to the B1U format -
;    which is a unfied data format that makes it possible to have one
;    read and one navigation routine.
; 
; USAGE
; 
; INPUT
; 
; OUTPUT
; 
; RESULT
;    The ISCCP B1U file is created
;    
; EXAMPLE
; 
; USES
; * b1qc.pro
; * b1read.pro
; * b1uread.pro
; * b1u_coastline_navigation.pro
; * b1u_get_center.pro
; 
; USED BY
; * isccp_b1u_production
; 
; NOTES
;    The B1U file is described by the B1U documentation. This routine
;    reads the B1 file using b1read, then modifies the structures
;    returned and outputs the data in a uniform format (which is
;    nearly the same for all B1 files). The data is read using
;    b1u_read.pro and navigated using b1u_nav.pro
;
;    Test on MET - Processed all MET files in ISCCP-test. Files
;                  created were similar in size (B1(size) = 299MB
;                  whereas B1U(size) = 301MB).
;    Test on GMS - Processed all GMS files in ISCCP-test.
;                  File sizes were slightly smaller.
;    Test on GOES9 Processed all GOES9 files in ISCCP-test.
;                  File sizes were slightly smaller.
;    Test on GOES7 Tested GOES-7 CSU
;                  I/O is fine, but navigation is off a bit.
;
; TODO

; BUGS
; 
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; 
; HISTORY
;    $Id: b1_2_b1u.pro 22799 2010-09-16 13:10:55Z kknapp $
;    03/30/2004  Created by Ken Knapp NCDC/RSAD
;    04/22/2004  Tested successfully on UWS data.
;    04/23/2004  Tested successfully on MET data.
;    04/26/2004  Tested successfully on GMS/GOES9 data.
;    04/27/2004  Tested successfully on CSU/GOES-8/9/10/12 data.
;    04/27/2004  Tested successfully on AES/GOES-8 data.
;    05/03/2004  Tested successfully on CSU/GOES-7 data.
;    05/03/2004  Tested successfully on CSU/GOES-6 data.
; 
; SOURCE


pro b1_2_b1u,file=file, $
             replace=replace, $
             b1ufile=b1ufile, $
             calfile=calfile, $
             skipcal=skipcal, $
             compress=compress, $
             debug=debug

;====================================================
;
; SET UP PARAMETERS
;
; IREQ defines if only required blocks are output
; IREQ = 1, all blocks are output
; IREQ = 0, only required blocks are output
IREQ = 1
;
; b1udir = location for the new b1u files
b1udir = getenv('ISCCPDIR')+'b1u/'

iskipcal=0
if keyword_set(skipcal) then iskipcal=1

ireplace = 0                      ;do not overwrite existing B1U files
if keyword_set(replace) then ireplace = 1 ;overwrite existing B1U files

icomp = 0
if keyword_set(compress) then icomp=1

;
;
;=====================================================

;REDO=1 means overwrite existing files
;REDO=0 means check the file and write if one isn't found
REDO = 0
if (ireplace) then redo = 1

print,'FILE:',file

; Create the B1U filename
    f = stripname(file)
    f = strsplit(f,'.',/extr)
    f(1) = 'B1U'
    f(2) = 'a'
    b1ufile = b1udir + f(4) +'/'+strjoin(f,'.')
    n = n_elements(f) 
    if (f(n-1) eq 'gz') then icomp=1
    ;check whether a compressed or uncompressed B1U file is present
    if (icomp) then begin
        ;checks for uncompressed file
        b1ucomp=b1udir + f(4) +'/'+strjoin(f(0:n-2),'.')
    endif else begin
        ;checks for a compressed file
        b1ucomp = b1ufile+'.gz'
    endelse
    b1uft = file_test(b1ufile)
    b1uct = file_test(b1ucomp)
    if ((b1uft or b1uct) and redo eq 0) then begin
        if (b1uft) then print,stripname(b1ufile),' exists' else $
          print,stripname(b1ucomp),' exists'
        u=1
    endif else begin
    
;    print,'-----------------------------'
    linhdr = 0
    
    fil = strsplit(stripname(file),'.',/extr)

    if (fil(1) eq 'B1') then begin
        b1read,file=file,image=image,area=area,nav=nav,rec=rec,geostr=geostr, $
               orbstr=orbstr,imgstr=imgstr,attstr=attstr,timstr=timstr, $
               chnstr=chnstr,tgrstr=tgrstr,linhdr=linhdr,calstr=calstr, $
               fhdr=fhdr,b1u=b1u,calfile=calfile,skipcal=iskipcal

        if n_elements(image) eq 1 then begin
            if (image(0) eq (-1) ) then begin
                print,'Cannot Convert B1 --> B1U: Can''t read B1 file'
                print,file
                goto,endd
            endif
        endif

        if n_elements(fhdr) eq 0 then fhdr = b1u.fhdr
    endif else if (fil(1) eq 'B1U') then begin
        b1uread,file=file,b1u=b1u
        fhdr=b1u.fhdr
    endif else begin
        message,'Unknown file type:'+fil(1)+' for file:'+stripname(file)
    endelse

;   Create the B1U filename
    f = stripname(file)
    f = strsplit(f,'.',/extr)
    f(1) = 'B1U'
    f(2) = 'a'
    b1ufile = b1udir + f(4)+'/'+strjoin(f,'.')
    nf=n_elements(f)
    if (icomp and f(nf-1) ne 'gz') then begin
        b1ufile=b1ufile+'.gz'
    endif

    b1utags=tag_names(b1u)
    
    nx = b1u.img(4)
    ny = b1u.img(3)
    nchan=b1u.img(5)
    chan=b1u.sat(2:2+nchan-1)
    irwin=where(strtrim(chan,2) eq 'IRWIN',nirwin)
    
;   get QC/QA information
    ;use the QCQA files where possible:
    if (file_test(remove_gz(b1ufile)+'.QCQA',/read)) then begin
        QC=lonarr(nchan,ny)
        a0=0l & a1 = 0l & a2 = 0l
        openr,1,remove_gz(b1ufile)+'.QCQA'
        readu,1,a0,a1,a2
        readu,1,qc
        close,1

        qc_line=qc
        
    endif else begin
        b1qc,'',qc_line,qc_pix,b1u=b1u,/nowrite,debug=debug
    endelse


    
;if keyword_set(debug) then stop

;   get center information
;   currently, only run this if file is not GVAR data

    if (nirwin eq 1 and ny gt 300 and b1u.img(13) eq 0) then begin
        irimg = reform(b1u.image(irwin,*,*))
        b1u_get_center,irimg,b1u.nav,center,b1u=b1u,meta=meta ;,/debug
if n_elements(meta) eq 0 then stop
        b1u.img(15) = meta(5)
        b1u.nav(151) = long(center.cen_lin_corr)
        b1u.nav(152) = long(center.cline_eq(0),0,1)
        b1u.nav(153) = long(center.cline_eq(1),0,1)
        b1u.nav(154) = long(center.cline_eq(2),0,1)
        b1u.nav(155) = long(center.cline_eq(3),0,1)

    endif
;if keyword_set(debug) then stop
;   META contains information on the space values used in the center
;   algorithm. This should be retained in b1u data to be used in
;   plotting the imagery. ... cince it tells what the IR space values
;   are, it should define whether imageery should be swapped or not.
;
;
;   get coastline navigation adjustment
   b1u_coastline_navigation,b1u,center,iroffset,visoffset


   b1u.img(16:17) =  iroffset
   b1u.img(18:19) = visoffset

    if (n_elements(image) gt 0) then begin
        if (image(0) eq (-1)) then begin
            goto,nextone
        endif
    endif

; Assign parameters to B1U structures
    b1u.rev(1) = '$Id: b1_2_b1u.pro 22799 2010-09-16 13:10:55Z kknapp $'
    b1u.rev(4) = systime()

; Prepare the headers
    MISSING = -9999l
    KEY = long(byte('B1UD'),0,1) 
    s = size(b1u.pre)
    NBLP = s(2)
    DATLOC = MISSING
    NSCAN = b1u.img(3)
    NELEM = b1u.img(4)
    NCHAN = b1u.img(5)
    NBYTE = b1u.img(6)
    NBIN  = long(b1u.img(14)) ;number of bins
    NHEAD = 0
    FILinf = [KEY, NBLP, DATLOC, NSCAN, NELEM, NCHAN, NHEAD]

; THE REVISION BLOCK IS REQUIRED
;intially, Revision block will start at byte 256 ... if for some
;reason this is too small, it will be changed later.
;prepare Revision block
    FILinf(6)++
    FILinf = [FILinf, 1l]       ;BTYPE=1l=REVinf block
    FILinf = [FILinf,256l]      ;BSTRT=128l=starting point
    FILinf = [FILinf,432l]      ;BLEN=432=6*72
    
; THE IMAGE BLOCK IS REQUIRED
    FILinf(6)++
    FILinf = [FILinf, 2l]       ;BTYPE=2l=IMGinf block
    i = n_elements(FILinf)
    FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
    FILinf = [FILinf,24l*4l]
    
; THE SATinf BLOCK is not required
    if (IREQ) then begin
        FILinf(6)++
        FILinf = [FILinf,3l]    ;SATinf
        i = n_elements(FILinf)
        FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
        FILinf = [FILinf,32l+nchan*26l]
    endif
    
; The NAVIGATION block is not required
    if (IREQ) then begin
        btype = 4l
        bsize = 800l
        GOESNEXT = 0
        if (n_elements(b1u.nav) eq 1) then begin
            ;If NAV is GVAR, then use different size
            GOESNEXT = 1
            btype = 9l
            bsize = 2816l
        endif
        FILinf(6)++
        FILinf = [FILinf,btype]    ;NAVinf
        i = n_elements(FILinf)
        FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
        FILinf = [FILinf,bsize]
    endif
    
;The CALIBRATION block is not required
    if (IREQ) then begin
        FILinf(6)++
        FILinf = [FILinf,5l]    ;CALinf
        i = n_elements(FILinf)
        FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
        FILinf = [FILinf,4L * (NCHAN*2*nbin + 1L)]
    endif
    
;The TEST GRID block is not required
    if (total(strpos(b1utags,'TGR') ge 0) gt 0) then begin
    if (IREQ and b1u.tgr(0) ne missing and b1u.tgr(0) ne 0) then begin
        FILinf(6)++
        FILinf = [FILinf,6l]    ;CALinf
        i = n_elements(FILinf)
        FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
        nrows = b1u.tgr(0)
        ncols = b1u.tgr(1)
        nbytes = (6+2*nrows*ncols)*4l
        FILinf = [FILinf,nbytes]    
    endif
    endif
    
;The OB1 block is not required
    if (IREQ) then begin
        FILinf(6)++
        FILinf = [FILinf,7l]    ;CALinf
        i = n_elements(FILinf)
        FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
        FILinf = [FILinf,n_elements(fhdr)]    
    endif
    
; THE LINE PREFIX & IMAGE BLOCK IS REQUIRED
    FILinf(6)++
    FILinf = [FILinf,8l]        ;CALinf
    i = n_elements(FILinf)
    FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
    FILinf(2) = FILinf(i-3)+FILinf(i-2)
    FILinf = [FILinf,nchan*nscan*(2*nblp+nelem*nbyte)]

; The QCinf block is not required
    if (IREQ) then begin
        FILinf(6)++
        FILinf = [FILinf,10l]    ;QCinf
        i = n_elements(FILinf)
        FILinf = [FILinf,FILinf(i-3)+FILinf(i-2)]
        FILinf = [FILinf,nchan*nscan*2]    
    endif

   
;
; CHECK TO SEE IF THE FILinf > 256 bytes
;
;
; Output the B1U file headers and data.
    print,'B1U==>',b1ufile
    openw,1,b1ufile,comp=icomp  ;+'.gz',/compress
    writeu,1,FILinf
    for itype=0,FILinf(6)-1 do begin
        type = FILinf(7+itype*3)
        point_lun,-1,here
;        print,'Position=',here,FILinf(7+itype*3+1)
        case type of            ;output the file header blocks
            1: begin            ;1=REVinf
                point_lun,1,FILinf(7+itype*3+1)
                writeu,1,byte(string(b1u.REV,format='(6a72)'))
;                printf,1,b1u.REV,format='(6a72)'
            end
            2: begin            ;2=IMGinf
                writeu,1,b1u.IMG
            end
            3: begin            ;3=SATinf
;                printf,1,b1u.SAT(0),format='(a16)'
                writeu,1,byte(string(b1u.SAT(0:1),format='(2a16)'))
;                printf,1,b1u.SAT(1),format='(a16)'
;                j=2
;                for i=0,nchan-1 do begin
;                    printf,1,b1u.SAT(j++),format='(a6)'
;                endfor
;                for i=0,nchan-1 do begin
;                    printf,1,b1u.SAT(j++),format='(a20)'
;                endfor
                writeu,1,byte(string(b1u.SAT(2:2+nchan-1),format='(20a6)'))
                writeu,1,byte(string(b1u.SAT(2+nchan:2+2*nchan-1),format='(20a20)'))
            end
            4: begin            ;4=NAVinf
                writeu,1,b1u.NAV
            end
            5: begin            ;5=CALinf
                writeu,1,b1u.CAL
            end
            6: begin            ;6=TGRinf
                writeu,1,b1u.TGR
            end
            7: begin            ;7=OB1inf
                writeu,1,fhdr
            end
            8: begin            ;PREinf/Image
                for j=0,nscan-1 do begin
                    for ich=0,nchan-1 do begin
                        writeu,1,b1u.pre(ich,*,j)
                        writeu,1,b1u.image(ich,*,j)
                    endfor
                endfor
            end
            9: begin            ;9=GVARinf
                writeu,1,b1u.nav
            end
            10: begin
                writeu,1,qc_line
            end
        endcase
    endfor
    close,1

    nextone:
endelse

endd:

end

;***
