;****I* idl/idl.ksmooth_internal
; NAME
; ksmooth
; 
; PURPOSE
; To perform some simple weighted averaging using a mask to ignore
; certain points. Binomial filter?
; 
; USAGE
; See idl.b1qc
; 
; INPUT
; data - array
; flag - array with same dimensions as data
; w    - number of times to apply the filter
; 
; OUTPUT
; Smoothed array.
; 
; RESULT
; EXAMPLE
; USES
; * Nothing
; 
; USED BY
; * b1qc.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; 
; HISTORY
; 
; SOURCE

function ksmooth,data,flag,w
;calculate the smooth values without the missing data
; FLAG=0 means ignore the point
;
;filter applies a binomial filter
n = n_elements(data)
ind=lindgen(n)
d = data
f = flag
k = 1
for iw = 1,w do begin
    num = fltarr(n)
    den = fltarr(n)
    for i=0,1 do begin
        num = num + f(ind+i*k) * d(ind+i*k)
        den = den + f(ind+i*k)
    endfor
    k = -1 * k
    d = num / (den>1)
    none=where(den eq 0.,nnone)
    f(*) = 1
    if (nnone gt 0) then begin
        num(none) = -1.
        f(none) = 0.
    endif   
endfor
return,d

end
;***

;****I* idl/idl.b1qc
; NAME:
;    b1qc
;
; PURPOSE:
;    Provide QC analysis of ISCCP B1 file. If QC file exists, the QC
;    information is read from the QC file and returned. Otherwise, QC
;    information is collected, written to file, then returned to the
;    main program.
;
; CATEGORY:
;    ISCCP
;
; INPUTS:
;
;
;
; KEYWORD PARAMETERS:
;    OVERWRITE - When set, do not look for existing QC file, but
;                create a new one.
;    UPDATE    - When set, checks version of the QC file found and
;                compares to current version. If QC file is older
;                version of this routine, then it reruns the QC.
;    QUIET     - When set,suppresses test output
;    VERSION   - Return the version string
;
; OUTPUTS:
;    qc_line- line-by-line QC flags
;
; OPTIONAL OUTPUTS:
;    qc_pix - When implemented, this routine will provide information
;             on a pixel-by-pixel basis.
;
; USES
; * idl.ksmooth (internal ... above)
; 
; * b1read.pro
; * b1uread.pro
; * stripname.pro
;
; USED BY
; * idl.b1_2_b1u
;
; MODIFICATION HISTORY:
;    $Id: b1qc.pro 22785 2010-09-16 12:45:53Z kknapp $
;
; SOURCE

pro b1qc,file,qc_line,qc_pix, $
         overwrite=overwrite,$
         update=update, $
         quiet=quiet, $
         version=version, $
         b1u=b1u, $
         nowrite=nowrite, $
         debug=debug


;the following is a list of files to test the QC algorithm on.
if (n_elements(file) eq 0 and not(keyword_set(b1u))) then begin
    file = '~/data/ISCCP-test/ISCCP.B1.0.GOE-7.1995.10.07.1531.CSU'
    file = '/raid/kknapp/data/isccp/b1u/1990/ISCCP.B1U.a.GOE-7.1990.02.14.2100.AES'
    file = '/raid/kknapp/data/isccp/b1u/1983/ISCCP.B1U.a.GOE-5.1983.07.31.1802.UWS'
;    file = '/raid/kknapp/data/isccp/b1u/1992/ISCCP.B1U.a.GMS-4.1992.08.15.0232.JMA'
;    file = '/raid/kknapp/data/isccp/b1u/1991/ISCCP.B1U.a.MET-4.1991.03.28.1430.ESA'
;    file = '/raid/kknapp/data/isccp/b1u/1988/ISCCP.B1U.a.MET-3.1988.11.16.1930.ESA'
;    file = '/raid/kknapp/data/isccp/b1u/1983/ISCCP.B1U.a.GOE-5.1983.11.16.2101.UWS'
;    file = '/raid/kknapp/data/isccp/b1u/1983/ISCCP.B1U.a.GOE-5.1983.12.27.0000.UWS'
    file = '/home/kknapp/DATA/isccp/b1u/2007/ISCCP.B1U.a.GOE-11.2007.06.01.0000.CSU.gz'
endif

ver = '$Id: b1qc.pro 22785 2010-09-16 12:45:53Z kknapp $'

if keyword_set(b1u) then goto,problemreading

if (n_elements(file) ne 1) then begin
    print,'Usage:'
    print,'b1qc,<FILE>,qc_img,qc_line [,qc_pix, /overwrite, /update]'
    return
endif

if keyword_set(debug) then testch=3
if keyword_set(debug) then testline=500

QCdir = getenv('ISCCPDIR')+'b1qc/'
f = stripname(file)
qcfile = QCdir + f + '.QC'

;fe = file_exists = result of file_test
fe = file_test(QCfile,/read)

if (not(fe)) then begin
    parts = strsplit(f,'.',/extr)
    QCdir = getenv('ISCCPDIR')+'b1u/'+parts(4)+'/'
    qcfile = QCdir + f + '.QCQA'
    fe = file_test(QCfile,/read)
endif


if (fe and not(keyword_set(overwrite))) then begin
    ;input the QC information
    on_ioerror,problemreading
    b1file=bytarr(100)
    qcver = bytarr(100)
    nch = 0l
    ny  = 0l
    openr,unit,QCfile,/get_lun
;    readu,unit,b1file
;    readu,unit,qcver
    qcver=0l
    readu,unit,qcver
    readu,unit,nch,ny
    QC_line = lonarr(nch,ny)
    readu,unit,QC_line
    close,unit
    free_lun,unit
    b1file = strtrim(b1file,2)
;    if (stripname(file) ne b1file) then begin
;        print,'ERROR: Filename do not match'
;        print,'Processed:',b1file
;        print,'Current:  ',stripname(file)
;        stop
;    endif
    if (keyword_set(update) and $
        strtrim(qcver) ne ver) then begin
        if (not(keyword_set(quiet))) then $
          print,'REPROCESSING DUE TO OLD VERSION'
        goto,problemreading
    endif
;    if (strtrim(qcver) ne ver and not(keyword_set(quiet))) then begin
;        print,'WARNING: QC file from old version of b1qc.pro'
;        print,'Old:    ',strtrim(qcver)
;        print,'Current:',ver
;    endif
    on_ioerror,null
endif else begin
    ;if no file (or problem reading it) then create a new one.
problemreading:
    ;read the image
    if (not(keyword_set(b1u))) then begin
        f = strsplit(f,'.',/extr)
        if (f(1) eq 'B1U') then begin
            b1uread,file=file,b1u=b1u
            image = b1u.image
            nch = b1u.img(5)
            ny = b1u.img(3)
            nx = b1u.img(4)
            nchan = nch
        endif else begin
            b1read,file=file,image=image,nav=nav,area=area,$
                   imgstr=imgstr,geostr=geostr,attstr=attstr,$
                   orbstr=orbstr,chnstr=chnstr,LINhdr=LINhdr, $
                   CALstr=CALstr,b1u=b1u
            nch = chnstr.nchan
            nx = imgstr.npix
            ny = imgstr.nlin
            nchan = chnstr.nchan
        endelse
    ;set up parameters
        n=n_elements(f)
        spc = f(n-1)
        sat = f(n-6)
    endif else begin
        image = b1u.image
        nch = b1u.img(5)
        ny = b1u.img(3)
        nx = b1u.img(4)
        nchan = nch
        file=b1u.rev(0)
    endelse
    ; Calculate statistics on a per scan line basis
    lnmx = fltarr(nchan,ny)
    lnmn = lnmx
    lnavg= lnmx
    lnstd= lnmx
    lnval= lnmx
    ;calculate stats of the entire line
    for ich=0,nchan-1 do begin
        im = reform(image(ich,*,*))
        lnmx(ich,*) = max(im,dim=1)
        lnmn(ich,*) = min(im,dim=1)
        sumx = total(im,1)
        sumx2= total(long(im)^2,1)
        lnavg(ich,*) = sumx / float(nx)
        lnstd(ich,*) = sqrt((nx*sumx2-sumx^2)/nx/(nx-1))
        lnval(ich,*) = total(im ne 0 and im ne 255,1)
    endfor
    ind = indgen(ny)
    dst = fltarr(nchan,ny)

;f90=fltarr(1800,2)
;openr,1,'~/for/tests/fort.13',/f77
;readu,1,f90
;close,1
;stop

    maxdn = 254.1
    if (b1u.img(14) gt 257) then maxdn = 1023

    if keyword_set(debug) then begin
        print,'Maxdn === ',maxdn
        print,'SUM=== Sum of each channel'
        print,long(total(total(image,2),2))

        window,4,xs=900,ys=900

        tvscl,congrid(image(testch,*,*),1,450,900)


        nny = findgen(ny)
        !p.multi=[1,2,1]
        plot,lnmx(testch,*),nny,ymarg=[0,0],yrange=[0,ny-1],ys=1,yticklen=0.01
        oplot,lnmn(testch,*),nny
        oplot,lnavg(testch,*),nny

    endif



    QC = long(lnavg)*0l
    ind = indgen(ny)
;========================================================
;                        QC TESTS
;========================================================
; TEST 1: Tests for dropped scans
;--------------------------------
    tval = 0
    ;MISS[nch,ny] is the flag for missing scan lines
    miss = lnstd lt 0.05 or $
           lnavg lt 0.05 or $
           lnavg ge maxdn
    qc1 = miss
    qc = qc + qc1*2^(tval)
    if keyword_set(debug) then begin
        print,'DEBUG:: Test 1:: Lines passing dropped scans'
        print,fix(total(miss eq 0,2))
        print,'lnavg,lnstd',lnavg(testch,testline), $
                            lnstd(testch,testline)
        print,'TEST ==>'
        print,'avg',lnavg(testch,testline:testline+1)
        print,'std',lnstd(testch,testline:testline+1)
        print,'mis',miss(testch,testline:testline+1)
        print,'<== TEST'
     endif
;========================================================
; TEST 2: Test for faint and/or noisy lines
;--------------------------------
; This test seems to work for (currently) all of the test
; files above. 
;
; Method: This routine starts out by skipping lines which
;         are MISSed or whose step in mean or std devation
;         are too large. Beyond that, it calculates an
;         optimal line using binomial filtering and skipping
;         missing lines. Then resets the flag for bad scans
;         using a simple difference between the mean and the
;         filtered mean.

    ;Calculate the steps in line-mean and line-std for the
    ;initial filter.
    stm = fltarr(nchan,ny)
    sts = stm
    for ich=0,nchan-1 do begin
        stm(ich,*) = abs(lnavg(ich,ind+1) - lnavg(ich,ind))
        sts(ich,*) = abs(lnstd(ich,ind+1) - lnstd(ich,ind))
    endfor
    ;the intial flag is MISS lines as well as large steps in 
    ;std dev or mean
    flag = (1-miss) and sts lt 2 and stm lt 4

    if keyword_set(debug) then begin
          print,' FLAG VALUES'
          print,miss(testch,testline),sts(testch,testline:testline+1),stm(testch,testline:testline+1)        
    endif

    ;FLAG=0 means bad lines
    ;FLAG=1 means good lines
    ;loop: iterate this 4 times
    for iapp=0,3 do begin
        mn_mn = fltarr(nchan,ny)
        ;calculate the binomial smoothed data
        for ich=0,nchan-1 do begin
            mn_mn( ich,*)= ksmooth(lnavg(ich,*),flag(ich,*),10)
        endfor

        if keyword_set(debug) then begin
            print,iapp+1,mn_mn(testch,testline),total(flag(testch,*))
        endif

;        plot,mn_mn(0,*)
;        wait,1.
        ;redefine the flag as being more than 3 counts away from the 
        ;binomial estimate. This second flag removes the dependence
        ;on the step size as well as standard deviation.
        for ich=0,nchan-1 do begin
            flag(ich,*) = abs(mn_mn(ich,*)-lnavg(ich,*)) lt 3
        endfor
    endfor
tval = 1
    qcline = 1 -flag
    qc = qc + (QCline)*2^tval
    if keyword_set(debug) then begin
        print,'DEBUG:: Test 2:: Lines passing faint/noisy scans'
        print,fix(total(qcline eq 0,2))
    endif
;========================================================
; TEST 3: Test for duplicate lines
;--------------------------------
;         This identifies adjacent lines which are duplicates of each
;         other and flags only the first as valid
    tval = 2
    qc3 = qc*0
    
    flg_dup = intarr(nch,ny)
    flg_dup(*) = 0
    for ic=0,nch-1 do begin
        im = reform(image(ic,*,*))
    ;sum along the scan line
        sums = total(im,1)
    ;find unique lines
        u = uniq(sums)
        nu = n_elements(u)
        if (nu ne ny) then begin
            ind = lindgen(nu)
            uu=[0,u]
            dl = uu[ind+1]-uu[ind]
            dups = where(dl gt 1,ndup)
            for idup=0,ndup-1 do begin
;            if (dups(idup) eq 0) then i1 = 0 else $
                i1 = (u(dups(idup)-1*(dups(idup) ne 0))+1)*(dups(idup) ne 0)
                i2 = u(dups(idup))
                for ii=i1+1,i2 do begin
                    if (array_equal(im(*,ii-1),im(*,ii))) then flg_dup(ic,ii)=1
                endfor
            endfor
        endif
    endfor
    qc3 = flg_dup
    qc = qc + qc3*2^tval
    if keyword_set(debug) then begin
        print,'DEBUG:: Test 3:: Lines passing line duplicates'
        print,fix(total(qc3 eq 0,2))
    endif

;========================================================
; TEST 4: Test for temporal inconsistencies
;--------------------------------
;         This identifies gaps in the temporal record stored in the
;         line headers of the B1 file
    tval = 3
    qc4 = qc*0

    ;Get julian day from filename
    f=strsplit(file,'.',/extr)
    year = long(f(4))
    month= long(f(5))
    day  = long(f(6))
    hhmm = long(f(7))
    jdayf = julday(month,day,year,hhmm/100,hhmm mod 100,0.)

    if (b1u.pre(0,3,40) gt 1900) then begin
        jdays = reform(julday(b1u.pre(*,3,*)*0+1,b1u.pre(*,4,*),b1u.pre(*,3,*)>1, $
                              b1u.pre(*,5,*)/100,b1u.pre(*,5,*) mod 100, $
                              b1u.pre(*,6,*) + b1u.pre(*,7,*)/1000.))
        qc4 = qc4 + (abs(jdays-jdayf) gt 0.5)
    endif

    ;turn off the mask when nearly every line is flagged as bad, 
    ;this generalyy happens when the header info is corrupt.
    if (total(qc4) lt n_elements(qc4)*0.9 ) then $
      qc = qc + qc4*2^tval

    if keyword_set(debug) then begin
        print,'DEBUG:: Test 4:: Lines passing Line header temporal test'
        print,fix(total(qc3 eq 0,2))
    endif

if not(keyword_set(debug)) then goto,nodisp
    print,'% Bad lines:'
    print,total(qc ne 0,2)/float(ny)
    print,'QC ALL LINES: OK (%OK)'
    for ich=0,nch-1 do begin
        
        print,ich+1,fix(total(qc(ich,*) eq 0)),' ('+strtrim(total(qc(ich,*) eq 0)/float(ny),2)+')'
    endfor


    stop

;=======================================
; Display result
    loadct,0,/sil
    color,'green',255
    dx=400
    dy = ny/2+1
    for ich=0,nchan-1 do begin
;        goto,blinkit
        window,xs=dx*2,ys=dy,/free,xpos = 100,ypos = 100, $
               title=stripname(file)+' Ch.'+string(ich+1,format='(i2)')
        img = reform(image(ich,*,*))
        im = bytarr(dx,dy)
        if ((ny mod 2) eq 1) then ll = 2 else ll = 1
        i1 = nx/2 - dx/4    & i2 = nx/2 + dx/4-1
        j1 = 0              & j2 = ny/2

        x = lindgen(200,ny/2) mod 200
        y = lindgen(200,ny/2)  /  200
        x = (x-100)*3 + nx/2

        tv,img(x,y)<254,0,0,/ord
        tv,img(x,y+ny/2)<254,dx,0,/ord

blinkit:
        img = reform(image(ich,*,*)) < 254
        for j=0,ny-1 do begin
            if (QC(ich,j) ne 0) then img(*,j) = 255
        endfor
        im = bytarr(dx,dy)
        i1 = nx/2 - dx/4   & i2 = nx/2 + dx/4-1
        j1 = 0             & j2 = ny/2-1


        

        tv,img(x,y),dx/2,0,/ord
        tv,img(x,y+ny/2),dx*3./2.,0,/ord
        goto,noblink
        window,1,xs=nx,ys=ny
        tv,bytscl(image(ich,*,*))<254,/ord
        window,0,xs=nx,ys=ny
        tvscl,img,/ord
        blink,[0,1],.75
        stop
noblink:
    endfor

NODISP:

    if (not(keyword_set(nowrite))) then begin

;========================================================
; Output results to the QC file
;--------------------------------
        nbuf1 = 100 - strlen(stripname(file))
        nbuf2 = 100 - strlen(ver)
        openw,unit,QCfile,/get_lun
        writeu,unit,stripname(file)
        writeu,unit,bytarr(nbuf1)
        writeu,unit,ver
        writeu,unit,bytarr(nbuf2)
        writeu,unit,long([nch,ny])
        writeu,unit,qc
        writeu,unit,lnavg
        writeu,unit,lnstd
        close,unit
        free_lun,unit
    endif
    qc_line = qc
    
endelse

VERSION = ver

end
;***
