;****I* idl/idl.b1read
;+
; NAME
;    b1read
;
; PURPOSE
;    To read all B1 files as provided by the SPCs
;
; CATEGORY
;    ISCCP
;
; SYNOPSIS:
;    b1read, file=file,images=images,
;           AREAstr=AREAstr,NAVstr=NAVstr,RECstr=RECstr,GEOstr=GEOstr,$
;           ORBstr=ORBstr,IMGstr=IMGstr,ATTstr=ATTstr,TIMstr=TIMstr,$
;           CHNstr=CHNstr,TGRstr=TGRstr,TEST=TEST
;
; INPUTS
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORDS
;    TEST=TEST:  Sets ioutput=1 [print debug info where available]
;
; OUTPUTS:
;    AREAstr:Return area information [varies by b1read_*]
;    NAVstr: Return nav. information [varies by b1read_*]
;    RECstr: Return rec  info        [varies by b1read_*]
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers of the data
;    CALstr: Calibration structure
;    LAT:    The pixel Latitude (deg) values if LL=1
;    LON:    The pixel Longitude(deg) values if LL=1
;    B1LAND: # of pixels in a 5x5 of IGBP that are land [0-25]
;    B1TYPE: IGBP Landcover type [17=water]
;    B1RVER: Returns version information on the b1read routines
;    FHDR:   Return a byte array of the entire B1 file header
;
;
; USES
;    External Routine:
;        geo_struct.pro ... sets up the structures
;        ... these external routines read the various B1 formats ...
;        b1read_aes_g7.pro
;        b1read_aes_g8.pro
;        b1read_cma_fy2.pro
;        b1read_csu_g6.pro
;        b1read_csu_g7.pro
;        b1read_csu_g8.pro
;        b1read_csu_g91012.pro
;        b1read_csu_met3.pro
;        b1read_eum_met.pro
;        b1read_eum_msg.pro
;        b1read_jma_gms.pro
;        b1read_jma_mts.pro
;        b1read_noa_goe.pro
;        b1read_uws_g56.pro
;        stripname.pro
;        reset.pro
;        b1read_calv2.pro
;        b1u_ij2ll.pro
;        b1uread.pro
;        mcidas_calibrate.pro
;        
;
; EXAMPLE
;       b1read,file='ISCCP.B1.0.MET-3.1993.10.07.1800.CSU',IMAGE=IMG
;             Read a MET-3 file and return the images in the IMG
;             variable.
;
; MODIFICATION HISTORY
;       $Id: b1read.pro 22671 2010-09-10 19:41:02Z kknapp $
;       Created     12/2003 - Ken Knapp NCDC
;       Modified 01/08/2004 - Ken Knapp NCDC
;                             To include Meteosat B1 files
;       Modified 03/01/2004 - Ken Knapp NCDC
;                             To include structures [to support
;                             navigation using GEO_LL2IJ and GEO_IJ2LL]
;               03/11/2004    Imported to CVS
;               03/15/2004    Adding the option to output the line headers
;               03/17/2004    Adding the option to receive Calibration
;               03/19/2004    Added LL,LAT,LON keywords
;                             Added LC, B1LAND,B1TYPE
;               03/30/2004    Added B1RVER
;                             Removed LL, LC
;       28Jun07 - Changes since last commit include: file compression,
;                 nominal calibration (noaacal, mtscal, msgcal,...),
;                 msg read, early GOES files (oldNOA),
;       01Jun09 - Now reads CSU-style files created at NCDC
;-
; SOURCE

pro b1read,file=file, $
           images=images, $
           AREAstr=AREAstr, $
           NAVstr=NAVstr,$
           RECstr=RECstr, $
           GEOstr=GEOstr,$      ;use with geo_ij2ll and geoll2ij
           ORBstr=ORBstr,$      ;keplarian orbital info
           IMGstr=IMGstr,$      ;image size and scan geometry
           ATTstr=ATTstr,$      ;satellite attitude info
           TIMstr=TIMstr,$      ;temporal data
           CHNstr=CHNstr,$      ;channel information
           TGRstr=TGRstr,$      ;the test grid data
           LINhdr=LINhdr,$      ;LINE HEADER
           CALstr=CALstr,$      ;Calibration Structure
           TEST=TEST,$          ;Sets IOUTPUT=1
           LAT=LAT,LON=LON,$
           B1LAND=B1LAND,B1TYPE=B1TYPE, $
           B1U=B1U, $           ;Returns B1-Uniform Data
           FHDR=FHDR, $         ;returns file header
           NOOUT=NOOUT, $       ;set to limit output
           MISSCAL=MISSCAL, $
           SKIPCAL=SKIPCAL, $
           calfile=calfile, $
           debug=debug, $
           ctable=ctable, $
           orig_cal_table=orig_cal_table


reset

b1readver = '$Id: b1read.pro 22671 2010-09-10 19:41:02Z kknapp $'

@geo_struct.pro

ioutput = 0
if (keyword_set(test)) then ioutput=1
if not(keyword_set(file)) then begin
    file = dialog_pickfile(path='/home/kknapp/data/ISCCP-test/',$
                           filter='I*B1*')
endif
if not(file_test(file)) then begin
    ;attempt to fix the path
    f=strsplit(file,'.',/extr)
    tryfile='/net/haze/home/raid/kknapp/data/isccp/b1/'+f(4)+'/'+file
    if (file_test(tryfile)) then begin
        file=tryfile
    endif else if (f(8) eq 'NOA') then begin
        tryfile='/net/haze/home/raid/kknapp/data/isccp/b1/oldNOA/'+file
        if (file_test(tryfile)) then begin
            file=tryfile
        endif else begin
            print,'File:',file
            print,'DOES NOT EXIST'
            goto,endd
        endelse
    endif
endif

b1 = strpos(file,'B1')
if (b1 lt 0) then begin
    print,'File NOT a B1 file'
    goto,endd
endif

f1 = strsplit(file,'.',/extract)
;print,f1

spc = f1(8)
year = f1(4)
sat = f1(3)
mon = f1(5)
day = f1(6)

compress=0
if (n_elements(f1) eq 10) then compress=1

g = strsplit(sat,'-',/extract)
;index = g(1)

if (ioutput) then print,'SPC=',spc,' SAT=',sat

if (spc eq 'UWS') then begin
    if (sat eq 'GOE-5' or sat eq 'GOE-6') then begin
        b1read_uws_g56,file=file,images=images,AREAstr=AREAstr,$  
          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
          LINhdr=LINhdr,B1U=b1u,FHDR=FHDR,compress=compress
    endif else begin
        print,'Invalid UWS file:',file
        images=-1
        goto,endd
    endelse
    if (n_elements(b1u) gt 0) then b1u.img(14) = 256
endif else if (spc eq 'CSU') then begin
    if (sat eq 'GOE-6' or sat eq 'GOE-7') then begin
        ;the CSU switch from mode A to AAA occured in 1987.
        ; all data on or after 3/26/1987 is mode AAA
        ; all data prior to that date is mode A
        switch2aaa = julday(3,26,1987)
        thisday    = julday(fix(mon),fix(day),fix(year))
        if (thisday lt switch2aaa) then begin
            b1read_csu_g6,file=file,images=images, $
                          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                          LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                          noaacal=noaacal
            area = -1
            if (n_elements(b1u) gt 0) then b1u.img(14) = 256
        endif else begin
            b1read_csu_g7,file=file,images=images, $ ;nav=nav
                          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                          LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                          noaacal=noaacal
            AREAstr=-1
            if (n_elements(b1u) gt 0) then b1u.img(14) = 256
        endelse
    endif else if (sat eq 'GOE-8') then begin
        b1read_csu_g8,file=file,images=images,AREAstr=AREAstr,$  ;area=area,nav=nav
          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                      LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                      noaacal=noaacal
        if (n_elements(b1u) gt 0) then b1u.img(14) = 256
    endif else if (sat eq 'GOE-9' or $
                   sat eq 'GOE-10' or $
                   sat eq 'GOE-11' or $
                   sat eq 'GOE-12') then begin
        b1read_csu_g91012,file=file,images=images,AREAstr=AREAstr,$ ;area=area,nav=nav
                          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                          LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u, $
                          compress=compress,debug=debug,noaacal=noaacal
    endif else if (sat eq 'MET-3') then begin
        b1read_csu_met3,file=file,image=images,$ ;nav=nav
          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                        B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                        metcal=metcal
        if (n_elements(b1u) gt 0) then b1u.img(14) = 256
        area = 0
    endif else begin
        print,'Invalid CSU File:',file
        images=-1
        goto,endd
    endelse
endif else if (spc eq 'AES' or spc eq 'MSC') then begin
        ;sometime along the way, AES became MSC, so
        ;there is no difference in the files.
    if (sat eq 'GOE-7') then begin
        b1read_aes_g7,file=file,images=images,AREAstr=AREAstr,$
                      NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                      ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                      LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                      noaacal=noaacal
        if (n_tags(b1u) gt 0) then b1u.img(14) = 256
    endif else if (sat eq 'GOE-8' or sat eq 'GOE-12') then begin
        b1read_aes_g8,file=file,images=images,REC=RECstr,AREAstr=AREAstr,$
                      NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                      ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                      LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                      noaacal=noaacal
    endif else begin
        print,'B1READ: Invalid AES File:',file
        print,'SPC:',spc,' SAT:',sat
        images=-1
        goto,endd
    endelse
endif else if (spc eq 'EUM' or spc eq 'ESA') then begin
    if (strmid(sat,0,3) eq 'MET' and fix(g(1)) lt 8) then begin
        b1read_eum_met,file=file,image=images,$
                       GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                       ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                       LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u,compress=compress, $
                       metcal=metcal,debug=debug
        nav = 0
        area = 0
    endif else if (strmid(sat,0,3) eq 'MSG') then begin
        b1read_eum_msg,file=file,image=images,$
                       GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                       ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                       LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u, $
                       compress=compress,msgcal=msgcal
        nav = 0
        area = 0
    endif else begin
        print,'Invalid EUM file:',file
        print,'SPC:',spc
        print,'SAT:',sat
        images=-1
        goto,endd
    endelse
    if n_elements(b1u) eq 0 then begin
        print,'Invalid EUM file:',file
        print,'SPC:',spc
        print,'SAT:',sat
        images=-1
        goto,endd
    endif
endif else if (spc eq 'JMA') then begin
    if (strmid(sat,0,3) eq 'GMS') then begin
        b1read_jma_gms,file=file,images=images,lnhdr=lnhdr,$
                       NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                       ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                       LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u, $
                       compress=compress,debug=debug,gmscal=gmscal
        nav=0
        if (n_elements(b1u) gt 0) then b1u.img(14) = 256
    endif else if (sat eq 'GOES9' or sat eq 'GOE-9') then begin
        b1read_jma_gms,file=file,images=images,lnhdr=lnhdr,$
                       NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                       ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                       LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u, $
                       compress=compress,debug=debug,gmscal=gmscal
        nav=0
        if (n_elements(b1u) gt 0) then b1u.img(14) = 256
    endif else if (strmid(sat,0,3) eq 'MTS') then begin
        b1read_jma_mts,file=file,images=images,mtcal=mtcal,b1u=b1u,fhdr=fhdr,compress=compress
        if (n_elements(b1u) gt 0) then b1u.img(14) = 1024
    endif else begin
        print,'Invalid GMS file:',file
        print,'SPC:',spc
        images=-1
        goto,endd
    endelse
endif else if (spc eq 'CMA') then begin
    if (sat eq 'FY2-C' or $
        sat eq 'FY2-E'     ) then begin
        b1read_cma_fy2,file=file,images=images,b1u=b1u,fhdr=fhdr,compress=compress, $
                       cmacal=cmacal,debug=debug
        nav=0
        if (n_elements(b1u) gt 0) then begin
            if (n_tags(b1u) gt 0) then begin
                tnames = tag_names(b1u)
                if ( total(strupcase(tnames) eq 'IMG') gt 0) then begin
                    b1u.img(14) = 1024
                endif
            endif else begin
                ;no structure to b1u so skip it
                goto,endd
            endelse
        endif
    endif else begin
        print,'Invalid CMA file:',file
        print,'SAT:',sat
        images=-1
        goto,endd
    endelse
endif else if (spc eq 'NOA') then begin
    if (sat eq 'SMS-1' or sat eq 'SMS-2' or $
        sat eq 'GOE-1' or sat eq 'GOE-2' or $
        sat eq 'GOE-3' or sat eq 'GOE-4' or $
        sat eq 'GOE-5' or sat eq 'GOE-6' or $
        sat eq 'GOE-7' or sat eq 'GOE-8') then begin
        b1read_noa_goe,file=file,images=images,b1u=b1u,fhdr=fhdr,compress=compress, $
                       debug=debug,noaacal=noaacal
        if (n_elements(b1u) gt 0) then begin
            if (n_tags(b1u) gt 0) then begin
                b1u.img(14) = 256 
            endif else begin
                images(0)=-1
                goto,endd
            endelse
        endif else begin
            images(0)=-1
            goto,endd
        endelse
    endif else if (sat eq 'GOE-11' or $
                   sat eq 'GOE-12' or $
                   sat eq 'GOE-13' or $
                   sat eq 'GOE-14' or $
                   sat eq 'GOE-15') then begin
;
;        this block reads files created at NCDC using the
;        CIRA-provided processing code
;
        b1read_csu_g91012,file=file,images=images,AREAstr=AREAstr,$ ;area=area,nav=nav
                          NAVstr=NAVstr,GEOstr=GEOstr,ORBstr=ORBstr,IMGstr=IMGstr,$
                          ATTstr=ATTstr,TIMstr=TIMstr,CHNstr=CHNstr,TGRstr=TGRstr, $
                          LINhdr=LINhdr,B1VER=b1spcver,FHDR=FHDR,b1u=b1u, $
                          compress=compress,debug=debug,noaacal=noaacal

    endif else begin
        print,'Invalid NOA file:',file
        print,'SAT=',sat
        images=-1
        goto,endd
    endelse
endif else if (spc eq 'INP') then begin
    b1uread,file=file,b1u=b1u
    images = b1u.image
    nbin = 256
    if (b1u.img(14) eq 1024) then nbin=1024
    nch = b1u.img(5)
    noaacal = fltarr(2,nch,nbin)
;FOR NOW IGNORE VIS CAL
    chans = [1,2,3,4,5]
    if (fix(g(1)) gt 11) then chans(4) = 6
    noaacal(*) = -1.
    for ichan=1,4 do begin
        mcidas_calibrate,areastr=b1u.nav.b1uarea,/tableonly, $
                         temperature=temp,radiance=rad, $
                         channel=chans(ichan),goessat=fix(g(1))
        noaacal(0,ichan,*) = rad
        noaacal(1,ichan,*) = temp
    endfor

endif else begin
    print,'This file is either:'
    print,'   a) Not a B1 file or'
    print,'   b) Not incorporated into this B1 reader'
    images=-1
    goto,endd
endelse

if (n_tags(b1u) eq 0) then goto,endd

nbin = 256
if (b1u.img(14) eq 1024) then nbin=1024


nch = b1u.img(5)
; Get Calibration information
; 
; Current version uses KT data. This is a binary version of the BT
; data. So it is faster to read.

if not(keyword_set(SKIPCAL)) then begin

;by default, set up calstr based on the nominal calibration from the
;provider
    calstr = fltarr(2,nch,nbin)
    cver = -999
    if (strmid(sat,0,3) eq 'MTS') then begin
                                ;use the MTS caliibration in lieu of ISCCP calibration
        calstr=fltarr(2,nch,nbin)
        if (nch eq 4) then begin
            calstr(1,*,*) = mtcal(0:3,*)
        endif else begin
            calstr(1,*,*) = mtcal
        endelse
        cver = -2
    endif else if (strmid(sat,0,3) eq 'MSG') then begin
        calstr=fltarr(2,nch,nbin)
        calstr(1,*,*) = msgcal
        cver=-3
    endif else if (strmid(sat,0,3) eq 'GMS') then begin
        calstr=gmscal
        cver=-4
    endif else if (spc eq 'NOA') then begin
        calstr=noaacal
        cver=-5
    endif else if (strmid(sat,0,3) eq 'MET') then begin
        calstr=metcal
        cver=-6
    endif else if (strmid(sat,0,3) eq 'GOE') then begin
        if ((strmid(sat,0,5) eq 'GOE-9' or $
             strmid(sat,0,5) eq 'GOES9') and $
            year gt 2000) then begin
            calstr = gmscal
            cver = -4
        endif else begin
            calstr=noaacal
            cver=-7
        endelse
    endif else if (strmid(sat,0,3) eq 'FY2') then begin
        calstr=cmacal
        cver=-8
    endif

    b1read_calv2,file,strtrim(b1u.sat(2:2+nch-1),2),nch,ctable, $
                 version=version,calfile=calfile,table=orig_cal_table

;   if a ctable is found the nfirst make any corrections based on the
;   type of channel
    if (n_elements(ctable) gt 0) then begin
        cver = -1

        b1u.rev(5) = stripname(calfile)
        s=size(ctable)

        ;If ISCCP provides 8-bit tables for 10-bit data
        if (s(3) ne nbin) then begin
            if (strmid(sat,0,3) eq 'MTS' or $
                strmid(sat,0,3) eq 'GOE') then begin
;               the following produces the correct index, so it will
;               work for the table values too
;               f=findgen(256)
;               i=findgen(1024)/4.
;               print,fix(interpol(f,f,i,/spline)*4.)
;       0       1       2       3       4       5       6       7       8       9
;      10      11      12      13      14      15      16      17      18      19
;      20      21      22      23      24      25      26      27      28      29
;     ...
;    1000    1001    1002    1003    1004    1005    1006    1007    1008    1009
;    1010    1011    1012    1013    1014    1015    1016    1017    1018    1019
;    1020    1021    1022    1023
                nctable=fltarr(2,nch,1024)
                ind=indgen(256)
                nind=findgen(1024)/4.
                for i=0,1 do begin
                    for j=0,nch-1 do begin
                        nctable(i,j,*) = interpol(reform(ctable(i,j,*)), $
                                                  ind, $
                                                  nind,/spline)
                    endfor
                endfor
;               The following invalidates any channels which ISCCP may
;               not be calibrating. The new assignment is made later
;               based on whether the ISCCP calibration actually
;               provides information ... or is blank
;                calstr=nctable
                oldctable=ctable
                ctable = nctable
            endif else begin
                print,'Calibration table mismatch for unknown problem.'
                print,'CalFile:',stripname(calfile)
                stop
            endelse
        endif
;the following assignment does not work when a channel is not
;calibrated by ISCCP. In this case, the calibration from the SPC
;should be used for channels without the ISCCP claibration
;        calstr = ctable
        for ich = 0,nch-1 do begin
;           only replace the nominal calibration if the ISCCP
;           calibration is not all zeroes
            if (total(ctable(1,ich,*)) gt 0.) then begin
                calstr(*,ich,*) = ctable(*,ich,*)
            endif else begin
;               if ISCCP doesn't have the calibration, then use the
;               MSG calibration, but reverse it to match the other
;               ISCCP calibration values
;               BUT we need to make sure not to reverse the Vis
;               channels.
                chname = strtrim(b1u.sat(2+ich),2)
                replace_table = reform(calstr(1,ich,*))
                if ( not(chname eq 'VSCHN' or chname eq 'VIS08' or chname eq 'IR16' or $
                         chname eq 'IRVAR' or chname eq 'VSVAR')) then begin
                    print,'Reversing ',ich,' ',chname
                    replace_table = reverse(replace_table)
                endif
                calstr(1,ich,*) = replace_table
            endelse
        endfor
        f = strsplit(stripname(calfile),'.',/extr)
        cver = fix(f(2))
    endif 

endif else begin
    calstr = fltarr(2,nch,nbin)
    cver = -1000
endelse

on_ioerror,null
;Get the latitude/longitude values if the LL keyword is set
if (arg_present(lon) or $
    arg_present(lat) or $
    arg_present(b1land) or $
    arg_present(b1type)) then begin
    ;nx = imgstr.npix
    ;ny = imgstr.nlin
    nx = b1u.img(4)
    ny = b1u.img(3)
    ii = lindgen(nx,ny) mod nx
    jj = lindgen(nx,ny) / nx

    ;using b1u_ij2ll instead of geo_ij2ll
    b1u_ij2ll,ii,jj,lat,lon,nav=b1u.nav
endif

if (arg_present(b1land) or arg_present(b1type)) then begin
    gd = where(lat gt (-90))
    nxlc = 43200l/5l
    nylc = 21600l/5l
    land = bytarr(nxlc,nylc)
    type = bytarr(nxlc,nylc)
    openr,1,getenv('DATDIR')+'/igbp/gigbp2_0ll.img.remap.5km'
    readu,1,land
    readu,1,type
    close,1

    ix = long((lon(gd)+180.)*120./5.)
    iy = long((90. - lat(gd)) * 120./5.) > 1

    b1type = byte(lat)*0+255
    b1land = byte(lat)*0+255
    b1land(gd) = land(ix,iy)
    b1type(gd) = type(ix,iy)
endif

if (n_elements(b1spcver) eq 1) then B1RVER = [b1readver,b1spcver]

if (n_elements(b1u) eq 1) then begin
    b1u.rev(2) = b1readver
    b1utags = tag_names(b1u)
    if (total(strpos(b1utags,'CAL') ge 0) gt 0) then begin
        b1u.cal = [cver,reform(long(calstr*1000l),2*nch*nbin)]
    endif else begin
        cal = [cver,reform(long(calstr*1000l),2*nch*nbin)]
        b1u = create_struct(b1u,'cal',cal)
    endelse
;    1ut = {rev:b1u.rev, $
;            img:b1u.img, $
;            sat:b1u.sat, $
;            nav:b1u.nav, $
;            cal:cal, $
;            tgr:b1u.tgr, $
;            pre:b1u.pre, $
;            image:b1u.image}
;    b1u=b1ut
;    b1ut=0
endif


endd:

end
;***
