;****I* idl/idl.b1read_aes_g7
;+
; NAME:
;    b1read_aes_g7
;
; PURPOSE:
;    To read ISCCP-B1 data created by the AES SPC for GOES-7 data.
;
; CATEGORY:
;    ISCCP
;
; SYNOPSIS:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    AREAstr: File header and header record info
;    NAVstr: Data from the OA file provided by AES
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;    
; COMMON BLOCKS:
;    These common blocks were used initially, but were replaced by the
;    structures above.
;               KEPLER - keplarian orbit elements
;               ATT - Satellite attitude parameters
;               IMAGE - Image coordinate constants
;               GREEN - greenwich right ascensions/times
;               
; USES
; * b1read_oa.pro
; * b1u_ij2ll.pro
; * cart2earth.pro
; * geo_struct.pro
; * get_file_size.pro
; * get_histry.pro
; * itime.pro
; * kep2cart.pro
; * navcheck.pro
; * plot_tle.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
; 
; USED BY
; * b1read.pro
; 
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        b1read_oa.pro  ... reads the OA files and returns navigation info
;
; MODIFICATION HISTORY:
;       $Id: b1read_aes_g7.pro 22672 2010-09-10 19:42:04Z kknapp $
;       Created 11/17/2003 - Ken Knapp NCDC
;       Modified01/06/2004 - Ken Knapp, NCDC
;                            Inserted common blocks for use with
;                            navigation routines
;       Modified03/02/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/15/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;                            Removed common blocks
;               06/03/2004   Added B1U functionality
;       * 28Jun07 - Changes since last commit: documentation, uses
;                   get_file_size.pro, compression, 
;-
; SOURCE

pro b1read_aes_g7,file=file,$     ;images=images,nav=nav,area=area
                  images=images, $ ;return image information in the images variable
                  AREAstr=AREAstr,$ ;area header in B1 file
                  NAVstr=NAVstr,$ ;navigation header
                  GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                  ORBstr=ORBstr,$ ;keplarian orbital info
                  IMGstr=IMGstr,$ ;image size and scan geometry
                  ATTstr=ATTstr,$ ;satellite attitude info
                  TIMstr=TIMstr,$ ;temporal data
                  CHNstr=CHNstr,$ ;channel information
                  LINhdr=LINhdr,$ ;LINE HEADERs
                  TGRstr=TGRstr,$ ;the test grid data
                  B1VER=B1VER,$ ;returns revision info
                  FHDR=FHDR,$   ;returns file header
                  b1u=b1u, $
                  compress=compress, $
                  noaacal=noaacal

b1ver = '$Id: b1read_aes_g7.pro 22672 2010-09-10 19:42:04Z kknapp $'

; set up the structures
@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = 'I*-7*AES'
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif
;GOES7 AES structure information
; this structure was deduced by inspecting the binary data files.

hdrrec = {head,$
          filename:bytarr(14),$ ;char*14 format: TTYYYYMMDDHHMM
          tdate:intarr(5),$     ;transmission date (yr,mon,day,hh,mm)
          sscan:0,$             ;startscan
          escan:0,$             ;end scan
          sline:0,$             ;start line
          eline:0,$             ;end line
          poffset:0,$           ;pixel offset
          soffset:0,$           ;scan offset
          sets:0}               ;number of AS or AC domain sets

header = {hdr,$
          type:bytarr(2),$
          cdate:bytarr(8),$
          ctime:bytarr(4),$
          year:0,$
          mon:0,$
          day:0,$
          hour:0,$
          min:0,$
          n:intarr(4),$
          spare:bytarr(1895) }
lnhdr = {lnhd, $
         year:0,$
         mon:0,$
         day:0,$
         hour:0,$
         min:0,$
         sec:0,$
         msec:0,$
         scnnum:0}

datarec={data,$
         hdr:lnhdr,$  ;yr,mon,day,hh,mm,100ths of sec, scan #
         data:bytarr(1911)}     ;vis/IR/Walsh-Hadamard data

; Get file information
filesize=get_file_size(file,status,comp=icomp)

goodrec = 1
if ((filesize mod 1927 ) ne 0) then begin
    print,'Warning: Non-integer number of records:',filesize/1927.
    goodrec = 0
endif

;open file
on_ioerror,problemread

;icomp=0
;if keyword_set(compress) then icomp=1

openr,unit,file,/swap_end,/get_lun,comp=icomp
readu,unit,header
point_lun,-1*unit,frank
point_lun,unit,0
readu,unit,hdrrec

if (total(header.spare) ne 0) then begin
    print,'Error: Header spare bytes should be 0''s but have sum of:', $
          total(header.spare)
    ;Checking to see if any data exists in the file
    point_lun,unit,1927
    d = bytarr(filesize-1927)
    readu,unit,d
    if (total(d ne 0) eq 0) then begin
        print,'All values zero.'
        goto,problemread
    endif
endif


if (arg_present(fhdr)) then begin
    fhdr = bytarr(1927)
    point_lun,unit,0
    readu,unit,fhdr
endif

;set up variables
nscans = hdrrec.escan-hdrrec.sscan+1
if (floor(filesize / 1927./3.) ne nscans) then nscans = floor(filesize / 1927. / 3.)
data = replicate(datarec,3,nscans)
ny = nscans
nx = 1911
images = -1
if (ny lt 5) then begin
    ;return if there are less than 5 scan lines
    print,'Returning ... too few lines:',ny
    images = -1
    return
endif
if (goodrec eq 1) then begin
;read imagery
    point_lun,unit,1927
    readu,unit,data
endif else begin
    point_lun,unit,1927
    i=0
    jk=3
    while (jk eq 3) do begin
        j = 0
        point_lun,-1*unit,here
        if ((filesize-here) lt 1927) then goto,done
        readu,unit,datarec
        if (i gt 0) then begin
            ;If the scan number changes a lot ... then scan lines
            ;have been skipped and the header is likely messed up,
            ;begin searching for a new header...
            if (abs(datarec.hdr.scnnum - old.hdr.scnnum) gt 10 or $
                datarec.hdr.year lt 1983 or $
                datarec.hdr.year gt 2010) then begin
;                print,'uh oh',old.hdr.scnnum,datarec.hdr.scnnum
                hok = 0
                j = 1l
                while (hok eq 0) do begin
                    ;print,'Bytes left:',filesize - (here+j)
                    if ((filesize - (here+j)) lt 1927) then goto,done
                    point_lun,unit,here+j
                    readu,unit,datarec
                    ;test to see if this header is ok, otherwise try 
                    ;another byte
                    yr = datarec.hdr.year
                    mo = datarec.hdr.mon
                    da = datarec.hdr.day
                    hr = datarec.hdr.hour
                    mi = datarec.hdr.min
                    if (yr ge 1983 and yr le 2010 and $
                        mo ge 1 and mo le 12 and $
                        da ge 1 and da le 31 and $
                        hr ge 0 and hr le 23 and $
                        mi ge 0 and mi le 59) then begin
                        ;good record found
                        restart = here+j
;                        print,'Old Header scan #:',old.hdr.scnnum
 ;                       print,'New Header scan #:',datarec.hdr.scnnum
  ;                      print,yr,mo,da,hr,mi,datarec.hdr.scnnum
                        curscn = datarec.hdr.scnnum
                        ileft = 0
                        on_ioerror,done
                        while (datarec.hdr.scnnum eq curscn) do begin
                            readu,unit,datarec
   ;                         print,datarec.hdr.scnnum
                            ileft = ileft + 1
                        endwhile
                        on_ioerror,null
                        ;If the following is not met, then there was
                        ;an errors while trying to determine which 
                        ;line this is ... so look for another line
                        if (datarec.hdr.scnnum ne (curscn+1)) then begin
                            j=j+1
                            goto,anotherline
                        endif

                        i = (curscn-data(0,0).hdr.scnnum + 1) * 3 - ileft 

                        point_lun,unit,restart
                        readu,unit,datarec
                        yr = datarec.hdr.year
                        if (yr lt 1983 or yr gt 2010) then stop
                        if (datarec.hdr.scnnum lt old.hdr.scnnum) then goto, done
                        hok = 1
                    endif else begin
                        j=j+1
                    endelse
anotherline:
                endwhile
            endif
        endif
storeit:
        yr = datarec.hdr.year
        mo = datarec.hdr.mon
        da = datarec.hdr.day
        hr = datarec.hdr.hour
        mi = datarec.hdr.min
;        print,'L1',yr,mo,da,hr,mi,datarec.hdr.scnnum,i / 3, i mod 3,j
        if ((i/3) ge nscans) then goto,done
        data(i mod 3,i / 3) = datarec
        i=i+1
        yr = datarec.hdr.year
        if (yr lt 1983 or yr gt 2010) then goto,problemread
        old = datarec
    endwhile
endelse
done:
close,unit
free_lun,unit
on_ioerror,null
images_ = data.data
images_ = transpose(images_,[1,0,2]) 
LINhdr = data.hdr

readOA:
;Get Navigation (OA = Orbital/Attitude) information from the OA file
f = strsplit(file,'.',/extr)
f(1) = 'B1OA'
oafile = strjoin(f,'.')
b1read_oa,file=oafile,oa=oa,rec=rec
if (rec(0) eq -1) then begin
    print,'No B1OA file'
    images = -1
    return
endif

navstr=oa
nav = oa
areastr = {header:header,$
           hdrrec:hdrrec}

;get julian day of EPOCH1
year = 1900 + oa.oa_date1 / 1000
jday = oa.oa_date1 mod 1000
etime = itime(oa.oa_time1/100./3600.)
hh =  etime / 10000
mm = (etime / 100) mod 100
ss = (etime / 10000)
ejday1 = julday(1,jday,year,hh,mm,ss)
ejday2 = ejday1 + 13.d0/24.d0


;Chebyshev polynomials
cx = nav.oa_cx / 2.d^13
cy = nav.oa_cy / 2.d^13
cz = nav.oa_cz / 2.d^13

; position in km
x1 = double(nav.oa_x1)/2.d^13
y1 = double(nav.oa_y1)/2.d^13
z1 = double(nav.oa_z1)/2.d^13
x2 = double(nav.oa_x2)/2.d^13
y2 = double(nav.oa_y2)/2.d^13
z2 = double(nav.oa_z2)/2.d^13

; velocity in km/hr
vx1 = double(nav.oa_vx1)/2.d^13
vy1 = double(nav.oa_vy1)/2.d^13
vz1 = double(nav.oa_vz1)/2.d^13
vx2 = double(nav.oa_vx2)/2.d^13
vy2 = double(nav.oa_vy2)/2.d^13
vz2 = double(nav.oa_vz2)/2.d^13

pdate = [nav.oa_date1,nav.oa_time1]
kdate = [nav.oa_epy, nav.oa_eph]
kep = [nav.oa_sma/1.e2, $
       nav.oa_ecc/1.e6, $
       nav.oa_inc/1.e3, $
       nav.oa_ran/1.e3, $
       nav.oa_ap/1.e3, $
       nav.oa_ma/1.e3]
navcheck,pdate, $
         [x1,y1,z1],[vx1,vy1,vz1], $
         [x2,y2,z2],[vx2,vy2,vz2], $
         cx,cy,cz,kdate,kep,result, $
         kepdate=kepdate,kepout=kepout
if (n_elements(result) eq 1) then begin
    print,'MAJOR ERROR: No Navigational Information in B1OA file'
endif

; Get orbital parameters from b3 file
;get b3 HISTRY information
ib3h = 0
if (oa.oa_date1 lt 75001) then ib3h = 1
if (result(0) eq (-1)) then ib3h = 1
if (ib3h) then begin
;   If this point is ever reached, the same tests will need to be
;   performed on the HISTRY data as was done on the nav data. This can
;   be performed later when and if the failure occurs.
;   This point will likely be reached if the epoch date is bad or all
;   the navigation info is missing.

    get_histry,file,orbstr=b3ORBstr,IMGstr=IMGstr,ATTstr=ATTstr, $
               TIMstr=TIMstr,TGRstr=TGRstr,histry=histry
    if (histry(0) eq -1) then begin
        images = -1
        return
    endif
    print,'Error: Using B3 HISTRY parameter'
    ;run nav check again except now use histry
    ; position in km
    x1 = double(histry(6))/2.d^13
    y1 = double(histry(7))/2.d^13
    z1 = double(histry(8))/2.d^13
    x2 = double(histry(102))/2.d^13
    y2 = double(histry(103))/2.d^13
    z2 = double(histry(104))/2.d^13
    
; velocity in km/hr
    vx1 = double(histry(9))/2.d^13
    vy1 = double(histry(10))/2.d^13
    vz1 = double(histry(11))/2.d^13
    vx2 = double(histry(105))/2.d^13
    vy2 = double(histry(106))/2.d^13
    vz2 = double(histry(107))/2.d^13
    
    pdate = [histry(0),histry(1)/100.]
    kdate = [histry(83),histry(84)]
    kep = [histry(85)/1.e2, $
           histry(86)/1.e6, $
           histry(87)/1.e3, $
           histry(90)/1.e3, $
           histry(89)/1.e3, $
           histry(88)/1.e3]
;Chebyshev polynomials
    cx = histry(48:58) / 2.d^13
    cy = histry(60:70) / 2.d^13
    cz = histry(72:82) / 2.d^13
    
    navcheck,pdate, $
             [x1,y1,z1],[vx1,vy1,vz1], $
             [x2,y2,z2],[vx2,vy2,vz2], $
             cx,cy,cz,kdate,kep,result, $
             kepdate=kepdate,kepout=kepout
endif
;Get time and date of the image
year = hdrrec(0).tdate(0)
mon  = hdrrec(0).tdate(1)
day  = hdrrec(0).tdate(2)
hh = hdrrec(0).tdate(3)
mm = hdrrec(0).tdate(4)
jday = julian(mon,day,(year mod 4) eq 0)

;get keplerien elements from NORAD TLE's for comparison
plot_tle,tle,sat='goes-7',year=year,jday=jday,$
         ORBstr=TLEORBstr
yymmdd = (year-1900l) * 10000l + mon * 100l + day
frtime = nav.ttime(0)*3600.d0 + nav.ttime(1)*60.d0 + nav.ttime(2)
frdate = yymmdd_2_yyjjj(yymmdd)
time1 = oa.oa_time1 * 1.e-2 ;time in seconds
ep_time = itime(time1/3600.)
ep_msec = (time1 mod 1.)*1000

;assign values 
spra1 = nav.oa_spr1 / 2.d0^21
spra2 = nav.oa_spra2 / 2.d0^21
spdc1 = nav.oa_spdc1 / 2.d0^21
spdc2 = nav.oa_spdc2 / 2.d0^21
pitch = nav.oa_zeta / 2.d0^21
roll  = nav.oa_rho  / 2.d0^21
yaw   = nav.oa_eta  / 2.d0^21

cen_lin = 7284 ;totscl * 4 = 1821 * 4
cen_pix = 7648 ;totsmp * 2 = 3822 * 2
linres = 8 ;nav.yirstp
pixres = 8 ;nav.xirstp
linang = 1.d0/728.586d0
pixang = 18.375d0/15288.d0 ; 2. * 9.1875 / 3822 / 4
spper = nav.oa_sper * 1.d-6

spix = 0.
sline = hdrrec.sline * 8
;
;Assign variables to structures
;
;

ejday1 = kepdate

;setup date/time info for output
caldat,kepdate,kmon,kday,kyear,khh,kmm,kss
jjj = round(kepdate - julday(1,1,kyear,khh,kmm,kss) + 1)
bestkepdate = kyear*1000l + jjj
bestkeptime = khh*10000l + kmm*100l + round(kss)
bestkepmsec = round((ss mod 1) * 1000)


time1 = oa.oa_time1 * 1.e-2 ;time in seconds
ep_time = itime(time1/3600.)
ep_msec = (time1 mod 1.)*1000
msec = (oa.oa_time1  mod 100) * 10

orbit.ep_date = yymmdd_2_yyjjj(oa.oa_epy)
orbit.ep_time = oa.oa_eph
orbit.ep_msec = 0
orbit.semima =  oa.oa_sma / 100.
orbit.incl =    oa.oa_inc / 1.e3
orbit.eccen =   oa.oa_ecc/ 1.e6
orbit.meana =   oa.oa_ma/ 1.e3
orbit.rgtasc =  oa.oa_ran/ 1.e3
orbit.perigee = oa.oa_ap/ 1.e3
orbit.sper = spper
ORBstr = orbit

att.roll = roll
att.pitch = pitch
att.yaw = yaw
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att
;if (oa.oa_time1 ne 0) then stop
sattim.d = 46800.               ;gt2-gt1
sattim.gt1 = oa.oa_time1 / 100.        ;gt1
sattim.gt2 = sattim.gt1+sattim.d ;gt2
sattim.gra1 = oa.oa_gra1/2.^21  ;gra1
sattim.gra2 = oa.oa_gra2/2.^21  ;gra2
sattim.spra1 = spra1
sattim.spra2 = spra2
sattim.spdc1 = spdc1
sattim.spdc2 = spdc2
sattim.cheb_x = 0.              ;nav.oa_cx
sattim.cheb_y = 0.              ;nav.oa_cy
sattim.cheb_z = 0.              ;nav.oa_cz
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

pickup:
;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = ny
imginfo.npix = nx
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

chninfo.nchan = 3
nchan = 3
chninfo.ch_name(0) = 'IRWIN'
chninfo.ch_name(1) = 'VSCHN'
chninfo.ch_name(2) = 'WLSHM'

if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '10' and f(6) eq '12' and f(7) eq '1801') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif
if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '07' and f(6) eq '29' and f(7) eq '0601') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif
if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '08' and f(6) eq '10' and f(7) eq '0001') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif
if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '08' and f(6) eq '14' and f(7) eq '1801') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif
if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '09' and f(6) eq '09' and f(7) eq '0601') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif
if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '12' and f(6) eq '10' and f(7) eq '0601') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif
if (f(3) eq 'GOE-7' and f(4) eq '1987' and f(5) eq '12' and f(6) eq '22' and f(7) eq '0001') then begin
    chninfo.ch_name(0) = 'IRWVP'
endif


chninfo.ch_info = 'none'
CHNstr = chninfo

; get parameters at start and end of scan

;determine first and last good scan
scannum = reform(linhdr(0,*).scnnum)
ny = n_elements(scannum)
ind = indgen(ny)
delta = scannum(1:ny-1)-scannum(0:ny-2)
;valid scan flag
valido = intarr(10000)
validn = intarr(10000)
;find 5 consecutive lines
del = 1                         ;median(delta)
for i=0,ny-11 do begin
    if (total(delta(i:i+4)) eq (5)) then break
endfor
if (i eq (ny-11)) then begin
    ;image has NO places with 10 consecutive scan lines
    print,'Noisy image: 5 consecutive lines do not exist'
goto,problemread
    stop
endif
istart = i
valido((istart)) = 1
if (scannum(istart) lt 0) then goto,problemread
if (scannum(istart) gt 9999) then goto,problemread
validn(scannum(istart)) = 1
scan1 = i
if (i ne 0) then begin
    last = istart
    for j=istart-1,0,-1 do begin
        jdel = scannum(last) - scannum(j)
        if (jdel ge del and jdel le (10*del) and scannum(j) ne 0) then begin
            valido((j)) = 1
            validn(scannum(j)) = 1
            last = j
        endif else begin
            if ((j-last) gt 100) then break
        endelse 
    endfor
    scan1 = last
endif
last = istart
for j=istart+1,3000 do begin
    if (j eq ny) then break
    jdel = scannum(j)-scannum(last)
    if (jdel ge del and jdel le (10*del) and scannum(j) ne 0) then begin
        valido((j)) = 1
        validn(scannum(j)) = 1
        last = j
    endif else begin
        if ((j - last) gt 100) then break
    endelse
endfor
scan2 = last
;adjust the image for missing scan lines
newny = scannum(scan2)-scannum(scan1)+1
if (newny ne ny) then begin
    print,newny-ny+1,' missing lines'
;    if (valido(0) ne 1) then begin
;        s = size(images_)
;        nch = 1
;        if (s(0) gt 2) then nch = s(1)
;        set_plot,'x'
;        window,xs=300,ys=300*nch
;        for i=0,nch-1 do begin
;            tvscl,congrid(images_(i,*,*),1,300,300),0,300*i
;        endfor
;        print,'Bad file? (yes or no)'
;        input=''
;        read,input
;        if (input eq 'yes') then begin
;            file_move,file, $
;                      '/raid/kknapp/data/isccp/b1/failed-read/'+stripname(file)
;            images = -1
;            return
;        endif else begin
;            stop
;        endelse
;        stop
;    endif
;    if (newny lt (ny-4)) then stop
    newimages = bytarr(nchan,nx,newny)
    newlinhdr = replicate(lnhdr,nchan,newny)
    oldind = where(valido eq 1)
    newind = where(validn eq 1)
    newind = newind - min(newind)
    newimages(*,*,newind) = images_(*,*,oldind)
    newlinhdr(*,newind) = linhdr(*,oldind)
    images_ = newimages
    linhdr = newlinhdr
    ;adjust old variables
    ny = newny
    imginfo.nlin = newny
    nscans = newny
    imgstr.nlin = newny
    scan1 = newind(0)
    scan2 = newind(n_elements(newind)-1)
endif

scans = [scan1,scan2]

; OLD METHOD for getting the 1st and last valid scan
; This method did not work well, was not robust, so it 
; failed easily.
;for i=0,ny-1 do begin
;    if (linhdr(0,i).scnnum ne 0) then break
;endfor
;scan1 = i
;for i=ny-1,0,-1 do begin
;    if (linhdr(0,i).scnnum ne 0) then break
;endfor
;scan2 = i
;scans = [scan1,scan2]

;get time of first and last valid scanline
;calculate seconds since epoch for each scan line
stime1 = (julday(linhdr(0,scan1).mon, $
                 linhdr(0,scan1).day, $
                 linhdr(0,scan1).year, $
                 linhdr(0,scan1).hour, $
                 linhdr(0,scan1).min, $
                 linhdr(0,scan1).sec + $
                 linhdr(0,scan1).msec/1000.) - ejday1)*3600.d0*24.d0
stime2 = (julday(linhdr(0,scan2).mon, $
                 linhdr(0,scan2).day, $
                 linhdr(0,scan2).year, $
                 linhdr(0,scan2).hour, $
                 linhdr(0,scan2).min, $
                 linhdr(0,scan2).sec + $
                 linhdr(0,scan2).msec/1000.) - ejday1)*3600.d0*24.d0

;get satellite sub-points at start and end of scan
; using best kepler values from NAVCHECK
kep2cart,kepout,stime1,sx1,sy1,sz1
kep2cart,kepout,stime2,sx2,sy2,sz2
; convert cartesian to Earth location
cart2earth,sx1,sy1,sz1, $
           (stime1/(3600.d0*24.d0)+ejday1), $
           glat1,glon1,grad1
cart2earth,sx2,sy2,sz2, $
           (stime2/(3600.d0*24.d0)+ejday1), $
           glat2,glon2,grad2
; Not sure, but it looks like the longitude is reversed??????
glon1 = - glon1 
glon2 = - glon2


;interpolate spin axis angles for scan1 and scan2
etim = nav.oa_time1 / 100.d0
ehh = floor(etim/3600.)
emm = floor(etim/60.) mod 60
ess = etim mod 60
frac1 = (julday(linhdr(0,scan1).mon, linhdr(0,scan1).day,linhdr(0,scan1).year, $
                linhdr(0,scan1).hour,linhdr(0,scan1).min,linhdr(0,scan1).sec + $
                linhdr(0,scan1).msec/1000.) - $
         julday(1,nav.oa_date1 mod 1000,1900+nav.oa_date1/1000, $
                ehh,emm,ess))*24.d0/13.
frac2 = (julday(linhdr(0,scan2).mon, linhdr(0,scan2).day,linhdr(0,scan2).year, $
                linhdr(0,scan2).hour,linhdr(0,scan2).min,linhdr(0,scan2).sec + $
                linhdr(0,scan2).msec/1000.) - $
         julday(1,nav.oa_date1 mod 1000,1900+nav.oa_date1/1000, $
                ehh,emm,ess))*24.d0/13.
;frac1 = stime1 / 3600.d0 / 13.d0
;frac2 = stime2 / 3600.d0 / 13.d0

sr1 = spra1
sr2 = spra2
sd1 = spdc1
sd2 = spdc2
;if (abs(sr1-sr2) gt 40. and sr2 ne 0. and ny gt 200) then stop
;if (abs(sd1-sd2) gt 40. and sd2 ne 0. and ny gt 200) then stop


if (sr1 eq 0.) then begin
    sspra1 = sr2
    sspra2 = sr2
endif else if (sr2 eq 0.) then begin
    sspra1 = sr1
    sspra2 = sr1
endif else begin
    sspra1 = sr1 + (sr2-sr1)*frac1
    sspra2 = sr1 + (sr2-sr1)*frac2
endelse 

if (sd1 eq 0.) then begin
    sspdc1 = sd2
    sspdc2 = sd2
endif else if (sd2 eq 0.) then begin
    sspdc1 = sd1
    sspdc2 = sd1
endif else begin
    sspdc1 = sd1 + (sd2-sd1)*frac1
    sspdc2 = sd1 + (sd2-sd1)*frac2
endelse

att.spra = -9999.               ;spnax1(0)
att.spdc = -9999.               ;spnax1(1)
ATTstr = att

TGRID.n    = 0
TGRstr=TGRID

nx = imgstr.npix
ny = imgstr.nlin
ii = [nx/2,nx/2]
jj = [1,ny-1]
b1u_ij2ll,ii,jj,lat,lon, $
          b1TIM=TIMstr,b1ATT=ATTstr, $
          b1ORB=ORBstr,b1IMG=IMGstr, $
          glat=glat,glon=glon,grad=grad


;CALIBRATION information for SMS & GOES 1-7

count=indgen(256)
noaacal = fltarR(2,nchan,256)
irloc = where(strtrim(chninfo.ch_name,2) eq 'IRWIN')
if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
    noaacal(1,irloc(0),0:176) = 330.-count(0:176)/2.
    noaacal(1,irloc(0),177:255)=418-count(177:255)
    wvl=11.e-4
    c = 1.44
    noaacal(0,irloc(0),*) = (1.195e-5)*(wvl^(-3))*(exp(c/wvl/noaacal(1,irloc(0),*))-1.)^(-1)
endif
irloc = where(strtrim(chninfo.ch_name,2) eq 'VSCHN')
if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
    noaacal(*,irloc(0),*) = -999.
endif

if keyword_set(debug) then stop



;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)
;---------------------------
;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver
;---------------------------
IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = itime(IMGstr.frtime/3600.)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder
;---------------------------
;Satellite information
i=0
SATinf(*) = 'Missing'
SATinf(i++) = 'GOES-7'
SATinf(i++) = 'VISSR'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor
;  GOES-7 channels info will be FWHM bounds ... estimated from the
;  ISCCP web page
g7info = ['0.52-0.73 um','10.3-12.1 um']
for j=0,nchan-1 do begin
    tmp = 'Walsh-Hadamard-XForm'
    if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g7info(0)
    if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g7info(1)
    SATinf(i++) = tmp
endfor
;---------------------------
; Navigation information
; Keplarian information
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10.
NAVinf(3) = IMGstr.cen_pix * 10.
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = oa.oa_sper          ;Seconds per revolution*10^6
;KEPSAT
;Set flag for origin of points
if (ib3h) then begin
    NAVinf(9) = 18
    ;Set up new flags in the Documentation table and include values here
endif else begin
    if (result(0) eq 2) then NAVinf(9) = 10 ;Kepler
    if (result(0) eq 1) then NAVinf(9) = 12 ;Kepler from position 2
    if (result(0) eq 0) then NAVinf(9) = 11 ;Kepler from position 1
endelse
NAVinf(10) = bestkepdate           ;oa.oa_date1
NAVinf(11) = bestkeptime
NAVinf(12) = bestkepmsec
NAVinf(13) = kepout(0)*1.e2
NAVinf(14) = kepout(2)*1.e3
NAVinf(15) = kepout(1)*1.e6
NAVinf(16) = kepout(5)*1.e3
NAVinf(17) = kepout(3)*1.e3
NAVinf(18) = kepout(4)*1.e3

;KEPFIT
NAVinf(19:28) = MISSING
;KEPTLE
NAVinf(29) = 13
NAVinf(30) = ORBstr.EP_date
NAVinf(31) = ORBstr.EP_time
NAVinf(32) = ORBstr.EP_msec
NAVinf(33) = ORBstr.semima *1.e2
NAVinf(34) = ORBstr.incl   *1.e3
NAVinf(35) = ORBstr.eccen  *1.e6
NAVinf(36) = ORBstr.meana  *1.e3
NAVinf(37) = ORBstr.rgtasc *1.e3
NAVinf(38) = ORBstr.perigee*1.e3
;ATTDAT
NAVinf(39) = round(ATTstr.roll * 1.e6)
NAVinf(40) = round(ATTstr.pitch* 1.e6)
NAVinf(41) = round(ATTstr.yaw  * 1.e6)
if (ib3h) then begin
    NAVinf(39:41) = histry([16,15,17])/2.d0^21*1.e6
endif
;ATTFIT
NAVinf(42:48) = MISSING
;RECINFO
NAVinf(49:51) = MISSING
;FITINFO
NAVinf(52:56) = MISSING
;POSDAT
NAVinf(57) = nav.oa_date1
NAVinf(58) = nav.oa_time1
NAVinf(59) = nav.oa_x1
NAVinf(60) = nav.oa_y1
NAVinf(61) = nav.oa_z1
NAVinf(62) = nav.oa_vx1
NAVinf(63) = nav.oa_vy1
NAVinf(64) = nav.oa_vz1
NAVinf(65) = nav.oa_spr1        ;be careful here, because this tag name is a typo
NAVinf(66) = nav.oa_spdc1
NAVinf(67) = nav.oa_zeta
NAVinf(68) = nav.oa_rho
NAVinf(69) = nav.oa_eta
NAVinf(70) = nav.oa_gamma
NAVinf(71) = nav.oa_sra1
NAVinf(72) = nav.oa_sdc1
NAVinf(73) = nav.oa_gra1
NAVinf(74) = nav.oa_est
NAVinf(75) = nav.oa_eet
NAVinf(76:85) = nav.oa_cb
NAVinf(86:96) = nav.oa_cx
NAVinf(97:107)= nav.oa_cy
NAVinf(108:118)=nav.oa_cz
NAVinf(119) = nav.oa_x2
NAVinf(120) = nav.oa_y2
NAVinf(121) = nav.oa_z2
NAVinf(122) = nav.oa_vx2
NAVinf(123) = nav.oa_vy2
NAVinf(124) = nav.oa_vz2
NAVinf(125) = nav.oa_spra2
NAVinf(126) = nav.oa_spdc2
NAVinf(127) = nav.oa_sraz       ;be careful, this tag name is a typo
NAVinf(128) = nav.oa_sdc2
NAVinf(129) = nav.oa_gra2
;LOCDAT
NAVinf(130) = 31
NAVinf(131) = 0
NAVinf(132) = round(glat(0)*1e6)
NAVinf(133) = round(glon(0)*1e6)
NAVinf(134) = round(grad(0)*1e3)
NAVinf(137) = ny-1
NAVinf(138) = round(glat(1)*1e6)
NAVinf(139) = round(glon(1)*1e6)
NAVinf(140) = round(grad(1)*1e3)
;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = 0
if (n_elements(result) eq 14) then NAVinf(146) = total(result(1:13))
NAVinf(147) = IMGstr.frdate
NAVinf(148) = IMGstr.frtime * 1000
NAVinf(149) = IMGstr.nlin
NAVinf(150) = IMGstr.npix
NAVinf(151:199) = MISSING
;---------------------------
; TEST GRID
TGRinf = MISSING
;---------------------------
; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = linhdr.scnnum ;absolute scan line
temp = linhdr.year * 0 + MISSING
gd = where(linhdr.year ne 0)
temp(gd) = linhdr(gd).year
PREinf(*,i++,*) = temp          ;year
temp(*) = MISSING
temp(GD) = julday(linhdr(gd).mon,linhdr(gd).day,linhdr(gd).year) - $
           julday(linhdr(gd).mon*0+1,linhdr(gd).day*0+1,linhdr(gd).year)+1 ;doy
PREinf(*,i++,*) = temp
PREinf(*,i++,*) = linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = linhdr.sec    ;Sec
PREinf(*,i++,*) = linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING ;eastern edge of earth
PREinf(*,i++,*) = MISSING ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images_,2) mod 256 ;checksum
;---------------------------
b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images_}      
;
; END B1U array definitions
;---------------------------------------------------

images = images_

goto,endd
problemread:
images=-1
print,'Problem Reading this file'
endd:

end

;***
