;****I* idl/idl.b1read_aes_g8
;+
; NAME:
;    b1read_aes_g8
;
; PURPOSE:
;    An input which reads ISSCP B1 data provided by AES for GOES-8
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    REC:    Navigation header in a FLTARR
;    NAVstr: Navigation data from the OA file
;    AREAstr:The file header
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; USES
; * b1read_oa.pro
; * bcd_time.pro
; * get_file_size.pro
; * mcidas_calibrate.pro
; * reset.pro
; * stripname.pro
; * tempfile.pro
; 
; USED BY
; * b1read.pro
; 
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        imcnav     ... calculates the effective spnax parameters
;        b1read_oa  ... reads the OA files and returns navigation info
;
; MODIFICATION HISTORY:
;       $Id: b1read_aes_g8.pro 22673 2010-09-10 19:44:00Z kknapp $
;       Modified03/01/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/15/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;               04/27/2004   Added B1U functionality
;               05/25/2004   fixed b1u.img(1) calculation
;       * 28Jun07 - Changes since last commit: noaacal keyword, uses
;                   get_file_size.pro,  compression, modify sat_id_num
;                   in AREAstr to allow calibration with
;                   mcidas_calibrate.pro 
; SOURCE

pro b1read_aes_g8,file=file,REC=REC,$ 
                  images=images, $;return image information
                  AREAstr=AREAstr,$
                  NAVstr=NAVstr,$ ;navigation header
                  GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                  ORBstr=ORBstr,$ ;keplarian orbital info
                  IMGstr=IMGstr,$ ;image size and scan geometry
                  ATTstr=ATTstr,$ ;satellite attitude info
                  TIMstr=TIMstr,$ ;temporal data
                  CHNstr=CHNstr,$ ;channel information
                  LINhdr=LINhdr,$ ;LINE HEADERs
                  TGRstr=TGRstr,$ ;the test grid data
                  B1VER=B1VER,$ ;returns revision info
                  FHDR=FHDR, $  ;returns file header
                  b1u=b1u, $
                  debug=debug, $
                  compress=compress, $
                  noaacal=noaacal

b1ver = '$Id: b1read_aes_g8.pro 22673 2010-09-10 19:44:00Z kknapp $'

; set up the structures
@geo_struct.pro


;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = 'ISCCP.B1.?.GOE-8*AES'
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif

if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    nav = -1
    images = -1
    area = -1
    goto,endd
endif

; define the block size of the AES file
block = 1284

;get file information
filesize=get_file_size(file,status,compress=icomp)
if (status ne 0) then stop

;get # of records
nr = filesize / block
if (filesize mod block) then begin
    print,'Non-standard AES file:',file
    stop
endif

;get the # of scans
nscan = (nr-1) / 5
if ((nr-1) mod 5) then begin
    print,'Unexpected number of channels'
    area=-1
    nav = -1
    images = -1
    stop
    goto,endd
endif

header = bytarr(block)

lnhdr = {hdr6, $
         sat:0,$                ;??
         scnnum:0, $            ;line number in instrument coord.
         spix:0}                ;starting pixel
scan = {hdr:lnhdr,line:bytarr(block-6)}

in = replicate(scan,5,nscan)


openr,1,file,/swap_end,comp=icomp
readu,1,header
readu,1,in
close,1

images = transpose(in.line,[1,0,2])
LINhdr = in.hdr

;if keyword_set(debug) then stop

; Return header in the area sector
area = header
AREAstr = area
fhdr = header
;Get Navigation (OA = Orbital/Attitude) information
f = strsplit(file,'.',/extr)
nf = n_elements(f) 
f(1) = 'B1OA'
oafile = strjoin(f,'.')
b1read_oa,file=oafile,oa=oa,rec=rec
if (n_elements(rec) eq 1 and rec(0) eq -1) then begin
    print,'No B1OA file: ',f
;    images = -1
;return
endif else begin
    nav=oa
    navstr=oa
;=======================================================
;  Assign NAVstr and AREAstr parameters using OA
;  This is used to define B1U Navigation information
;Define GOES-8 header information
AREAG8  = { B1UAREA,               $
            area_status:0L,$    ; Area status
            version_num:0L,$    ; Area version number
            sat_id_num:0L,$     ; Satellite ID (SSS)
            img_date:0L,$       ; Year and Julian day (YYDDD)
            img_time:0L,$       ; Time of image (HHMMSS)
            north_bound:0L,$    ; Upper left line in Sat coords (Y-Coord)
            west_vis_elem:0L,$  ; Upper left element in Sat coords (X-Coord)
            z_coor:0L,$         ; Upper z-coord (Z-Coord)
            num_line:0L,$       ; Number of lines in image (Y-SIZE)
            num_elem:0L,$       ; Number of elememts in image (X-SIZE)
            bytes_per_pixel:0L, $ ; Number of bytes per data element
            line_res:0L,$       ; Line resolution (Y-RES)
            elem_res:0L,$       ; Element resolution (X-RES)
            num_chan:0L,$       ; Number of bands (Z-RES)


            num_byte_ln_prefix:0L,      $ ; Number of bytes in line prefix (multiple of 4)
            proj_num:0L,$       ; Project number
            creation_date:0L,$  ; Creation date (YYYDDD)
            creation_time:0L,$  ; Creation time (HHMMSS)
            sndr_filter_map:0L,$ ; Filter map for soundings
            img_id_num:0L,$     ; Image ID number
            id:lonarr(4),$      ; Reserved for radar appications
            comment:bytarr(32),$; 32 char comments
            pri_key_calib:0L,$  ; Calibration colicil (area number)
            pri_key_nav:0L,$    ; Primary navigation codicil  (data)
            sec_key_nav:0L,$    ; Secondary navigation codicil (nav)
            val_code:0L,$       ; Validity code
            pdl:lonarr(8),$     ; PDL in packed-byte format
            band8:0L,$          ; Where band-8 came from
            act_img_date:0L,$   ; Actual image start day (YYYDDD)
            act_img_time:0L,$   ; Actual image start time (HHMMSS)
            act_start_scan:0L,$ ; Actual start scan
            len_prefix_doc:0L,$ ; Length of prefix documentation (DOC)
            len_prefix_calib:0L,$;Length of prefix calibration (CAL)
            len_prefix_lev:0L,$ ; Length of prefix level (LEV)
            src_type:bytarr(4),$; Source type
            calib_type:bytarr(4),$;Calibration type
            avg_or_sample:0L,$  ; Data was averaged (0) or sampled (1)
            poes_signal:0L,$    ; LAC, GAC, HRPT
            poes_up_down:0L,$   ; POES ascending/descending
            orig_src_type:bytarr(4),$;Original source type of data
            reserved:lonarr(7)} ; Reserved (6 calib pointer)
;Navigation structure
IMGR_SIN = { IMGR_SIN, mag_sinu:0L, phase_ang_sinu:0L }
; Imager repeat monomial T.
IMGR_MON = { IMGR_MON,            $
             order_appl_sinu:0L,  $
             order_mono_sinu:0L,  $
             mag_mono_sinu:0L,    $
             phase_ang_sinu:0L,   $
             ang_from_epoch:0L    }
;  Imager repeat T.
IMGR_REP = { IMGR_REP,                        $
             exp_mag:0L,                      $
             exp_time_const:0L,               $
             mean_att_ang_const:0L,           $
             num_sinu_per_angle:0L,           $
             sinusoid:REPLICATE(IMGR_SIN,15), $
             num_mono_sinu:0L,                $
             monomial:REPLICATE(IMGR_MON,4)   }
NAVG8  = { NAVstr_aes,                          $
           nav_type:bytarr(4), ID:bytarr(4), IMC_status:0L,        $
           Flip:0L, spare1:0L, ref_long:0L, ref_rad_dist:0L,       $
           ref_lat:0L, ref_orb_yaw:0L, ref_att_roll:0L,            $
           ref_att_pitch:0L, ref_att_yaw:0L, epoch_time:BYTARR(8), $ ; BCD_TIME
           start_time:0L, IMC_corr_roll:0L, IMC_corr_pitch:0L,     $
           IMC_corr_yaw:0L, ref_long_change:LONARR(13),            $
           ref_rad_dist_change:LONARR(11), sine_lat:LONARR(9),     $
           sine_orb_yaw:LONARR(9), solar_rate:0L,exp_start_time:0L,$
           roll_att:IMGR_REP, spare2:LONARR(10), more1:bytarr(4),  $
           gvar1:bytarr(4), pitch_att:IMGR_REP, yaw_att:IMGR_REP,  $
           spare3:LONARR(16), more2:bytarr(4), gvar2:bytarr(4),    $
           roll_misalgn:IMGR_REP, pitch_misalgn:IMGR_REP,          $
           img_date:0L, img_time:0L, instr:0L, spare4:LONARR(9),   $
           ns_cyl:0L,ew_cyl:0L,ns_inc:0L, ew_inc:0L,               $
           more3:bytarr(4),gvar3:bytarr(4),spare5:LONARR(126),     $
           more4:bytarr(4),gvar4:bytarr(4),spare6:LONARR(127)}
; MAKE assignments based on OA data

case f(3) of 
    'GOE-8':  AREAG8.sat_id_num = 70
    'GOE-9':  AREAG8.sat_id_num = 72
    'GOE-10': AREAG8.sat_id_num = 74
    'GOE-11': AREAG8.sat_id_num = 76
    'GOE-12': AREAG8.sat_id_num = 78
endcase 


NAVG8.imc_status     = OA.imc_flag
AREAG8.north_bound   = OA.rec_st_scn
AREAG8.west_vis_elem = OA.rec_st_pix
AREAG8.line_res      = 8
AREAG8.elem_res      = 16
AREAG8.num_line      = (OA.rec_en_scn - OA.rec_st_scn + 1) / AREAG8.line_res
AREAG8.num_elem      = (OA.rec_en_pix - OA.rec_st_pix + 1) / AREAG8.elem_res
NAVG8.ref_long       = OA.ref_long              * 1.d7
NAVG8.ref_rad_dist   = OA.ref_rad_dist_from_nom * 1.d7
NAVG8.ref_lat        = OA.ref_lat               * 1.d7
NAVG8.ref_orb_yaw    = OA.ref_orb_yaw           * 1.d7
NAVG8.ref_att_roll   = OA.ref_roll              * 1.d7
NAVG8.ref_att_pitch  = OA.ref_pitch             * 1.d7
NAVG8.ref_att_yaw    = OA.ref_yaw               * 1.d7
NAVG8.epoch_time     = byte(long(nav.bcd_epoch_time),0,8)
NAVG8.start_time     = OA.start_time_from_epoch * 1.d2
NAVG8.imc_corr_roll  = OA.imc_roll              * 1.d7
NAVG8.imc_corr_pitch = OA.imc_pitch             * 1.d7
NAVG8.imc_corr_yaw   = OA.imc_yaw               * 1.d7
NAVG8.ref_long_change= OA.delta_long            * 1.d7
NAVG8.ref_rad_dist_change=OA.delta_rad_dist     * 1.d7
NAVG8.sine_lat       = OA.sin_geo_lat           * 1.d7
NAVG8.sine_orb_yaw   = OA.sin_orb_yaw          * 1.d7
NAVG8.solar_rate     = OA.solar_rate            * 1.d7
NAVG8.exp_start_time = OA.exp_start_time_from_epoch * 1.d2
NAVG8.ew_cyl = long(OA.nadir_offsets.EW_NAD_CY)
NAVG8.ew_inc = long(OA.nadir_offsets.EW_NAD_INC)
NAVG8.ns_cyl = long(OA.nadir_offsets.NS_NAD_CY)
NAVG8.ns_inc = long(OA.nadir_offsets.NS_NAD_INC)
;------------------------------------
;NAVG8.roll_att      = OA.att_miss[0]
NAVG8.roll_att.exp_mag            = OA.att_miss[0].exp_mag * 1.d7
NAVG8.roll_att.exp_time_const     = OA.att_miss[0].exp_tim * 1.d2
NAVG8.roll_att.mean_att_ang_const = OA.att_miss[0].mn_att_ang * 1.d7
NAVG8.roll_att.num_sinu_per_angle = OA.att_miss[0].num_sin
NAVG8.roll_att.num_mono_sinu      = OA.att_miss[0].num_mon_sin
NAVG8.roll_att.sinusoid.mag_sinu        = OA.att_miss[0].sinusoids.magnitude * 1.d7
NAVG8.roll_att.sinusoid.phase_ang_sinu  = OA.att_miss[0].sinusoids.phs_ang   * 1.d7
NAVG8.roll_att.monomial.order_appl_sinu = OA.att_miss[0].polynomial.order_app
NAVG8.roll_att.monomial.order_mono_sinu = OA.att_miss[0].polynomial.order_1st
NAVG8.roll_att.monomial.mag_mono_sinu   = OA.att_miss[0].polynomial.mag           * 1.d7
NAVG8.roll_att.monomial.phase_ang_sinu  = OA.att_miss[0].polynomial.phs_ang       * 1.d7
NAVG8.roll_att.monomial.ang_from_epoch  = OA.att_miss[0].polynomial.ang_frm_epoch * 1.d7
;NAVG.pitch_att     = OA.att_miss[1]
NAVG8.pitch_att.exp_mag            = OA.att_miss[1].exp_mag * 1.d7
NAVG8.pitch_att.exp_time_const     = OA.att_miss[1].exp_tim * 1.d2
NAVG8.pitch_att.mean_att_ang_const = OA.att_miss[1].mn_att_ang * 1.d7
NAVG8.pitch_att.num_sinu_per_angle = OA.att_miss[1].num_sin
NAVG8.pitch_att.num_mono_sinu      = OA.att_miss[1].num_mon_sin
NAVG8.pitch_att.sinusoid.mag_sinu        = OA.att_miss[1].sinusoids.magnitude * 1.d7
NAVG8.pitch_att.sinusoid.phase_ang_sinu  = OA.att_miss[1].sinusoids.phs_ang   * 1.d7
NAVG8.pitch_att.monomial.order_appl_sinu = OA.att_miss[1].polynomial.order_app
NAVG8.pitch_att.monomial.order_mono_sinu = OA.att_miss[1].polynomial.order_1st
NAVG8.pitch_att.monomial.mag_mono_sinu   = OA.att_miss[1].polynomial.mag           * 1.d7
NAVG8.pitch_att.monomial.phase_ang_sinu  = OA.att_miss[1].polynomial.phs_ang       * 1.d7
NAVG8.pitch_att.monomial.ang_from_epoch  = OA.att_miss[1].polynomial.ang_frm_epoch * 1.d7
;NAVG.yaw_att       = OA.att_miss[2]
NAVG8.yaw_att.exp_mag            = OA.att_miss[2].exp_mag * 1.d7
NAVG8.yaw_att.exp_time_const     = OA.att_miss[2].exp_tim * 1.d2
NAVG8.yaw_att.mean_att_ang_const = OA.att_miss[2].mn_att_ang * 1.d7
NAVG8.yaw_att.num_sinu_per_angle = OA.att_miss[2].num_sin
NAVG8.yaw_att.num_mono_sinu      = OA.att_miss[2].num_mon_sin
NAVG8.yaw_att.sinusoid.mag_sinu        = OA.att_miss[2].sinusoids.magnitude * 1.d7
NAVG8.yaw_att.sinusoid.phase_ang_sinu  = OA.att_miss[2].sinusoids.phs_ang   * 1.d7
NAVG8.yaw_att.monomial.order_appl_sinu = OA.att_miss[2].polynomial.order_app
NAVG8.yaw_att.monomial.order_mono_sinu = OA.att_miss[2].polynomial.order_1st
NAVG8.yaw_att.monomial.mag_mono_sinu   = OA.att_miss[2].polynomial.mag           * 1.d7
NAVG8.yaw_att.monomial.phase_ang_sinu  = OA.att_miss[2].polynomial.phs_ang       * 1.d7
NAVG8.yaw_att.monomial.ang_from_epoch  = OA.att_miss[2].polynomial.ang_frm_epoch * 1.d7
;NAVG.roll_misalgn  = OA.att_miss[3]
NAVG8.roll_misalgn.exp_mag            = OA.att_miss[3].exp_mag * 1.d7
NAVG8.roll_misalgn.exp_time_const     = OA.att_miss[3].exp_tim * 1.d2
NAVG8.roll_misalgn.mean_att_ang_const = OA.att_miss[3].mn_att_ang * 1.d7
NAVG8.roll_misalgn.num_sinu_per_angle = OA.att_miss[3].num_sin
NAVG8.roll_misalgn.num_mono_sinu      = OA.att_miss[3].num_mon_sin
NAVG8.roll_misalgn.sinusoid.mag_sinu        = OA.att_miss[3].sinusoids.magnitude * 1.d7
NAVG8.roll_misalgn.sinusoid.phase_ang_sinu  = OA.att_miss[3].sinusoids.phs_ang   * 1.d7
NAVG8.roll_misalgn.monomial.order_appl_sinu = OA.att_miss[3].polynomial.order_app
NAVG8.roll_misalgn.monomial.order_mono_sinu = OA.att_miss[3].polynomial.order_1st
NAVG8.roll_misalgn.monomial.mag_mono_sinu   = OA.att_miss[3].polynomial.mag           * 1.d7
NAVG8.roll_misalgn.monomial.phase_ang_sinu  = OA.att_miss[3].polynomial.phs_ang       * 1.d7
NAVG8.roll_misalgn.monomial.ang_from_epoch  = OA.att_miss[3].polynomial.ang_frm_epoch * 1.d7
;NAVG.pitch_misalgn = OA.att_miss[4]
NAVG8.pitch_misalgn.exp_mag            = OA.att_miss[4].exp_mag * 1.d7
NAVG8.pitch_misalgn.exp_time_const     = OA.att_miss[4].exp_tim * 1.d2
NAVG8.pitch_misalgn.mean_att_ang_const = OA.att_miss[4].mn_att_ang * 1.d7
NAVG8.pitch_misalgn.num_sinu_per_angle = OA.att_miss[4].num_sin
NAVG8.pitch_misalgn.num_mono_sinu      = OA.att_miss[4].num_mon_sin
NAVG8.pitch_misalgn.sinusoid.mag_sinu        = OA.att_miss[4].sinusoids.magnitude * 1.d7
NAVG8.pitch_misalgn.sinusoid.phase_ang_sinu  = OA.att_miss[4].sinusoids.phs_ang   * 1.d7
NAVG8.pitch_misalgn.monomial.order_appl_sinu = OA.att_miss[4].polynomial.order_app
NAVG8.pitch_misalgn.monomial.order_mono_sinu = OA.att_miss[4].polynomial.order_1st
NAVG8.pitch_misalgn.monomial.mag_mono_sinu   = OA.att_miss[4].polynomial.mag           * 1.d7
NAVG8.pitch_misalgn.monomial.phase_ang_sinu  = OA.att_miss[4].polynomial.phs_ang       * 1.d7
NAVG8.pitch_misalgn.monomial.ang_from_epoch  = OA.att_miss[4].polynomial.ang_frm_epoch * 1.d7
;=======================================================
;  Assign other parameters where possible
ew_offset = long(string(area(34:38)))
if (ew_offset eq 4129) then ew_offset = 4441
if (ew_offset eq 5973) then ew_offset = 6261
AREAG8.west_vis_elem = ew_offset
ns_offset = long(string(area(22:26)))
AREAG8.north_bound = ns_offset

;=======================================================
endelse


;MISSING OA FILE
if (rec(0) eq -1) then begin
    area=-1
    nav = -1
    images = fix(images)
    images(0) = -1
    file_move,file,'/home/kknapp/DATA/isccp/b1/missingb1oa/'+stripname(file),/overwrite
    print,'Missing OA file ... returning'
    goto,endd
endif
;
; convert values to useful info

f1 = strsplit(file,'.',/extract)
;print,f1
spc = f1(8)
year = f1(4)
sat = f1(3)
g = strsplit(sat,'-',/extract)
index = g(1)

EW_NAD_CY  = long(nav.nadir_offsets.EW_NAD_CY)
NS_NAD_CY  = long(nav.nadir_offsets.NS_NAD_CY)
NS_NAD_INC = long(nav.nadir_offsets.NS_NAD_INC)
EW_NAD_INC = long(nav.nadir_offsets.EW_NAD_INC)
name = tempfile()
openw,temp,name,/get_lun,/swap_endian
writeu,temp,nav.bcd_epoch_time
close,temp
bsd = bytarr(8)
openr,temp,name
readu,temp,bsd
close,temp
free_lun,temp
file_delete,name
bcd_time,bsd([4,3,2,1,7,6,5,0]),nye,nde,nhe,nme,sec,msec ;,/iout
se = double(float(sec) + msec / 1000.)
imc = long(nav.imc_flag)
ny = long(string(area(2:5)))
mon = fix(f1(5))
day = fix(f1(6))
nd = julday(mon,day,ny)-julday(1,1,ny)+1
nh = fix(f1(7)) / 100
nm = fix(f1(7)) mod 100
s = 0.d0
nhe = 0
elem_res = 16
line_res = 8

;
; assign values to the structures
cen_pix= (EW_NAD_CY*6136d0 + EW_NAD_INC) * 1d0 + 1.d0
cen_lin= (NS_NAD_CY*6136d0 + NS_NAD_INC) * 8d0 / 28d0 + 4.5d0
linres = line_res
pixres = elem_res
linang = 28.d-6 * 180.d0 / !dpi
pixang = 16.d-6 * 180.d0 / !dpi
sline = ns_offset
spix =  ew_offset

dlat = 0.087d
dlon = -79.62d
key = 1l
iimmcc = 01
instr = 1l

;imcnav,dlat,dlon,line1,pixel1,key,iimmcc,instr,NAV,AREA,$
;  glat=glat,glon=glon,rec=rec,file=file,grad=grad, $
;  rpy=rpy,spnax=spnax,ma=ma,iout=0

;spnax = spnax*180.d0/!dpi mod 360.d0
;if (glon lt (-180)) then glon = glon + 360.d0
;if (glon gt 180)    then glon = glon - 360.d0

;roll = rpy(0)
;pitch = rpy(1)
;yaw = rpy(2)
;
; ASSIGN PARAMETERS TO STRUCTURES
;
imginfo.frdate = (ny-1900)*1000+nd
imginfo.frtime = nh*10000l + nm*100l
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = nscan ;areastr.num_line
imginfo.npix = block - 6 ;areastr.num_elem
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

;att.roll = roll
;att.pitch = pitch
;att.yaw = yaw
;need to calculate the current spra and spdc first
;att.spra = spnax(0)
;att.spdc = spnax(1)
;att.rma = ma(0)
;att.pma = ma(1)
ATTstr = att

;geo_nav.lat = glat * 180.d0/!dpi 
;geo_nav.lon = glon * 180.d0/!dpi 
;geo_nav.radius = grad
;geo_nav.rgtasc = spnax(0)
;geo_nav.decl = spnax(1)
;geo_nav.GOESNEXT = 1
;geo_nav.rma = ma(0)
;geo_nav.pma = ma(1)
;GEOstr = geo_nav

chninfo.nchan = 5
chninfo.ch_name(0) = 'VSCHN'
chninfo.ch_name(1) = 'IRNIR'
chninfo.ch_name(2) = 'IRWVP'
chninfo.ch_name(3) = 'IRWIN'
chninfo.ch_name(4) = 'IRSPL'
if (sat eq 'GOE-12') then chninfo.ch_name(4) = 'IRCO2'
chninfo.ch_info = 'none'
CHNstr = chninfo

;=======================================================
;  Assign other parameters where possible
AREAG8.img_date = imginfo.frdate
AREAG8.img_time = imginfo.frtime
NAVG8.img_date   = imginfo.frdate
NAVG8.img_time   = imginfo.frtime*1000l
;=======================================================

nchan = 5
;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(144)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

;IMAGE INFORMATION
IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = long(IMGstr.frtime)
IMGinf(2) = MISSING     ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 1                  ;neither Imager or Sounder
IMGinf(14) = 256                ;# bins ... should add option for 10-bit data (1024) when 
                                ;data goes to 10-bit in June 2006

;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)

SATinf(*) = 'Missing'
sn = fix(s(1))
SATinf(i++) = 'GOES-'+s(1)
SATinf(i++) = 'Imager'

for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  GMS channel info will be FWHM bounds ... estimated from the
;  ISCCP web page

if (sat eq 'GOE-8') then begin
    goes8info  = ['0.53-0.73 um','3.78-4.03 um','6.47-7.03 um','10.2-11.2 um','11.5-12.5 um']
    for j=0,nchan-1 do begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes8info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes8info(3)
        if (chninfo.ch_name(j) eq 'IRSPL') then tmp = goes8info(4)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes8info(2)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes8info(1)
        SATinf(i++) = tmp
    endfor
endif else if (sat eq 'GOE-12') then begin
    goes12info = ['0.53-0.77 um','3.77-4.03 um','5.72-7.34 um','10.2-11.2 um','13.0-13.7 um']
    for j=0,nchan-1 do begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes12info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes12info(3)
        if (chninfo.ch_name(j) eq 'IRCO2') then tmp = goes12info(4)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes12info(2)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes12info(1)
        SATinf(i++) = tmp
    endfor
endif else begin
    print,'Unknown SATELLITE:',sat
    stop
endelse

nbin = 256
if (areag8.bytes_per_pixel eq 2) then nbin = 1024
noaacal = fltarr(2,nchan,nbin)

;FOR NOW IGNORE VIS CAL
chans = [1,2,3,4,5]
if (fix(s(1)) gt 11) then chans(4) = 6

noaacal(*) = -1.
for ichan=1,4 do begin
    mcidas_calibrate,areastr=areag8,/tableonly, $
                     temperature=temp,radiance=rad, $
                     channel=chans(ichan),debug=debug
    noaacal(0,ichan,*) = rad
    noaacal(1,ichan,*) = temp
endfor
if keyword_set(debug) then stop




; TEST GRID
TGRinf = MISSING

; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
ny = IMGstr.nlin


PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = linhdr.scnnum ;absolute scan line
PREinf(*,i++,*) = MISSING
PREinf(*,i++,*) = MISSING       ;year
PREinf(*,i++,*) = MISSING       ;doy
PREinf(*,i++,*) = MISSING       ;HHMM
PREinf(*,i++,*) = MISSING       ;Sec
PREinf(*,i++,*) = MISSING       ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = linhdr.sat    ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

temp = {b1unav: NAVG8, b1uarea:AREAG8}

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:temp,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------


endd:
end

;***
