;****I* idl/idl.b1read_csu_g6
;+
; NAME:
;      b1read_csu_g6
;
; PURPOSE:
;      To read ISCCP-B1 data created by the CSU SPC for GOES-6
;      data. 
;
; CATEGORY:
;      ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    NAVstr: The navigation header
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; USES
; * b1u_ij2ll.pro
; * geo_struct.pro
; * get_file_size.pro
; * itime.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_csu_g6.pro 22763 2010-09-16 12:16:03Z kknapp $
;       Created 11/14/2003 - Ken Knapp NCDC
;       Modified01/02/2004 - Ken Knapp, NCDC
;                            Inserted common blocks for use with
;                            navigation routines
;       Modified03/01/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/16/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;               05/03/2004   Added B1U functionality
;       *28Jun07 - Added robodoc documentation 
;-
; SOURCE

pro b1read_csu_g6,file=file, $  ;filename
                  images=images, $;return image information in the images variable
                  NAVstr=NAVstr,$ ;navigation header
                  GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                  ORBstr=ORBstr,$ ;keplarian orbital info
                  IMGstr=IMGstr,$ ;image size and scan geometry
                  ATTstr=ATTstr,$ ;satellite attitude info
                  TIMstr=TIMstr,$ ;temporal data
                  CHNstr=CHNstr,$ ;channel information
                  LINhdr=LINhdr,$ ;LINE HEADERs
                  TGRstr=TGRstr,$ ;the test grid data
                  B1VER=B1VER,$ ;returns revision info
                  FHDR=FHDR,$   ;returns file header
                  b1u=b1u, $
                  compress=compress, $
                  noaacal=noaacal

b1ver = '$Id: b1read_csu_g6.pro 22763 2010-09-16 12:16:03Z kknapp $'

; set up the structures
@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = ['I*-6*CSU']
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

; CHECK to see if the file is compressed
ICOMP = 0
if (strmid(file,strlen(file)-3,3) eq '.gz') then ICOMP=1
if keyword_set(compress) then icomp=1

;Define the Navigation header
nv={COMVAS,$
    MKHEAD:bytarr(44),$         ;STANDARD CSU HEADER BY M. WHITCOMB 
    DATTYP:intarr(2), $         ;
    MAXRLN:0,$                  ;maximum record length in all records in this file
    YEAR:0,$                    ;year of first scan line of image from satellite transmissio
    DAY:0,$                     ;DAY OF THE YEAR (1 TO 366) of first scan line in images
    HOUR:0,$                    ;GMT hour transmitted by satellite   
    MINUTE:0,$                  ;minute transmitted by satellite
    SECOND:0,$                  ;second transmitted by satellite
    PYR:0,$                     ;preparation year
    PDY:0, $                    ;day 
    IRVOFF:0,$                  ;east west visible pixel shift to line up with IR
    NCHANL:0,$                  ;number of channels included in this file (1,2,9 etc.)
    CHID:intarr(9),$            ;identifier of each channel (0=ir,1 to 8=vis, 9=average
    IDSAT:intarr(2), $          ;ID of satellite (1)=NUMBER, (2)=LETTER
    NPXAV:intarr(9),$           ;number of pixels added together to form the output  
                                ;usually 1 for IR, 32 or 64 for vis
    AVTY:intarr(2),$            ;avg proc. for ir/vis chan. IR:no avg usually, 
                                ;VIS: 0=no avrg, 1 call to VSUM in ATOB1
    SAMPLE:intarr(9), $         ;sample interval for each channel  
    AVINTV:intarr(9),$          ;averageing interval for each channel 
    NIRSCN:0,$                  ;NUMBER OF IR SCAN LINES  
    NVSCN:0,$                   ;NUMBER OF VIS SCAN LINES
    NIREL:0,$                   ;NUMBER OF IR ELEMENTS IN IR SCAN LINES
    NVSEL:0, $                  ;NUMBER OF VIS ELEMTENTS IN VIS SCAN LINES the 
                                ;following numbers are in units of full res. vis data
    NIR:0,$                     ;north IR scan number (first one)
    SIR:0,$                     ;south IR scan number (last)
    WIR:0,$                     ;west IR element number (first)
    EIR:0,$                     ;east IR element number (last)
    NVS:0,$                     ;north vis scan number 
    SVS:0,$                     ;south vis scan number 
    WVS:0,$                     ;west vis element number ( full resolution )
    EVS:0, $                    ;east vis element number 
    PROGCD:intarr(3,3),$        ;
    LFORM:0,$                   ;OUTPUT FORMAT (3=HED,DATA; 4=COMPRESSED MANY HED,DATA)
    CALFLG:intarr(11),$         ;
    NDON:intarr(2), $           ;number of scan lines output of each type (IR , VIS)
    IRSEN:0,$                   ;IR SENSOR NUMBER (1 OR 2) transmitted from the satellite
    NSOFF:0,$                   ;north south offset done a NESS before transmission
    EWOFF:0,$                   ;east west offset done at NESS   BETWEEN IR AND VIS
    XIRSTP:0,$                  ;STEP BETWEEN PIXELS OF IR IN EAST WEST
    YIRSTP:0,$                  ;STEP BETWEEN PIXELS OF IR IN NORTH SOUTH
    XVSSTP:0,$                  ; STEP BETWEEN PIXELS OF VIS IN EAST WEST
    YVSSTP:0, $                 ;STEP BETWEEN PIXELS OF VIS IN NORTH SOUTH
    IMGSCN:0,$                  ;
    LASTSN:0,$                  ; LAST SCAN LINE RECORDED TO OUTPUT UNIT
    BGSTIM:0,$                  ;
    ENGTIM:0,$                  ;
    NESVCD:0,$                  ;
    NESECD:0, $                 ;
    DUMMY:intarr(7),$           ;
    NAVFLG:0,$                  ;NAVIGATION FLAG, 0=NO NAV, 1=SAT TRANSMITED,
                                ;2=UPDATED BY LANDMARK 
    NAVPR4:lonarr(60),$         ;
    NAVPR2:intarr(16), $        ;
    LATN:0,$                    ;north latitude used to calc fiducials (degrees*10)
    LONW:0,$                    ;west longitude used to calc fiducals (degrees*10)
    DLAT:0,$                    ;latidude step used between fiducals (degrees*10) 
    DLON:0,$                    ;longitude step used between fiducals (degrees*10)
    NLAT:0,$                    ;number of step calculated (6 or 0)
    NLON:0, $                   ;number of steps calculated (6 or 0)
    LTN:intarr(6,6),$           ;full resolution line number coresponding to a lat, lon
    LNN:intarr(6,6), $          ;full resolution element number at lat lon
    CIRSTR:intarr(256),$        ;conversion between input count and stored byte for IR
    CIRENG:lonarr(256),$        ;conversion between stored byte and energy (W/m**2/ster)
    CVSSTR:intarr(64,8),$       ;conversion between input count for channel isen and stored byte
    CVSENG:lonarr(256), $       ;conversion between stored byte and energy for vis(W/m**2/ster)
    NESHED:intarr(64)}          ;
;from FORTRAN SOURCE CODE -------- 
;C       PROGCD  = program versions                                              
;C               (1,1) = VESION A LEVEL, (2,1)=YEAR, (3,1)=MONTH                 
;C               (1,2) = VERSION B1 LEVEL, (2,2)=YEAR, (3,2)=MONTH               
;C               (1,3) = VERSION B2 , (2,3)=YEAR, (3,3)=MONTH
;C       CALFLG = CALIBRATION TYPE FLAGES (0=NO ENTRY IN TABLES,                 
;C               1=NOMINAL VALUES, 2=NORMALIZED AT CSU,                          
;C               3=CALIBRATION CENTER UPDATED                                    
;C               AS OF MAY, 83 ONLY TYPE 1 IS AVAILABLE                          
;C               (-1) refers to CIRENG  the averaged IR counts                   
;C               (0)  refers to CIRSTR  the raw IR counts                        
;C               (1..8) refers to CVSSTR  the raw vis counts                     
;C               (9)  refers to CVSENG  the averaged vis counts                  
;C       NAVPR4 = navigation parameters I*4 PARAMS see NAVTR SUBROUTINE          
;C       1 SPVANG THE SPIN VECTOR ANGLE                                          
;C       2 RASCEN THE RIGHT ASCENSION OF SPIN AXIS                               
;C       3 DECLIN THE DECLINATION                                                
;C       4 ZETA,PITCH                                                            
;C       5 ROLL,RHO                                                              
;C       6 YAW,ETA                                                               
;C       7 SPINRA THE SPIN RATE                                                  
;C       8 IEDATE THE DATE OF NAVIGATION PARAMETER FORMATION                     
;C       9 IETIME THE TIME OF NAVIGATION PARAMETER FORMATION                     
;C       10 SEMIMA  THE SEMIMAJOR AXIS LENGTH OF THE ORBIT                       
;C       11 OECCEN  THE ORBIT ECCENTRICITY                                       
;C       12 ORBINC  THE ORBIT PLANE INCLINATION TO EQUITORIAL PLANE              
;C       13 OANOML  THE ORBIT ANOMALY                                            
;C       14 PERIGE  THE PERIGEE ANGLE                                            
;C       15 ASNODE  THE ASSENDING NODE ANGLE                                     
;C       16 DATE1   THE DATE OF THIS IMAGE                                       
;C       17 TAIME1  THE TIME OF THIS IMAGE                                       
;C       18 GRA1                                                                 
;C       19 GRA2                                                                 
;C       20 SPRA2   THE SPIN VECTOR ANGLE AT TIME 2                              
;C       21 SPDC2   THE DECLINATION AT TIME 2                                    
;C       28 TO 60   THE CHEBYSHEV POLYNOMIAL COEFICIENTS                         
;C       22      EMPTY                                                           
;C               THE FOLLOWING ARE USED FOR LANDMARK NAVIGATION                  
;C       23 BETA1   THE BETA COUNT (SUN ANGLE IN VIEW) AT TIME 1                 
;C       24 BETA2   THE BETA COUNT AT TIME 2                                     
;C       25 GAMAIN  THE INITIAL GAMA PIXEL COUNT ( NOT YET)                      
;C       26 GAMADJ  THE GAMA PIXEL COUNT ADJUSTMENT                              
;C       27 GAMDOT  THE TIME RATE CHANGE OF GAMA     
;;                            
;C       NAVPR2 = navigation parameter I*2 PARAMS                                
;C       1 TO 6   THE TIME OF BETA1  (YR,DAY,HOUR,MIN,SEC,CSEC)                  
;C       7       THE SCAN LINE NUMBER FOR BETA1                                  
;C       8 TO 13  THE TIME OF BETA2                                              
;C       14      THE SCAN LINE NUMBER FOR BETA2                                  
;C       15      NORTH SOUTH OFFSET                                              
;C       16      EAST WEST OFFSET BETWEEN VIS AND IR FROM TRANSMISSION           
;C               NOTE WHEN HANDLING I*4 VALUES ONE MUST BE CONSISTANT            
;C               WITH THE DEC, VAX CONVENSIONS, IF YOU DON'T HAVE                
;C               THE RIGHT COMPUTER, YOU MAY NEED TO CHANGE THE                  
;C               STORAGE FORMAT                                                  
; end FORTRAN SOURCE CODE ------------

; the hd8 header was designed for the tape format
; it indicates the following physical record contents
head8 = {h8,$
         hdrlen:0,$             ;length of header (bytes)
         nrec:0,$               ;number of logical records in this physical record
         nbytes:0,$             ;total bytes in physical record
         unknown:0,$            ;unknown
         recnum:0,$             ;physical record number
         spare:intarr(3)}       ;spares
;

; CURRENTLY am unsure of this head24 setup.... when copied from
; Garrett's code, am missing two variables, so not sure if all the
; names are correct.
head24 = {h24,$
          hdrlen:0,$            ;1     length of header (bytes)
          nelem:0,$             ;2     width of image
          nscan:0,$             ;3     height of image
          year:0,$              ;4     year
          day:0,$               ;5     day
          hour:0,$              ;6     hour
          min:0,$               ;7     minute
          sec:0,$               ;8     sec
          msec:0,$              ;9     millisecond 
          ir_res_scan:0,$       ;0     ir res. scan line number
          ir_res_scan_rel:0,$   ;1     ir scan line number with count starting at first line transmitted
          ns_samp:0,$           ;2     n-s sampling or averaging interval in full res coordinates
          ew_smp:0,$            ;3     e-w sampling or averaging interval
          east_edge:0,$         ;4     east edge
          west_edge:0,$         ;5     west edge
          unknown:0,$           ;6     65 or 66?
          ch_num:0,$            ;7     channel number (0=ir,1...8=full res vis, 9=average vis)
          unk2:0,$              ;8     CHECK FLAGS WITHIN DOC FOR NON DATA (GRAY WEDGE,BLACK) 
          npix:0,$              ;9     1,16,32 = number of pixels included in average
          nbytes:0,$            ;0     bytes / pixel
          unk3:0,$              ;1     0
          line_num:0,$
          tmp:intarr(2)}        ;2     full resolution line number
;        HED(20)=B(125+4)                                                        
;C               THE NORTH SOUTH OFFSET                                          
;        HED(21)=B(126+4)                                                        
;C               THE EAST WEST OFFSET                                            
;        HED(22)=IBCD(B(56+4),B(57+4))                                           
;C               THE LEFT HORIZON POINT                                          
;        HED(23)=IBCD(B(58+4),B(59+4))                                           
;C               THE RIGHT HORIZON POINT                                         
;        HED(24)=B(8+4)                                                          
          
          
; Get file information
; = file_info(file)
filesize=get_file_size(file,status,compress=icomp)

skip4 = 0
f = strsplit(file,'.',/extr)

; open file
openr,unit,file,/get_lun,compress=ICOMP ;,/swap_end

tryagain:
point_lun,unit,0

;for 1989 files, skip the first 4 bytes
if (f(4) eq '1989' or skip4 eq 1) then begin
    r = 0
    readu,unit,r
endif

; read navigation header

readu,unit,nv

; get file header info
if (arg_present(fhdr)) then begin
    point_lun,-unit,here
    point_lun,unit,0
    fhdr = bytarr(here)
    readu,unit,fhdr
endif

;image parameters
nrsize=0
nvax=0
if (nv.xirstp lt 1 or nv.xirstp gt 20) then begin
    if (nv.xvsstp lt 1 or nv.xvsstp gt 20) then begin
        dx = 12
    endif else dx = nv.xvsstp
endif else dx = nv.xirstp
if (nv.yirstp lt 1 or nv.yirstp gt 20) then begin
    if (nv.yvsstp lt 1 or nv.yvsstp gt 20) then begin
        dy = 12
    endif else dy = nv.yvsstp
endif else dy = nv.yirstp

nx=(nv.eir-nv.wir)/dx + 1
ny=(nv.sir-nv.nir+1)/dy + 1


if (nx lt 0 or ny lt 0 or $
    nx gt 3000 or ny gt 3000) then begin
    if (skip4 eq 0) then begin
        skip4 = 1
        goto,tryagain
    endif else stop
endif 

imageir=bytarr(nx,ny)
imagevs=bytarr(nx,ny)

ipzero=0
prefix=0
hd24=intarr(24)
hd8=intarr(8)
j=ny
i1=0
i2=nx-1
ji=0
jv=0
irchan = 0
vschan = 0
linhdr = replicate(head24,2,ny)
while not(eof(unit)) do begin
    readu,unit,head8
    nrec = head8.nrec
    in = bytarr(nx)
    for i=0,nrec-1 do begin
        readu,unit,head24

        if (head24.year lt 1950 or $
            head24.year gt 2050) then begin
            ;following clause included for:
            ;    ISCCP.B1.0.GOE-6.1984.04.21.1746.CSU
            if (head24.year eq 2984) then begin
                head24.year = 1984 
            endif else if (head24.year eq 2985) then begin
                head24.year = 1985
            endif else if (head24.year eq 1945 and head24.hdrlen eq 48) then begin
                head24.year=1985
            endif else if (head24.year eq 1585 and head24.hdrlen eq 48) then begin
                head24.year=1985
            endif else if (head24.year eq 3986 and head24.hdrlen eq 48) then begin
                head24.year=1986
            endif else stop     ;goto,cantread
        endif
        nread = head24.nelem - 48
        if (nread gt 0) then begin
            in = bytarr(nread)
            readu,unit,in
        endif else in(*) = 255
        if (head24.ch_num eq 0) then begin
            irchan = 1
            imageir(*,ji) = in(i1:i2)
            linhdr(0,ji) = head24
            ji=ji+1
        endif else if (head24.ch_num eq 9) then begin
            vschan = 1
            imagevs(*,jv) = in(i1:i2)
            linhdr(1,jv) = head24
            jv=jv+1
        endif else begin
            if (head24.ch_num eq (-1) and $
                head24.hour eq (-1)) then begin
                ;Missing data?
                qwerty = 1
            endif else begin
                goto,cantread
            endelse
        endelse            
    endfor
endwhile
close,unit
free_lun,unit

nchan = vschan + irchan
if (nchan eq 2) then begin
    images = [[[imageir]],[[imagevs]]]
    images=transpose(images,[2,0,1])
endif else begin
    images = imageir
    images = reform(images,1,nx,ny)
    linhdr = reform(linhdr(0,*))
endelse
NAVstr = nv

;set up common blocks for use with navigation
navpr2 = NAVstr(0).navpr2
navpr4 = NAVstr(0).navpr4
; set up kepler COMMON BLOCK
semima = double(NAVPR4(10-1))/100.
incl   = double(NAVPR4(12-1))/1000. 
eccen  = double(NAVPR4(11-1))/1.e6
meana  = double(NAVPR4(13-1))/1000.
rasc   = double(NAVPR4(15-1))/1000.
perigee= double(NAVPR4(14-1))/1000. 
edate  = yymmdd_2_yyjjj(NAVPR4(8-1)) ;YYMMDD
etime  = NAVPR4(9-1) ;HHMMSS 
msec = 0.0

if (semima lt 1.) then goto,cantread

;set up att COMMON BLOCK
roll  = double(NAVPR4(5-1)) /2.^21
pitch = double(NAVPR4(4-1)) /2.^21
yaw   = double(NAVPR4(6-1)) /2.^21
SPRA1 = double(NAVPR4(2-1)) /2.^21                                         
SPRA2 = double(NAVPR4(20-1))/2.^21                                        
SPDC1 = double(NAVPR4(3-1)) /2.^21                                         
SPDC2 = double(NAVPR4(21-1))/2.^21                                        
;set up image COMMON BLOCK
sline = NAVSTR.nir
spix  = NAVstr.wir
cen_lin = 7284 ;totscl * 4 = 1821 * 4
cen_pix = 7648 ;totsmp * 2 = 3822 * 2
linres = NAVstr.yirstp
pixres = NAVstr.xirstp
linang = 1.d0/728.586d0
pixang = 18.375d0/15288.d0 ; 2. * 9.1875 / 3822 / 4
IH1=NAVPR2(3-1)
IM1=NAVPR2(4-1)
SEC=double(NAVPR2(5-1))+double(NAVPR2(6-1))*.01
SPPER = 1.E-6*double(NAVPR4(7-1))
TEMP = 60l*IH1+IM1
TEMP=TEMP*60
TIME = TEMP + SEC
KSCAN=NAVPR2(7-1)
FRTIME = TIME - (KSCAN - 1)*SPPER

IY=(NAVPR2(1-1) mod 100)
TEMP=IY
TEMP=TEMP*1000l+NAVPR2(2-1)
DATE=TEMP
DATE1=DATE
IF (FRTIME lt 0) then begin
    FRTIME = 86400 + FRTIME
    DATE = DATE1 - 1.
endif
frdate = date
;set up green COMMON BLOCK
GRA1 = double(NAVPR4(18-1))/2.^21
GRA2 = double(NAVPR4(19-1))/2.^21

gra1 = 0.d0
gra2 = 0.d0

D = 13.d0*3600d0
gt1 = .01*double(NAVPR4(17-1)) ;time of this image in seconds
gt2 = gt1 + d 

;chebyshev polynomials
ch_x = double(NAVstr.navpr4(27:37))/2.d0^13
ch_y = double(NAVstr.navpr4(38:48))/2.d0^13
ch_z = double(NAVstr.navpr4(49:59))/2.d0^13

;
;Assign variables to structures
;

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
orbit.sper = spper
ORBstr = orbit

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = ny
imginfo.npix = nx
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

att.roll = roll
att.pitch = pitch
att.yaw = yaw
;need to calculate the current spra and spdc first
;att.spra = spnax(0)
;att.spdc = spnax(1)
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

sattim.d = d
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = spra1            ;-999.d0
sattim.spra2 = spra2            ;-999.d0
sattim.spdc1 = spdc1            ;-999.d0
sattim.spdc2 = spdc2            ;-999.d0
sattim.cheb_x = ch_x
sattim.cheb_y = ch_y
sattim.cheb_z = ch_z
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

chninfo.nchan = nchan
chninfo.ch_name(0) = 'IRWIN'
chninfo.ch_name(1) = 'VSCHN'
chninfo.ch_info = 'none'
CHNstr = chninfo



ATTstr = att

TGRID.n    = 6
TGRID.slat = NAVstr.latn / 10.
TGRID.slon = NAVstr.lonw / 10.
TGRID.dlat = NAVstr.dlat / 10.
TGRID.dlon = NAVstr.dlon / 10.
TGRID.pixl(0:TGRID.N-1,0:TGRID.N-1) = NAVstr.lnn
TGRID.line(0:TGRID.N-1,0:TGRID.N-1) = NAVstr.ltn
TGRstr=TGRID


; get first and last scan
nx = imgstr.npix
ny = imgstr.nlin
ii = [nx/2,nx/2]
jj = [1,ny-1]
b1u_ij2ll,ii,jj,lat,lon, $
          b1TIM=TIMstr,b1ATT=ATTstr, $
          b1ORB=ORBstr,b1IMG=IMGstr, $
          glat=glat,glon=glon,grad=grad


;CALIBRATION information for SMS & GOES 1-7

count=indgen(256)
noaacal = fltarR(2,nchan,256)
irloc = where(strtrim(chninfo.ch_name,2) eq 'IRWIN')
if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
    noaacal(1,irloc(0),0:176) = 330.-count(0:176)/2.
    noaacal(1,irloc(0),177:255)=418-count(177:255)
    wvl=11.e-4
    c = 1.44
    noaacal(0,irloc(0),*) = (1.195e-5)*(wvl^(-3))*(exp(c/wvl/noaacal(1,irloc(0),*))-1.)^(-1)
endif
irloc = where(strtrim(chninfo.ch_name,2) eq 'VSCHN')
if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
    noaacal(*,irloc(0),*) = -999.
endif

if keyword_set(debug) then stop



;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = itime(IMGstr.frtime/3600.)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = head24.npix             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder

;Satellite information
i=0
ipo = (strsplit(f(3),'-',/extr))[1]
SATinf(*) = 'Missing'
SATinf(i++) = 'GOES-'+(strsplit(f(3),'-',/extr))[1]
SATinf(i++) = 'VISSR'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  GOES-7 channels info will be FWHM bounds ... estimated from the
;  ISCCP web page
g7info = ['0.52-0.73 um','10.3-12.1 um']
g6info = ['0.52-0.70 um','10.4-12.1 um']
for j=0,nchan-1 do begin
    if (ipo eq '7') then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g7info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g7info(1)
    endif else if (ipo eq '6') then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g6info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g6info(1)       
    endif else begin
        message,'ERROR: Satellite ID does not match allowed:'+ipo
    endelse
    SATinf(i++) = tmp
endfor

;---------------------------
; Navigation information
; Keplarian information
;IMGDAT
NAVinf(0) = sline
NAVinf(1) = spix
NAVinf(2) = cen_lin*10l
NAVinf(3) = cen_pix*10l
NAVinf(4) = linres
NAVinf(5) = pixres
NAVinf(6) = linang*1.e9
NAVinf(7) = pixang*1.e9
NAVinf(8) = spper*1e6         ;Seconds per revolution*10^6

; Keplarian information
NAVinf(09) = 10
NAVinf(10) = edate
NAVinf(11) = etime
NAVinf(12) = msec
NAVinf(13) = semima *1e2
NAVinf(14) = incl   *1e3
NAVinf(15) = eccen  *1e6
NAVinf(16) = meana  *1e3
NAVinf(17) = rasc   *1e3
NAVinf(18) = perigee*1e3
;other Keplerian info
NAVinf(19:28) = MISSING
; other Keplerian info
NAVinf(29:38) = MISSING
; Attitude data
NAVinf(39) = roll * 1.e6
NAVinf(40) = pitch* 1.e6
NAVinf(41) = yaw  * 1.e6
; ATTFIT
NAVinf(42:48) = MISSING
; RECINFO & FITINFO
NAVinf(49:56) = MISSING
; POSDAT
NAVinf(57:129) = MISSING
NAVinf(57) = navpr4(15) 
NAVinf(58) = navpr4(16)
NAVinf(65) = navpr4(1)
NAVinf(66) = navpr4(2)
NAVinf(73) = navpr4(17)
NAVinf(86:96)  = navpr4(27:27)
NAVinf(97:107) = navpr4(38:48)
NAVinf(108:118)= navpr4(49:59)
NAVinf(125) = navpr4(19)
NAVinf(126) = navpr4(20)
NAVinf(129) = navpr4(18)
;LOCDAT
NAVinf(130:142) = MISSING
NAVinf(131) = 1
NAVinf(132) = glat(0)*1e6
NAVinf(133) = glon(0)*1e6
NAVinf(134) = grad(0)*1e3
NAVinf(137) = ny-1
NAVinf(138) = glat(1)*1e6
NAVinf(139) = glon(1)*1e6
NAVinf(140) = grad(1)*1e3
NAVinf(130) = MISSING
;MISC
NAVinf(143) = 0
NAVinf(144) = nv.irvoff
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = frdate
NAVinf(148) = frtime*1000.
NAVinf(149) = ny
NAVinf(150) = nx

NAVinf(151:199) = MISSING

; TEST GRID
TGRinf = MISSING
if (total(navstr.ltn) ne 0) then begin
    latstart=-50.
    lonstart = 175.
    dlat = -20.
    dlon = 20.
    TGRinf = lonarr(6+2*6*6)
    i=0
    TGRinf(i++) = 6             ;nrows
    TGRinf(i++) = 6             ;ncols
    TGRinf(i++) = latstart*10
    TGRinf(i++) = lonstart*10
    TGRinf(i++) = dlat*10
    TGRinf(i++) = dlon*10
    for j=0,5 do begin
        for k=0,5 do begin
            TGRinf(i++) = NAVstr.ltn(j,k)
            TGRinf(i++) = NAVstr.lnn(j,k)
        endfor
    endfor
endif


; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = linhdr.IR_RES_SCAN ;absolute scan line
PREinf(*,i++,*) = linhdr.year   ;year
PREinf(*,i++,*) = linhdr.day    ;doy
PREinf(*,i++,*) = linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = linhdr.sec    ;Sec
PREinf(*,i++,*) = linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING ;eastern edge of earth
PREinf(*,i++,*) = MISSING ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------
goto,endd

cantread:
file_move,file,getenv('ISCCPDIR')+'b1/failed-read/'+stripname(file)
print,'Bad file'
images = -1
endd:
end
;***
