;****I* idl/idl.b1read_csu_g7
;+
; NAME:
;    b1read_csu_g7
;
; PURPOSE:
;    To read ISCCP-B1 data created by the CSU SPC for GOES-7
;    data. 
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    NAVstr: Navigation header from B1 file
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    These common blocks were used initially, but were replaced by the
;    structures above.
;               KEPLER - keplarian orbit elements
;               ATT - Satellite attitude parameters
;               IMAGE - Image coordinate constants
;               GREEN - greenwich right ascensions/times
;
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        latlon2ij  ... sets up the navigation information
;
; ERROR:
;    There still seems to be an error in the navigation by ~1-3 pixels.
;
; USES
; * b1u_ij2ll.pro
; * b1u_ll2ij.pro
; * geo_struct.pro
; * get_file_size.pro
; * itime.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_csu_g7.pro 22764 2010-09-16 12:16:38Z kknapp $
;       Created 11/19/2003 - Ken Knapp NCDC
;       Modified01/02/2004 - Ken Knapp, NCDC
;                            Inserted common blocks for use with
;                            navigation routines
;       Modified03/01/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/16/2004   Added the option to return line headers
;                            Commented out the common blocks
;               03/30/2004   Added B1VER
;                            Removed common blocks
;               04/22/2004   Added B1U functionality
;     * 28Jun07 - Added robodoc header
;-
; SOURCE
pro b1read_csu_g7,file=file,$     ;images=images,area=area,nav=nav
                  images=images, $;return image information in the images variable
                  NAVstr=NAVstr,$ ;navigation header
                  GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                  ORBstr=ORBstr,$ ;keplarian orbital info
                  IMGstr=IMGstr,$ ;image size and scan geometry
                  ATTstr=ATTstr,$ ;satellite attitude info
                  TIMstr=TIMstr,$ ;temporal data
                  CHNstr=CHNstr,$ ;channel information
                  LINhdr=LINhdr,$ ;LINE HEADERs
                  TGRstr=TGRstr,$ ;the test grid data
                  B1VER=B1VER,$ ;returns revision info
                  FHDR=FHDR,$   ;returns file header
                  b1u=b1u, $
                  compress=compress, $
                  noaacal=noaacal

b1ver = '$Id: b1read_csu_g7.pro 22764 2010-09-16 12:16:38Z kknapp $'

@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = ['I*-7*CSU']
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

;get file information
; = file_info(file)
filesize=get_file_size(file,compress=icomp)

;Navigation Header
Nav={NavHdr,NHBUF:0,MKHEAD:bytarr(44),DATTYP:intarr(2), $
     MAXRLN:0,YEAR:0,DAY:0,HOUR:0,MINUTE:0,SECOND:0,PYR:0,PDY:0, $
     IRVOFF:0,NCHANL:0,CHID:intarr(9),IDSAT:intarr(2), $
     NPXAV:intarr(9),AVTY:intarr(2),SAMPLE:intarr(9), $
     AVINTV:intarr(9),NIRSCN:0,NVSCN:0,NIREL:0,NVSEL:0, $
     NIR:0,SIR:0,WIR:0,EIR:0,NVS:0,SVS:0,WVS:0,EVS:0, $
     PROGCD:intarr(3,3),LFORM:0,CALFLG:intarr(13),NDON:intarr(2), $
     IRSEN:0,NSOFF:0,EWOFF:0,XIRSTP:0,YIRSTP:0,XVSSTP:0,YVSSTP:0, $
     IMGSCN:0,LASTSN:0,BGSTIM:0,ENGTIM:0,NESVCD:0,NESECD:0, $
     DUMMY:intarr(7),NAVFLG:0,NAVPR4:lonarr(60),NAVPR2:intarr(16), $
     CENTSEC:0,LATN:0,LONW:0,DLAT:0,DLON:0,NLAT:0,NLON:0, $
     LTN:intarr(6,6),LNN:intarr(6,6), $
     CALVIR4:lonarr(3),CALVIR:intarr(2,4),CALVIS:intarr(7,8), $
     BDUM:bytarr(3),BIRAAA:bytarr(1024),DUMPAD:bytarr(37)}
;
; head24 and head8 are copied from the CSU G7 b1read file
;
; the hd8 header was designed for the tape format
; it indicates the following physical record contents
head8 = {h8,$
         hdrlen:0,$             ;length of header (bytes)
         nrec:0,$               ;number of logical records in this physical record
         nbytes:0,$             ;total bytes in physical record
         unknown:0,$            ;unknown
         recnum:0,$             ;physical record number
         spare:intarr(3)}       ;spares

; CURRENTLY am unsure of this head24 setup.... when copied from
; Garrett's code, am missing two variables, so not sure if all the
; names are correct.
head24 = {h24,$
          hdrlen:0,$            ;1     length of header (bytes)
          nelem:0,$             ;2     width of image
          nscan:0,$             ;3     height of image
          year:0,$              ;4     year
          day:0,$               ;5     day
          hour:0,$              ;6     hour
          min:0,$               ;7     minute
          sec:0,$               ;8     sec
          msec:0,$              ;9     millisecond 
          ir_res_scan:0,$       ;0     ir res. scan line number
          ir_res_scan_rel:0,$   ;1     ir scan line number with count starting at first line transmitted
          ns_samp:0,$           ;2     n-s sampling or averaging interval in full res coordinates
          ew_smp:0,$            ;3     e-w sampling or averaging interval
          east_edge:0,$         ;4     east edge
          west_edge:0,$         ;5     west edge
          unknown:0,$           ;6     65 or 66?
          ch_num:0,$            ;7     channel number (0=ir,1...8=full res vis, 9=average vis)
          unk2:0,$              ;8     CHECK FLAGS WITHIN DOC FOR NON DATA (GRAY WEDGE,BLACK) 
          npix:0,$              ;9     1,16,32 = number of pixels included in average
          nbytes:0,$            ;0     bytes / pixel
          unk3:0,$              ;1     0
          line_num:0,$
          tmp:intarr(2)}        ;2     full resolution line number
;        HED(20)=B(125+4)                                                        
;C               THE NORTH SOUTH OFFSET                                          
;        HED(21)=B(126+4)                                                        
;C               THE EAST WEST OFFSET                                            
;        HED(22)=IBCD(B(56+4),B(57+4))                                           
;C               THE LEFT HORIZON POINT                                          
;        HED(23)=IBCD(B(58+4),B(59+4))                                           
;C               THE RIGHT HORIZON POINT                                         
;        HED(24)=B(8+4)                                                          


;open file
icomp=0
if keyword_set(compress) then icomp=1
OPENR, unit, file, /GET_LUN, comp=icomp ;,/swap_end

; read navigation header
readu,unit,nav

; get file header info
if (arg_present(fhdr)) then begin
    point_lun,-unit,here
    point_lun,unit,0
    fhdr = bytarr(here)
    readu,unit,fhdr
endif

;Image parameters
nx=(nav.eir-nav.wir)/nav.xirstp+1
ny=(nav.sir-nav.nir+1)/nav.yirstp+1
nrsize=0
nvax=0
imageir=bytarr(nx,ny)
imagevs=bytarr(nx,ny)
ipzero=0
prefix=0
i1 = 0
i2 = nx-1
jv = 0
ji = 0
irchan = 0
vschan = 0

linhdr = replicate(head24,2,ny)

while not(eof(unit)) do begin
    readu,unit,head8
    nrec = head8.nrec
    in = bytarr(nx)
;    print,'Record:',nrec
    for i=0,nrec-1 do begin
        readu,unit,head24
        nread = head24.nelem - 48
        if (nread gt 0) then begin
            in = bytarr(nread)
            readu,unit,in
        endif else in(*) = 255
        if (head24.ch_num eq 8) then begin
            irchan = 1
            if (ji lt ny) then begin
                imageir(*,ji) = in(i1:i2)
                linhdr(0,ji) = head24
            endif
            ji=ji+1
        endif else if (head24.ch_num eq 13) then begin
            vschan = 1
            if (jv lt ny) then begin
                imagevs(*,jv) = in(i1:i2)
                linhdr(1,jv) = head24
            endif
            jv=jv+1
        endif else begin
            stop
        endelse
    endfor
endwhile
close,unit
free_lun,unit


nchan = vschan + irchan
if (nchan eq 2) then begin
    images = [[[imageir]],[[imagevs]]]
    images=transpose(images,[2,0,1])
endif else begin
    images = imageir
    images = reform(images,1,nx,ny)
    linhdr = reform(linhdr(0,*))
endelse
NAVstr = nav
;images = [[[imageir]],[[imagevs]]]
;images = transpose(images,[2,0,1])

;set up common blocks for use with navigation
navpr2 = NAVstr(0).navpr2
navpr4 = NAVstr(0).navpr4
; set up kepler COMMON BLOCK
semima = double(NAVPR4(10-1))/100.
incl   = double(NAVPR4(12-1))/1000. 
eccen  = double(NAVPR4(11-1))/1.e6

if (eccen gt 1.) then eccen = 0.

meana  = double(NAVPR4(13-1))/1000.
rasc   = double(NAVPR4(15-1))/1000.
perigee= double(NAVPR4(14-1))/1000. 
edate  = yymmdd_2_yyjjj(NAVPR4(8-1)) ;YYMMDD
etime  = NAVPR4(9-1) ;HHMMSS 
msec = 0.0
;set up att COMMON BLOCK
roll  = double(NAVPR4(5-1)) /2.^21
pitch = double(NAVPR4(4-1)) /2.^21
yaw   = double(NAVPR4(6-1)) /2.^21
SPRA1 = double(NAVPR4(2-1)) /2.^21                                         
SPRA2 = double(NAVPR4(20-1))/2.^21                                        
SPDC1 = double(NAVPR4(3-1)) /2.^21                                         
SPDC2 = double(NAVPR4(21-1))/2.^21                                        
;set up image COMMON BLOCK
sline = NAVSTR.nir
spix  = NAVstr.wir
cen_lin = 7284 ;totscl * 4 = 1821 * 4
cen_pix = 7648 ;totsmp * 2 = 3822 * 2
linres = NAVstr.yirstp
pixres = NAVstr.xirstp
linang = 1.d0/728.586d0
pixang = 18.375d0/15288.d0 ; 2. * 9.1875 / 3822 / 4
IH1=NAVPR2(3-1)
IM1=NAVPR2(4-1)
SEC=double(NAVPR2(5-1))+double(NAVPR2(6-1))*.01
SPPER = 1.E-6*double(NAVPR4(7-1))
TEMP = 60l*IH1+IM1
TEMP=TEMP*60
TIME = TEMP + SEC
KSCAN=NAVPR2(7-1)
FRTIME = TIME - (KSCAN - 1)*SPPER

IY=(NAVPR2(1-1) mod 100)
TEMP=IY
TEMP=TEMP*1000l+NAVPR2(2-1)
DATE=TEMP
DATE1=DATE
IF (FRTIME lt 0) then begin
    FRTIME = 86400 + FRTIME
    DATE = DATE1 - 1.
endif
frdate = date
;set up green COMMON BLOCK
GRA1 = double(NAVPR4(18-1))/2.^21
GRA2 = double(NAVPR4(19-1))/2.^21

gra1 = 0.d0
gra2 = 0.d0

D = 13.d0*3600d0
gt1 = .01*double(NAVPR4(17-1))
gt2 = gt1 + d 

;
;Assign variables to structures
;

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
orbit.sper = spper
ORBstr = orbit

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = ny
imginfo.npix = nx
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

att.roll = roll
att.pitch = pitch
att.yaw = yaw
;need to calculate the current spra and spdc first
;att.spra = spnax(0)
;att.spdc = spnax(1)
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

sattim.d = d
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = spra1
sattim.spra2 = spra2
sattim.spdc1 = spdc1
sattim.spdc2 = spdc2
sattim.cheb_x = navpr4(27:37)/(2.d^13) ;-999.d0;ch_x
sattim.cheb_y = navpr4(38:48)/(2.d^13) ;-999.d0;ch_y
sattim.cheb_z = navpr4(49:59)/(2.d^13) ;-999.d0;ch_z
sattim.cheb_a = -999.d0
if (eccen gt 0) then sattim.cheb_a = 1.0d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

chninfo.nchan = nchan
chninfo.ch_name(1) = 'VSCHN'
chninfo.ch_name(0) = 'IRWIN'
chninfo.ch_info = 'none'
CHNstr = chninfo


;Call navigation info to get satellite information
inlat = 0.0d0
inlon = -100.d0

b1u_ll2ij,inlat,inlon,i,j, $
          glat=glat,glon=glon,grad=grad, $
          b1TIMstr=TIMstr,b1ORBstr=ORBstr, $
          b1ATTstr=ATTstr,b1IMGstr=IMGstr

if (glon lt (-180)) then glon = glon + 360.d0
if (glon gt 180)    then glon = glon - 360.d0


ATTstr = att

; The test grid seems to be non-existant in this file format?
; The data is missing in the ltn and lnn variables.
TGRID.n    = 0
if (tgrid.n gt 0) then begin
    TGRID.slat = NAVstr.latn / 10.
    TGRID.slon = NAVstr.lonw / 10.
    TGRID.dlat = NAVstr.dlat / 10.
    TGRID.dlon = NAVstr.dlon / 10.
    TGRID.pixl(0:TGRID.N-1,0:TGRID.N-1) = NAVstr.lnn
    TGRID.line(0:TGRID.N-1,0:TGRID.N-1) = NAVstr.ltn
endif
TGRstr=TGRID


; get first and last scan

;call latlon for first and last scan in image
; get positions
b1u_ij2ll,1,imgstr.sline,lat,lon, $
          glat=glat1c,glon=glon1c,grad=grad1c, $
          b1TIMstr=TIMstr,b1ORBstr=ORBstr,b1ATTstr=ATTstr,b1IMGstr=IMGstr

b1u_ij2ll,1,imgstr.sline+IMGstr.nlin*IMGstr.linres,lat,lon, $
          glat=glat2c,glon=glon2c,grad=grad2c, $
          b1TIMstr=TIMstr,b1ORBstr=ORBstr,b1ATTstr=ATTstr,b1IMGstr=IMGstr


;CALIBRATION information for SMS & GOES 1-7

count=indgen(256)
noaacal = fltarR(2,nchan,256)
irloc = where(strtrim(chninfo.ch_name,2) eq 'IRWIN')
if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
    noaacal(1,irloc(0),0:176) = 330.-count(0:176)/2.
    noaacal(1,irloc(0),177:255)=418-count(177:255)
    wvl=11.e-4
    c = 1.44
    noaacal(0,irloc(0),*) = (1.195e-5)*(wvl^(-3))*(exp(c/wvl/noaacal(1,irloc(0),*))-1.)^(-1)
endif
irloc = where(strtrim(chninfo.ch_name,2) eq 'VSCHN')
if (irloc(0) ge 0 and irloc(0) lt nchan) then begin
    noaacal(*,irloc(0),*) = -999.
endif

if keyword_set(debug) then stop


;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = itime(IMGstr.frtime/3600.)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = head24.npix             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder

;Satellite information
i=0
SATinf(*) = 'Missing'
f = strsplit(file,'.',/extr)
ipo = (strsplit(f(3),'-',/extr))[1]
SATinf(i++) = 'GOES-'+ipo
SATinf(i++) = 'VISSR'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  GOES-7 channels info will be FWHM bounds ... estimated from the
;  ISCCP web page
g7info = ['0.52-0.73 um','10.3-12.1 um']
g6info = ['0.52-0.70 um','10.4-12.1 um']
for j=0,nchan-1 do begin
    if (ipo eq '7') then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g7info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g7info(1)
    endif else if (ipo eq '6') then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g6info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g6info(1)       
    endif else begin
        message,'ERROR: Satellite ID does not match allowed:'+ipo
    endelse
    SATinf(i++) = tmp
endfor


; Navigation information
; Keplarian information
i=0
NAVinf(*) = MISSING
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10.
NAVinf(3) = IMGstr.cen_pix * 10.
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = ORBstr.sper*1e6          ;Seconds per revolution*10^6

;print,i
NAVinf( 9) = 10
NAVinf(10) = ORBstr.ep_date
NAVinf(11) = ORBstr.ep_time
NAVinf(12) = ORBstr.ep_msec
NAVinf(13) = ORBstr.semima*100.
NAVinf(14) = ORBstr.incl * 1000.
NAVinf(15) = ORBstr.eccen*1e6
NAVinf(16) = ORBstr.meana*1.e3
NAVinf(17) = ORBstr.rgtasc*1.e3
NAVinf(18) = ORBstr.perigee*1.e3

;Second set of kepler elements
NAVinf(19:28) = MISSING
;Third set of kepler elements
NAVinf(29:38) = MISSING

;ATTDAT
NAVinf(39) = round(ATTstr.roll * 1.e6)
NAVinf(40) = round(ATTstr.pitch* 1.e6)
NAVinf(41) = round(ATTstr.yaw  * 1.e6)
;ATTFIT
NAVinf(42:48) = MISSING
;RECINFO
NAVinf(49:51) = MISSING
;FITINFO
NAVinf(52:56) = MISSING
;POSDAT
NAVinf(57) = navpr4(7)
NAVinf(58) = navpr4(8)
NAVinf(59) = MISSING
NAVinf(60) = MISSING
NAVinf(61) = MISSING
NAVinf(62) = MISSING
NAVinf(63) = MISSING
NAVinf(64) = MISSING
NAVinf(65) = navpr4(1)
NAVinf(66) = navpr4(2)
NAVinf(67) = navpr4(3)
NAVinf(68) = navpr4(4)
NAVinf(69) = navpr4(5)
NAVinf(70) = MISSING
NAVinf(71) = MISSING
NAVinf(72) = MISSING
NAVinf(73) = navpr4(17)
NAVinf(74) = MISSING
NAVinf(75) = MISSING
NAVinf(76:85) = MISSING
NAVinf(86:96) = navpr4(27:37)
NAVinf(97:107)= navpr4(38:48)
NAVinf(108:118)=navpr4(49:59)
NAVinf(119) = MISSING
NAVinf(120) = MISSING
NAVinf(121) = MISSING
NAVinf(122) = MISSING
NAVinf(123) = MISSING
NAVinf(124) = MISSING
NAVinf(125) = navpr4(19)
NAVinf(126) = navpr4(20)
NAVinf(127) = MISSING
NAVinf(128) = MISSING
NAVinf(129) = navpr4(18)
;LOCDAT
NAVinf(130) = 31
NAVinf(131) = IMGstr.sline
NAVinf(132) = round(glat1c*1e6)
NAVinf(133) = round(glon1c*1e6)
NAVinf(134) = round(grad1c*1e3)
NAVinf(137) = IMGstr.sline+IMGstr.nlin*IMGstr.linres
NAVinf(138) = round(glat2c*1e6)
NAVinf(139) = round(glon2c*1e6)
NAVinf(140) = round(grad2c*1e3)
;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = IMGstr.frdate
NAVinf(148) = IMGstr.frtime * 1000
NAVinf(149) = IMGstr.nlin
NAVinf(150) = IMGstr.npix
NAVinf(151:199) = MISSING



; TEST GRID
TGRinf = MISSING
if (total(navstr.ltn) ne 0) then begin
    latstart=-50.
    lonstart = 175.
    dlat = -20.
    dlon = 20.
    TGRinf = lonarr(6+2*6*6)
    i=0
    TGRinf(i++) = 6             ;nrows
    TGRinf(i++) = 6             ;ncols
    TGRinf(i++) = latstart*10
    TGRinf(i++) = lonstart*10
    TGRinf(i++) = dlat*10
    TGRinf(i++) = dlon*10
    for j=0,5 do begin
        for k=0,5 do begin
            TGRinf(i++) = NAVstr.ltn(j,k)
            TGRinf(i++) = NAVstr.lnn(j,k)
        endfor
    endfor
endif


; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = linhdr.IR_RES_SCAN ;absolute scan line
PREinf(*,i++,*) = linhdr.year   ;year
PREinf(*,i++,*) = linhdr.day    ;doy
PREinf(*,i++,*) = linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = linhdr.sec    ;Sec
PREinf(*,i++,*) = linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING ;eastern edge of earth
PREinf(*,i++,*) = MISSING ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------


;stop
endd:
end
;***
