;****I* idl/idl.b1read_csu_g8
;+
; NAME:
;    b1read_csu_g8
;
; PURPOSE:
;    To read ISCCP-B1 data created by the CSU ISCCP SPC for GOES-8
;    data. The data is in a format similar to the McIdas format,
;    except for the inclusion of a record header which was used in
;    writing data to tape.
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    AREAstr:Mcidas area header
;    NAVstr: Mcidas Navigation header
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    None.
;
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        imcnav     ... sets up the navigation structures [not NAVstr]
;
; USES
; * bcd_time.pro
; * bitprint.pro
; * bittest.pro
; * geo_struct.pro
; * get_file_size.pro
; * mcidas_calibrate.pro
; * navtorec.pro
; * range.pro
; * reset.pro
; * stripname.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_csu_g8.pro 22765 2010-09-16 12:17:19Z kknapp $
;       Created 11/16/2003 - Ken Knapp NCDC
;       Modified03/01/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/16/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;               04/26/2004   Added B1U
;    * 28Jun07 - Added robodoc header.
;-
; SOURCE

pro b1read_csu_g8,file=file, $    ;images=images,area=area,nav=nav
                  images=images, $;return image information in the images variable
                  AREAstr=AREAstr,$
                  NAVstr=NAVstr,$ ;navigation header
                  GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                  ORBstr=ORBstr,$ ;keplarian orbital info
                  IMGstr=IMGstr,$ ;image size and scan geometry
                  ATTstr=ATTstr,$ ;satellite attitude info
                  TIMstr=TIMstr,$ ;temporal data
                  CHNstr=CHNstr,$ ;channel information
                  LINhdr=LINhdr,$ ;LINE HEADERs
                  TGRstr=TGRstr,$ ;the test grid data
                  B1VER=B1VER, $ ;Returns revision information
                  FHDR=FHDR,$   ;returns file header
                  b1u=b1u, $
                  compress=compress, $
                  noaacal=noaacal

b1ver = '$Id: b1read_csu_g8.pro 22765 2010-09-16 12:17:19Z kknapp $'
;***
; set up the structures
@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = 'ISCCP.B1.*-8*CSU'
    file = dialog_pickfile(path='~/data/ISCCP-test/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

;get file information
; = file_info(file)
filesize=get_file_size(file,compress=icomp)

;Define GOES-8 header information
AREAstr = { G8AREA,               $
            area_status:0L,$    ; Area status
            version_num:0L,$    ; Area version number
            sat_id_num:0L,$     ; Satellite ID (SSS)
            img_date:0L,$       ; Year and Julian day (YYDDD)
            img_time:0L,$       ; Time of image (HHMMSS)
            north_bound:0L,$    ; Upper left line in Sat coords (Y-Coord)
            west_vis_elem:0L,$  ; Upper left element in Sat coords (X-Coord)
            z_coor:0L,$         ; Upper z-coord (Z-Coord)
            num_line:0L,$       ; Number of lines in image (Y-SIZE)
            num_elem:0L,$       ; Number of elememts in image (X-SIZE)
            bytes_per_pixel:0L, $ ; Number of bytes per data element
            line_res:0L,$       ; Line resolution (Y-RES)
            elem_res:0L,$       ; Element resolution (X-RES)
            num_chan:0L,$       ; Number of bands (Z-RES)
            num_byte_ln_prefix:0L,      $ ; Number of bytes in line prefix (multiple of 4)
            proj_num:0L,$       ; Project number
            creation_date:0L,$  ; Creation date (YYYDDD)
            creation_time:0L,$  ; Creation time (HHMMSS)
            sndr_filter_map:0L,$ ; Filter map for soundings
            img_id_num:0L,$     ; Image ID number
            id:lonarr(4),$      ; Reserved for radar appications
            comment:bytarr(32),$; 32 char comments
            pri_key_calib:0L,$  ; Calibration colicil (area number)
            pri_key_nav:0L,$    ; Primary navigation codicil  (data)
            sec_key_nav:0L,$    ; Secondary navigation codicil (nav)
            val_code:0L,$       ; Validity code
            pdl:lonarr(8),$     ; PDL in packed-byte format
            band8:0L,$          ; Where band-8 came from
            act_img_date:0L,$   ; Actual image start day (YYYDDD)
            act_img_time:0L,$   ; Actual image start time (HHMMSS)
            act_start_scan:0L,$ ; Actual start scan
            len_prefix_doc:0L,$ ; Length of prefix documentation (DOC)
            len_prefix_calib:0L,$;Length of prefix calibration (CAL)
            len_prefix_lev:0L,$ ; Length of prefix level (LEV)
            src_type:bytarr(4),$; Source type
            calib_type:bytarr(4),$;Calibration type
            avg_or_sample:0L,$  ; Data was averaged (0) or sampled (1)
            poes_signal:0L,$    ; LAC, GAC, HRPT
            poes_up_down:0L,$   ; POES ascending/descending
            orig_src_type:bytarr(4),$;Original source type of data
            reserved:lonarr(7)} ; Reserved (6 calib pointer)
;Navigation structure
IMGR_SIN = { IMGR_SIN, mag_sinu:0L, phase_ang_sinu:0L }
;
; Imager repeat monomial T.
;
IMGR_MON = { IMGR_MON,            $
             order_appl_sinu:0L,  $
             order_mono_sinu:0L,  $
             mag_mono_sinu:0L,    $
             phase_ang_sinu:0L,   $
             ang_from_epoch:0L    }
;
;  Imager repeat T.
;
IMGR_REP = { IMGR_REP,                        $
             exp_mag:0L,                      $
             exp_time_const:0L,               $
             mean_att_ang_const:0L,           $
             num_sinu_per_angle:0L,           $
             sinusoid:REPLICATE(IMGR_SIN,15), $
             num_mono_sinu:0L,                $
             monomial:REPLICATE(IMGR_MON,4)   }
NAVstr = { NAVstr,                          $
           nav_type:bytarr(4),              $
           ID:bytarr(4),                    $
           IMC_status:0L,                   $
           Flip:0L,                         $
           spare1:0L,                       $
           ref_long:0L,                     $
           ref_rad_dist:0L,                 $
           ref_lat:0L,                      $
           ref_orb_yaw:0L,                  $
           ref_att_roll:0L,                 $
           ref_att_pitch:0L,                $
           ref_att_yaw:0L,                  $
           epoch_time:BYTARR(8),            $ ; BCD_TIME
           start_time:0L,                   $
           IMC_corr_roll:0L,                $
           IMC_corr_pitch:0L,               $
           IMC_corr_yaw:0L,                 $
           ref_long_change:LONARR(13),      $
           ref_rad_dist_change:LONARR(11),  $
           sine_lat:LONARR(9),              $
           sine_orb_yaw:LONARR(9),          $
           solar_rate:0L,                   $
           exp_start_time:0L,               $
           roll_att:IMGR_REP,               $
           spare2:LONARR(10),               $
           more1:bytarr(4),                 $
           gvar1:bytarr(4),                 $
           pitch_att:IMGR_REP,              $
           yaw_att:IMGR_REP,                $
           spare3:LONARR(16),               $
           more2:bytarr(4),                 $
           gvar2:bytarr(4),                 $
           roll_misalgn:IMGR_REP,           $
           pitch_misalgn:IMGR_REP,          $
           img_date:0L,                     $
           img_time:0L,                     $
           instr:0L,                        $
           spare4:LONARR(9),                $
           ns_cyl:0L,                       $
           ew_cyl:0L,                       $
           ns_inc:0L,                       $
           ew_inc:0L,                       $
           more3:bytarr(4),                 $
           gvar3:bytarr(4),                 $
           spare5:LONARR(126),              $
           more4:bytarr(4),                 $
           gvar4:bytarr(4),                 $
           spare6:LONARR(127)               }

on_ioerror,notape
icomp=0
if keyword_set(compress) then icomp=1

openr,unit,file,/get_lun,comp=icomp

readu,unit,areastr
POINT_LUN, unit, AREAstr.SEC_KEY_NAV
ReadU, unit, NavStr

ipzero=areastr.pri_key_nav
nch=areastr.num_chan
nx=areastr.num_elem
ny=areastr.num_line
res=[areastr.elem_res,areastr.line_res]
prefix=areastr.num_byte_ln_prefix
bytes=areastr.bytes_per_pixel

hd=intarr(prefix/2)

if (prefix ne 80) then stop

; old header ... is WRONG
;hd = {val_code:0l,$
;      crc:bytarr(2),$
;      scan_stat:0l,$
;      time:bytarr(8),$
;      blkhdr:bytarr(30),$
;      other:intarr(16)}
;new header from CIRA web site ... does not work
;hd = {linesize:0l, $
;      line_num:0l, $
;      west:0l, $
;      nwide:0l, $
;      chan_id:0l, $
;      xave:0l, $
;      num_chan:0l, $
;      xstp:0l, $
;      val:0l, $
;      time:lonarr(2), $
;      blk:0l,$
;      other:lonarr(8)}
;
; This line prefix definition works!!
hd = {first:intarr(16), $
      time:lonarr(2), $
      last:lonarr(10)}


; TESTING
;record = {hdr:intarr(prefix/2),scan:bytarr(nch,nx)}
;input = {tape:bytarr(8),line:replicate(record,3)}
;point_lun,unit,ipzero
;for i=0,4 do begin
;    readu,unit,input
;    print,input.line.hdr([0,1,4,36,38])
;endfor
;stop

;in = intarr(40000l)

if (arg_present(fhdr)) then begin
    point_lun,unit,0
    fhdr = bytarr(ipzero)
    readu,unit,fhdr
endif

point_lun,unit,ipzero


dm=bytarr(nch,nx)
img=bytarr(nch,nx,ny)

linhdr = replicate(hd,ny)

tape=intarr(4)
;tape(0) = 8 == length of tape header
;tape(1) = nrec == # of scan lines for this tape record
;tape(2) = nbytes == # of bytes in this tape record
;tape(3) = UNKNOWN

point_lun,unit,ipzero
iline=0
line = {hdr:hd, $
        scan:dm}


;nftr = floor(ny/3.)
;rec = replicate(line,3)
;tp = {tape:intarr(4), $
;       rc:rec}
;tps = replicate(tp,nftr)
;readu,unit,tps
;stop
while not(eof(unit)) do begin
    readu,unit,tape
    nrec = tape(1)
    if (tape(0) ne 8) then goto,notape
    rec = replicate(line,nrec)
    readu,unit,rec
    img(*,*,iline:iline+nrec-1) = rec.scan
    linhdr(iline:iline+nrec-1) = rec.hdr
    iline = iline + nrec
endwhile
on_ioerror,null
goto,closeit

notape:
;##################################################
; THIS calculation looks nice, but it doesn't work!
;
; Looks like IPZERO is wrong
;# of full tape records
;nftr = floor(ny / 3.)
;# of tape records
;ntr = ceil(ny / 3.)
;# of lines in the last partial record
;nps = ny mod 3
;tape record length
;reclen = (nx * nch + 80)*3 + 8
;preclen= (nx * nch + 80)*nps + 8
;data length
;datalen = long(reclen)*long(nftr) + long(ntr-nftr)*long(preclen)
;find starting point
;newipzero = filesize - datalen
;if (newipzero lt 0) then stop

;##############################################
; This section attempts to find where the validity code of the header
; is valid, ... but for ny=1306, it finds 1304 ... looks like two
; lines are bad??
;
;point_lun,unit,10000
;len = (filesize - 10000)
;h = bytarr(len)
;readu,unit,h
; the header starts with [0, 4, 107]
; if this doesn't change, than the following should work
;compare = [0b,0b,4b,0b,107b,0b]
;nc = n_elements(compare)
;match = where(h eq compare(0),nmatch)
;if (nmatch eq 0) then goto,nomatch
;for ic = 1,nc-1 do begin
;    mm = where(h(match+ic) eq compare(ic),nmatch)
;    if (nmatch eq 0) then goto,nomatch
;    stop
;    match = match(mm)
;endfor
;
;nomatch:
;stop

;#######################################
; This method is close, .. I think that the tape record is sometimes
; split up... but is too difficult to figure out for now.
;
;point_lun,unit,10000
;h=intarr(20000)
;readu,unit,h
;gd = where(h eq 8,ngd)
;if (ngd eq 0) then stop
;newipzero = 10000 + gd(0)*2
;tape2 = intarr(2)
;tape1 = 0
;tape3 = intarr(3)
;point_lun,unit,newipzero
;iline=-1l
;while not(eof(unit)) do begin
;    readu,unit,tape2   
;    print,tape2
;    if (tape2(0) ne 8) then stop
;    for i=0,2 do begin
;        readu,unit,hd
;        print,'Time:',hd.time
;        iline=iline+1l
;        readu,unit,dm
;        img(*,*,iline) = dm
;    endfor
;    tape2o = tape2
;    readu,unit,tape2
;    print,tape2o,tape2,format='(4i8)'
;endwhile

;#####################################
; move bad files to failed-read directory and move on
CLOSE, unit
FREE_LUN, unit
out = getenv('ISCCPDIR')+'b1/failed-read/'+ $
  stripname(file)
print,'Error:',file
file_move,file,out
images = -1
goto,endd


closeit:
CLOSE, unit
FREE_LUN, unit

images = img

nav  = NAVstr
area = AREAstr

;
; SET UP PARAMETERS
;
EW_NAD_CY  = nav.ew_cyl
EW_NAD_INC = nav.ew_inc
NS_NAD_CY  = nav.ns_cyl
NS_NAD_INC = nav.ns_inc
bcd_time,nav.epoch_time,year,jday,hh,mm,sec,msec
nye = long(year) & nde = long(jday)
nhe = long(hh) & nme = long(mm)
se = double(sec) + msec / 1000.d0
imgdate = nav.img_date
imgtime = nav.img_time
ny = long(imgdate / 1000 ) + 1900l
nd = long(imgdate mod 1000)
nh = long(imgtime / 10000000)
nm = long((imgtime / 100000) mod 100)
s = double(imgtime mod 100000) / 1000.d0
IMC = long(bittest(nav.imc_status,7))
ew_offset = area.west_vis_elem
ns_offset = area.north_bound
line_res = area.line_res
elem_res = area.elem_res

cen_pix= (EW_NAD_CY*6136d0 + EW_NAD_INC) * 1d0 + 1.d0
cen_lin= (NS_NAD_CY*6136d0 + NS_NAD_INC) * 8d0 / 28d0 + 4.5d0
linres = line_res
pixres = elem_res
linang = 28.d-6 * 180.d0 / !dpi
pixang = 16.d-6 * 180.d0 / !dpi
sline = ns_offset
spix =  ew_offset

navtorec,NAV,rec

dlat = 0.087d
dlon = -79.62d
key = 1l
iimmcc = 0
instr = 1l
;imcnav,dlat,dlon,line1,pixel1,key,iimmcc,instr,NAVstr,AREAstr,$
;  glat=glat,glon=glon,rec=rec,file=file,grad=grad, $
;  rpy=rpy,spnax=spnax,ma=ma

;spnax = spnax*180.d0/!dpi mod 360.d0
;if (glon lt (-180)) then glon = glon + 360.d0
;if (glon gt 180)    then glon = glon - 360.d0

;roll = rpy(0)
;pitch = rpy(1)
;yaw = rpy(2)

;
; ASSIGN PARAMETERS TO STRUCTURES
;
imginfo.frdate = imgdate
imginfo.frtime = imgtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = areastr.num_line
imginfo.npix = areastr.num_elem
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

;att.roll = roll
;att.pitch = pitch
;att.yaw = yaw
;need to calculate the current spra and spdc first
;att.spra = spnax(0)
;att.spdc = spnax(1)
;att.rma = ma(0)
;att.pma = ma(1)
ATTstr = att

;geo_nav.lat = glat * 180.d0/!dpi 
;geo_nav.lon = glon * 180.d0/!dpi 
;geo_nav.radius = grad
;geo_nav.rgtasc = spnax(0)
;geo_nav.decl = spnax(1)
;geo_nav.GOESNEXT = 1
;geo_nav.rma = ma(0)
;geo_nav.pma = ma(1)
;GEOstr = geo_nav

chninfo.nchan = nch
nchan = nch
noaacal = fltarr(2,nchan,256)
noaacal(*) = -1.

if (nch eq 7) then begin
    chninfo.ch_name(0) = 'IRVAR'
    chninfo.ch_name(1) = 'VSCHN'
    chninfo.ch_name(2) = 'IRNIR'
    chninfo.ch_name(3) = 'IRWVP'
    chninfo.ch_name(4) = 'IRWIN'
    chninfo.ch_name(5) = 'IRSPL'
    chninfo.ch_name(6) = 'VSVAR'
    chninfo.ch_info = 'none'
endif else begin
    print,'Unknown channel order'
    print,nch,' channels'
    print,bitprint(area.sndr_filter_map)
    images=fix(images)
    images(0) = -1
    return
    stop
endelse
CHNstr = chninfo



nchan = nch

noaacal = fltarr(2,nchan,256)
;FOR NOW IGNORE VIS CAL
;chans = [1,2,3,4,5]

noaacal(*) = -1.
;for ichan=1,4 do begin
;    mcidas_calibrate,areastr=areastr,/tableonly, $
;                     temperature=temp,radiance=rad, $
;                     channel=chans(ichan)
;    noaacal(0,ichan+1,*) = rad
;    noaacal(1,ichan+1,*) = temp
;endfor


;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

;IMAGE INFORMATION
IMGinf(*) = MISSING
IMGinf(0) = area.img_date       ;IMGstr.frdate
IMGinf(1) = area.img_time       ;round(long(imgstr.frtime)/1000.)
IMGinf(2) = AREA.sat_id_num     ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = area.bytes_per_pixel ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = AREA.val_code
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 1                  ;neither Imager or Sounder

;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)

SATinf(*) = 'Missing'
sn = fix(s(1))
SATinf(i++) = 'GOES-'+s(1)
SATinf(i++) = 'Imager'

for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  GMS channel info will be FWHM bounds ... estimated from the
;  ISCCP web page

goes8info  = ['0.53-0.73 um','3.78-4.03 um','6.47-7.03 um','10.2-11.2 um','11.5-12.5 um','Vis Variability','IR Variability']

for j=0,nchan-1 do begin
    if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes8info(0)
    if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes8info(3)
    if (chninfo.ch_name(j) eq 'IRSPL') then tmp = goes8info(4)
    if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes8info(2)
    if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes8info(1)
    if (chninfo.ch_name(j) eq 'VSVAR') then tmp = goes8info(5)
    if (chninfo.ch_name(j) eq 'IRVAR') then tmp = goes8info(6)
    SATinf(i++) = tmp
endfor


; TEST GRID
TGRinf = MISSING

; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
ny = IMGstr.nlin

tm = linhdr.time
tm1 = byte(tm,0,8,ny) / 16
tm2 = byte(tm,0,8,ny) mod 16
year= tm1(3,*)*1000l + tm2(3,*)*100l + tm1(2,*)*10l + tm2(2,*)
doy = tm1(1,*)*100l  + tm2(1,*)*10l +  tm1(0,*)
hhmm =tm2(0,*)*1000l + tm1(7,*)*100l + tm2(7,*)*10l + tm1(6,*)
ss   =tm2(6,*)*10l + tm1(5,*)
msec =tm2(5,*)*100l + tm1(4,*)*10l + tm2(4,*)

PREinf(*,i++,*) = rebin(reform(linhdr.first(5),1,ny),nch,ny) ;relative scanline
PREinf(*,i++,*) = MISSING       ;absolute scan line
PREinf(*,i++,*) = MISSING
PREinf(*,i++,*) = rebin(year,nchan,ny) ;year
PREinf(*,i++,*) = rebin(doy ,nchan,ny) ;doy
PREinf(*,i++,*) = rebin(hhmm,nchan,ny) ;HHMM
PREinf(*,i++,*) = rebin(ss  ,nchan,ny) ;Sec
PREinf(*,i++,*) = rebin(msec,nchan,ny) ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

temp = {b1unav: NAVstr, b1uarea:AREAstr}

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:temp,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------


endd:
end
