;****I* idl/idl.b1read_csu_g91012
;+
; NAME:
;    b1read_csu_g91012
;
;
; PURPOSE:
;    To read ISCCP-B1 data created by the CSU ISCCP SPC for GOES-8
;    data. The data is in a format similar to the McIdas format,
;    except for the inclusion of a record header which was used in
;    writing data to tape.
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    AREAstr:Mcidas area header
;    NAVstr: Mcidas Navigation header
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    None.
;
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        imcnav     ... sets up the navigation structures [not NAVstr]
;
; USES
; * bcd_time.pro
; * bittest.pro
; * geo_struct.pro
; * get_file_size.pro
; * mcidas_calibrate.pro
; * navtorec.pro
; * range.pro
; * reset.pro
; * stripname.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_csu_g91012.pro 22766 2010-09-16 12:18:08Z kknapp $
;       Created 11/16/2003 - Ken Knapp NCDC
;       Modified03/01/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/15/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;               04/26/2004   Added B1U
;               05/25/2004   Adjusting the Imgtime (b1u.img(1)) calc ???
;   * 28Jun07 - Added robodoc header
;-
; SOURCE

pro b1read_csu_g91012,file=file,$ ;images=images,area=area,nav=nav
  images=images, $              ;return image information in the images variable
  AREAstr=AREAstr,$
  NAVstr=NAVstr,$               ;navigation header
  GEOstr=GEOstr,$               ;use with geo_ij2ll and geoll2ij
  ORBstr=ORBstr,$               ;keplarian orbital info
  IMGstr=IMGstr,$               ;image size and scan geometry
  ATTstr=ATTstr,$               ;satellite attitude info
  TIMstr=TIMstr,$               ;temporal data
  CHNstr=CHNstr,$               ;channel information
  LINhdr=LINhdr,$               ;LINE HEADERs
  TGRstr=TGRstr,$               ;the test grid data
  B1VER=B1VER,$                 ;returns revision info
  FHDR=FHDR,$                   ;returns file header
  b1u=b1u, $
  compress=compress, $
  debug=debug, $
  noaacal=noaacal

b1ver = '$Id: b1read_csu_g91012.pro 22766 2010-09-16 12:18:08Z kknapp $'

@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = ['I*-9*CSU','I*-10*CSU','I*-12*CSU']
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

;get file information
; = file_info(file)
filesize=get_file_size(file,compress=icomp)

;Define GOES-8 header information
;AREA Structure
AREAstr = { AREAg9,               $
            area_status:0L,$    ; Area status
            version_num:0L,$    ; Area version number
            sat_id_num:0L,$     ; Satellite ID (SSS)
            img_date:0L,$       ; Year and Julian day (YYDDD)
            img_time:0L,$       ; Time of image (HHMMSS)
            north_bound:0L,$    ; Upper left line in Sat coords (Y-Coord)
            west_vis_elem:0L,$  ; Upper left element in Sat coords (X-Coord)
            z_coor:0L,$         ; Upper z-coord (Z-Coord)
            num_line:0L,$       ; Number of lines in image (Y-SIZE)
            num_elem:0L,$       ; Number of elememts in image (X-SIZE)
            bytes_per_pixel:0L, $ ; Number of bytes per data element
            line_res:0L,$       ; Line resolution (Y-RES)
            elem_res:0L,$       ; Element resolution (X-RES)
            num_chan:0L,$       ; Number of bands (Z-RES)
            num_byte_ln_prefix:0L,      $ ; Number of bytes in line prefix (multiple of 4)
            proj_num:0L,$       ; Project number
            creation_date:0L,$  ; Creation date (YYYDDD)
            creation_time:0L,$  ; Creation time (HHMMSS)
            sndr_filter_map:0L,$ ; Filter map for soundings
            img_id_num:0L,$     ; Image ID number
            id:lonarr(4),$      ; Reserved for radar appications
            comment:bytarr(32),$; 32 char comments
            pri_key_calib:0L,$  ; Calibration colicil (area number)
            pri_key_nav:0L,$    ; Primary navigation codicil  (data)
            sec_key_nav:0L,$    ; Secondary navigation codicil (nav)
            val_code:0L,$       ; Validity code
            pdl:lonarr(8),$     ; PDL in packed-byte format
            band8:0L,$          ; Where band-8 came from
            act_img_date:0L,$   ; Actual image start day (YYYDDD)
            act_img_time:0L,$   ; Actual image start time (HHMMSS)
            act_start_scan:0L,$ ; Actual start scan
            len_prefix_doc:0L,$ ; Length of prefix documentation (DOC)
            len_prefix_calib:0L,$ ; Length of prefix calibration (CAL)
            len_prefix_lev:0L,$ ; Length of prefix level (LEV)
            src_type:bytarr(4),$ ;Source type
            calib_type:bytarr(4),$;Calibration type
            avg_or_sample:0L,$  ; Data was averaged (0) or sampled (1)
            poes_signal:0L,$    ; LAC, GAC, HRPT
            poes_up_down:0L,$   ; POES ascending/descending
            orig_src_type:bytarr(4),$;Original source type of data
            reserved:lonarr(7)} ; Reserved (6 calib pointer)
;Navigation structure
IMGR_SIN = { IMGR_SIN, mag_sinu:0L, phase_ang_sinu:0L }
;
; Imager repeat monomial T.
;
IMGR_MON = { IMGR_MON,            $
             order_appl_sinu:0L,  $
             order_mono_sinu:0L,  $
             mag_mono_sinu:0L,    $
             phase_ang_sinu:0L,   $
             ang_from_epoch:0L    }
;
;  Imager repeat T.
;
IMGR_REP = { IMGR_REP,                        $
             exp_mag:0L,                      $
             exp_time_const:0L,               $
             mean_att_ang_const:0L,           $
             num_sinu_per_angle:0L,           $
             sinusoid:REPLICATE(IMGR_SIN,15), $
             num_mono_sinu:0L,                $
             monomial:REPLICATE(IMGR_MON,4)   }
NAVstr = { NAVstr,                          $
           nav_type:bytarr(4),              $
           ID:bytarr(4),                    $
           IMC_status:0L,                   $
           Flip:0L,                         $
           spare1:0L,                       $
           ref_long:0L,                     $
           ref_rad_dist:0L,                 $
           ref_lat:0L,                      $
           ref_orb_yaw:0L,                  $
           ref_att_roll:0L,                 $
           ref_att_pitch:0L,                $
           ref_att_yaw:0L,                  $
           epoch_time:BYTARR(8),            $ ; BCD_TIME
           start_time:0L,                   $
           IMC_corr_roll:0L,                $
           IMC_corr_pitch:0L,               $
           IMC_corr_yaw:0L,                 $
           ref_long_change:LONARR(13),      $
           ref_rad_dist_change:LONARR(11),  $
           sine_lat:LONARR(9),              $
           sine_orb_yaw:LONARR(9),          $
           solar_rate:0L,                   $
           exp_start_time:0L,               $
           roll_att:IMGR_REP,               $
           spare2:LONARR(10),               $
           more1:bytarr(4),                 $
           gvar1:bytarr(4),                 $
           pitch_att:IMGR_REP,              $
           yaw_att:IMGR_REP,                $
           spare3:LONARR(16),               $
           more2:bytarr(4),                 $
           gvar2:bytarr(4),                 $
           roll_misalgn:IMGR_REP,           $
           pitch_misalgn:IMGR_REP,          $
           img_date:0L,                     $
           img_time:0L,                     $
           instr:0L,                        $
           spare4:LONARR(9),                $
           ns_cyl:0L,                       $
           ew_cyl:0L,                       $
           ns_inc:0L,                       $
           ew_inc:0L,                       $
           more3:bytarr(4),                 $
           gvar3:bytarr(4),                 $
           spare5:LONARR(126),              $
           more4:bytarr(4),                 $
           gvar4:bytarr(4),                 $
           spare6:LONARR(127)               }

blkheader = {bid:0b,$           
             wsize:0b,$
             wcount:0,$
             pid:0,$
             rflag:0b,$
             vernum:0b,$
             dval:0b,$
             ascii:0b,$
             spare:0b,$
             range:0b,$
             bcount:0,$
             spare2:0,$
             time:bytarr(8),$
             spare3:0l,$
             err:0}

; This line header structure HD describes the line header as defined
; in McIdas and GVAR documentation [e.g., Table 3-5 defines the
; BlockHeader structure]. However, the structure does not seem to
; match the data. It looks like there is a time code in the data, but
; not where it is supposed to be. Not sure what to do. Will leave it
; this way for now. I may learn something from other data sets, or
; maybe should contact Garrett for help.
;
; This header does not work.
;hd = {val_code:0,$
;      scnnum:0,$
;      crc:bytarr(2),$
;      scan_stat:0l,$
;      time:bytarr(8),$
;      blkhdr:blkheader,$
;      other:intarr(16)}
hd = {first:intarr(12), $
      time:lonarr(2), $
      last:lonarr(12)}

on_ioerror,problem

;icomp=0
;if keyword_set(compress) then icomp=1
NAVstr1=NAVstr
openr,unit,file,/get_lun,comp=icomp
readu,unit,areastr
POINT_LUN, unit, 256
ReadU, unit, NavStr1
;CHECK endian-ness
iswap=0
if (areastr.version_num ne 4) then begin
    iswap=1
    close,unit
    free_lun,unit
    openr,unit,file,/get_lun,/swap_endian,comp=icomp
    readu,unit,areastr
    if (areastr.version_num ne 4) then begin
        print,'Problem with AREA file from CSU.'
        print,'File does not appear to have correct format'
        stop
    endif
endif

if (areastr.sec_key_nav ne 256) then begin
    print,'B1READ_CSU_G891012.pro: Problem in hardcoded read. Nav header not at 256. Change lines above'
    stop
endif
POINT_LUN, unit, AREAstr.SEC_KEY_NAV
ReadU, unit, NavStr

;is AREAstr OK but NAVstr swapped? If so, how should the rest of the
;file be read?
if (navstr.img_date lt 0) then begin
;   close it up    
    close,unit
    free_lun,unit
;   reopen with "wrong" endian swapping
    if (iswap eq 0) then begin
        openr,unit,file,/get_lun,/swap_endian,comp=icomp
    endif else begin
        openr,unit,file,/get_lun,comp=icomp        
    endelse
    POINT_LUN, unit, AREAstr.SEC_KEY_NAV
    ReadU, unit, NavStr
;   close again
    close,unit
    free_lun,unit
;   Now open the "right" way
    if (iswap eq 0) then begin
        openr,unit,file,/get_lun,comp=icomp
    endif else begin
        openr,unit,file,/get_lun,comp=icomp,/swap_endian
    endelse
endif


ipzero=areastr.pri_key_nav
nch=areastr.num_chan
nx=areastr.num_elem
ny=areastr.num_line
res=[areastr.elem_res,areastr.line_res]
prefix=areastr.num_byte_ln_prefix
bytes=areastr.bytes_per_pixel

if (arg_present(fhdr)) then begin
    point_lun,unit,0
    fhdr = bytarr(ipzero)
    readu,unit,fhdr
endif

point_lun,unit,ipzero

if (prefix ne 0) then begin
    if (prefix eq 80) then begin
        if (bytes eq 1) then begin
            input = {header:hd,$
                     scan:bytarr(nch,nx)}
        endif else if (bytes eq 2) then begin
            input = {header:hd,$
                     scan:intarr(nch,nx)}            
        endif else begin 
            stop
        endelse
    endif else stop
endif else begin
    input = bytarr(nch,nx)
endelse
in = replicate(input,ny)

readu,unit,in

goto,noproblem
problem:
;Error in reading the file:
print,'--> I/O Error: ',stripname(file)
images = -1
close,unit
free_lun,unit
return
noproblem:
close,unit
free_lun,unit
on_ioerror,null

if (prefix ne 0) then begin
    images = in(*).scan 
    LINhdr = in(*).header
endif else begin
    images = in
endelse

if (bytes eq 2) then images = images / 32

area = areastr
nav = navstr
;
; SET UP PARAMETERS
;
EW_NAD_CY  = nav.ew_cyl
EW_NAD_INC = nav.ew_inc
NS_NAD_CY  = nav.ns_cyl
NS_NAD_INC = nav.ns_inc
bcd_time,nav.epoch_time,year,jday,hh,mm,sec,msec
nye = long(year) & nde = long(jday)
nhe = long(hh) & nme = long(mm)
se = double(sec) + msec / 1000.d0
imgdate = nav.img_date
imgtime = nav.img_time / 1000
ny = long(imgdate / 1000 ) + 1900l
nd = long(imgdate mod 1000)
nh = long(imgtime / 10000000)
nm = long((imgtime / 100000) mod 100)
s = double(imgtime mod 100000) / 1000.d0
IMC = long(bittest(nav.imc_status,7))
ew_offset = area.west_vis_elem
ns_offset = area.north_bound
line_res = area.line_res
elem_res = area.elem_res

cen_pix= (EW_NAD_CY*6136d0 + EW_NAD_INC) * 1d0 + 1.d0
cen_lin= (NS_NAD_CY*6136d0 + NS_NAD_INC) * 8d0 / 28d0 + 4.5d0
linres = line_res
pixres = elem_res
linang = 28.d-6 * 180.d0 / !dpi
pixang = 16.d-6 * 180.d0 / !dpi
sline = ns_offset
spix =  ew_offset

navtorec,NAV,rec

;dlat = 0.087d
;dlon = -79.62d
;key = 1l
;iimmcc = 0
;instr = 1l
;imcnav,dlat,dlon,line1,pixel1,key,iimmcc,instr,NAVstr,AREAstr,$
;  glat=glat,glon=glon,rec=rec,file=file,grad=grad, $
;  rpy=rpy,spnax=spnax,ma=ma

;spnax = spnax*180.d0/!dpi mod 360.d0
;if (glon lt (-180)) then glon = glon + 360.d0
;if (glon gt 180)    then glon = glon - 360.d0

;roll = rpy(0)
;pitch = rpy(1)
;yaw = rpy(2)

;
; ASSIGN PARAMETERS TO STRUCTURES
;
imginfo.frdate = imgdate
imginfo.frtime = imgtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = areastr.num_line
imginfo.npix = areastr.num_elem
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

;att.roll = roll
;att.pitch = pitch
;att.yaw = yaw
;;need to calculate the current spra and spdc first
;att.spra = spnax(0)
;att.spdc = spnax(1)
;att.rma = ma(0)
;att.pma = ma(1)
ATTstr = att

;geo_nav.lat = glat * 180.d0/!dpi 
;geo_nav.lon = glon * 180.d0/!dpi 
;geo_nav.radius = grad
;geo_nav.rgtasc = spnax(0)
;geo_nav.decl = spnax(1)
;geo_nav.GOESNEXT = 1
;geo_nav.rma = ma(0)
;geo_nav.pma = ma(1)
;GEOstr = geo_nav

f = strsplit(stripname(file),'.',/extr)

chninfo.nchan = nch
chninfo.ch_name(0) = 'VSCHN'
chninfo.ch_name(1) = 'IRNIR'
chninfo.ch_name(2) = 'IRWVP'
chninfo.ch_name(3) = 'IRWIN'
chninfo.ch_name(4) = 'IRSPL'
if (f(3) eq 'GOE-12') then chninfo.ch_name(4) = 'IRCO2'
chninfo.ch_name(5) = 'VSVAR'
chninfo.ch_name(6) = 'IRVAR'
chninfo.ch_info = 'none'
CHNstr = chninfo

nchan = nch

;Set up NOAA Cal
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)

nbin = 256
if (areastr.bytes_per_pixel eq 2) then nbin = 1024
noaacal = fltarr(2,nchan,nbin)

;FOR NOW IGNORE VIS CAL
chans = [1,2,3,4,5]
if (fix(s(1)) gt 11) then chans(4) = 6

noaacal(*) = -1.
for ichan=1,4 do begin
    mcidas_calibrate,areastr=areastr,/tableonly, $
                     temperature=temp,radiance=rad, $
                     channel=chans(ichan),goessat=fix(s(1))
    noaacal(0,ichan,*) = rad
    noaacal(1,ichan,*) = temp
endfor
if keyword_set(debug) then stop



;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(144)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

;IMAGE INFORMATION
IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = (IMGstr.frtime)
IMGinf(2) = AREA.sat_id_num     ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = area.bytes_per_pixel ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = AREA.val_code
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 1                  ;neither Imager or Sounder
IMGinf(14) = 256
if (areastr.bytes_per_pixel eq 2) then IMGinf(14) = 1024
;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)

SATinf(*) = 'Missing'
sn = fix(s(1))
SATinf(i++) = 'GOES-'+s(1)
SATinf(i++) = 'Imager'

for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  GMS channel info will be FWHM bounds ... estimated from the
;  ISCCP web page

goes9info  = ['0.52-0.74 um','3.78-4.04 um','6.50-7.00 um','10.2-11.2 um','11.6-12.5 um','Vis Variability','IR Variability']
goes10info = ['0.53-0.77 um','3.79-4.04 um','6.45-7.02 um','10.3-11.0 um','11.6-12.5 um','Vis Variability','IR Variability']
goes11info = ['0.53-0.79 um','3.78-4.03 um','6.47-7.04 um','10.2-11.2 um','11.6-12.5 um','Vis Variability','IR Variability']
goes12info = ['0.53-0.77 um','3.77-4.03 um','5.72-7.34 um','10.2-11.2 um','13.0-13.7 um','Vis Variability','IR Variability']
goes13info = ['0.54-0.71 um','3.73-4.08 um','5.90-7.28 um','10.2-11.1 um','13.0-13.7 um','Vis Variability','IR Variability']

for j=0,nchan-1 do begin
    if (sn eq 9) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes9info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes9info(3)
        if (chninfo.ch_name(j) eq 'IRSPL') then tmp = goes9info(4)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes9info(2)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes9info(1)
        if (chninfo.ch_name(j) eq 'VSVAR') then tmp = goes9info(5)
        if (chninfo.ch_name(j) eq 'IRVAR') then tmp = goes9info(6)
    endif else if (sn eq 10) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes10info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes10info(3)
        if (chninfo.ch_name(j) eq 'IRSPL') then tmp = goes10info(4)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes10info(2)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes10info(1)
        if (chninfo.ch_name(j) eq 'VSVAR') then tmp = goes10info(5)
        if (chninfo.ch_name(j) eq 'IRVAR') then tmp = goes10info(6)
    endif else if (sn eq 11) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes11info(0)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes11info(1)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes11info(2)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes11info(3)
        if (chninfo.ch_name(j) eq 'IRSPL') then tmp = goes11info(4)
        if (chninfo.ch_name(j) eq 'VSVAR') then tmp = goes11info(5)
        if (chninfo.ch_name(j) eq 'IRVAR') then tmp = goes11info(6)        
    endif else if (sn eq 12) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes12info(0)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes12info(1)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes12info(2)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes12info(3)
        if (chninfo.ch_name(j) eq 'IRCO2') then tmp = goes12info(4)
        if (chninfo.ch_name(j) eq 'VSVAR') then tmp = goes12info(5)
        if (chninfo.ch_name(j) eq 'IRVAR') then tmp = goes12info(6)
    endif else if (sn eq 13) then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = goes12info(0)
        if (chninfo.ch_name(j) eq 'IRNIR') then tmp = goes12info(1)
        if (chninfo.ch_name(j) eq 'IRWVP') then tmp = goes12info(2)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = goes12info(3)
        if (chninfo.ch_name(j) eq 'IRCO2') then tmp = goes12info(4)
        if (chninfo.ch_name(j) eq 'VSVAR') then tmp = goes12info(5)
        if (chninfo.ch_name(j) eq 'IRVAR') then tmp = goes12info(6)
     endif else begin
        stop
    endelse
    SATinf(i++) = tmp
endfor



; TEST GRID
TGRinf = MISSING

; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
ny = IMGstr.nlin

tm = linhdr.time
tm1 = byte(tm,0,8,ny) / 16
tm2 = byte(tm,0,8,ny) mod 16
year= tm1(3,*)*1000l + tm2(3,*)*100l + tm1(2,*)*10l + tm2(2,*)
doy = tm1(1,*)*100l  + tm2(1,*)*10l +  tm1(0,*)
hhmm =tm2(0,*)*1000l + tm1(7,*)*100l + tm2(7,*)*10l + tm1(6,*)
ss   =tm2(6,*)*10l + tm1(5,*)
msec =tm2(5,*)*100l + tm1(4,*)*10l + tm2(4,*)
PREinf(*,i++,*) = MISSING    ;relative scanline
PREinf(*,i++,*) = rebin(reform(linhdr.first(1),1,ny),nch,ny) ;absolute scan line
PREinf(*,i++,*) = MISSING
PREinf(*,i++,*) = rebin(year,nchan,ny) ;year
PREinf(*,i++,*) = rebin(doy ,nchan,ny) ;doy
PREinf(*,i++,*) = rebin(hhmm,nchan,ny) ;HHMM
PREinf(*,i++,*) = rebin(ss  ,nchan,ny) ;Sec
PREinf(*,i++,*) = rebin(msec,nchan,ny) ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

temp = {b1unav: NAVstr, b1uarea:AREAstr}

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:temp,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------


endd:
end
;***
