;****I* idl/idl.b1read_csu_met3
;+
; NAME:
;    b1read_csu_met3
;
; PURPOSE:
;    To read ISCCP-B1 data created by CSU for Meteosat data
;    This Meteosat data covered the GEO gap for the Americas
;    prior to GOES-8.
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    NAVstr: Navigation info
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    None.
;
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;
; USES
; * ftime.pro
; * geo_struct.pro
; * get_file_size.pro
; * gha.pro
; * itime.pro
; * plot_tle.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_csu_met3.pro 22767 2010-09-16 12:20:12Z kknapp $
;       Created 01/15/2004 - Ken Knapp NCDC
;       Modified03/04/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/30/2004   Added B1VER
;
;-
; SOURCE

pro b1read_csu_met3,file=file, $ ;images=images,
                    NAVstr=NAVstr,$ ;nav=nav
                    images=images, $ ;return image information in the images variable
                    GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                    ORBstr=ORBstr,$ ;keplarian orbital info
                    IMGstr=IMGstr,$ ;image size and scan geometry
                    ATTstr=ATTstr,$ ;satellite attitude info
                    TIMstr=TIMstr,$ ;temporal data
                    CHNstr=CHNstr,$ ;channel information
                    TGRstr=TGRstr,$ ;the test grid data
                    B1VER=B1VER,$ ;returns revision info
                    FHDR=FHDR,$   ;returns file header
                    b1u=b1u, $
                    compress=compress, $
                    metcal=metcal

b1ver = '$Id: b1read_csu_met3.pro 22767 2010-09-16 12:20:12Z kknapp $'

; set up the structures
@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    file = dialog_pickfile(path='~/data/ISCCP-test',filter='*MET-*CSU')
    print,file
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

; = file_info(file)
filesize=get_file_size(file,comp=icomp)

filhdr = {unk1:0,$
        year:0,$                ;4-digit year
        jday:0,$                ;day of year
        time:0,$                ;HHMM (UTC)
        unk2:intarr(7),$        ;??
        sat:bytarr(8),$         ;8 character string
        unk3:intarr(35)}

rechdr=intarr(4)

scan = {hdr:intarr(8),$
        line:bytarr(1250)}


icomp=0
if keyword_set(compress) then icomp=1

openr,unit,file,/get_lun,comp=icomp ;,/swap_end
readu,unit,filhdr

if (arg_present(fhdr)) then begin
    point_lun,-unit,here
    fhdr = bytarr(here)
    point_lun,unit,0
    readu,unit,fhdr
endif

nchan = filhdr.unk2(3)
image = bytarr(1250,nchan,1250)
icount=0
ok = 1
while (not(eof(unit)) and ok) do begin
    readu,unit,rechdr
;    if (rechdr(1) ne 12) then begin
        ;for some reason, the last record is often missing a line or 
        ;more so this just skips the last line since it is generally
        ;zeros ... because it is off the full disk.
;        point_lun,-unit,here
;        left = (filesize - here) / 1266.
;        if (left ne rechdr(1)) then goto,endd
;        data=replicate(scan,left)
;        ns = left / nchan
;    endif else begin
;        nline = rechdr(1)
;        ns=nline/nchan
;        data=replicate(scan,nchan,ns)
;    endelse
    if (rechdr(1) eq 12) then begin
        nline = rechdr(1)
        ns=nline/nchan
        data=replicate(scan,nchan,ns)
        readu,unit,data
        image(*,*,icount:icount+ns-1) = data.line
        icount=icount+ns
        old=data
    endif else begin
        ok = 0
    endelse
endwhile
endd:
close,unit
free_lun,unit

nav=filhdr
images = transpose(image,[1,0,2])


;date/time
f=strsplit(file,'.',/extr)
year = long(f(4))
mon = long(f(5))
day = long(f(6))
time = long(f(7))
hh = time / 100l
mm = time mod 100l
frtime = hh*10000l + mm*100l
yymmdd = (year-1900)*10000l + mon*100l + day
frdate = yymmdd_2_yyjjj(yymmdd)


; THE Meteosat files do not provide any keplarian elements.
; However, they could be retrieved from the TLE files in the future.
orbit.ep_date = -999.           ;edate
orbit.ep_time = -999.           ;etime
orbit.ep_msec = -999.           ;msec
orbit.semima = -999.            ;semima
orbit.incl = -999.              ;incl
orbit.eccen = -999.             ;eccen
orbit.meana = -999.             ;meana
orbit.rgtasc = -999.            ;rasc
orbit.perigee = -999.           ;perigee
orbit.sper = -999.              ;spper
ORBstr = orbit

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = 1250.5         ;cen_lin
imginfo.cen_pix = 1250.5         ;cen_pix
imginfo.linres = 2              ;linres
imginfo.pixres = 2              ;pixres
imginfo.nlin = 1250             ;ny
imginfo.npix = 1250             ;nx
imginfo.sline = 0               ;sline
imginfo.spix = 0                ;spix
imginfo.linang = 0.0072d0       ;linang
imginfo.pixang = 0.0072d0       ;pixang
IMGstr = imginfo

ATTstr = att


chninfo.nchan = nchan
chninfo.ch_name(0) = 'VSCHN'
chninfo.ch_name(1) = 'IRWIN'
chninfo.ch_name(2) = 'IRWVP'
if (nchan eq 2) then begin
    chninfo.ch_name(0) = 'IRWIN'
    chninfo.ch_name(1) = 'IRWVP'
    chninfo.ch_name(2) = ' '
endif

if (f(3) eq 'MET-3' and f(4) eq '1992' and f(5) eq '12' and f(6) eq '30' and f(7) eq '1230') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1U.a.MET-3.1993.07.22.2030.CSU
if (f(3) eq 'MET-3' and f(4) eq '1993' and f(5) eq '07' and f(6) eq '22' and f(7) eq '2030') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1U.a.MET-3.1993.08.01.2030.CSU
if (f(3) eq 'MET-3' and f(4) eq '1993' and f(5) eq '08' and f(6) eq '01' and f(7) eq '2030') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1U.a.MET-3.1993.08.05.1130.CSU
if (f(3) eq 'MET-3' and f(4) eq '1993' and f(5) eq '08' and f(6) eq '05' and f(7) eq '1130') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1U.a.MET-3.1993.10.06.2130.CSU
if (f(3) eq 'MET-3' and f(4) eq '1993' and f(5) eq '10' and f(6) eq '06' and f(7) eq '2130') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1U.a.MET-3.1994.01.11.1530.CSU
if (f(3) eq 'MET-3' and f(4) eq '1994' and f(5) eq '01' and f(6) eq '11' and f(7) eq '1530') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1.0.MET-3.1993.04.24.2100.CSU
if (f(3) eq 'MET-3' and f(4) eq '1993' and f(5) eq '04' and f(6) eq '24' and f(7) eq '2100') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif
;ISCCP.B1.0.MET-3.1993.07.21.1130.CSU
;ISCCP.B1U.a.MET-3.1993.07.21.1130.CSU
if (f(3) eq 'MET-3' and f(4) eq '1993' and f(5) eq '07' and f(6) eq '21' and f(7) eq '1130') then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif

;the following files are special groups of files that are incorrect
;using the above tests.

ch_1_irwin = ['ISCCP.B1U.a.MET-3.1994.01.04.0030.CSU.gz', $
              'ISCCP.B1U.a.MET-3.1994.01.07.0030.CSU.gz', $
              'ISCCP.B1U.a.MET-3.1994.01.08.0030.CSU.gz', $
              'ISCCP.B1U.a.MET-3.1994.01.11.0330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.16.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.20.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.22.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.24.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.26.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.30.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.09.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.10.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.14.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.23.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.27.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.29.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.12.31.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.01.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.02.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.03.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.07.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.09.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.11.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.16.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.18.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.20.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.21.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.22.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.26.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.27.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.28.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.29.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.30.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.31.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.04.01.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.04.02.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.04.04.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.05.29.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.06.20.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.19.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.21.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.23.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.01.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.03.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.06.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.07.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.08.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.09.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.10.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.11.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.12.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.13.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.14.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.17.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.18.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.19.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.20.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.21.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.23.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.25.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.26.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.27.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.28.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.29.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.31.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.01.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.02.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.03.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.04.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.05.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.08.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.10.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.13.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.14.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.15.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.16.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.17.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.18.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.20.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.22.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.23.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.24.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.25.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.26.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.27.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.28.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.29.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.10.18.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.12.0030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.09.0530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.09.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.09.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.09.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.09.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.09.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.10.0230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.10.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.10.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.10.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.10.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.11.0230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.11.0530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.11.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.11.1430.CSU.gz']

ch_1_irwvp = ['ISCCP.B1U.a.MET-3.1992.09.27.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.10.08.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.10.09.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.10.12.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.10.19.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.19.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.02.05.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.08.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1995.01.17.1530.CSU.gz']

ch_2_vsirwin = ['ISCCP.B1U.a.MET-3.1992.10.07.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.10.17.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.12.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.18.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.19.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.21.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.28.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.29.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.29.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.29.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.29.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.30.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.11.30.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.04.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.04.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.10.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.17.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.18.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.18.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.19.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.19.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.19.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.19.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.20.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.20.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.20.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.20.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.21.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.21.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.21.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.21.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.22.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.22.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.22.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.29.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.30.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.30.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.30.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1992.12.30.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.02.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.02.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.04.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.04.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.05.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.05.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.06.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.07.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.08.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.09.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.09.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.11.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.11.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.11.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.12.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.12.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.12.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.19.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.20.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.20.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.20.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.21.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.22.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.23.1900.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.25.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.26.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.27.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.28.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.30.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.01.31.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.02.02.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.02.05.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.02.13.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.02.13.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.25.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.27.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.27.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.27.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.27.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.27.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.28.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.28.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.28.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.28.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.03.28.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.02.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.05.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.09.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.19.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.19.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.20.0230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.20.0530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.20.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.20.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.20.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.20.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.23.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.04.24.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.01.1200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.04.1200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.05.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.05.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.14.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.15.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.16.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.21.1200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.27.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.05.28.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.06.01.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.06.04.1200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.06.13.1200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.06.20.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.06.21.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.08.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.08.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.08.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.08.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.12.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.13.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.14.1500.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.15.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.19.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.20.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.20.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.20.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.20.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.20.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.21.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.21.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.21.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.21.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.21.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.22.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.22.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.22.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.22.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.22.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.23.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.23.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.23.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.23.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.23.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.24.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.24.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.24.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.24.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.24.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.25.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.25.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.25.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.25.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.25.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.26.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.26.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.26.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.26.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.26.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.27.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.27.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.27.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.27.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.27.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.28.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.28.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.28.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.28.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.28.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.29.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.29.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.29.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.30.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.30.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.30.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.30.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.31.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.31.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.31.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.31.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.07.31.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.01.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.01.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.01.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.01.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.01.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.02.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.02.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.02.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.02.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.02.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.03.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.03.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.03.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.03.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.03.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.04.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.04.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.04.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.04.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.04.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.05.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.05.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.05.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.05.2030.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.05.2330.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.06.1130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.06.1430.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.06.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.07.1800.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.08.18.2100.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.12.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.13.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.13.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.13.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.13.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.14.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.14.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.14.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.14.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.15.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.15.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.15.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.16.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.16.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.16.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.16.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.17.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.17.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.17.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.17.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.18.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.18.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.18.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.18.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.19.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.19.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.19.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.19.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.20.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.20.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.20.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.20.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.21.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.21.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.21.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.21.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.09.28.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.05.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.05.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.06.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.06.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.06.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.06.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.07.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.07.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.07.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.07.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.08.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.08.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.08.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.08.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.09.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.09.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.09.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.09.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.10.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.10.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.10.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.10.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.11.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.11.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.11.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.11.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.12.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.12.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.12.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.12.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.13.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.13.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.13.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.13.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.14.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.14.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.14.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.14.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.15.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.15.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.15.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.15.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.16.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.16.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.18.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.19.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.19.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.19.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.19.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.20.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.20.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.20.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.20.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.21.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.21.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.21.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.21.2130.CSU.gz',$ 
'ISCCP.B1U.a.MET-3.1993.10.22.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.22.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.22.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.22.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.23.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.23.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.23.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.23.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.24.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.24.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.24.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.24.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.25.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.25.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.25.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.25.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.26.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.26.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.26.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.26.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.27.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.27.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.27.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.27.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.10.28.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.01.2200.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.10.1300.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.20.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.20.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.20.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.21.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.21.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.21.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.21.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.22.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.22.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.22.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.11.22.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1993.12.23.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.05.1600.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.11.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.11.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.18.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.01.24.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.08.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.09.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.13.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.19.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.02.26.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.04.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.03.25.1730.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.04.10.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.04.30.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.05.18.2130.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.06.22.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.09.16.1830.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.10.10.1530.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.06.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.09.1230.CSU.gz',$
'ISCCP.B1U.a.MET-3.1994.11.24.2130.CSU.gz']

partname = strjoin(f([3,4,5,6,7,8]),'.')

if (           total(strpos(ch_2_vsirwin,partname) ge 0) gt 0) then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif else if (total(strpos(ch_1_irwvp,partname)   ge 0) gt 0) then begin
    chninfo.ch_name(0) = 'IRWVP'
endif else if (total(strpos(ch_1_irwin,partname)   ge 0) gt 0) then begin
    chninfo.ch_name(0) = 'IRWIN'
endif



chninfo.ch_info = 'none'
CHNstr = chninfo

;geo_nav.lat = 0.d0
;geo_nav.lon = -75.d0
;geo_nav.radius = 6.611d0*6378.144d ;grad
;geo_nav.rgtasc = 0.d0           ;spnax(0)
;geo_nav.decl = 90.0d0           ;spnax(1)
;geo_nav.GOESNEXT = 0
;geo_nav.rma = 0.0d0
;geo_nav.pma = 0.0d0
;GEOstr = geo_nav

TGRstr=TGRID

; GET orbit info from tle
plot_tle,tle,    $
  glat = glat1c, $
  glon = glon1c, $
  coord= coord,  $
  gra  = gra1c,  $
  sat  = strlowcase(f(3)), $
  year = year,   $
  jday = frdate mod 1000, $
  postime = ftime(frtime)*3600.+0.01,$
  ORB = TLEORB


reclon = 0.d0
;if (year gt 1997 and f(3) eq 'MET-5') then reclon =  63.0d0
;if (year gt 1993 and f(3) eq 'MET-3') then reclon = -75.0d0
;put glon1c within -180 to 180. deg
if (glon1c gt 180.) then glon1c = glon1c - 360.
;get reclon from glon1c
if   (abs(glon1c-  0.0) lt 3.)  then reclon =  0. else $
  if (abs(glon1c- 63.0) lt 3.) then reclon = 63.0 else $
  if (abs(glon1c+ 75.0) lt 3.) then reclon =-75.0 else begin
    RECLON = glon1c
    print,'Strange Normalized longitude:',glon1c
endelse

; Get Keplerian elements for optimally-perfect geostationary satellite
;     define the epoch time as the 00:00:00UTC on the date of the image
jday = frdate mod 1000
angle = gha(julday(1,jday,year,0,0,0),/degree)
; set up kepler elements
semima = 42166.264d0
eccen = 0.d0
incl = 0.d-7
meana = RECLON + angle 
perigee = 0.d0 
rgtasc = 0.d0 
; THE Meteosat files do not provide any keplarian elements.
; However, they could be retrieved from the TLE files in the future.
orbit.ep_date = frdate           ;edate
orbit.ep_time = 0l           ;etime
orbit.ep_msec = 0l           ;msec
orbit.semima = semima            ;semima
orbit.incl = incl              ;incl
orbit.eccen = eccen             ;eccen
orbit.meana = meana             ;meana
orbit.rgtasc = rgtasc            ;rasc
orbit.perigee = perigee           ;perigee
orbit.sper = 0.6              ;spper
ORBstr = orbit

metcal=fltarr(2,nchan,256)

;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = long(IMGstr.frtime)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = chninfo.nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 1                   ;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder

;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)
sn = fix(s(1))
SATinf(*) = 'Missing'
SATinf(i++) = 'Meteosat-'+s(1)
SATinf(i++) = 'Missing'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  METEOSAT channel info will be FWHM bounds ... calculated from the
;  ISCCP web page
metinfo = [['0.56-0.95 um','10.7-12.1 um','5.76-7.22 um'], $
           ['0.49-0.98 um','10.6-12.3 um','5.80-7.23 um'], $           
           ['0.50-0.93 um','10.7-11.9 um','5.78-6.85 um'], $           
           ['0.46-0.87 um','10.6-11.8 um','5.73-6.84 um'], $           
           ['0.47-0.86 um','10.7-12.0 um','5.77-6.81 um'], $           
           ['0.44-0.96 um','10.6-12.5 um','5.74-7.01 um']]         
for j=0,nchan-1 do begin
    if (chninfo.ch_name(j) eq 'VSCHN') then tmp = metinfo(0,sn-2)
    if (chninfo.ch_name(j) eq 'IRWIN') then tmp = metinfo(1,sn-2)
    if (chninfo.ch_name(j) eq 'IRWVP') then tmp = metinfo(2,sn-2)
    SATinf(i++) = tmp
endfor

; Navigation information
; Keplarian information
i=0
NAVinf(*) = MISSING    
;Image navigation information
NAVinf(0) = IMGstr.sline      ;area.ulline
NAVinf(1) = IMGstr.spix       ;area.ulelem
NAVinf(2) = IMGstr.cen_lin*10.    ;nav.piclin
NAVinf(3) = IMGstr.cen_pix*10.    ;nav.eletot / 2
NAVinf(4) = IMGstr.linres     ;area.lineres
NAVinf(5) = IMGstr.pixres     ;area.elemres
NAVinf(6) = -IMGstr.linang     *1.e9 
NAVinf(7) = -IMGstr.pixang     *1.e9
NAVinf(8) = 6*1.e7
;KEPSAT
NAVinf(9) = 15                  ;Kepler from "perfect" geostationary
NAVinf(10) = frdate
NAVinf(11) = 0l
NAVinf(12) = 0l
NAVinf(13) = semima *1.e2
NAVinf(14) = incl   *1.e3
NAVinf(15) = eccen  *1.e6
NAVinf(16) = meana  *1.e3
NAVinf(17) = rgtasc *1.e3
NAVinf(18) = perigee*1.e3
;KEPTLE
NAVinf(19) = 13                 ;Elements from nearest TLE
NAVinf(20) = TLEORB.EP_date
NAVinf(21) = TLEORB.EP_time
NAVinf(22) = TLEORB.EP_msec*1.e3
NAVinf(23) = TLEORB.semima *1.e2
NAVinf(24) = TLEORB.incl   *1.e3
NAVinf(25) = TLEORB.eccen  *1.e6
NAVinf(26) = TLEORB.meana  *1.e3
NAVinf(27) = TLEORB.rgtasc *1.e3
NAVinf(28) = TLEORB.perigee*1.e3
;Third set of kepler elements
NAVinf(29:38) = MISSING

;ATTDAT
NAVinf(39:41) = MISSING
;ATTFIT
NAVinf(42:48) = MISSING
;RECINFO
NAVinf(49) = 1
NAVinf(50) = 0
NAVinf(51) = reclon * 1.e3
;FITINFO
NAVinf(52:56) = MISSING
;POSDAT
NAVinf(57:129) = MISSING
NAVinf(65) =   0l
NAVinf(66) =  -90l*(2l^21)
NAVinf(125)=   0l
NAVinf(126)=  -90l*(2l^21)
;LOCDAT
NAVinf(130) = 31
NAVinf(131) = IMGstr.sline
NAVinf(132) = round(glat1c*1e6)
NAVinf(133) = round(glon1c*1e6)
NAVinf(134) = MISSING
NAVinf(137:140) = MISSING
;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = IMGstr.frdate
NAVinf(148) = ftime(IMGstr.frtime) * 1000. * 3600.
NAVinf(149) = IMGstr.nlin
NAVinf(150) = IMGstr.npix
NAVinf(151:199) = MISSING



; TEST GRID
TGRinf = MISSING


; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = MISSING       ;absolute scan line
PREinf(*,i++,*) = MISSING       ;CHNUM
PREinf(*,i++,*) = MISSING       ;linhdr.year   ;year
PREinf(*,i++,*) = MISSING       ;linhdr.day    ;doy
PREinf(*,i++,*) = MISSING       ;linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = MISSING       ;linhdr.sec    ;Sec
PREinf(*,i++,*) = MISSING       ;linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------
end
;***
