;****I* idl/idl.b1read_eum_met
;+
; NAME:
;    b1read_eum_met
;
; PURPOSE:
;    To read ISCCP-B1 data created by EUM for Meteosat data
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    None.
;
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;
; USES
; * ftime.pro
; * geo_struct.pro
; * get_file_size.pro
; * gha.pro
; * itime.pro
; * plot_tle.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
;       $Id: b1read_eum_met.pro 22768 2010-09-16 12:22:01Z kknapp $
;       Created 01/08/2004 - Ken Knapp NCDC
;       Modified03/04/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/16/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;               04/23/2004   Added B1U functionality
;               05/25/2004   Corrected b1u.img(1) ???
;
;-
; SOURCE 

pro b1read_eum_met,file=file,$    ;images=images
                   images=images, $ ;return image information in the images variable
                   GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                   ORBstr=ORBstr,$ ;keplarian orbital info
                   IMGstr=IMGstr,$ ;image size and scan geometry
                   ATTstr=ATTstr,$ ;satellite attitude info
                   TIMstr=TIMstr,$ ;temporal data
                   CHNstr=CHNstr,$ ;channel information
                   LINhdr=LINhdr,$ ;LINE HEADERs
                   TGRstr=TGRstr,$ ;the test grid data
                   B1VER=B1VER,$ ;returns revision info
                   FHDR=FHDR,$    ;returns file header
                   b1u=b1u, $
                   compress=compress, $
                   metcal=metcal, $
                   debug=debug

b1ver = '$Id: b1read_eum_met.pro 22768 2010-09-16 12:22:01Z kknapp $'

;reset global variables
reset

@geo_struct.pro


; Get a file name if none provided
if not(keyword_set(file)) then begin
    file = dialog_pickfile(path='~/data/ISCCP-test',filter='*MET-*')
    print,file
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

;date/time
f=strsplit(file,'.',/extr)
year = long(f(4))
mon = long(f(5))
day = long(f(6))
time = long(f(7))
hh = time / 100l
mm = time mod 100l
frtime = hh*10000l + mm*100l
yymmdd = (year-1900)*10000l + mon*100l + day
frdate = yymmdd_2_yyjjj(yymmdd)

jday = frdate mod 1000

;Ignore MET-7 data during move from prime meridian to 57.5 deg East
if (year eq 2006 and (jday ge 182 and jday le 304) and f(3) eq 'MET-7') then begin
    print,'B1READ_EUM_MET: Ignore MET-7 during move to IODC'
    images = -1
    return
endif


; = file_info(file)
filesize = get_file_size(file,comp=icomp)
;print,filesize,icomp
recl = 15202
nrec = filesize / recl
if (nrec eq 313) then nchan = 3 else $
  if (nrec eq 209) then nchan = 2 else $
  if (nrec eq 105) then nchan = 1 else $
  nchan = 0

; See note below on FASTREAD
;goto,fastread
; BEGIN "SLOW READ" ??
header = intarr(recl/2)
scan = {hdr:intarr(16/2),line:bytarr(1250)}

if (nchan eq 0) then begin
    print,'Invalid Meteosat file'
    print,'NREC=',nrec
    print,'FileSize=',filesize
    images=-1
    goto,endd
endif
;CPR = channels per record
cpr = 12/nchan
record = {hdr:intarr(8/2),data:replicate(scan,nchan,cpr),dummy:0}

linhdr = intarr(8,nchan,1250)
images = bytarr(1250,nchan,1250)

icomp = 0
if keyword_set(compress) then icomp=1

on_ioerror,badfile

openr,1,file,/swap_end,comp=icomp
readu,1,header

fhdr = byte(header,0,15202)

nl = 0
while not(eof(1)) do begin
    readu,1,record
    images(*,*,nl:nl+cpr-1) = record.data.line
    linhdr(*,*,nl:nl+cpr-1) = record.data.hdr
    nl += cpr
endwhile
close,1

on_ioerror,null

images = transpose(images,[1,0,2])
linhdr = transpose(linhdr,[1,2,0])
goto,done
; END "SLOW READ" ???

; Attempting to READ data faster
; BEGIN "FAST READ" ???
fastread:
;
;    The "FASTREAD" here did not work. It runs slower than the
;    original code. This may be due to some page faulting ... but
;    can't be sure. Should retest this algorithm on the new machine
;    which has more RAM. Until then, will not mess with I/O.

nc = nchan                      ;# of channels
nl = nrec * 12 / nc           ;# of lines

;Iarray = intarr(filesize/2)   ;Entire file
Barray = bytarr(filesize)
openr,1,file ;,/swap_end
readu,1,Barray                  ;Read entire file
close,1
;calculate Iarray
ind = lindgen(filesize/2)*2
Iarray = ishft(fix(Barray[ind]),8) + fix(Barray[ind+1])

; assign header
ind = lindgen(8,nc,nl)
ilin = (ind / 8) mod 12
irec = (ind / (8*12)) + 1
II = (Ind mod 8) + (633l*12l+5l)*(Irec-1)+633*Ilin + 7605l
header = Iarray(II)
; assign image
ind = lindgen(1250,nc,nl)
ilin = (ind / 1250) mod 12
irec = (ind / (1250l * 12)) + 1l
II = 15226l + (ind mod 1250) + 1266l*ilin + 15202l*(irec-1)
img = Barray(II)
images = transpose(img,[1,0,2])
linhdr = transpose(header,[1,2,0])

done:
; END   "FAST READ"??




;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = 1250.5         ;cen_lin
imginfo.cen_pix = 1249.5         ;cen_pix
imginfo.linres = 2              ;linres
imginfo.pixres = 2              ;pixres
imginfo.nlin = 1250             ;ny
imginfo.npix = 1250             ;nx
imginfo.sline = 0               ;sline
imginfo.spix = 0                ;spix
imginfo.linang = -0.0072d0       ;linang
imginfo.pixang = -0.0072d0       ;pixang
IMGstr = imginfo

;small correction as determined in talks with George Huffman
;Around MArch 28, 2008
;This will cause new B1U files to be created for all MET files.
imginfo.cen_pix=imginfo.cen_pix+4


att.spra = 0.
att.spdc = 90.
ATTstr = att


chninfo.nchan = nchan
if (nchan eq 2) then begin
    ;put in conditions for when the channels are incorrect
    if (f(3) eq 'MET-2' and f(4) eq '1985' and f(5) eq '03' and f(6) eq '28' and f(7) eq '1430') then begin
        chninfo.ch_name(0) = 'VSCHN'
        chninfo.ch_name(1) = 'IRWIN'
    endif else if (f(3) eq 'MET-2' and f(4) eq '1987' and f(5) eq '12' and f(6) eq '07' and f(7) eq '0830') then begin
        chninfo.ch_name(0) = 'VSCHN'
        chninfo.ch_name(1) = 'IRWIN'
    endif else begin
        chninfo.ch_name(0) = 'IRWIN'
        chninfo.ch_name(1) = 'IRWVP'
    endelse
endif else begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
    chninfo.ch_name(2) = 'IRWVP'
endelse

;the following files are special groups of files that are incorrect
;using the above tests.

ch_2_badbad = ['ISCCP.B1U.a.MET-3.1988.11.16.1930.ESA.gz',$
               'ISCCP.B1U.a.MET-3.1989.02.01.1130.ESA.gz',$
               'ISCCP.B1U.a.MET-3.1989.02.01.1430.ESA.gz',$
               'ISCCP.B1U.a.MET-3.1989.02.01.1730.ESA.gz']


ch_2_bad_irwin = ['ISCCP.B1U.a.MET-2.1983.09.15.1830.ESA.gz',$
                  'ISCCP.B1U.a.MET-2.1985.11.20.0830.ESA.gz',$
                  'ISCCP.B1U.a.MET-2.1987.09.09.0830.ESA.gz',$
                  'ISCCP.B1U.a.MET-2.1987.09.09.1130.ESA.gz',$
                  'ISCCP.B1U.a.MET-2.1988.05.09.1130.ESA.gz',$
                  'ISCCP.B1U.a.MET-2.1988.05.09.1430.ESA.gz']


ch_2_vsirwin = ['ISCCP.B1U.a.MET-2.1983.10.20.1730.ESA.gz',$
'ISCCP.B1U.a.MET-2.1983.10.22.1230.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.01.11.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.01.18.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.08.29.1430.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.11.01.0831.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.11.09.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.11.10.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1984.12.05.1130.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.02.13.0930.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.03.06.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.03.28.1430.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.04.16.1430.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.04.23.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.10.17.1130.ESA.gz',$
'ISCCP.B1U.a.MET-2.1985.11.12.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1986.01.29.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1986.04.07.0630.ESA.gz',$
'ISCCP.B1U.a.MET-2.1986.04.07.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.01.08.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.01.24.1430.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.01.28.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.02.15.1130.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.02.25.1130.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.05.04.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.05.06.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.07.01.0730.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.07.03.1430.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.09.29.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.12.07.0830.ESA.gz',$
'ISCCP.B1U.a.MET-2.1987.12.30.0830.ESA.gz',$
'ISCCP.B1U.a.MET-3.1988.11.16.0730.ESA.gz',$
'ISCCP.B1U.a.MET-3.1988.11.16.1030.ESA.gz',$
'ISCCP.B1U.a.MET-3.1988.11.16.1330.ESA.gz',$
'ISCCP.B1U.a.MET-3.1988.11.16.1630.ESA.gz',$
'ISCCP.B1U.a.MET-3.1989.02.28.1430.ESA.gz',$
'ISCCP.B1U.a.MET-4.1992.09.08.0730.ESA.gz']


partname = strjoin(f([3,4,5,6,7,8]),'.')

if (           total(strpos(ch_2_vsirwin,partname) ge 0) gt 0) then begin
    chninfo.ch_name(0) = 'VSCHN'
    chninfo.ch_name(1) = 'IRWIN'
endif else if (total(strpos(ch_2_badbad,partname)   ge 0) gt 0) then begin
    chninfo.ch_name(0) = 'BAD'
    chninfo.ch_name(1) = 'BAD'
endif else if (total(strpos(ch_2_bad_irwin,partname)   ge 0) gt 0) then begin
    chninfo.ch_name(0) = 'BAD'
    chninfo.ch_name(1) = 'IRWIN'
endif


chninfo.ch_info = 'none'
CHNstr = chninfo

reclon = 0.d0
if (year gt 1997 and f(3) eq 'MET-5') then reclon = 63.d0

;geo_nav.lat = 0.d0
;geo_nav.lon = 0.d0
;if (year gt 1997 and f(3) eq 'MET-5') then geo_nav.lon = 63.d0
;geo_nav.radius = 6.611d0*6378.144d ;grad
;geo_nav.rgtasc = 1.d-5           ;spnax(0)
;geo_nav.decl = 90.0d0           ;spnax(1)
;geo_nav.GOESNEXT = 0
;geo_nav.rma = 0.0d0
;geo_nav.pma = 0.0d0
;GEOstr = geo_nav

sattim.spdc1 = -90.0d0 
sattim.spdc2 = -90.0d0 
sattim.spra1 = 0.d0
sattim.spra2 = 0.d0
TIMstr = SATtim



; GET orbit info from tle
plot_tle,tle,    $
  glat = glat1c, $
  glon = glon1c, $
  coord= coord,  $
  gra  = gra1c,  $
  sat  = strlowcase(f(3)), $
  year = year,   $
  jday = frdate mod 1000, $
  postime = ftime(frtime)*3600. + 0.01,$
  ORB = TLEORB
TGRstr=TGRID
;define longitude to which the image was rectified
if   (abs(glon1c-  0.0) lt 25. or $
      abs(glon1c-360.0) lt 25)  then RECLON =   0 else $ ;Europe/Africa
  if (abs(glon1c- 63.0) lt 11.) then RECLON =  63 else $ ;Indian Ocean
  if (abs(glon1c+ 75.0) lt 10.) then RECLON = -75 else $ ;GOES-East
  if (year lt 1984) then             RECLON =   0 else $
  if ((year eq 1992 and f(3) eq 'MET-4') or $
      (year eq 1998 and f(3) eq 'MET-5' and mon eq 3) or $
      (year eq 1993 and f(3) eq 'MET-5') or $
      (year eq 1991 and f(3) eq 'MET-5') or $
      (year eq 1988 and f(3) eq 'MET-2') or $
      (year eq 1986 and f(3) eq 'MET-2') or $
      (year eq 1990 and f(3) eq 'MET-3') or $
      (year eq 1990 and f(3) eq 'MET-4') or $
      (year eq 1992 and f(3) eq 'MET-5')) then begin
    RECLON = 0.0
    glon1c = 0.
endif else begin
;   Normally the error causes image to be set to -1, but in this case,
;   we want to keep the image (in case the location is wrong), thus
;   only the 1st element is set to -1, however, we need to convert to
;   intarr to allow negative numbers.
    print,'Unknown Rectification longitude:',glon1c
    images = fix(images)
    images(0) = -1
    stop
    return
endelse

;HARDCODE MET-5 location to IODC after 1997
if ((year gt 1998 and f(3) eq 'MET-5') or $
    (year eq 1998 and f(3) eq 'MET-5' and mon gt 6)) then RECLON = 63.d0

;HARDCODE MET-7 location after move to IODC
if (f(3) eq 'MET-7' and (year gt 2006 or (year eq 2006 and jday ge 305))) then reclon = 57.d0

; Get Keplerian elements for optimally-perfect geostationary satellite
;     define the epoch time as the 00:00:00UTC on the date of the image
angle = gha(julday(1,jday,year,0,0,0),/degree)
; set up kepler elements
semima = 42166.264d0
eccen = 0.d0
incl = 0.d-7
meana = RECLON + angle 
perigee = 0.d0 
rgtasc = 0.d0 

; THE Meteosat files do not provide any keplarian elements.
; However, they could be retrieved from the TLE files in the future.
orbit.ep_date = frdate           ;edate
orbit.ep_time = 0l           ;etime
orbit.ep_msec = 0l           ;msec
orbit.semima = semima            ;semima
orbit.incl = incl              ;incl
orbit.eccen = eccen             ;eccen
orbit.meana = meana             ;meana
orbit.rgtasc = rgtasc            ;rasc
orbit.perigee = perigee           ;perigee
orbit.sper = 0.6              ;spper
ORBstr = orbit

;Set up calibration for times when none is available

;From some REALLY early ISCCP documents ...
;Radiance = f * S * (Count-I)
; Channel 2 (IRWIN): S=0.046  W / (m^2 ster count); I=5; f=0.937
; Channel 3 (IRWVP): S=0.00865W / (m^2 ster count); I=6; f=1.000

metcal = fltarr(2,nchan,256)
;count = indgen(256)
;metcal(0,1,*) = 0.937 * 0.046   * (count-5.)
;metcal(0,2,*) = 1.000 * 0.00865 * (count-6.)

;Will need to convert these radiances to temperature
;c1 = 1.19104e-8
;c2 = 1.43877
;nu = 1e4/11.5
;metcal(1,1,*) = c2 * nu / alog(1. + c1 / metcal(0,1,*) * nu^3)
;nu = 1e4/6.7
;metcal(1,2,*) = c2 * nu / alog(1. + c1 / metcal(0,1,*) * nu^3)

;RADIANCE from MET-2: ISCCP.BT.2.MET-2.1983.07.01.1200.ESA.gz
;but because the data is from a BT file, the table needs to be
;inverted (using IDL:reverse) to match the original B1 files (because
;GISS inverted the radiances between the B2 and B3 files)

ich=0
if (nchan eq 3) then begin
    metcal(0,0,*) = $
      reverse([0.000, 0.000, 0.000, 0.580, 1.160, 1.740, 2.320, 2.900, 3.480, $
               4.060, 4.640, 5.220, 5.800, 6.380, 6.960, 7.540, 8.120, 8.700, $
               9.280, 9.860, 10.440, 11.020, 11.600, 12.180, 12.760, 13.340, 13.920, $
               14.500, 15.080, 15.660, 16.240, 16.820, 17.400, 17.980, 18.560, 19.140, $
               19.720, 20.300, 20.880, 21.460, 22.040, 22.620, 23.200, 23.780, 24.360, $
               24.940, 25.520, 26.100, 26.680, 27.260, 27.840, 28.420, 29.000, 29.580, $
               30.160, 30.740, 31.320, 31.900, 32.480, 33.060, 33.640, 34.220, 34.800, $
               35.380, 35.960, 36.540, 37.120, 37.700, 38.280, 38.860, 39.440, 40.020, $
               40.600, 41.180, 41.760, 42.340, 42.920, 43.500, 44.080, 44.660, 45.240, $
               45.820, 46.400, 46.980, 47.560, 48.140, 48.720, 49.300, 49.880, 50.460, 51.040, 51.620, $
               52.200, 52.780, 53.360, 53.940, 54.520, 55.100, 55.680, 56.260, 56.840, 57.420, 58.000, $
               58.580, 59.160, 59.740, 60.320, 60.900, 61.480, 62.060, 62.640, 63.220, 63.800, 64.380, $
               64.960, 65.540, 66.120, 66.700, 67.280, 67.860, 68.440, 69.020, 69.600, 70.180, 70.760, $
               71.340, 71.920, 72.500, 73.080, 73.660, 74.240, 74.820, 75.400, 75.980, 76.560, 77.140, $
               77.720, 78.300, 78.880, 79.460, 80.040, 80.620, 81.200, 81.780, 82.360, 82.940, 83.520, $
               84.100, 84.680, 85.260, 85.840, 86.420, 87.000, 87.580, 88.160, 88.740, 89.320, 89.900, $
               90.480, 91.060, 91.640, 92.220, 92.800, 93.380, 93.960, 94.540, 95.120, 95.700, 96.280, $
               96.860, 97.440, 98.020, 98.600, 99.180, 99.760, 100.340, 100.920, 101.500, 102.080, 102.660, $
               103.240, 103.820, 104.400, 104.980, 105.560, 106.140, 106.720, 107.300, 107.880, 108.460, 109.040, $
               109.620, 110.200, 110.780, 111.360, 111.940, 112.520, 113.100, 113.680, 114.260, 114.840, 115.420, $
               116.000, 116.580, 117.160, 117.740, 118.320, 118.900, 119.480, 120.060, 120.640, 121.220, 121.800, $
               122.380, 122.960, 123.540, 124.120, 124.700, 125.280, 125.860, 126.440, 127.020, 127.600, 128.180, $
               128.760, 129.340, 129.920, 130.500, 131.080, 131.660, 132.240, 132.820, 133.400, 133.980, 134.560, $
               135.140, 135.720, 136.300, 136.880, 137.460, 138.040, 138.620, 139.200, 139.780, 140.360, 140.940, $
               141.520, 142.100, 142.680, 143.260, 143.840, 144.420, 145.000, 145.580, 146.160, 146.740])
    ich=ich+1
endif
metcal(0,ich,*) = $
  reverse([20.220, 20.140, 20.050, 19.970, 19.890, 19.810, 19.730, 19.650, 19.570, 19.490, $
           19.410, 19.330, 19.250, 19.170, 19.080, 19.000, 18.920, 18.840, 18.760, 18.680, $
           18.600, 18.520, 18.440, 18.360, 18.280, 18.200, 18.110, 18.030, 17.950, 17.870, $
           17.790, 17.710, 17.630, 17.550, 17.470, 17.390, 17.310, 17.220, 17.140, 17.060, $
           16.980, 16.900, 16.820, 16.740, 16.660, 16.580, 16.500, 16.420, 16.340, 16.250, $
           16.170, 16.090, 16.010, 15.930, 15.850, 15.770, 15.690, 15.610, 15.530, 15.450, $
           15.360, 15.280, 15.200, 15.120, 15.040, 14.960, 14.880, 14.800, 14.720, 14.640, $
           14.560, 14.480, 14.390, 14.310, 14.230, 14.150, 14.070, 13.990, 13.910, 13.830, $
           13.750, 13.670, 13.590, 13.500, 13.420, 13.340, 13.260, 13.180, 13.100, 13.020, 12.940, 12.860, 12.780, $
           12.700, 12.620, 12.530, 12.450, 12.370, 12.290, 12.210, 12.130, 12.050, 11.970, 11.890, 11.810, 11.730, $
           11.640, 11.560, 11.480, 11.400, 11.320, 11.240, 11.160, 11.080, 11.000, 10.920, 10.840, 10.760, 10.670, $
           10.590, 10.510, 10.430, 10.350, 10.270, 10.190, 10.110, 10.030, 9.950, 9.870, 9.780, 9.700, 9.620, $
           9.540, 9.460, 9.380, 9.300, 9.220, 9.140, 9.060, 8.980, 8.900, 8.810, 8.730, 8.650, 8.570, 8.490, $
           8.410, 8.330, 8.250, 8.170, 8.090, 8.010, 7.920, 7.840, 7.760, 7.680, 7.600, 7.520, 7.440, 7.360, $
           7.280, 7.200, 7.120, 7.040, 6.950, 6.870, 6.790, 6.710, 6.630, 6.550, 6.470, 6.390, 6.310, 6.230, $
           6.150, 6.070, 5.980, 5.900, 5.820, 5.740, 5.660, 5.580, 5.500, 5.420, 5.340, 5.260, 5.180, 5.090, $
           5.010, 4.930, 4.850, 4.770, 4.690, 4.610, 4.530, 4.450, 4.370, 4.290, 4.210, 4.120, 4.040, 3.960, $
           3.880, 3.800, 3.720, 3.640, 3.560, 3.480, 3.400, 3.320, 3.230, 3.150, 3.070, 2.990, 2.910, 2.830, $
           2.750, 2.670, 2.590, 2.510, 2.430, 2.350, 2.260, 2.180, 2.100, 2.020, 1.940, 1.860, 1.780, 1.700, $
           1.620, 1.540, 1.460, 1.370, 1.290, 1.210, 1.130, 1.050, 0.970, 0.890, 0.810, 0.730, 0.650, 0.570, $
           0.490, 0.400, 0.320, 0.240, 0.160, 0.080, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000])
ich=ich+1
metcal(0,ich,*) = $
  reverse([2.150, 2.150, 2.140, 2.130, 2.120, 2.110, 2.100, 2.090, 2.080, 2.080, 2.070, 2.060, 2.050, $
           2.040, 2.030, 2.020, 2.020, 2.010, 2.000, 1.990, 1.980, 1.970, 1.960, 1.950, 1.950, 1.940, $
           1.930, 1.920, 1.910, 1.900, 1.890, 1.890, 1.880, 1.870, 1.860, 1.850, 1.840, 1.830, 1.830, $
           1.820, 1.810, 1.800, 1.790, 1.780, 1.770, 1.760, 1.760, 1.750, 1.740, 1.730, 1.720, 1.710, $
           1.700, 1.700, 1.690, 1.680, 1.670, 1.660, 1.650, 1.640, 1.630, 1.630, 1.620, 1.610, 1.600, $
           1.590, 1.580, 1.570, 1.570, 1.560, 1.550, 1.540, 1.530, 1.520, 1.510, 1.510, 1.500, 1.490, $
           1.480, 1.470, 1.460, 1.450, 1.440, 1.440, 1.430, 1.420, 1.410, 1.400, 1.390, 1.380, 1.380, $
           1.370, 1.360, 1.350, 1.340, 1.330, 1.320, 1.310, 1.310, 1.300, 1.290, 1.280, 1.270, 1.260, $
           1.250, 1.250, 1.240, 1.230, 1.220, 1.210, 1.200, 1.190, 1.190, 1.180, 1.170, 1.160, 1.150, $
           1.140, 1.130, 1.120, 1.120, 1.110, 1.100, 1.090, 1.080, 1.070, 1.060, 1.060, 1.050, 1.040, $
           1.030, 1.020, 1.010, 1.000, 0.990, 0.990, 0.980, 0.970, 0.960, 0.950, 0.940, 0.930, 0.930, $
           0.920, 0.910, 0.900, 0.890, 0.880, 0.870, 0.870, 0.860, 0.850, 0.840, 0.830, 0.820, 0.810, $
           0.800, 0.800, 0.790, 0.780, 0.770, 0.760, 0.750, 0.740, 0.740, 0.730, 0.720, 0.710, 0.700, $
           0.690, 0.680, 0.670, 0.670, 0.660, 0.650, 0.640, 0.630, 0.620, 0.610, 0.610, 0.600, 0.590, $
           0.580, 0.570, 0.560, 0.550, 0.540, 0.540, 0.530, 0.520, 0.510, 0.500, 0.490, 0.480, 0.480, $
           0.470, 0.460, 0.450, 0.440, 0.430, 0.420, 0.420, 0.410, 0.400, 0.390, 0.380, 0.370, 0.360, $
           0.350, 0.350, 0.340, 0.330, 0.320, 0.310, 0.300, 0.290, 0.290, 0.280, 0.270, 0.260, 0.250, $
           0.240, 0.230, 0.220, 0.220, 0.210, 0.200, 0.190, 0.180, 0.170, 0.160, 0.160, 0.150, 0.140, $
           0.130, 0.120, 0.110, 0.100, 0.100, 0.090, 0.080, 0.070, 0.060, 0.050, 0.040, 0.030, 0.030, $
           0.020, 0.010, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000])
ich=0
if (nchan eq 3) then begin
    metcal(1,ich,*) = $
      reverse([0.000, 0.000, 0.000, 0.004, 0.007, 0.011, 0.015, 0.018, 0.022, 0.025, 0.029, 0.033, 0.036, 0.040, $
               0.044, 0.047, 0.051, 0.055, 0.058, 0.062, 0.066, 0.069, 0.073, 0.076, 0.080, 0.084, 0.087, 0.091, $
               0.095, 0.098, 0.102, 0.106, 0.109, 0.113, 0.117, 0.120, 0.124, 0.127, 0.131, 0.135, 0.138, 0.142, $
               0.146, 0.149, 0.153, 0.157, 0.160, 0.164, 0.168, 0.171, 0.175, 0.178, 0.182, 0.186, 0.189, 0.193, $
               0.197, 0.200, 0.204, 0.208, 0.211, 0.215, 0.218, 0.222, 0.226, 0.229, 0.233, 0.237, 0.240, 0.244, $
               0.248, 0.251, 0.255, 0.259, 0.262, 0.266, 0.269, 0.273, 0.277, 0.280, 0.284, 0.288, 0.291, 0.295, $
               0.299, 0.302, 0.306, 0.310, 0.313, 0.317, 0.320, 0.324, 0.328, 0.331, 0.335, 0.339, 0.342, 0.346, $
               0.350, 0.353, 0.357, 0.360, 0.364, 0.368, 0.371, 0.375, 0.379, 0.382, 0.386, 0.390, 0.393, 0.397, $
               0.401, 0.404, 0.408, 0.411, 0.415, 0.419, 0.422, 0.426, 0.430, 0.433, 0.437, 0.441, 0.444, 0.448, 0.452, $
               0.455, 0.459, 0.462, 0.466, 0.470, 0.473, 0.477, 0.481, 0.484, 0.488, 0.492, 0.495, 0.499, 0.503, 0.506, $
               0.510, 0.513, 0.517, 0.521, 0.524, 0.528, 0.532, 0.535, 0.539, 0.543, 0.546, 0.550, 0.553, 0.557, 0.561, $
               0.564, 0.568, 0.572, 0.575, 0.579, 0.583, 0.586, 0.590, 0.594, 0.597, 0.601, 0.604, 0.608, 0.612, 0.615, $
               0.619, 0.623, 0.626, 0.630, 0.634, 0.637, 0.641, 0.645, 0.648, 0.652, 0.655, 0.659, 0.663, 0.666, 0.670, $
               0.674, 0.677, 0.681, 0.685, 0.688, 0.692, 0.696, 0.699, 0.703, 0.706, 0.710, 0.714, 0.717, 0.721, 0.725, $
               0.728, 0.732, 0.736, 0.739, 0.743, 0.746, 0.750, 0.754, 0.757, 0.761, 0.765, 0.768, 0.772, 0.776, 0.779, $
               0.783, 0.787, 0.790, 0.794, 0.797, 0.801, 0.805, 0.808, 0.812, 0.816, 0.819, 0.823, 0.827, 0.830, 0.834, $
               0.838, 0.841, 0.845, 0.848, 0.852, 0.856, 0.859, 0.863, 0.867, 0.870, 0.874, 0.878, 0.881, 0.885, 0.888, $
               0.892, 0.896, 0.899, 0.903, 0.907, 0.910, 0.914, 0.918, 0.921])
    ich=ich+1
endif

metcal(1,ich,*) = $
  reverse([336.060, 335.710, 335.350, 335.000, 334.620, 334.260, 333.910, 333.570, 333.230, 332.880, 332.530, 332.180, 331.790, $
           331.440, 331.090, 330.730, 330.360, 330.000, 329.650, 329.290, 328.910, 328.550, 328.180, 327.820, 327.450, 327.090, $
           326.730, 326.330, 325.970, 325.590, 325.220, 324.840, 324.470, 324.090, 323.730, 323.330, 322.970, 322.590, 322.220, $
           321.840, 321.450, 321.060, 320.690, 320.280, 319.900, 319.520, 319.130, 318.740, 318.350, 317.970, 317.580, 317.160, $
           316.770, 316.390, 316.000, 315.610, 315.230, 314.830, 314.400, 314.000, 313.600, 313.200, 312.800, 312.400, 312.000, $
           311.600, 311.170, 310.760, 310.340, 309.930, 309.520, 309.100, 308.690, 308.280, 307.830, 307.410, 307.000, 306.590, $
           306.170, 305.750, 305.320, 304.860, 304.450, 304.030, 303.610, 303.180, 302.740, 302.300, 301.860, 301.390, 300.960, $
           300.520, 300.070, 299.630, 299.190, 298.740, 298.300, 297.810, 297.370, 296.920, 296.460, 296.000, 295.560, 295.110, $
           294.620, 294.150, 293.690, 293.230, 292.760, 292.280, 291.810, 291.350, 290.840, 290.360, 289.880, 289.400, 288.920, $
           288.440, 287.960, 287.480, 286.960, 286.460, 285.960, 285.460, 284.960, 284.460, 283.960, 283.420, 282.910, 282.390, $
           281.880, 281.380, 280.870, 280.350, 279.820, 279.230, 278.700, 278.170, 277.640, 277.090, 276.570, 276.040, 275.500, $
           274.910, 274.360, 273.810, 273.240, 272.670, 272.100, 271.520, 270.950, 270.360, 269.800, 269.200, 268.620, 268.050, $
           267.450, 266.850, 266.200, 265.600, 265.000, 264.400, 263.790, 263.160, 262.530, 261.890, 261.210, 260.580, 259.950, $
           259.320, 258.670, 258.000, 257.330, 256.670, 255.940, 255.280, 254.590, 253.890, 253.220, 252.530, 251.820, 251.060, $
           250.350, 249.630, 248.880, 248.130, 247.410, 246.690, 245.930, 245.070, 244.310, 243.530, 242.730, 241.930, 241.130, $
           240.290, 239.470, 238.570, 237.730, 236.920, 236.000, 235.140, 234.230, 233.360, 232.380, 231.460, 230.500, 229.540, $
           228.580, 227.580, 226.580, 225.580, 224.450, 223.420, 222.360, 221.270, 220.180, 219.000, 217.900, 216.700, 215.400, $
           214.200, 212.900, 211.670, 210.330, 209.000, 207.630, 206.000, 204.570, 203.000, 201.500, 199.860, 198.140, 196.430, $
           194.670, 192.500, 190.500, 188.400, 186.200, 184.000, 181.400, 178.750, 175.750, 172.330, 168.670, 164.670, 160.000, $
           154.500, 147.000, 137.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000])
ich=ich+1
metcal(1,ich,*) = $
  reverse([272.770, 272.690, 272.540, 272.380, 272.230, 272.150, 272.000, 271.830, 271.670, 271.580, 271.420, 271.250, 271.170, $
           271.000, 270.830, 270.670, 270.580, 270.420, 270.250, 270.080, 270.000, 269.830, 269.670, 269.500, 269.420, 269.250, $
           269.080, 269.000, 268.820, 268.640, 268.450, 268.360, 268.180, 268.000, 267.830, 267.750, 267.580, 267.420, 267.330, $
           267.170, 267.000, 266.820, 266.730, 266.550, 266.360, 266.180, 266.090, 265.900, 265.700, 265.500, 265.400, 265.200, $
           265.000, 264.910, 264.730, 264.550, 264.360, 264.270, 264.090, 263.900, 263.700, 263.600, 263.400, 263.200, 263.100, $
           262.900, 262.700, 262.500, 262.400, 262.200, 262.000, 261.800, 261.700, 261.500, 261.300, 261.200, 261.000, 260.800, $
           260.600, 260.500, 260.300, 260.100, 259.890, 259.780, 259.560, 259.330, 259.110, 259.000, 258.780, 258.560, 258.440, $
           258.220, 258.000, 257.780, 257.670, 257.440, 257.220, 257.000, 256.880, 256.630, 256.380, 256.250, 256.000, 255.780, $
           255.560, 255.440, 255.220, 255.000, 254.750, 254.630, 254.380, 254.130, 254.000, 253.750, 253.500, 253.250, 253.130, $
           252.880, 252.630, 252.380, 252.250, 252.000, 251.710, 251.430, 251.290, 251.000, 250.750, 250.630, 250.380, 250.130, $
           249.830, 249.670, 249.330, 249.000, 248.750, 248.630, 248.380, 248.130, 248.000, 247.670, 247.330, 247.000, 246.860, $
           246.570, 246.290, 246.000, 245.830, 245.500, 245.170, 245.000, 244.710, 244.430, 244.140, 244.000, 243.600, 243.200, $
           242.860, 242.710, 242.430, 242.140, 241.800, 241.600, 241.200, 240.830, 240.670, 240.330, 240.000, 239.600, 239.400, $
           239.000, 238.600, 238.200, 238.000, 237.600, 237.200, 237.000, 236.600, 236.200, 235.800, 235.600, 235.200, 234.800, $
           234.400, 234.200, 233.750, 233.250, 232.800, 232.600, 232.200, 231.750, 231.500, 231.000, 230.500, 230.000, 229.750, $
           229.250, 228.750, 228.250, 228.000, 227.330, 226.750, 226.500, 226.000, 225.330, 224.670, 224.330, 223.750, 223.250, $
           222.670, 222.330, 221.670, 221.000, 220.670, 220.000, 219.000, 218.330, 218.000, 217.330, 216.500, 215.500, 215.000, $
           214.330, 213.500, 212.500, 212.000, 211.000, 210.000, 209.500, 208.500, 207.500, 206.000, 205.500, 204.500, 203.000, $
           202.000, 201.000, 199.000, 197.500, 197.000, 195.000, 193.000, 191.000, 189.000, 187.000, 183.000, 179.000, 177.000, $
           172.000, 162.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000, 0.000])

if keyword_set(debug) then stop



;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = long(IMGstr.frtime)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = chninfo.nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 0                   ;N-S orientation
IMGinf(8) = 1                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder
IMGinf(14) = 256                ;# of bins ... should add option for MET-8 soon.

;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)
sn = fix(s(1))
SATinf(*) = 'Missing'
SATinf(i++) = 'Meteosat-'+s(1)
SATinf(i++) = 'Missing'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor

;  METEOSAT channel info will be FWHM bounds ... calculated from the
;  ISCCP web page
metinfo = [['0.56-0.95 um','10.7-12.1 um','5.76-7.22 um'], $
           ['0.49-0.98 um','10.6-12.3 um','5.80-7.23 um'], $           
           ['0.50-0.93 um','10.7-11.9 um','5.78-6.85 um'], $           
           ['0.46-0.87 um','10.6-11.8 um','5.73-6.84 um'], $           
           ['0.47-0.86 um','10.7-12.0 um','5.77-6.81 um'], $           
           ['0.44-0.96 um','10.6-12.5 um','5.74-7.01 um']]         
for j=0,nchan-1 do begin
    tmp = 'Missing'
    if (chninfo.ch_name(j) eq 'VSCHN') then tmp = metinfo(0,sn-2)
    if (chninfo.ch_name(j) eq 'IRWIN') then tmp = metinfo(1,sn-2)
    if (chninfo.ch_name(j) eq 'IRWVP') then tmp = metinfo(2,sn-2)
    SATinf(i++) = tmp
endfor
;---------------------------
; Navigation information - NEW B1U NAV ARRAY
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10.
NAVinf(3) = IMGstr.cen_pix * 10.
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = 0.6*1.e6            ;Seconds per revolution*10^6
;KEPSAT
NAVinf(9) = 15                  ;Kepler from "perfect" geostationary
NAVinf(10) = frdate
NAVinf(11) = 0l
NAVinf(12) = 0l
NAVinf(13) = semima *1.e2
NAVinf(14) = incl   *1.e3
NAVinf(15) = eccen  *1.e6
NAVinf(16) = meana  *1.e3
NAVinf(17) = rgtasc *1.e3
NAVinf(18) = perigee*1.e3
;KEPTLE
NAVinf(19) = 13                 ;Elements from nearest TLE
NAVinf(20) = TLEORB.EP_date
NAVinf(21) = TLEORB.EP_time
NAVinf(22) = TLEORB.EP_msec*1.e3
NAVinf(23) = TLEORB.semima *1.e2
NAVinf(24) = TLEORB.incl   *1.e3
NAVinf(25) = TLEORB.eccen  *1.e6
NAVinf(26) = TLEORB.meana  *1.e3
NAVinf(27) = TLEORB.rgtasc *1.e3
NAVinf(28) = TLEORB.perigee*1.e3
;KEPFIT
NAVinf(29:38) = MISSING
;ATTDAT
NAVinf(39:41) = 0l
;ATTFIT
NAVinf(42:48) = MISSING
;RECINFO
NAVinf(49) = 1
NAVinf(50) = 0
NAVinf(51) = reclon * 1.e3
;FITINFO
NAVinf(52:56) = MISSING
;POSDAT
NAVinf(57:129) = MISSING
NAVinf(65) =   0l
NAVinf(66) =  -90l*(2l^21)
NAVinf(125)=   0l
NAVinf(126)=  -90l*(2l^21)
;LOCDAT
NAVinf(130:142) = MISSING
;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = IMGstr.frdate
NAVinf(148) = ftime(IMGstr.frtime) * 1000. * 3600l
NAVinf(149) = IMGstr.nlin
NAVinf(150:199) = MISSING

;--------------------------
; TEST GRID
TGRinf = MISSING


; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = linhdr(*,*,2) ;absolute scan line
PREinf(*,i++,*) = MISSING       ;CHNUM
PREinf(*,i++,*) = MISSING       ;linhdr.year   ;year
PREinf(*,i++,*) = MISSING       ;linhdr.day    ;doy
PREinf(*,i++,*) = MISSING       ;linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = MISSING       ;linhdr.sec    ;Sec
PREinf(*,i++,*) = MISSING       ;linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = linhdr(*,*,5) ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------
goto,endd
badfile:
;in the event the read of the file failed, then don't process it.
;e.g., 1998/ISCCP.B1.0.MET-6.1998.05.08.1430.EUM.gz is a bad file.
images = -1
return

endd:
end
;***
