;****I* idl/idl.b1read_eum_msg
;+
; NAME:
;    b1read_eum_msg
;
; PURPOSE:
;    To read ISCCP-B1 data created by EUM for Meteosat Second
;    Generation (MET-8/MET-9)
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    B1VER : Return version information
;    FHDR:   Return a byte array of the entire B1 file header
;
; COMMON BLOCKS:
;    None.
;
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;
; USES
; * ftime.pro
; * geo_struct.pro
; * get_file_size.pro
; * gha.pro
; * itime.pro
; * plot_tle.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; MODIFICATION HISTORY:
; * 2010-08 - krk - Added robodoc header
;
;-
; SOURCE 

pro b1read_eum_msg,file=file,$    ;images=images
                   images=images, $ ;return image information in the images variable
                   GEOstr=GEOstr,$ ;use with geo_ij2ll and geoll2ij
                   ORBstr=ORBstr,$ ;keplarian orbital info
                   IMGstr=IMGstr,$ ;image size and scan geometry
                   ATTstr=ATTstr,$ ;satellite attitude info
                   TIMstr=TIMstr,$ ;temporal data
                   CHNstr=CHNstr,$ ;channel information
                   LINhdr=LINhdr,$ ;LINE HEADERs
                   TGRstr=TGRstr,$ ;the test grid data
                   B1VER=B1VER,$ ;returns revision info
                   FHDR=FHDR,$    ;returns file header
                   b1u=b1u, $
                   compress=compress, $
                   msgcal=msgcal
;HARDCODED MATERIAL
reset
;file='/home/kknapp/data/isccp/eumetsat/MSG1-SEVI-MSGISB1-0100-0100-20040127084500.000000000Z-1969.nat'
;file='/home/raid/kknapp/data/isccp/b1/MSG/ISCCP.B1.0.MSG-1.2007.01.08.0000.EUM'
;file='/home/kknapp/EUMETSAT-DVDs/ISCCP.B1.0.MSG-1.2006.01.31.1200.EUM'
;file='/home/kknapp/EUMETSAT-DVDs/ISCCP.B1.0.MET-2.1983.11.10.2030.EUM'


b1ver = '$Id: b1read_eum_msg.pro 22769 2010-09-16 12:22:39Z kknapp $'
;reset global variables
reset
@geo_struct.pro
; Get a file name if none provided
if not(keyword_set(file)) then begin
    file = dialog_pickfile(path='~/data/ISCCP-test',filter='*MET-*')
    print,file
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif
; = file_info(file)
filesize = get_file_size(file,comp=icomp)




; Set up input arrays and structs

isccp_hdr = {ftn:bytarr(2),$    ;[0b,1b]
             nrec:0,$           ;# of physical recs ... 313?
             yrd:0,$            ;year of data acquisition
             dayd:0,$           ;day of data acsuisition
             hrd:0,$            ;hour of acq.
             mind:0,$           ;minute of acq
             yrp:0,$            ;year processed
             dayp:0,$           ;day processed
             hrp:0,$            ;hour proc
             minp:0,$           ;minute proc
             nchan:0,$          ;# of channels
             chid:intarr(12),$  ;
             sat:bytarr(8),$    ;satellite name 'MSG1    '??
             npix:intarr(12),$  ;
             pixavg:intarr(12),$ ;
             avgmthd:intarr(12),$ ;
             bits:intarr(12),$  ;
             subsamp:intarr(12),$ ;
             calcoeff:intarr(12),$ ;
             space:intarr(12)$  ;
       }                        
packet_hdr=bytarr(429)
packet_subhdr=bytarr(198)
mpef_hdr=bytarr(172)

ln_hdr = intarr(5,11,1237)
images = bytarr(11,1237,1237)

in_hdr = intarr(5)
in_lin = bytarr(1237)


;read in the file
fhdr = bytarr(1021)
openr,unit,file,/swap_end,comp=icomp,/get
readu,unit,fhdr
point_lun,unit,0
readu,unit,packet_hdr
readu,unit,packet_subhdr
readu,unit,mpef_hdr
readu,unit,isccp_hdr
for j=0,1236 do begin
    for ich=0,10 do begin
        readu,unit,in_hdr
        readu,unit,in_lin
        ln_hdr(*,ich,j) = in_hdr
        images(ich,*,j) = in_lin
    endfor
endfor
close,unit
free_lun,unit

;reassign packet header information for the byte data
ph=packet_hdr
ph_ver = ph(0)
ph_type = ph(1)
ph_subtype = ph(2)
ph_srdid = ph(3)
ph_envid = ph(4)
ph_src_inst = ph(5)
ph_swid = swap_endian(long(ph(6:9),0,1))
ph_srccpu = ph(10:13)
ph_destid = ph(14)
ph_destenv = ph(15)
ph_datalen = swap_endian(long(ph(16:19),0,1))
ph_filler = ph(20:21)

sh = packet_subhdr
sh_ver = sh(0)
sh_svctyp = sh(1)
sh_subtyp = sh(2)
sh_nomday = swap_endian(fix(sh(3:4),0,1))
sh_nomsec = swap_endian(long(sh(5:8),0,1))
sh_satid  = swap_endian(fix(sh(9:10),0,1))
sh_filler = fix(sh(11:12))

mh = mpef_hdr

ih = isccp_hdr

chninfo.nchan = ih.nchan
nchan = ih.nchan
if (ih.nchan eq 11) then begin
    ;from MSG data documentation
    MSG_ch_name = ['NoChannel','VIS06','VIS08','IR16','IR39','WV62','WV73','IR87','IR97','IR108','IR120','IR134']
    ;Reset names for common ISCCP channels
    MSG_ch_name(1) = 'VSCHN'
    MSG_ch_name(4) = 'IRNIR'
    MSG_ch_name(5) = 'IRWVP'
    MSG_ch_name(9) = 'IRWIN'
    MSG_ch_name(10)= 'IRCO2'
endif else begin
    print,'Unknown number of MSG channels'
    stop
endelse
chninfo.ch_name(0:11) = MSG_ch_name
chninfo.ch_info = 'none'
CHNstr = chninfo


;date/time
f=strsplit(file,'.',/extr)
year = long(f(4))
mon = long(f(5))
day = long(f(6))
time = long(f(7))
hh = time / 100l
mm = time mod 100l
frtime = hh*10000l + mm*100l
yymmdd = (year-1900)*10000l + mon*100l + day
frdate = yymmdd_2_yyjjj(yymmdd)

;FROM MSG_navigation.f90 [from EUMETSAT]

;! NOTE: Please be aware, that the program assumes the MSG image is
;! ordered in the operational scanning direction which means from south
;! to north and from east to west. With that the VIS/IR channels contains
;! of 3712 x 3712 pixels, start to count on the most southern line and the
;! most eastern column with pixel 1,1.
;! COFF/LOFF are the offsets for column and line which are basically 1856
;! and 1856 for the VIS/IR channels and refer to the middle of the image 
;! (centre pixel). The values regarding the High Resolution Visible Channel 
;! (HRVis) will be made available in a later issue of this software.
;!
;! CFAC/LFAC are responsible for the image "spread" in the NS and EW
;! directions. They are calculated as follows:
;
;! CFAC = LFAC = 2^16 / delta
;! with
;! delta = 83.843 micro Radian (size of on VIS/IR MSG pixel)
;! CFAC     = LFAC     =   781648343.404  rad^-1 for VIS/IR
;!
;! which should be rounded to the nearest integer as stated in Ref [1].
;! CFAC     = LFAC     =   781648343  rad^-1 for VIS/IR


; OK THROUGH HERE


;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = 1856.         ;cen_lin
imginfo.cen_pix = 1856.         ;cen_pix
imginfo.linres = 3              ;linres
imginfo.pixres = 3              ;pixres
imginfo.nlin = 1237             ;ny
imginfo.npix = 1237             ;nx
imginfo.sline = 0               ;sline
imginfo.spix = 0                ;spix
imginfo.linang = -83.843d0*180.d0/!dpi*1e-6   ;-0.0072d0       ;linang
imginfo.pixang = -83.843d0*180.d0/!dpi*1e-6   ;-0.0072d0       ;pixang
IMGstr = imginfo

att.spra = 0.
att.spdc = 90.
ATTstr = att

reclon = 0.d0
;CURRENTLY NO RESOURCE WHEN AN MSG IS OVER INDIAN OCEAN

sattim.spdc1 = -90.0d0 
sattim.spdc2 = -90.0d0 
sattim.spra1 = 0.d0
sattim.spra2 = 0.d0
TIMstr = SATtim

plot_tle,tle,    $
  glat = glat1c, $
  glon = glon1c, $
  coord= coord,  $
  gra  = gra1c,  $
  sat  = strlowcase(f(3)), $
  year = year,   $
  jday = frdate mod 1000, $
  postime = ftime(frtime)*3600.,$
  ORB = TLEORB
TGRstr=TGRID




;geo_nav.lat = 0.d0
;geo_nav.lon = 0.d0
;if (year gt 1997 and f(3) eq 'MET-5') then geo_nav.lon = 63.d0
;geo_nav.radius = 6.611d0*6378.144d ;grad
;geo_nav.rgtasc = 1.d-5           ;spnax(0)
;geo_nav.decl = 90.0d0           ;spnax(1)
;geo_nav.GOESNEXT = 0
;geo_nav.rma = 0.0d0
;geo_nav.pma = 0.0d0
;GEOstr = geo_nav


; GET orbit info from tle


; Get Keplerian elements for optimally-perfect geostationary satellite
;     define the epoch time as the 00:00:00UTC on the date of the image
jday = frdate mod 1000
angle = gha(julday(1,jday,year,0,0,0),/degree)
; set up kepler elements


;USE the value from MSG_navigation.f90
semima = 42166.d0
eccen = 0.d0
incl = 0.d-7
meana = RECLON + angle 
perigee = 0.d0 
rgtasc = 0.d0 

; THE Meteosat files do not provide any keplarian elements.
; However, they could be retrieved from the TLE files in the future.
orbit.ep_date = frdate           ;edate
orbit.ep_time = 0l           ;etime
orbit.ep_msec = 0l           ;msec
orbit.semima = semima            ;semima
orbit.incl = incl              ;incl
orbit.eccen = eccen             ;eccen
orbit.meana = meana             ;meana
orbit.rgtasc = rgtasc            ;rasc
orbit.perigee = perigee           ;perigee
orbit.sper = 0.6              ;spper
ORBstr = orbit

;set up MSG cal
;  get # of bits
nbits = ih.bits(0)
nbin = 2l^nbits
if (nbin eq 256) then begin
    msgcal=fltarr(nchan,nbin)
endif else if (nbin eq 1024) then begin
    msgcal=fltarr(nchan,nbin)
endif else begin
    print,'B1_READ_MSG: Unknown # of bins:',nbin
    stop
endelse

c1 = 1.19104e-5
c2 = 1.43877e0
if (strtrim(ih.sat,2) eq 'MSG1') then begin 
;   from "A planned Change to the MSG Level1.5 Image Product Radiance
;   Definition" in http://www.eumetsat.int/groups/ops/documents/document/pdf_msg_planned_change_level15.pdf
;   The values are specifc to MSG-1 (MET-8)
    nu = [0., 0., 0., 2567.33, 1598.103, 1362.081, 1149.069, 1034.343, 930.647, 839.660, 752.387]

;   Values from pdf_ten_052058_msg1_vis6_calib.pdf downloaded from:
;   http://www.eumetsat.int/Home/Main/Access_to_Data/Meteosat_Meteorological_Products/Calibration/SP_1119512219822
;   Values represent Im/!PI
;   Values are specific to MSG-1 but I have no input for MSG-2 yet
    TOA = [20.76322, 23.24066, 19.85346]
endif else if (strtrim(ih.sat,2) eq 'MSG2') then begin
;   from "A planned Change to the MSG Level1.5 Image Product Radiance
;   Definition" in http://www.eumetsat.int/groups/ops/documents/document/pdf_msg_planned_change_level15.pdf
;   The values are specifc to MSG-2 (MET-9)
    nu = [0., 0., 0., 2568.832,1600.548, 1360.330, 1148.620, 1035.289, 931.700, 836.445, 751.792]
;   Values are specific to MSG-1 but I have no input for MSG-2 yet
    TOA = [20.76322, 23.24066, 19.85346]
endif else begin
    print,'B1_READ_MSG: Unknown Satellite: ->',strtrim(ih.sat,2),'<-'
endelse


count=indgen(nbin)*4
;window,xs=1000,ys=1000
;!p.multi=[0,3,4]
for i=0,nchan-1 do begin
    space = (long([ih.space(i),0],0,1)/1000.)[0]
    calcoef= (long([ih.calcoeff(i),0],0,1)/100000.)[0]
;    print,i,space,calcoef
    radiance = calcoef*(count-space)
    if (i gt 2) then begin
        temp = c2*nu(i) / (alog(1.0e0 + (nu(i)^3)*c1/radiance))
        msgcal(i,*) = temp
    endif else begin
        ;radiance = calcoef*count - space
        msgcal(i,*) = radiance / TOA(i)
    endelse
    ;plot,radiance,chars=2
;    plot,msgcal(i,*),chars=2,/yno
endfor




;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = long(IMGstr.frtime)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = chninfo.nchan
IMGinf(6) = 1                   ;areadir.bytes
IMGinf(7) = 0                   ;N-S orientation
IMGinf(8) = 1                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder
IMGinf(14) = nbin               ;# of bins ... should add option for MET-8 soon.

;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)
sn = fix(s(1))
SATinf(*) = 'Missing'
SATinf(i++) = 'Meteosat-'+strtrim(fix(s(1))+7,2)
SATinf(i++) = 'Missing'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j+1)
endfor

;  METEOSAT channel info will be FWHM bounds ... calculated from the
;  ISCCP web page
;metinfo = [['0.56-0.95 um','10.7-12.1 um','5.76-7.22 um'], $
;           ['0.49-0.98 um','10.6-12.3 um','5.80-7.23 um'], $           
;           ['0.50-0.93 um','10.7-11.9 um','5.78-6.85 um'], $           
;           ['0.46-0.87 um','10.6-11.8 um','5.73-6.84 um'], $           
;           ['0.47-0.86 um','10.7-12.0 um','5.77-6.81 um'], $           
;           ['0.44-0.96 um','10.6-12.5 um','5.74-7.01 um']]         
;for j=0,nchan-1 do begin
;    if (chninfo.ch_name(j) eq 'VSCHN') then tmp = metinfo(0,sn-2)
;    if (chninfo.ch_name(j) eq 'IRWIN') then tmp = metinfo(1,sn-2)
;    if (chninfo.ch_name(j) eq 'IRWVP') then tmp = metinfo(2,sn-2)
;    SATinf(i++) = tmp
;endfor
;---------------------------
; Navigation information - NEW B1U NAV ARRAY
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10.
NAVinf(3) = IMGstr.cen_pix * 10.
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = 0.6*1.e6            ;Seconds per revolution*10^6
;KEPSAT
NAVinf(9) = 15                  ;Kepler from "perfect" geostationary
NAVinf(10) = frdate
NAVinf(11) = 0l
NAVinf(12) = 0l
NAVinf(13) = semima *1.e2
NAVinf(14) = incl   *1.e3
NAVinf(15) = eccen  *1.e6
NAVinf(16) = meana  *1.e3
NAVinf(17) = rgtasc *1.e3
NAVinf(18) = perigee*1.e3
;KEPTLE
NAVinf(19) = 13                 ;Elements from nearest TLE
NAVinf(20) = TLEORB.EP_date
NAVinf(21) = TLEORB.EP_time
NAVinf(22) = TLEORB.EP_msec*1.e3
NAVinf(23) = TLEORB.semima *1.e2
NAVinf(24) = TLEORB.incl   *1.e3
NAVinf(25) = TLEORB.eccen  *1.e6
NAVinf(26) = TLEORB.meana  *1.e3
NAVinf(27) = TLEORB.rgtasc *1.e3
NAVinf(28) = TLEORB.perigee*1.e3
;KEPFIT
NAVinf(29:38) = MISSING
;ATTDAT
NAVinf(39:41) = 0l
;ATTFIT
NAVinf(42:48) = MISSING
;RECINFO
NAVinf(49) = 1
NAVinf(50) = 0
NAVinf(51) = reclon * 1.e3
;FITINFO
NAVinf(52:56) = MISSING
;POSDAT
NAVinf(57:129) = MISSING
NAVinf(65) =   0l
NAVinf(66) =  -90l*(2l^21)
NAVinf(125)=   0l
NAVinf(126)=  -90l*(2l^21)
;LOCDAT
NAVinf(130:142) = MISSING
;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = IMGstr.frdate
NAVinf(148) = ftime(IMGstr.frtime) * 1000. * 3600l
NAVinf(149) = IMGstr.nlin
NAVinf(150:199) = MISSING

;--------------------------
; TEST GRID
TGRinf = MISSING


; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = ln_hdr(1,*,*) ;relative scanline
PREinf(*,i++,*) = ln_hdr(2,*,*) ;absolute scan line
PREinf(*,i++,*) = ln_hdr(0,*,*) ;CHNUM
PREinf(*,i++,*) = MISSING       ;linhdr.year   ;year
PREinf(*,i++,*) = MISSING       ;linhdr.day    ;doy
PREinf(*,i++,*) = MISSING       ;linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = MISSING       ;linhdr.sec    ;Sec
PREinf(*,i++,*) = MISSING       ;linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      

;stop                            ;WORKING ON THIS
;_______________________________________________
;===============================================
;-----------------------------------------------
;...............................................


;
; END B1U array definitions
;---------------------------------------------------

endd:

;stop

end
;***
