;****I* idl/idl.b1read_jma_mts
; NAME
; b1read_jma_mts
; 
; PURPOSE
; Read ISCCP B1 files from JMA for the MTSAT-1R and MTSAT-2
; satellites.

; USAGE
; This routine should not be called directly (unless tsting). The
; b1read.pro should be used by the user.
; 
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; USES
; * ftime.pro
; * geo_struct.pro
; * gha.pro
; * plot_tle.pro
; * reset.pro
; * stripname.pro
; * yymmdd_2_yyjjj.pro
;
; USED BY
; * b1read.pro
;
; CREATION DATE
; ~2006
; 
; HISTORY
; * 2007    - Changes since last commit: compression and calibration
;             header (mtcal)
; * 28Jun07 - Added robodoc header.
; 
; SOURCE

pro b1read_jma_mts,file=file,$    ;images=images
                   images=images, $ ;return image information in the images variable
                   LINhdr=LINhdr,$ ;LINE HEADERs
                   B1VER=B1VER,$ ;returns revision info
                   FHDR=FHDR,$    ;returns file header
                   b1u=b1u, $
                   mtcal=mtcal, $
                   compress=compress
b1ver = '$Id: b1read_jma_mts.pro 22771 2010-09-16 12:27:05Z kknapp $'


fhdr=0b
reset
@geo_struct.pro

;Block definitions
prihdr = {reclen:0, $           ;should be 16
          ftype:0b, $           ;0 = image
          hdrlen:0l,$           ;total header length in octets
          datlen:lonarr(2)}     ;data field length
imgstr = {reclen:0,$            ;should be 9
          bpp:0b,$              ;bits per pixel
          npix:0,$              ;# of pixels / columns
          nlin:0,$              ;# of lines
          comp:0b}              ;compression flag
imgnav = {reclen:0,$            ;should be 51
          pname:bytarr(32),$    ;projection name: GEOS (140.0)
          cfac:0l,$             ;column scaling factor
          lfac:0l,$             ;line scaling factor
          coff:0l,$             ;column offset
          loff:0l}              ;line offset
timsta = {reclen:0,$            ;should be 10
          stamp:0b,$            ;time stamp
          day:0,$               ;days starting at 1/1/1958
          tod:0l}               ;time of day in milliseconds
imgsid = {reclen:0,$
          seq:0b,$
          nseg:0b, $
          seqlin:0}
if (n_elements(file) eq 0) then begin
    file='/raid/kknapp/data/isccp/b1-rt/mtsat/ISCCP.B1.0.MTS-1.2005.06.18.2332.JMA'
endif
byte = 0b
int2 = 0

mtcal = fltarr(5,1024)
chinfo=strarr(5)

icomp=0
if keyword_set(compress) then icomp=1

openr,lun,file,/get_lun,/swap_en,comp=icomp
old = 0l
ich = 0
while not(eof(lun)) do begin
    ;0=Primary Header
    readu,lun,byte
    if (byte ne 0) then begin
        message,'Incorrect Header type: Expected 0, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,prihdr
    endelse
    ;1=Image Structure
    readu,lun,byte
    if (byte ne 1) then begin
        message,'Incorrect Header type: Expected 1, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,imgstr
    endelse
    ;2=Image Navigation
    readu,lun,byte
    if (byte ne 2) then begin
        message,'Incorrect Header type: Expected 2, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,imgnav
    endelse
    ;3=Image Data Function
    readu,lun,byte
    if (byte ne 3) then begin
        message,'Incorrect Header type: Expected 3, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,int2
        imgdat = bytarr(int2-3)
        readu,lun,imgdat
    endelse
    ;4=Annotation
    readu,lun,byte
    if (byte ne 4) then begin
        message,'Incorrect Header type: Expected 4, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,int2
        annota = bytarr(int2-3)
        readu,lun,annota
    endelse
s = strsplit(string(annota),':',/extr)
chinfo(ich) = s(1)

    ;5=Time Stamp
    readu,lun,byte
    if (byte ne 5) then begin
        message,'Incorrect Header type: Expected 5, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,timsta
    endelse
    ;128=Image Segment Identification
    readu,lun,byte
    if (byte ne 128) then begin
        message,'Incorrect Header type: Expected 128, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,imgsid
    endelse
    ;130=Image Compensation Information Header
    readu,lun,byte
    if (byte ne 130) then begin
        message,'Incorrect Header type: Expected 130, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,int2
        imgcom = bytarr(int2-3)
        readu,lun,imgcom
    endelse
    ;131=Image Observation time header
    readu,lun,byte
    if (byte ne 131) then begin
        message,'Incorrect Header type: Expected 131, found:'+ $
                strtrim(string(byte))
    endif else begin
        readu,lun,int2
        imgtim = bytarr(int2-3)
        readu,lun,imgtim
    endelse 
    ;132=Image Quality Information Header
    readu,lun,byte
    if (byte ne 132) then begin
        message,'Incorrect Header type: Expected 132, found:'+ $
                strtrim(string(byte))
    endif else begin    
        readu,lun,int2
        imgqua = bytarr(int2-3)
        readu,lun,imgqua
    endelse 
    nx = imgstr.npix
    ny = imgstr.nlin
    if ( n_elements(img) lt 10) then begin
        if (imgstr.bpp eq 16) then begin
            img = intarr(nx,ny)
            image = intarr(5,nx,ny)            
        endif else begin
            img = bytarr(nx,ny)
            image = bytarr(5,nx,ny)
        endelse
    endif
    readu,lun,img
    image(ich,*,*) = img
    ich=ich+1
    point_lun,-lun,here
    old=here


    ;Derive Calibration information from the IMGDAT header
    cinfo = string(imgdat>32b)
    carray=strsplit(cinfo,' ',/extr)
    c = 3
    i1 = 0
    i2 = 0
    q= strsplit(carray(c),':=',/extr)
    i1 = fix(q(0))
    y1 = float(q(1))
    while (i2 ne 1023) do begin
        c=c+1
        q= strsplit(carray(c),':=',/extr)
        i2 = fix(q(0))
        y2 = float(q(1))
        n = i2 - i1 + 1
        mtcal(ich-1,i1:i2) = y1+findgen(n)/float(n-1)*(y2-y1)
        i1=i2
        y1=y2
    endwhile   
endwhile
closeit:
close,lun
free_lun,lun

images=image(0:ich-1,*,*)

;Image navigation information
;Get subsat longitude from string(imgnav.pname)
;set up Kepler stuff identical to Meteosat except with new longitude
;Store calibration information in separate header...
;convert other imgnav stuff to pixang/linang info

;date/time
f=strsplit(file,'.',/extr)
year = long(f(4))
mon = long(f(5))
day = long(f(6))
time = long(f(7))
hh = time / 100l
mm = time mod 100l
frtime = hh*10000l + mm*100l
yymmdd = (year-1900)*10000l + mon*100l + day
frdate = yymmdd_2_yyjjj(yymmdd)
nchan = ich
;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = imgnav.loff-1 ;1250.5         ;cen_lin
imginfo.cen_pix = imgnav.coff   ;1249.5         ;cen_pix
imginfo.linres = 1              ;linres
imginfo.pixres = 1              ;pixres
imginfo.nlin = ny               ;ny
imginfo.npix = nx               ;nx
imginfo.sline = 0               ;sline
imginfo.spix = 0                ;spix
imginfo.linang = 65536.0e0 / float(imgnav.lfac) ;-0.0072d0       ;linang
imginfo.pixang = 65536.0e0 / float(imgnav.cfac) ;pixang
IMGstr = imginfo

att.spra = 0.
att.spdc = 90.
ATTstr = att

chninfo.nchan = nchan
chninfo.ch_name(0) = 'IRWIN'
chninfo.ch_name(1) = 'IRSPL'
chninfo.ch_name(2) = 'IRWVP'
chninfo.ch_name(3) = 'IRNIR'
if (nchan eq 5) then begin
    chninfo.ch_name(4) = 'VSCHN'
endif
;from a web site ... these are likely not the exact FWHM values
if (f(3) eq 'MTS-1' or f(3) eq 'MTS-2') then begin
    chninfo.ch_info(0) = '10.3-11.3 um' ;IR1
    chninfo.ch_info(1) = '11.5-12.5 um' ;IR2
    chninfo.ch_info(2) = '6.5-7.0 um' ;IR3
    chninfo.ch_info(3) = '3.5-4.0 um' ;IR4
    chninfo.ch_info(4) = '0.55-0.90 um' ;VIS
    if (f(3) eq 'MTS-1') then tsat = 'mtsat-1r'
    if (f(3) eq 'MTS-2') then tsat = 'mtsat-2'
endif else begin 
    stop
endelse 


CHNstr = chninfo


f=strsplit(string(imgnav.pname),')',/extr)
g = strsplit(f(0),'(',/extr)

reclon = float(g(1))

sattim.spdc1 = -90.0d0 
sattim.spdc2 = -90.0d0 
sattim.spra1 = 0.d0
sattim.spra2 = 0.d0
TIMstr = SATtim

; GET orbit info from tle
plot_tle,tle,    $
  glat = glat1c, $
  glon = glon1c, $
  coord= coord,  $
  gra  = gra1c,  $
  sat  = tsat, $
  year = year,   $
  jday = frdate mod 1000, $
  postime = ftime(frtime)*3600.,$
  ORB = TLEORB
TGRstr=TGRID

; Get Keplerian elements for optimally-perfect geostationary satellite
;     define the epoch time as the 00:00:00UTC on the date of the image
jday = frdate mod 1000
angle = gha(julday(1,jday,year,0,0,0),/degree)
; set up kepler elements
semima = 42166.264d0
eccen = 0.d0
incl = 0.d-7
meana = RECLON + angle 
perigee = 0.d0 
rgtasc = 0.d0 

;Get the scan line times
mjd=dblarr(ny)
hdr = string(imgtim>32b)
h=strsplit(hdr,'L',/extr)

n = n_elements(h)
h1 = strsplit(h(0),':= ',/extr)

for i=1,n-1 do begin
    h2 = strsplit(h(i),':= ',/extr)
    i1 = fix(h1(1))-1
    i2 = (fix(h2(1))-1)<(ny-1)
    n = i2-i1+1
    mjd(i1:i2) = double(h1(3)) + findgen(n) / float(n-1) * (double(h2(3)) - double(h1(3)))
    h1=h2
endfor

jul = dblarr(ny)
ltime = intarr(5,ny)
for i=0,ny-1 do begin
    jul(i) = mjd(i) + 2400000.5d0
    caldat,jul(i),mon,day,year,hour,min,sec
    doy = julday(mon,day,year)-julday(1,1,year)+1
    ltime(*,i) = [year,doy,hour*100+min,floor(sec),round((sec mod 1) * 1000)]
endfor



;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,IMGstr.nlin)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = b1ver

IMGinf(*) = MISSING
IMGinf(0) = IMGstr.frdate
IMGinf(1) = long(IMGstr.frtime)
IMGinf(2) = MISSING             ;areadir.sss
IMGinf(3) = IMGstr.nlin         ;areadir.nline
IMGinf(4) = IMGstr.npix         ;areadir.nelem
IMGinf(5) = chninfo.nchan
IMGinf(6) = 2                   ;areadir.bytes
IMGinf(7) = 0                   ;N-S orientation
IMGinf(8) = 1                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = MISSING
IMGinf(11) = MISSING
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder
IMGinf(14) = 1024

;Satellite information
i=0
f = strsplit(stripname(file),'.',/extr)
s = strsplit(f(3),'-',/extr)
sn = fix(s(1))
SATinf(*) = 'Missing'
SATinf(i++) = 'MTSAT-1R'
SATinf(i++) = 'Missing'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor


for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_info(j)
endfor
;---------------------------
; Navigation information - NEW B1U NAV ARRAY
;IMGDAT
NAVinf(0) = IMGstr.sline
NAVinf(1) = IMGstr.spix
NAVinf(2) = IMGstr.cen_lin * 10.
NAVinf(3) = IMGstr.cen_pix * 10.
NAVinf(4) = IMGstr.linres
NAVinf(5) = IMGstr.pixres
NAVinf(6) = IMGstr.linang*1.e9
NAVinf(7) = IMGstr.pixang*1.e9
NAVinf(8) = 0.6*1.e6            ;Seconds per revolution*10^6
;KEPSAT
NAVinf(9) = 15                  ;Kepler from "perfect" geostationary
NAVinf(10) = frdate
NAVinf(11) = 0l
NAVinf(12) = 0l
NAVinf(13) = semima *1.e2
NAVinf(14) = incl   *1.e3
NAVinf(15) = eccen  *1.e6
NAVinf(16) = meana  *1.e3
NAVinf(17) = rgtasc *1.e3
NAVinf(18) = perigee*1.e3
;KEPTLE
NAVinf(19) = 13                 ;Elements from nearest TLE
NAVinf(20) = TLEORB.EP_date
NAVinf(21) = TLEORB.EP_time
NAVinf(22) = TLEORB.EP_msec*1.e3
NAVinf(23) = TLEORB.semima *1.e2
NAVinf(24) = TLEORB.incl   *1.e3
NAVinf(25) = TLEORB.eccen  *1.e6
NAVinf(26) = TLEORB.meana  *1.e3
NAVinf(27) = TLEORB.rgtasc *1.e3
NAVinf(28) = TLEORB.perigee*1.e3
;KEPFIT
NAVinf(29:38) = MISSING
;ATTDAT
NAVinf(39:41) = 0l
;ATTFIT
NAVinf(42:48) = MISSING
;RECINFO
NAVinf(49) = 1
NAVinf(50) = 0
NAVinf(51) = reclon * 1.e3
;FITINFO
NAVinf(52:56) = MISSING
;POSDAT
NAVinf(57:129) = MISSING
NAVinf(65) =   0l
NAVinf(66) =  -90l*(2l^21)
NAVinf(125)=   0l
NAVinf(126)=  -90l*(2l^21)
;LOCDAT
NAVinf(130:142) = MISSING
;MISC
NAVinf(143) = 0                 ;GOES-NEXT Flag=0
NAVinf(144) = MISSING
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = IMGstr.frdate
NAVinf(148) = ftime(IMGstr.frtime) * 1000. * 3600l
NAVinf(149) = IMGstr.nlin
NAVinf(150:199) = MISSING

;--------------------------
; TEST GRID
TGRinf = MISSING


; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*,i++,*) = MISSING       ;relative scanline
PREinf(*,i++,*) = MISSING       ;absolute scan line
PREinf(*,i++,*) = MISSING       ;CHNUM
PREinf(*,i++,*) = rebin(reform(ltime(0,*),1,1,ny),nchan,1,ny) ;linhdr.year   ;year
PREinf(*,i++,*) = rebin(reform(ltime(1,*),1,1,ny),nchan,1,ny) ;linhdr.day    ;doy
PREinf(*,i++,*) = rebin(reform(ltime(2,*),1,1,ny),nchan,1,ny) ;linhdr.hour*100+linhdr.min ;HHMM
PREinf(*,i++,*) = rebin(reform(ltime(3,*),1,1,ny),nchan,1,ny) ;linhdr.sec    ;Sec
PREinf(*,i++,*) = rebin(reform(ltime(4,*),1,1,ny),nchan,1,ny) ;linhdr.msec   ;MSec
PREinf(*,i++,*) = MISSING       ;eastern edge of earth
PREinf(*,i++,*) = MISSING       ;western edge of earth
PREinf(*,i++,*) = MISSING       ;detector ID
PREinf(*,i++,*) = MISSING       ;validity code
PREinf(*,i++,*) = total(images,2) mod 256 ;checksum

b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------

endd:
end
;***
