;****I* idl/idl.b1read_oa
;+
; NAME:
;    b1read_oa
;
; PURPOSE:
;    To read ISCCP-B1 OA files from AES
;
; CATEGORY:
;    ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read_aes_g7/8
;
; INPUTS:
;    file = filename of B1 data
;    
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    oa:  the orbit/attitude data in a structure
;    rec: the orbit/attitude data in a FLTARR
;    
; COMMON BLOCKS:
;    None.
;
; RESTRICTIONS:
;    External Routine:
;        def_oapre96   ... defines the OA structure for dates before
;                                                                 1996
;        def_oapost96  ... defines the OA structure for dates after 1996
;        def_oagoes7   ... defines the OA structure for GOES-7
;        reset         ... perform basic housekeeping
;
; USES
; * def_oagoes7.pro
; * def_oapost96.pro
; * def_oapre96.pro
; * get_file_size.pro
; * remove_gz.pro
; * reset.pro
; 
; USED BY
; * b1read_aes_g7.pro
; * b1read_aes_g8.pro
; 
; MODIFICATION HISTORY:
;       $Id: b1read_oa.pro 22773 2010-09-16 12:28:53Z kknapp $
;       Created 01/08/2004 - Ken Knapp NOAA/NCDC
;       Modified03/02/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;
;-
; SOURCE

pro b1read_oa,file=file,oa=oa,rec=rec

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = 'I*OA*'
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    ;print,'Non-existent file: '+stripname(file)
    ;test by removing gz
    no_gz = remove_gz(file)
    if (file ne no_gz) then begin
        if (file_test(no_gz)) then begin
            file=no_gz
            goto,use_it
        endif
    endif
    oa = -1
    rec = -1
    goto,endd
endif
use_it:
; = file_info(file)
size = get_file_size(file,comp=icomp)
f = strsplit(file,'.',/extr)

openr,unit,file,/get_lun,/swap_end,comp=icomp

if (size eq 2807 or size eq 3007 or size ne 5163) then begin
    ;read header information
    filename=' '
    satid = ' '
    firstscan = ' '
    dummy = ' '
    firstpix = ' '
    ;get OA data----------->    pre 7/1996    -or-      post 7/1996
    if (size eq 2807) then oa = def_oapre96() else oa = def_oapost96()
    ;read in ASCII for newer files, binary for older ones
    rec = dblarr(336)
    if (fix(f(4)) gt 2003) then begin
        readf,unit,filename,satid,firstscan,dummy,firstpix
        point_lun,-1*unit,here
        readf,       unit,oa
        point_lun,   unit,here
        readf,       unit,a
        readf,       unit,rec
        if (oa.rec_st_pix eq 4129) then oa.rec_st_pix = 4441
        if (oa.rec_st_pix eq 5973) then oa.rec_st_pix = 6261
    endif else begin
        readf,unit,format='(a14,1x,a6,3(1x,a5))',filename,satid,$
              firstscan,dummy,firstpix
        point_lun,unit,39
        readu,    unit,OA
        point_lun,unit,39+8
        readu,    unit,rec
    endelse
endif else if (size eq 5163) then begin
    ;GOES-7 OA data
    b = 0b
    oa = def_oagoes7()
    readu,unit,b
    readu,unit,oa
    rec = 1
endif else begin
    print,'Corrupt OA file:',file
    oa=-1
    goto,endd
endelse
point_lun,-1*unit,here
close,unit
free_lun,unit

endd:

end

;***
