function choosevar,input
if (max(input) eq min(input)) then begin
    return,max(input)
endif else begin
    s = sort(input)
    u = uniq(input(s))
    n = n_elements(u)
    nn = intarr(n)
    nn(0) = u(0)+1
    ind = indgen(n-1)+1
    nn(ind) = u(ind)-u(ind-1)
    mx = max(nn,maxloc)
    return,input(s(u(maxloc)))
endelse
end
;****I* idl/idl.b1read_uws_g56
;+
; NAME:
;    b1read_uws_g56
;
; PURPOSE:
;    To read ISCCP-B1 data created by the UWS SPC for GOES-5 and 6
;    data. Data is in the AES format as described in the SSEC
;    documentation folder.
;
; CATEGORY:
;      ISCCP
;
; CALLING SEQUENCE:
;    This routine is called from b1read. The user should use that read
;    routine to read any and all B1 data.
;
; INPUTS:
;    file = filename of B1 data [if not provided, then the
;    dialog_pickfile will prompt user to select a file]
;
; OPTIONAL INPUTS:
;    IOUTPUT: Print debug info
;
; KEYWORD PARAMETERS:
;    All keywords are outputs
;
; OUTPUTS:
;    AREAstr:Old McIdas area header
;    NAVstr: Old McIdas nvaigation header
;    IMAGES: The image file. The return is BYTARR[NCHAN,NX,NY]
;    
;      Structures common to all B1 read routines:
;    GEOstr: Used to navigate the data using geo_ll2ij/geo_ij2ll
;    ORBstr: Keplarian orbital information
;    IMGstr: Image information [size, resolution, ...]
;    ATTstr: Attitude info [roll, pitch, yaw, ...]
;    TIMstr: Timing data [greenwich right asc, chebyshev polynomials, ...]
;    CHNstr: Channel information [names, number, ...]
;    TGRstr: Test grid data [used to test navigation in old data]
;    LINhdr: The line headers in the data
;    FHDR:   Return a byte array of the entire B1 file header
;    B1U:    Structure with B1-Uniform data info [to write B1U files]
;
; common BLOCKS:
;    These common blocks were used initially, but were replaced by the
;    structures above.
;               KEPLER - keplarian orbit elements
;               ATT - Satellite attitude parameters
;               IMAGE - Image coordinate constants
;               GREEN - greenwich right ascensions/times
; USES
; * asctab.pro
; * b1u_ij2ll.pro
; * equiv_ken.pro
; * ftime.pro
; * get_file_size.pro
; * geo_struct.pro
; * itime.pro
; * kep2cart.pro
; * navcheck.pro
; * plot_tle.pro
; * reset.pro
; * stdev.pro
; * stripname.pro
; * yyjjj_2_julday.pro
; * yymmdd_2_yyjjj.pro
; 
; USED BY
; * b1read.pro
; 
; RESTRICTIONS:
;     External routines:
;        reset.pro  ... performs housekeeping tasks
;        geo_struct ... defines structures
;        latlon2ij  ... perform navigation calculations
;
; MODIFICATION HISTORY:
;       $Id: b1read_uws_g56.pro 22774 2010-09-16 12:29:16Z kknapp $
;       Created 11/14/2003 - Ken Knapp NCDC
;       Modified01/02/2004 - Ken Knapp, NCDC
;                            Inserted common blocks for use with
;                            navigation routines
;       Modified03/04/2004 - Ken Knapp, NCDC
;                            Added structures which will replace
;                            common blocks.
;               03/11/2004   Imported to CVS
;               03/16/2004   Added the option to return line headers
;               03/30/2004   Added B1VER
;                            Removed common blocks
;
;-
; SOURCE
pro b1read_uws_g56,file=file,$ 
                   images=images, $ ;return image information 
                   AREAstr=AREAstr,$ ;
                   ioutput=ioutput,$ ;
                   NAVstr=NAVstr,$ ;navigation header
                   GEOstr=GEOstr,$ ;use with geo_ij2ll & geoll2ij
                   ORBstr=ORBstr,$ ;keplarian orbital info
                          IMGstr=IMGstr,$ ;image size and scan geometry
                          ATTstr=ATTstr,$ ;satellite attitude info
                          TIMstr=TIMstr,$ ;temporal data
                          CHNstr=CHNstr,$ ;channel information
                          LINhdr=LINhdr,$ ;LINE HEADERs
                          TGRstr=TGRstr,$ ;the test grid data
                          FHDR=FHDR,$ ;returns file header
                          B1U=B1U, $ ;returns B1U block information
                          compress=compress

b1ver = '$Id: b1read_uws_g56.pro 22774 2010-09-16 12:29:16Z kknapp $'
;***
linhdr = 0
lf = 0
@geo_struct.pro

;reset global variables
reset

; Get a file name if none provided
if not(keyword_set(file)) then begin
    filters = ['I*-5*UWS','I*-6*UWS']
    file = dialog_pickfile(path='~/data/isccp/b1/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

ICOMP = 0
if (strmid(file,strlen(file)-3,3) eq '.gz') then ICOMP=1
if keyword_set(compress) then icomp=1

if keyword_Set(ioutput) then ioutput=1 else ioutput = 0
ioutput=0
; This old data set does not use ASCII but EBDIC
asctab = [ ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ','.','<','(','+','|','&',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ','!','$','*',')',';','^','-','/',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ',',','%','_','>','?',$
           ' ',' ',' ',' ',' ',' ',' ',' ',' ','`',':','#','@','\',$
           '=','"',' ','a','b','c','d','e','f','g','h','i',' ',' ',$
           ' ',' ',' ',' ',' ','j','k','l','m','n','o','p','q','r',$
           ' ',' ',' ',' ',' ',' ',' ','~','s','t','u','v','w','x',$
           'y','z',' ',' ',' ','[',' ',' ',' ',' ',' ',' ',' ',' ',$
           ' ',' ',' ',' ',' ',' ',' ',']',' ',' ','{','A','B','C',$
           'D','E','F','G','H','I',' ',' ',' ',' ',' ',' ','}','J',$
           'K','L','M','N','O','P','Q','R',' ',' ',' ',' ',' ',' ',$
           '\\',' ','S','T','U','V','W','X','Y','Z',' ',' ',' ',' ',$
           ' ',' ','0','1','2','3','4','5','6','7','8','9',' ',' ',$
           ' ',' ',' ',' ']
        ;define area information
areadir = {area,$
           valid:0l,$           ;valid if 0
           format:0l,$          ; always 4 (as of June 1985)
           sss:0l,$             ;satellite id #
           date:0l,$            ;nominal date: YYJJJ
           time:0l,$            ;nominal time: HHMMSS
           ulline:0l,$          ;upper left line
           ulelem:0l,$          ;upper left element
           unused:0l,$          ;not used
           nline:0l,$           ; # of lines
           nelem:0l,$           ; # of elems
           bytes:0l,$           ; # of bytes / element
           lineres:0l,$         ;line resolution: spacing in image-lines 
                                ;between consecutive area lines
           elemres:0l,$         ;element resolution: spacing in image-elements 
                                ;between consecutive area elements
           nchans:0l,$          ;maximum number of bands per line of area
           presize:0l,$         ;length of line prefix in bytes. Indicated
                                ;by the sum of W59,W50,W51 (+4 if W36
                                ;is present)
           proj:0l,$            ;mcidas user project number
           c_date:0l,$          ;creation date
           c_time:0l,$          ;creation time
           fmap:0l,$            ;filter map 32-bit vector for multi-channel
                                ;if a bit=1 then there are data for
                                ;that band in the area. the rightmost
                                ;bit is for band 1
           other:lonarr(5),$    ;internal use
           memo:bytarr(32),$    ;EBCDIC available for comment
           other2:lonarr(3),$   ;internal use
           validity:0l,$        ;validity code
                                ;if this matches the 1st 4 bytes of
                                ;the line prefix, the line is valid
                                ;otherwise,line is invalid
           pdl:lonarr(8),$      ;exists if image is mode AA or AAA
           band8:0l,$           ;origin of band 8 (if mode AA)
           a_date:0l,$          ;actual start date
           a_time:0l,$          ;actual start time
           a_scan:0l,$          ;actual starting scan
           a_pred:0l,$          ;line prefix documentation - bytes - W49
           a_prec:0l,$          ;line prefix calibration - bytes - W50
           a_prem:0l,$          ;line prefix level map - bytes - W51
           src:bytarr(4),$      ;image source:'VISR','VAS','AAA','ERBE',...
           caltype:bytarr(4),$  ;calibration type: units
                                ;'RAW','TEMP','BRIT',...
           other3:lonarr(11)}   ;internal use

navdir = {nav,$
          type:bytarr(4),$      ;'GOES'
          iddate:0l,$           ;ID and date: SSSYYJJJ
          time:0l,$             ;nominal start time of image
          orbit:0l,$            ;orbit type (=1 for GOES)        ORBITAL
          etimy:0l,$            ;epoch date:YYMMDD               PARAMETERS
          etimh:0l,$            ;epoch time:HHMMSS
          semima:0l,$           ;semi major axis KM*100
          eccen:0l,$            ;eccentricity * 100000
          orbinc:0l,$           ;orbital inclination deg*1000
          meana:0l,$            ;mean anomoly deg*1000
          perigee:0l,$          ;argument of perigee deg*1000
          asnode:0l,$           ;right ascension of asc. node deg*1000
          declin:0l,$           ;declin. sat. axis: dddmmss (+ =north)    SATELLITE
          rascen:0l,$           ;rgt. asc. of sat. axis.                  PARAMETERS
          piclin:0l,$           ;picture center line #
          spinp:0l,$            ;spin period: period in usec or revolutions/min.
          deglin:0l,$           ;total sweep angle, line direction DDDMMSS
          lintot:0l,$           ;# scan lines NNLLLLL NN = # sensors, LLL: # scans NN*LL
          degele:0l,$           ;sweep angle element direction
          eletot:0l,$           ;# elem in scan line
          pitch:0l,$            ;forward leaning  DDDMMSS
          yaw:0l,$              ;sideways leaning DDDMMSS
          roll:0l,$             ;rotation         DDDMMSS
          reserved:0l,$         ;  OTHER
          iajust:0l,$           ;east-west adjustment value              PARAMETERS
          iajtim:0l,$           ;time of iajust from first valid landmark
          res2:0l,$
          iseang:0l,$           ;angle between vissr and sun sensor DDDMMSS
          skew:0l,$             ;for a later implementation
          res3:0l,$
          bt1_scan:0l,$         ;scan line of beta 1
          bt1_time:0l,$         ;time of above scan HHMMSS
          bt1_tim2:0l,$         ;time (continued) millisec * 10
          bt1_cnt:0l,$          ;beta count 1
          bt2_scan:0l,$         ;scan line of beta 1
          bt2_time:0l,$         ;time of above scan HHMMSS
          bt2_tim2:0l,$         ;time (continued) millisec * 10
          bt2_cnt:0l,$          ;beta count 1
          gamma:0l,$            ;element offset * 100
          gamdot:0l,$           ;element drift per hour * 100
          res4:lonarr(80),$
          memo:bytarr(32) }     ;EBCDIC comment
          
; Get file information
; = file_info(file)
filesize = get_file_size(file,comp=icomp)

;Define format specific variables

fhdrflag = 0
;Block header
rechdr={Block,$
        type: bytarr(4), $
        desc:lonarr(5)}
rsize = 4120 ;record size
recn = 0l    ;record counter
dsize = 4096 ;size of data record 4096 +rechdr = 4120
word_flag = 0
; open file
openr,unit,file,/swap_end,/get_lun,compre=ICOMP
images = 0
prefix = 0
;read each record until the end of the fle
while not(eof(unit)) do begin
    point_lun,unit,recn*rsize
    readu,unit,rechdr
    recn = recn + 1
    ;convert Header type to ASCII
    type = strtrim(strjoin(asctab(rechdr.type)),2)
;    if (type ne 'WORD') then print,type,prefix
    if (type eq 'END') then begin
        if (ioutput eq 1) then print,type
        all = bytarr(4096)
        readu,unit,all
        if (word_flag eq 1) then begin
            if (n_elements(images) lt 20) then begin
                if (prefix gt 0) then begin
                    headers = image(0:prefix-1,*)
                    lf = 1
                    if (stdev(headers(*,0:100)) lt 4 or $
                       total(abs(headers(120,*)-image(252,*))) eq 0.) then begin
                        ;In some files(e.g., ISCCP.B1.0.GOE-5.1983.12.12.1500.UWS)
                        ;the header is replaced by a portion of the image
                        ;In this case, erase the header
                        headers(*) = 0.
                        lf = 0
                    endif
                    image = image(prefix:nelem+prefix-1,*)
                    s = size(headers)
                    ny = s(2)
                    hdr = reform(headers,long(prefix)*ny)
                    lhdr = equiv_ken(hdr,/tolong)
                    linhdr = headers
                endif
                images = image
                nav = navdir
                area=areadir
            endif else begin
                if (prefix ne 0) then begin
                    headers = image(0:prefix-1,*)
                    image = image(prefix:nelem+prefix-1,*)
                    ;LINHDR is in the Mode A line prefix format
                    if (n_elements(linhdr) lt 5) then $
                      linhdr = headers else $
                      linhdr = [[[linhdr]],[[headers]]]
                    lf = 1
                endif
                images = [[[images]],[[image]]]
                area = [area,areadir]
                nav  = [nav ,navdir]
            endelse
            word_flag = 0
        endif
    endif
    if (type eq 'HDR') then begin
        if (ioutput eq 1) then print,'HDR'
        ;AREA Header information
        dataorigin = rechdr.desc(1)
    endif 
    if (type eq 'DIR') then begin
        if (ioutput eq 1) then print,'AREA'
        ;AREA information
        readu,unit,areadir
        nelem = areadir.nelem
        nbytes = areadir.bytes
        nchans = areadir.nchans
        prefix = areadir.presize
        if (prefix gt 0) then scnhdr = bytarr(prefix)
        image = bytarr(areadir.nelem+(prefix>0),areadir.nline)
;       print,prefix,areadir.nelem,areadir.nline
    endif
    if (type eq 'NAV') then begin
        if (ioutput eq 1) then print,type
        ;navigation information
        point_lun,-1*unit,here
        nnaavv = lonarr(128)
        readu,unit,nnaavv
        point_lun,unit,here
        readu,unit,navdir
    endif
    if (type eq 'CHEB') then begin
        if (ioutput eq 1) then print,type
        ;special navigation information for GOES areas
        ;contains information from the IR documentation
        cheborigin = rechdr.desc(0)
        chebblock = lonarr(512/4)
;        chebblock = bytarr(512)        
        readu,unit,chebblock
    endif
    if (type eq 'WORD') then begin
        if (fhdrflag eq 0) then begin
            fhdrflag = 1
            fhdrend = (recn-1)*rsize
        endif
        ;IMAGE information
        wordno = rechdr.desc(0)
        nch = nchans > 1
        words_per_line = (nelem * nbytes * nch)/4
        area_line = floor(wordno / words_per_line)
        area_element = wordno mod words_per_line
        ndata = rechdr.desc(4)*4
        start = rechdr.desc(0)*4
        in = bytarr(ndata)
        readu,unit,in
        image(start:start+ndata-1) = in
        word_flag = 1
    endif
endwhile

if (arg_present(fhdr)) then begin
    point_lun,unit,0
    fhdr = bytarr(fhdrend)
    readu,unit,fhdr
;    stop
endif

close,unit
free_lun,unit
s=size(images)
siz=s
if (s(0) eq 3) then images = transpose(images,[2,0,1]) else $
                    images = reform(images,1,areadir.nelem,areadir.nline)
navtmp = nav
areatmp=area
if (n_elements(nav) gt 1) then begin
    nav=nav(0)
    area=area(0)
endif

ny = s(2)

;get rid of missing linhdr
slinhdr = size(linhdr)
if (slinhdr(0) eq 3) then begin
    replacelin = reform(linhdr(*,*,0))
    if (total(replacelin) eq 0) then replacelin = reform(linhdr(*,*,1))
    linhdr = replacelin
endif

;===============================================================================
; set up line header block
if (lf eq 1) then begin
    pre_retrace = reform(linhdr(0,*)) ;retrace: 254=scanner retrace
    pre_sname =   reform(linhdr(1,*)) ;space craft name
    pre_fcode =   reform(linhdr(3,*)) ;frame code: 254=picture transmission
; ccode:0b,$;change code:254=1st line picture fcode=254,or last line+1 if fcode=1
    pre_ccode =   reform(linhdr(4,*))
    pre_scode =   reform(linhdr(5,*)) ;step code:254=normal line, 1 = line not to be used
;  ldelay:0b,$                   ;line delay:denotes delay to be introduced by user
    pre_ldelay =  reform(linhdr(6,*))
    pre_irsel =   reform(linhdr(7,*)) ;IR selection: 1=IR1, 2=IR2,4=AUG
    pre_gray =    reform(linhdr(8,*)) ;gray scale status: 254=gray scale
    pre_dtm =     reform(linhdr(9,*)) ;direct transmission mode254=28Mb/s,1=14Mb/s
    pre_scancount=reform(fix(linhdr(10,*))/16*1000+ $
                         fix(linhdr(10,*) mod 16)*100+ $
                         linhdr(11,*)/16*10+ $
                         (linhdr(11,*) mod 16))
    pre_scanmode =reform(linhdr(12,*)) ;A=1,B=2,C=4,D=8
    pre_betacount=reform(long(linhdr(13,*))*256l*256l + $
                         long(linhdr(14,*))*256l + $
                         long(linhdr(15,*)))
    pre_gridflag =reform(linhdr(16,*)) ;grid flag:1=no grid information
    pre_SYNCHERR =reform(linhdr(17,*)*128l+linhdr(18,*)/2) ;synch error
    pre_biterr   =reform(linhdr(19,*)*32 + linhdr(20,*)/8) ;synch error
    pre_seterr   =reform(linhdr(21,*)) ;setup error: 254=set up error
    pre_comperr  =reform(linhdr(22,*)*256l+linhdr(23,*)) ;computer error messages
    pre_yy1      =reform(linhdr(26,*)/16*10 + (linhdr(26,*) mod 16)) ;year 2 most sig. digits
    pre_yy2      =reform(linhdr(27,*)/16*10 + (linhdr(27,*) mod 16)) ;year 2 least sig digits
    pre_year = pre_yy1*100 + pre_yy2
    pre_doy1     =reform(linhdr(28,*)/16*10 + (linhdr(28,*) mod 16)) ;day of year 2 most sig digits
    pre_doy2     =reform(linhdr(29,*)/16*10 + (linhdr(29,*) mod 16)) ;day of year 2 least sig digits
    pre_doy = pre_doy1*100 + pre_doy2
    pre_hour     =reform(linhdr(30,*)/16*10 + (linhdr(30,*) mod 16)) ;  hour:0b
    pre_min      =reform(linhdr(31,*)/16*10 + (linhdr(31,*) mod 16))
    pre_sec      =reform(linhdr(32,*)/16*10 + (linhdr(32,*) mod 16))
    pre_msec     =reform(linhdr(33,*)/16*10 + (linhdr(33,*) mod 16)) ;millisecond x 10
    pre_black    =reform(linhdr(34,*)) ;black enable: 254=annotation transmission
    pre_calib    =reform(linhdr(35,*)) ;254=C-Cal not used, otherwise ...
    pre_synclk   =reform(linhdr(36,*)) ;bit frame sync lock
    pre_limscn   =reform(linhdr(37,*)) ;limited scan mode indicator
    pre_scmode   =reform(linhdr(38,*)) ;sample control mode
    pre_vchcon   =reform(linhdr(39,*)) ;visible channel connection
    pre_scandir  =reform(linhdr(40,*)) ;scan direction 254=normal north-south scan
    pre_bpmod    =reform(linhdr(41,*)) ;bi phase modulation flag: 254=on
    pre_pllerr   =reform(linhdr(43,*)) ;PLL Error light: 254=error condition
    pre_test     =reform(linhdr(44,*)) ;test data
    pre_random   =reform(linhdr(45,*)) ;data randomization: 254=on
    pre_sunpulse =reform(linhdr(46,*)) ;Sun pulse select: 254=digital, 1=analog
    pre_nessmode =reform(linhdr(47,*)) ;NESS Mode Set
    pre_lscee    =reform(linhdr(48,*)) ;limited scan command encoder enable: 254=on
    pre_digsun   =reform(linhdr(49,*)) ;digital sun pulse
    pre_biterl   =reform(linhdr(50,*)) ;bit error light: 254=on
    pre_mnirdiff =reform(linhdr(51,*)) ;mean IR difference
    pre_rmsirdif =reform(linhdr(52,*)) ;RMS IR difference
    pre_cortab   =reform(linhdr(53,*)*256l+linhdr(54,*)) ;correction table
    pre_wedge    =reform(linhdr(55,*)*256l+linhdr(56,*)) ;western edge of earth
    pre_eedge    =reform(linhdr(57,*)*256l+linhdr(58,*)) ;eastern edge of earth
    pre_scanline =reform(linhdr(60,*)*256l+linhdr(61,*)) ;scan line number
    pre_missed   =reform(linhdr(63,*)) ;missed correlation indicator
    pre_nbslip   =reform(linhdr(64,*)) ;# of bit slip interuptions
    pre_ingest   =reform(linhdr(66,*)) ;inget of this scan line
    pre_tcode    =reform(linhdr(79,*)) ;telemetry code word
    pre_telem    =reform(linhdr(80:97,*)) ;telemetry
    pre_oa       =reform(linhdr(98:123,*)) ;26 bytes of O/A data


;set up navigation parameters
    bn = pre_oa(0,*)
    mfi =pre_oa(1,*)
    word = swap_endian(long(pre_oa(2:25,*),0,6,areadir.nline))

; assign navigation parameters from line header
    cx = dblarr(11)
    cy = dblarr(11)
    cz = dblarr(11)
    cb = dblarr(10)
    l = where(bn eq 1 and mfi eq 1)
    date1 = choosevar(word(0,l))
    time1 = choosevar(word(1,l))
    
    l = where(bn eq 1 and mfi eq 2)
    x1 =  choosevar(word(0,l))/2.d^13
    y1 =  choosevar(word(1,l))/2.d^13
    z1 =  choosevar(word(2,l))/2.d^13
    vx1 = choosevar(word(3,l))/2.d^13
    vy1 = choosevar(word(4,l))/2.d^13
    vz1 = choosevar(word(5,l))/2.d^13

    l = where(bn eq 2 and mfi eq 1)
    sper  = choosevar(word(0,l))
    spra1 = choosevar(word(1,l))/2.d^21
    spdc1 = choosevar(word(2,l))/2.d^21
    zeta  = choosevar(word(3,l))/2.d^21
    rho   = choosevar(word(4,l))/2.d^21
    eta   = choosevar(word(5,l))/2.d^21

    l = where(bn eq 2 and mfi eq 2)
    gamma = choosevar(word(0,l))/2.d^21
    names = choosevar(word(1,l))
    id    = choosevar(word(2,l))
    sra1  = choosevar(word(3,l))/2.d^21
    sdc1  = choosevar(word(4,l))/2.d^21
    gra1  = choosevar(word(5,l))/2.d^21

    l = where(bn eq 3 and mfi eq 1)
    est   = choosevar(word(0,l))
    eet   = choosevar(word(1,l))
    fper  = choosevar(word(2,l))
    tc    = choosevar(word(3,l))
    ce0   = choosevar(word(4,l))
    ce1   = choosevar(word(5,l))

    l = where(bn eq 3 and mfi eq 2)
    ce2   = choosevar(word(0,l))
    ce3   = choosevar(word(1,l))
    ce0   = choosevar(word(2,l))
    cr1   = choosevar(word(3,l))
    cr2   = choosevar(word(4,l))
    cr3   = choosevar(word(5,l))
;CHEBYSHEV BETA
    l = where(bn eq 4 and mfi eq 1)
    for j=0,5 do begin
        cb(j) = choosevar(word(j,l))/273.d/2.d^11
    endfor
    l = where(bn eq 4 and mfi eq 2)
    for j=0,3 do begin
        cb(j+6) = choosevar(word(j,l))/273.d/2.d^11
    endfor
; CHEBYSHEV CX
    l = where(bn eq 5 and mfi eq 1)
    for j=0,5 do begin
        cx(j) = choosevar(word(j,l))/2.d^13
    endfor
    l = where(bn eq 5 and mfi eq 2)
    for j=0,4 do begin
        cx(j+6) = choosevar(word(j,l))/2.d^13
    endfor
; CHEBYSHEV CX
    l = where(bn eq 6 and mfi eq 1)
    for j=0,5 do begin
        cy(j) = choosevar(word(j,l))/2.d^13
    endfor
    l = where(bn eq 6 and mfi eq 2)
    for j=0,4 do begin
        cy(j+6) = choosevar(word(j,l))/2.d^13
    endfor
; CHEBYSHEV CX
    l = where(bn eq 7 and mfi eq 1)
    for j=0,5 do begin
        cz(j) = choosevar(word(j,l))/2.d^13
    endfor
    l = where(bn eq 7 and mfi eq 2)
    for j=0,4 do begin
        cz(j+6) = choosevar(word(j,l))/2.d^13
    endfor
    
    epy   = choosevar(word(5,l))
    
    l = where(bn eq 8 and mfi eq 1)
    eph   = choosevar(word(0,l))
    sma   = choosevar(word(1,l))/100.d0
    ecc   = choosevar(word(2,l))/1000000.d0
    inc   = choosevar(word(3,l))/1000.d0
    ma    = choosevar(word(4,l))/1000.d0
    ap    = choosevar(word(5,l))/1000.d0
    
    l = where(bn eq 8 and mfi eq 2)
    ran   = choosevar(word(0,l))/1000.d0
    sbscan= choosevar(word(1,l))/100.d0
    sbsamp= choosevar(word(2,l))/100.d0
    sblat = choosevar(word(3,l))/100.d0
    sblong= choosevar(word(4,l))/100.d0
    oyaw  = choosevar(word(5,l))/100.d0
    
    l = where(bn eq 9 and mfi eq 1)
    sbscanb=choosevar(word(0,l))
    sbsampb=choosevar(word(1,l))
    sblatb= choosevar(word(2,l))
    sblonb= choosevar(word(3,l))
    oyawb = choosevar(word(4,l))
    
    l = where(bn eq 9 and mfi eq 2)
    x2 =  choosevar(word(0,l))/2.d^13
    y2 =  choosevar(word(1,l))/2.d^13
    z2 =  choosevar(word(2,l))/2.d^13
    vx2 = choosevar(word(3,l))/2.d^13
    vy2 = choosevar(word(4,l))/2.d^13
    vz2 = choosevar(word(5,l))/2.d^13
    
    l = where(bn eq 10 and mfi eq 1)
    spra2 = choosevar(word(1,l))/2.d^21
    spdc2 = choosevar(word(2,l))/2.d^21
    deltal= choosevar(word(3,l))/2.d^21
    deltas= choosevar(word(4,l))/2.d^21
    
    l = where(bn eq 10 and mfi eq 2)
    sra2  = choosevar(word(3,l))/2.d^21
    sdc2  = choosevar(word(4,l))/2.d^21
    gra2  = choosevar(word(5,l))/2.d^21
    
;test the parameters
    navcheck,[date1,time1], $
             [x1,y1,z1],[vx1,vy1,vz1], $
             [x2,y2,z2],[vx2,vy2,vz2], $
             cx,cy,cz, $
             [epy,eph],[sma,ecc,inc,ran,ap,ma],result
    
; find bounds of valid scans
    ny = areadir.nline
    for i=0,ny-1 do begin
        if (i eq 0) then begin
            if ((pre_scancount(i)+1) eq (pre_scancount(i+1)) and $
                pre_scode(i) eq 254) then break
        endif else begin
            if ((pre_scancount(i)-1) eq (pre_scancount(i-1)) and $
                pre_scode(i) eq 254) then break
        endelse
    endfor
    scan1 = i
    for i=ny-1,0,-1 do begin
        if (i eq 0) then begin
            if ((pre_scancount(i)+1) eq (pre_scancount(i+1)) and $
                pre_scode(i) eq 254) then break
        endif else begin
            if ((pre_scancount(i)-1) eq (pre_scancount(i-1)) and $
                pre_scode(i) eq 254) then break
        endelse
    endfor
    scan2 = i

;get julian date of each scan line
    gd = where(pre_year gt 2 and pre_doy lt 367)
    
    jj = julday(pre_doy(gd)*0+1,pre_doy(gd),pre_year(gd), $
                pre_hour(gd),pre_min(gd),pre_sec(gd)+pre_msec(gd)/100.)
    t1 = julday(1,date1 mod 1000,date1/1000+1900,0,0,time1/100.)
    t1=t1(0)
    lintim = dblarr(ny)
    lintim(*) = t1
    lintim(gd) = jj
;get location at start and end of image
    sc = [scan1,scan2]
;tim is # of seconds since epoch for 1st and last scan
    epoch1 = yyjjj_2_julday(yymmdd_2_yyjjj(epy),time=eph,/itime)
    tim = (lintim(sc)-epoch1)*24.d0*3600.d0

    kep2cart,[sma,ecc,inc,ran,ap,ma],tim,x,y,z, $
             glat=glat,glon=glon,grad=grad,sat=lintim(sc)
    u = 2.d*(lintim(sc)-t1)/13.d - 1.
    if (abs(spra1-spra2) gt 70.) then stop
    if (abs(spdc1-spdc2) gt 70.) then stop
    spra = 0.5d0 * (spra1 + spra2 + (spra2-spra1)*u)
    spdc = 0.5d0 * (spdc1 + spdc2 + (spdc2-spdc1)*u)

    grad1c = grad(0)
    grad2c = grad(1)
    glat1c = glat(0)
    glat2c = glat(1)
    glon1c = glon(0)
    glon2c = glon(1)
    spra1c = spra(0)
    spra2c = spra(1)
    spdc1c = spdc(0)
    spdc2c = spdc(1)
endif else begin
    ;//////////////////////////////
    ;SET UP WHEN LINHDR IS MISSING
    ;\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
    ny = areadir.nline
    for i=0,ny/2 do begin
        if (total(image(*,i)) gt 100) then break
    endfor
    scan1 = i
    for i=ny-1,ny/2,-1 do begin
        if (total(image(*,i)) gt 100) then break
    endfor
    scan2 = i

endelse


; Get TLE from NORAD data
f = strsplit(file,'.',/extr)
sat=f(3)
st = strsplit(sat,'-',/extr)
plot_tle,tle,sat='goes-'+st(1), $
         year=1900+area.date/1000l, $
         jday=area.date mod 1000l,$
         ORBstr=TLEORBstr

; GET NAVIGATION INFORMATION


; set up kepler COMMON BLOCK
semima = double(nav.semima)/100.
incl   = double(nav.orbinc)/1000. 
eccen  = double(nav.eccen)/1.e5
meana  = double(nav.meana)/1000.
rasc   = double(nav.asnode)/1000.
perigee= double(nav.perigee)/1000. 
edate  = yymmdd_2_yyjjj(nav.etimy) ;YYMMDD
etime  = NAV.etimh ;HHMMSS 
msec = 0.0

;set up att COMMON BLOCK
roll  = double(ftime(nav.roll))
pitch = double(ftime(nav.pitch))
yaw   = double(ftime(nav.yaw))
SPRA1 = double(ftime(nav.rascen))
SPRA2 = double(ftime(nav.rascen))
SPDC1 = double(ftime(nav.declin))
SPDC2 = double(ftime(nav.declin))

;set up image COMMON BLOCK
sline = area.ulline
spix  = area.ulelem
cen_lin = nav.piclin
cen_pix =nav.eletot / 2
linres = area.lineres
pixres = area.elemres
linang = double(ftime(nav.deglin))/double((nav.lintot/100000)*(nav.lintot mod 100000))  ;deg/line
linang = -linang
pixang = double(ftime(nav.degele))/double(nav.eletot) ;deg/pixel
pixang = -pixang
; get image time
if (area.sss gt 25 and nav.bt1_scan gt 0) then begin
    time1a = ftime(nav.bt1_time) + nav.bt1_tim2 / 10. / 60. / 60.
endif else begin
    time1a = ftime(nav.time)
endelse
frtime = time1a * 3600.d0
spper = nav.spinp / 1.d6

frdate = nav.iddate mod 100000l

;set up green COMMON BLOCK
GRA1 = 0.d0 
GRA2 = 0.d0 
gt1 =  0.d0 
gt2 =  13.d0 *3600.d0

navstr=navtmp
areastr=areatmp
;
;Assign variables to structures
;

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
orbit.ep_date = edate
orbit.ep_time = etime
orbit.ep_msec = msec
orbit.semima = semima
orbit.incl = incl
orbit.eccen = eccen
orbit.meana = meana
orbit.rgtasc = rasc
orbit.perigee = perigee
orbit.sper = spper
ORBstr = orbit

;All values are directly from the common block, so this if can be
;copied to each B1READ routine
imginfo.frdate = frdate
imginfo.frtime = frtime
imginfo.cen_lin = cen_lin
imginfo.cen_pix = cen_pix
imginfo.linres = linres
imginfo.pixres = pixres
imginfo.nlin = AREAdir.nline
imginfo.npix = AREAdir.nelem
imginfo.sline = sline
imginfo.spix = spix
imginfo.linang = linang
imginfo.pixang = pixang
IMGstr = imginfo

att.roll = roll
att.pitch = pitch
att.yaw = yaw
;need to calculate the current spra and spdc first
att.spra = spra1                ;spnax(0)
att.spdc = spdc1                ;spnax(1)
att.rma = 0.d0
att.pma = 0.d0
ATTstr = att

sattim.d = gt2-gt1
sattim.gt1 = gt1
sattim.gt2 = gt2
sattim.gra1 = gra1
sattim.gra2 = gra2
sattim.spra1 = spra1 ;-999.d0
sattim.spra2 = spra1 ;-999.d0
sattim.spdc1 = spdc1 ;-999.d0
sattim.spdc2 = spdc1 ;-999.d0
sattim.cheb_x = -999.d0         ;ch_x
sattim.cheb_y = -999.d0         ;ch_y
sattim.cheb_z = -999.d0         ;ch_z
sattim.cheb_a = -999.d0
sattim.cheb_b = -999.d0
sattim.cheb_c = -999.d0
TIMstr = sattim

if (siz(0) eq 3) then nchan = siz(3) else nchan = 1
;if (nchan eq 1) then stop
chninfo.nchan = nchan
chninfo.ch_name(1) = 'IRWIN'
chninfo.ch_name(0) = 'VSCHN'
if (nchan eq 1) then chninfo.ch_name(0) = 'IRWIN'
chninfo.ch_info = 'none'
CHNstr = chninfo

AREAstr=areatmp
NAVstr=navtmp

nx = imgstr.npix
ny = imgstr.nlin
ii = [nx/2,nx/2]
jj = [1,ny-1]
b1u_ij2ll,ii,jj,lat,lon, $
          b1TIM=TIMstr,b1ATT=ATTstr, $
          b1ORB=ORBstr,b1IMG=IMGstr, $
          glat=glat,glon=glon,grad=grad
glon = glon * 1e6
glat = glat * 1e6
grad = grad * 1e3

;---------------------------------------------------
;\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/\_/
;
; BEGIN B1U array definitions
;
MISSING = -9999l
REVinf = strarr(6)
IMGinf = lonarr(24)
SATinf = strarr(34)
NAVinf = lonarr(200)
PREinf = intarr(nchan,20,areadir.nline)

;REVISION INFORMATION
REVinf(*) = 'Missing'
REVinf(0) = stripname(file)
REVinf(3) = '$Id: b1read_uws_g56.pro 22774 2010-09-16 12:29:16Z kknapp $'

IMGinf(*) = MISSING
IMGinf(0) = areadir.date
IMGinf(1) = areadir.time
IMGinf(2) = areadir.sss
IMGinf(3) = areadir.nline
IMGinf(4) = areadir.nelem
IMGinf(5) = nchan
IMGinf(6) = areadir.bytes
if (lf eq 1) then IMGinf(7) = total(pre_scandir eq 254) gt 10 else IMGinf(7) = 1;N-S orientation
IMGinf(8) = 0                   ;E-W orientation
IMGinf(9) = MISSING             ;Vis averaging: -999=MISSING
IMGinf(10) = areadir.valid eq 0
IMGinf(11) = areadir.validity
IMGinf(12) = 0                  ;no image byte offset
IMGinf(13) = 0                  ;neither Imager or Sounder

;Satellite information
i=0
SATinf(*) = 'Missing'
SATinf(i++) = 'GOES-'+st(1)
SATinf(i++) = 'VISSR'
for j=0,nchan-1 do begin
    SATinf(i++) = chninfo.ch_name(j)
endfor
;  GOES-5 channels
;info will be FWHM bounds ... estimated from the ISCCP web page
g5info = ['0.52-0.69 um','10.4-12.1 um']
g6info = ['0.52-0.70 um','10.4-12.1 um']
for j=0,nchan-1 do begin
    if (st(1) eq '5') then begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g5info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g5info(1)
    endif else begin
        if (chninfo.ch_name(j) eq 'VSCHN') then tmp = g6info(0)
        if (chninfo.ch_name(j) eq 'IRWIN') then tmp = g6info(1)
    endelse
    SATinf(i++) = tmp
endfor
;---------------------------
; Navigation information
; Keplarian information
;IMGDAT
NAVinf(0) = area.ulline
NAVinf(1) = area.ulelem
NAVinf(2) = nav.piclin * 10.
NAVinf(3) = nav.eletot * 10. / 2
NAVinf(4) = area.lineres
NAVinf(5) = area.elemres
NAVinf(6) = round(-(double(ftime(nav.deglin))/ $
                    double((nav.lintot/100000)*(nav.lintot mod 100000)))*1e9)
NAVinf(7) = round(-(double(ftime(nav.degele))/double(nav.eletot))*1e9)
NAVinf(8) = spper*1e6         ;Seconds per revolution*10^6

; Keplarian information
NAVinf(09) = 10
NAVinf(10) = yymmdd_2_yyjjj(nav.etimy)
NAVinf(11) = NAV.etimh
NAVinf(12) = 0l
NAVinf(13) = nav.semima
NAVinf(14) = nav.orbinc
NAVinf(15) = nav.eccen*10l
NAVinf(16) = nav.meana
NAVinf(17) = nav.asnode
NAVinf(18) = nav.perigee
;other Keplerian info
NAVinf(19:28) = MISSING
; other Keplerian info
NAVinf(29:38) = MISSING
; Attitude data
NAVinf(39) = round(ftime(nav.roll) * 1.e6)
NAVinf(40) = round(ftime(nav.pitch)* 1.e6)
NAVinf(41) = round(ftime(nav.yaw)  * 1.e6)
; ATTFIT
NAVinf(42:48) = MISSING
; RECINFO & FITINFO
NAVinf(49:56) = MISSING
; POSDAT
if (lf) then begin
    NAVinf(57) = date1
    NAVinf(58) = time1
    NAVinf(59) = round(x1*2.^13)
    NAVinf(60) = round(y1*2.^13)
    NAVinf(61) = round(z1*2.^13)
    NAVinf(62) = round(vx1*2.^13)
    NAVinf(63) = round(vy1*2.^13)
    NAVinf(64) = round(vz1*2.^13)
    NAVinf(65) = round(spra1*2.^21)
    NAVinf(66) = round(spdc1*2.^21)
    NAVinf(67) = round(zeta*2.^21)
    NAVinf(68) = round(rho*2.^21)
    NAVinf(69) = round(eta*2.^21)
    NAVinf(70) = round(gamma*2.^21)
    NAVinf(71) = round(sra1*2.^21)
    NAVinf(72) = round(sdc1*2.^21)
    NAVinf(73) = round(gra1*2.^21)
    NAVinf(74) = round(est*2.^21)
    NAVinf(75) = round(eet*2.^21)
    NAVinf(76:85)  = round(cb*273.*2.^11)
    NAVinf(86:96)  = round(cx*2.^13)
    NAVinf(97:107) = round(cy*2.^13)
    NAVinf(108:118)= round(cz*2.^13)
    NAVinf(119) = round(x2*2.^13)
    NAVinf(120) = round(y2*2.^13)
    NAVinf(121) = round(z2*2.^13)
    NAVinf(122) = round(vx2*2.^13)
    NAVinf(123) = round(vy2*2.^13)
    NAVinf(124) = round(vz2*2.^13)
    NAVinf(125) = round(spra2*2.^21)
    NAVinf(126) = round(spdc2*2.^21)
    NAVinf(127) = round(sra2*2.^21)
    NAVinf(128) = round(sdc2*2.^21)
    NAVinf(129) = round(gra2*2.^21)
;LOCDAT
endif
NAVinf(130:142) = MISSING
NAVinf(131) = 1
NAVinf(132) = glat(0)
NAVinf(133) = glon(0)
NAVinf(134) = grad(0)
NAVinf(137) = ny-1
NAVinf(138) = glat(1)
NAVinf(139) = glon(1)
NAVinf(140) = grad(1)
NAVinf(130) = MISSING
;MISC
NAVinf(143) = 0
NAVinf(144) = nav.iajust
NAVinf(145) = MISSING
NAVinf(146) = MISSING
NAVinf(147) = frdate
NAVinf(148) = frtime
NAVinf(149) = AREAdir.nline
NAVinf(150) = AREAdir.nelem

NAVinf(151:199) = MISSING


; TEST GRID - None provided from UWS
TGRinf = MISSING

; Line prefix information
;PREinf = intarr(nchan,20,areadir.nline)
i=0
PREinf(*) = MISSING
if (lf) then begin
    PREinf(*,i++,*) = MISSING   ;relative scanline
    PREinf(*,i++,*) = rebin(reform(pre_scancount,1,1,ny),nchan,1,ny) ;absolute scan line
    PREinf(*,i++,*) = rebin(reform(indgen(nchan),nchan,1,1),nchan,1,ny) ; channel number
    PREinf(*,i++,*) = rebin(reform(pre_year,1,1,ny),nchan,1,ny) ;year
    PREinf(*,i++,*) = rebin(reform(pre_doy,1,1,ny),nchan,1,ny) ;doy
    PREinf(*,i++,*) = rebin(reform(pre_hour*100+pre_min,1,1,ny),nchan,1,ny) ;HHMM
    PREinf(*,i++,*) = rebin(reform(pre_sec,1,1,ny),nchan,1,ny) ;Sec
    PREinf(*,i++,*) = rebin(reform(pre_msec*10,1,1,ny),nchan,1,ny) ;MSec
    PREinf(*,i++,*) = rebin(reform(pre_eedge,1,1,ny),nchan,1,ny) ;eastern edge of earth
    PREinf(*,i++,*) = rebin(reform(pre_wedge,1,1,ny),nchan,1,ny) ;western edge of earth
    PREinf(*,i++,*) = MISSING   ;detector ID
    PREinf(*,i++,*) = MISSING   ;validity code
    PREinf(*,i++,*) = total(images,2) mod 256 ;checksum
endif
b1u = {rev:REVinf,$
       img:IMGinf,$
       sat:SATinf,$
       nav:NAVinf,$
       tgr:TGRinf,$
       pre:PREinf,$
       image:images}      
;
; END B1U array definitions
;---------------------------------------------------
;stop
endd:
end
