;****** grisat/grisat
; NAME
;  GriSat
; PURPOSE
; Gridded Satellite data
;
; The concept is to provide satellite data in a gridded file to
; simplify access to satellite data for non-satellite experts. Users
; should still learn about the proper use of satellite data, however,
; data access is no longer a huindrance with GriSat. As initially
; implemented, the GriSat data consist of ISCCP B1 data. However, a
; similar gridding of AVHRR and SSMI data at different resolutions
; would likely be beneficial to the community.
; 
; The data are provided in netCDF and CF compliant conventions to
; cimplify access by automated tools.
;
; NOTES
;
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
;
; CREATION DATE
; 2009-2010
;
; HISTORY
;***
;****** grisat-b1/grisat-b1
; NAME
; GriSat-B1
; 
; PURPOSE
; Gridded Satellite data - B1
;
; The data are brightness temperatures at channels available from
; ISCCP B1 data. The global channels are:
; - visible (0.6 um)
; - infrared window (11 um)
; - infrared water vapor (6.7 um)
;
; The data are provided in netCDF and CF compliant conventions to
; cimplify access by automated tools.
;
; NOTES
; The processing of GriSat-B1 data is  based on this IDL
; routine. However, many things need to be in place prior to this last
; step. The following is a summary of the Processing steps required to
; create GriSat-B1 data:
;
; 1. Convert ISCCP B1 files to B1U files.
; Data from the archive are in numerous formats, so processing is
; simplified by unifying the format. The file b1_2_b1u.pro does the
; bulk of the work in reformating and creating the B1U files.
;
; 2. Create Match files
; The Match files are lists of temporally co-located satellite
; files. The files are created by match_b1u_poes.pro
; In general, this lists coincident B1U files along with SSMI, HIRS
; and ISCCP B2 AVHRR files. This simplifies looking for matching
; pixels by providing the list of temporally located files. Match
; files are produced at 3-hr resolution and point to files located
; across NCDC servers (in current configuration, using the nfs
; connections.
; 
; 3. Co-locate ISCCP B1U with HIRS all sky radiances
; This step is performed by a Fortran routine:
; b1u_HIRS-path-AS_matchup.f90
; This process matches B1U observations with HIRS when both have
; nearly the same viewing geometry. The output of this routine is then
; used to create a calibration correction that normalizes B1U
; calibration to HIRS data. It currently matches data for the IRWIN,
; IRWVP and IRSPL channels. The output of these steps are called the
; calibration match files.
;
; 4. Create monthly calibration coefficients
; Process the instantaneous calibration match files (output from step
; 3) to monthly corrections. This is presently performed via a
; complicated process of automated and manual steps. The final output
; of this step is the calibration tables, which are then used by
; b1u_2_grisat.pro to create GriSat-B1 files. The calibration
; coefficient table is created by 
;
; 5. Create GriSat-B1 data.
; Run this algorithm to create grisat-b1 data. This is generally run
; from a script and trolled (troll.batch_runner)
; 
; USES
; idl.b1u_2_grisat (b1u_2_grisat.pro)
;
; TODO
;
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
;
; CREATION DATE
; 2009-2010
;
; HISTORY
;***

;****I* idl/idl.b1u_2_grisat
; NAME
; b1u_2_grisat
; 
; PURPOSE
; To reformat, remap and calibrate ISCCP B1U files as netCDF data.
; 
; USAGE
; IDL> b1u_2_grisat,'/home/kknapp/DATA/isccp/match-poes/1984/B1U-POES.1984.11.26.12.match'
; 
; INPUT
; year - year for which to process gridded data
; istart(optional) - index at which to restart gridding
; debug(optional) - debug flag
; parallax(opitonal) - perform the parallax correction on IRWIN (NOT
;                      IMPLEMENTED YET)
; 
; mfilein(optional) - specify one match-poes file to process (useful
;                     when running with troll.batch_runner
; allonly(optional) - Only process the ALL files, not individual basins.
; VZAcorr(obsolete) - Limb-correct IRWIN temps. However, this is now
;                     the default, so the option is obsolete.
;                     
; OUTPUT
; Files are output to:
;  '~/datasan/grisat/'+cyear
;  
; USES
; * Files from the isccp_b1u_production
; * Results of b1u_calibration_procedure
; * 
; * idl.angles
; * idl.b1u_ij2ll
; * idl.b1u_ll2ij
; * idl.b1u_dn2cal
; * idl.b1uread
; * idl.date_jul2iso
; * idl.date_mmm_2_mm
; * idl.flines
; * idl.geo_hirs_limb_corr
; * idl.i2xy
; * idl.parallax_correct_swath
; * idl.read_match_poes
; * idl.remove_gz
; * idl.satellite_temp2image.pro 
; * idl.stripname
; * idl.systime_iso
; * idl.szacorr
; 
; USED BY
; 
; NOTES
; 
; TODO
;    
; BUGS
; * The szacorr seems to still leave seams. An new correction should
;   be derived from the HIRS/B1U matchups
; 
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; * ~Jan. 2007
; 
; HISTORY
; * 25Jun07 - Modifying for output to work with NCO
; * 01Feb09 - MOdifying to output grisat files ... instead of HURSAT
; * 01Mar09 - Added second layer of observations to keep all VZAs
; 
; SOURCE

pro b1u_2_grisat,year, $
                 istart=istart, $
                 debug=debug, $
                 noparallax=noparallax, $
                 VZAcorr=VZAcorr, $
                 mfilein=mfilein, $ ;input the matchup file
                 allonly=allonly, $
                 nostorm=nostorm, $ ;don't include TC storm information
                 ay=ay,$        ;for Alisa, only process ALL data and apply the intercalibration
                 timer=timer, $
                 noplot=noplot, $
                 nosparse=nosparse
timer,/start,num=9
cvsver='$Id: b1u_2_grisat.pro 22801 2010-09-16 13:26:22Z kknapp $'

;FOR CALIBRATION ir==IRWIN, wv==Water Vapor

;VERSION NUMBER
ircfile = 'b1u.calibration.correction.IRWIN.v15'
wvcfile = 'b1u.calibration.correction.IRWVP.v14'
;_________________________
hursatver='beta'
sdir      ='basin'
hursatname='grisat-b1'
subdir = sdir+'-'+hursatver
ncdir = getenv('HURSATDIR') + '/'+subdir+'/'
ncdir = '/net/glaze/home/kknapp/datasan/grisat/'
ang = 0.07
caldir = getenv('ISCCPDIR')+'/calib/'
matdir=getenv('ISCCPDIR')+'/match-poes/'
vzalimit = 85.
;_________________________

MISSING = -31999
MISSINGb= 255b

counter=0

;gridonly=1 <- do not write out sparse data, but only the complete
;              grids
;gridonly=0 <- allow mostly empty arrays to be sparse
if keyword_set(nosparse) then gridonly=1 else gridonly=0

print,'GRIDDING: (0) 2D&Sparse, (1) 2D only  === ',gridonly

if n_elements(year) eq 0 then year = 2005
if n_elements(satfilt) eq 0 then satfilt='*'
if n_elements(istart) eq 0 then istart=0
idebug=0
if keyword_set(debug) then begin
    idebug=debug
    mfilein = '/home/kknapp/DATA/isccp/match-poes/2006/B1U-POES.2006.10.20.09.match'
    mfilein = '/home/kknapp/DATA/isccp/match-poes/2006/B1U-POES.2006.09.01.06.match'
    mfilein = '/home/kknapp/DATA/isccp/match-poes/2006/B1U-POES.2006.10.30.15.match'
    mfilein = '/home/kknapp/DATA/isccp/match-poes/2008/B1U-POES.2008.09.01.09.match'

    mfilein = '/home/kknapp/DATA/isccp/match-poes/1981/B1U-POES.1981.01.02.03.match'
    mfilein = '/home/kknapp/DATA/isccp/match-poes/1985/B1U-POES.1985.08.31.18.match'
    mfilein = '/home/kknapp/DATA/isccp/match-poes/1997/B1U-POES.1997.08.31.18.match'
;    mfilein = '/home/kknapp/DATA/isccp/match-poes/1990/B1U-POES.1990.10.07.18.match'

    mfilein = '/home/kknapp/DATA/isccp/match-poes/2005/B1U-POES.2005.08.28.12.match';Katrina

    mfilein = '/home/kknapp/DATA/isccp/match-poes/2009/B1U-POES.2009.10.06.06.match'
    mfilein = '/home/kknapp/DATA/isccp/match-poes/1993/B1U-POES.1993.04.20.18.match'
;    mfilein = '/home/kknapp/DATA/isccp/match-poes/2006/B1U-POES.2006.01.20.09.match'
endif
if keyword_set(mfilein) then begin
    f=strsplit(mfilein,'.',/extr)
    year = fix(f(1))
endif

nostorm=1

istart=long(istart)
year1=year
year2=year

nbasins = 1
basins = ['ALL']
basin_name = ['Global Grid']

;If set, only produce the ALL basin
runbasins=nbasins
if keyword_set(allonly) then runbasins=1

latmin = -70
latmax =  70.
lonmin = -180.
lonmax =  180.

;Set up the initial grid
ngx = ceil((lonmax-lonmin)/ang)
ngy = ceil((latmax-latmin)/ang)

;if keyword_set(debug) then window,xs=ngx/4,ys=ngy/4*3



lat1 = findgen(ngy) * ang + latmin
lon1 = findgen(ngx) * ang + lonmin

lat = rebin(reform(lat1,1,ngy),ngx,ngy)
lon = rebin(reform(lon1,ngx,1),ngx,ngy)

corner_lat = [0., 0., 0., 0., 0.]
corner_lon = [0., 63., 140., -135., -75.]




;FINAL CALIBRATION
;get final calibration information from 

;IRWIN

calfile = caldir + ircfile
ncal = flines(calfile) - 1
openr,1,calfile
calinfo=' '
readf,1,calinfo
ircname = strarr(ncal)
ircslope = fltarr(ncal)
ircoffset= fltarr(ncal)
s=''
for i=0,ncal-1 do begin
    readf,1,s
    g=strsplit(s,' ',/extr)
    ircname(i)   = g(0)
    ircslope(i)  = g(1)
    ircoffset(i) = g(2)
endfor
close,1

;IRWVP

calfile = caldir + wvcfile
ncal = flines(calfile) - 1
openr,1,calfile
calinfo=' '
readf,1,calinfo
wvcname = strarr(ncal)
;wvcalcoef = fltarr(4,ncal)
;wvcalstat = fltarr(2,ncal)
wvcslope =  fltarr(ncal)
wvcoffset=  fltarr(ncal)
s=''
for i=0,ncal-1 do begin
    readf,1,s
    g=strsplit(s,' ',/extr)
    wvcname(i)   = g(0)
    wvcslope(i) = g(1)
    wvcoffset(i)= g(2)
;    wvcalcoef(*,i) = g(1:4)
;    wvcalstat(*,i) = g(5:6)
endfor
close,1


;remove N17 from the beginning
;wvcname = strmid(wvcname,4,100)



;grids to hold primary observations
;satidi = bytarr(ngx,ngy)
;satidv = satidi
;satidw = satidi

;irtemp = fltarr(ngx,ngy)
;wvtemp = fltarr(ngx,ngy)
;vsalb = wvtemp

;gdvzair = intarr(ngx,ngy)       ;wvtemp
;gdvzawv = gdvzair               ;wvtemp
;gdvzavs = gdvzair               ;wvtemp

;hold the secondary observations (when another satellite is present,
;these will be the observations with the next highest VZA)
;satidi2 = satidi
;satidw2 = satidi
;satidv2 = satidi

;irtemp2 = irtemp
;wvtemp2 = wvtemp
;vsalb2  = vsalb

;gdvzair2 = gdvzair               ;wvtemp
;gdvzawv2 = gdvzair               ;wvtemp
;gdvzavs2 = gdvzair               ;wvtemp

;tertiary observations for IR only
;satidi3 = satidi

;irtemp3 = irtemp

;gdvzair3 = gdvzair               ;wvtemp


;directory containing B1 files
for iyear=year1,year2 do begin
    yrdir = string(iyear,format='(i4.4)')

    if keyword_set(mfilein) then begin
        mfiles = mfilein
        nmfiles = 1
        istart=0
    endif else begin
                                ;get list of merged files
        mfiles = file_search(matdir+yrdir+'/B1U-POES.*',count=nmfiles)
    endelse

    ;for each merged file:
    for imfile=istart,nmfiles-1 do begin
        mfile=mfiles(imfile)
        mf = strsplit(stripname(mfile),'.',/extr)
        myear = fix(mf(1))
        mmon  = fix(mf(2))
        mdom  = fix(mf(3))
        mhh   = fix(mf(4))
        read_match_poes,file=mfile,b1u=geolist
;geolist=reverse(geolist)
        ngeo = n_elements(geolist)
        print,imfile,' ',stripname(mfiles(imfile)),ngeo
        numsat=0
        satlat=fltarr(10)
        satlon=fltarr(10)
        satrad=fltarr(10)
        satname  =strarr(10)
        b1u_file=strarr(10)
        b1_file =strarr(10)
        irsatslp=fltarr(10)
        irsatoff=fltarR(10)
        wvsatslp=fltarr(10)
        wvsatoff=fltarR(10)
        
        vs_rad_slope=fltarr(10)
        vs_dc_slope= fltarr(10)
        vs_dc_offset=fltarr(10)

        ;wvsatcoef=fltarr(2,10)
        satidi = replicate(MISSINGb,ngx,ngy)
        satidv = satidi
        satidw = satidi
        satidi2 = satidi
        satidv2 = satidi
        satidw2 = satidi
        satidi3 = satidi
        final_cal_flag = 0
        ;for each geostationary satellite
        numgeo=ngeo

        vschnpresent=0
        irwvppresent=0
        irwinpresent=0
        
        igstart=0
        if keyword_set(debug) then igstart=0

        for igeo=igstart,numgeo-1 do begin
;        for igeo=1,1 do begin
            gfile = geolist(igeo)
            ;read the file
            b1uread,file=gfile,b1u=b1u

            tagnames = tag_names(b1u)
            if n_elements(tagnames) eq 0 then $
              message,'ISCCP B1U files returned no B1U structure: '+gfile

            b1u_file(igeo) = stripname(gfile)
            b1_file(igeo)  = strtrim(b1u.rev(0),2)
            nch = b1u.img(5)
            nx = b1u.img(4)
            ny = b1u.img(3)

            ;is there enough scan lines to count?
            if (ny lt 150) then goto,nextgeo

            ;can the satellite see the grid?
; This test was used for HURSAT basin when each basin was calculated
; individually, it has no equivalent for global application since each
; satellite should see some part of the grid. Thus nok will be hard
; coded to 1.
; The problem was that the eclipse replacement image was not seeingthe
; Equator and the test was used for known satellite
; subpoints. However, these aren't always observed. So will skip this
; test for gridding, however the navigation call is used to get
; satellite subpoints, so will keep it for now

            b1u_ll2ij,corner_lat,corner_lon,ci,cj,nav=b1u.nav, $
                      glat=glat,glon=glon,grad=grad
;            ok = where(ci ge 0 and ci lt nx and $
;                       cj ge 0 and cj lt ny,nok)
nok=1
if (nok eq 0) then stop
            ;set up satellite variables
            satname(igeo) = strtrim(b1u.sat(0),2)
            satlat(igeo) = glat(0)
            satlon(igeo) = glon(0)
            satrad(igeo) = grad(0)

;IGNORE FY-2C
            ;IGNORE FY-2C data for now.
            if (strmid(satname(igeo),0,2) eq 'FY') then begin
                ;ignore FY only if MTSAT-1R or -2 exists at this time
                if (total(strpos(geolist,'MTS-1') ge 0) gt 0 or $
                    total(strpos(geolist,'MTS-2') ge 0) gt 0) then nok = 0
            endif

;IGNORE MET-5
            ;Also, IGNORE MET-5 data after MET-7 begins Indian Ocean Coverage
            ;BUT ONLY IF MET-7 data is present
            ;although this test should really
            ;depend on MET-7 producing useful data.. except for Nov. 2006
            if (strpos(gfile,'MET-5') ge 0 and $
                total(strpos(geolist,'MET-7') ge 0) gt 0) then begin
                ;check dates
                if (myear gt 2006 or $
                    (myear eq 2006 and mmon gt 11)) then nok = 0
            endif else if (strpos(gfile,'MET-7') ge 0 and $
                total(strpos(geolist,'MET-5') ge 0) gt 0) then begin
                ; if current sat=MET-7 and MET-5 is present and
                ; current time is Nov. 2006, then ignore MET-7
                ;check dates
                if (myear eq 2006 and mmon eq 11) then nok = 0
            endif
;IGNORE MSG-1
            ;IGNORE MSG-1 when MET-7 is present before Oct. 2006
            ;although this test should really
            ;depend on MET-7 producing useful data..
            if (strpos(gfile,'MSG-1') ge 0 and $
                total(strpos(geolist,'MET-7') ge 0) gt 0) then begin
                ;check dates
                if (myear lt 2006 or $
                    (myear eq 2006 and mmon lt 11)) then nok = 0
            endif
nok=1
            print,satname(igeo),nok
            ;if so, include the observations in the grid
            if (nok gt 0) then begin
                if (numsat eq 0) then begin
                    ;set up the variables on the first pass through
                    irtemp = replicate(-1.,ngx,ngy)
                    wvtemp = irtemp
                    vsalb  = irtemp

                    gdvzair = replicate(1000,ngx,ngy)
                    gdvzawv = gdvzair
                    gdvzavs = gdvzair

                    gdvzair2 = gdvzair
                    gdvzawv2 = gdvzair
                    gdvzavs2 = gdvzair                    

                    gdvzair3 = gdvzair
                    
                    irtemp2 = irtemp
                    wvtemp2 = irtemp
                    vsalb2  = irtemp
                    
                    irtemp3 = irtemp
                    

                    ;get TS information on the first pass through
                    ;read_jk_hurr,iyear,num,tcname,span,tclat,tclon,tctime,tcind, $
                    ;             wind=tcwind,tcname=wholename,pres=tcpres
                endif

                ;read in the QC information
                if (total(strpos(tagnames,'QC') ge 0) gt 0) then begin
                    qc = b1u.qc
                endif else begin
                    gfile_nogz = remove_gz(gfile)               

;                   This option uses b1qc to read an recalc QC as
;                   needed. HOWEVER, b1qc is not mature.
;                   b1qc,gfile_nogz,qc


                    qc=get_b1u_qcqa(gfile_nogz,nch,ny)
;                    if (file_test(gfile_nogz+'.QCQA')) then begin
;                        a0=0l & a1 = 0l & a2 = 0l
;                        openr,1,gfile_nogz+'.QCQA'
;                        readu,1,a0,a1,a2
;                        qc = lonarr(
;                        readu,1,qc
;                        close,1
;                    endif
                endelse

                ;navigate the grid points
                ;select points most likely visible from satellite
                dlon = abs(lon-glon(0))
                fixit = where(dlon gt 180,nfix)
                if (nfix gt 0) then begin
                    dlon(fixit) = 360. - dlon(fixit)
                endif

;                navigate grid points near the sub satellite point.
;                limit was found by slowly decreasing making sure nuse
;                does not change. 74 deg changes nuse, so keeping 75. 
                nearby = where(dlon lt 75.)
dlon=0b
                chans = b1u.sat(2:2+nch-1)
                btemp = b1u_dn2cal(chans,b1u,frac=frac,debug=debug)
                vs_rad_slope(igeo) = frac(0)
                vs_dc_slope(igeo)  = frac(1)
                vs_dc_offset(igeo) = frac(2)
if keyword_set(debug) then stop
                    ;if calibration fails, then btemp=-100, so
                    ;I added the "and n_elements(btemp) gt 100" below
                    ;to the if statements
                if (n_elements(btemp) gt 100) then begin

                ;navigate each grid point
;print,memory()/1024/1024

if keyword_set(debug) then print,'Navigating grid points'
if keyword_set(timer) then timer,/start,number=0

b1u_ll2ij,lat(nearby),lon(nearby),dgridi,dgridj, $
          nav=b1u.nav,glon=glon,chunk=10000l

gridi=round(dgridi)
gridj=round(dgridj)
dgridi=0
dgridj=0

if keyword_set(timer) then timer,/stop,number=0,/print

;print,memory()/1024/1024

;no use timing call to angles because it is so fast (< 1% of total run
;time)
;if keyword_set(debug) then print,'Grid point angles'
;if keyword_set(timer) then timer,/start,number=1
angles,lat(nearby),lon(nearby),0.,0.,temp1,temp2,vza,azi, $
       /satonly,inglon = glon(0) + 0.0001
;if keyword_set(timer) then timer,/stop,number=1
;print,memory()/1024/1024

                ;keep points with vza < gdvza which are visible to
                ;the satellite
;timer,/start
                use= where(gridi ge 0 and gridi lt nx and $
                           gridj ge 0 and gridj lt ny and $
                           vza lt vzalimit,nuse)
nne = n_elements(nearby) 
print,nuse,nne,float(nuse)/float(nne)
;stop
                if (nuse gt 100) then begin
                    numsat=numsat+1

                    gvarflag = b1u.img(13)
                    chans = b1u.sat(2:2+nch-1)
                    
                    ;IRWIN - set and calibrate the IR Window channel
                    ich = where(strtrim(b1u.sat,2) eq 'IRWIN')-2
                    ich = ich(0)
                    irwin=-1
                    if (ich ge 0 and n_elements(btemp) gt 100 ) then begin

                        irwinpresent=1
                        irwin=ich

                        tempir = irtemp * 0.

;                       old method prior to parallax corection
;                        tempir(nearby(use)) = btemp(ich+gridi(use)*0,gridi(use),gridj(use))

;                        will keep this original definition since
;                        parallax correction should have no impact the
;                        satid parameter at this point
                        satid = replicate(0b,ngx,ngy)
                        satid(nearby(use)) = igeo

                        ;get the calibration correction
                        gf = strsplit(gfile,'.',/extr)
                        if (gf(3) eq 'GOES9') then gf(3) = 'GOE-9'
                        satmon=strjoin(gf([3,4,5]),'.')
                        cal_index = where(satmon eq ircname,id)
                        cal_slope = 1.
                        cal_offset = 0
                        if (id ne 1) then begin
                            print,'IRWIN Error: Can not find correct calibration ID'
                            print,id
                            print,satmon
;                            stop
; rather than stopping, values should be ignored and continue
; processing ... this sets the values to zero which means the data
; won't overwrite data from other files.
                            cal_slope = 0.
                            cal_offset= 0.
                        endif else begin
                            final_cal_flag=final_cal_flag+1
                            cal_slope = ircslope(cal_index(0))
                            cal_offset= ircoffset(cal_index(0))
                        endelse
                        irsatslp(igeo) = cal_slope
                        irsatoff(igeo) = cal_offset
                        ;apply the calibration correction
                        ;tempir = tempir * cal_slope + cal_offset

                        ;perform VZA IR correction
                        ;if keyword_set(VZAcorr) then begin
                        doy = b1u.img(0) mod 1000

;                       if keyword_set(debug) then begin

;                       attempt correction prior to remapping
;                       likely the better option
                        sattemp = reform(btemp(ich,*,*))
                        sattemp = sattemp * cal_slope + cal_offset
;                       navigate
                        imgi = lindgen(nx,ny) mod nx
                        imgj = lindgen(nx,ny)  /  nx
;timer,/stop,/print
;print,memory()/1024/1024
                        
                        if keyword_set(debug) then print,'Navigating satellite points'
if keyword_set(timer) then timer,/start,number=3
                        b1u_ij2ll,imgi,imgj,dimglat,dimglon,nav=b1u.nav,glon=glon,chunksize=1000000
imglat = float(dimglat)
imglon = float(dimglon)
dimglat=0
dimglon=0
if keyword_set(timer) then timer,/stop,number=3

                        ;only work with point that are ;visible: |lat| < 88,
                        valid = where(abs(imglat) lt 88. and $
                                      sattemp gt 100 and $
                                      sattemp lt 400,nvalid)
                        if (nvalid eq 0) then goto,nextgeo

                        if keyword_set(debug) then print,'Calculating angles'

;no use timing call to angles because it is very fast
;if keyword_set(timer) then timer,/start,number=4
                        angles,imglat(valid),imglon(valid),0.,0.,temp1,temp2,imgvza,imgazi, $
                               /satonly,inglon = glon(0) + 1.e-4
;timer,/stop,number=4

;                       no use timing call to szacorr because it is very fast
                        irtempc = szacorr(sattemp(valid),imglat(valid),imgvza,doy)
                        sattemp(valid) = irtempc


                        xy = i2xy(nearby(use),size(tempir))

                        sxy=size(xy)
                        if (n_elements(use) ne sxy(2)) then stop

                        ;is szacorr corrup[ting IR temps?
;                        ;values are valid but when we get here, there
;                        are still come invalid values??

                       ;check output and skip GEO if not
;                       enough valid points remain.
                        check_irtempc = where(irtempc gt 100 and $
                                              irtempc lt 400,ngoodirtempc)
                        if (ngoodirtempc lt 10) then goto,nextgeo

                        if (keyword_set(noparallax)) then begin

                            ;no parallax correction
                            tempir(nearby(use)) = sattemp(gridi(use),gridj(use))

                        endif else begin
                            ;parallax correction
                            if keyword_set(debug) then print,'Parallax correction'
                            if keyword_set(timer) then timer,/start,number=6
                            parallax_correct_swath,irtempc,imgvza,imgazi,imglat(valid),irtempcp,remapp, $
                              dx=dx,dy=dy,/donly,height=height
                            if keyword_set(timer) then timer,/stop,number=6
                            

;                       reproject dx/dy
                            if keyword_set(debug) then print,'Remapping with parallax'
                            if keyword_set(timer) then timer,/start,number=7
                            dxmap = reform(btemp(0,*,*))*0.
                            dymap = dxmap
                            
                            dxmap(valid) = dx
                            dymap(valid) = dy
                            
                            
                            tempirc = tempir*0. + 400.
                            
                            tempirc(xy(0,*)+dxmap(gridi(use),gridj(use)), $
                                    xy(1,*)+dymap(gridi(use),gridj(use))) = $
                              sattemp(gridi(use),gridj(use))

;no remapping
;window,1,xs=1200,ys=1200
;tvscl,-(tempir>200<300),-42,-400
;remapped
;window,0,xs=1200,ys=1200
;tvscl,-(tempirc>200<300),-42,-400


;                       filling data
                            low = where(azi lt 0.,nlow)
                            if (nlow gt 0) then azi(low) = azi(low) + 360.

                            tempircr = tempirc

;nexti/j are the nearby grid points based on azimuth angle
;these are used to fill missing points
;since they use azi, they are already grid(nearby) ... still need the
;(use) subset of nearby
                            nextj = fix((azi(use) gt 112.5 and azi(use) lt 247.5) - $
                                        (azi(use) gt 292.5 or azi(use) lt 67.5)+1b)-1
                            nexti = fix((azi(use) lt 337.5 and azi(use) gt 202.5) - $
                                        (azi(use) gt 22.5 and azi(use) lt 157.5)+1b)-1
                            if keyword_set(timer) then timer,/stop,number=7
;                       so replace values where current values are missing and nearby
;                       (nexti/j) points are valid ... a complicated where statement might replace
;                        a slow loop
                            if keyword_set(debug) then print,'Filling parallax gaps'
                            if keyword_set(timer) then timer,/start,number=8
                            itry=0
                            fill = where(tempircr(nearby(use)) gt 390.,nfill_new)

                            if (nfill_new gt 0) then begin
                            
                            lefti = replicate(-1l,nfill_new)
                            leftj = replicate(-1l,nfill_new)
                            righti= replicate(-1l,nfill_new)
                            rightj= replicate(-1l,nfill_new)
                            nfilled = 0l
                            orig_fill = fill
again:
                            nfill_orig=nfill_new
;slow loop
                            for ifill=0l,nfill_orig-1 do begin
                                gi = xy(0,fill(ifill))
                                gj = xy(1,fill(ifill))
                                checki = gi + nexti(fill(ifill))
                                checkj = (gj + nextj(fill(ifill)))>0<(ngy-1)
                                checki = (checki + ngx * (checki lt 0)) mod ngx 
                                if (tempirc(checki,checkj) gt 100 and $
                                    tempirc(checki,checkj) lt 380) then begin
                                    tempircr(gi,gj) = tempirc(checki,checkj)
                                    lefti(nfilled)  = gi
                                    leftj(nfilled)  = gj
                                    righti(nfilled) = checki
                                    rightj(nfilled) = checkj
                                    nfilled++
                                endif
                            endfor
                            itry++
                            fill = where(tempircr(nearby(use)) gt 390.,nfill_new)
                            tempirc = tempircr
                            if (itry lt 60 and (nfill_orig-nfill_new) gt 10) then goto,again
                            if keyword_set(timer) then timer,/stop,number=8
;window,2,xs=1200,ys=1200
;tvscl,-(tempircr>200<300),-42,-400
                            endif
                            if keyword_set(timer) then timer,/start,number=1
                            
                            tempir = tempircr
                            
                        endelse
                    endif 


                    ;IRWVP - set and calibrate the IR Water Vapor Channel
                    ich = where(strtrim(b1u.sat,2) eq 'IRWVP')-2
                    irwvp = -1
                    ich = ich(0)
                    if (ich ge 0 and n_elements(btemp) gt 100) then begin
                        irwvppresent = 1

                        irwvp = ich
                        tempwv = irtemp * 0.
;                        original method
;                        tempwv(nearby(use)) = btemp(ich+gridi(use)*0,gridi(use),gridj(use))
;                        new method using parallax remapping
                        if (keyword_set(noparallax) or irwin lt 0) then begin
                            tempwv(xy(0,*), $
                                   xy(1,*)) = $
                              btemp(ich+gridi(use)*0,gridi(use),gridj(use))
                        endif else begin
                            tempwv(xy(0,*)+dxmap(gridi(use),gridj(use)), $
                                   xy(1,*)+dymap(gridi(use),gridj(use))) = $
                              btemp(ich+gridi(use)*0,gridi(use),gridj(use))

                            if (nfilled gt 0) then begin
                                tempwvf = tempwv
                                for i=0l,nfilled-1 do begin
                                    tempwvf(lefti(i),leftj(i)) = $
                                      tempwvf(righti(i),rightj(i))
                                endfor
                                tempwv = tempwvf
                                tempwvf=0.
                            endif
                        endelse

                        ;perform calibration correction
                        gf = strsplit(gfile,'.',/extr)
                        if (gf(3) eq 'GOES9') then gf(3) = 'GOE-9'
                        satmon=strjoin(gf([3,4,5]),'.')
                        cal_index = where(satmon eq wvcname,id)
                        cal_slope = 1.
                        cal_offset = 0
                        if (id ne 1) then begin
                            print,'IRWVP Error: Can not find correct calibration ID'
                            print,id
                            print,satmon

;                            stop
; rather than stopping, values should be ignored and continue
; processing ... this sets the values to zero which means the data
; won't overwrite data from other files.

                            cal_slope = 0.
                            cal_offset= 0.

                        endif else begin
                            cal_slope = wvcslope(cal_index(0))
                            cal_offset= wvcoffset(cal_index(0))
                        endelse
                        wvsatslp(igeo) = cal_slope
                        wvsatoff(igeo) = cal_offset
                        tempwv = tempwv * cal_slope + cal_offset
                            
                            ;apply calibration correction for N17/HIRS
;                        wv_vza_corr = geo_hirs_limb_corr(gf(3),tempwv(nearby(use)),vza(use))
                        
                        apply_vza_corr=0
                        if (apply_vza_corr) then begin
                            wv_vza_corr = geo_hirs_limb_corr('N14',tempwv(nearby(use)),vza(use))
                            tcorr = tempwv*0
                            tcorr(nearby(use)) = wv_vza_corr
                            tempwv = tempwv + tcorr
                            tcorr=0 & wv_vza_corr=0
                        endif
                    endif
                    ;VSCHN - set and calibrate the Visible Channel
                    ich = where(strtrim(b1u.sat,2) eq 'VSCHN')-2
                    ich = ich(0)
                    ivschn = ich
                    if (ich ge 0 and n_elements(btemp) gt 100) then begin
                        vschnpresent=1
                        tempvs = irtemp * 0.
                        if (keyword_set(noparallax) or irwin lt 0) then begin
                            tempvs(nearby(use)) = $
                              btemp(ich+gridi(use)*0,gridi(use),gridj(use))
                        endif else begin
                            ;new method using parallax remapping
                            tempvs(xy(0,*)+dxmap(gridi(use),gridj(use)), $
                                   xy(1,*)+dymap(gridi(use),gridj(use))) = $
                              btemp(ich+gridi(use)*0,gridi(use),gridj(use))
                            if (nfilled gt 0) then begin
                                tempvsf = tempvs
                                for i=0l,nfilled-1 do begin
                                    tempvsf(lefti(i),leftj(i)) = $
                                      tempvsf(righti(i),rightj(i))
                                endfor
                                tempvs = tempvsf
                                tempvsf=0.
                            endif
                        endelse

                        ;perform vis calibration correction
                        
                    endif
                    
                    if keyword_set(timer) then timer,/stop,number=1
                    
                    if keyword_set(timer) then timer,/start,number=2
                    ;replace the final grid with current satellite data

                    vzause = fix(vza(use)*10.)

                    if (irwin ge 0) then begin
                        
                        replace = where(vzause lt gdvzair(nearby(use)) and $         ;VZA is lower
                                        qc(irwin+gridj(use)*0,gridj(use)) eq 0 and $   ;scan line is valid
                                        tempir(nearby(use)) gt 51. and $               ;obs. is valid
                                        tempir(nearby(use)) lt 400.,nrep)              ;obs. is valid

                        back =   where(vzause ge gdvzair(nearby(use)) and $          ;VZA is greater
                                       vzause lt gdvzair2(nearby(use)) and $         ; vza is lower
                                       qc(irwin+gridj(use)*0,gridj(use)) eq 0 and $    ;scan line is valid
                                       tempir(nearby(use)) gt 51. and $                ;obs. is valid
                                       tempir(nearby(use)) lt 400.,nback)

                        tert =   where(vzause ge gdvzair2(nearby(use)) and $          ;VZA is greater
                                       vzause lt gdvzair3(nearby(use)) and $         ; vza is lower
                                       qc(irwin+gridj(use)*0,gridj(use)) eq 0 and $    ;scan line is valid
                                       tempir(nearby(use)) gt 51. and $                ;obs. is valid
                                       tempir(nearby(use)) lt 400.,ntert)

                        if (nrep gt 0) then begin
                                ;data which will be overwritten but should be kept
                            irtemp3( nearby(use(replace))) = irtemp2( nearby(use(replace)))
                            satidi3( nearby(use(replace))) = satidi2( nearby(use(replace)))
                            gdvzair3(nearby(use(replace))) = gdvzair2(nearby(use(replace)))

                            irtemp2( nearby(use(replace))) = irtemp( nearby(use(replace)))
                            satidi2( nearby(use(replace))) = satidi( nearby(use(replace)))
                            gdvzair2(nearby(use(replace))) = gdvzair(nearby(use(replace)))

                            irtemp(nearby(use(replace))) = tempir(nearby(use(replace)))
                            gdvzair(nearby(use(replace)))= vzause(replace)
                            satidi(nearby(use(replace))) = igeo
                        endif 

                        if (nback gt 0) then begin
                            irtemp3(nearby(use(back)))  = irtemp2(nearby(use(back)))
                            satidi3(nearby(use(back)))  = satidi2(nearby(use(back)))
                            gdvzair3(nearby(use(back))) = gdvzair2(nearby(use(back)))

                            irtemp2(nearby(use(back)))  = tempir(nearby(use(back)))
                            satidi2(nearby(use(back)))  = igeo
                            gdvzair2(nearby(use(back))) = vzause(back)
                        endif

                        if (ntert gt 0) then begin
                            irtemp3(nearby(use(tert)))  = tempir(nearby(use(tert)))
                            satidi3(nearby(use(tert)))  = igeo
                            gdvzair3(nearby(use(tert))) = vzause(tert)
                        endif

                    if (irwvp ge 0) then begin
                        replace = where(vzause lt gdvzawv(nearby(use)) and $
                                        qc(irwvp+gridj(use)*0,gridj(use)) eq 0 and $   ;scan line is valid
                                        tempwv(nearby(use)) gt 51. and $                ;obs. is valid
                                        tempwv(nearby(use)) lt 400.,nrep)              ;obs. is valid
                        back =    where(vzause ge gdvzawv(nearby(use)) and $
                                        vzause lt gdvzawv2(nearby(use)) and $         ; vza is lower
                                        qc(irwvp+gridj(use)*0,gridj(use)) eq 0 and $    ;scan line is valid
                                        tempwv(nearby(use)) gt 51. and $                ;obs. is valid
                                        tempwv(nearby(use)) lt 400.,nback)              ;obs. is valid
                        if (nrep gt 0) then begin
                                ;data which will be overwritten but should be kept
                            wvtemp2( nearby(use(replace))) = wvtemp( nearby(use(replace)))
                            satidw2( nearby(use(replace))) = satidw( nearby(use(replace)))
                            gdvzawv2(nearby(use(replace))) = gdvzawv(nearby(use(replace)))

                            wvtemp(nearby(use(replace))) = tempwv(nearby(use(replace)))                           
                            satidw(nearby(use(replace))) = igeo
                            gdvzawv(nearby(use(replace)))= vzause(replace)
                        endif
                        if (nback gt 0) then begin
                            wvtemp2(nearby(use(back))) = tempwv(nearby(use(back)))
                            satidw2(nearby(use(back))) = igeo
                            gdvzawv2(nearby(use(back))) = vzause(back)
                        endif 
                    endif 
                    if (ivschn ge 0) then begin
                        replace = where(vzause lt gdvzavs(nearby(use)) and $
                                        qc(ivschn+gridj(use)*0,gridj(use)) eq 0,nrep)
                        back =    where(vzause ge gdvzavs(nearby(use)) and $
                                        vzause lt gdvzavs2(nearby(use)) and $ ; vza is lower
                                        qc(ivschn+gridj(use)*0,gridj(use)) eq 0,nback)
                        if (nrep gt 0) then begin
                                ;data which will be overwritten but should be kept
                            vsalb2(  nearby(use(replace))) = vsalb( nearby(use(replace)))
                            satidv2( nearby(use(replace))) = satidv( nearby(use(replace)))
                            gdvzavs2(nearby(use(replace))) = gdvzavs(nearby(use(replace)))

                            vsalb(nearby(use(replace))) = tempvs(nearby(use(replace)))
                            satidv(nearby(use(replace))) = igeo
                            gdvzavs(nearby(use(replace))) = vzause(replace)
                        endif
                        if (nback gt 0) then begin
                            vsalb2(nearby(use(back))) = tempvs(nearby(use(back)))
                            satidv2(nearby(use(back))) = igeo
                            gdvzavs2(nearby(use(back))) = vzause(back)
                        endif
if keyword_set(timer) then timer,/stop,number=2
                    endif
if keyword_set(debug) and not(keyword_set(noplot)) then begin
    window,0,xs=1800,ys=900
;    loadct,0,/sil
;tvscl,congrid(-(irtemp(800:1700,800:1250)>200<300),1800,900)
;tvscl,congrid(-(irtemp>200<300),1800,900)
;stop

    irimg = satellite_temp2image(congrid(irtemp,900,300),/cira)
    tv,irimg,0,600
    !p.multi=[6,2,3]
    color,'magenta',6
    map_set,/cont,limit=[-70,-180,70,180],/nobord,xmarg=[0,0],ymarg=[0,0],col=6,/noerase    

    irimg = satellite_temp2image(congrid(irtemp2,900,300),/cira)
    tv,irimg,0,300
    !p.multi=[4,2,3]
    color,'magenta',6
    map_set,/cont,limit=[-70,-180,70,180],/nobord,xmarg=[0,0],ymarg=[0,0],col=6,/noerase    

    irimg = satellite_temp2image(congrid(irtemp3,900,300),/cira)
    tv,irimg,0,0
    !p.multi=[2,2,3]
    color,'magenta',6
    map_set,/cont,limit=[-70,-180,70,180],/nobord,xmarg=[0,0],ymarg=[0,0],col=6,/noerase    


    loadct,39,/sil
    color,'red',0
    color,'orange',1
    color,'yellow',2
    color,'green',3
    color,'blue',4
    color,'violet',5
    color,'white',6
    color,'white',255b


    tv,  congrid(satidi,900,300)<8,900,600
    !p.multi=[5,2,3]
    map_set,/cont,limit=[-70,-180,70,180],/nobord,xmarg=[0,0],ymarg=[0,0],col=6,/noerase    

    tv,  congrid(satidi2,900,300)<8,900,300
    !p.multi=[3,2,3]
    map_set,/cont,limit=[-70,-180,70,180],/nobord,xmarg=[0,0],ymarg=[0,0],col=6,/noerase    

    tv,  congrid(satidi3,900,300)<8,900,00
    !p.multi=[1,2,3]
    map_set,/cont,limit=[-70,-180,70,180],/nobord,xmarg=[0,0],ymarg=[0,0],col=6,/noerase    


    png,'GriSat-Construction-'+strtrim(counter,2)
    counter++

;    tvscl,-(congrid(wvtemp, 900,300)>200<300),0,600
;    tvscl,-(congrid(wvtemp2,900,300)>200<300),0,300
;    loadct,39,/sil
;    tvscl,  congrid(satidi2,900,300)<8
;    tvscl,-(congrid(wvtemp2-wvtemp,900,300)>(-20)<20),0,0
;    tvscl,(congrid(vsalb, 900,300)>0<.6),0,600
;    tvscl,(congrid(vsalb2,900,300)>0<.6),0,300
stop
;    tvscl,-(congrid(wvtemp2-wvtemp,900,300)>(-20)<20),0,0
endif
endif
;if keyword_set(debug) then stop
                endif ; nuse
            endif               ;btemp
        endif                   ;nok
nextgeo:
    endfor                      ;igeo

    if keyword_set(timer) then begin
        timer,/print,number=0,comment='Navigating grid points'
        timer,/print,number=3,comment='Navigating satellite points'
;        timer,/print,number=4
;        timer,/print,number=5,comment='VZA Correction'
        timer,/print,number=6,comment='Parallax correction'
        timer,/print,number=7,comment='Remapping with parallax'
        timer,/print,number=8,comment='Filling parallax gaps'
        timer,/print,number=1,comment='Remap other channels'
        timer,/print,number=2,comment='Sort channels by VZA'
    endif

        ;output to netcdf file

;if keyword_set(ay) then begin
;    for i=0,numgeo-1 do begin
;        w=where(satid eq i,n)
;        if (n gt 0) then begin
;            print,i,satname(i),mean(wvtempo(w)),mean(wvtempl(w)),mean(wvtempc(w)), $
;                  format='(i,a15,3f10.3)'
;        endif
;    endfor
;endif

if keyword_set(timer) then timer,/start,num=0
        for ibasin=0,runbasins-1 do begin
            basin=basins(ibasin)

;basin='SP'

            blatmin = -70
            blatmax = 70.
            blonmin = -180.
            blonmax =  180.

            ncdir1=ncdir+yrdir
            ncdir1=ncdir1

            ;Set up the initial grid

            ;assign values from huge grid to local grid
            i1 = round((blonmin-lonmin)/ang)
            i2 = round((blonmax-lonmin)/ang) mod (ngx)
            j1 = round((blatmin-latmin)/ang)
            j2 = round((blatmax-latmin)/ang)
            ;ngxg = ceil((blonmax-blonmin)/ang)+1
            ;ngyg = ceil((blatmax-blatmin)/ang)+1
            ngyg = (j2-j1)+1
            if (i2 gt i1) then begin
                ngxg = (i2-i1)+1
            endif else begin
                ngxg = ngx - (i1-i2) + 1
            endelse
;                ngxg = ngx
;                ngyg = ngy
;                irtemp1 = irtemp
;                wvtemp1 = wvtemp
;                vsalb1 = vsalb
;                satid1 = satid
;                lat1g = lat1
;                lon1g = lon1
            print,'GRID SIZE=',ngxg,ngyg

                    ;isolate only the current storms.

            ngoodtemp = total(irtemp gt 200 and irtemp lt 320)

            if (ngoodtemp gt 1e2 or keyword_set(debug) ) then begin
;            save,/all
;            stop
;writenetcdf:
;            restore
            ;define NetCDF filename
                t=strsplit(stripname(mfile),'.',/extr)
                f=t

                ncfile = ncdir1 + '/'+strjoin([strupcase(hursatname),f(1:4),hursatver,'nc'],'.')

;               determine whether irtemp2/3 will be gridded or compressed
                ;where ngx*ngy*2 = Npoints * 6
                ; LHS: 2 required to store a 2 byte int
                ; RHS: 6=4+2: 4=store indices of values
                ;             2=store 2 byte values

                lowertempflag=100.
                uppertempflag=390.

                if (gridonly eq 1) then begin
                    isparse2flag=0
                    isparse2flagwv=0
                    isparse2flagvs=0
                    isparse3flag=0
                endif else begin

                if ((total(irtemp2 gt lowertempflag and $
                           irtemp2 lt uppertempflag)/(ngx*ngy)) gt (2./6.)) then $
                  isparse2flag=0 else $
                  isparse2flag=1

                if ((total(wvtemp2 gt lowertempflag and $
                           wvtemp2 lt uppertempflag)/(ngx*ngy)) gt (2./6.)) then $
                  isparse2flagwv=0 else $
                  isparse2flagwv=1

                if ((total(vsalb2 gt (-0.05) and $
                           vsalb2 lt (20))/(ngx*ngy)) gt (2./6.)) then $
                  isparse2flagvs=0 else $
                  isparse2flagvs=1

                ;where ngx*ngy*3 = Npoints * 6
                ; LHS: 3=2+1
                ;        1=another satid parameter is required
                ;        2=required to store a 2 byte int
                ; RHS: 7=4+2+1: 4=store indices of values
                ;               2=store 2 byte values
                ;               1=store another satid value
                if ((total(irtemp3 gt lowertempflag and $
                           irtemp3 lt uppertempflag)/(ngx*ngy)) gt (3./7.)) then $
                  isparse3flag=0 else $
                  isparse3flag=1
                endelse

                ;set up parameters if using sparse array

                ;nsparseX = 1 <- data not written
                ;nsparseX = 2 <- 2-D matrix used to store valeus
                ;nsparseX > 2 <- 1-D sparse array used to store values
                nsparse2=2
                irwin2sparse = [-1.,-1.]
                sparse2 = [-1,-1]
                if (isparse2flag) then begin
                    isparse3flag=1
                    sparse2 = where(irtemp2 gt lowertempflag and $
                                    irtemp2 lt uppertempflag, nsparse2)
                    if (nsparse2 lt 10) then begin
                        nsparse2=1
                        irwin2sparse = [-1.]
                    endif else begin
                        irwin2sparse = irtemp2(sparse2)
                    endelse
                endif

                nsparse2wv=2
                wvtemp2sparse = [-1.,-1.]
                sparse2wv = [-1,-1]
                if (isparse2flagwv) then begin
                    sparse2wv = where(wvtemp2 gt lowertempflag and $
                                      wvtemp2 lt uppertempflag, nsparse2wv)
                    if (nsparse2wv lt 10) then begin
                        nsparse2wv=1
                        wvtemp2sparse = [-1.]
                    endif else begin
                        wvtemp2sparse = wvtemp2(sparse2wv)
                    endelse
                endif

                nsparse2vs=2
                vsalb2sparse = [-1.,-1.]
                sparse2vs = [-1,-1]
                if (isparse2flagvs) then begin
                    sparse2vs = where(vsalb2 gt (-0.05) and $
                                      vsalb2 lt 20,nsparse2vs)
                    if (nsparse2vs lt 10) then begin
                        nsparse2vs=1
                        vsalb2sparse = [-1.]
                    endif else begin
                        vsalb2sparse = vsalb2(sparse2vs)
                    endelse
                endif


                nsparse3=2
                irwin3sparse = [-1.,-1.]
                satidi3sparse = [-1,-1]
                sparse3=[-1,-1]
                if (isparse3flag) then begin
                    sparse3 = where(irtemp3 gt lowertempflag,nsparse3)
                    if (nsparse3 lt 10) then begin
                        nsparse3=1
                        irwin3sparse = [-1.]
                        satidi3sparse = [-1]
                    endif else begin
                        irwin3sparse = irtemp3(sparse3)
                        satidi3sparse = satidi3(sparse3)
                    endelse
                endif


            ;define Global Attributes
                cyr = f(1)
                cmn = f(2)
                cdy = f(3)

            date_made = systime()
            q=strsplit(date_made,' ',/extr)
            newdate = q(3)
            mm=date_mmm_2_mm(q(1))
            newdate = q(4)+'-'+mm+'-'+q(2)+' '+newdate
            now = systime_iso()
            tstart = date_jul2iso(julday(f(2),f(3),f(1),f(4),0,0))
            tend   = date_jul2iso(julday(f(2),f(3),f(1),f(4),30,0))

            cur_time = round((julday(f(2),f(3),f(1),f(4),0,0) - julday(1,1,1970,0,0,0))*24.)
            
            ;open NetCDF file
            id = ncdf_create(ncfile,/clobber)
            ;_____________________________________________________________________________
            ;Global Attributes
            ; Data Set info
            ncdf_attput,id,'title', $
                        'GriSat-B1 - Gridded Satellite (GriSat) ISCCP B1 Observations',/char,/global
            ncdf_attput,id,'summary', $
                        'Geostationary data remapped to equal angle to facilitate processing.',/char,/global
            ncdf_attput,id,'Conventions','CF-1.4',/char,/global
            ncdf_attput,id,'Metadata_Conventions', $
                        'Unidata Dataset Discovery v1.0',/char,/global
            ncdf_attput,id,'keywords', $
                        'EARTH SCIENCE > Atmosphere > Clouds > Cloud Top Temperature '    +string(10b)+ $
                        'EARTH SCIENCE > Atmosphere > Clouds > Cloud Height  '    +string(10b)+ $
                        'EARTH SCIENCE > Atmosphere > Atmospheric Radiation > Reflectance '    +string(10b)+ $
                        'EARTH SCIENCE > Atmosphere > Atmospheric Radiation > Longwave Radiation ',/char,/global
            ncdf_attput,id,'keywords_vocabulary', $
                        'GCMD Science Keywords, '+string(10b)+ $
                        'http://gcmd.gsfc.nasa.gov/Resources/valids/gcmd_parameters.html',/char,/global

            ; version info
            ncdf_attput,id,'cvs_info_b1u2grisat',cvsver,/char,/global
            ncdf_attput,id,'version_info',hursatver,/char,/global

            ncdf_attput,id,'date_created',newdate,/global,/char
            ncdf_attput,id,'date_created_format2',    systime(/UTC)+' UTC',/char,/global
            ncdf_attput,id,'date_created_format1',strtrim(long(systime(/sec)),2)+ $
                        ' seconds since 1 Jan 1970 00:00 UTC',/char,/global
            ncdf_attput,id,'creator_name','Ken Knapp, NOAA/NESDIS/NCDC/RSAD',/global,/char
            ncdf_attput,id,'creator_url', $
                        'http://www.ncdc.noaa.gov/ '+string(10b)+ $
                        'http://www.ncdc.noaa.gov/oa/rsad/isccpb1/',/global,/char
            ncdf_attput,id,'creator_email', $
                        'NCDC.Satorder@noaa.gov '+string(10b)+ $
                        'Ken.Knapp@noaa.gov',/char,/global
            ncdf_attput,id,'institution', $
                        'NOAA/NESDIS/National Climatic Data Center',/char,/global
            ncdf_attput,id,'date_modified', $
                        now,/char,/global
            ncdf_attput,id,'date_issued', $
                        now,/char,/global

            ;GeoSpatial info
            ncdf_attput,id,'Projection',         'Mercator',  /char, /global
            ncdf_attput,id,'cdm_data_type',      'Grid',      /char, /global
            ncdf_attput,id,'geospatial_lat_min',lat1(0),     /float,/global
            ncdf_attput,id,'geospatial_lat_max',lat1(ngy-1),/float,/global
            ncdf_attput,id,'geospatial_lon_min',lon1(0),     /float,/global
            ncdf_attput,id,'geospatial_lon_max',lon1(ngx-1),/float,/global
            ncdf_attput,id,'geospatial_lat_units', $
                        'degrees_north',/char,/global
            ncdf_attput,id,'geospatial_lat_resolution', $
                        ang,/float,/global
            ncdf_attput,id,'geospatial_lon_units', $
                        'degrees_east',/char,/global
            ncdf_attput,id,'geospatial_lon_resolution', $
                        ang,/float,/global

            ;Temporal info
            ncdf_attput,id,'base_date',  [float(cyr),float(cmn),float(cdy)],/float,/global
            ncdf_attput,id,'time_coverage_start',tstart,      /char, /global
            ncdf_attput,id,'time_coverage_end',  tend,        /char, /global

            ;Satellite info
            ncdf_attput,id,'_satid','_satid attributes are comma delimited arrays with'+string(10b)+ $
                        'satellite information that corresponds to the satid variable (-1 is missing)',/char,/global
            ncdf_attput,id,'Satellite_Name_satid',     strjoin(satname(0:ngeo-1),','),/char,/global
            ncdf_attput,id,'B1U_File_satid',strjoin(geolist,string([44b,10b])),/char,/global
            ncdf_attput,id,'IRWIN_calibration_file', ircfile,           /char,/global
            ncdf_attput,id,'IRWVP_calibration_file', wvcfile,           /char,/global
            ncdf_attput,id,'Corrections','IRWIN has VZA correction,'+string(10b)+ $
                        'All channels have parallax correction,'+string(10b)+ $
                        'All channels have parallax gaps filled.',           /char,/global


            ;Define Dimensions
            nlon =   ncdf_dimdef(id,'lon',ngx)
            nlat =   ncdf_dimdef(id,'lat',ngy)
            num  =   ncdf_dimdef(id,'Ngeo',ngeo)
            strln =  ncdf_dimdef(id,'StrLen',50)
            ntime=   ncdf_dimdef(id,'time',/unlimited) ;record variable

;           always write out nspars information
            nspar2 = ncdf_dimdef(id,'sparse2ir',nsparse2)
            nspar3 = ncdf_dimdef(id,'sparse3ir',nsparse3)
            nspar2wv = ncdf_dimdef(id,'sparse2wv',nsparse2wv)
            nspar2vs = ncdf_dimdef(id,'sparse2vs',nsparse2vs)

            ;Define Variables
            vid_lat = ncdf_vardef(id,'lat',[nlat],/float)
            ncdf_attput,id,vid_lat,'units','degrees_north'
            ncdf_attput,id,vid_lat,'long_name','Latitude'
            ncdf_attput,id,vid_lat,'actual_range',[blatmin, blatmax],/float
            
            vid_lon = ncdf_vardef(id,'lon',[nlon],/float)
            ncdf_attput,id,vid_lon,'units','degrees_east'
            ncdf_attput,id,vid_lon,'long_name','Longitude'
            ncdf_attput,id,vid_lon,'actual_range',[blonmin,blonmax],/float

            vid_time = ncdf_vardef(id,'time',[ntime],/long)
            ncdf_attput,id,vid_time,'units','hours since 1970-01-01 00:00:00'
            ncdf_attput,id,vid_time,'long_name','time'

            vid_slat = ncdf_vardef(id,'satlat',[num,ntime],/float)
            ncdf_attput,id,vid_slat,'long_name','Sub-Satellite Latitude'
            ncdf_attput,id,vid_slat,'units','degrees_north'
            ncdf_attput,id,vid_slat,'valid_range',[-20,20],/float

            vid_slon = ncdf_vardef(id,'satlon',[num,ntime],/float)
            ncdf_attput,id,vid_slon,'long_name','Sub-Satellite Longitude'
            ncdf_attput,id,vid_slon,'units','degrees_east'
            ncdf_attput,id,vid_slon,'valid_range',[-180,360],/float

            vid_srad = ncdf_vardef(id,'satrad',[num,ntime],/float)
            ncdf_attput,id,vid_srad,'long_name','Sub-Satellite Orbital Radius'
            ncdf_attput,id,vid_srad,'units','km'
            ncdf_attput,id,vid_srad,'valid_range',[38000.,44000.],/float

            vid_snam = ncdf_vardef(id,'satname',[strln,num,ntime],/char)
            ncdf_attput,id,vid_snam,'long_name','Satellite Name'

            vid_b1u = ncdf_vardef(id,'b1ufile',[strln,num,ntime],/char)
            ncdf_attput,id,vid_b1u,'long_name','B1U File Name'

            vid_b1 = ncdf_vardef(id,'b1file',[strln,num,ntime],/char)
            ncdf_attput,id,vid_b1,'long_name','B1 Filename'
            ncdf_attput,id,vid_b1,'Note','In the event of an nfs error, B1UFile will not match B1File'

            vid_cals = ncdf_vardef(id,'calslp_irwin',[num,ntime],/float)
            ncdf_attput,id,vid_cals,'long_name','Slope used in IRWIN calibration correction'
            ncdf_attput,id,vid_cals,'equation','IRWIN = VZA_correction(IRWIN*CALSLP_IRWIN + CALOFF_IRWIN)'
            ncdf_attput,id,vid_cals,'Note','Usage of correction provided by satid_ir'

            vid_calo = ncdf_vardef(id,'caloff_irwin',[num,ntime],/float)
            ncdf_attput,id,vid_calo,'long_name','Offset used in IRWIN calibration correction'

            vid_calsw = ncdf_vardef(id,'calslp_irwvp',[num,ntime],/float)
            ncdf_attput,id,vid_calsw,'long_name','Slope used in IRWVP calibration correction'
            ncdf_attput,id,vid_calsw,'equation','IRWIN = VZA_correction(IRWVP*CALSLP_IRWVP + CALOFF_IRWVP)'
            ncdf_attput,id,vid_calsw,'Note','Usage of correction provided by satid_wv'

            vid_calow = ncdf_vardef(id,'caloff_irwvp',[num,ntime],/float)
            ncdf_attput,id,vid_calow,'long_name','Offset used in IRWIN calibration correction'
            ncdf_attput,id,vid_calow,'Note','Usage of correction provided by satid_wv'

            vid_vrs = ncdf_vardef(id,'vis_rad_slope',[num],/float)
            ncdf_attput,id,vid_vrs,'long_name','Conversion factor to calculate radiance from reflectance'
            ncdf_attput,id,vid_vrs,'units','watts m-2 sr-1'
            ncdf_attput,id,vid_vrs,'Radiance_note','Radiance is calculated via: '+string(10b)+ $
                            'Rad = VSCHN * vis_rad_slope(satid)'

            vid_vds = ncdf_vardef(id,'vis_dc_slope',[num],/float)
            ncdf_attput,id,vid_vds,'long_name','Conversion factor to calculate original digital count from reflectance'
            ncdf_attput,id,vid_vds,'units',' '
            ncdf_attput,id,vid_vds,'DigitalCount_note','Original digital count is calculated via: '+string(10b)+ $
                            'DC = VSCHN * vis_dc_slope(satid) + vis_dc_offset(satid)'

            vid_vdo = ncdf_vardef(id,'vis_dc_offset',[num],/float)
            ncdf_attput,id,vid_vdo,'long_name','Conversion offset to calculate original digital count from reflectance'
            ncdf_attput,id,vid_vdo,'units',' '

;           originally, I didn't output variables when a channel was
;           missing, because this messes up the NCO procedure. It is
;           better to leave those channels present (so NCO finds
;           them), but also leave them blank.
;           So I'm commenting out the following if statements that
;           make the variable definitions conditional

;            if (irwinpresent) then begin
            vid_ir = ncdf_vardef(id,'irwin',[nlon,nlat,ntime],/short)
            ncdf_attput,id,vid_ir,'long_name','Brightness Temperature (~11um)'
            ncdf_attput,id,vid_ir,'units','Kelvin'
            ncdf_attput,id,vid_ir,'Note','Nadir-most observations'
            ncdf_attput,id,vid_ir,'scale_factor',0.01,/float
            ncdf_attput,id,vid_ir,'add_offset',  200.,/float
            ncdf_attput,id,vid_ir,'valid_range',[100.,375.],/float
            ncdf_attput,id,vid_ir,'missing_value',MISSING,/short
            ncdf_attput,id,vid_ir,'_FillValue',MISSING,/short
            if (final_cal_flag gt 0) then begin
                ncdf_attput,id,vid_ir,'Note_A','Calibration correction of Knapp (2007) applied before VZA correction.'
            endif else begin
                ncdf_attput,id,vid_ir,'Note_A','Calibration correction of Knapp (2007) not applied'
            endelse
            ncdf_attput,id,vid_ir,'Note_B','The IRWIN data has been View Zenith corrected '+string(10b)+ $
                        'using the Joyce (NCEP) algorithm'

            if (isparse2flag) then begin
                vid_ir2 = ncdf_vardef(id,'irwin_2',[nspar2,ntime],/short)
            endif else begin
                vid_ir2 = ncdf_vardef(id,'irwin_2',[nlon,nlat,ntime],/short)
            endelse
            ncdf_attput,id,vid_ir2,'long_name','Brightness Temperature (~11um)'
            ncdf_attput,id,vid_ir2,'units','Kelvin'
            ncdf_attput,id,vid_ir2,'Note','Non nadir-most observations'
            ncdf_attput,id,vid_ir2,'scale_factor',0.01,/float
            ncdf_attput,id,vid_ir2,'add_offset',  200.,/float
            ncdf_attput,id,vid_ir2,'valid_range',[100.,375.],/float
            ncdf_attput,id,vid_ir2,'missing_value',MISSING,/short
            ncdf_attput,id,vid_ir2,'coordinates','lon lat'
            ncdf_attput,id,vid_ir2,'_FillValue',MISSING,/short
            if (final_cal_flag gt 0) then begin
                ncdf_attput,id,vid_ir2,'Note_A','Calibration correction of Knapp (2007) applied before VZA correction.'
            endif else begin
                ncdf_attput,id,vid_ir2,'Note_A','Calibration correction of Knapp (2007) not applied'
            endelse
            ncdf_attput,id,vid_ir2,'Note_B','The IRWIN data has been View Zenith corrected '+string(10b)+ $
                        'using the Joyce (NCEP) algorithm'


            if (isparse3flag) then begin
                vid_ir3 = ncdf_vardef(id,'irwin_3',[nspar3,ntime],/short)
            endif else begin
                vid_ir3 = ncdf_vardef(id,'irwin_3',[nlon,nlat,ntime],/short)
            endelse
            ncdf_attput,id,vid_ir3,'long_name','Brightness Temperature (~11um)'
            ncdf_attput,id,vid_ir3,'units','Kelvin'
            ncdf_attput,id,vid_ir3,'Note','Non nadir-most observations'
            ncdf_attput,id,vid_ir3,'scale_factor',0.01,/float
            ncdf_attput,id,vid_ir3,'add_offset',  200.,/float
            ncdf_attput,id,vid_ir3,'valid_range',[100.,375.],/float
            ncdf_attput,id,vid_ir3,'missing_value',MISSING,/short
            ncdf_attput,id,vid_ir3,'coordinates','lon lat'
            ncdf_attput,id,vid_ir3,'_FillValue',MISSING,/short
            if (final_cal_flag gt 0) then begin
                ncdf_attput,id,vid_ir3,'Note_A','Calibration correction of Knapp (2007) applied before VZA correction.'
            endif else begin
                ncdf_attput,id,vid_ir3,'Note_A','Calibration correction of Knapp (2007) not applied'
            endelse
            ncdf_attput,id,vid_ir3,'Note_B','The IRWIN data has been View Zenith corrected '+string(10b)+ $
                        'using the Joyce (NCEP) algorithm'


;            endif
;            if (irwvppresent) then begin
                vid_wv = ncdf_vardef(id,'irwvp',[nlon,nlat,ntime],/short)
                ncdf_attput,id,vid_wv,'long_name','Brightness Temperature (~6.7um)'
                ncdf_attput,id,vid_wv,'units','Kelvin'
                ncdf_attput,id,vid_wv,'Note','Nadir-most observations'
                ncdf_attput,id,vid_wv,'scale_factor',0.01,/float
                ncdf_attput,id,vid_wv,'add_offset',  200.,/float
                ncdf_attput,id,vid_wv,'valid_range',[100.,375.],/float
                ncdf_attput,id,vid_wv,'missing_value',MISSING,/short
                ncdf_attput,id,vid_wv,'_FillValue',MISSING,/short
                if keyword_set(ay) then begin
                    ncdf_attput,id,vid_wv,'Note','Limb-corrected and inter-calibrated using HIRS on NOAA-17'
                endif

            if (isparse2flagwv) then begin
                vid_wv2 = ncdf_vardef(id,'irwvp_2',[nspar2wv,ntime],/short)
            endif else begin
                vid_wv2 = ncdf_vardef(id,'irwvp_2',[nlon,nlat,ntime],/short)
            endelse
                ncdf_attput,id,vid_wv2,'long_name','Brightness Temperature (~6.7um)'
                ncdf_attput,id,vid_wv2,'units','Kelvin'
                ncdf_attput,id,vid_wv2,'Note','Non nadir-most observations'
                ncdf_attput,id,vid_wv2,'scale_factor',0.01,/float
                ncdf_attput,id,vid_wv2,'add_offset',  200.,/float
                ncdf_attput,id,vid_wv2,'valid_range',[100.,375.],/float
                ncdf_attput,id,vid_wv2,'missing_value',MISSING,/short
                ncdf_attput,id,vid_wv2,'_FillValue',MISSING,/short
                if keyword_set(ay) then begin
                    ncdf_attput,id,vid_wv2,'Note_A','Limb-corrected and inter-calibrated using HIRS on NOAA-17'
                endif
;            endif
;            if (vschnpresent) then begin
                vid_vs = ncdf_vardef(id,'vschn',[nlon,nlat,ntime],/short)
                ncdf_attput,id,vid_vs,'long_name','Visible reflectance in (~0.6um)'
                ncdf_attput,id,vid_vs,'units','Reflectance (unitless)'
                ncdf_attput,id,vid_vs,'Note','Nadir-most observations'
                ncdf_attput,id,vid_vs,'scale_factor',1./25000.,/float
                ncdf_attput,id,vid_vs,'add_offset',1.,/float
                ncdf_attput,id,vid_vs,'valid_range',[-0.2,5.],/float
                ncdf_attput,id,vid_vs,'missing_value',MISSING,/short
                ncdf_attput,id,vid_vs,'_FillValue',MISSING,/short
                ncdf_attput,id,vid_vs,'Radiance_note','Radiance is calculated via: '+string(10b)+ $
                            'Rad = VSCHN * vis_rad_slope(satid)'
                ncdf_attput,id,vid_vs,'DigitalCount_note','Original digital counts are calculated via: '+string(10b)+ $
                            'DC = VSCHN * vis_dc_slope(satid) + vis_dc_offset(satid)'


            if (isparse2flagvs) then begin
                vid_vs2 = ncdf_vardef(id,'vschn_2',[nspar2vs,ntime],/short)
            endif else begin
                vid_vs2 = ncdf_vardef(id,'vschn_2',[nlon,nlat,ntime],/short)
            endelse
                ncdf_attput,id,vid_vs2,'long_name','Visible reflectance in (~0.6um)'
                ncdf_attput,id,vid_vs2,'units','Reflectance (unitless)'
                ncdf_attput,id,vid_vs2,'Note','Non nadir-most observations'
                ncdf_attput,id,vid_vs2,'scale_factor',1./25000.,/float
                ncdf_attput,id,vid_vs2,'add_offset',1.0,/float
                ncdf_attput,id,vid_vs2,'valid_range',[-0.2,5.],/float
                ncdf_attput,id,vid_vs2,'missing_value',MISSING,/short
                ncdf_attput,id,vid_vs2,'_FillValue',MISSING,/short
;            endif

            vid_sati = ncdf_vardef(id,'satid_ir',[nlon,nlat,ntime],/byte)
            ncdf_attput,id,vid_sati,'long_name','IRWIN Satellite Index (packed)'
            ncdf_attput,id,vid_sati,'valid_range',[0,254],/byte
            ncdf_attput,id,vid_sati,'variable','irwin'
            ncdf_attput,id,vid_sati,'Note', $
                        'The SATID value corresponds to the index for the satellite '+string(10b)+ $
                        'variables (SATLAT, SATLON, SATRAD, SATNAME). It derives from '+string(10b)+ $
                        'the satellites used in the IRWIN composite.',/char
            ncdf_attput,id,vid_sati,'Usage', $
                        'The SATID can be used in conjunction with the sub-satellite '+string(10b)+ $
                        'latitude and longitude to determine satellite view and azimuth angles.',/char
            ncdf_attput,id,vid_sati,'Requirement', $
                        'Convert from signed to unsigned bytes by adding 128b    '+string(10b)+ $
                        'to satid_ir prior to unpacking.',/char
            ncdf_attput,id,vid_sati,'Unpacking', $
                        'SATID  = satid_ir / 15   '+string(10b)+ $
                        'SATID2 = satid_ir % 15 [where % is the modulo function]',/char
            ncdf_attput,id,vid_sati,'Unpacked', $
                        '14 is missing'
            
            ncdf_attput,id,vid_sati,'missing_value',MISSINGb,/short

            vid_satw = ncdf_vardef(id,'satid_wv',[nlon,nlat,ntime],/byte)
            ncdf_attput,id,vid_satw,'long_name','IRWVP Satellite Index (packed)'
            ncdf_attput,id,vid_satw,'valid_range',[0,254],/byte
            ncdf_attput,id,vid_satw,'variable','irwvp'
            ncdf_attput,id,vid_satw,'Note','See notes on packing and unpacking for satid_ir'

            vid_satv = ncdf_vardef(id,'satid_vs',[nlon,nlat,ntime],/byte)
            ncdf_attput,id,vid_satv,'long_name','VSCHN Satellite Index (packed)'
            ncdf_attput,id,vid_satv,'valid_range',[0,254],/byte
            ncdf_attput,id,vid_satv,'variable','vschn'
            ncdf_attput,id,vid_satv,'Note','See notes on packing and unpacking for satid_ir'


;            if (isparse2flag) then begin
            vid_sp2 = ncdf_vardef(id,'sparse2ir',[nspar2],/long)
            ncdf_attput,id,vid_sp2,'channel','irwin'
            ncdf_attput,id,vid_sp2,'compress','lat lon'
            ncdf_attput,id,vid_sp2,'Note','Result of sparse2ir = where(irwin2 gt lowertempflag,nsparse2)'
            ncdf_attput,id,vid_sp2,'Size', $
                        '1=Missing (no obs. this layer)' + string(10b) + $
                        '2=Data layer is not sparse, but is a 2D (lat,lon) matrix' + string(10b) + $ $
                        ' > 2 = Data layer is stored using a sparse array'
;            endif
;            if (isparse2flagwv) then begin
            vid_sp2wv = ncdf_vardef(id,'sparse2wv',[nspar2wv],/long)
            ncdf_attput,id,vid_sp2wv,'channel','irwvp'
            ncdf_attput,id,vid_sp2wv,'compress','lat lon'
            ncdf_attput,id,vid_sp2wv,'Note','Result of sparse2wv = where(wvtemp2 gt lowertempflag,nsparse2wv)'
            ncdf_attput,id,vid_sp2wv,'Size','See sparse2ir'

;            endif
;            if (isparse2flagvs) then begin
            vid_sp2vs = ncdf_vardef(id,'sparse2vs',[nspar2vs],/long)
            ncdf_attput,id,vid_sp2vs,'channel','vsalb'
            ncdf_attput,id,vid_sp2vs,'compress','lat lon'
            ncdf_attput,id,vid_sp2vs,'Note','Result of sparse2wv = where(wvtemp2 gt lowertempflag,nsparse2wv)'
            ncdf_attput,id,vid_sp2vs,'Size','See sparse2ir'
;            endif
            vid_sp3 = ncdf_vardef(id,'sparse3ir',[nspar3],/long)
            ncdf_attput,id,vid_sp3,'channel','irwin'
            ncdf_attput,id,vid_sp3,'compress','lat lon'
            ncdf_attput,id,vid_sp3,'Note','Result of sparse3 = where(irwin3 gt lowertempflag,nsparse3)'
            ncdf_attput,id,vid_sp3,'Size','See sparse2ir'


            if (isparse3flag) then begin
                vid_sat3 = ncdf_vardef(id,'satid_ir3',[nspar3],/byte)
            endif else begin
                vid_sat3 = ncdf_vardef(id,'satid_ir3',[nlon,nlat,ntime],/byte)
            endelse
            ncdf_attput,id,vid_sat3,'long_name','IRWIN Satellite Index (packed)'
            ncdf_attput,id,vid_sat3,'valid_range',[0,254],/byte
            ncdf_attput,id,vid_sat3,'variable','irwin_3'
            ncdf_attput,id,vid_sat3,'Note', $
                        'The SATID_IR3 value corresponds to the index for the satellite '+string(10b)+ $
                        'variables (SATLAT, SATLON, SATRAD, SATNAME). It derives from '+string(10b)+ $
                        'the satellites used in the IRWIN_3 composite.',/char
            ncdf_attput,id,vid_sat3,'Usage', $
                        'The SATID can be used in conjunction with the sub-satellite '+string(10b)+ $
                        'latitude and longitude to determine satellite view and azimuth angles.',/char
            ncdf_attput,id,vid_sat3,'Unpacked', $
                        '14 is missing'


            ncdf_control,id,/endef

            ;Set variables
            ncdf_varput,id,vid_lat,lat1
            ncdf_varput,id,vid_lon,lon1
            ncdf_varput,id,vid_time,cur_time

            mir = where(irtemp lt 0.,nmir)
            mwv = where(wvtemp lt 0.,nmwv)
            mvs = where(vsalb  lt 0.,nmvs)

            outir = fix(round((irtemp-200.)/0.01) >(-30000)<30000)
            outwv = fix(round((wvtemp-200.)/0.01) >(-30000)<30000)
            outvs = fix(round((vsalb-1.   )*25000)>(-30000)<30000)

            if (nmir gt 0) then outir(mir) = MISSING
            if (nmwv gt 0) then outwv(mwv) = MISSING
            if (nmvs gt 0) then outvs(mvs) = MISSING

            if (irwinpresent gt 0) then ncdf_varput,id,vid_ir,outir
            if (irwvppresent gt 0) then ncdf_varput,id,vid_wv,outwv
            if (vschnpresent gt 0) then ncdf_varput,id,vid_vs,outvs

            if (isparse2flag)   then irout = irwin2sparse  else irout = irtemp2
            if (isparse2flagwv) then wvout = wvtemp2sparse else wvout = wvtemp2
            if (isparse2flagvs) then vsout = vsalb2sparse  else vsout = vsalb2

            mir = where(irout lt 0.,nmir)
            mwv = where(wvout lt 0.,nmwv)
            mvs = where(vsout  lt 0.,nmvs)

            outir = fix(round((irout-200.)/0.01) >(-30000)<30000)
            outwv = fix(round((wvout-200.)/0.01) >(-30000)<30000)
            outvs = fix(round((vsout-1.  )*25000)>(-30000)<30000)

            if (nmir gt 0) then outir(mir) = MISSING
            if (nmwv gt 0) then outwv(mwv) = MISSING
            if (nmvs gt 0) then outvs(mvs) = MISSING

            if (irwinpresent gt 0) then ncdf_varput,id,vid_ir2,outir
            if (irwvppresent gt 0) then ncdf_varput,id,vid_wv2,outwv
            if (vschnpresent gt 0) then ncdf_varput,id,vid_vs2,outvs

            if (isparse3flag) then begin
                irout = irwin3sparse 
                satid_ir3 = satidi3sparse
            endif else begin
                irout = irtemp3
                satid_ir3 = satidi3
            endelse

            mir = where(irout lt 0.,nmir)
            outir = fix(round((irout-200.)/0.01) >(-30000)<30000)
            if (nmir gt 0) then outir(mir) = MISSING
            ncdf_varput,id,vid_ir3,outir

            ncdf_varput,id,vid_sat3, satid_ir3


            ncdf_varput,id,vid_sp2,  sparse2
            ncdf_varput,id,vid_sp2wv,sparse2wv
            ncdf_varput,id,vid_sp2vs,sparse2vs
            ncdf_varput,id,vid_sp3,  sparse3

            ;these arrays are always 2-D
            ncdf_varput,id,vid_sati, (satidi<14b)*15b + (satidi2<14b)
            ncdf_varput,id,vid_satv, (satidv<14b)*15b + (satidv2<14b)
            ncdf_varput,id,vid_satw, (satidw<14b)*15b + (satidw2<14b)



            ncdf_varput,id,vid_slat,satlat(0:ngeo-1)
            ncdf_varput,id,vid_slon,satlon(0:ngeo-1)
            ncdf_varput,id,vid_srad,satrad(0:ngeo-1)
            ncdf_varput,id,vid_snam,satname(0:ngeo-1)
            ncdf_varput,id,vid_b1u,b1u_file(0:ngeo-1)
            ncdf_varput,id,vid_b1, b1_file( 0:ngeo-1)

            if (final_cal_flag gt 0) then begin
                ncdf_varput,id,vid_cals,  irsatslp(0:ngeo-1)
                ncdf_varput,id,vid_calo,  irsatoff(0:ngeo-1)

                ncdf_varput,id,vid_calsw,  wvsatslp(0:ngeo-1)
                ncdf_varput,id,vid_calow,  wvsatoff(0:ngeo-1)

                ncdf_varput,id,vid_vrs,  vs_rad_slope(0:ngeo-1)
                ncdf_varput,id,vid_vds,  vs_dc_slope(0:ngeo-1)
                ncdf_varput,id,vid_vdo,  vs_dc_offset(0:ngeo-1)
            endif

            


            ;close netcdf file
            ncdf_close,id
;            spawn,'gzip -f '+ncfile
;            spawn,'ncks -a -4 -L 2 '+ncfile+' -O '+strtrim(ncfile,2)+'4'
;            spawn,'rm '+ncfile

        endif
if keyword_set(timer) then timer,/stop,num=0
;if keyword_set(timer) then timer,/print,num=0,comment='Writing netCDF file'
timer,/stop,/print,num=9
        if keyword_set(debug) then begin
;            closewin          
        endif
    endfor                      ;imfile
endfor 
endfor 

end

;***
