program b1u_HIRS_as_matchup
!****p* /match_hirs_b1u
! NAME
! match_hirs_b1u
! SYNOPSIS
! Performs colocations of HIRS with ISCCP B1U data. Colocations are
! limited in space and time.
!
! INPUTS
! Command line: 
! $ match_hirs_b1u <match_file>
! where:
! - <match_file> is a matchup file containing ISCCP B1U files and
!   corresponding POES files, including: ISCCP B2, HIRS AS, and
!   HIRS L1C data.
!
! OUTPUT
! A binary file is created for each matchup, reporting on the HIRS and B1U
! statistics associated with the matchup.
!
! USES
! Enchilada, Enchilada_Internal
! AUTHOR
! Ken Knapp - NCDC/RSAD - October 2005
!
! NOTES
! * New compilation does not use compile but the kmod Makefile.
! * Compilation: make.match 
!    $ compile -mei -p b1u_HIRS-path-AS_matchup.f90 -o match_hirs_b1u
! * Hardcoded options include:
!   * db1pix - search +/-DB1PIX B1U pixels from HIRS center for colocation
!              with HIRS. This allows a 25x25 box to be used to find B1U pixels
!              which fall within the HIRS footprint
!   * dtimes - difference in time allowed for matchups expressed in days 
!              currently=2/24 (2 hours). While, 2 hours allows a previous
!              orbit, the change in time generally is overcome by errors in 
!              surface heating and cloud motion.
!   * dmulimit-The difference in mu between HIRS and B1U. The limit in VZA varies
!              nonlinearly with VZA, so dMU is better. Sensitivity studies showed
!              dMU~0.1 was reasonable, so here, will only keep obs within 0.2
!   * fov_frac-The B1U stats are collected for 3 levels. Currently, for B1U pixels 
!              falling exactly within the footprint, then expanding the footprint
!              by one half and 1 full B1U pixel ... this allows for B1U pixels whose
!              pixel center is outside the HIRS footprint, but a portion of which
!              may be within the HIRS footprint.
!   * alphag - This represents the scan angle (in radians) of an 8km footprint GEO
!              satellite (e.g., it is twice the GOES-8 IR scan angle).
! 
! TODO
! * GEO-GEO calibrations should be included in the future.
!***
  use kmod_isccp
  use kmod_math
  use kmod_time
  use kmod_geometry
  use service_routines, only:fstat,lstat
  
  !implicit none

  TYPE hirs_pathfinder_as
     sequence
     integer(kind=2) :: num
     real(kind=8) :: time
     real(kind=4) :: lat, lon
     real(kind=4)    :: sza, alt
     integer(kind=1) :: ref
     real(kind=4),dimension(19) :: tb
     character(len=3) :: name
  END TYPE hirs_pathfinder_as

  !Matchup file info
  integer(kind=4), parameter :: ntypemax=5, nsatmax=20
  character(len=120) :: matchup_file
  integer(kind=4), dimension(ntypemax) :: npersat
  character(len=120), dimension(ntypemax,nsatmax) :: satfile
  character(len=120), dimension(:), allocatable :: f, ff
  integer(kind=4) :: nf, mon, dom,year, jday, hh
  real(kind=8) :: curjul, dtimes
  real(kind=4) :: dmulimit
  !HIRS data
  integer(kind=4), dimension(20) :: nrec_f
  integer(kind=4), dimension(20) :: filestat
  integer(kind=1) :: ispot, iref
  integer(kind=2) :: ilon, ilat, iline, iszen, ialt
  real(kind=4) :: xlon, xtime, xlat, xszen, xalt, xrefl, xtbc, xtb(19)
  integer(kind=4) :: lline, lspot
  integer(kind=2), dimension(19) :: itb
  integer(kind=4) :: itime
  real(kind=4)    :: oldtime
  type(hirs_pathfinder_as), allocatable, dimension(:,:) :: h_data 
  type(hirs_pathfinder_as),              dimension(56)  :: fullscan
  integer(kind=4) :: nrec, nlines
  integer(kind=4) :: hnx, hny
  real(kind=8), allocatable, dimension(:) :: poest !Satellite position in Cartesian coord.
  real(kind=4), allocatable, dimension(:) :: poesx, poesy, poesz !Satellite position in Cartesian coord.
  real(kind=4), allocatable, dimension(:,:) :: hx, hy, hz !Satellite position in Cartesian coord.
  real(kind=4), allocatable, dimension(:,:) :: hvza, hazi, hsza, hazn !View angles for each footprint
  real(kind=4), allocatable, dimension(:,:) :: hvx, hvy, hvz !HIRS View vector (Satellite to surface)
  character(len=3), allocatable, dimension(:,:) :: hname
  character(len=3), dimension(nsatmax) :: c_sat
  integer(kind=4) :: nsat, nummat, isat
  integer :: nwi, satnum
  integer(kind=4), allocatable, dimension(:) :: hok !array of indices w/i dtime of GEO
  !GEO data
  real(kind=4), dimension(nsatmax) :: geolon, geolat
  integer(kind=4) :: istat, ich(3), oj
  character(len=5),dimension(3) :: chn = (/ 'IRWIN','IRWVP','IRNIR' /)
  type(b1u_data) :: b1u
  real(kind=4), dimension(3) :: subsat
  real(kind=4), dimension(:,:), allocatable :: hi, hj, inlat, inlon, hb1v, b1lat, b1lon, &
       &                                       b1i, b1j, b1x, b1y, b1z
  integer(kind=4), dimension(:,:), allocatable :: hii, hjj
  real(kind=4), dimension(13) :: center
  real(kind=4), allocatable, dimension(:,:,:)  :: btemp
  integer(kind=4), allocatable, dimension(:,:) :: qc
  !GEO-LEO matchups
  real(kind=4), dimension(:,:), allocatable :: blat, blon, bi, bj, bx, by, bz,bglat, &
       &                                       brad, magbv, bang
  real(kind=4), dimension(3) :: fov_frac, fov_lim, fov_n
  integer(kind=4) :: qi, qj, hi1, hi2, hj1, hj2
  real(kind=4) :: av8, sd8, av12,sd12, av19, sd19, vv, av
  real(kind=8) :: sumw, sumx, sumxx
  real(kind=8), dimension(3,3) :: sw, sx, sxx, dx, dxx
  integer :: lf, iq, ic
  real(kind=4), dimension(2) :: xq
  real, dimension(12) :: prof
  !Calculations
  real (kind=4) :: glat1, glat2, mag, alt, ft, px, py, pz,tod, &
       & radlat, radlat1, radlat2, maghv, alphag, d1, d2,beta, adj,&
       & erad, clat, slat, clon, slon, glat
  character(len=120) :: mdir, mfile

  real(kind=4) :: hmu, bmu, dmu

  real :: test1,test2, test3
  integer :: test_position, ilastlat, ilastlon
  

  !Matchup limits
  integer(kind=4) :: dB1pix
  !MISC Variables
  integer(kind=4) :: i, j, ll, ki, kj, icount, idebug, jch
  character(len=1) :: cdebug

  real(kind=4) :: aout,bout

  !Limits in the matchups
  ! Limit of 0.625 is the HIRS FOV. We expand that by 3 km (which is half the 
  ! B1U FOV (4 km) plus some extra since the B1U FOV is nominally 4 km, but is 
  ! larger away from nadir, so we estimate it to be 3km (6 km FOV)
  db1pix = 12             !search +/-12 B1U pixels from HIRS center for colocation
  dtimes = 2.d0 / 24.d0    !difference in time allowed by matchup (in days)
  fov_frac = (/ 0., 0.5, 1.0 /)
  alphag = 1.118e-4 * 2.  !radians: scan angle of 8km GOES FOV
  dmulimit=0.2

  allocate (bi(db1pix*2+1,db1pix*2+1), bj(db1pix*2+1,db1pix*2+1))
  allocate (bx(db1pix*2+1,db1pix*2+1))
  allocate (by(db1pix*2+1,db1pix*2+1))
  allocate (bz(db1pix*2+1,db1pix*2+1))
  allocate (bglat(db1pix*2+1,db1pix*2+1))
  allocate (brad(db1pix*2+1,db1pix*2+1))
  allocate (magbv(db1pix*2+1,db1pix*2+1))
  allocate (bang(db1pix*2+1,db1pix*2+1))
  allocate (blat(db1pix*2+1,db1pix*2+1), blon(db1pix*2+1,db1pix*2+1))

  !Get matchup file information
  call getarg(1,matchup_file)
  call getarg(2,cdebug)

  idebug = 0
  if (len_trim(cdebug) == 1) idebug = 1

  if (len_trim(matchup_file) < 3) then
     matchup_file = '/raid/kknapp/data/isccp/match-poes/1998/'//&
          &         'B1U-POES.1998.07.01.00.match'
!     matchup_file = '/raid/kknapp/data/isccp/match-poes/2002/'//&
!          &         'B1U-POES.2002.01.01.12.match'
!     matchup_file = '/raid/kknapp/data/isccp/match-poes/2001/'//&
!          &         'B1U-POES.2001.07.01.00.match'
!     matchup_file = '/raid/kknapp/data/isccp/match-poes/1986/'//&
!          &         'B1U-POES.1986.12.25.18.match'
!     matchup_file = '/raid/kknapp/data/isccp/match-poes/1995/'//&
!          &         'B1U-POES.1995.04.29.21.match'
  end if

  !Get day of year
  call strsplit(trim(matchup_file),'.',f,nf)
  read (f(2),'(i4)') year
  read (f(3),'(i2)') mon
  read (f(4),'(i2)') dom
  read (f(5),'(i2)') hh
  print*,'_____________Starting:',trim(matchup_file)
  print*,mon,dom,year
  jday = nint(julday(mon,dom,year)-julday(1,1,year)+1)
  print*,'Jday:',jday
  curjul = julday(mon,dom,year,hh,00,0.0)
  print*,'Julian Date:',curjul
  
  !Read in the matchup file information
  open (unit=1, file=trim(matchup_file))
  do j=1,4
     read (1,*) npersat(j)
     do i=1,npersat(j)
        read (1,'(a)') satfile(j,i)
     end do
  end do
  close (1)
  ! satfile(1,*) = B1U files
  print*,'=== B1U FILES ==='
  print '(1a)',satfile(1,1:npersat(1))
  ! satfile(2,*) = B2  files
  ! satfile(3,*) = L1C files
  ! satfile(4,*) = HIRS Pathfinder All Sky Files

  if (npersat(4) < 1) then
     print*,'No HIRS Pathfinder AS data file present!!'
     print*,trim(matchup_file)
  end if

  !Read in the HIRS Pathfinder All Sky data
  hnx = 56
  print*,'Number of files:',npersat
  print*,'Reading HIRS data'
  nrec = 0
  nsat = 0
  do k=1,npersat(4)
     call strsplit(trim(satfile(4,k)),'.',f,nf)
     read (f(3),'(1x,i2)') year
     year = year + 1900
     if (year < 1970) year = year + 100

     open (unit=1,file=trim(satfile(4,k)))
     i = fstat(1,filestat)
     close(1)
     !print*,k,nf,filestat

     !The 112 byte records are for files provided by Lei Shi, which are different 
     !than those provided by Darren Jackson (which have 54-byte records.
     if (nf == 6) then
        nrec_f(k) = filestat(8) / 112
     else
        nrec_f(k) = filestat(8) / 54
     endif
     nrec = nrec +  nrec_f(k)  !54 or 112 bytes per All Sky record
     print*,trim(satfile(4,k)),nrec_f(k),nrec

     !gather satellite name
     call strsplit(trim(satfile(4,k)),'.',f,n)
     if (nsat == 0) then
        nsat=nsat+1
        c_sat(nsat) = trim(f(2))
     else
        iflag=0
        do i=1,nsat
           if (c_sat(i) == trim(f(2))) iflag=iflag+1
        end do
        if (iflag == 0) then
           nsat=nsat+1
           c_sat(nsat)=trim(f(2))
        endif
     endif
  end do
  nlines = nrec / 48
  !print*,'NREC',nrec
  !print*,'NLINES',nlines
  allocate (h_data(hnx,nlines))

  print '(a,i3,a2,4(a3,'',''))','# of Sats:',nsat,': ',c_sat(1:nsat)
  j = 0
  oldtime = 0
  !For each HIRS file...
  do k=1,npersat(4)

     !Get date info from the matchup filename
     call strsplit(trim(satfile(4,k)),'.',f,nf)
     read (f(3),'(1x,i2)') year
     mon = 1
     read (f(4),'(1x,i3)') dom
     year = year + 1900
     if (year < 1970) year = year + 100
     !print*,year,dom

     read (f(2),'(1x,i2)') satnum
     if (nf == 6) then 
        open(unit=1,file=trim(satfile(4,k)),access='direct',recl=112)
     else
        open(unit=1,file=trim(satfile(4,k)),access='direct',recl=54)
     endif
        
     ifscan = 0
     ispot = 1
     oldtime=0.
     inext = 0

     ilastlat = -99900
     ilastlon = -99900

     do i=1,nrec_f(k)
        
        if (nf == 6) then 
           read (1,rec=i) xtime, xlon, xlat, xszen, xalt, lspot, lline, xrefl, xtbc, xtb
           iref = int(within(nint(xrefl*100.),minimum=0,maximum=105),1)
           ispot = int(lspot,1)
           iline = int(lline,2)
           ilat =  int(xlat*100)
           ilon =  int((xlon-180)*100.)
        else
           read (1,rec=i) itime, ilon, ilat, iline, ispot, iszen, ialt, iref, itb
           xtime = real(itime,4)/100.
           xlat = real(ilat) / 100.
           xlon = real(ilon) / 100. + 180.
           xszen = iszen / 100.
           xalt = real(ialt) / 10.
           xtb = float(itb)/100. + 100.
        endif

        !Increment Day of Month when the time goes from 86400sec  to 0sec
        if (   (xtime-oldtime) < (-86000.) .and. & !change -1000. to -86000. b/c that 
                                                    !           is what a real day change is
             &  xtime > 0.                 .and. &
             &  oldtime > 86350.          .and. & !this avoids changing day when the 
             &  inext == 0) then
           !print*,'CHANGE',oldtime,itime,dom,dom+1
           dom=dom+1
           inext = 1
           oldtime=xtime
        end if

        ! This test checks to make sure the latitude is changing.
        ! To fix error with file: HIRS.N07.Y84.D206.AS
        if (i == 1)  then
           test_position=1 
        else
           test_position = abs(ilat - ilastlat) + &
                &          abs(ilon - ilastlon)
        end if
        print*,ilastlat,ilat,ilastlon,ilon,test_position
        ilastlat=ilat
        ilastlon=ilon

        !store data
        if (   ispot > 0 .and. &
             & test_position /= 0) then
           !Time is stored as True Julian Date
           fullscan(ispot)%num = satnum
           fullscan(ispot)%time= julday(mon,dom,year,0,0,xtime)
           !Convert lat/lon and solar zenith to degrees
           fullscan(ispot)%lat = xlat
           fullscan(ispot)%lon = xlon
           fullscan(ispot)%sza = xszen
           !Altitude in km
           fullscan(ispot)%alt = xalt
           !Reflectance is already in %
           fullscan(ispot)%ref = iref
           !Convert Tb to Kelvin
           fullscan(ispot)%tb  = xtb
           fullscan(ispot)%name = trim(f(2))
        end if

        !record data from fullscan to h_data
!print*,ispot,iline,xtime,oldtime,j,ifscan
        if (ifscan == 0) then
           if (ispot == 1 .and. xtime > oldtime) then
              lastgoodspot=1
              ifscan = 1
           end if
        else
           if (ispot == (lastgoodspot+1) .and. xtime > oldtime) then
              lastgoodspot=lastgoodspot+1
              oldtime=xtime
           else
              if (ispot == 1 .and. lastgoodspot > 50) then
                 !store data
                 j=j+1
!print*,fullscan%name
                 h_data(:,j) = fullscan
                 lastgoodspot=1
                 oldtime=xtime
              else
                 !if this spot /= 1, then part of the scan is missing, so we won't keep it
                 ifscan = 0
              end if
           end if
        end if
     end do
     close(1)
  end do
  hny = j

  print*,'HNY=',hny
  do i=1,nsat
!     print*,c_sat(i),'=',count(h_data%name == c_sat(i))
  enddo

  !Allocate HIRS parameters
  allocate (poest(hny), poesx(hny), poesy(hny), poesz(hny))

  !Calculate HIRS parameters
  !  Get satellite position using near nadir points, then interpolate 
  !  new position for each time
  do j=1,hny
     poest(j) = (h_data(28,j)%time + h_data(29,j)%time) / 2.
     radlat1 = earth_radius(h_data(28,j)%lat,geolat=glat1)
     radlat2 = earth_radius(h_data(29,j)%lat,geolat=glat2)
     glat1=glat1/rad
     glat2=glat2/rad
     !nadir position on the Earth
     poesx(j) = 0.5 * (radlat1*cos(glat1) * cos(h_data(28,j)%lon/rad) + &
          &            radlat2*cos(glat2) * cos(h_data(29,j)%lon/rad))
     poesy(j) = 0.5 * (radlat1*cos(glat1) * sin(h_data(28,j)%lon/rad) + &
          &            radlat2*cos(glat2) * sin(h_data(29,j)%lon/rad))
     poesz(j) = 0.5 * (radlat1*sin(glat1) + radlat2*sin(glat2))
     mag = sqrt(poesx(j)**2 + poesy(j)**2 + poesz(j)**2)
     alt = 0.5 * (h_data(28,j)%alt + h_data(29,j)%alt)
     poesx(j) = poesx(j) * (mag+alt) / mag
     poesy(j) = poesy(j) * (mag+alt) / mag
     poesz(j) = poesz(j) * (mag+alt) / mag
  end do

!  s1 = count(poest < (curjul-dtime))+1
!  s2 = count(poest < (curjul+dtime))
!  nwi = s2 - s1 + 1

  call idl_where(abs(poest-curjul)<=dtimes,hok,nwi)

  print*,'HIRS scans:',nwi
  if (nwi < 1) then
     goto 999
  endif

  allocate (hx(hnx,nwi), hy(hnx,nwi), hz(hnx,nwi))
  allocate (hsza(hnx,nwi), hazn(hnx,nwi), hvza(hnx,nwi), hazi(hnx,nwi))
  allocate (hvx(hnx,nwi), hvy(hnx,nwi), hvz(hnx,nwi), hname(hnx,nwi))

  !For each point,
  do oj=1,nwi
     jj = oj    ! jj [1,nwi]
     j = hok(oj)! j  [1,nrec]
     do i=1,hnx
        !Calculate satellite position
        if (j == 1) then 
           ij1 = 1
           ij2 = 2
        else if (j == hny) then
           ij1 = hny-1
           ij2 = hny
        else 
           if (i <=28) then
              ij1 = j-1
              ij2 = j
           else
              ij1 = j
              ij2 = j+1
           endif
        end if
        if ((poest(ij2)-poest(ij1)) == 0) then
           ft = 0.
        else
           ft = (h_data(i,j)%time - poest(ij1)) / (poest(ij2)-poest(ij1))
        endif
        hx(i,jj) = poesx(ij1) + (poesx(ij2)-poesx(ij1)) * ft
        hy(i,jj) = poesy(ij1) + (poesy(ij2)-poesy(ij1)) * ft
        hz(i,jj) = poesz(ij1) + (poesz(ij2)-poesz(ij1)) * ft
        !Calculate view angles.
        radlat = earth_radius(h_data(i,j)%lat,geolat=glat1)
        glat1=glat1/rad
        px = radlat * (cos(glat1) * cos(h_data(i,j)%lon/rad))
        py = radlat * (cos(glat1) * sin(h_data(i,j)%lon/rad))
        pz = radlat * (sin(glat1))
        !Calculate the view angles - point on Earth to POES
        call zenazi_cartesian((/px,py,pz/),(/hx(i,jj),hy(i,jj),hz(i,jj)/),&
             &                hvza(i,jj),hazi(i,jj))
        !View vector used in finding nearby B1U points
        hvx(i,jj) = px - hx(i,jj)
        hvy(i,jj) = py - hy(i,jj)
        hvz(i,jj) = pz - hz(i,jj)
        !Calculate solar angles
        if (h_data(i,j)%time > 1.) then
           call caldat(h_data(i,j)%time,mon,dom,year)
           jday = (julday(mon,dom,year)-julday(1,1,year)+1)
           tod = real(h_data(i,j)%time - julday(mon,dom,year,0,0,0.0),4)*24.
           call solar_angle(h_data(i,j)%lat,h_data(i,j)%lon,jday,&
                &           tod,&                        !tod = time of day
                &           hsza(i,jj),hazn(i,jj))
        else
           hsza(i,jj) = -1.
           hazn(i,jj) = -400.
        endif
           hname(i,jj) = h_data(i,j)%name
           
        
     end do
  end do
  do i=1,nsat
     print*,c_sat(i),'=',count(hname == c_sat(i))
  enddo
!stop

  allocate (inlat(hnx,nwi), inlon(hnx,nwi))
  allocate (hb1v(hnx,nwi))
  allocate (hi(hnx,nwi), hj(hnx,nwi))
  allocate (hii(hnx,nwi), hjj(hnx,nwi))

  do i=1,nwi
     inlat(1:56,i) = h_data(1:56,hok(i))%lat
     inlon(1:56,i) = h_data(1:56,hok(i))%lon
  end do


  !Perform HIRS-B1U matchup for each B1U satellite
  do igeo=1,npersat(1)

!!     print*,'Time0:',dtime(xq)

     !Store info on each satellite for GEO-GEO matchups
     print*,'GEO:   ',trim(stripname(satfile(1,igeo)))
     istat = b1uread(trim(satfile(1,igeo)),b1u)

     !Not getting center currently
     !call b1u_get_center(trim(satfile(1,igeo)),center)
     
     !Problem (at least with FY-2C) when getting QC and the QCQA file is missing. 
     !When readit=0, FY2-C file QC is corrupt. The work around seems to be just 
     !leave readit=1, but this slows down I/O and processing.... oh well.
     call b1u_get_qc(trim(satfile(1,igeo)),b1u,qc,readit=1,debug=idebug)

     !prepare matchup filename
     call strsplit(trim(satfile(1,igeo)),'.',ff,nf)
     mdir='/net/glaze/home/kknapp/DATA/isccp/b1ucal/matchup/'//trim(ff(5))//'/'
     mfile = trim(ff(4))//'.'//&
          &  trim(ff(5))//'.'//&
          &  trim(ff(6))//'.'//&
          &  trim(ff(7))//'.'//&
          &  trim(ff(8))//'.'//&
          &  trim(ff(9))

     !Get GEO sublat,lon
     ny = b1u%img(3)
     nx = b1u%img(4)
     print*,'NX/NY:',nx,ny
     allocate (b1i(nx,ny), b1j(nx,ny))
     allocate (b1x(nx,ny), b1y(nx,ny), b1z(nx,ny))
     allocate (b1lat(nx,ny), b1lon(nx,ny))

     allocate (btemp(3,nx,ny))
     call b1u_dn2cal(chn,b1u,btemp,order=ich)
!!     print*,'Time1:',dtime(xq)

     if (idebug == 1) then
        do jch=1,3
           print*,jch,minval(btemp(jch,:,:)),maxval(btemp(jch,:,:)),count(qc(ich(jch),:) == 0)
        enddo
        do jch=1,5
           print*,jch,count(qc(jch,:) == 0),count(qc(jch,:) /= 0)
           
        enddo
        
     endif


     !Navigate HIRS foot prints
!!print*,'HIRS size: ',size(inlat)
     
     b1u%nav(149) = ny

     if (b1u%img(13) == 1) then
        call b1u_ll2ij(inlat,inlon,hi,hj,b1u%nav_gvar,vza=hb1v, subsat=subsat, center=center)
     else
        call b1u_ll2ij(inlat,inlon,hi,hj,b1u%nav,     vza=hb1v, subsat=subsat, center=center)
     endif
     hii = nint(hi)
     hjj = nint(hj)

!!     print*,'Time3:',dtime(xq)


     !Navigate the entire B1U image  ... takes 5.4 seconds
     do i=1,nx  
        b1i(i,:) = i
     end do
     do j=1,ny
        b1j(:,j) = j
     end do
     if (b1u%img(13) == 1) then
        call b1u_ij2ll(b1i, b1j ,b1lat, b1lon,&
             &         b1u%nav_gvar,subsat=subsat, center=center)
     else
        call b1u_ij2ll(b1i,b1j, b1lat, b1lon,&
             &         b1u%nav,     subsat=subsat, center=center)
     endif
     print*,'subsat:',subsat
     geolat(igeo) = subsat(1)
     geolon(igeo) = subsat(2)

!!     print*,'Time2a:',dtime(xq)

     !Get Cartesian coordinates of the entire B1U image
     do i=1,nx
        do j=1,ny
           if (abs(b1lat(i,j)) < 90.) then
              glat = atan(tan(b1lat(i,j)/rad)*omf2)
              call sincos(glat,slat,clat)
              call sincos(b1lon(i,j)/rad,slon,clon)
              erad = sqrt(1.d0+(omf2-1.0)*slat*slat)*ae
              b1x(i,j) = erad * clat * clon
              b1y(i,j) = erad * clat * slon
              b1z(i,j) = erad * slat
           end if
        end do
     end do

!!     print*,'Time2b:',dtime(xq)
print*,'Total possible matchups:',count(hii > db1pix      .and. &
     &         hii < (nx-db1pix) .and. &
     &         hjj > db1pix      .and. &
     &         hjj < (ny-db1pix) .and. &
     &         abs(1./cos(hb1v/rad) - 1./cos(hvza/rad)) < dmulimit .and. &
     &         hb1v < 85.)

     do isat=1,nsat

        nummat = count(hii > db1pix      .and. &
             &         hii < (nx-db1pix) .and. &
             &         hjj > db1pix      .and. &
             &         hjj < (ny-db1pix) .and. &
             &         abs(1./cos(hb1v/rad) - 1./cos(hvza/rad)) < dmulimit .and. &
             &         hb1v < 85.        .and. &
             &         hname == c_sat(isat))

        !print*,c_sat(isat),' # of possible matchups:',nummat

        if (nummat > 0) then
           print*,'CHANNELS:',ich

           !open output matchup file
           print*,trim(mdir)//c_sat(isat)//'.'//trim(mfile),nummat
           open (unit=22,file=trim(mdir)//c_sat(isat)//'.'//trim(mfile),form='binary')
           !Number of B1 channels for comparison
           write (22) ich
           icount=0
           prof(:) = 0.0
           do j=1,nwi
              jj = hok(j)
              do i=1,hnx
                 hmu = 1./cos(hvza(i,j)/rad) !cos(HIRS VZA)
                 bmu = 1./cos(hb1v(i,j)/rad) !cos(B1U VZA)
                 dmu = hmu - bmu
                 if (   hii(i,j)  > db1pix      .and. &
                      & hii(i,j)  < (nx-db1pix) .and. &
                      & hjj(i,j)  > db1pix      .and. &
                      & hjj(i,j)  < (ny-db1pix) .and. &
                      & hb1v(i,j) < 85.         .and. &
                      & abs(dmu)  < dmulimit    .and. &
                      & hname(i,j) == c_sat(isat)) then
                    !For each HIRS foot print
                    !Calculate the B1U lat/lon based on DB1PIX
                    ! OK ... the lats and lons are being reproduced by the navigation ...
!!$qwerty = dtime(xq)
                    blat = b1lat(hii(i,j)-db1pix:hii(i,j)+db1pix,&
                         &       hjj(i,j)-db1pix:hjj(i,j)+db1pix)
                    blon = b1lon(hii(i,j)-db1pix:hii(i,j)+db1pix,&
                         &       hjj(i,j)-db1pix:hjj(i,j)+db1pix)
                    bx = b1x(hii(i,j)-db1pix:hii(i,j)+db1pix,&
                         &   hjj(i,j)-db1pix:hjj(i,j)+db1pix)
                    by = b1y(hii(i,j)-db1pix:hii(i,j)+db1pix,&
                         &   hjj(i,j)-db1pix:hjj(i,j)+db1pix)
                    bz = b1z(hii(i,j)-db1pix:hii(i,j)+db1pix,&
                         &   hjj(i,j)-db1pix:hjj(i,j)+db1pix)
                    
!!$prof(2) = prof(2) + dtime(xq)
                    magbv = sqrt(bx*bx + by*by + bz*bz)
                    maghv = sqrt(hvx(i,j)*hvx(i,j) + &
                         &       hvy(i,j)*hvy(i,j) + &
                         &       hvz(i,j)*hvz(i,j))
                    do    ki=1,2*db1pix+1
                       do kj=1,2*db1pix+1
                          if (magbv(ki,kj) > 10.) then
                             bx(ki,kj) = bx(ki,kj) - hx(i,j)
                             by(ki,kj) = by(ki,kj) - hy(i,j)
                             bz(ki,kj) = bz(ki,kj) - hz(i,j)
                             !Determine the angle between the B1 vectors and the 
                             !  HIRS view vector
                             magbv(ki,kj) = sqrt(bx(ki,kj)*bx(ki,kj) + &
                                  &              by(ki,kj)*by(ki,kj) + &
                                  &              bz(ki,kj)*bz(ki,kj))
                             !angle between view vectors
                             ! cos(angle) = A dot B / (mag(A)*mag(B))
                             bang(ki,kj) = (hvx(i,j)*bx(ki,kj) + hvy(i,j)*by(ki,kj) + &
                                  &         hvz(i,j)*bz(ki,kj)) / maghv / magbv(ki,kj)
!!$                       bang(ki,kj) = acos(within(bang(ki,kj),&
!!$                            &                    minimum=-1.e0,maximum=1.0e0))*rad
                       !bang(ki,kj) = bang(ki,kj)
                          end if
                       end do
                    end do
!!$prof(3) = prof(3) + dtime(xq)
!!$print*,i,j,minval(bang),maxval(bang)
              !
              !The angular limit varies with view angles
                    dlon = abs(subsat(2) - inlon(i,j))
                    if (dlon > 180) dlon = 360 - dlon
                    !The following only works for satellites near the equator
                    !because it assumes the spherical triangle is a right triangle
                    beta=cos(dlon/rad) * cos(inlat(i,j)/rad)
                    beta=max(min(beta,1.0e0),-1.0e0)
                    beta = acos(beta)
                    d1 = subsat(3) * sin(beta) / (sin((180.-hb1v(i,j))/rad))
                    d2 = maghv
                    adj = d1 * cos(hvza(i,j)/rad) / (d2 * cos(hb1v(i,j)/rad))
                    do l=1,3
!!$                 fov_lim(l) = 0.625 + &  !AlphaH - HIRS FOV
!!$                      &       fov_frac(l) * alphag * adj*rad
                       !FOV_LIM is now cos(FOV_LIM)
                       fov_lim(l) = cos((0.625 + &  !AlphaH - HIRS FOV
                            &            fov_frac(l) * alphag * adj*rad)/rad)
                       !bang = angle
                       !fov_n(l) = count(bang < fov_lim(l))
                       !bang = cos(angle)
                       fov_n(l) = count(bang >= fov_lim(l))
                    end do
!!$              if (i == 28) then
!!$                 print '(2i5,2f8.2,2f8.3,3(f8.3,i4))', i,j,inlat(i,j),inlon(i,j),&
!!$                   &                  minval(bang),maxval(bang),&
!!$                   &                  (fov_lim(l),nint(fov_n(l)),l=1,3)
!!$                 print '(10x,2f8.2,2f8.3)',inlat(i,j)-b1lat(hii(i,j),hjj(i,j)),&
!!$                      & inlon(i,j)-b1lon(hii(i,j),hjj(i,j)),bang(db1pix+1,db1pix+1)
!!$                 !print*,d1,d2,hvza(i,j),hb1v(i,j)
!!$              end if
!!$prof(4) = prof(4) + dtime(xq)

                    !get average/stdev values of HIRS
                    if (i == 1) then 
                       hi1 = 1
                       hi2 = 3
                    else if (i == 56) then
                       hi1 = 54
                       hi2 = 56
                    else
                       hi1 = i-1
                       hi2 = i+1
                    end if
                    if (jj == 1) then 
                       hj1 = 1
                       hj2 = 3
                    else if (jj == hny) then
                       hj1 = hny-2
                       hj2 = hny
                    else
                       if (h_data(i,jj+1)%num == h_data(i,jj-1)%num) then
                          hj1 = jj-1
                          hj2 = jj+1
                       else
                          if (h_data(i,jj+1)%num == h_data(i,jj)%num) then
                             hj1 = jj
                             hj2 = jj+2
                          else
                             hj1 = jj-2
                             hj2 = jj
                          end if
                       end if
                    end if
!!$prof(5) = prof(5) + dtime(xq)
                    
                    call avevar(reshape(h_data(hi1:hi2,hj1:hj2)%tb(8),(/9/)) ,9,av8,sd8)
                    call avevar(reshape(h_data(hi1:hi2,hj1:hj2)%tb(12),(/9/)),9,av12,sd12)
                    call avevar(reshape(h_data(hi1:hi2,hj1:hj2)%tb(19),(/9/)),9,av19,sd19)
                    
                    !get parameters for B1U
                    call zenazi_angles((/ inlat(i,j),inlon(i,j)/), subsat, vv,av)
!print*,(/ inlat(i,j),inlon(i,j)/), subsat, vv,av

!print*,'Testing View angle calculation'
!call zenazi_angles((/0., -75. /), (/0.,-80.,42175./),vv,av)
!print*,vv,av


!!$prof(6) = prof(6) + dtime(xq)
                    
                    !Convert angles to less than 180. for conversion to int
                    if (inlon(i,j) > 180.) inlon(i,j) = inlon(i,j) - 360.
                    if (hazi(i,j) > 180.) hazi(i,j) = hazi(i,j) - 360.
                    if (hazn(i,j) > 180.) hazn(i,j) = hazn(i,j) - 360.
                    if (av        > 180.) av        = av        - 360.
                    
                    !write HIRS output
                    icount=icount+1
                    write (22) h_data(i,jj)%time
                    write (22) int(h_data(i,jj)%num,2)  !int(h_data(i,jj)%num,4)
                    write (22) int(inlat(i,j)*100.,2),int(inlon(i,j)*100.,2) !inlat(i,j),inlon(i,j)
                    write (22) int(hsza(i,j)*100.,2), int(hazn(i,j)*100.,2)  !hsza(i,j), hazn(i,j)
                    write (22) int(hvza(i,j)*100.,2), int(hazi(i,j)*100.,2)  !p
print*
print*,'HIRS'
print*,'Time  ',h_data(i,jj)%time
print*,'Num   ',int(h_data(i,jj)%num,2)
print*,'LatLon',int(inlat(i,j)*100.,2),int(inlon(i,j)*100.,2)
print*,'szaAzn',int(hsza(i,j)*100.,2), int(hazn(i,j)*100.,2)
print*,'vzaAzi',int(hvza(i,j)*100.,2), int(hazi(i,j)*100.,2)
print*,'Temp  ',int((h_data(i,jj)%tb(8) -50.)*100.,2),  &
                         & int((av8-50.)*100.,2),  int(within(sqrt(sd8),maximum=100.)*100.,2)

                    if (ich(1) > 0) write (22) int((h_data(i,jj)%tb(8) -50.)*100.,2),  &
                         & int((av8-50.)*100.,2),  int(within(sqrt(sd8),maximum=100.)*100.,2)

                    if (ich(2) > 0) write (22) int((h_data(i,jj)%tb(12)-50.)*100.,2), &
                         & int((av12-50.)*100.,2), int(within(sqrt(sd12),maximum=100.)*100.,2)
                    if (ich(3) > 0) write (22) int((h_data(i,jj)%tb(19)-50.)*100.,2), &
                         & int((av19-50.)*100.,2), int(within(sqrt(sd19),maximum=100.)*100.,2)

                    write (22) int(vv*100.,2), int(av*100.,2) !B1U view and azimuth angles
                    !Move QI/QJ loop to here and do it only once ... collecting sums for each limit and each channel
                    
                    iq = 1
                    
                    if (iq == 1) then !an attempt to speed things up
                       sw  = 0.d0
                       sx  = 0.d0
                       sxx = 0.d0
                       dx  = 0.d0
                       dxx = 0.d0
                       do    qi=1,2*db1pix+1
                          do qj=1,2*db1pix+1
                             do lf=2,2 !FOV
                                !                       if (bang(qi,qj) <= fov_lim(lf)) then
                                if (bang(qi,qj) >= fov_lim(lf)) then
                                   do ic = 1,3 !channel
                                      if (ich(ic) > 0) then
                                         if (qc(ich(ic),hjj(i,j)+qj-db1pix) == 0) then
                                            sw(lf,ic) = sw(lf,ic) + 1.0d0
                                            sx(lf,ic) = sx(lf,ic) + real(btemp(ic,hii(i,j)+qi-db1pix,&
                                                 &                                hjj(i,j)+qj-db1pix),8)
                                            sxx(lf,ic)=sxx(lf,ic) + real(btemp(ic,hii(i,j)+qi-db1pix,&
                                                 &                                hjj(i,j)+qj-db1pix),8) * &
                                                 &                  real(btemp(ic,hii(i,j)+qi-db1pix,&
                                                 &                                hjj(i,j)+qj-db1pix),8)
                                            dx(lf,ic) = dx(lf,ic) + real(b1u%image%scan(ich(ic),hii(i,j)+qi-db1pix,&
                                                 &                                              hjj(i,j)+qj-db1pix),8)
                                            dxx(lf,ic)= dxx(lf,ic)+ real(b1u%image%scan(ich(ic),hii(i,j)+qi-db1pix,&
                                                 &                                              hjj(i,j)+qj-db1pix),8) * &
                                                 &                  real(b1u%image%scan(ich(ic),hii(i,j)+qi-db1pix,&
                                                 &                                              hjj(i,j)+qj-db1pix),8)
                                         end if
                                      end if
                                   end do
                                end if
                             end do
                          end do
                       end do
                       !output results
                       do lf=2,2
                          write (22) int(fov_n(lf),2)
                          do ic=1,3
                             if (ich(ic) > 0) then
                                !Center B1 DN
                                write (22) int(b1u%image%scan(ich(ic),hii(i,j),hjj(i,j)),2)
                                if (sw(lf,ic) > 1.) then
                                   !Avg DN
                                   write (22) int(nint((dx(lf,ic)/sw(lf,ic)-512.)*50.),2)
                                   !Std Dev DN
                                   write (22) int(nint(sqrt(within((dxx(lf,ic)*sw(lf,ic)-dx(lf,ic)*dx(lf,ic))/&
                                        &                             (sw(lf,ic))/(sw(lf,ic)-1.0d0),minimum=0.0d0))*100.),2)
                                   !avg temp
                                   aout = int(nint((sx(lf,ic)/sw(lf,ic)-50.)*100.),2)
                                   write (22) int(nint((sx(lf,ic)/sw(lf,ic)-50.)*100.),2)
                                   !temp std dev
                                   bout = int(nint(sqrt(within((sxx(lf,ic)*sw(lf,ic)-sx(lf,ic)*sx(lf,ic))/&
                                        &                             (sw(lf,ic))/(sw(lf,ic)-1.0d0),minimum=0.0d0))*100.),2)
                                   write (22) int(nint(sqrt(within((sxx(lf,ic)*sw(lf,ic)-sx(lf,ic)*sx(lf,ic))/&
                                        &                             (sw(lf,ic))/(sw(lf,ic)-1.0d0),minimum=0.0d0))*100.),2)
print*,'GEO'
print*,'DNij',int(b1u%image%scan(ich(ic),hii(i,j),hjj(i,j)),2)
print*,'MnDN',int(nint((dx(lf,ic)/sw(lf,ic)-512.)*50.),2)
print*,'SdDN',int(nint(sqrt(within((dxx(lf,ic)*sw(lf,ic)-dx(lf,ic)*dx(lf,ic))/&
                                        &                             (sw(lf,ic))/(sw(lf,ic)-1.0d0),minimum=0.0d0))*100.),2)
print*,'MnTb',int(nint((sx(lf,ic)/sw(lf,ic)-50.)*100.),2)
print*,'SDTb',int(nint(sqrt(within((sxx(lf,ic)*sw(lf,ic)-sx(lf,ic)*sx(lf,ic))/&
                                        &                             (sw(lf,ic))/(sw(lf,ic)-1.0d0),minimum=0.0d0))*100.),2)
                                else if (sw(lf,ic) > 0.) then
                                   aout=int(nint((dx(lf,ic)/sw(lf,ic)-512.)*50.),2)
                                   bout=-1.
                                   write (22) int(nint((dx(lf,ic)/sw(lf,ic)-512.)*50.),2), int(-51.*100,2)
                                   write (22) int(nint((sx(lf,ic)/sw(lf,ic)-50.)*100.),2), int(-51.*100,2)
                                else
                                   aout=-1.
                                   bout=-1.
                                   write (22) int(-51.*100.,2), int(-51.*100.,2)
                                   write (22) int(-51.*100.,2), int(-51.*100.,2)
                                end if
                                
                             end if

                          end do
                       end do
                    else if (iq == 0) then !the original routine
                       print*,'wrong IQ'
                       stop
                    else
                       print*,'IQ Must be set.'
                       print*,'IQ=0 was the original calculation'
                       print*,'IQ=1 is trying to speed it up'
                    endif
!!$prof(7) = prof(7) + dtime(xq)
                    
                 end if
              end do
           end do
        endif
     enddo!isat
     close (22)
!!$print*,'Profile'
!!$print*,prof(1:7)

     deallocate (b1i, b1j, b1lat, b1lon, b1x, b1y, b1z, btemp)
  end do
  !Perform GEO-GEO Matchups

  !Output GEO-GEO matchup files

999 continue
end program b1u_HIRS_as_matchup
