;****I* idl/idl.b1u_ij2ll
; NAME:
;    B1U_IJ2LL
;
; PURPOSE:
;    To convert line/pixel coordinate point(s) of a satellite image to
;    latitude/longitude.
;
; CATEGORY:
;    B1U Navigation
;
; INPUTS:
;    ipoints - Pixel locations (default is in image coordinates)
;    jpoints - Line locations (default is in image coordinates)
;    NAV     - B1U Navigation array (LONARR(200)) from b1uread
;
; KEYWORD PARAMETERS:
;    SATCOOR - Optionally input line/pixels in original satellite
;              coordinates.
;    IOUTPUT - Optionally output debugging information
;    RADIANS - Optionally output lat/lon as radians (instead of the
;              default units)
;
; OUTPUTS:
;    latitude (degrees)
;    longitude  (degrees)
;
; Optional outputs:
 
;    glat,glon - Subsatellite latitude/longitude (degrees)
;    grad      - Satellite distance from center of Earth (km)
;
;
; USES
; * atan2.pro
; * b1u_imc.pro
; * cart2earth.pro
; * cheb2cart.pro
; * const.pro
; * ftime.pro
; * gha.pro
; * kep2cart.pro
; 
; 
; USED BY
; * b1read_aes_g7.pro
; * b1read_cma_fy2.pro
; * b1read_csu_g6.pro
; * b1read_csu_g7.pro
; * b1read_jma_gms.pro
; * b1read_noa_goe.pro
; * b1read_uws_g56.pro
; * b1read.pro
; * b1u_get_center.pro
; 
;    
; MODIFICATION HISTORY:
;    $Id: b1u_ij2ll.pro 22796 2010-09-16 13:06:12Z kknapp $
;
; SOURCE

pro b1u_ij2ll,ipoints,jpoints,outlat,outlon, $
              NAV=NAV, $        ;b1u navigation array
              GLAT=GLAT,GLON=GLON,GRAD=GRAD, $ ;satellite position
              SATCOOR=SATCOOR, $
              IOUTPUT=IOUTPUT, $
              RADIANS=RADIANS, $
              CHUNKSIZE=CHUNKSIZE, $
              b1TIMstr=b1TIMstr, $ ;provided for continuity with older navigation
              b1ORBstr=b1ORBstr, $ ;provided for continuity with older navigation
              b1ATTstr=b1ATTstr, $ ;provided for continuity with older navigation
              b1IMGstr=b1IMGstr, $ ;provided for continuity with older navigation
              NCDF=NCDF,$          ;provided for compatibility with netCDF files
              SKEW=SKEW, $
              CENTER=CENTER, $
              SCANANGLE=SCANANGLE, $ ;return the scan angle for the pixels
              debug=debug

;Set up navigation constants
@const.pro

if keyword_set(SKEW) then begin
    skew_fact = skew
endif else begin 
    skew_fact = 0.0d0
endelse 


; The chunksize parameter works differently fot ij2ll than
; ll2ij. Likely, the ll2ij is faster when chunksize is small because
; it iterates. ll2ij, conversely, does not iterate, so is fastest when
; it does NOT iterate
if not(keyword_set(CHUNKSIZE)) then CHUNKSIZE=10000000l

;set up the output flag
iout = 0
if keyword_set(IOUTPUT) then iout = 1
; don't output with too many points
if (n_elements(ipoint) gt 2) then iout = 0

; If NAV is a structure, not an array, then 
if (n_elements(nav) gt 0 and $
    n_elements(nav) lt 5) then begin
    AREAstr = nav.b1uarea
    NAVstr  = nav.b1unav
    if keyword_set(SATCOOR) then begin
        ip = ipoints
        jp = jpoints
    endif else begin
        ip = AREAstr.west_vis_elem + ipoints*AREAstr.elem_res
        jp = AREAstr.north_bound + jpoints*AREAstr.line_res
    endelse
    ;call imcnav to convert lat/lon -> i/j
    b1u_imc,outlat,outlon,jp,ip,0,0L,1L,NAVstr,AREAstr,iout=iout, $
            glat=glat,glon=glon,grad=grad,SCANANGLE=SCANANGLE
    glat=glat*180./!pi
    glon=glon*180./!pi
    return
    stop
endif
if keyword_set(nav) then begin 
    if (nav(199) ne -9999) then begin
        skew_fact = nav(199) * 1.e-6
    endif
endif

roll = 0.d0 & pitch = 0.d0 & yaw = 0.d0

;WHICH navigation information is provided??
inavb1u = 0
if (n_elements(nav) eq 200) then inavb1u = 1 ; nav from b1u structure
if (n_elements(b1ORBstr) eq 1 and $
    n_elements(b1ATTstr) eq 1 and $
    n_elements(b1IMGstr) eq 1) then inavb1u = 2
if keyword_set(ncdf) then inavb1u = 3

icheb = 0                       ;Are Chebyshev polynomials present??
case inavb1u of
    1: begin
;       set up VARIABLES from the NAV array
;       Kepler elements
        kep = [nav(13)*1.d-2, $
               nav(15)*1.d-6, $
               nav(14)*1.d-3, $
               nav(17)*1.d-3, $
               nav(18)*1.d-3, $
               nav(16)*1.d-3]
;       Epoch time (in julian days)
        ep_date = nav(10)       ;YYJJJ
        ep_time = nav(11)       ;HHMMSS
        ep_msec = nav(12)       ;MsecMsecMsec
;       CHECK FOR BAD NAVIGATION INFORMATION
        if (ep_date eq 0 or $
            ep_date eq -9999) then begin
           ;If Keplerian elements is missing from the B1 header,
           ;try using the secondary Kepler values.
;           Other Kepler elements
            kep = [nav(23)*1.d-2, $
                   nav(25)*1.d-6, $
                   nav(24)*1.d-3, $
                   nav(27)*1.d-3, $
                   nav(28)*1.d-3, $
                   nav(26)*1.d-3]
            ep_date = nav(20)   ;YYJJJ
            ep_time = nav(21)   ;HHMMSS
            ep_msec = nav(22)   ;MsecMsecMsec
        endif
;       CHECK FOR BAD NAVIGATION INFORMATION
        if (ep_date eq 0 or $
            ep_date eq -9999) then begin
           ;If Keplerian elements is missing from the B1 header,
           ;try using the secondary Kepler values.
;           Other Kepler elements
            kep = [nav(33)*1.d-2, $
                   nav(35)*1.d-6, $
                   nav(34)*1.d-3, $
                   nav(37)*1.d-3, $
                   nav(38)*1.d-3, $
                   nav(36)*1.d-3]
            ep_date = nav(30)   ;YYJJJ
            ep_time = nav(31)   ;HHMMSS
            ep_msec = nav(32)   ;MsecMsecMsec
        endif
;       set up attitude parameters
        if (nav(39) ne -9999) then roll   =  nav(39)*1.d-6
        if (nav(40) ne -9999) then pitch  =  nav(40)*1.d-6
        if (nav(41) ne -9999) then yaw    = -nav(41)*1.d-6
;       image information
        cen_lin = nav(2)/10.
        if keyword_set(center) then begin
            if n_elements(center) ne 0 then cen_lin = cen_lin + center.cen_lin_corr(0)/10.
        endif
        cen_pix = nav(3)/10.
        linres = nav(4)
        pixres = nav(5)
        sline  = nav(0)
        spix   = nav(1)
        linang = nav(6)*1.d-9
        pixang = nav(7)*1.d-9

        cen_lin_img = (float(cen_lin) - float(sline))/ float(linres)
        if (nav(160) eq 1) then begin
            c0 = float(nav(161))*1.e-7 * 10.^(nav(162))
            c1 = float(nav(163))*1.e-7 * 10.^(nav(164))
            c2 = float(nav(165))*1.e-7 * 10.^(nav(166))
        endif else begin
            c0 = (float(cen_pix) -float(spix))/ float(pixres)
            c1 = 0.0e0
            c2 = 0.0e0
        endelse

        ny = nav(149)

        spper  = nav(8)*1.d-6   ;spin period in seconds/revolution
        imgdate = nav(147)
        imgtime = nav(148)/1000.      ;seconds since start of day * 1000
;       scan information
        scan1  = nav(131)
        sspra1 = nav(135)*1d-6
        sspdc1 = nav(136)*1d-6
        scan2  = nav(137)
        sspra2 = nav(141)*1d-6
        sspdc2 = nav(142)*1d-6
        spra1 = nav(65)  / 2.d0^21
        spdc1 = nav(66)  / 2.d0^21
        spra2 = nav(125) / 2.d0^21
        spdc2 = nav(126) / 2.d0^21
        if (nav(146) gt 0) then icheb = 1
        chebx = nav(86:96)  /2.d0^13
        cheby = nav(97:107) /2.d0^13
        chebz = nav(108:118)/2.d0^13
        ;Use JMA fit if RMS > 0
        if (nav(52) gt 0) then begin
            spra1 = nav(45) * 1.e-6
            spra2 = nav(47) * 1.e-6
            spdc1 = nav(46) * 1.e-6
            spdc2 = nav(48) * 1.e-6
            roll  = nav(42) * 1.e-6
            pitch = nav(43) * 1.e-6
            yaw   = nav(44) * 1.e-6
        endif

        if not(nav(151) eq nav(152) and $
               nav(151) eq (-9999l)) then begin
            center = {cen_lin_corr:0., $
                      cline_eq: fltarr(4)}
            center.cen_lin_corr = float(nav(151))
            center.cline_eq(0)  = float(nav(152),0,1)
            center.cline_eq(1)  = float(nav(153),0,1)
            center.cline_eq(2)  = float(nav(154),0,1)
            center.cline_eq(3)  = float(nav(155),0,1)
        endif


    end
    2: begin
;       set up VARIABLES from the B1 structures
        if keyword_set(b1ORBstr) then begin
            ep_date  = b1ORBstr.ep_date
            ep_time  = b1ORBstr.ep_time
            ep_msec   = b1ORBstr.ep_msec
            semima = b1ORBstr.semima
            incl   = b1ORBstr.incl 
            eccen  = b1ORBstr.eccen
            meana  = b1ORBstr.meana
            rasc   = b1ORBstr.rgtasc
            perigee= b1ORBstr.perigee 
            spper  = b1ORBstr.sper
            kep = [semima,eccen,incl, $
                   rasc,perigee,meana]
        endif
        if keyword_set(b1ATTstr) then begin
            roll = b1ATTstr.roll
            pitch= b1ATTstr.pitch
            yaw  = b1ATTstr.yaw
            spra = b1ATTstr.spra
            spdc = b1ATTstr.spdc
        endif
        if keyword_set(b1IMGstr) then begin
            imgdate=  b1IMGstr.frdate  ;= frdate
            imgtime=  b1IMGstr.frtime  ;= frtime
            cen_lin= b1IMGstr.cen_lin  ;= cen_lin
            if keyword_set(center) then begin
                if n_elements(center) ne 10 then cen_lin = cen_lin + center.cen_lin_corr(0)/10.
            endif
            cen_pix= b1IMGstr.cen_pix  ;= cen_pix
            linres=  b1IMGstr.linres   ;= linres
            pixres=  b1IMGstr.pixres   ;= pixres
            sline=   b1IMGstr.sline    ;= sline
            spix =   b1IMGstr.spix     ;= spix
            linang = b1IMGstr.linang   ;= linang
            pixang = b1IMGstr.pixang   ;= pixang
            cen_lin_img = (float(cen_lin) - float(sline))/ float(linres)
            cen_pix_img = (float(cen_pix) - float(spix))/ float(pixres)
        endif
        if keyword_set(b1TIMstr) then begin
            d     = b1TIMstr.d    ;= d
            gt1   = b1TIMstr.gt1  ;= gt1
            gt2   = b1TIMstr.gt2  ;= gt2
            gra1  = b1TIMstr.gra1 ;= gra1
            gra2  = b1TIMstr.gra2 ;= gra2
            spra1 = b1TIMstr.spra1 ;= -999.d0
            spra2 = b1TIMstr.spra2 ;= -999.d0
            spdc1 = b1TIMstr.spdc1 ;= -999.d0
            spdc2 = b1TIMstr.spdc2 ;= -999.d0
            chebx  = b1TIMstr.cheb_x ;= ch_x
            cheby  = b1TIMstr.cheb_y ;= ch_y
            chebz  = b1TIMstr.cheb_z ;= ch_z
            ch_a  = b1TIMstr.cheb_a ;= -999.d0
            ch_b  = b1TIMstr.cheb_b ;= -999.d0
            ch_c  = b1TIMstr.cheb_c ;= -999.d0
        endif
    end
    3: begin
;       set up VARIABLES from the netCDF file
        stop
    end
    0: begin
        print,'No navigation information provided.'
        stop
    end
endcase

if keyword_set(debug) then begin
    print,'Nav Info:'
    print,'RPY   :',roll,pitch,yaw
    print,'Kepler:',semima,incl,eccen,meana,rasc,perigee,spper
endif

;*****************
; Time information
;*****************
; Epoch time
year = ep_date/1000 + 1900 * (ep_date/1000 lt 1900)
jday = ep_date mod 1000
ejulday = julday(1,jday,year,0,0,0) + $
          ftime(ep_time) / 24.d0 + $
          ep_msec / 1000.d0 / 3600.d0 / 24.d0
;image time 
iyear = imgdate/1000 + 1900 * (floor(imgdate/1000) lt 1900)
idoy  = imgdate mod 1000
imgjulday = julday(1,idoy,iyear,0,0,0) + $
            imgtime / 3600.d0 / 24.d0
if (iout) then print,'IMG TIME:',iyear,idoy,imgtime
if (iout) then print,'EPOCH   :',year,jday,ep_time
if (iout) then print,'KEPLER:',kep

;*****************
; Satellite spin axis declination & right ascension
;*****************
spinmiss = 0
if (iout) then print,'spra1,2,spdc1,2',spra1,spra2,spdc1,spdc2
if ((spra1 eq spra2 and spra1 eq 0.) and $
    (spdc1 eq spdc2 and spdc1 eq 0.)) then begin
    spinmiss = 1
    ;get satellite position using image time and kep2cart
    dtime = (imgjulday - ejulday) * 24.d0 * 3600.d0
    kep2cart,kep,dtime,xs1,xs2,xs3
    ;get subsat using cart2earth
    time = ejulday + dtime/3600.d0/24.d0
    cart2earth,xs1,xs2,xs3,time,glat,glon,grad
    if (spra1 eq spra2 and spra1 eq 0.) then begin
        ;approximate using the subsatellite longitude
        spra = -glon / rad
    endif
    if (spdc1 eq spdc2 and (spdc1 eq 0. or spdc1 eq 90.)) then begin
        ;approximate using the sub satellite latitude
        spdc = (-90. + glat) / rad
    endif
    if (iout) then print,'CALCed SPRA, SPDC:'
    if (iout) then print,SPRA, SPDC
endif



if (iout) then print,'RPY',roll,pitch,yaw

;omf is constant that relates to the flattening of the Earth
omf2 = (be/ae)^2

;Make sure IP & JP are in image coordinates
ip = ipoints
jp = jpoints
if (keyword_set(SATCOOR)) then begin
    ip = (ipoints-spix)/pixres
    jp = (jpoints-sline)/linres
endif else begin

;CONVERT i,j to scan/elevation angles
;scan = (ip*pixres + spix - cen_pix)*pixang/rad
;elev = (cen_lin - (jp*linres+sline))*linang/rad
;dline = cen_lin - (jp*linres+sline)
;scan = (ip*pixres + spix - cen_pix + dline*skew_fact)*pixang/rad
;elev = dline*linang/rad

    if (n_elements(c0) gt 0) then cen_pix_img = c0 + c1 * jp + c2 * jp * jp
    if keyword_set(center) then begin
        if n_elements(center) ne 10 then begin
            xtemp = findgen(ny) - ny/2.
            correction = center.cline_eq(0) + $
                         center.cline_eq(1)*xtemp + $
                         center.cline_eq(2)*xtemp*xtemp + $
                         center.cline_eq(3)*xtemp*xtemp*xtemp
            
            cen_pix_img = (float(cen_pix) - float(spix) + correction) / float(pixres) ;cen_pix_img(jp)
            cen_pix_img = cen_pix_img(jp)
        endif
    endif
    scan = (ip - cen_pix_img) * pixres * pixang/rad
    elev = (cen_lin_img - jp) * linres * linang/rad
    ;elev = ((cen_lin_img - jp) * pixres - sline) * linang/rad
endelse 

if (iout) then print,'scan, elev'
if (iout) then print,scan,elev

if keyword_set(SCANANGLE) then begin
    outlat = elev
    outlon = scan
    return
endif


;reform ipoint/jpoint
insize = size(ipoints)
npoints = n_elements(ipoints)
scans = reform([scan],npoints)
elevs = reform([elev],npoints)
jp_1d = reform([jp],npoints)

lattt = dblarr(npoints)
lonnn = dblarr(npoints)

nchunk = ceil(npoints/float(chunksize))
nps = npoints

for ichunk = 0l,nchunk-1 do begin
chlow = (ichunk  )*chunksize
chhigh= ((ichunk+1)*chunksize-1) < (nps-1)
scan = scans(chlow:chhigh)
elev = elevs(chlow:chhigh)
npoints = chhigh - chlow + 1
jp = jp_1d(chlow:chhigh)

;======
;  VS
;======
;Get satellite viewing angle from the scan/elevation by correcting for
;the roll/pitch/yaw angles
useold = 1
if (useold) then begin
;==================================================
;  The following is the INCORRECT calculation of
;  the VS vector. It uses the old method which
;  has an incorrect treatment of YAW
;==================================================
;convert elev/scan to instrument pointing vector VS
    capr = cos(scan+roll/rad)
    sapr = sin(scan+roll/rad)
    cppe = cos(elev+pitch/rad)
    sppe = sin(elev+pitch/rad)
    syaw = sin(yaw/rad)
    vs = [[CAPR*CPPE-SAPR*SYAW*SPPE],$
          [SAPR*CPPE+CAPR*SYAW*SPPE],$
          [-COS(YAW/rad)*SPPE]]
    vs1 = vs(0) & vs2 = vs(1) & vs3 = vs(2)
;==================================================
endif else begin
;==================================================
;  The following is the CORRECT calculation of
;  the VS vector. It uses the NEW method.
;==================================================
    e_ = tan(elev + pitch/rad)
    s_ = tan(scan  + roll /rad)
    vs3 = s_*sin(yaw/rad) - e_*cos(yaw/rad)
    vs2 = s_*cos(yaw/rad) - vs3*tan(yaw/rad)
    vs1 = vs2*0.d0 + 1.d0
    vs=[[vs1],[vs2],[vs3]]
;==================================================
endelse
if (iout) then print,'VS - Satellite view vector'
if (iout) then help,vs
if (iout) then print,vs ;([vs1,vs2,vs3])

;calculate the time of the observation since epoch (in seconds)
dtime = (imgjulday - ejulday) * 24.d0 * 3600.d0 + $ ;time since epoch at start of image
        (jp+sline/float(linres)) * spper
if (iout) then print,'IMG time, Epoch Time'
if (iout) then print,imgjulday,ejulday,format='(2f20.10)'
if (iout) then print,'Diff=',(imgjulday - ejulday),' days'
if (iout) then print,(imgjulday - ejulday) * 24.d0,(jp+sline/float(linres))
if (iout) then print,jp,sline,spper,linres
if (iout) then print,'DTIME:',reform([dtime])

;if (iout) then stop

; get satellite location in cartesian coordinates
if (icheb and $
    max(dtime) le (13.d0*3600.d0) and $
    min(dtime) ge 0.d0 and $
    total(abs(chebx)) ne 0.) then begin
;   Use Chebyshev polynomials to determine Cartesian coordinates
    cheb2cart,chebx,dtime,xs1
    cheb2cart,cheby,dtime,xs2
    cheb2cart,chebz,dtime,xs3
endif else begin
;   Otherwise use Kepler elements
    kep2cart,kep,dtime,xs1,xs2,xs3
endelse
if (iout) then print,'Sat position'
if (iout) then print,(reform([xs1,xs2,xs3]))
;get greenwich hour angle
time = imgjulday + (jp+sline/float(linres))*spper/3600.d0/24.d0
if (iout) then print,'GHA TIME:',(reform([time])-ejulday)*24.d0
gra = gha(time)

; get satellite location in Earth coordinates if requested
if (arg_present(GLAT) or $
    arg_present(glon) or $
    arg_present(grad) or iout) then begin

    cart2earth,xs1,xs2,xs3,time,glat,glon,grad

    if (iout) then print,'gat,glon,grad'
    if (iout) then print,glat,glon,grad 
end

px=xs1 & py=xs2 & pz=xs3
r = [[px],[py],[pz]]/ae
sathgt = sqrt(xs1^2+xs2^2+xs3^2)

;Calculate the SPRA and SPDC angles ... the new method uses the SPRA
;for the first and last scan line, instead of interpolating over time
;again.
if (spinmiss eq 0) then begin
    spra = spra1 + dtime/3600./13. * (spra2-spra1)
    spdc = spdc1 + dtime/3600./13. * (spdc2-spdc1)
    spra = spra / rad
    spdc = spdc / rad
    if (spra2 eq 0.) then spra = spra1 / rad
    if (spdc2 eq 0.) then spdc = spdc1 / rad
endif
if (iout) then print,'spra,spdc'
if (iout) then print,spra*rad,spdc*rad
;stop
;======
;   S
;======
;Calculate the S transformation matrix
n = npoints
s=dblarr(3,3,n)
cdc = cos(spdc)
s(2,0,*) = cdc*cos(spra)
s(2,1,*) = cdc*sin(spra)
s(2,2,*) = sin(spdc)
PDOTS = PX*S(2,0,*) + PY*S(2,1,*) + PZ*S(2,2,*)
PSQ = PX^2 + PY^2 + PZ^2
S1DEN = SQRT(PSQ - PDOTS^2)
S(0,0,*) = (-PX + PDOTS*S(2,0,*))/S1DEN                                     
S(0,1,*) = (-PY + PDOTS*S(2,1,*))/S1DEN
S(0,2,*) = (-PZ + PDOTS*S(2,2,*))/S1DEN
S(1,0,*)=S(2,1,*)*S(0,2,*)-S(2,2,*)*S(0,1,*)
S(1,1,*)=S(2,2,*)*S(0,0,*)-S(2,0,*)*S(0,2,*)
S(1,2,*)=S(2,0,*)*S(0,1,*)-S(2,1,*)*S(0,0,*)
if (iout) then print,'S matrix'
if (iout) then print,(s)

;======
;  VC
;======
;transform VS (pointing vector) to earth-centered fixed coordinates (VC)
;to do this, we need the S vector from satcrd
;vc = transpose(s) # vs
vc = dblarr(n,3)
vc(*,0) = vs(*,0)*s(0,0,*) + vs(*,1)*s(1,0,*) + vs(*,2)*s(2,0,*)
vc(*,1) = vs(*,0)*s(0,1,*) + vs(*,1)*s(1,1,*) + vs(*,2)*s(2,1,*)
vc(*,2) = vs(*,0)*s(0,2,*) + vs(*,1)*s(1,2,*) + vs(*,2)*s(2,2,*)

if (iout) then print,'VC'
if (iout) then print,reform((vc))

;find where vc crosses the Earth's surface
;solving a quadratic equation for H
;H is the distance from the satellite to where vc strikes the Earth
off = rebin(reform([1.d0,1.d0,1.d0/omf2],1,3),n,3)
q1 = total(vc*vc*off,2)         ;[1.d0,1.d0,1.d0/omf2])
q2 = total(vc* r*off,2)         ;[1.d0,1.d0,1.d0/omf2])
q3 = total(r * r*off,2)-1.d0    ;[1.d0,1.d0,1.d0/omf2]) - 1.0d0
d = q2*q2-q1*q3
on = where(d ge 0.d0,non,comp = off,ncomp = noff)

if (iout) then print,'d: ',d
;set up the initial lat/lon arrays with missing values
lat = dblarr(n)
lat(*) = -999.
lon = lat
; get locations of on-earth points
if (non gt 0) then begin        ;d ge 0.d0) then begin
    h = (-q2(on)-sqrt(d(on)))/q1(on)
if (iout) then print,'h: ',h
;given distance H, can determine X: the point on the earth's surface
    X = r(on,*) + rebin(reform(h,non,1),non,3)*vc(on,*)
if (iout) then print,'x: ',x
;if (iout) then stop
;get latitude
    lat(on) = atan(x(*,2) / omf2 / sqrt(x(*,0)^2+x(*,1)^2))
;get longitude in intertial coord.
    lon(on) = atan2(x(*,1),x(*,0))
;subtract greenwich RA for real longitude
    lon(on) = (lon(on) - gra(on)) mod (2.d0*!dpi)
    lon(on) = lon(on) - 2.d0*!dpi*(lon(on) gt !dpi)
    lon(on) = lon(on) + 2.d0*!dpi*(lon(on) lt (-1.d0*!dpi))
    lon(on) = lon(on) * rad
    lat(on) = lat(on) * rad
endif

outlon = lon 
outlat = lat 

if keyword_set(RADIANS) then begin
    outlat = outlat / rad
    outlon = outlon / rad
endif
if (iout) then print,'lat/lon'
if (iout) then print,outlat,outlon
;if (iout) then stop

lattt(chlow:chhigh) = outlat
lonnn(chlow:chhigh) = outlon
endfor

outlat = lattt
outlon = lonnn
if (nps eq 1) then return
outlat = reform([outlat],insize(1:insize(0)))
outlon = reform([outlon],insize(1:insize(0)))

end

;***
