;****I* idl/idl.b1u_ll2ij
; NAME:
;    B1U_LL2IJ
;
; PURPOSE:
;    To convert latitude/longitude to image (or satellite) coordinates.
;
; INPUTS:
;    inlat - latitude (scalar or array) in degrees
;    inlon - longitude (scalar or array) in degrees
;    NAV   - B1U Navigation array
;
; KEYWORD PARAMETERS:
;    RADIANS - Optionally, the input lat/lon values are provided in
;              radians
;    IOUTPUT - Optionally output debugging information
;    SATCOOR - Optionally output line/pixels in original satellite
;              coordinates.
;
; OUTPUTS:
;    ipoints - Pixel location of lat/lon coordinates in image
;              coordinates.
;    jpoints - Line location of lat/lon coordinates in image
;              coordinates
; Optional
;    glat    - Subsatellite latitude locations (deg)
;    glon    - Subsatellite longitude locations (deg)
;    grad    - Satellite distance to center of Earth (km)
;
; USES
; - const.pro      - provides an interconsistent set of constants across many IDL routines
; - b1u_imc.pro    - processes navigation of the GVAR files
; - cart2earth.pro - Convert Cartesian coordinates to Earth
;                    coordinates.
; - cheb2cart.pro  - Calculate Cartesian coordinates from Chbyshev polynomials
; - ftime.pro
; - gha.pro
; - kep2cart.pro   - Convert keplerian parameters to a Cartesian position using a
;                    time since epoch
; - unit.pro       - only in debug mode (when iloop = 1)
;
; 
; USED BY
; This common utility is used by nearly any routine that processes
; ISCCP B1 data.
; 
; Examples include:
; * b1read_cma_fy2.pro
; * b1read_csu_g7.pro
; * b1u_2_grisat.pro
; * b1read_noa_goe.pro
; * b1read.pro
; * b1u_get_center.pro
; * jma_bestfit.pro
; * cma_bestfit.pro
; * b1u_coastline_navigation.pro
; * 
;
; NOTES
; This routine is meant to be identical to the Fortran routine of the
; same name (b1u_ll2ij).
; 
; MODIFICATION HISTORY:
;    $Id: b1u_ll2ij.pro 22795 2010-09-16 13:05:53Z kknapp $
;
; SOURCE

pro b1u_ll2ij,inlat,inlon,ipoint,jpoint, $
              NAV=NAV, $
              GLAT=GLAT,GLON=GLON,GRAD=GRAD, $
              SATCOOR=SATCOOR, $
              RADIANS=RADIANS, $
              IOUTPUT=IOUTPUT, $
              CHUNKSIZE=CHUNKSIZE, $
              b1TIMstr=b1TIMstr, $ ;provided for continuity with older navigation
              b1ORBstr=b1ORBstr, $ ;provided for continuity with older navigation
              b1ATTstr=b1ATTstr, $ ;provided for continuity with older navigation
              b1IMGstr=b1IMGstr, $ ;provided for continuity with older navigation
              NCDF=NCDF, $      ;provided for compatibility with netCDF files
              SKEW=SKEW, $      ;use new parameter to adjust for spin rate errors
              CENTER=CENTER, $
              SCANANGLE=SCANANGLE, $ ;return the scan angle to the pixels
              ERRORFLAG=ERRORFLAG, $ ;ERRORFLAG=0 --> Success, ERRORFLAG=1 --> NO CONVERGENCE
              debug=debug


;set up the output flag
iout = 0
if keyword_set(IOUTPUT) then iout = 1
if keyword_set(debug) then iout = 1

;Set up navigation constants
@const.pro
;Divide up the work into chunks so it processes faster

if not(keyword_set(CHUNKSIZE)) then CHUNKSIZE=1000l

; If NAV is a structure, not an array, then 
if (n_elements(nav) gt 0 and $
    n_elements(nav) lt 5) then begin
    AREAstr = nav.b1uarea
    NAVstr  = nav.b1unav
    
    ;call imcnav to convert lat/lon -> i/j
    b1u_imc,inlat,inlon,line,pixel,1L,0L,1L,NAVstr,AREAstr, $
            glat=glat,glon=glon,grad=grad,SCANANGLE=SCANANGLE,ioutput=ioutput
    glon = glon * 180./!pi
    glat = glat * 180./!pi
    if keyword_set(SATCOOR) then begin
        ipoint = pixel
        jpoint = line
    endif else begin 
        jpoint = (line - AREAstr.north_bound)/AREAstr.line_res
        ipoint = (pixel - AREAstr.west_vis_elem) / AREAstr.elem_res
    endelse
    return
    stop
endif

if keyword_set(SKEW) then begin
    skew_fact = skew
endif else begin 
    skew_fact = 0.0d0
endelse 
if keyword_set(nav) then begin 
    if (nav(199) ne -9999) then begin
        skew_fact = nav(199) * 1.e-6
    endif
endif

;WHICH navigation information is provided??
inavb1u = 0
if (n_elements(nav) eq 200) then inavb1u = 1 ; nav from b1u structure
if (n_elements(b1ORBstr) eq 1 and $
    n_elements(b1ATTstr) eq 1 and $
    n_elements(b1IMGstr) eq 1) then inavb1u = 2
if keyword_set(ncdf) then inavb1u = 3
icheb = 0                       ;Are Chebyshev polynomials present??

roll = 0.d0 & pitch = 0.d0 & yaw = 0.d0

case inavb1u of
    1: begin
;       set up VARIABLES from the NAV array
;       Kepler elements
        kep = [nav(13)*1.d-2, $
               nav(15)*1.d-6, $
               nav(14)*1.d-3, $
               nav(17)*1.d-3, $
               nav(18)*1.d-3, $
               nav(16)*1.d-3]
;       Epoch time (in julian days)
        ep_date = nav(10)       ;YYJJJ
        ep_time = nav(11)       ;HHMMSS
        ep_msec = nav(12)       ;MsecMsecMsec
;       CHECK FOR BAD NAVIGATION INFORMATION
        if (ep_date eq 0 or $
            ep_date eq -9999) then begin
           ;If Keplerian elements is missing from the B1 header,
           ;try using the secondary Kepler values.
;           Other Kepler elements
            kep = [nav(23)*1.d-2, $
                   nav(25)*1.d-6, $
                   nav(24)*1.d-3, $
                   nav(27)*1.d-3, $
                   nav(28)*1.d-3, $
                   nav(26)*1.d-3]
            ep_date = nav(20)   ;YYJJJ
            ep_time = nav(21)   ;HHMMSS
            ep_msec = nav(22)   ;MsecMsecMsec
        endif
;       CHECK FOR BAD NAVIGATION INFORMATION
        if (ep_date eq 0 or $
            ep_date eq -9999) then begin
           ;If Keplerian elements is missing from the B1 header,
           ;try using the secondary Kepler values.
;           Other Kepler elements
            kep = [nav(33)*1.d-2, $
                   nav(35)*1.d-6, $
                   nav(34)*1.d-3, $
                   nav(37)*1.d-3, $
                   nav(38)*1.d-3, $
                   nav(36)*1.d-3]
            ep_date = nav(30)   ;YYJJJ
            ep_time = nav(31)   ;HHMMSS
            ep_msec = nav(32)   ;MsecMsecMsec
        endif
;       set up attitude parameters
        if (nav(39) ne -9999) then roll   =  nav(39)*1.d-6
        if (nav(40) ne -9999) then pitch  =  nav(40)*1.d-6
        if (nav(41) ne -9999) then yaw    = -nav(41)*1.d-6
;       image information
        cen_lin = nav(2)/10.
        if keyword_set(center) then begin
            if (iout) then print, 'Center line: '+strtrim(cen_lin,2)+' > '+strtrim(cen_lin + center.cen_lin_corr(0)/10.,2)
            if n_elements(center) ne 10 then cen_lin = cen_lin + center.cen_lin_corr(0)/10.
        endif
        cen_pix = nav(3)/10.
        linres = nav(4)
        pixres = nav(5)
        sline  = nav(0)
        spix   = nav(1)
        linang = nav(6)*1.d-9
        pixang = nav(7)*1.d-9

        cen_lin_img = (float(cen_lin) - float(sline))/ float(linres)
        if (nav(160) eq 1) then begin
            c0 = float(nav(161))*1.e-7 * 10.^(nav(162))
            c1 = float(nav(163))*1.e-7 * 10.^(nav(164))
            c2 = float(nav(165))*1.e-7 * 10.^(nav(166))
        endif else begin
            c0 = (float(cen_pix) -float(spix))/ float(pixres)
            c1 = 0.0e0
            c2 = 0.0e0
        endelse
        ny = nav(149)

        spper  = nav(8)*1.d-6   ;spin period in seconds/revolution
        imgdate = nav(147)
        imgtime = nav(148)/1000. ;seconds since start of day 
;       scan information
        scan1  = nav(131)
        sspra1 = nav(135)*1d-6
        sspdc1 = nav(136)*1d-6
        scan2  = nav(137)
        sspra2 = nav(141)*1d-6
        sspdc2 = nav(142)*1d-6
        two21 = 2.d0^21
        spra1 = nav(65)  / two21 ;2.d0^21
        spdc1 = nav(66)  / two21 ;2.d0^21
        spra2 = nav(125) / two21 ;2.d0^21
        spdc2 = nav(126) / two21 ;2.d0^21
        if (nav(146) gt 0) then icheb = 1
        two13=2.d0^13
        chebx = nav(86:96)  /two13 ;2.d0^13
        cheby = nav(97:107) /two13 ;2.d0^13
        chebz = nav(108:118)/two13 ;2.d0^13
        ;Use JMA fit if RMS > 0
        if (nav(52) gt 0) then begin
            spra1 = nav(45) * 1.e-6
            spra2 = nav(47) * 1.e-6
            spdc1 = nav(46) * 1.e-6
            spdc2 = nav(48) * 1.e-6
            roll  = nav(42) * 1.e-6
            pitch = nav(43) * 1.e-6
            yaw   = nav(44) * 1.e-6
        endif
        gra1 = nav(73) /two21   ;2.d0^21
        gra2 = nav(129)/two21   ;2.d0^21

        if not(nav(151) eq nav(152) and $
               nav(151) eq (-9999l)) then begin
            center = {cen_lin_corr:0., $
                      cline_eq: fltarr(4)}
            center.cen_lin_corr = float(nav(151))
            center.cline_eq(0)  = float(nav(152),0,1)
            center.cline_eq(1)  = float(nav(153),0,1)
            center.cline_eq(2)  = float(nav(154),0,1)
            center.cline_eq(3)  = float(nav(155),0,1)
        endif

;print,'LL2IJ'
;print,cen_lin_img,c0,c1,c2
;print,kep
;print,roll,pitch,yaw
    end
    2: begin
;       set up VARIABLES from the B1 structures
        if keyword_set(b1ORBstr) then begin
            ep_date  = b1ORBstr.ep_date
            ep_time  = b1ORBstr.ep_time
            ep_msec   = b1ORBstr.ep_msec
            semima = b1ORBstr.semima
            incl   = b1ORBstr.incl 
            eccen  = b1ORBstr.eccen
            meana  = b1ORBstr.meana
            rasc   = b1ORBstr.rgtasc
            perigee= b1ORBstr.perigee 
            spper  = b1ORBstr.sper
            kep = [semima,eccen,incl, $
                   rasc,perigee,meana]
        endif
        if keyword_set(b1ATTstr) then begin
            roll = b1ATTstr.roll
            pitch= b1ATTstr.pitch
            yaw  = b1ATTstr.yaw
            spra = b1ATTstr.spra
            spdc = b1ATTstr.spdc
        endif
        if keyword_set(b1IMGstr) then begin
            imgdate=  b1IMGstr.frdate  ;= frdate
            imgtime=  b1IMGstr.frtime  ;= frtime
            cen_lin= b1IMGstr.cen_lin  ;= cen_lin
            if keyword_set(center) then begin
                if n_elements(center) ne 10 then cen_lin = cen_lin + center.cen_lin_corr(0)/10.
            endif
            cen_pix= b1IMGstr.cen_pix  ;= cen_pix
            linres=  b1IMGstr.linres   ;= linres
            pixres=  b1IMGstr.pixres   ;= pixres
            sline=   b1IMGstr.sline    ;= sline
            spix =   b1IMGstr.spix     ;= spix
            linang = b1IMGstr.linang   ;= linang
            pixang = b1IMGstr.pixang   ;= pixang
            cen_lin_img = (float(cen_lin) - float(sline))/ float(linres)
            cen_pix_img = (float(cen_pix) - float(spix))/ float(pixres)

            ny = b1imgstr.nlin

        endif
        if keyword_set(b1TIMstr) then begin
            d     = b1TIMstr.d    ;= d
            gt1   = b1TIMstr.gt1  ;= gt1
            gt2   = b1TIMstr.gt2  ;= gt2
            gra1  = b1TIMstr.gra1 ;= gra1
            gra2  = b1TIMstr.gra2 ;= gra2
            spra1 = b1TIMstr.spra1 ;= -999.d0
            spra2 = b1TIMstr.spra2 ;= -999.d0
            spdc1 = b1TIMstr.spdc1 ;= -999.d0
            spdc2 = b1TIMstr.spdc2 ;= -999.d0
            chebx  = b1TIMstr.cheb_x ;= ch_x
            cheby  = b1TIMstr.cheb_y ;= ch_y
            chebz  = b1TIMstr.cheb_z ;= ch_z
            ch_a  = b1TIMstr.cheb_a ;= -999.d0
            if (ch_a(0) eq 1.0d0) then icheb = 1
            ch_b  = b1TIMstr.cheb_b ;= -999.d0
            ch_c  = b1TIMstr.cheb_c ;= -999.d0
        endif
    end
    3: begin
;       set up VARIABLES from the netCDF file
        stop
    end
    0: begin
        print,'No navigation information provided.'
        stop
    end
endcase

;*****************
; Time information
;*****************
year = ep_date/1000 + 1900 * (ep_date/1000 lt 1900)
jday = ep_date mod 1000
ejulday = julday(1,jday,year,0,0,0) + $
          ftime(ep_time) / 24.d0 + $
          ep_msec / 1000.d0 / 3600.d0 / 24.d0
;image time 
iyear = floor(imgdate/1000) + 1900 * (floor(imgdate/1000) lt 1900)
idoy  = imgdate mod 1000
imgjulday = julday(1,idoy,iyear,0,0,0) + $
            imgtime / 3600.d0 / 24.d0
frtime = imgjulday


if keyword_set(debug) then begin
    print,'Nav Info:========================================'
    print,'EPOCH :',ep_date,ep_time,ep_msec
    print,'Kepler:',kep,format='(a,6f)'
    print,'RPY   :',roll,pitch,yaw
    print,'spra  :',spra1,spra2
    print,'spdc  :',spdc1,spdc2
;    print,'GRA   :',gra1,gra2   ;GRA is not used
    print,'center:',cen_lin,cen_pix
    print,'res   :',linres,pixres
    print,'off   :',sline,spix
    print,'JUL   :',ejulday
    print,'IMG   :',imgjulday,imgdate,iyear,idoy,imgtime
    print,'Nav Info:========================================'
    iout=0
endif

;*****************
; Satellite spin axis declination & right ascension
;*****************
spinmiss = 0
if ((spra1 eq spra2 and spra1 eq 0.) and $
    (spdc1 eq spdc2 and spdc1 eq 0.)) then begin
    spinmiss = 1
    ;get satellite position using image time and kep2cart
    dtime = (imgjulday - ejulday) * 24.d0 * 3600.d0
    kep2cart,kep,dtime,xs1,xs2,xs3
    ;get subsat using cart2earth
    time = ejulday + dtime/3600.d0/24.d0
    cart2earth,xs1,xs2,xs3,time,glat,glon,grad
    if (spra1 eq spra2 and spra1 eq 0.) then begin
        ;approximate using the subsatellite longitude
        spra = -glon / rad
    endif
    if (spdc1 eq spdc2 and (spdc1 eq 0. or spdc1 eq (90.))) then begin
        ;approximate using the sub satellite latitude
        spdc = (-90. + glat) / rad
    endif
endif

;if keyword_set(center) then spra = -90. / rad



;omf is constant that relates to the flattening of the Earth
omf2 = (be/ae)*(be/ae)          ;^2

; don't output with too many points
;if (n_elements(inlat) gt 2) then iout = 0

if (iout) then print,'lat/lon'
if (iout) then print,inlat,inlon

if (iout) then print,'RPY',roll,pitch,yaw


;resize the input lat/lon and retain the dimensions to return i/j
if keyword_set(RADIANS) then div = 1.d0 else div = rad
insize = size([inlat])
npoints = n_elements(inlat)

latt = reform([inlat],npoints) / div
lonn = reform([inlon],npoints) / div
jps = dblarr(npoints)
ips = dblarr(npoints)

;nchunk = floor(npoints/chunksize)+1
nchunk = ceil(npoints/float(chunksize))
nps = npoints
for ichunk = 0l,nchunk-1 do begin
chlow = (ichunk  )*chunksize
chhigh= ((ichunk+1)*chunksize-1) < (nps-1)
lat = latt(chlow:chhigh)
lon = lonn(chlow:chhigh)
npoints = chhigh - chlow + 1
;this is a constant that relates to # of radians per line ... but this
;is at a different resolution that linang
amul = 1.0986e-2 / rad

;CALCULATIONS START
;
; estimate the time the point was scanned
phi1  = atan(sin(lat)/(6.61d0-cos(lat))) - pitch/rad
;t_scn1 =    frtime + ((phi1/linang*rad)+cen_lin)*spper/linres
dtime = (imgjulday - ejulday) * 24.d0 * 3600.d0 + $ ;time since epoch at start of image
        (cen_lin - (phi1/linang*rad))*spper/linres
if (iout) then print,'First T_scan:',dtime,(cen_lin - (phi1/linang*rad))*spper/linres
;set loop variables
phiold = phi1
dsline = 10 + phiold*0.
ndsline = total(dsline ne 0)
try=0

iloop = 0

; iterate the i/j coordinates until the change in line is zero
while (ndsline ne 0) do begin
    try=try+1
    
    if (iloop) then print,'=========== TRY:',try
    
    lasttime = dtime
    if (iloop) then print,'DTIME:',reform([dtime])

    ;compute normalized coordinates of the point on Earth in
    ;Earth-centered fixed coordinates
    if (icheb and $
        max(dtime) le (13.d0*3600.d0) and $
        min(dtime) ge 0.d0 and $
        total(abs(chebx)) ne 0.) then begin
;   Use Chebyshev polynomials to determine Cartesian coordinates
        cheb2cart,chebx,dtime,xs1
        cheb2cart,cheby,dtime,xs2
        cheb2cart,chebz,dtime,xs3
    endif else begin
;   Otherwise use Kepler elements
        kep2cart,kep,dtime,xs1,xs2,xs3
    endelse

    if (iloop) then print,'SAT POSITION'
    if (iloop) then print,reform([xs1,xs2,xs3])

    px=xs1 & py=xs2 & pz=xs3
    grad = sqrt(xs1*xs1+xs2*xs2+xs3*xs3)

;   get greenwich hour angle
    time = ejulday + dtime/3600.d0/24.d0

    cart2earth,xs1,xs2,xs3,time,glat,glon,grad
    if (iloop) then print,'SAT POSITION'
    if (iloop) then print,reform([glat,glon,grad])

    if (iloop) then print,'GHA TIME:',(reform(time)-ejulday)*24.d0
    gra = gha(time)

;    if (gra1 ne gra2) then begin
;;        for some reason, interpolating over GRA1/2 doesn't work
;        if (gra1 gt gra2) then gra2 = gra2 + 360.d0
;;        gra = gra1 + (gra2-gra1) * dtime / (13.d0*3600.d0)
;    endif else begin
;        gra = gha(time)
;    endelse

    ;R = position of satellite normalized by eqtorial earth radius
    r = [[xs1], $
         [xs2], $
         [xs3]] / ae
    if (iloop) then print,reform(r)
    ; X = position of earth location in fixed coordinates
    ;convert geographic to geocentric latitude
    philat = atan(omf2 * tan(lat))
    ;normalize longitude with greenwich rgt. asc.
    cartlon = lon + gra
    p = omf2 - 1.d0

    sinphilat=sin(philat)
    cosphilat=cos(philat)

    earthradius = sqrt(1.d0 + p*sinphilat*sinphilat)

    x = rebin(reform(earthradius,npoints,1),npoints,3)* $
        [[cosphilat*cos(cartlon)],$
         [cosphilat*sin(cartlon)],$
         [sinphilat]]

    ;Dotproduct to determine which pixels are visible
    n = npoints
    dot = total(x * r * rebin(reform([1.,1.,(ae/be)*(ae/be)],1,3),n,3),2)

    if (iloop) then print,'X - Earth location'
    if (iloop) then print,reform(x)

   ;compute the pointing vector W = X - R
   ; points from  Satellite to Earth-point
    w = x - r
    if (iloop) then print,'W = X - R'
    if (iloop) then print,reform(w)

;   Calculate the SPRA and SPDC angles ... the new method uses the SPRA
;   for the first and last scan line, instead of interpolating over time
;   again.
    if (spinmiss eq 0) then begin
        spra = spra1 + dtime/3600./13. * (spra2-spra1)
        spdc = spdc1 + dtime/3600./13. * (spdc2-spdc1)
        spra = spra / rad
        spdc = spdc / rad
        if (spra2 eq 0.) then spra = spra1 / rad
        if (spdc2 eq 0.) then spdc = spdc1 / rad
    endif
    if (iloop) then print,'spra,spdc'
    if (iloop) then print,spra*rad,spdc*rad

;   =======
;      S
;   =======
;   Calculate the S transformation matrix
    s=dblarr(3,3,n)
    cdc = cos(spdc)
    s(2,0,*) = cdc*cos(spra)
    s(2,1,*) = cdc*sin(spra)
    s(2,2,*) = sin(spdc)
    PDOTS = PX*S(2,0,*) + PY*S(2,1,*) + PZ*S(2,2,*)
    PSQ = PX*PX + PY*PY + PZ*PZ
    S1DEN = SQRT(PSQ - PDOTS*PDOTS)
    S(0,0,*) = (-PX + PDOTS*S(2,0,*))/S1DEN                                     
    S(0,1,*) = (-PY + PDOTS*S(2,1,*))/S1DEN
    S(0,2,*) = (-PZ + PDOTS*S(2,2,*))/S1DEN
    S(1,0,*)=S(2,1,*)*S(0,2,*)-S(2,2,*)*S(0,1,*)
    S(1,1,*)=S(2,2,*)*S(0,0,*)-S(2,0,*)*S(0,2,*)
    S(1,2,*)=S(2,0,*)*S(0,1,*)-S(2,1,*)*S(0,0,*)
    if (iloop) then print,'S matrix'
    if (iloop) then print,unit(s)

;   =======
;      VC
;   =======
    vc = w*ae
    if (iloop) then print,'VC'
    if (iloop) then print,reform(unit(vc))
;   =======
;      VS
;   =======
;   vs = s#vc
;   replaced multiplication with the following which calculates
;   every point
    vs = dblarr(n,3)
    vs(*,0) = vc(*,0)*s(0,0,*) + vc(*,1)*s(0,1,*) + vc(*,2)*s(0,2,*)
    vs(*,1) = vc(*,0)*s(1,0,*) + vc(*,1)*s(1,1,*) + vc(*,2)*s(1,2,*)
    vs(*,2) = vc(*,0)*s(2,0,*) + vc(*,1)*s(2,1,*) + vc(*,2)*s(2,2,*)
       
    vs1 = reform(vs(*,0))
    vs2 = reform(vs(*,1))
    vs3 = reform(vs(*,2))
    if (iloop) then print,'VS'
    if (iloop) then print,unit(reform(vs))

;   Get satellite viewing angle from the scan/elevation by corecting for
;   the roll/pitch/yaw angles
    useold = 1
    if (useold) then begin
        SIGMA = ATAN(VS2/VS1)
        TANYAW=  TAN(YAW/rad)
        COSYAW=  COS(YAW/rad)
;print,vs1,vs2,vs3
;print,tanyaw,yaw
;stop
        vs12vs22 = VS1*VS1+VS2*VS2

        XI=ATAN(VS3*TANYAW/SQRT(vs12vs22-VS3*VS3*TANYAW*TANYAW))
        PHInew=ATAN(-VS3/(COSYAW*COS(XI)*SQRT(vs12vs22)))-PITCH/rad
        theta = xi + sigma - roll/rad
    endif else begin
        s_= (vs2 + vs3*tan(yaw/rad))/cos(yaw/rad)
        e = -(vs3/cos(yaw/rad)-s_*tan(yaw/rad))
        theta = atan(s_/vs1) - roll/rad
        phinew= atan(e/vs1) - pitch/rad
    endelse
    
    if (iloop) then print,'scan, elev'
    if (iloop) then print,theta,phinew

;   t_scn1 =  frtime + ((phinew/linang*rad)+cen_lin)/linres*spper
;    dtime = (imgjulday - ejulday) * 24.d0 * 3600.d0 + $ ;time since epoch at start of image
;        ((phi1/linang*rad)+cen_lin)*spper/linres

; the following calculation matches the time from ij2latlon, and
; creates a perfect inverse for the function. This has been tested
; with GOE-5 and GOES-6 (UWS) data
    dtime = (imgjulday - ejulday) * 24.d0 * 3600.d0 + $ ;time since epoch at start of image
            (cen_lin - (phinew/linang*rad))/(linres)*spper

;   Check iteration of the line number
    if (iloop) then print,'LOOP'
    dsline = round(((phiold/amul*linres) - (phinew/amul*linres))*10)
    ndsline = total(dsline ne 0)
;stop
    phiold = phinew
    if (iloop) then print,reform(dsline),ndsline
    if (try gt 1000) then begin
        print,'NO CONVERGENCE'
        ndsline = 0.
        ERRORFLAG=1
        goto,failednav
        ;instead of stopping, need to error out.
        stop
    endif

endwhile

;output for debugging
if (iout) then print,'FINISHED--------------------------------'
if (iout) then print,'DTIME:',reform([dtime])
if (iout) then print,'SAT POSITION'
if (iout) then print,reform([xs1,xs2,xs3])
if (iout) then print,'GHA TIME:',(reform(time)-ejulday)*24.d0
if (iout) then print,reform(r)
if (iout) then print,'X - Earth location'
if (iout) then print,reform(x)
if (iout) then print,'W = X - R'
if (iout) then print,reform(w)
if (iout) then print,'spra,spdc'
if (iout) then print,spra*rad,spdc*rad
if (iout) then print,'S matrix'
if (iout) then print,unit(s)
if (iout) then print,'VC'
if (iout) then print,reform(unit(vc))
if (iout) then print,'VS'
if (iout) then print,unit(reform(vs))
if (iout) then print,'LOOP'
if (iout) then print,reform(dsline),ndsline


; CALCULATE I/J IN SATELLITE COORDINATES
;dline  = phinew/linang*rad
;ipoint = cen_pix + theta /pixang*rad - skew_fact*dline
;jpoint = cen_lin - phinew/linang*rad

;convert scan/elev to img resolution i/j
;middle scan (center.cscan) is not as accurate as the raw information
;from what I can tell. So will not use the following to substitute for
;cen_lin_img
;stop
;if keyword_set(center) then cen_lin_img = center.cscan
;print,cen_lin_img(0)
jpoint = cen_lin_img - (phinew/linang*rad)/float(linres)
if (n_elements(c0) gt 0) then cen_pix_img = c0 + c1 * jpoint + c2 * jpoint * jpoint ;- 1.0d0
if keyword_set(center) then begin
    if n_elements(center) ne 10 then begin 
        xtemp = findgen(ny) - ny/2.
        correction = center.cline_eq(0) + $
                      center.cline_eq(1)*xtemp + $
                      center.cline_eq(2)*xtemp*xtemp + $
                      center.cline_eq(3)*xtemp*xtemp*xtemp
        cen_pix_img = (float(cen_pix) - float(spix) + correction)/ float(pixres)

    ;center.cline is stored in image coordinates, so need to
    ;convert jpoint to img coord. to use:
        if (iout) then print,'Center pixel: '+strtrim((float(cen_pix) - float(spix))/ float(pixres),2)+ $
                             ' > '+strtrim(cen_pix_img(jpoint),2)
        cen_pix_img = cen_pix_img(jpoint)
    endif
endif 
ipoint = cen_pix_img + (theta/pixang*rad )/float(pixres)
;convert img resolution into satellite resolution
ipoint = ipoint * pixres + spix
jpoint = jpoint * linres + sline


; IF SAT. SOORDINATES ARE NOT REQUESTED THEN CONVERT TO IMAGE
; COORDINATES VIA:
; (Ii,Ji) = img. coord.
; (Is,Js) = sat. coord.
; Ii = (Is - spix) / pixres
; Ji = (Js - sline)/ linres
;
; NOTE: There is the possibility that some satellites do not report
; sline and spix in full resolution ... that will need to be checked
; and corrected in the read routine.
;

;convert satellite resolution to img resolution, unless
; sat. res. is requested
if not(keyword_set(SATCOOR)) then begin
    ipoint = (ipoint - spix) / float(pixres)
    jpoint = (jpoint - sline) / float(linres)
endif


if (iout) then print,'Scan/elev'
if (iout) then print,theta,phinew
if (iout) then begin
    print,'Factors:',cen_pix_img,pixang,rad,pixres,spix,linang,sline
endif
;assign non-visible pixels (over horizon) to -999
bad = where(dot lt 1.,nbad)
if (nbad gt 0) then begin
    ipoint(bad) = -9999.
    jpoint(bad) = -9999.
endif

jps(chlow:chhigh) = jpoint
ips(chlow:chhigh) = ipoint
endfor
ipoint = ips
jpoint = jps

ipoint = reform([ipoint],insize(1:insize(0)))
jpoint = reform([jpoint],insize(1:insize(0)))

;   get satellite location in Earth coordinates if requested
if (arg_present(GLAT) or $
    arg_present(glon) or $
    arg_present(grad)) then begin
    cart2earth,xs1,xs2,xs3,time, $
               glat,glon,grad
    if (iout) then print,'gat,glon,grad'
    if (iout) then print,glat,glon,grad 
end
ERRORFLAG=0


;if (iout) then stop


return




failednav:
ipoint = ips
jpoint = jps
ipoint(*) = -1
jpoint(*) = -1
ipoint = reform([ipoint],insize(1:insize(0)))
jpoint = reform([jpoint],insize(1:insize(0)))
print,'FAILED NAV'
return



end

;***
