;****I* idl/idl.b1uread
; NAME:
;    b1uread
;
; PURPOSE:
;    To read the B1U file format and return all data in the b1u
;    structure (where each block is a separate part of the
;    structure).
;
; KEYWORD PARAMETERS:
;    file: The b1u filename, if non is provided, then one is selected
;          using dialog_pickfile
;    b1u:  The contents of the b1u file in a structure
;    headers: only read the headers (saves I/O when entire file is not
;             needed) - it returns after CALIBRATION header
;
;
; EXAMPLE:
;
; USES
; * no other IDL scripts
; 
; USED BY
; * b1u_2_grisat.pro
; * b1qc.pro
; * b1read.pro
; 
;
; MODIFICATION HISTORY:
;    $Id: b1uread.pro 22794 2010-09-16 13:05:30Z kknapp $
;
;
; SOURCE
;


pro b1uread,file=file,b1u=b1u,headers=headers, $
            debug=debug, $
            ignore=ignore
;reset

;=================================================================
; DEFINE THE GVAR Header
;
;Define GOES-8 header information
AREAG8  = { B1UAREA,               $
            area_status:0L, version_num:0L, sat_id_num:0L,$
            img_date:0L, img_time:0L, north_bound:0L,$ 
            west_vis_elem:0L, z_coor:0L,num_line:0L,num_elem:0L,$
            bytes_per_pixel:0L, line_res:0L, elem_res:0L,$
            num_chan:0L, num_byte_ln_prefix:0L, proj_num:0L,$
            creation_date:0L,creation_time:0L, sndr_filter_map:0L,$
            img_id_num:0L, id:lonarr(4), comment:bytarr(32),$
            pri_key_calib:0L, pri_key_nav:0L, sec_key_nav:0L,$
            val_code:0L, pdl:lonarr(8), band8:0L, act_img_date:0L,$
            act_img_time:0L, act_start_scan:0L, len_prefix_doc:0L,$
            len_prefix_calib:0L,len_prefix_lev:0L,src_type:bytarr(4),$
            calib_type:bytarr(4),avg_or_sample:0L,poes_signal:0L,$
            poes_up_down:0L,orig_src_type:bytarr(4),reserved:lonarr(7)}
;Navigation structure
IMGR_SIN = { IMGR_SIN, mag_sinu:0L, phase_ang_sinu:0L }
; Imager repeat monomial T.
IMGR_MON = { IMGR_MON, $
             order_appl_sinu:0L, order_mono_sinu:0L,  $
             mag_mono_sinu:0L, phase_ang_sinu:0L, ang_from_epoch:0L}
;  Imager repeat T.
IMGR_REP = { IMGR_REP, $
             exp_mag:0L, exp_time_const:0L, mean_att_ang_const:0L,$
             num_sinu_per_angle:0L, sinusoid:REPLICATE(IMGR_SIN,15), $
             num_mono_sinu:0L, monomial:REPLICATE(IMGR_MON,4)   }
NAVG8  = { NAVstr,                          $
           nav_type:bytarr(4), ID:bytarr(4), IMC_status:0L,        $
           Flip:0L, spare1:0L, ref_long:0L, ref_rad_dist:0L,       $
           ref_lat:0L, ref_orb_yaw:0L, ref_att_roll:0L,            $
           ref_att_pitch:0L, ref_att_yaw:0L, epoch_time:BYTARR(8), $ ; BCD_TIME
           start_time:0L, IMC_corr_roll:0L, IMC_corr_pitch:0L,     $
           IMC_corr_yaw:0L, ref_long_change:LONARR(13),            $
           ref_rad_dist_change:LONARR(11), sine_lat:LONARR(9),     $
           sine_orb_yaw:LONARR(9), solar_rate:0L,exp_start_time:0L,$
           roll_att:IMGR_REP, spare2:LONARR(10), more1:bytarr(4),  $
           gvar1:bytarr(4), pitch_att:IMGR_REP, yaw_att:IMGR_REP,  $
           spare3:LONARR(16), more2:bytarr(4), gvar2:bytarr(4),    $
           roll_misalgn:IMGR_REP, pitch_misalgn:IMGR_REP,          $
           img_date:0L, img_time:0L, instr:0L, spare4:LONARR(9),   $
           ns_cyl:0L,ew_cyl:0L,ns_inc:0L, ew_inc:0L,               $
           more3:bytarr(4),gvar3:bytarr(4),spare5:LONARR(126),     $
           more4:bytarr(4),gvar4:bytarr(4),spare6:LONARR(127)}
ntemp = {b1unav: NAVG8, b1uarea:AREAG8}
; END DEFINE GVAR Header
;=================================================================
; Select file to read
if (not(keyword_set(file))) then begin
    file = dialog_pickfile(path=getenv('ISCCPDIR')+'b1u/', $
                          filter ='*GOE-*')
endif

f = strsplit(file,'.',/extr)
icomp = 0
if (f(n_elements(f)-1) eq 'gz') then icomp = 1


; read in FILinf
FIL = lonarr(7)
if not(file_test(file)) then return
openr,lun,file,compress=icomp,/get_lun
readu,lun,FIL

if keyword_set(debug) then print,'First attempt:',fil

;Check Endian-ness of data
if (FIL(0) ne 1146433858) then begin
;  The Key 1146433858 is the 4-byte representation of the ASCII 
;  string "B1UD"
;               IDL> print,string(1146433858,0,4))
;               B1UD
    close,lun
    free_lun,lun
    openr,lun,file,compress=icomp,/swap_endian,/get_lun
    readu,lun,FIL
    if keyword_set(debug) then print,'Second attempt:',fil
endif
inf = lonarr(3*FIL(6))
readu,lun,inf

if keyword_set(debug) then print,'inf: ',inf

FILinf = [FIL,inf]
b1u = {FIL:FILinf}
NHEAD = FIL(6)
NCHAN = FIL(5)
NELEM = FIL(4)
NSCAN = FIL(3)
NBLP  = FIL(1)

for itype=0,nhead-1 do begin
    type = inf(itype*3)
;print,type
    case type of
        1:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.rev'
            point_lun,lun,inf(itype*3+1)
;print,inf(itype*3+1)
            REVinf = strarr(6)
            b72 = bytarr(72)
            for ii=0,5 do begin
                readu,lun,b72
                REVinf(ii) = string(b72)
            endfor
            b1u = create_struct(['REV'],REVinf,b1u)
        end
        2:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.img'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then begin
                message,'Read Error:'+string(itype)+string(type)+string(here)
                print,here, inf(itype*3+1)
                stop
            endif
            IMGinf = lonarr(24)
            readu,lun,IMGinf
            b1u = create_struct(['IMG'],IMGinf,b1u)
            nbyte=b1u.img(6)
            nbin = 256
            if (b1u.img(14) gt 100) then nbin = b1u.img(14)
        end
        3:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.sat'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
            SATinf = strarr(2+nchan*2)
            b16 = bytarr(16)
            readu,lun,b16
            SATinf(0) = string(b16)
            if keyword_set(debug) then print,'String: ',satinf(0)
            readu,lun,b16
            SATinf(1) = string(b16)
            if keyword_set(debug) then print,'String: ',satinf(1)
            j=2
            b6 = bytarr(6)
            b20= bytarr(20)
            if keyword_set(debug) then print,'nchan: ',nchan
            for ic = 0,nchan-1 do begin
                readu,lun,b6
                SATinf(j++) = string(b6)
                if keyword_set(debug) then print,'string: ',j,string(b6)
            endfor
            for ic = 0,nchan-1 do begin
                readu,lun,b20
                SATinf(j++) = string(b20)
                if keyword_set(debug) then print,'string: ',j,string(b20)
            endfor
            b1u = create_struct(['SAT'],SATinf,b1u)
        end
        4:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.nav'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
            NAVinf = lonarr(200)
            readu,lun,NAVinf
            b1u = create_struct(['NAV'],NAVinf,b1u)
        end
        5:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.cal'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
;            CALinf = lonarr(7681)

            CALinf = lonarr(2*nchan*nbin+1)
            readu,lun,CALinf
            b1u = create_struct(['CAL'],CALinf,b1u)
            if keyword_set(headers) then begin
                close,lun
                free_lun,lun
                return
            endif
        end
        6:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.tgr'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
            TGRinf = lonarr(inf(itype*3+2)/4)
            readu,lun,TGRinf
            b1u = create_struct(['TGR'],TGRinf,b1u)
        end
        7:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.fhdr'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
            FHDR = bytarr(inf(itype*3+2))
            readu,lun,FHDR
            b1u = create_struct(['FHDR'],FHDR,b1u)
        end
        8:begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.pre and image'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
            if (nbyte eq 1) then begin
                img = {prefix:intarr(nblp), $
                       scan:  bytarr(nelem)}
            endif else if (nbyte eq 2) then begin
                img = {prefix:intarr(nblp), $
                       scan:  intarr(nelem)}
            endif else begin
                message,'Incorrect b1u.img(6) value'
            endelse
            imgg = replicate(img,nchan,nscan)
            readu,lun,imgg
            pre = imgg.prefix
            pre = transpose(pre,[1,0,2])
            image = imgg.scan
            image = transpose(image,[1,0,2])           
            b1u = create_struct(['PRE'],PRE,b1u)
            b1u = create_struct(['IMAGE'],IMAGE,b1u)
        end
        9: begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.nav (ntemp)'
            point_lun,-lun,here
            if (here ne inf(itype*3+1) and not(keyword_set(ignore))) then $
              message,'Read Error:'+string(itype)+string(type)+string(here)
            readu,lun,ntemp
            b1u = create_struct(['NAV'],ntemp,b1u)
        end
        10: begin
            if keyword_set(debug) then print,'Reading: '+strtrim(itype,2)+' b1u.qc'
            point_lun,-lun,here
; previous bug in b1_2_b1u.pro caused errors in the calculation of the
; size of the image array. So can ignore this check for a while
; ... until all B1U files have been recreated.
;            if (here ne inf(itype*3+1)) then message,'Read Error:'+string(itype)+string(type)+string(here)
            qc = lonarr(nchan,nscan)
            readu,lun,qc
            b1u=create_struct(['QC'],qc,b1u)
        end
    endcase
endfor
point_lun,-lun,here
close,lun
free_lun,lun
finfo = file_info(file)
;if (here ne finfo.size) then print,'Incomplete read'

end

;***
