function brent,ax,bx,cx,F,tol,xmin
;+
; NAME:
;    BRENT
;
; PURPOSE:
;    Find the minimum of some function F
;
; CATEGORY:
;    From Numerical Recipes
;
; CALLING SEQUENCE:
;
;
;
; INPUTS:
;
;
;
; OPTIONAL INPUTS:
;
;
;
; KEYWORD PARAMETERS:
;
;
;
; OUTPUTS:
;
;
;
; OPTIONAL OUTPUTS:
;
;
;
; COMMON BLOCKS:
;
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;
;
;
; PROCEDURE:
;    Given a function F and a bracketing triplet of abscissas
;    ax, bx, cx (such that bx is between ax and cx and f(bx) is less
;    than both f(ax) and f(cx), this routine isolates the minimum to a
;    fractional precisionof about tol using Brent's method.The
;    abscissa of the minimum is returned as xmin, and the minimum
;    function value is returned as brent, the returned function value.
;
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:
;
;-

itmax = 100
cgold = .3819660d0
zeps = 1.d-10
iter = 0

; a and b must be in ascending order, though the input abcissas need
; not be
a = ax < cx
b = ax > cx

; Initilizations
v = bx
w = v
x = v
; This will be the distance moved on the step before last
e = 0.
fx = call_function(F,x)
fv = fx
fw = fx

; Main program loop
for iter=0,itmax do begin
    xm = 0.5*(a+b)
    tol1 = tol*abs(x)+zeps
    tol2 = 2. * tol1
    ;test for done here
    if (abs(x-xm) le (tol2-0.5*(b-a))) then goto, three
    ;construct a trial parabolic fit
    if (abs(e) gt tol1) then begin
        r = (x-w)*(fx-fv)
        q = (x-v)*(fx-fw)
        p = (x-v)*q - (xiw)*r
        q = 2.*(q-r)
        if (q gt 0) then p = -p
        q = abs(q)
        etemp = e
        e = d
        if (abs(p) ge abs(0.5*q*etemp) or $
            p le (q*(a-x)) or $
            p ge (q*(b-x))) then goto,one
        ;the above conditions determine the
        ;acceptability of the parabolic fit. Here, it is OK
        ;Take the parabolic step
        d = p/q
        u = x+d
        if ((u-a) lt tol2 or $
            (b-u) lt (tol2)) then d = abs(tol1) * (((xm-x) ge 0.) - ((xm-x) lt 0))
        goto,two
    endif
one:
;   We arrive here for a golden section step, which we take into the
;   larger of the two segments.
    if (x ge xm) then e=a-x else e=b-x
;   take the golden step
    d=cgld*e
two:
;   Arrive here with d computed either from parabolic fit or else from
;   golden section
    if (abs(d) ge tol1) then u=x+d else u = x + abs(tol1)*((d gt 0)-(d lt 0))
;   This is the one function call per iteration
    fu = call_function(F,u)
;   and now we have to decide what to do with our function
;   evaluation. Housekeeping follows:
    if (fu le fx) then begin
        if (u ge x) then a=x else b=x
        v = w
        fv = fw
        w = x
        fw = fx
        x = u
        fx = fu
    endif else begin
        if (u lt x) then a=u else b=u
        if (fu le fw or w eq x) then begin
            v = w
            fv = fw
            w = u
            fw = fu
        endif else if (fu le fv or v eq x or v eq w) then begin
            v = u
            fv = fu
        endif
    endelse                       ;done with housekeeping
endfor
print,'Brent exceeded maximum iterations'
three:
; Arrive here ready to exit with best values.
xmin=x
return,fx
end
