;****** b1ucal/b1u_calibration_procedure
; NAME
; b1u_calibration_procedure
; 
; PURPOSE
; To use HIRS as a basis for ISCCP B1U calibration.
; 
; NOTES
; The following is the general flow of calibrating ISCCP B1U data
; using HIRS Pathfinder All Sky data.
;
; 1. Contact Lei Shi and have her update the HIRS Pathfinder All Sky
; files through 2008.
;
; 2. Ensure the files are visible in the $DATA/hirs/
; subdirectories. This is done through symbollic links to her files on
; crystal.
;
; 3. Update ISCCP B1U files. See isccp_b1u_production for more
; information on this process.
; 
; 4. Run match_b1u_poes.pro through the latest year to ensure that ISCCP
; B1U files and HIRS clear sky files can be matched up.
;
; 5. Run the Fortran matchup routine:
;
; 6. ...
; 
; USES
; * match_poes.pro
; 
; USED BY
; The following routines use the calibration results of this process: 
; * idl.b1u_2_grisat
; 
; 
; TODO
; * Change calibration from linear regression of T vs T to (T-T)
;   vs. T. T vs T leads to biases when correlation decreases because
;   linear regresssion tends toward a zero slope. Using the anomaly
;   would remove this bias.
;   
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Fri Oct  5 10:36:07 EDT 2007
; 
; HISTORY
; * 2007Oct05 - Modifying to remport values instead of NaN
;               (particularly useful for SMS)
; * 2009Apr07 - Adding more documentation
; 
; 
;***
;****I* b1ucal/idl.cal_b1u_correct
; NAME
; cal_b1u_correct
; 
; PURPOSE
; To analyze the collected monthly matchups between HIRS and ISCCP B1
; and provide a monthly calibration correction for use in calibrating
; ISCCP B1 observations for climate purposes.
; 
; USAGE
; INPUT
; OUTPUT
; RESULT
; The output is the calibration file to the directory:
; /home/kknapp/idl/isccp/calib/
; 
; EXAMPLE
; USES
; * blank.pro
; * circle.pro [idl/lib]
; * cl.pro
; * cleanplot.pro [idl/dl]
; * correlation.pro
; * flines.pro
; * legend.pro [idl/dl]
; * mean.pro
; * multiplot.pro
; * percentiles.pro
; * png.pro
; * reset.pro
; * stdev.pro

; 
; USED BY
; * b1u_calibration_procedure
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; Fri Oct  5 10:36:07 EDT 2007
; 
; HISTORY
; * 2007Oct05 - Modifying to remport values instead of NaN
;               (particularly useful for SMS)
; 
; SOURCE

pro cal_b1u_correct, $
  irwvp=irwvp,$                 ;Calibrate IRWVP (Default=IRWIN)
  showold=showold, $
  overwrite=overwrite

;***
 
cl


if keyword_set(IRWVP) then begin
    CHANNEL='IRWVP'
    lowcorr0 = 0.75
endif else begin
    CHANNEL='IRWIN'
    lowcorr0 = 0.99
endelse


;SET UP -----------------------------------------------------
verid='v15'                     ;new version
oldver='v14'                    ;version to show when comparing older calibrations
caldir='2010-06'                ;subdirectory of Proportionate

if keyword_set(IRWVP) then begin
    file='/home/kknapp/DATA/isccp/b1ucal/monthly/Proportionate/'+caldir+'/'+CHANNEL+'.cal.edit.sorted.fix'
endif else begin
    file='/home/kknapp/DATA/isccp/b1ucal/monthly/Proportionate/'+caldir+'/'+CHANNEL+'.cal.edit.sorted.fix'
endelse
;------------------------------------------------------------

version='$Id: cal_b1u_correct.pro 22788 2010-09-16 12:50:55Z kknapp $'
calfile='~/idl/isccp/calib/b1u.calibration.correction.'+CHANNEL+'.'+verid

;grab old calibration if requested
if keyword_set(showold) then begin
    oldcal = '~/idl/isccp/calib/b1u.calibration.correction.'+CHANNEL+'.'+oldver
    nl=flines(oldcal)-1
    oldsatmon=strarr(nl)
    oldslope = fltarr(nl)
    oldoff   = fltarr(nl)
    oldsat   = strarr(nl)
    oldtime  = fltarr(nl)
    s=''
    openr,lun,oldcal,/get_lun
    readf,lun,s                 ;version header
    for i=0,nl-1 do begin
        readf,lun,s
        t=strsplit(s,' ',/extr)
        oldsatmon(i) = t(0)
        oldslope(i) = t(1)
        oldoff(i) = t(2)
        ss=strsplit(t(0),'.',/extr)
        oldsat(i) = ss(0)
        oldtime(i) = ss(1) + (ss(2)-0.5)/12.
    endfor
    close,lun
    free_lun,lun
endif



;STEP 1 - READ IN THE IRWIN file

gsat=strarr(30)
date=fltarr(30,200)
final1=fltarr(5)
finalall=fltarr(30,200,5)
nmon=intarr(30)
s=' '
;if (CHANNEL eq 'IRWVP') then file=file +'.oldNOAA'
;file='/home/raid/kknapp/data/isccp/b1ucal/monthly/Proportionate/IRWIN.cal.edit.sorted.fix'
;the '...sorted.fix' file has been 'fixed' using hirs.fix_cal_file on
;the .edit file then sorting the IRWIN.cal.edit.fix --> IRWIN.cal.edit.sorted.fix
nl = flines(file)
stat=fltarr(6,11)
openr,lun,file,/get_lun
isat=0
nsat=0
;initial variables with first line
readf,lun,s
dat=strsplit(s,' ',/extr)
dd=strsplit(dat(0),'.',/extr)
satmon=strjoin(dd(1:3),'.')
stat(isat,*) = dat(2:12)
gsatold=dd(1)
imon = 0
isat = 1
olddd=dd
for i=1,nl-1 do begin
;print,isat,imon,nsat
    readf,lun,s
    dat=strsplit(s,' ',/extr)
    dd=strsplit(dat(0),'.',/extr)
    newsatmon=strjoin(dd(1:3),'.')
    gsatnew=dd(1)
;    print,newsatmon,'=',satmon
    if (newsatmon ne satmon) then begin
;STEP 2 - CHOOSE THE CORRECTION COEFFICIENTS PER EACH SATELLITE-MONTH
        k=where(stat(*,1) gt lowcorr0,nk)
        final1(*) = -90.
        if (nk eq 0) then begin
            print,satmon,' poor correlation',reform(stat(0:isat-1,1))
        endif else begin
            ;if (satmon eq 'GOE-8.1998.08') then stop
            final1(0) = total(stat(k,0))
            final1(1) = total(stat(k,1)*stat(k,0))/final1(0)
            wgt=stat(k,1)*stat(k,0)/total(stat(k,1)*stat(k,0))
            final1(2) = total(stat(k,2)*wgt)
            final1(3) = total(stat(k,3)*wgt)
            final1(4) = total(stat(k,4)*wgt)
        endelse
        ;check if GEO satellite has changed
        date(nsat,imon) = float(olddd(2)) + (float(olddd(3))-0.5)/12.
        finalall(nsat,imon,*) = final1
        gsat(nsat) = gsatold
        if (gsatold ne gsatnew) then begin
            nmon(nsat) = imon
            imon=0
            nsat=nsat+1
        endif else begin
            imon=imon+1
        endelse
        gsatold = gsatnew
        olddd=dd
        isat = 0
        stat(*) = 0.
    endif
    stat(isat,*) = dat(2:12)
    isat=isat+1
    satmon=newsatmon
endfor
close,lun
free_lun,lun
nmon(nsat) = imon
k=where(stat(*,1) gt 0.95,nk)
if (nk eq 0) then begin
;   print,gsat(nsat),' poor correlation',reform(stat(0:isat,1))
endif else begin
    final1(0) = total(stat(k,0))
    final1(1) = total(stat(k,1)*stat(k,0))/final1(0)
    wgt=stat(k,1)*stat(k,0)/total(stat(k,1)*stat(k,0))
    final1(2) = total(stat(k,2)*wgt)
    final1(3) = total(stat(k,3)*wgt)
    final1(4) = total(stat(k,4)*wgt)
;   print,satmon,final1
endelse
;check if GEO satellite has changed
date(nsat,imon) = float(olddd(2)) + (float(olddd(3))-0.5)/12.
finalall(nsat,imon,*) = final1
gsat(nsat) = gsatold

;STEP 3 - CHECK VALUES TEMPORALLY FOR CONSISTENCY
corr = reform(finalall(*,*,1))
slop = reform(finalall(*,*,2))
yint = reform(finalall(*,*,3))
num  = reform(finalall(*,*,0))

;design flag for data to skip in computing final cofficients
flag = corr gt 0.98
for isat=0,nsat do begin
    lowcorr = lowcorr0
    if (isat eq 7) then lowcorr = lowcorr-0.01
; FIRST flag was simple < 0.99 or more than 2 sigmas away
    cor_bar = mean( corr(isat,0:nmon(isat)))
    cor_var = stdev(corr(isat,0:nmon(isat)))
    flag(isat,0:nmon(isat)) =  (corr(isat,0:nmon(isat))         gt  lowcorr  and $
                                corr(isat,0:nmon(isat))-cor_bar gt (-cor_var*2))
    ;previous accidentally flags anomalously high correlations
;    if (total((corr(isat,0:nmon(isat))-cor_bar) gt cor_var*2) gt 0) then stop
; SECOND flag: calculate mean and variance of correlation > 0.98
;              then throw out points outside 2 sigma
;    cor_bar = total(corr(isat,0:nmon(isat))*flag(isat,0:nmon(isat)))/ $
;              total(flag(isat,0:nmon(isat)))
;    cor_var = sqrt((total((corr(isat,0:nmon(isat))-cor_bar)^2)*flag(isat,0:nmon(isat)))/ $
;              (total(flag(isat,0:nmon(isat)))-1.))
;    flag(isat,0:nmon(isat)) = corr(isat,0:nmon(isat)) gt  0.98 and $
;                              abs(corr(isat,0:nmon(isat))-cor_bar) lt cor_var*2
endfor

;Add a few values by hand

if file_test(calfile) and not(keyword_set(overwrite))then begin
    message,'Calibration file already exists: '+calfile
endif

openw,lun,calfile,/get_lun
printf,lun,'# Version: ',version

slopf=slop
yintf=yint
numf = num
;calculate temporally adjusted values.
;this uses a binomial filter which:
;  a) ignores bad points and
;  b) ignores the jump at 2001
for isat=0,nsat do begin

;only process IRWIN or sats with IRWVP
    process = not(keyword_set(IRWVP)) or $
              (keyword_set(IRWVP)  and $
               (gsat(isat) ne 'GMS-2' and $
                gsat(isat) ne 'GOE-7' and $
                gsat(isat) ne 'GOE-6' and $
                gsat(isat) ne 'GOE-2' and $
                gsat(isat) ne 'GOE-1'  $
               ))

    if (process) then begin

    print
    print,'SATELLITE:',isat,' ',gsat(isat)
c = reform(corr(isat,0:nmon(isat)))
cp = percentiles(c,val=[0.05])
    print,'CORR: ',min(c),cp(0),mean(c),median(c),max(c)


;A dateswitch is a temporal boundary across which, the temporal
;averaging should NOT occur.
; For example, at a boundary between when ISCCP calibration is and is
; not available will cause a discontinuity. The averaging should not
; happen across this boundary. Another example is the calibration
; problem noticed in the 2001.75 (Sep/Oct)
; 
;the original loop assumed only one dateswitch could happen, but now
;with more processing, more than one is possible
;
;New loop structure will loop over each period with dateswitch being
;an array that encompasses all possible dates.

    ;all dates will be after         the initial value
    dateswitch = [1970]
    ;dates are placed in chronological order to ensure they are in order
    if (gsat(isat) eq 'GMS-2') then dateswitch = [dateswitch,1984.2]
    if (gsat(isat) eq 'GMS-4') then dateswitch = [dateswitch,1995.1]
    if (gsat(isat) eq 'GOE-5') then dateswitch = [dateswitch,1982.9]
    dateswitch=[dateswitch,2001.75]
    if (gsat(isat) eq 'MSG-1') then dateswitch = [dateswitch,2005.9]

    dateswitch=[dateswitch,2009. + 1/12.+1./265] ;ISCCP Cal ends in Jan. 2009

    ;last value is the absolute maximum -
    ;assumes code won't run after 2100
    dateswitch=[dateswitch,2100.]

    nperiods = n_elements(dateswitch)-1 
;    g=where(date(isat,0:nmon(isat)) ltdateswitch,ng,comp=gcomp,ncomp=ngcomp)
    print,'ISAT:nperiods',isat,nperiods,dateswitch
    for i=0,nperiods-1 do begin
;        if (i eq 1) then begin
;            g=gcomp
;            ng=ngcomp
;        endif
        g = where(date(isat,0:nmon(isat)) ge dateswitch(i) and $
                  date(isat,0:nmon(isat)) lt dateswitch(i+1),ng) 
        if (ng gt 4) then begin

            if keyword_set(IRWVP) then begin
                ;currently, do not smooth IRWVP data
                slopf(isat,g) = slop(isat,g)
                yintf(isat,g) = yint(isat,g)
                slopf(isat,g) = convol(reform(slop(isat,g)*flag(isat,g) + $
                                              (1-flag(isat,g))*(-100.)), $
                                       [1,4,6,4,1],/norm,/edge_trun,invalid=-100.)
                yintf(isat,g) = convol(reform(yint(isat,g)*flag(isat,g) + $
                                              (1-flag(isat,g))*(-100.)), $
                                       [1,4,6,4,1],/norm,/edge_trun,invalid=-100.)
            endif else begin
                slopf(isat,g) = convol(reform(slop(isat,g)*flag(isat,g) + $
                                              (1-flag(isat,g))*(-100.)), $
                                       [1,4,6,4,1],/norm,/edge_trun,invalid=-100.)
                yintf(isat,g) = convol(reform(yint(isat,g)*flag(isat,g) + $
                                              (1-flag(isat,g))*(-100.)), $
                                       [1,4,6,4,1],/norm,/edge_trun,invalid=-100.)
            endelse

            if (gsat(isat) eq 'GOE-1') then begin
                slopf(isat,g) = 0.
                yintf(isat,g) = 0.
            endif
            ;replace NaNs by continuously
            ;smoothing the data until bad data are gone
            ok = where(abs(slopf(isat,g)) lt 10.,nok,comp=bad,ncomp=nbad)
            ntimes = 0
            while (nbad gt 0) do begin
                sl = reform(slopf(isat,g))
                yi = reform(yintf(isat,g))
                if (ntimes gt 40) then begin
                    message,'Problem processing data',/info
                    print,reform(slop(isat,g))
                    print,reform(slopf(isat,g))
                    stop
                endif
                sl(bad) = -100.
                yi(bad) = -100.
                newsl = convol(sl,[1,4,6,4,1],/norm,/edge_trun,invalid=-100.)
                newyi = convol(yi,[1,4,6,4,1],/norm,/edge_trun,invalid=-100.)
                sl(bad) = newsl(bad)
                yi(bad) = newyi(bad)
                ok = where(abs(sl) lt 10.,nok,comp=bad,ncomp=nbad)
                slopf(isat,g) = sl
                yintf(isat,g) = yi
                ntimes++
            endwhile
            
        endif else if (ng gt 0) then begin
            ;this path is not prepared for IRWVP
;            if keyword_set(IRWVP) then stop
            ;This only fixes SMS-1 points
            ;The following produces -NaN when there are too few points
                slopf(isat,g) = total(slop(isat,g)*flag(isat,g))/total(flag(isat,g))
                yintf(isat,g) = total(yint(isat,g)*flag(isat,g))/total(flag(isat,g))
                if (total(flag(isat,g)) lt 1) then begin
                    print,'Replacing NaNs with...'
                    print,reform(slopf(isat,g))
                ;if there are no valid flags, then 
                ;if slop > 0.9, use it,
                    slopf(isat,g) = slop(isat,g)
                    yintf(isat,g) = yint(isat,g)
                ; otherwise report slope=1, off=0.
                    default = where(corr(isat,g) lt 0.9,ndef)
                    if (ndef gt 0) then begin
                        slopf(isat,g(default)) = 1.0e0
                        yintf(isat,g(default)) = 0.0e0
                    endif
                    print,reform(slopf(isat,g))
                ;stop
                endif 
        endif 
    endfor

    ;linearly interpolate between periods of 4 months or more of gaps.
    if (isat eq 13) then begin
;        i1=20
;        i2=26
;        slopf(isat,i1:i2) = slopf(isat,i1) + (slopf(isat,i2)-slopf(isat,i1)) / $
;                                             (date(isat,i2)   -date(isat,i1)) * $
;                                             (date(isat,i1:i2)-date(isat,i1))
;        yintf(isat,i1:i2) = yintf(isat,i1) + (yintf(isat,i2)-yintf(isat,i1)) / $
;                                             (date(isat,i2)   -date(isat,i1)) * $
;                                             (date(isat,i1:i2)-date(isat,i1))
    endif
    if (isat eq 2) then begin
;        stop
;        i1=32
;        i2=35
;        slopf(isat,i1:i2) = slopf(isat,i1) + (slopf(isat,i2)-slopf(isat,i1)) / $
;                                             (date(isat,i2)   -date(isat,i1)) * $
;                                             (date(isat,i1:i2)-date(isat,i1))
;        yintf(isat,i1:i2) = yintf(isat,i1) + (yintf(isat,i2)-yintf(isat,i1)) / $
;                                             (date(isat,i2)   -date(isat,i1)) * $
;                                             (date(isat,i1:i2)-date(isat,i1))        
    endif
    if (isat eq 8) then begin
;        i1=36
;        i2=39
;        slopf(isat,i1:i2) = slopf(isat,i1) + (slopf(isat,i2)-slopf(isat,i1)) / $
;                                             (date(isat,i2)   -date(isat,i1)) * $
;                                             (date(isat,i1:i2)-date(isat,i1))
;        yintf(isat,i1:i2) = yintf(isat,i1) + (yintf(isat,i2)-yintf(isat,i1)) / $
;                                             (date(isat,i2)   -date(isat,i1)) * $
;                                             (date(isat,i1:i2)-date(isat,i1))
    endif
    if (isat eq 16) then begin
;        slopf(isat,nmon(isat)-1:nmon(isat)) = slop(isat,nmon(isat)-1:nmon(isat))
;        yintf(isat,nmon(isat)-1:nmon(isat)) = yint(isat,nmon(isat)-1:nmon(isat))        
    endif
    circle,/fill
    cleanplot,/sil
    blank,255

    if keyword_set(showold) then multiplot,[1,4] else multiplot,[1,3] 

    ;CORRELATION PLOT
    plot,date(isat,0:nmon(isat)),corr(isat,0:nmon(isat)), $
         psym=-6,/yno,title=gsat(isat),chars=1.5, $
         xrange=[date(isat,0)-1,date(isat,nmon(isat))+0.5],xs=1,col=0,min=-50.
    legend,['Correlation'],chars=1.5,textc=0
    oplot,[0,2100],[lowcorr,lowcorr],col=0
    multiplot
    ;SLOPE PLOT
    plot,date(isat,0:nmon(isat)),slop(isat,0:nmon(isat)), $
         psym=-6,/yno,title=gsat(isat),chars=1.5, $
         xrange=[date(isat,0)-1,date(isat,nmon(isat))+0.5],xs=1,col=0,min=-50.

    nqw=0
    if keyword_set(showold) then begin
        qw = wherE(gsat(isat) eq oldsat,nqw)
        if (nqw gt 1) then begin
            sqw = sort(oldtime(qw))
            oplot,oldtime(qw(sqw)),oldslope(qw(sqw)),thick=3,col=180
            
        endif
    endif

    oplot,date(isat,0:nmon(isat)),slop(isat,0:nmon(isat))*flag(isat,0:nmon(isat)), $
          psym=8,syms=1.5,col=0
    oplot,date(isat,0:nmon(isat)),slopf(isat,0:nmon(isat)),thick=3,col=100
    legend,['Slope'],chars=1.5,textc=0
    multiplot

    ;OFFSET PLOT
    plot,date(isat,0:nmon(isat)),yint(isat,0:nmon(isat)), $
         psym=-6,/yno,title=gsat(isat),chars=1.5, $
         xrange=[date(isat,0)-1,date(isat,nmon(isat))+0.5],xs=1,col=0,min=-50.
    oplot,date(isat,0:nmon(isat)),yint(isat,0:nmon(isat))- $
          (100*(flag(isat,0:nmon(isat)) eq 0)), $
          psym=8,syms=1.5,col=0

    if keyword_set(showold) and nqw gt 1 then oplot,oldtime(qw(sqw)),oldoff(qw(sqw)),thick=3,col=180
    oplot,date(isat,0:nmon(isat)),yintf(isat,0:nmon(isat)),thick=3,col=100
    legend,['Offset'],chars=1.5,textc=0

    if keyword_set(showold) then begin
        multiplot
        temp1 = 210.
        temp2 = 300.
        newt1 = reform(temp1 * slopf(isat,0:nmon(isat)) + yintf(isat,0:nmon(isat)))
        newt2 = reform(temp2 * slopf(isat,0:nmon(isat)) + yintf(isat,0:nmon(isat)))
        oldt1 = temp1 * oldslope(qw(sqw))        + oldoff(qw(sqw))
        oldt2 = temp2 * oldslope(qw(sqw))        + oldoff(qw(sqw))
        diff1 = newt1-oldt1
        diff2 = newt2-oldt2
        lowt = (min(diff1)<min(diff2))<(-1)
        hight= (max(diff1)>max(diff2))>(1)
        plot,date(isat,0:nmon(isat)),yint(isat,0:nmon(isat)), $
             psym=-6,/yno,/nodata,chars=1.5, $
             xrange=[date(isat,0)-1,date(isat,nmon(isat))+0.5],xs=1,col=0,min=-50., $
             yrange=[lowt,hight],ytitle='dTemp'
        oplot,oldtime(qw(sqw)),diff1,thick=3,col=0
        oplot,oldtime(qw(sqw)),diff2,thick=3,col=100
        w1 = where(abs(diff1) gt 0.2,nw1)
        w2 = where(abs(diff2) gt 0.2,nw2)
        if (nw1 gt 0) then begin
            circle
            plots,oldtime(qw(sqw(w1))),diff1(w1),psym=8,col=0
        endif
        if (nw2 gt 0) then begin
            circle
            plots,oldtime(qw(sqw(w2))),diff2(w2),psym=8,col=0
        endif

        oplot,[0,3000],[0,0],col=0
        legend,strtrim(fix([temp1,temp2]),2)+' K',thick=[3,3],col=[0,100],textc=0,lines=0
        legend,'T(new) - T(old)',/right,textc=0
    endif

    multiplot,/reset
    png,'ISCCP-Cal-Corr-'+CHANNEL+'-'+gsat(isat)
;stop
    for i=0,nmon(isat) do begin
        year=floor(date(isat,i))
        mon = round((date(isat,i) mod 1) * 12 + 0.5)
        printf,lun,gsat(isat),'.',strtrim(year,2),'.', $
              string(mon,form='(i2.2)'), $
              ' ',string(slopf(isat,i),format='(f10.6)'), $
               ' ',string(yintf(isat,i),format='(f10.3)'), $
               ' ',string(num(isat,i),format='(f10.0)'), $
               ' ',string(flag(isat,i),format='(i2)')
    endfor
endif 
endfor  
close,lun
free_lun,lun



stop
for isat=0,nsat do begin
    window,isat,xs=600,ys=800
    cleanplot,/sil
    multiplot,[1,3]
    cor_bar = mean( corr(isat,0:nmon(isat)))
    cor_var = stdev(corr(isat,0:nmon(isat)))
    for ist=1,3 do begin
    ;print,gsat(isat),total(abs(corr(isat,0:nmon(isat))-cor_bar) gt cor_var*3)
        plot,date(isat,0:nmon(isat)) mod 10000,finalall(isat,0:nmon(isat),ist), $
             /yno,psym=6,title=gsat(isat),chars=2
        oplot,date(isat,0:nmon(isat)) mod 10000,finalall(isat,0:nmon(isat),ist)+ $
              (-100*(abs(finalall(isat,0:nmon(isat),1)-cor_bar) gt cor_var*2 or $
                     finalall(isat,0:nmon(isat),1) lt 0.99)),psym=1
        multiplot
    endfor
endfor


for isat=0,nsat do begin
endfor


stop

end
