;****I* idl/idl.cart2earth
;+
; NAME:
;    idl.cart2earth
;
; PURPOSE:
;    Convert cartesian coordinates at a specifc time to Earth-centered
;    coordinates of latitude, longitude and distance from center.
;
; CATEGORY:
;    Navigation
;
; INPUTS:
;    X, Y, Z - Cartesian coordinates X, Y, Z in km
;    julday  - Julian date from IDL julday algorithm
;
; KEYWORD PARAMETERS:
;    RADIANS - Optionally output lat/lon in radians
;    TWOPI   - Optionally output longitude in range [0, 360.]
;
; OUTPUTS:
;    lat,lon - Sub-satellite latitude and longitude (degrees)
;              lat = [-90,90]
;              lon = [-180,180]
;    radius  - distance from center of Earth (km)
;
; USES
; * gha.pro
; * const.pro
;
; USED BY
; * b1read_aes_g7.pro
; * navcheck.pro
; * b1u_ij2ll.pro
; * b1u_ll2ij.pro
;
; MODIFICATION HISTORY:
;    $Id: cart2earth.pro 22776 2010-09-16 12:34:07Z kknapp $
;-
; SOURCE

pro cart2earth,x,y,z,julday,lat,lon,radius, $
               RADIANS=RADIANS, $
               TWOPI=TWOPI

;get Greenwich hour angle
angle = gha(julday)

@const.pro

radius = sqrt(x*x + y*y + z*z)
declination = asin(z / radius)
rasc = atan2(y,x)

;convert declination (which is geocentric latitude) to geographic
;latitude
lat = atan(be*be * tan(declination) / (ae*ae))

lon = (rasc - angle) mod (2.d0*!dpi)
lon = lon + 2.0 * !dpi * (lon lt (-!dpi))
if keyword_set(TWOPI) then lon = lon + 2.0 * !dpi * (lon lt 0.)

if not(keyword_set(RADIANS)) then begin
    lat = lat * 180.d0 / !dpi
    lon = lon * 180.d0 / !dpi
endif

end
;***
