;****I* idl/idl.cart2kep
;+
; NAME:
;    idl.cart2kep
;
; PURPOSE:
;    Convert a position vector with corresponding velocity to
;    keplerian elements.
;
; CATEGORY:
;    Navigation
;
; INPUTS:
;    Input position is distance from center of Earth in km
;    Input velocity is km/hour
;
; KEYWORD PARAMETERS:
;    None
;
; OUTPUTS:
;    Output Keplerian elements is an array: DBLARR(6)
;    kep(0) = semi-major axis (km)
;    kep(1) = orbit eccentricity
;    kep(2) = Orbital inclination (degrees)
;    kep(3) = right ascension of the ascending node (degrees)
;    kep(4) = argument of perigee (degrees)
;    kep(5) = mean anomaly (degrees)
;
; PROCEDURE:
;    This routine is based on the equations found in the Earth
;    Location User's Guide (Revision 1, March 1998) NOAA/NESDIS
;    document DRL 504-11. It has been tested in conjunction with
;    kep2cart.pro.
;
;
; MODIFICATION HISTORY:
;    $Id: cart2kep.pro 22776 2010-09-16 12:34:07Z kknapp $
;    
; USES
; * atan2.pro
; 
; USED BY
; * navcheck.pro
; 
;-
; SOURCE

pro cart2kep,pos1,vel1,kep

rad = 180.d / !dpi

pos = pos1 * 1.d3               ;convert to meters
vel = vel1 * 1.d3 / 3600.d0     ;convert to m/s

px = pos(0)
py = pos(1)
pz = pos(2)
vx = vel(0)
vy = vel(1)
vz = vel(2)

; H is the angular momentum vector
hvec = crossp(pos,vel)
pvec = pos
vvec = vel

hx = hvec(0)
hy = hvec(1)
hz = hvec(2)

;mu = Earth's gravitational constant
;mu = G * Me
mu = 3.9860044d14 ;m^3/s^2

PdotV = total(pos*vel)

;calculate the magnitudes of the vectors
H = sqrt(total(hvec^2))
R = sqrt(total(pos^2))
V = sqrt(total(vel^2))

if ((hx - (py*vz - pz*vy)) ne 0.) then stop
if ((hy + (px*vz - pz*vx)) ne 0.) then stop
if ((hz - (px*vy - py*vx)) ne 0.) then stop

;Right ascending node
RANODE = atan2(hx,-hy)
RANODE = RANODE + 2.d0*!dpi * (RANODE lt 0.d0)
;psis
psis = (vz*R - pz*PDOTV/R)/H

;Ls
Ls = pz/R

;inclination
INCL = asin(sqrt(Ls^2 + psis^2))

U = atan2(Ls,psis)

;semi-major axis
SEMIMA = R / (2.d0 - (R*v^2)/mu)
;eccentricity
p = ((R*V)^2 - PDOTV^2)/mu
ECCEN = sqrt(1.d0 - p/SEMIMA)

;true anomaly
TA = atan2(pdotv*sqrt(p/mu),p-r)

;perigee
PERIGEE = u - TA
PERIGEE = PERIGEE + 2.d0*!dpi * (PERIGEE lt 0.d0)

arg = sqrt((1d0-eccen)/(1d0+eccen))
;eccentric anomaly
EA = 2d0 * atan(arg*tan(TA/2.d0))
EA = EA + 2.d0*!dpi * (EA lt 0.d0)

;Mean Anomaly
MEANA = EA - ECCEN*sin(EA)
MEANA = MEANA + 2.d0*!dpi * (MEANA lt 0.d0)

KEP = [SEMIMA/ 1d3, ECCEN, INCL*rad, RANODE*rad, PERIGEE*rad, MEANA*rad]
return
print,'Original'
print,kep
; ========================================

pvec = pos1 * 1.d3               ;convert to meters
vvec = vel1 * 1.d3 / 3600.d0     ;convert to m/s

px = pvec(0)
py = pvec(1)
pz = pvec(2)
vx = vvec(0)
vy = vvec(1)
vz = vvec(2)

; H is the angular momentum vector
hvec = crossp(pvec,vvec)

hx = hvec(0)
hy = hvec(1)
hz = hvec(2)

;mu = Earth's gravitational constant
;mu = G * Me
mu = 3.9860044d14 ;m^3/s^2

PdotV = total(pvec*vvec)

;calculate the magnitudes of the vectors
H = sqrt(total(hvec^2))
R = sqrt(total(pvec^2))
V = sqrt(total(vvec^2))

rgtasc = atan2(hx,-hy)

incl = atan(sqrt(hx^2+Hy^2)/hz)

semima = r / (2.d0 - (r*v^2)/mu)

q = ((r*v)^2 - pdotv^2)/mu

eccen = sqrt(1.d0-q/semima)

Ls = pz / r

if (incl eq 0) then begin
    if   (ls gt 0.0d0) then u = 90.d0 else $
      if (ls lt 0.0d0) then u = 270.d0 else $
                            u = 0.d0
    u = u * !dpi / 180.d0
endif else begin
    u = asin(ls/sin(incl))
endelse

if ((q-r) eq 0.d0) then ta = u else ta = atan2(pdotv*sqrt(q/mu),q-r)

if (eccen eq 0.d0) then perigee = 0. else perigee = u - ta

arg = sqrt((1.d0-eccen)/(1.d0+eccen))

EA = 2.d0 * atan(arg * tan(ta / 2.d0))

meana = ea - eccen * sin(ea)
print,'New'
print,semima/1.d3,incl*rad,eccen,meana*rad,ranode*rad,perigee*rad

stop

end

;****
