;****I* idl/idl.gha
;+
; NAME:
;    idl.gha
;
; PURPOSE:
;    To calculate the greenwich hour angle.
;
; CATEGORY:
;    Navigation
;
; INPUTS:
;    Julian - The julian date (as returned by the julday IDL routine)
;
; KEYWORD PARAMETERS:
;    DEGREES - Returns angle in degrees (instead of radians).
;
;
; OUTPUTS:
;    Returns the Greenwich hour angle for the given time in radians
;    (optionally in degrees).
;
;
; PROCEDURE:
;    This routine is used to convert the satellite position in
;    inertial coordinates to earth coordinates (e.g., longitude). The
;    initial values are taken from Kidder and Vonder Haar and from
;    some old NOAA GOES navigation code:
;
;    Kidder and Vonder Haar values:
;       100.38641 deg at 0000UTC on 1 Jan 1990
;    From NOAA
;       
;    Other values can be added by including them in the DATE array and
;    the corresponding hour angle in the GHA0 array
;
; MODIFICATION HISTORY:
;    $Id: gha.pro 22776 2010-09-16 12:34:07Z kknapp $
;    
; USES
; Nothing (outside IDL internal routine)
; 
; USED BY
; * cart2earth.pro
; * kep2cart.pro
; * navcheck.pro
; * b1read_csu_met3.pro
; * b1read_eum_met.pro
; * b1read_eum_msg.pro
; * b1read_jma_gms.pro
; * b1read_jma_mts.pro
; * b1u_ij2ll.pro
; * b1u_ll2ij.pro
; 
; SOURCE
;-

function gha,julian, $
             DEGREES=DEGREES, $
             debug=debug

if keyword_set(debug) then iout=1 else iout = 0
;Rotation of earth
rot = 7.292115922d-5            ;rad/sec
;Conversion of degrees to radians
dtor = !dpi / 180.d0

;Set up for intialization points
np = 1
DATE = dblarr(np)
GHA0 = dblarr(np)
;Kidder and Vonder Haar, 1995
DATE(0) = julday(1,1,1990,0,0,0)
GHA0(0) = 100.38641d0 * dtor

; FIND CLOSEST INITIALIZATION POINT
;  If another point is added, will need to a search for the nearest
;  point in time.
INear = 0
if (iout) then print,'ang0',gha0(inear)
if (iout) then print,'julian',julian
if (iout) then print,'dtime ',julian-date(inear)

gha = gha0(inear) + (julian-date(inear)) * rot * 86400.d0
gha = gha mod (2.0d0 * !dpi)
gha = gha + (2.0d0*!dpi) * (gha lt 0)
if keyword_set(DEGREES) then gha = gha / dtor
return,gha
end
;***
