;****I* idl/idl.i2xy
;+
; NAME:
;      i2xy
;
; PURPOSE:
;         Convert an array index to an N dimensional array index.
;
; CALLING SEQUENCE:
;         INDEX = i2xy(I,size(ARRAY))
;
; INPUTS:
;         INDEX0 : Long/Int/Byte
;         SIZEOFARRAY - The result of the call SIZE(ARRAY) where the
;                       array. This passes just the information on the
;                       array size, which reduces unnecessary I/O if
;                       the entire array were passed.
;
; EXAMPLE:
;       IDL> f = randomu(1,10,10) 
;       IDL> help,f 
;       F               FLOAT     = Array[10, 10]
;       IDL> i = where(f gt 0.9) 
;       IDL> print,i 
;           4          19          31          37          38          39
;          45          51          62          68
;       IDL> i2 = i2xy(i,size(f)) 
;       IDL> help,i2 
;       I2              LONG      = Array[2, 10]
;       IDL> print,i2 
;           4           0
;           9           1
;           1           3
;           ... (and so on)
;       IDL> print,f(i2(0,0),i2(1,0)) 
;             0.930436
;       IDL> print,f(i2(0,1),i2(1,1)) 
;             0.909208
; USES
; * cmproduct.pro
; 
; USED BY
; * b1u_2_grisat.pro
; 
; *Many others
; (a very useful and widely used tool in the Knapp library)
; 
; MODIFICATION HISTORY
;        Written by Kenneth Knapp - CIRA
;                   visiting NOAA/NESDIS/ORA
;                   now at NOAA/NESDIS/NCDC
;        08-12-2003 - Documentation header added
;        06-16-2010 - Chenged Docs to Robodoc header
;
; SOURCE

function i2xy,index0,sizeofarray

index = index0
s=sizeofarray
n = s(0)
sz = s(1:n)
ind = sz
ind(*) = 0
nel = n_elements(index0)
;ind = rebin(reform(ind,n,1),n,nel)
if (max(index0) ge s(n+2)) then return,-1

;this loop reduces the number of times cmproduct is called ... since
;it is not efficient for small numbers when called repeatedly

prodsize = sz
for i=1,n-1 do begin
    prodsize(i) = cmproduct(sz(0:i))
endfor

;timer,/start
;for j = 0l,nel-1 do begin
;    temp = index(j)
;    for i=n-2,0,-1 do begin
;;        nn = cmproduct(sz(0:i))
;;        ind(i+1,j) = floor(temp / prodsize(i))
;        ind(i+1,j) = (temp / prodsize(i))
;        temp = temp - ind(i+1,j) * prodsize(i)
;    endfor
;ind(0,j) = temp
;endfor
;timer,/stop,/print

;This was the initial replacement 
;ind1 = replicate(-1,n,nel)
;t = index
;for i=n-2,0,-1 do begin
;    ind1(i+1,*) = (t / prodsize(i))
;    t = t - ind1(i+1,*) * prodsize(i)
;endfor
;ind1(0,*) = t
;timer,/stop,/print


;This implementation is faster ... except the result needs to be
;transposed ... which makes it roughly the same speed.
; MEANING: Set up matrices to process (*,i) instead of (i,*)
ind2 = replicate(-1,nel,n)
t = index
for i=n-2,0,-1 do begin
    ind2(*,i+1) = (t / prodsize(i))
    t = t - ind2(*,i+1) * prodsize(i)
endfor
ind2(*,0) = t
ind=transpose(ind2)


;stop
;if (nel eq 1) then ind = reform(ind,1,n)
return,ind
end

;***
