;****I* idl/idl.kep2cart
;+
; NAME:
;    kep2cart
;
; PURPOSE:
;    To locate a satellite in its orbit around a mass (e.g., Earth,
;    Sun) using the Keplerian elements and the time passed since
;    Epoch. The returned position is in the pweudo-inertial reference
;    frame where x points from center of Earth to the Sun at vernal
;    equinox, the z coordinate points along the Earth's spin axis
;    northward and the y coordinate completes the right handed
;    coordinate system.
;
; CATEGORY:
;    Navigation
;
; INPUTS:
;    kep:    The keplerian element array
;            kep(0) = semi-major axis (km)
;            kep(1) = orbit eccentricity
;            kep(2) = Orbital inclination (degrees)
;            kep(3) = right ascension of the ascending node (degrees)
;            kep(4) = argument of perigee (degrees)
;            kep(5) = mean anomaly (degrees)
;    intime: The time elapsed since epoch (sec) (can be an array)
;
; OPTIONAL INPUTS:
;    sattim: The time in Julian of the position (used to
;            calculate sub-satellite positions.
;
; KEYWORD PARAMETERS:
;    NOPERTURB - Set this flag to optionally not calculate the orbital
;                perturbations due to an oblate Earth (useful when not
;                calculating position of point orbitting Earth).
;
;
; OUTPUTS:
;    x,y,z - [same dimensions of intime] Array or scalar values of the
;            satellite position.
;
; OPTIONAL OUTPUTS:
;    glon - [same dimensions of intime] Array or scalar values of
;           sub-satellite longitude [only calculated if SATTIM is set]
;    glat - [same dimensions of intime] Array or scalar values of
;           sub-satellite latitude [only calculated if SATTIM is set]
;
; USES
; * atan2.pro
; * const.pro
; * gha.pro
; 
; USED BY
; * navcheck.pro
; * plot_tle.pro
; * b1read_uws_g56.pro
; * b1u_2_ij2ll.pro
; * b1u_2_ll2ij.pro
; 
; MODIFICATION HISTORY:
;    $Id: kep2cart.pro 22777 2010-09-16 12:34:52Z kknapp $
;-
; SOURCE 

pro kep2cart,kep, $
             intime, $          ;time since epoch in seconds
             xfin,yfin,zfin,$            ;position in inertial coordinates
             SATTIM=SATTIM,$    ;julian time used to calculate GRA (for glat,glon)
             GLON=GLON, $       ;return the sub satellite latitude  (deg)
             GLAT=GLAT, $       ;return the sub satellite longitude (deg)
             GRAD=GRAD, $       ;return the sub satellite longitude (deg)
             NOPERTURB=NOPERTURB,$ ;don't include the perturbations
             TLESTR=TLESTR
if keyword_set(TLESTR) then begin
    semima = (7.537122734d13 / tlestr.motion / tlestr.motion)^(1./3.)
    incl   = tlestr.incl
    eccen  = tlestr.eccn
    meana  = tlestr.mnan
    rasc   = tlestr.rasc
    perigee= tlestr.prge
endif else begin
;assign parameters
    semima  = kep(0)
    eccen   = kep(1)
    incl    = kep(2)
    rasc    = kep(3)
    perigee = kep(4)
    meana   = kep(5)
endelse
@const.pro

; Number of input points
npoints = n_elements(intime)
stime = size(intime)
time = intime
if (npoints gt 1) then time = reform(intime,npoints)

;Calculate changes in the orbit
;using orbit perturbations 
;from Kidder & VonderHaar, p25
    ;mean motion constant
a = semima                      ;km
a_m=a*1.d3                      ;meters
n = sqrt(gme / (a_m*a_m*a_m))    ;s^-1
e = eccen
i = incl / rad

sini = sin(i)
sini2=sini*sini

dmdt = n  * (1.d0 + $
             1.5d0*j2*(ree/a)*(ree/a)*sqrt((1.d0-e*e)^(-3))*$
             (1.d0 - 1.5d0*(sini2)))

dradt = -dmdt * (1.5d0*j2*(ree/a)*(ree/a)*cos(i)/(1.d0-e*e)^2)



dwdt  =  dmdt * (1.5d0*j2*(ree/a)*(ree/a)*(2.d0-2.5d0*(sini2))/(1.d0-e*e)^2)

nodper = 2.d0*!dpi/(dmdt+dwdt)

if keyword_set(NOPERTURB) then begin
    dmdt = n
    dradt = 0.d0
    dwdt = 0.d0
endif 


;mean motion constant (n bar)
ammc = dmdt

; mean anomaly
anom = time * ammc + meana / rad

; solve for eccentric anomaly (e)
ecan1 = ammc * time
for i=0,4 do begin
    ecan2 = anom + eccen*sin(ecan1)
    ecan1=ecan2
endfor
ecanom = ecan1

;true anomaly
cosecanom = cos(ecanom)
tranom = asin((cosecanom - eccen) / (1.d0 - eccen*cosecanom))
ang=ecanom
n = floor((ang+!pi)/!pi)
m = -1 + 2*((n mod 2) eq 0)
tranom2 = !pi*n+m*tranom-!pi/2.
tranom=tranom2

;calculate radius (r)
radius = semima * (1.d0 - eccen*eccen) / (1.d0 + eccen*cos(tranom))

;locate satellite in orbital plane
x = radius * cos(tranom)
y = radius * sin(tranom)
z = tranom * 0.d0

;rotate axis about argument of perigee
sw = sin(perigee/rad + dwdt*time)
cw = cos(perigee/rad + dwdt*time)
xp = x*cw - y*sw
yp = x*sw + y*cw
zp = z

;rotate axis through inclination
si = sin(incl/rad)
ci = cos(incl/rad)
xpp = xp
ypp = yp*ci - zp*si
zfin = yp*si + zp*ci

;rotate axis through right ascension
so = sin(rasc/rad + dradt*time)
co = cos(rasc/rad + dradt*time)
xfin = xpp*co - ypp*so
yfin = xpp*so + ypp*co
;zfin = zpp


if keyword_set(sattim) then begin 

    sat_ra = sqrt(xfin*xfin + yfin*yfin + zfin*zfin)
    sat_de = asin(zfin / sat_ra)
    sat_rt = atan2(yfin,xfin)

;convert satellite declination (geocentric latitude) to
;geographic latitude
lat = atan(be*be*tan(sat_de)/(ae*ae))


;convert satellite right ascension to longitude by
;subtracting greenwich right ascension
    sat_de = asin(zfin / sat_ra)
    glat = atan(be*be*tan(sat_de)/(ae*ae)) * rad
    gr_rasc = gha(sattim)

    sat_rt = atan2(yfin,xfin)
    lon = (sat_rt - gr_rasc) mod (2.d0*!dpi)
    glon = (lon + 2.d0*!dpi*(lon lt 0.))*rad
    grad = sqrt(xfin*xfin + yfin*yfin + zfin*zfin)
endif

;stop

;set x,y,z to the same dimensions as intime
;x = xfin
;y = yfin
;z = zfin
end
;***
