module kmod_geometry
!****h* /kmod_geometry
! NAME
! kmod_geometry
! SYNOPSIS
! Useful geometrical routines
! USES
! * kmod_constants
! * kmod_math
! * kmod_utilities
! * kmod_time
!***
  use kmod_constants
  use kmod_math
  use kmod_utilities
  use kmod_time

  implicit none

  INTERFACE azimuth
     MODULE PROCEDURE azimuth_scalar
     MODULE PROCEDURE azimuth_rank1
     MODULE PROCEDURE azimuth_rank2
  END INTERFACE
  INTERFACE cart2earth
     MODULE PROCEDURE cart2earth_rank1
     MODULE PROCEDURE cart2earth_scalar
  END INTERFACE
  INTERFACE conv_lat_geolat
     MODULE PROCEDURE conv_lat_geolat_scalar
     MODULE PROCEDURE conv_lat_geolat_rank1
     MODULE PROCEDURE conv_lat_geolat_rank2
  END INTERFACE
  INTERFACE earth2cart
     MODULE PROCEDURE earth2cart_rank1
     MODULE PROCEDURE earth2cart_rank2
     MODULE PROCEDURE earth2cart_scalar
  END INTERFACE earth2cart
  INTERFACE earth_radius
     MODULE PROCEDURE earth_radius_rank1
     MODULE PROCEDURE earth_radius_rank2
     MODULE PROCEDURE earth_radius_scalar
  END INTERFACE earth_radius
  INTERFACE gha
     MODULE PROCEDURE gha_rank1
     MODULE PROCEDURE gha_scalar
  END INTERFACE
  INTERFACE kep2cart
     MODULE PROCEDURE kep2cart_rank1
     MODULE PROCEDURE kep2cart_scalar
  END INTERFACE
  INTERFACE solar_angle
     MODULE PROCEDURE solar_angle_scalar
     MODULE PROCEDURE solar_angle_lr1_tsc
     MODULE PROCEDURE solar_angle_rank1
     MODULE PROCEDURE solar_angle_scalar_lnhdr
     MODULE PROCEDURE solar_angle_rank1_lnhdr
     MODULE PROCEDURE solar_angle_rank2_lnhdr
  END INTERFACE
  INTERFACE viewzenith
     MODULE PROCEDURE viewzenith_scalar
     MODULE PROCEDURE viewzenith_rank1
     MODULE PROCEDURE viewzenith_rank2
  END INTERFACE
  contains
!#######################################################################
!                                                          BEGIN AZIMUTH
!****f* kmod_geometry/azimuth
! NAME
! azimuth
! SYNOPSIS
! azimuth_angle = azimuth(lat, lon, slat, slon)
! FUNCTION
! Calculate azimuth to a location(s) defined by (lat,lon) on 
! the Earth from one point: slat, slon.
! INPUTS
! lat - r*4 - latitude  (scalar or arrays rank 1 or 2)
! lon - r*4 - longitude  (scalar or arrays rank 1 or 2.)
! slat - r*4 - latitude of fixed point -  (scalar)
! slon - r*4 - latitude of fixed point -  (scalar)
! 
! OUTPUT
! azimuth_angle - 0-360 degrees (clockwise from North) (scalar or arrays rank 1 or 2)
!
! NOTES
! All variables are passed in units of degrees.
! USES
! within
! 
!***
  function azimuth_rank2 (lat, lon, slat, slon ) result (output)
    !external variables
    real(kind=4), dimension(:,:),intent(in) :: lat !degrees
    real(kind=4), dimension(:,:),intent(in) :: lon !deg
    real(kind=4), intent(in) :: slat               !deg
    real(kind=4), intent(in) :: slon               !deg
    real(kind=4), dimension(size(lat,1),size(lat,2)) :: output !deg
    !internal variables
    real(kind=4), dimension(:),allocatable :: latt, lonn, azi
    integer :: n, nx, ny

    n = size(lat)
    nx = size(lat,1)
    ny = size(lat,2)
    allocate (latt(n),lonn(n),azi(n))
    latt = reshape(lat,(/n/))
    lonn = reshape(lon,(/n/))
    azi = azimuth(latt,lonn,slat,slon)
    output = reshape(azi,(/nx,ny/))
    deallocate (latt, lonn, azi)
  end function azimuth_rank2

  function azimuth_rank1 (lat, lon, slat, slon ) result (output)
    !external variables
    real(kind=4), dimension(:),intent(in) :: lat !degrees
    real(kind=4), dimension(:),intent(in) :: lon !deg
    real(kind=4), intent(in) :: slat             !deg
    real(kind=4), intent(in) :: slon             !deg
    real(kind=4), dimension(size(lat)) :: output !deg
    !internal variables
    real(kind=4), dimension(size(lat)) :: colat1, colat2, dlon, gc

    colat1 = pi/2. - lat/rad
    colat2 = pi/2. - slat/rad

    dlon = slon - lon
    where (dlon > 180.)
       dlon = dlon - 360.
    end where
    where (dlon < (-180.)) 
       dlon = dlon + 360.
    end where
    dlon=dlon/rad

    !The spherical triangle is defined by the North Pole, the Earth
    !point and the point below the satellite.
    !GC = the great circle between PT and the satellite nadir
    GC = cos(colat1)*cos(colat2) + sin(colat1)*sin(colat2)*cos(dlon)
    gc = within(gc,-1.0,1.0)
    GC = acos(GC)

    where (abs(gc*rad) > 1.e-4)
       output = sin(dlon)*sin(colat2) / sin(gc)
       output = within(output,-1.0e0,1.0e0)
       output = asin(output)*rad
    end where
    where (abs(gc*rad) <= 1.e-4)
       output = 0.
    end where
    !convert azimuth to a range of 0-360 deg clockwise from North
    where ((lat < slat) .and. (output < 0.) )
       output = output + 360.
    end where
    where (lat >= slat)
       output = 180. - output
    end where
    
    
  end function azimuth_rank1

  function azimuth_scalar (lat, lon, slat, slon ) result (output)
    !external variables
    real(kind=4), intent(in) :: lat !degrees
    real(kind=4), intent(in) :: lon !deg
    real(kind=4), intent(in) :: slat             !deg
    real(kind=4), intent(in) :: slon             !deg
    real(kind=4)  :: output !deg
    !internal variables
    real(kind=4), dimension(1) :: azi1
    
    azi1 = azimuth( (/ lat /),(/ lon /),slat, slon )
    output = azi1(1)

  end function azimuth_scalar
!                                                            END AZIMUTH
!#######################################################################
!#######################################################################
!                                                       BEGIN CART2EARTH
!****r* kmod_geometry/cart2earth
! NAME
! cart2earth
! SYNOPSIS
! call cart2earth(x, y, z, julian, lat, lon, radius)
! FUNCTION
! Converts Cartesian coordinates to Earth-relative coordinates.
! INPUTS
! * x (km)
! * y (km)
! * z (km)
!   * r*8 - rank2
!   * r*8 - rank1
!   * r*8 - scalar
!   * same type & rank as ip
! * julian - r*8 - Julian Date
! 
! OUTPUT
!  * lat (degrees)
!  * lon  (degrees)
!  * radius (km)
!    * same type and rank as x,y,z
!
! USES 
! gha
! 
! NOTES
!***
!                                                      cart2earth_scalar
  subroutine cart2earth_scalar ( xin, yin, zin, julian, lat, lon, radius )
    implicit none
    !External variables
    real(kind=8), intent(in) :: xin, yin, zin
    real(kind=8), intent(in) :: julian
    real(kind=8), intent(out):: lat, lon, radius

    real(kind=8), dimension(1) :: lat_out, lon_out, rad_out
    
    call cart2earth( (/xin/), (/yin/), (/zin/), julian, &
                     lat_out, lon_out, rad_out)
    lat = lat_out(1)
    lon = lon_out(1)
    radius = rad_out(1)

    return
  end subroutine cart2earth_scalar
!                                                       cart2earth_rank1
  subroutine cart2earth_rank1 ( xin, yin, zin, julian, lat, lon, radius )
    implicit none
    !External variables
    real(kind=8), intent(in), dimension(:) :: xin, yin, zin
    real(kind=8), intent(in) :: julian
    real(kind=8), intent(out), dimension(:) :: lat, lon, radius
    !Internal variables
    real(kind=8), dimension(size(xin)) :: angles, declination, rasc
    
    angles = gha(julian)

    radius = sqrt(xin**2 + yin**2 + zin**2)
    declination = zin / radius
    where(declination > 1.d0)
       declination = 1.d0
    end where
    where (declination < (-1.0))
       declination = -1.0d0
    end where
    declination = asin(within(zin / radius,-1.0d0,1.0d0))
    rasc = atan2(yin,xin)

    !convert declination (which is geocentric latitude) to geographic
    !latitude
    lat = atan(be**2 * tan(declination) / ae**2)
    lon = mod((rasc - angles) , (2.d0*dpi))
    where (lon < (-dpi))
       lon = lon + 2.0d0 * dpi
    end where

    return
  end subroutine cart2earth_rank1
!                                                         END CART2EARTH
!#######################################################################
!#######################################################################
!                                                  BEGIN CONV_LAT_GEOLAT
!****f* kmod_geometry/conv_lat_geolat
! NAME
! conv_lat_geolat
! SYNOPSIS
! geolat = conv_lat_geolat(lat)
! FUNCTION
! Converts geodetic (i.e., every day, run-of-the-mill) latitude
! to geocentric latitude
! INPUTS
! * latitude (deg)
!   * r*4 - rank2
!   * r*4 - rank1
!   * r*4 - scalar
! 
! RESULT
! * geolat (deg) - same type and rank as lat
!
!***
  real function conv_lat_geolat_scalar(lat)
    real(kind=4), intent(in) :: lat !in degrees
    conv_lat_geolat_scalar = atan(tan(lat/rad)*omf2)*rad
    return
  end function conv_lat_geolat_scalar

  function conv_lat_geolat_rank1(lat) result (output)
    real(kind=4), intent(in), dimension(:) :: lat !in degrees
    real(kind=4), dimension(size(lat)) :: output
    output = atan(tan(lat/rad)*omf2)*rad
    return
  end function conv_lat_geolat_rank1

  function conv_lat_geolat_rank2(lat) result (output)
    real(kind=4), intent(in), dimension(:,:) :: lat !in degrees
    real(kind=4), dimension(size(lat,1), size(lat,2)) :: output
    output = atan(tan(lat/rad)*omf2)*rad
    return
  end function conv_lat_geolat_rank2
!                                                    END CONV_LAT_GEOLAT
!#######################################################################
!#######################################################################
!                                                  BEGIN CONV_GEOLAT_LAT
!****f* kmod_geometry/conv_geolat_lat
! NAME
! conv_geolat_lat
! SYNOPSIS
! lat = conv_geolat_lat(geolat)
! FUNCTION
! Converts geocentric latitude to (i.e., every day, run-of-the-mill) 
! geodetic latitude
! INPUTS
! * geocentric latitude (deg)
!   * r*4 - rank2
!   * r*4 - rank1
!   * r*4 - scalar
! 
! RESULT
! * geolat (deg) - same type and rank as lat
!
!***
  real function conv_geolat_lat(geolat)
    real(kind=4), intent(in) :: geolat !in degrees
    conv_geolat_lat = atan(tan(geolat/rad)/omf2)*rad
    return
  end function conv_geolat_lat
!                                                    END CONV_GEOLAT_LAT
!#######################################################################
!#######################################################################
!                                                     BEGIN EARTH_RADIUS
!****f* kmod_geometry/earth_radius
! NAME
! earth_radius
! SYNOPSIS
! radius = earth_radius(lat,geolat,sglat,cglat)
! FUNCTION
! Computes the radius of the Earth at lat.
! INPUTS
! * latitude (deg)
!   * r*4 - rank2
!   * r*4 - rank1
!   * r*4 - scalar
! 
! RESULT
! * radius (km)
! Optional outputs:
!   * geolat - returns geocentric latitude (deg)
!   * sglat = sin(geolat) - which keeps from computing it again.
!   * cglat = cos(geolat) - ditto
! USES
! conv_lat_geolat, sincos
!***
  real function earth_radius_scalar ( lat, geolat, sglat, cglat )
    real(kind=4) :: lat
    real(kind=4), optional :: sglat, cglat
    real(kind=4), optional:: geolat
    real(kind=4), dimension(1) :: geolatt, out, slat, clat
    
    out = earth_radius( (/ lat /), geolat = geolatt, sglat=slat, cglat=clat) 

    earth_radius_scalar = out(1)
    if (present(geolat)) geolat = geolatt(1)
    if (present(sglat))  sglat  = slat(1)
    if (present(cglat))  cglat  = slat(1)

  end function earth_radius_scalar

  function earth_radius_rank2 ( lat, geolat, sglat, cglat ) result ( output )
    real(kind=4), dimension(:,:) :: lat
    real(kind=4), dimension(:,:), optional :: geolat, sglat, cglat
    real(kind=4), dimension(size(lat,1), size(lat,2)) :: output
    real(kind=4), dimension(size(lat)) :: geolatt, radout, slat, clat
    integer :: n, n1, n2
    n = size(lat)
    n1= size(lat,1)
    n2= size(lat,2)
    
    radout =  earth_radius(reshape(lat,(/ n /)),geolat=geolatt, sglat=slat, cglat=clat)

    output = reshape(radout, (/ n1, n2 /) )
    if (present(geolat)) geolat = reshape(geolatt, (/ n1,n2 /) )
    if (present(sglat))  sglat  = reshape(clat, (/ n1,n2 /) )
    if (present(cglat))  cglat  = reshape(slat, (/ n1,n2 /) )

  end function earth_radius_rank2

  function earth_radius_rank1(lat,geolat,sglat,cglat) result ( output )
    !Return the radius of the earth at the geographic latitude LAT
    !in km
    real(kind=4), dimension(:), intent(in) :: lat
    real(kind=4), dimension(:), intent(inout), optional :: geolat, sglat, cglat
    real(kind=4), dimension(size(lat)) :: output
    real(kind=4), dimension(size(lat)) :: geolatt, clat, slat

    geolatt = conv_lat_geolat(lat)
    call sincos(geolatt/rad,slat,clat)

    !output = sqrt(1.d0+(omf2-1.0)*sin(geolatt/rad)**2)*ae
    output = sqrt(1.d0+(omf2-1.0)*slat*slat)*ae

    if (present(geolat)) geolat = geolatt
    if (present(sglat)) sglat = slat
    if (present(cglat)) cglat = clat
    

    return
  end function earth_radius_rank1
!                                                       END EARTH_RADIUS
!#######################################################################
!#######################################################################
!                                                       BEGIN EARTH2CART
!****r* kmod_geometry/earth2cart
! NAME
! earth2cart
! SYNOPSIS
! call earth2cart(lat,lon,x,y,z)
! FUNCTION
! Computes the Cartesian location (x,y,z) based on lat,lon of a 
! point on the earth
! INPUTS
! * latitude (deg)
! * longitude (deg)
!   * r*4 - rank2
!   * r*4 - rank1
!   * r*4 - scalar
! 
! RESULT
! * x, y, z (km)
!   * same type and rank as latitude
! USES
! earth_radius, sincos
!***
  subroutine earth2cart_scalar ( lat, lon, x, y, z )
    real (kind=4) :: lat, lon, x, y, z
    real (kind=4), dimension(1) :: ax, ay, az
    call earth2cart((/ lat /), (/ lon /), ax, ay, az)
    x = ax(1)
    y = ay(1)
    z = az(1)
    return
  end subroutine earth2cart_scalar
  subroutine earth2cart_rank2 ( lat, lon, x, y, z )
    real(kind=4), dimension(:,:) :: lat, lon, x, y, z
    real(kind=4), dimension(size(lat)) :: ax, ay, az
    integer(kind=4) :: n, n1, n2
    n = size(lat)
    n1 = size(lat,dim=1)
    n2 = size(lat,dim=2)
    call earth2cart(reshape(lat, (/ n /) ), &
         &            reshape(lon, (/ n /) ), ax, ay, az)
    x = reshape(ax, (/n1,n2/) )
    y = reshape(ay, (/n1,n2/) )
    z = reshape(az, (/n1,n2/) )
    
  end subroutine earth2cart_rank2

  subroutine earth2cart_rank1 ( lat, lon, x, y, z )
    real (kind=4), dimension(:) :: lat, lon, x, y, z
    real (kind=4), dimension(size(lat)) :: glat
    integer :: i, n
    real(kind=4) :: radius, rlon
    real(kind=4) :: slat, slon, clat, clon
    
    n = size(lat)
    do i=1,n
       if (lat(i) >= (-90.) .and. lat(i) <= 90.) then
          radius = earth_radius ( lat(i), geolat=glat(i), sglat=slat, cglat=clat )
          rlon = lon(i) / rad
          !clat = sqrt(1.-slat*slat) !cos(glat(i)/rad)
          !slat = sqrt(1.-clat*clat)
          !clon = cos(rlon)
          call sincos(rlon,slon,clon)
          x(i) = radius * clat * clon
          y(i) = radius * clat * slon
          z(i) = radius * slat
       end if
    end do
  end subroutine earth2cart_rank1

!                                                         END EARTH2CART
!#######################################################################
!#######################################################################
!                                                              BEGIN GHA
!****f* kmod_geometry/gha
! NAME
! gha
! SYNOPSIS
! angle = gha(time)
! FUNCTION
! Calculates the greenwich hour angle.
! INPUTS
! * time - Julian Date (as output from julday)
!   * scalar or rank1 r*8
! 
! RESULT
! * angle (deg) same rank and type as input time
!    Returns the Greenwich hour angle for the given time in radians
!   (optionally in degrees).
! NOTES
!    This routine is used to convert the satellite position in
!    inertial coordinates to earth coordinates (e.g., longitude). The
!    initial values are taken from Kidder and Vonder Haar:
!
!    Kidder and Vonder Haar value:
!       100.38641 deg at 0000UTC on 1 Jan 1990
! TODO
!    Should include other points which might be more accurate for times
!    not too close to 1990. .. such as the one from NOAA.
!    From NOAA
!
!    Other values can be added by including them in the DATE array and
!    the corresponding hour angle in the GHA0 array
! USES
! julday
!***
!                                                             gha_scalar
  function gha_scalar ( julian )
    implicit none
    !External variables
    real(kind=8) :: gha_scalar
    real(kind=8), intent(in) :: julian
    real(kind=8), dimension(1) :: gha_out
    gha_out = gha( (/julian/) )
    gha_scalar = gha_out(1)
  end function gha_scalar
!                                                              gha_rank1
  function gha_rank1 ( julian ) result (gha_)
    implicit none
    !External variables
    real(kind=8), intent(in), dimension(:) :: julian
    real(kind=8), dimension(size(julian)) :: gha_
    !Internal variables
    real(kind=8) :: rot, date, gha0

    !Rotation of earth (radians per second)
    rot = 7.292115922d-5
    !Kidder and Vonder Haar, 1995
    date = julday(1,1,1990,0,0,0.)
    gha0 = 100.38641d0 / drad

    gha_ = gha0 + (julian - date) * rot * 86400.d0
    gha_ = mod(gha_,2.d0*dpi)
    where (gha_ < 0.d0)
       gha_ = gha_ + 2.0d0 * dpi
    end where

    return
  end function gha_rank1
!                                                                END GHA
!#######################################################################
!#######################################################################
!                                                         BEGIN KEP2CART
!****r* kmod_geometry/kep2cart
! NAME
! kep2cart
! SYNOPSIS
! call kep2cart(kepler,time,x,y,z)
! FUNCTION
! Calculates the Cartesian location based on keplerian elements
! and Julian Date.
! INPUTS
! * kepler - real*8(6)
!    * kepler(1) = Semimajor axis (km)
!    * kepler(2) = Eccentricity
!    * kepler(3) = Inclination (deg)
!    * kepler(4) = Right ascension of scending node
!    * kepler(5) = Argument of Perigee
!    * kepler(6) = Mean Anomaly
! * time - r*8(scalar or rank1)  - Julian Date (output from julday)
! RESULT
! * x,y,z (r*8: same rank as inputs) - Cartesian locations (km)
! NOTES
!    To locate a satellite in its orbit around a mass (e.g., Earth,
!    Sun) using the Keplerian elements and the time passed since
!    Epoch. The returned position is in the pseudo-inertial reference
!    frame where x points from center of Earth to the Sun at vernal
!    equinox, the z coordinate points along the Earths spin axis
!    northward and the y coordinate completes the right handed
!    coordinate system.
!
!    These calculations are based on equations from Kidder & Vonder Haar, 1995:
!    Introduction to Satellite Meteorology
! USES
! sincos
!***
!                                                        kep2cart_scalar
  subroutine kep2cart_scalar ( kepler, time, xout, yout, zout )
    implicit none
    ! External variables
    real(kind=8), intent(in), dimension(6) :: kepler
    real(kind=8), intent(in)  :: time
    real(kind=8), intent(out) :: xout, yout, zout
    !Internal variables
    real(kind=8), dimension(1) :: x, y, z

    call kep2cart(kepler,(/time/),x,y,z)
    xout = x(1) ; yout = y(1) ; zout = z(1)

    return
  end subroutine kep2cart_scalar
!                                                         kep2cart_rank1
  subroutine kep2cart_rank1 ( kepler, time, xout, yout, zout )
    implicit none
    ! External variables
    real(kind=8), intent(in), dimension(6) :: kepler
    real(kind=8), intent(in), dimension(:) :: time
    real(kind=8), intent(out), dimension(:) :: xout, yout, zout
    ! Internal variables
    real(kind=8) :: semima, incl, eccen, meana, rasc, perigee
    real(kind=8) :: a, nn, e, sin_s, cos_s, i
    real(kind=8) :: dmdt, dradt, dwdt, nodper, ammc
    real(kind=8), dimension(size(time)) :: anom, ecan1, ecanom, &
         tranom, radius
    real(kind=8), dimension(size(time)) :: x,    y  !,    z
    real(kind=8), dimension(size(time)) :: xp,   yp !,   zp
    real(kind=8), dimension(size(time)) :: xpp,  ypp,  zpp
!    real(kind=8), dimension(size(time)) :: xppp, yppp, zppp
    real(kind=8), dimension(size(time)) :: sin_, cos_
    integer(kind=4), dimension(size(time)) :: n, m
    integer(kind=4) :: j, k, np
    np=size(time)
    !Assign Kepler elements
    semima = kepler(1)
    eccen  = kepler(2)
    incl   = kepler(3)
    rasc   = kepler(4)
    perigee= kepler(5)
    meana  = kepler(6)
    
    !Calculate changes in the orbit using the orbit perturbations from
    !Kidder and Vonder Haar, 1995, page 25

    ! MEAN MOTION CONSTANT = dmdt
    a = semima
    nn = sqrt(gme / ((a*1.d3)*(a*1.d3)*(a*1.d3))) !s^-1
    e = eccen
    i = incl / drad
    
    dmdt = nn  * (1.d0 + &
                 1.5d0*j2*(ree/a)*(ree/a) * sqrt((1.d0-e*e)**(-3)) * &
                 (1.d0 - 1.5d0*(sin(i)*sin(i))))

    dradt = -dmdt * (1.5d0*j2*(ree/a)*(ree/a) * cos(i) / (1.d0 - e*e)/(1.d0 - e*e) )

    dwdt  =  dmdt * (1.5d0*j2*(ree/a)*(ree/a) * (2.d0 - 2.5d0 * (sin(i)*sin(i))) / &
                    ((1.d0-e*e)*(1.d0-e*e) ))

    nodper = 2.d0*dpi / (dmdt + dwdt)

    !mean motion constant (n bar)
    ammc = dmdt

    !mean anomaly
    anom = time * ammc + meana / drad

    !solve for eccentric anomaly (e)
    ecan1 = ammc * time
    do j=0,6
       do k=1,np
          ecan1(k) = anom(k) + eccen*sin(ecan1(k))
       end do
    end do
    ecanom = ecan1

    !true anomaly
    tranom = cos(ecanom)
    tranom = (tranom - eccen) / (1.d0 - eccen*tranom)
!!$    tranom = (cos(ecanom) - eccen) / (1.d0 - eccen*cos(ecanom))
    tranom = asin(within(tranom,-1.0d0,1.0d0))
!!$    ang=ecanom
!!$    n = int((ang+pi)/pi)
    n = int((ecanom+pi)/pi)
    m = -1 
    where (mod(n,2) == 0)
       m = 1
    end where
    tranom = dpi*real(n,8) + real(m,8) * tranom - dpi/2.d0

    !cos_ = cos(tranom)
    !sin_ = sin(tranom)
    call sincos(tranom(1:np),sin_(1:np),cos_(1:np))


    !calculate radius (r)
    radius = semima * (1.d0 - eccen*eccen) / (1.d0 + eccen*cos_)

    !locate satellite in orbital plane
    do j=1,np
       x(j) = radius(j) * cos_(j)
       y(j) = radius(j) * sin_(j)
       !z(j) = 0. !tranom * 0.d0
    end do

    !rotate axis about argument of perigee
    call sincos(perigee/drad + dwdt*time(1:np),sin_(1:np),cos_(1:np))
    !sin_= sin(perigee/rad + dwdt*time)
    !cos_= cos(perigee/rad + dwdt*time)
    do j=1,np
       xp(j) = x(j)*cos_(j) - y(j)*sin_(j)
       yp(j) = x(j)*sin_(j) + y(j)*cos_(j)
    end do
    !zp = z !Z=0.

    !rotate axis through inclination
    sin_s = sin(i)
    cos_s = cos(i)
    do j=1,np
       xpp(j) = xp(j)
       ypp(j) = yp(j)*cos_s !- zp*sin_ !ZP = Z = 0.
       zpp(j) = yp(j)*sin_s !+ zp*cos_ !ZP = Z = 0.
    end do

    !rotate axis through right ascension
    call sincos(rasc/drad + dradt*time,sin_,cos_)
    do j=1,np
!!$       sin_s = sin(rasc/rad + dradt*time(j))
!!$       cos_s = cos(rasc/rad + dradt*time(j))
       xout(j) = xpp(j)*cos_(j) - ypp(j)*sin_(j)
       yout(j) = xpp(j)*sin_(j) + ypp(j)*cos_(j)
       zout(j) = zpp(j)
    end do

    !set x,y,z to the same dimensions as intime
!!$    xout = xppp
!!$    yout = yppp
!!$    zout = zppp

    return
  end subroutine kep2cart_rank1
!                                                           END KEP2CART
!#######################################################################
!#######################################################################
!                                                      BEGIN SOLAR_ANGLE
!****r* kmod_geometry/solar_angle
! NAME
! solar_angle
! SYNOPSIS
! call solar_angle(lat,lon,day,time,sza,azi)
! FUNCTION
! Calculates solar zenith and azimuth angles.
! INPUTS
! * lat - real*4 - deg
! * lon - real*4 - deg
! * day - i*4 - day of year
! * time- real*4 - fractional hour of day (e.g., output from ftime) (UTC)
!
! OUTPUT
! * sza - real*4 - deg
! * azi - real*4 - deg - optional
!
! OPTIONS
! There are numerous input options available:
! TIME CALCULATED PRIOR TO CALL
! * ONE POINT AT ONE TIME
!   * call solar_angle(lat,lon,day,time,sza,azi)
!     * lat,lon,day,time,sza,azi are all scalars.
! * MANY POINTS AT ONE TIME
!     * lat,lon, sza, azi are rank1
!     * day, time are scalar
! * MANY POINTS AT MANY TIMES
!     * lat,lon,day,time,sza,azi are all rank1.
! TIME CALCULATED FROM LINE HEADER
! * MANY POINTS AT MANY TIMES
!   * call solar_angle(lat,lon,iy,b1u.img,linhdr,sza,azi
!     * lat, lon, sza, azi are rank2 or rank1 or scalar
!     * iy is same rank as lat (from the b1u_ll2ij routine)
!     * b1u.img - array from the b1uread routine
!     (This method is useful for getting info on recently navigated points).
!
! USES
! ftime
!***
  subroutine solar_angle_scalar  ( lat, lon, day, time, sza, azi )
    !Externals
    real(kind=4),    intent(in)    :: lat !latitude in degrees
    real(kind=4),    intent(in)    :: lon !longitude in degrees
    integer(kind=4), intent(in)    :: day !day of year (1-366)
    real(kind=4),    intent(in)    :: time !time of day (UTC in decimal format)
    real(kind=4),    intent(inout) :: sza !Solar zenith angle in degrees
    real(kind=4),    intent(inout), optional :: azi !Azimuth angle (degrees)
    !internals
    real(kind=4), dimension(1) :: sza_d, azi_d

    if (present(azi)) then
       call solar_angle((/lat/), (/lon/), (/day/), (/time/), sza_d, azi_d)
    else
       call solar_angle((/lat/), (/lon/), (/day/), (/time/), sza_d)
    end if
    sza = sza_d(1)
    if (present(azi)) azi = azi_d(1)
  end subroutine solar_angle_scalar

  subroutine solar_angle_lr1_tsc ( lat, lon, day, time, sza, azi )
    !Externals
    real(kind=4),    dimension(:), intent(in)    :: lat !latitude in degrees
    real(kind=4),    dimension(:), intent(in)    :: lon !longitude in degrees
    integer(kind=4), intent(in)    :: day !day of year (1-366)
    real(kind=4),    intent(in)    :: time !time of day (UTC in decimal format)
    real(kind=4),    dimension(:), intent(inout) :: sza !Solar zenith angle in degrees
    real(kind=4),    dimension(:), intent(inout), optional :: azi !Azimuth angle (degrees)
    !Internals
    
    call solar_angle(lat, lon, spread(day,1,size(lat)), spread(time,1,size(lat)), sza, azi)
  end subroutine solar_angle_lr1_tsc

  subroutine solar_angle_rank1   ( lat, lon, day, time, sza, azi )
    !Externals
    real(kind=4),    dimension(:), intent(in)    :: lat !latitude in degrees
    real(kind=4),    dimension(:), intent(in)    :: lon !longitude in degrees
    integer(kind=4), dimension(:), intent(in)    :: day !day of year (1-366)
    real(kind=4),    dimension(:), intent(in)    :: time !time of day (UTC in decimal format)
    real(kind=4),    dimension(:), intent(inout) :: sza !Solar zenith angle in degrees
    real(kind=4),    dimension(:), intent(inout), optional :: azi !Azimuth angle (degrees)
    !Internals
    real(kind=4), dimension(size(lat)) :: declin  !Solar declination angle
    real(kind=4), dimension(size(lat)) :: hourang !Hour angle
    real(kind=4), dimension(size(lat)) :: mu_not  !sin(SZA)
    real(kind=4), dimension(size(lat)) :: GC      !Great Circle length
    real(kind=4), dimension(size(lat)) :: dlon    !difference in longitude
    
    hourang = ( 12. - time ) * 15.
    declin = -23.45 * cos(day*2.*pi/365. + 10.*2.*pi/365.)

    dlon = hourang-lon
    where (dlon < (-180.)) 
       dlon = dlon + 360.
    end where
    where (dlon > 180.) 
       dlon = dlon - 360.
    end where
    mu_not = -1.
    where (lat > (-90.) .and. (lat < 90.))
       mu_not = sin(lat/rad) * sin(declin/rad) + &
            &   cos(lat/rad) * cos(declin/rad) * cos(dlon/rad)
    end where
    mu_not = within(mu_not,minimum=-1.0,maximum=1.0)
    sza = acos(mu_not) * rad
    where (lat < (-90.) .or. lat > 90.) 
       sza = -99999.
    end where
    
    if (present(azi)) then
       !The spherical triangle is defined by the North Pole, the Earth
       !point and the point below the Sun.
       !GC = the great circle between PT and the Sun nadir
       GC = cos((90.-lat)/rad)*cos((90.-declin)/rad) + &
          & sin((90.-lat)/rad)*sin((90.-declin)/rad)*cos(dlon/rad)
       GC = within(GC,minimum=-1.0e0,maximum=1.0e0)
       GC = acos(GC)
       azi = sin(dlon/rad) * sin((90.-declin)/rad) / sin(GC)
       azi = within(azi,maximum=1.0e0,minimum=-1.0e0)
       azi = asin(azi)*rad
       where (lat > declin)
          azi = 180. - azi
       end where
       where (lat < declin .and. azi < 0.)
          azi = azi + 360.
       end where

    end if
    
  end subroutine solar_angle_rank1

  subroutine solar_angle_rank2_lnhdr ( lat, lon, iy, img, linhdr, sza, azi )
    !External variables
    real(kind=4), dimension(:,:),intent(in) :: lat
    real(kind=4), dimension(:,:),intent(in) :: lon
    real(kind=4), dimension(:,:),intent(in) :: iy
    integer(kind=4), dimension(0:19), intent(in) :: img
    integer(kind=2), dimension(:,:),intent(in) :: linhdr
    real(kind=4), dimension(size(lat,1),size(lat,2)), intent(out) :: sza
    real(kind=4), dimension(size(lat,1),size(lat,2)), intent(out), optional :: azi
    !internal variables
    integer(kind=4) :: n, nx, ny
    real(kind=4), dimension(size(lat)) :: latt, lonn, iyy, szaa,azii

    n = size(lat)
    nx = size(lat,1)
    ny = size(lat,2)

    latt = reshape(lat,(/ n /) )
    lonn = reshape(lon,(/ n /) )
    iyy  = reshape(iy ,(/ n /) )
    
    if (present(azi)) then
       call solar_angle(latt, lonn, iyy, img, linhdr, szaa, azii)
       azi = reshape(azii, (/ nx,ny /) )
    else
       call solar_angle(latt, lonn, iyy, img, linhdr, szaa, azii)
    endif
    sza = reshape(szaa, (/ nx, ny /) )
  end subroutine solar_angle_rank2_lnhdr

  subroutine solar_angle_scalar_lnhdr ( lat, lon, iy, img, linhdr, sza, azi )
    !External variables
    real(kind=4), intent(in) :: lat
    real(kind=4), intent(in) :: lon
    real(kind=4), intent(in) :: iy
    integer(kind=4), dimension(0:19), intent(in) :: img
    integer(kind=2), dimension(:,:),intent(in) :: linhdr
    real(kind=4), intent(out) :: sza
    real(kind=4), intent(out), optional :: azi
    !internal variables
    real(kind=4), dimension(1) :: latt, lonn, iyy, szaa,azii


    latt(1) = lat
    lonn(1) = lon
    iyy(1)  = iy
    
    if (present(azi)) then
       call solar_angle(latt, lonn, iyy, img, linhdr, szaa, azii)
       azi = azii(1)
    else
       call solar_angle(latt, lonn, iyy, img, linhdr, szaa)
    endif
    sza = szaa(1)
  end subroutine solar_angle_scalar_lnhdr

  subroutine solar_angle_rank1_lnhdr ( lat, lon, iy, img, linhdr, sza, azi )
    !External variables
    real(kind=4), dimension(:),intent(in) :: lat
    real(kind=4), dimension(:),intent(in) :: lon
    real(kind=4), dimension(:),intent(in) :: iy
    integer(kind=4), dimension(0:19), intent(in) :: img
    integer(kind=2), dimension(:,:),intent(in) :: linhdr
    real(kind=4), dimension(:), intent(out) :: sza
    real(kind=4), dimension(:), intent(out), optional :: azi
    !Internal variables
    integer(kind=4), dimension(size(lat)) :: hhmm, jday
    integer(kind=2), dimension(size(lat)) :: sec, msec
    real (kind=4),   dimension(size(lat)) :: time
    integer(kind=4) :: n, i
    real (kind=4),   dimension(size(lat)) :: hourang, declin, mu_not, beta
    real (kind=4),   dimension(size(lat)) :: slat, slon, h, ns

    n = size(lat)
    !print*,ubound(linhdr) = 20 1354
    !print*,lbound(linhdr) =  1    1

    !Get date & time of each line from the linhdr
    do i=1,n
       hhmm(i) = linhdr(6,nint(iy(i)))
       sec(i)  = linhdr(7,nint(iy(i)))
       msec(i) = linhdr(8,nint(iy(i)))
       jday(i) = linhdr(5,nint(iy(i)))
    end do
    time = ftime(hhmm/100, mod(hhmm,100),real(sec)+real(msec)/1000)
!!$    print*,'Time:',sum(time)/real(n)
!!$    print*,'hhmm:',hhmm(1:n:n/20)
!!$    print*,'sec:',sec(1:n:n/20)
!!$    print*,'msec',msec(1:n:n/20)
!!$    print*,'Time',time(1:n:n/20)

    if (((sum(time)/real(n)) < 0) .or. ((sum(time)/real(n)) >= 24.)) then
       hhmm = img(1) / 100
       sec  = mod(img(1),100)
       msec = 0.
       time = ftime(hhmm/100, mod(hhmm,100),real(sec)+real(msec)/1000)
       jday = mod(img(0),1000)
!       print*,'No Time information in line header: New time:',time(1)
    end if


    hourang = -1. * (time - 12.) * 15.
    declin = -23.45 * cos(real(jday)*2.*pi/365. + 10.*2.*pi/365.)
    mu_not = -1.
    where (lat > (-90) .and.lat < 90.)
       mu_not = sin(lat/rad)*sin(declin/rad) + &
                cos(lat/rad)*cos(declin/rad)*cos((hourang-lon)/rad)
    end where
!    where (mu_not >   1. ) ; mu_not =  1. ; end where
!    where (mu_not < (-1.)) ; mu_not = -1. ; end where
    mu_not = within(mu_not, minimum=-1.0,maximum=1.0)
    sza = acos(mu_not) * rad
    where (lat < (-90.) .or. lat > 90.) 
       sza = -99999.
    end where

    if (present(azi)) then
        beta = 100.
        slon = hourang
        slat = declin
        where ((lat < 90) .and. (lat > (-90)))
           beta = acos(cos((lat-slat)/rad) * cos((slon-lon)/rad))
        end where
        where ((lon > (-180)) .and. (lon < 180))
           azi = sin((slon-lon)/rad) / sin(beta)
        end where
!!$        where (azi > 1.)
!!$           azi = 1.0
!!$        end where
!!$        where (azi < (-1.))
!!$           azi = -1.0
!!$        end where
        azi = asin(within(azi,-1.0,1.0))*rad
        where (azi < 0.) 
           azi = azi + 360.
        end where
        h = mod((lon-slon), 360.)
        ns = lat - slat
        where (h < 0)
           h = h + 360.
        end where
        !points north east of slat,slon
        where ((ns > 0) .and. (h < 180.))
           azi = 540. - azi
        end where
        !points north west of slat,slon
        where ((ns > 0) .and. (h > 180.))
           azi = 180. - azi
        end where
        !bad points
        where ((lat < -90.) .or. (lat > 90))
           azi = -99999.
        end where
     endif
  end subroutine solar_angle_rank1_lnhdr
!                                                        END SOLAR_ANGLE
!#######################################################################
!#######################################################################
!                                                           BEGIN VARSOL
!****f* kmod_geometry/varsol
! NAME
! varsol
! SYNOPSIS
! ddm = varsol(jday)
! FUNCTION
!       correction factor for Earth-Sun dist
! INPUTS
!   JDAY : integer - day of year (range i 1 to 366)
! OUTPUT
!    Returns the Dsol value. The Earth-Sun Distance normalized by the
!    average distance.
! OPTIONS
! NOTES
!       from varsol.f (from 6S model)
!***
  pure function varsol ( jday )
    !External
    integer(kind=4), intent(in) :: jday !Day of year
    real(kind=4) :: varsol
    !Internal
    real(kind=4) :: M
    
    M = 0.9856*real(jday-4,4)*rad
    varsol = 1. / ((1.-0.01673*cos(M))**2)

  end function varsol
!                                                             END VARSOL
!#######################################################################
!#######################################################################
!                                                       BEGIN VIEWZENITH
!****f* kmod_geometry/viewzenith
! NAME
! viewzenith
! SYNOPSIS
! vza = viewzenith(lat,lon,glat,glon)
! FUNCTION
! Calculate the view zenith angle (in degrees) from the lat/lon of 
! a location on the Earth and a satellite in geostationary orbit at 
! location glat/glon
! INPUTS
! Inputs can be of rank1, rank2 or scalar and are real*4's:
! * lat - deg
! * lon - deg
! * glat - deg
! * glon - deg
! OUTPUT
! vza - deg - Real*4 with same rank as input.
!***
  pure function viewzenith_rank1 ( lat, lon, glat, glon ) result (viewzenith)
    !External
    Real(kind=4), dimension(:),intent(in) :: lat, lon
    real(kind=4), intent(in) :: glat, glon
    real(kind=4), dimension(size(lat)) :: viewzenith
    !internal
    real(kind=8), dimension(size(lat)) :: beta

    !Routine assumes an optimal geostationary orbit radius
    beta = acos(within(cos((lat-glat)/rad) * cos((glon-lon)/rad),&
         &             minimum=-1.0, maximum=1.0))
    viewzenith = (42166.264d0*sin(beta) / &
                  sqrt(1.818402116d9 - 5.36080966d8*cos(beta)))
    viewzenith = asin(within(viewzenith,-1.0,1.0)) * rad
  end function viewzenith_rank1

  pure function viewzenith_rank2 ( lat, lon, glat, glon ) result (viewzenith)
    !External
    Real(kind=4), dimension(:,:),intent(in) :: lat, lon
    real(kind=4),intent(in) :: glat, glon
    real(kind=4), dimension(size(lat,1),size(lat,2)) :: viewzenith
    !internal
    real(kind=8), dimension(size(lat,1),size(lat,2)) :: beta

    !Routine assumes an optimal geostationary orbit radius
    beta = 0.
    where(lat > (-90) .and. lat < (90.))
       beta = acos(within(cos((lat-glat)/rad) * cos((glon-lon)/rad),-1.0,1.0))
    end where
!!$    beta = acos(within(cos((lat-glat)/rad) * cos((glon-lon)/rad),&
!!$         &             minimum=-1.0, maximum=1.0))
    viewzenith = (42166.264d0*sin(beta) / &
                  sqrt(1.818402116d9 - 5.36080966d8*cos(beta)))
    viewzenith = asin(within(viewzenith,-1.0,1.0)) * rad
    where(lat < (-90) .or. lat > (90.))
       viewzenith = -1.
    end where
  end function viewzenith_rank2

  pure real(kind=4) function viewzenith_scalar ( lat, lon, glat, glon )
    !External
    Real(kind=4),intent(in) :: lat, lon, glat, glon
    !internal
    real(kind=8) :: beta

    !Routine assumes an optimal geostationary orbit radius
    beta = acos(within(cos((lat-glat)/rad) * cos((glon-lon)/rad),&
         &             minimum=-1.0, maximum=1.0))
    viewzenith_scalar = (42166.264d0*sin(beta) / &
                  sqrt(1.818402116d9 - 5.36080966d8*cos(beta)))
    viewzenith_scalar = asin(within(viewzenith_scalar,&
         &                          -1.0,1.0)) * rad
  end function viewzenith_scalar
!                                                         END VIEWZENITH
!#######################################################################
!#######################################################################
!                                              BEGIN VIEW_ZENITH_CORRECT
!****r* kmod_geometry/view_zenith_correct
! NAME
! view_zenith_correct
! SYNOPSIS
! performs the correction
! FUNCTION
! INPUTS
!     INPUT:           uncorrected Tb, satellite zenith angle,
!                      Julian day, latitude
! OUTPUT
!     OUTPUT:          Seasonal, Latitudinal, Tb dependent
!                           Zenith angle IR Tb correction
! NOTES
!     programmer:      Bob Joyce
!     date created:    3/2/2000
! USES
! VIEW_ZENITH_READ_MODELS
!***
  subroutine view_zenith_correct (error,errorlat,Tb,sza,jday,xlat,Tbcorr)
    !external variables
    integer(kind=4), parameter :: J5MAX = 240
    integer(kind=4), parameter :: ZABINMX = 64
    integer(kind=4), parameter :: TMPBINMX = 170
    integer(kind=4), parameter :: ISEASMX = 8
    real(kind=4),dimension(ZABINMX,TMPBINMX) :: error
    real(kind=4),dimension(J5MAX,ISEASMX)    :: errorlat
    real(kind=4), intent(inout) :: tb
    real(kind=4), intent(in) :: sza !Satellite zenith angle
                                    !NOT to be confused with SOLAR zenith angle
    real(kind=4), intent(in) :: jday
    real(kind=4), intent(in) :: xlat
    real(kind=4), intent(out) :: tbcorr
    
    !internal variables
    real(kind=4), parameter    :: ZAMAX = 87.0
    integer(kind=4), parameter :: ISTYR = 1999

    real(kind=4), parameter    :: XMISSING=-9999


    real(kind=4), parameter    :: SZAANGMN=26.
    real(kind=4), parameter    :: SZATMPMN=160.
    real(kind=4), parameter    :: YEARDYS=365
    real(kind=4), parameter    :: XLATMX=60
    real(kind=4), parameter    :: XLATMN=-60

    integer(kind=4), parameter :: ZERO=0
    integer(kind=4), parameter :: ONE=1

    real(kind=4) xlatbinint
    real(kind=4), dimension(iseasmx) ::seasjday
    integer j5x, icday, iz, it
    real diff1, diff2, w1, w2, xlnth, xerrorlat, xjday, temp

    xjday = jday
    seasjday = (/ 57.5, 104.5, 148.5, 206.5, 268.5, 326.5, 395., 422.5 /)
    XLATBININT=0.5

    !add yearly day total if after feb 26 (when model began)
    if (xjday < seasjday(ONE)) xjday = xjday + YEARDYS

    !Find latitude bin for target
    if (xlat > XLATMX) then
       j5x = ONE
    else if ((xlat > XLATMN) .and. (xlat < XLATMX)) then
       j5x = (XLATMX - xlat)/XLATBININT + ONE
    else
       j5x = J5MAX
    end if

    !Check to see if current day is exactly seasonal model
    do icday = 1, ISEASMX-1
       if (xjday == seasjday(icday)) then
          xerrorlat = errorlat(j5x,icday)
          exit
       else
          if ((xjday > seasjday(icday)) .and. &
              (xjday < seasjday(icday+1))) then
             !Find day difference from current day and early and late seasonal tables
             diff1 = xjday - seasjday(icday)
             diff2 = seasjday(icday+1) - xjday
             xlnth = seasjday(icday+1) - seasjday(icday)
             w1    = diff2/xlnth
             w2    = diff1/xlnth
             xerrorlat = errorlat(j5x,icday)*w1 + errorlat(j5x,icday+1)*w2
             exit
          endif
       endif
    end do

    !Add zenith angle correction to IR
    iz = (sza-SZAANGMN) + 1
    it = (Tb-SZATMPMN) + 1

    if (iz >= 1)then
       if (iz > ZABINMX) iz=ZABINMX
       if (it < 1) it=1
       if (it > TMPBINMX) it=TMPBINMX
       temp = Tb - xerrorlat*error(iz,it)
    else
       temp = Tb
    endif

    !if sza > 87 or if Tb is non positive set to missing
    if ((sza > ZAMAX) .or. (sza < ZERO) .or. (Tb < ZERO)) then
!       Tb     = XMISSING
       temp = XMISSING
    endif
    tbcorr = temp
  end subroutine view_zenith_correct
!                                                END VIEW_ZENITH_CORRECT
!#######################################################################
!#######################################################################
!                                          BEGIN VIEW_ZENITH_READ_MODELS
!****r* kmod_geometry/VIEW_ZENITH_READ_MODELS
! NAME
! VIEW_ZENITH_READ_MODELS
! SYNOPSIS
!    call view_zenith_read_models(error,errorlat,itype)
! FUNCTION
! Reads in the tables to be used by the correction algorithm.
! NOTES
!  INCLUDE FOLLOWING CODE (through END INCLUDE) IN MAIN PROGRAM
!    integer(kind=4), parameter :: J5MAX = 240
!    integer(kind=4), parameter :: ZABINMX = 64
!    integer(kind=4), parameter :: TMPBINMX = 170
!    integer(kind=4), parameter :: ISEASMX = 8
!    integer(kind=4) :: itype
!    real(kind=4),dimension(ZABINMX,TMPBINMX) :: error
!    real(kind=4),dimension(J5MAX,ISEASMX)    :: errorlat
!
!    call view_zenith_read_models(error,errorlat,itype)
!
!  END INCLUDE
!
!
!  programmer:      Bob Joyce
!  date created:    3/2/2000
!  purpose:         Subroutine read zenith angle corrections models
!  For itype enter "0" if using PC or LINUX, "1" if SGI or HP
!
! USES
! EnchiladaDataDir
!
!***
  subroutine VIEW_ZENITH_READ_MODELS ( error, errorlat, itype )

    integer(kind=4), parameter :: J5MAX = 240
    integer(kind=4), parameter :: ZABINMX = 64
    integer(kind=4), parameter :: TMPBINMX = 170
    integer(kind=4), parameter :: ISEASMX = 8

    !correction table at equator
    real(kind=4),dimension(ZABINMX,TMPBINMX) :: error
    !correction table for latitude and season
    real(kind=4),dimension(J5MAX,ISEASMX)    :: errorlat
    !linux,PC: ITYPE=0 or for IBM, Sun, etc. ITYPE=1
    integer(kind=4),intent(in)               :: itype

    real difflat(J5MAX,ISEASMX)
    character(len=98) :: regridin, errorlatin
    integer j5, iseas

    !Read in IR correction table
    !regridin ="/export-1/lnx57/wd52rj/mcidas/src/sza/error_regrid"
    regridin = EnchiladaDataDir//'error_regrid'
    open (unit=10, file = trim(regridin), access = 'direct',&
          form='unformatted', recl=64*170*4)
    read (10,rec=1) error
    close (10)
    if (itype == 1) then
       call SWAP46(error,ZABINMX,TMPBINMX)
    endif
    !Read in latitude adjustment
    !errorlatin =" ... latitudinal-error_annual-cycle-model"
    errorlatin = EnchiladaDataDir//'latitudinal-error_annual-cycle-model'
    open (unit=22, file = trim(errorlatin), &
         access = 'direct',form='unformatted', recl=240*2*4)
    do iseas = 1, ISEASMX
       read (22, rec=iseas) (errorlat(j5,iseas),j5=1,J5MAX),&
            (difflat(j5,iseas),j5=1,J5MAX)
    end do
    close (22)
    if (itype == 0) then
       call  SWAP46(errorlat,J5MAX,ISEASMX)
    endif
  contains
    SUBROUTINE SWAP46(A,N,M)
      !REVERSE ORDER OF BYTES IN INTEGER*4 WORD, or REAL*4
      !External variables
      integer (kind=4),intent(in) :: N, M
      real(kind=4),intent(inout),dimension(:,:)  :: A
      !Internal variables
      real(kind=4) ::ITEMP
      CHARACTER(len=1) JTEMP(4)
      CHARACTER(len=1) KTEMP
      integer(kind=4) :: i, j
      
!      EQUIVALENCE (JTEMP,ITEMP) ... use transfer instead
      DO I = 1,N
         do j=1, m
            ITEMP    = A(I,j)
            jtemp = transfer(itemp,jtemp)
            KTEMP    = JTEMP(4)
            JTEMP(4) = JTEMP(1)
            JTEMP(1) = KTEMP
            KTEMP    = JTEMP(3)
            JTEMP(3) = JTEMP(2)
            JTEMP(2) = KTEMP
            itemp = transfer(jtemp,itemp)
            A(I,j)     = ITEMP
         end do
      END DO
    end SUBROUTINE SWAP46
  end subroutine VIEW_ZENITH_READ_MODELS
!                                            END VIEW_ZENITH_READ_MODELS
!#######################################################################
!#######################################################################
!                                                           BEGIN ZENAZI
!****r* kmod_geometry/zenazi
! NAME
! zenazi
! SYNOPSIS
! call zenazi(point,sat,vza,azi)
! FUNCTION
! Calculates zenith (and azimuth) angles from point and satellite locations.
! INPUTS
! * point
!   Either:
!   * real*4(3) - Cartesian coordinates (km)
!   * real*4(2) - (/lat,lon/) coordinates (deg)
! * sat - real*4(3) - (/lat(deg),lon(deg),radius(deg)/) 
!         for example, subsat output from b1u_ij2ll
! OUTPUT
! * vza - real*4 - zenith angle (deg)
! * azi - real*4 - azimuth angle (deg) - optional
! USES 
! earth_radius, conv_lat_geolat, azimuth
! NOTES
!***
   subroutine zenazi_angles ( pt, sat, vza, azi )
    !Externals
    real, dimension(2), intent(in) :: pt !point on earth in lat/lon
    real, dimension(3), intent(in) :: sat !sat location lat,lon,radius
    real,intent(out) :: vza, azi
    !Internals
    real, dimension(3) :: pt_cart, sat_cart
    real :: glat, radlat, lon

    !convert Earth point to Cartesian
    radlat = earth_radius(pt(1),geolat=glat)
    glat=glat/rad
    lon=pt(2)/rad
    pt_cart = (/ radlat * cos(glat) * cos(lon),&
         &       radlat * cos(glat) * sin(lon),&
         &       radlat * sin(glat) /)

    !Convert satellite point to Cartesian
    glat = conv_lat_geolat(sat(1))/rad
    lon = sat(2) / rad
    sat_cart = (/ sat(3) * cos(glat) * cos(lon),&
         &        sat(3) * cos(glat) * sin(lon),&
         &        sat(3) * sin(glat) /)

    call zenazi_cartesian(pt_cart, sat_cart, vza, azi)
    
  end subroutine zenazi_angles
  subroutine zenazi_cartesian ( pt, sat, vza, azi )
    !Externals 
    real, dimension(3), intent(in) :: pt
    real, dimension(3), intent(in) :: sat
    real, intent(inout) :: vza, azi
    !Internals
    real :: satlat, satlon, plat, plon
    real, dimension(3) :: view
    real :: magp, mags, magv

    
    !View zenith angle is the angle subtended by the Earth point and the
    !view vector
    view = pt - sat
    magp = sqrt(sum(pt*pt))
    mags = sqrt(sum(sat*sat))
    magv = sqrt(sum(view*view))
    if (abs(magp) < 1.e-1 .or. abs(magv) < 1.e-1) then
       vza = -99.
       azi = -99.
       return
    end if
    !print*,pt,sat,view
    vza = 180. - rad * &
         & acos(within((view(1)*pt(1) + view(2)*pt(2) + view(3)*pt(3)) / &
         &                  magv / magp,-1.0e0,1.0e0))
    !Get the azimuth from the 2 sets of lat/lon
    !All angles calculated below are in radians except AZIMUTH
    satlat = asin(within(sat(3)/mags,-1.0e0,1.0e0))*rad
    satlat = conv_geolat_lat(satlat)
    satlon = atan2(sat(2),sat(1))*rad

    plat = asin(within(pt(3) / magp,-1.0e0,1.0e0))*rad
    plat = conv_geolat_lat(plat)
    plon = atan2(pt(2),pt(1))*rad
    azi = azimuth(plat, plon, satlat, satlon)

    return
  end subroutine zenazi_cartesian
!                                                             END ZENAZI
!#######################################################################



end module kmod_geometry
