module kmod_isccp
!****h* /kmod_isccp
! NAME
! kmod_isccp
! SYNOPSIS
! Files used in manipulating ISCCP data and files
! USES
! kmod_mcidas, kmod_utilities, kmod_math, kmod_geometry
!***
  
  use kmod_mcidas
  use kmod_utilities
  use kmod_math
  use kmod_geometry
  use kmod_time

  implicit none

!****s* kmod_isccp/b1u_satinfo
! NAME
! b1u_satinfo
! SYNOPSIS
! Store satellite and sensor information for B1U data.
! SOURCE
! 
  TYPE b1u_satinfo
     sequence
     character(len=16) :: satnam
     character(len=16) :: sennam
     character(len=6), allocatable, dimension(:) :: chname
     character(len=20), allocatable, dimension(:) :: chinfo
  END TYPE b1u_satinfo
!***
!****s* kmod_isccp/b1u_calinfo
! NAME
! b1u_calinfo
! SYNOPSIS
! Store calibration information for B1U data.
! SOURCE
! 
  TYPE b1u_calinfo
     SEQUENCE
     integer(kind=4) :: cver
     integer(kind=4), allocatable, dimension(:,:,:) :: ctable
  END TYPE b1u_calinfo
!***
!****s* kmod_isccp/b1u_tgrinfo
! NAME
! b1u_tgrinfo
! SYNOPSIS
! Store test grid information for B1U data.
! NOTES
! JMA, CMA and early GOES files provided test grids which provide
! lat/lon pairings for a fixed grid on the earth. In particular,
! the JMA and CMA files were used to calculate orbit and attitude 
! information for the Kepler navigation.
! SOURCE
! 
  TYPE b1u_tgrinfo
     sequence
     integer(kind=4) :: nrows
     integer(kind=4) :: ncols
     integer(kind=4) :: lat1
     integer(kind=4) :: lon1
     integer(kind=4) :: dlat
     integer(kind=4) :: dlon
     integer(kind=4), allocatable, dimension(:,:,:) :: grid
  END TYPE b1u_tgrinfo
!***
!****s* kmod_isccp/b1u_scanline
! NAME
! b1u_scanline
! SYNOPSIS
! Store information on the scan line and scan header.
! SOURCE
! 
  TYPE b1u_scanline
     sequence
     integer(kind=2), allocatable,dimension(:,:,:) :: prefix
     integer(kind=2), allocatable, dimension(:,:,:) :: scan
  END TYPE b1u_scanline
!***
!****s* kmod_isccp/b1u_gvar
! NAME
! b1u_gvar
! SYNOPSIS
! Store gvar information in the b1u header.
! SOURCE
! 
  TYPE b1u_gvar
     SEQUENCE
     type(gvar_nav) :: b1unav
     type(gvar_area)  :: b1uarea
  END TYPE b1u_gvar
!***
!****s* kmod_isccp/b1u_data
! NAME
! b1u_data
! SYNOPSIS
! Data structure for ISCCP B1U data.
! SOURCE
!
  TYPE b1u_data
     sequence
     character(len=72), dimension(0:5) :: rev
     integer(kind=4), dimension(0:19) :: img
     type(b1u_satinfo) :: sat
     integer(kind=4), dimension(0:199) :: nav
     type(b1u_calinfo) :: cal
     type(b1u_tgrinfo) :: tgr
     integer(kind=1), allocatable, dimension(:) :: fhdr
     type(b1u_scanline) :: image
     type(b1u_gvar) :: nav_gvar
  END TYPE b1u_data
!***

  !ISCCP B2 Data
  TYPE isccp_b2_noa_visinfo
     SEQUENCE
     integer(kind=4) :: solar_filter
     integer(kind=4) :: equiv_width
  END TYPE isccp_b2_noa_visinfo
  TYPE isccp_b2_noa_irinfo
     SEQUENCE
     integer(kind=4) :: wavenumber
     integer(kind=4) :: c1
     integer(kind=4) :: c2
  END TYPE isccp_b2_noa_irinfo
  TYPE isccp_b2_noa_hdrrec
     SEQUENCE
     integer(kind=4) :: SatID
     integer(kind=4) :: NChan    !# of channels on sensor
     integer(kind=4) :: NChanAct !# of active channels
     integer(kind=4) :: zero1    !Spare1
     integer(kind=4), dimension(12) :: Spectral !Spectral bandwidths
     type(isccp_b2_noa_visinfo) :: cal_ch1
     type(isccp_b2_noa_visinfo) :: cal_ch2
     type(isccp_b2_noa_visinfo) :: cal_ch3a
     type(isccp_b2_noa_irinfo)  :: cal_ch3b
     type(isccp_b2_noa_irinfo)  :: cal_ch4
     type(isccp_b2_noa_irinfo)  :: cal_ch5
     real(kind=4), dimension(3) :: rcc_a, rcc_b, rcc_d, rcc_rsp !48 bytes used 
     !                                         track nonlinearity
     integer(kind=4), dimension(2611-48) :: zero2 !spare
  END TYPE isccp_b2_noa_hdrrec
  TYPE isccp_b2_noa_date
     SEQUENCE
     integer(kind=2) :: Year !4-digit year
     integer(kind=2) :: Day  !Day of year
     integer(kind=4) :: Time !UTC time of day in miliseconds
  END TYPE isccp_b2_noa_date
  TYPE isccp_b2_noa_phyrec
     SEQUENCE
     integer(kind=2) :: seq
     character(len=1) :: ascii
     character(len=1) :: SatID
     type(isccp_b2_noa_date) :: time_scan1
     type(isccp_b2_noa_date) :: time_create
     character(len=42) :: name
     integer(kind=1), dimension(10) :: blank
  END TYPE isccp_b2_noa_phyrec
  TYPE isccp_b2_noa_latlon
     SEQUENCE
     integer(kind=4) :: lat
     integer(kind=4) :: lon
  END TYPE isccp_b2_noa_latlon
  TYPE isccp_b2_noa_vscal
     SEQUENCE
     integer(kind=4) :: slope1
     integer(kind=4) :: int1
     integer(kind=4) :: slope2
     integer(kind=4) :: int2
     integer(kind=4) :: intersect
  END TYPE isccp_b2_noa_vscal
  TYPE isccp_b2_noa_ircal
     SEQUENCE
     integer(kind=4) :: coef1
     integer(kind=4) :: coef2
     integer(kind=4) :: coef3
  END TYPE isccp_b2_noa_ircal
  TYPE isccp_b2_noa_logrec
     SEQUENCE
     integer(kind=2) :: seq
     type (isccp_b2_noa_date) :: time_scan
     integer(kind=2) :: scanline_bit
     integer(kind=4) :: quality_bit
     integer(kind=4) :: scanline_flag
     integer(kind=2),dimension(3) :: calib_bit
     type(isccp_b2_noa_vscal) :: cal_ch1
     type(isccp_b2_noa_vscal) :: cal_ch2
     type(isccp_b2_noa_vscal) :: cal_ch3a
     type(isccp_b2_noa_ircal) :: cal_ch3b
     type(isccp_b2_noa_ircal) :: cal_ch4
     type(isccp_b2_noa_ircal) :: cal_ch5
     integer(kind=2), dimension(17) :: sza
     type(isccp_b2_noa_latlon), dimension(17) :: loc
     integer(kind=4), dimension(3) :: zero
  END TYPE isccp_b2_noa_logrec

  TYPE isccp_b2_noa_datrec
     SEQUENCE
     integer(kind=1), dimension(5,65) :: scan
     integer(kind=1), dimension(27) :: zero
  END TYPE isccp_b2_noa_datrec

  TYPE isccp_b2_noa_oldchinfo
     SEQUENCE
     integer(kind=4) :: lower, upper
  END TYPE isccp_b2_noa_oldchinfo
  TYPE isccp_b2_noa_oldhdr
     SEQUENCE
     integer(kind=4) :: SatCode, Nchan
     integer(kind=1), dimension(8) :: zero
     type(isccp_b2_noa_oldchinfo), dimension(5) :: ChInfo
  END TYPE isccp_b2_noa_oldhdr
  TYPE isccp_b2_noa_oldprh
     SEQUENCE
     integer(kind=2) :: SEQ
     integer(kind=1) :: Blank1, SatID
     integer(kind=2) :: Blank2
     integer(kind=1),dimension(6) :: Time_Scan
     integer(kind=2) :: Blank3
     integer(kind=1),dimension(6) :: Time_Made
     character(len=42) :: name
     integer(kind=1), dimension(10) :: Blank4
  END TYPE isccp_b2_noa_oldprh
  TYPE isccp_b2_noa_oldlrh
     SEQUENCE
     integer(kind=2) :: SEQ
     integer(kind=1),dimension(6) :: Time_Scan
     integer(kind=4) :: Qual
     integer(kind=4), dimension(2,5) :: Cal
     integer(kind=1), dimension(17) :: SZA
     integer(kind=1), dimension(3) :: Blank1
     integer(kind=2), dimension(2,17) :: Loc
     integer(kind=1), dimension(12) :: Blank2
  END TYPE isccp_b2_noa_oldlrh
  INTERFACE b1u_ij2ll
     MODULE PROCEDURE b1u_ij2ll_kepler_scalar
     MODULE PROCEDURE b1u_ij2ll_kepler_rank1
     MODULE PROCEDURE b1u_ij2ll_kepler_rank2
     MODULE PROCEDURE b1u_ij2ll_gvar_scalar
     MODULE PROCEDURE b1u_ij2ll_gvar_rank1
     MODULE PROCEDURE b1u_ij2ll_gvar_rank2
  END INTERFACE
  INTERFACE b1u_ll2ij
     MODULE PROCEDURE b1u_ll2ij_kepler_scalar
     MODULE PROCEDURE b1u_ll2ij_kepler_rank1
     MODULE PROCEDURE b1u_ll2ij_kepler_rank2
     MODULE PROCEDURE b1u_ll2ij_gvar_scalar
     MODULE PROCEDURE b1u_ll2ij_gvar_rank1
     MODULE PROCEDURE b1u_ll2ij_gvar_rank2
  END INTERFACE
  INTERFACE view_zenith_correct_isccp
     MODULE PROCEDURE vzci_rank1
     MODULE PROCEDURE vzci_rank2
     MODULE PROCEDURE vzci_scalar
  END INTERFACE

contains 
!#######################################################################
!                                                        BEGIN B1GETFILE
!****f* kmod_isccp/b1getfile
! NAME
! b1getfile
! SYNOPSIS
! file1 = b1getfile(b1file,dtime)
! FUNCTION
! Find the B1U file based on the time of the file B1FILE and
! a change in time DTIME. This is useful for comparing observations 
! at the same time on different days.
! INPUTS
! b1file - char(len=*) - full path to the B1U FILE
! dtime - real*4 - difference in time in days
! OUTPUT
! file1 - char(len=120) - Name of the B1U file dtime days
!         away from B1FILE, or 'MISSING' if none found.
! USES
! caldat, julday
!***
  character(len=120) function b1getfile(file,dtime)
    !External variables
    character(len=*), intent(in):: file
    real(kind=4), intent(in)     ::   dtime !in days
    !Internal variables
    character(len=120), dimension(:),allocatable :: f,path
    integer(kind=4) ::  n, i, np, it
    integer mon, day, year, hh, mm
    integer omon, oday, oyear, ohh, omm, ohhmm
    real ss, oss
    real(kind=8)  ojday
    logical there

    ! Get Date from filename
    call strsplit(file,'/',path,np)
    call strsplit(trim(path(np)),'.',f,n)
    read (f(5),*) oyear
    read (f(6),*) omon
    read (f(7),*) oday
    read (f(8),*) ohhmm
    ohh = ohhmm / 100
    omm = mod(ohhmm,100)
    oss=30.0
    ss = 30.0
    !Adjust to new time
    ojday = julday(omon,oday,oyear,ohh,omm,oss)

    !Get new file
    call caldat(ojday+dtime,mon,day,year,hh,mm,ss)

    !Check new filename by only changing the dates (same HHMM)
    write (f(5),'(i4.4)') year
    write (f(6),'(i2.2)') mon
    write (f(7),'(i2.2)') day
    write (f(8),'(i4.4)') hh*100+mm
    write (path(np),'(20(a,1x))') (trim(f(i)),i=1,n)
 !   write (path(np-1),'(i4.4)') year
    do i=1,len_trim(path(np))
       if (path(np)(i:i) == ' ') path(np)(i:i) = '.'
    enddo
    write (b1getfile,'(20(a,1x))') (trim(path(i)),i=1,np)
    do i=1,len_trim(b1getfile)
       if (b1getfile(i:i) == ' ') b1getfile(i:i) = '/'
    enddo
    if (file(1:1) == '/') b1getfile = '/'//b1getfile
       
    !If the file exists, then return
    inquire (file=trim(b1getfile),exist=there)
    if (there) return
    !If not, search DTIME +/- 1 hour for the file 
    do it=0,120
       call caldat(ojday+dtime*1.d0+dble(it-60)/60.d0/24.d0,mon,day,year,hh,mm,ss)
       write (f(5),'(i4.4)') year
       write (f(6),'(i2.2)') mon
       write (f(7),'(i2.2)') day
       write (f(8),'(i4.4)') hh*100+mm
       write (path(np),'(20(a,1x))') (trim(f(i)),i=1,n)
!       write (path(np-1),'(i4.4)') year
       do i=1,len_trim(path(np))
          if (path(np)(i:i) == ' ') path(np)(i:i) = '.'
       enddo
       write (b1getfile,'(20(a,1x))') (trim(path(i)),i=1,np)
       do i=1,len_trim(b1getfile)
          if (b1getfile(i:i) == ' ') b1getfile(i:i) = '/'
       enddo
       if (file(1:1) == '/') b1getfile = '/'//b1getfile
       inquire (file=trim(b1getfile),exist=there)
       if (there) return
    end do
    !If file still not found, error out and return missing
    print*,'-------------- File not found ---------------'
    print*,'File: ',file
    print*,'dtime:',dtime
    print*,'Tried:',trim(b1getfile)
    b1getfile = 'MISSING'
    stop
  end function b1getfile
!                                                          END B1GETFILE
!#######################################################################
!#######################################################################
!                                                          BEGIN B1UREAD
!****f* kmod_isccp/b1uread
! NAME
! b1uread
! SYNOPSIS
! istatus = b1uread(file, b1u)
! FUNCTION
! This reads in the b1u file and returns the data in the b1u structure.
! INPUTS
! file - char(len=*) - full path to the B1U file
! 
! OUTPUT
! B1U - type(b1u_data) - the B1U data returned in a structure
! istatus - i*4 - successful=1, failed=0
!
! NOTES
! Some of the parameters in b1u_data are allocatable. Here is how they are
! declared:
! *   allocate (b1u%sat%chname(nchan))
! *   allocate (b1u%sat%chinfo(nchan))
! *   allocate (b1u%fhdr(len_ob1))
! *   allocate (b1u%image%prefix(0:19,nchan,nscan))
! *   allocate (b1u%image%scan(nchan,nelem,nscan))
! *   allocate (b1u%cal%ctable(2,nchan,nbin))
!
! USES 
! swap_endian
! 
!***
  integer function b1uread(infile,b1u)
    !External variables
    character(len=*) infile
    type(b1u_data) ::  b1u
   !Internal variables
    character(len=120) :: file, unc_file
    integer(kind=4) :: key, nblp, datloc, nscan, nelem, nchan, nhead,&
         &             nbyte
    integer(kind=4), allocatable, dimension(:) :: btype, bstrt, blen

    integer(kind=4) :: i, j, k, ii
    integer(kind=2), dimension(20)   ::  inpre
    integer(kind=1), allocatable,dimension(:) :: inscan
    integer(kind=2), allocatable,dimension(:) :: inscan2
    integer(kind=4), allocatable, dimension(:) :: incal, gridin
    integer(kind=4), dimension(704) :: gvarin
    integer(kind=4) :: incver
    integer(kind=4) :: len_ob1, nb
    logical :: swapit, there, iszipped, tempfileflag
    !-------------------------------------------------------------
    swapit = .false.
    nb = 256
    !how to define b1u
    b1uread = 0

    file=infile

    !open file
!    print*,trim(file)
    inquire (file=trim(file),exist = there)
    if (there) then
    else
       print*,'Missing file: ',trim(file)
       stop
    endif

    !Check for gzipped files
    iszipped=is_zipped(file)

    if (iszipped) then
12     unc_file = '/tmp/UncompressedB1uFile.'//trim(Random_String(8))
       inquire(file=trim(unc_file),exist=tempfileflag)
       if (tempfileflag) goto 12
       call system('gunzip -c '//trim(file)//' > '//trim(unc_file))
       file = unc_file
    endif

    open (unit=1,file=trim(file),form='binary')
    !
    !read the file information block
    !
    read (1) key, nblp, datloc, nscan, nelem, nchan, nhead
    if (key /= 1146433858) then 
       swapit = .true.
       key = swap_endian(key)
       if (key /= 1146433858) then 
          print*,'ERROR: File is NOT a B1U file'
          print*,'FILE: ',trim(infile)
          stop
       end if
       nblp = swap_endian(nblp)
       datloc = swap_endian(datloc)
       nscan  = swap_endian(nscan)
       nelem  = swap_endian(nelem)
       nchan  = swap_endian(nchan)
       nhead  = swap_endian(nhead)
    endif
    ! allocate the header arrays
    allocate (btype(nhead), bstrt(nhead), blen(nhead))
    do i=1,nhead
       read (1) btype(i), bstrt(i), blen(i)
       if (swapit) then
          btype(i) = swap_endian(btype(i))
          bstrt(i) = swap_endian(bstrt(i))
          blen(i)  = swap_endian(blen(i))
       end if
       if (btype(i) == 7) len_ob1 = blen(i)
    enddo
    
    !
    !allocate arrays
    if (allocated(b1u%sat%chname))  deallocate(b1u%sat%chname)
    if (allocated(b1u%sat%chinfo))  deallocate (b1u%sat%chinfo)
    if (allocated(b1u%fhdr))        deallocate (b1u%fhdr)
    if (allocated(b1u%image%prefix))deallocate (b1u%image%prefix)
    if (allocated(b1u%image%scan))  deallocate (b1u%image%scan)
    if (allocated(b1u%cal%ctable))  deallocate (b1u%cal%ctable)
    if (allocated(incal))           deallocate (incal)
    if (allocated(inscan))          deallocate (inscan)
    if (allocated(inscan2))          deallocate (inscan2)

    allocate (b1u%sat%chname(nchan))
    allocate (b1u%sat%chinfo(nchan))
    allocate (b1u%fhdr(len_ob1))
    allocate (b1u%image%prefix(0:19,nchan,nscan))
    allocate (b1u%image%scan(nchan,nelem,nscan))
    allocate (inscan(nelem))
    allocate (inscan2(nelem))
    !
    ! Loop over file headers and input them as needed
    do i=1,nhead
!print*,btype(i), bstrt(i),blen(i)
       call point_lun(1,bstrt(i))
       select case (btype(i))
       case (1)
          read (1) b1u%rev
       case (2)
          read (1) b1u%img
          if (swapit) b1u%img = swap_endian(b1u%img)
          nbyte = b1u%img(6)
          if (b1u%img(14) == 1024) nb = 1024
          allocate (b1u%cal%ctable(2,nchan,nb))
          allocate (incal(2*nchan*nb))
       case (3)
          read (1) b1u%sat%satnam, &
                   b1u%sat%sennam
          do j=1,nchan
             read (1) b1u%sat%chname(j)
          enddo
          do j=1,nchan
             read (1) b1u%sat%chinfo(j)
          enddo
       case (4)
          read (1) b1u%nav
          if (swapit) b1u%nav = swap_endian(b1u%nav)
       case (5)  ! Calibration header
          read (1) incver,incal !b1u%cal
          if (swapit) then
             b1u%cal%cver = swap_endian(incver)
             b1u%cal%ctable = reshape(swap_endian(incal), (/2,nchan,nb/) )
          else
             b1u%cal%cver = incver
             b1u%cal%ctable = reshape(incal, (/2,nchan,nb/) )
          endif
       case (6)  ! Test grid
          read (1) b1u%tgr%nrows,  b1u%tgr%ncols,  b1u%tgr%lat1,  &
                   b1u%tgr%lon1,   b1u%tgr%dlat,   b1u%tgr%dlon
          allocate (gridin(2 * b1u%tgr%nrows * b1u%tgr%ncols))
          read (1) gridin
          if (swapit) then
             b1u%tgr%nrows = swap_endian(b1u%tgr%nrows)
             b1u%tgr%ncols = swap_endian(b1u%tgr%ncols)
             b1u%tgr%lat1  = swap_endian(b1u%tgr%lat1)
             b1u%tgr%lon1  = swap_endian(b1u%tgr%lon1)
             b1u%tgr%dlat  = swap_endian(b1u%tgr%dlat)
             b1u%tgr%dlon  = swap_endian(b1u%tgr%dlon)
             gridin = swap_endian(gridin)
          endif
          if (allocated(b1u%tgr%grid)) deallocate(b1u%tgr%grid)
          allocate (b1u%tgr%grid(2,b1u%tgr%nrows,b1u%tgr%ncols))
          b1u%tgr%grid = reshape(gridin, (/2,b1u%tgr%nrows,b1u%tgr%ncols/))
          deallocate (gridin)
!          read (1) b1u%tgr%grid
       case (7)  ! Original B1 header
          read (1) b1u%fhdr
       case (8)  ! Prefix and Images
          do j=1,nscan
             do k=1,nchan
                if (nbyte == 1) then
                   read (1) inpre,inscan
                   b1u%image%scan(k,:,j) = inscan
                   do ii=1,nelem
                      if (b1u%image%scan(k,ii,j) < 0) then
                         b1u%image%scan(k,ii,j) = 256 + b1u%image%scan(k,ii,j)
                      endif
                      if (b1u%image%scan(k,ii,j) < 0) stop
                   enddo
                end if
                if (nbyte == 2) then
                   read (1) inpre,inscan2
                   if (swapit) inscan2=swap_endian(inscan2)
                   b1u%image%scan(k,:,j) = inscan2
                end if
                if (swapit) inpre = swap_endian(inpre)
                b1u%image%prefix(:,k,j) = inpre
             enddo
          enddo
       case (9)  ! GVAR Navigation information
          if (swapit) then
             read (1) gvarin
             open (unit=15,file='DeleteThisFile',form='binary')
             write (15) swap_endian(gvarin)
             close (15)
             open (unit=15,file='DeleteThisFile',form='binary')
             read  (15) b1u%nav_gvar
             close (15)
          else
             read (1) b1u%nav_gvar
          endif
       end select
    enddo
    close(1)
    b1uread = 1
    if (iszipped) then
       call system('rm '//trim(unc_file))
    endif


  end function b1uread
!                                                            END B1UREAD
!#######################################################################
!#######################################################################
!                                                BEGIN B1U_CHANNEL_INDEX
!****f* kmod_isccp/b1u_channel_index
! NAME
! b1u_channel_index
! SYNOPSIS
! index = b1u_channel_index(b1u.sat.chname,channel)
! FUNCTION
! Returns the index of the location where CHNAME equals
! the requested channel.
! INPUTS
! * b1u.sat.chname - list of channel names provided by b1uread
!   * char(len=6), dimension (:)
!   * see b1u_data, b1u_satinfo
! * channel - char(len=*)
! OUTPUT
! * index - integer*4 - range: [1,nchan]
! NOTES
! * returns -1 if channel non-existant in the b1u information provided.
!***

  integer (kind=4) function b1u_channel_index( chname, channel )
    !External variables
    character(len=6), intent(in),dimension(:) :: chname
    character(len=*), intent(in) :: channel
    !Internal variables
    integer(kind=4) :: n, i
    n = size(chname)
    do i=1,n
       if (trim(adjustl(chname(i))) == trim(adjustl(channel))) exit
    end do
    if (i > n) then
       b1u_channel_index = -1
    else
       b1u_channel_index = i
    end if
  end function b1u_channel_index
!                                                  END B1U_CHANNEL_INDEX
!#######################################################################
!#######################################################################
!                                                       BEGIN B1U_DN2CAL
  subroutine b1u_dn2cal ( channels, b1u, temp, order )
!****r* kmod_isccp/b1u_dn2cal
! NAME
! b1u_dn2cal
! SYNOPSIS
! call b1u_dn2cal(channels, b1u, temp, order)
! FUNCTION
! Applies the calibration tables stored in the b1u structure
! INPUTS
! channels - char(len=*)(:) - Channel names (IRWIN, IRWVP, ...) to be calibrated
! b1u      - b1u_data - b1u data from the b1uread algorithm
! OUTPUT
! 
! NOTES
! Incomplete documentation.
! USES
!***
    !Externals
    character(len=*), dimension(:), intent(in)    :: channels
    type(b1u_data), intent(in)                    :: b1u
    real(kind=4), dimension(:,:,:), intent(inout) :: temp
    integer(kind=4), dimension(size(temp,1)), optional :: order
    !internals
    integer(kind=4) :: scale, off
    integer(kind=4) :: nc, ic
    integer(kind=4) :: nx, ny, nch, iir, nb, year, time
    integer(kind=2), dimension(size(temp,2),size(temp,3)) :: img
    real(kind=4), dimension(size(temp,2),size(temp,3))    :: atemp
    logical, dimension(size(temp,2),size(temp,3))         :: mask
    real(kind=4), allocatable, dimension(:,:,:)   :: caltable
    real(kind=4) :: mxt, pval, med, upfrac
    real(kind=4), dimension(4) :: mom
    character(len=120) :: filename
    character(len=3)   :: sat
    character(len=120), allocatable, dimension(:) :: f
    integer :: nf

    integer(kind=4) :: i, j, k
    integer :: nny, x1
    real :: mn, sdd
    real(kind=4), dimension(:),allocatable :: sp, sd, scan1, sp_

    integer(kind=4), dimension(:), allocatable :: ind, s, ss
    integer(kind=4) :: nok, nnok

    !input channel information
    nc = size(channels)

    if (present(order)) order(:) = -1

    !b1u data
    nx = b1u%img(4)
    ny = b1u%img(3)
    nch= b1u%img(5)
    nb = 256
    if (b1u%img(14) == 1024) nb = 1024

    nb=size(b1u%cal%ctable,3)
    allocate (caltable(2,nch,nb))
    do i=1,2
       do j=1,nch
          do k=1,nb
             caltable(i,j,k) = b1u%cal%ctable(i,j,k) / 1000.
          end do
       end do
    end do

    !Skip the IR stuff if only the visible channel is requested.
    if (nc == 1 .and. trim(adjustl(channels(1))) == 'VSCHN') goto 113

    !Default values of scale and offset
    off = nb-1
    scale = -1

    !Determing orientation of the IRWIN channel
    iir = b1u_channel_index(b1u%sat%chname,'IRWIN')
    print*,'IRWIN:',iir,'    CVER=', b1u%cal%cver
    if (iir > nch .or. iir < 1) then
       print*,'No IRWIN channel found.'
       print*,'CHNAME:',b1u%sat%chname
       stop
    endif

    !Calibrate the IRWIN channel
    img(:,:) = b1u%image%scan(iir,:,:)

    filename=b1u%rev(0)
    do i=1,120
       if (filename(i:i+1) == 'B1') then
          sat = filename(i+5:i+7)
          exit
       endif
    enddo
    call strsplit(trim(filename),'.',f,nf)
    read ((f(5)),'(i4)') year
    read ((f(8)),'(i4)') time


!   -20< cver <-1 means the calibration table is not from ISCCP
!   so it can be used without any modification
!   MTS=-2
!   MSG=-3
!   GMS=-4
!   [spc] NOA=-5  (GOES files produced by NCDC)
!   MET=-6
!   GOE=-7
!   FY2=-8
    if (b1u%cal%cver > (-20) .and. b1u%cal%cver < (-1)) then
       off   = 0
       scale = 1
       goto 113
    endif


    if (sat == 'GMS' .and. year < 1983) then
       off   = 0
       scale = 1
       goto 113
    endif
!!$ This is likely a wrong statement ... not sure why it was coded this way.
!!$    if (sat == 'GOE' .and. b1u%cal%cver == (-1)) then
!!$       off   = 0
!!$       scale = 1
!!$       goto 113
!!$    endif

    !MTS with ISCCP calibration should be oriented using ..
    if (sat == 'MTS' .and. b1u%cal%cver == (-1)) then
       off   = 0
       scale = 1
       goto 113
    endif

!!**CSU makes a large image of 1023 with the real values as a small subsector
!!**My calibration algorithm likely switches the table based on the 1023 values
!!**If (GOES-11 and 2006 and time=0800/1000) then off=nb-1 and slope=-1

if (trim(adjustl(f(4))) == 'GOE-11' .and. year == 2006 .and. &
    (time == 1000 .or. time == 800)) then 
    off = nb-1
    scale = -1.
    goto 113   !,calall
endif


!!$ The following no longer applies for MSG as it now has ISCCP
!!$ calibration, so sometimes needs to be inverted.
!!$    if (sat == 'MSG') then
!!$       off   = 0
!!$       scale = 1
!!$       goto 113
!!$    endif


    if (sat == 'GOE') then
       !;for GOES satellites ... the space values are never zeroed out,
       !;so if we can determine what space is, then we know the orientation
       !;first try to get center
       nny = ny-41
       x1=10
114    continue
       !deallocate (sp, sd, scan1, s, sp_, ss)
       mom = idl_moment(img(x1:x1+10,10:10+nny))
       mn = mom(1)
       sdd= mom(2)
       if (sdd > 10) then
          !if the std dev is large, then we need to look at portions of the 
          !box to find locally smooth values
          if (allocated(sp))    deallocate (sp)
          if (allocated(sd))    deallocate (sd)
          if (allocated(scan1)) deallocate (scan1)
          allocate (sp(nny), sd(nny), scan1(nny))
          sd(:) = 100.
          do i=1,nny
             mom=idl_moment(img(x1:x1+10,10+i:10+i+3))
             sp(i) = mom(1)
             sd(i) = mom(2)
             scan1(i) = sum(img(:,10+i:10+i+3))
          end do
          !use sd < 10 and scan > 0
          call idl_where(sd < 10. .and. scan1 > 1,ind,nok)
          if (nok > 0) then
             if (allocated(s))     deallocate (s)
             allocate (s(nok))
             !sort sd
             call qsort(sd(ind),s,nok)
             !get the lowest 20% of the std dev's
             nnok = nok * 0.2
             !the grab the darkest points with the lowest std dev's
             if (allocated(sp_))   deallocate (sp_)
             if (allocated(ss))    deallocate (ss)
             allocate (sp_(nnok), ss(nnok))
             sp_(:) = sp(ind(s(1:nnok)))
             call qsort(sp_,ss,nnok)
             mn = sp_(ss(within(nint(nnok*0.15),minimum=1)))
          else
             if (x1 < 35) then
                x1=x1+10
                goto 114
             else
                mom=idl_moment(sp)
                if (mom(2) < 1) then
                   mn = mom(1)
                else
                   print*,'Uh Oh. Calibration went awry when trying to find space value'
                   stop
                endif
             endif
          endif
       endif
!print*,'MN:',mn
       !if the std deviation is low, then use the 
       !value of this block as the space value
       if (mn > 200) then
          off = 0
          scale = 1
       else
          off = nb-1
          scale = -1
       endif
       goto 113
    end if
 

    do i=1,nx
       do j=1,ny
          atemp(i,j) = caltable(2,iir,(off+scale*b1u%image%scan(iir,i,j))+1)
       end do
    end do
    mxt = maxval(caltable(2,iir,:))

    !Return missing values if most of the imge is missing
    mask = atemp > 200 .and. atemp < (mxt-1.)
    pval = (count(b1u%image%scan(iir,:,:) == 0 .or. &
         &        b1u%image%scan(iir,:,:) == (nb-1)))/ &
         &                 size(b1u%image%scan(iir,:,:))
    if ( count(mask) < 10000 .or.pval > 0.90) then
       temp = -999.0
       return
    endif

    !Determine moments of IRWIN
    mom = idl_moment(atemp,mask=mask)

    !instead of calculating the median, just figure out which side 
    !of the mean it lies on
    upfrac=real(count(atemp > mom(1) .and. mask))/real(count(mask))
    !determine orientation of IRWIN
    if (upfrac > 0.5 .and. mom(3) < 0) then
       !this situation is normal for IRWIN temperatures over the Earth
    else
       if (upfrac < 0.5 .and. mom(3) > 0) then
          !OK, non-normal situation
          !reverse caltable and see if it becomes normal
          off = 0
          scale = 1
          do i=1,nx
             do j=1,ny
                atemp(i,j) = caltable(2,iir,(off+scale*b1u%image%scan(iir,i,j))+1)
             end do
          end do
          mask = atemp > 200 .and. atemp < (mxt-1.)
          mom = idl_moment(atemp,mask=mask)

          upfrac=real(count(atemp > mom(1) .and. mask))/real(count(mask))
          if (upfrac > 0.5 .and. mom(3) < 0) then
             !this situation is normal for IRWIN temperatures over the Earth
          else
             !neither orientation works
             print*,'Error in B1U_DN2CAL: ERROR 1'
             print*,'Moment',mom
             print*,'upfrac',upfrac
             stop
          endif
       else
          !The initial orientation is unknown
          print*,'Error in B1U_DN2CAL: ERROR 2'
          print*,'Moment',mom
          print*,'upfrac',upfrac
          stop
       endif
    endif
    !Now that the SCALE and OFF are known (i.e., the orientation
    !of the IR imagery), calibrate the requested channels.
113 continue
print*,'Calibration: Scale and Offset: ',scale,off

    do ic=1,nc
       atemp = -1.
       iir = b1u_channel_index(b1u%sat%chname,channels(ic))
       if (present(order)) then
          order(ic) = iir
       end if
       !Removed the condition that img(i,j) < 255 since future sats will have 10-bit values
       if (iir > 0 .and. iir <= nch) then
          if (trim(adjustl(channels(ic))) == 'VSCHN') then
             do i=1,nx
                do j=1,ny
                   atemp(i,j) = caltable(2,iir,&
                        &       within(b1u%image%scan(iir,i,j)+1,1,nb))
                end do
             end do
          else
             do i=1,nx
                do j=1,ny
                   atemp(i,j) = caltable(2,iir,&
                        &       within((b1u%image%scan(iir,i,j)*scale+off)+1,1,nb))
                end do
             end do
          end if
       end if
       temp(ic,:,:) = atemp
    end do
    return
  end subroutine b1u_dn2cal
!                                                         END B1U_DN2CAL
!#######################################################################
!#######################################################################
!                                                   BEGIN B1U_GET_ANGLES
!****r* kmod_isccp/b1u_get_angles
! NAME
! b1u_get_angles
! SYNOPSIS
! call b1u_get_angles( b1u, lat, lon, vza, azi, sza, azn, subsat )
! FUNCTION
! This routine calculates viewing an illumination angles based on
! the b1u structure and lat/lon's provided.
! INPUTS
! b1u - type(b1u_data) - b1u information provided by b1uread
! lat - r*4 - rank2 - latitude 
! lon - r*4 - rank2 - longitude
! 
! OUTPUT
! subsat(3) - sub-satellite [lat(deg),lon(deg),radius(km)]
! - The remaining outputs are optional, real*4 rank2 allocatable arrays
! vza - view zenith angle (deg)
! azi - view azimuth angle (deg) (0 to 360, clockwise from North)
! sza - solar zenith angle at time of observation (deg)
! azn - solar azimuth angle (deg) (0 to 360, clockwise from North)
!
! NOTES
! See Enchilada for units of lat/lon
! USES 
! b1u_ij2ll
! solar_angle
! 
!***

  subroutine b1u_get_angles ( b1u, lat, lon, vza, azi, sza, azn, subsat )
    !Externals
    type(b1u_data), intent(in) :: b1u
    real(kind=4), allocatable, dimension(:,:), intent(inout) :: lat 
    real(kind=4), allocatable, dimension(:,:), intent(inout) :: lon
    real(kind=4), allocatable, dimension(:,:), intent(inout), optional :: vza 
    real(kind=4), allocatable, dimension(:,:), intent(inout), optional :: azi 
    real(kind=4), allocatable, dimension(:,:), intent(inout), optional :: sza 
    real(kind=4), allocatable, dimension(:,:), intent(inout), optional :: azn 
    real(kind=4), dimension(3), intent(inout), optional :: subsat
    !internals
    integer (kind=4) :: nx, ny
    integer (kind=4), allocatable, dimension(:,:) :: i, j
    integer ii, jj
    real(kind=4), dimension(3) :: subs

    !Deallocate variables if present
    if (allocated(lat)) deallocate(lat)
    if (allocated(lon)) deallocate(lon)
    
    if (present(vza)) then
       if (allocated(vza)) deallocate (vza)
    endif
    if (present(azi)) then
       if (allocated(azi)) deallocate (azi)
    endif
    if (present(sza)) then
       if (allocated(sza)) deallocate (sza)
    endif
    if (allocated(azn)) deallocate (azn)
    
    if (allocated(i)) deallocate(i)
    if (allocated(j)) deallocate(j)
    
    !Set up parameters and navigate the imagery
    nx = b1u%img(4)
    ny = b1u%img(3)
    
    allocate (i(nx,ny), j(nx,ny))
    do jj=1,ny
       j(:,jj) = jj
    end do
    do ii=1,nx
       i(ii,:) = ii
    end do

    allocate (lat(nx,ny), lon(nx,ny))
    if (present(vza)) then
       allocate (vza(nx,ny))
       if (b1u%img(13) == 1) then
          call b1u_ij2ll(real(i), real(j), lat, lon, b1u%nav_gvar, subsat=subs, vza=vza)
       else
          call b1u_ij2ll(real(i), real(j), lat, lon, b1u%nav, subsat=subs,vza=vza)
       endif
    else
       if (b1u%img(13) == 1) then
          call b1u_ij2ll(real(i), real(j), lat, lon, b1u%nav_gvar, subsat=subs)
       else
          call b1u_ij2ll(real(i), real(j), lat, lon, b1u%nav, subsat=subs)
       endif
    end if
    !Get view angles
    if (present(azi)) then
       allocate(azi(nx,ny))
       azi(:,:) = -400.
    end if
    if (present(sza) .or. present(azn)) then
       allocate (sza(nx,ny))
       allocate (azn(nx,ny))
       sza(:,:) = -1.
       azn(:,:) = -400.
    end if

    if (present(azi)) then
       azi = azimuth(lat, lon,subs(1), subs(2))
    end if
    if (present(sza) .or. present(azn)) then 
       call solar_angle(lat,lon,real(j),b1u%img,b1u%image%prefix(:,1,:),sza,azn)
    end if

    deallocate (i, j)

    if (present(subsat)) subsat = subs

  end subroutine b1u_get_angles
!                                                   BEGIN B1U_GET_ANGLES
!#######################################################################
!#######################################################################
!                                                   BEGIN B1U_GET_CENTER
  subroutine b1u_get_center ( file, center )
!****r* kmod_isccp/b1u_get_center
! NAME
! b1u_get_center
! SYNOPSIS
! call b1u_get_center(file,center)
!
! FUNCTION
! To read in the derived B1U image center information (derived by
! idl.b1u_finalcenter).
! INPUTS
! * file - B1U filename (character(len=*))
! 
! OUTPUT
! * center : real(kind=4), dimension(13)
!   center(1:4) = binomial fit information
!   center(5:6) = range of the derived center pixel values
!   center(7)   = correction for the center scan angle
!   center(8:11)= moments of the on-earth IR distribution:
!                 mean, std. dev., skew, kurtosis
!   center(12)  = median of the on-earth IR distribution
!   center(13)  = average space count
!
! 
! NOTES
!***
    character(len=*), intent(in) :: file
    real(kind=4),     intent(inout), dimension(13) :: center
    character(len=120) :: cfile
    
    !If the file does not exist, then return -1. in center
    center(:) = -1.
    cfile=trim(remove_gz(file))//'.center'
    open(unit=17,file=trim(cfile),err=20,form='binary',status='old')
    read(17,err=20,end=20) center
    close(17)
    return
    !try reading the smaller data
20  print*,'Error reading center file:'
    print*,trim(cfile)
    
    center(:) = -99.

    return
    
  end subroutine b1u_get_center
!                                                     END B1U_GET_CENTER
!#######################################################################
!#######################################################################
!                                                       BEGIN B1U_GET_QC
!****f* kmod_isccp/b1u_get_qc
! NAME
! b1u_get_qc
!
! SYNOPSIS
! call b1u_get_qc(file, b1u, qc, overwrite, readit)
! FUNCTION
! This routine returns the quality information on a line-by-line
! basis for the scans in a B1U file. The data is read from the 
! QCQA file (if it's missing, then the QCQA info is calculated)
!
! INPUTS
! See source below for type information:
! * file - filename of the B1U file
! * b1u  - b1u data structure
!
! OUTPUT
! * qc   - QC info for each band/channel and scan line
!
! OPTIONS
! * overwrite - flag to overwrite QCQA file if it exists
! * readit    - flag to read the b1u data (i.e., b1u data not 
!               provided by parent routine)
! * debug     - Print debugging output
! USES
! b1uread, smooth
!
! SOURCE

  subroutine b1u_get_qc (file, b1u, qc, overwrite, readit, debug)
    !Externals
implicit none
    character(len=*), intent(in) ::  file
    type(b1u_data), intent(inout) :: b1u
    integer(kind=4), allocatable, dimension(:,:), intent(inout) :: qc
    integer(kind=4), optional,intent(in) :: overwrite, readit, debug
!***
    !Internals
    character(len=120) :: qcfile
    integer :: istat
    logical :: there, process, debugflag
    !image info
    integer :: nch, nx, ny, ich, i, j, tval
    !Stats
    integer, parameter :: nlines = 3000, nchans=12
    real(kind=4), dimension(nchans,nlines) :: lnmx, lnmn,&
         &        lnavg, lnstd, stm, sts, mnmn
    integer(kind=4), dimension(nchans,nlines) :: miss, flag
    integer(kind=4), dimension(nchans,nlines,nlines) :: img
    real(kind=4), dimension(nlines) :: sumx2
    character(len=120), dimension(:), allocatable:: f
    integer(kind=4) :: nf
    integer :: year, dom, hhmm
    real(kind=8) :: jdayimg,jdayline
    integer :: version, inversion, i1, month
    integer :: maxdn, num


    maxdn = 254
    VERSION = 1
    !VERSION 1
    !Uses 4 tests: 
    ! 1. Dropped scans
    ! 2. Faint lines
    ! 3. Duplicate scans
    ! 4. temporal inconsistancies
    
    process = .false.
    if (present(overwrite)) process = .true.
    if (allocated(qc)) deallocate(qc)

    debugflag = .false.
    if (present(debug)) then
       if (debug /= 0) debugflag = .true.
    endif
    
    !Find QC file and read if it exists
    !print*,'=',trim(file),'='
    qcfile=trim(remove_gz(file))//'.QCQA'
    !qcfile = trim(file)//'.QCQA'
    inquire (file=trim(qcfile),exist=there)
    if (there) then
       print*,'Found =',trim(qcfile),'='
    else
       print*,'Not Found =',trim(qcfile),'='
    endif
    if (.not. (there)) process = .true.
    if (there .and. .not. (process)) then
       open (unit=2,file=trim(qcfile),form='binary',err=22)
       !read in info
       read (2) nch, ny, inversion
       if (inversion /= version) goto 22
       allocate (qc(nch,ny))
       read (2,err=22) qc
       goto 23
22     process = .true.
23     close (2)
    end if
    if (process) then
       !Read b1u file if needed
       if (present(readit)) then
          istat = b1uread(trim(file),b1u)
          print*,b1u%img
          if (b1u%img(14) > 257) maxdn = 1023
       endif
       nch = b1u%img(5)
       ny = b1u%img(3)
       nx = b1u%img(4)
       if (nx > nlines) then
          print*,'NX is too large. NX=',nx
          print*,'Change parameter NLINES and recompile.'
          stop
       end if
       if (ny > nlines) then
          print*,'NY is too large. NY=',ny
          print*,'Change parameter NLINES and recompile.'
          stop
       end if
       if (nch > nchans) then
          print*,'NCH is too large. NCH=',NCH
          print*,'Change parameter NCHANS and recompile.'
          stop
       end if
       img(1:nch,1:nx,1:ny) = b1u%image%scan(1:nch,1:nx,1:ny)
       if (debugflag) print*,'TOTAL POINTS:',sum(img(1:nch,1:nx,1:ny))
       !Calculate statistics
       do ich=1,nch
          lnmn(ich,1:ny) = minval(img(ich,1:nx,1:ny),dim=1)
          lnmx(ich,1:ny) = maxval(img(ich,1:nx,1:ny),dim=1)
          lnavg(ich,1:ny)= sum(img(ich,1:nx,1:ny),dim=1)
          sumx2(1:ny) = sum(float(img(ich,1:nx,1:ny))**2,dim=1)
          !fixing sqrt(-dx) problem
          !removing instability
          !lnstd(ich,1:ny) = sqrt((nx*sumx2(1:ny)-lnavg(ich,1:ny)**2)/&
          !     &                 float(nx*(nx-1)))
          lnstd(ich,1:ny) = sqrt(within((sumx2(1:ny)-(lnavg(ich,1:ny)**2)/real(nx))&
               &                                              /real(nx-1),minimum=0.0))
          lnavg(ich,1:ny) = lnavg(ich,1:ny) / float(nx)
       end do
         
       allocate (qc(nch,ny))
       QC(1:nch,1:ny) = 0

       ! TEST 1: Tests for dropped scans
       tval = 0
       miss = 0
       miss(1:nch,1:ny) = eval(lnstd(1:nch,1:ny) < 0.05 .or. &
            &                  lnavg(1:nch,1:ny) < 0.05 .or. &
            &                  lnavg(1:nch,1:ny) > maxdn)
       qc(1:nch,1:ny) = qc(1:nch,1:ny) + miss(1:nch,1:ny)*2**tval
       if (debugflag) then 
          print*,'Lines passing Test 1: Dropped scans'
          print*,(count((flag(ich,1:ny)) == 0),ich=1,nch)
       endif
       
       ! TEST 2: Test for faint and/or noisy lines
       !; Method: This routine starts out by skipping lines which
       !;         are MISSed or whose step in mean or std devation
       !;         are too large. Beyond that, it calculates an
       !;         optimal line using binomial filtering and skipping
       !;         missing lines. Then resets the flag for bad scans
       !;         using a simple difference between the mean and the
       !;         filtered mean.
       !Calculate the steps in the line-mean and line-std
       sts = 0.
       stm = 0.
       do ich=1,nch
          do i=2,ny
             sts(ich,i) = abs(lnstd(ich,i) - lnstd(ich,i-1))
             stm(ich,i) = abs(lnavg(ich,i) - lnavg(ich,i-1))
          end do
       end do
       flag=1
       !flag=1 is good line, flag=0 is bad line
       flag(1:nch,1:ny) = eval(miss(1:nch,1:ny) == 0  .and. &
            &                   sts(1:nch,1:ny)  < 2. .and. &
            &                   stm(1:nch,1:ny)  < 4.)
       
       !The above initially conservative filter is a means to 
       !get the best estimate of the signal, it will be redefined 
       !below
       
       !Apply the following filter 4 times
       do j=1,4
          !Apply the binomial smoothing using flag to ignore
          !bad points
          do ich=1,nch
             mnmn(ich,1:ny) = smooth(lnavg(ich,1:ny),10,&
                  &                  flag=flag(ich,1:ny) == 1,&
                  &                  binomial=1)
          end do
          
          !Redefine the flag as being more than 3 counts away from
          !the binomial estimate. This second flag removes the 
          !dependence on the step size.
          !flag=1 is good line, flag=0 is bad line
          flag(1:nch,1:ny) = &
               & eval(abs(mnmn(1:nch,1:ny)-lnavg(1:nch,1:ny)) < 3)
       end do
       tval = 1
       qc(1:nch,1:ny) = qc(1:nch,1:ny) + (1-flag(1:nch,1:ny))*2**tval

       if (debugflag) then 
          print*,'Lines passing Test 2: Faint or noisy lines'
          print*,(count((1-flag(ich,1:ny)) == 0),ich=1,nch)
       endif
       ! TEST 3: Test for duplicate lines
       !FLAG=0 is OK, FLAG=1 means duplicate line
       flag(:,:) = 0
       miss(:,:) = 0
       !Sum the scan for each channel
       miss(1:nch,1:ny) = sum(img(1:nch,1:nx,1:ny),dim=2)
       !For each channel
       do ich=1,nch
          i1=1
          do i=2,ny
             !First check the sum to see if duplicate is possible
             if (miss(ich,i1) == miss(ich,i)) then
                if (all(img(ich,1:nx,i1) == img(ich,1:nx,i))) then
                   !Lines are identical
                   flag(ich,i) = 1
                else
                   !Lines may have same sum, but aren't identical
                   i1 = i
                end if
             else
                !If not possible, then lines aren't identical
                i1=i
             endif
          end do
       end do
       tval = 2
       qc(1:nch,1:ny) = qc(1:nch,1:ny) + (flag(1:nch,1:ny))*2**tval
       if (debugflag) then 
          print*,'Lines passing Test 3: Duplicate lines'
          print*,(count(flag(ich,1:ny) == 0),ich=1,nch)
       endif
       ! TEST 4: Test for temporal inconsistencies
       flag(:,:) = 0
       miss(:,:) = 0
       tval = 3
       !Get the time from the B1U filename
       call strsplit(trim(file),'.',f,nf)
       read ((f(5)),'(i4)') year
       read ((f(6)),'(i4)') month
       read ((f(7)),'(i4)') dom
       read ((f(8)),'(i4)') hhmm
       jdayimg = julday(month,dom,year,hhmm/100,mod(hhmm,100),0.)
       !Compare this time with the time of each scan line (if present)
       do ich=1,nch
          do i=1,ny
             jdayline = julday(1,&
                  &            int(b1u%image%prefix(4,ich,i)),&
                  &            max(int(b1u%image%prefix(3,ich,i)),1),&
                  &            int(b1u%image%prefix(5,ich,i)/100),&
                  &            mod(int(b1u%image%prefix(5,ich,i)),100),&
                  &            float(b1u%image%prefix(6,ich,i)) + &
                  &            float(b1u%image%prefix(7,ich,i))/1000.)
             
             if (abs(jdayimg-jdayline) > 0.5 .or. &
                  jdayline < (jdayimg-1./24.)) flag(ich,i) = 1
          end do
       end do
       if (debugflag) then 
          print*,'Lines passing Test 4: Temporal consistency'
          print*,(count(flag(ich,1:ny) == 0),ich=1,nch)          
       endif
       !turn off the mask when nearly every line is flagged as bad, 
       !this generally happens when the header info is corrupt.
       if (sum(flag(1:nch,1:ny)) < (nch*ny*0.9)) &
            & qc(1:nch,1:ny) = qc(1:nch,1:ny) + (flag(1:nch,1:ny))*2**tval
       
       !Output the results to a file
       
       if (debugflag) then 
          print*,'% Bad lines:'
          print*,(count(qc(ich,1:ny) /= 0) / real(ny)*100,ich=1,nch)
       endif
       if (debugflag) then 
          print*,'QC ALL LINES: OK (%OK)'
          do ich=1,nch
             num = count(qc(ich,1:ny) == 0)
             print*,ich,num,' (',num/float(ny)*100,') '
          enddo
       endif
       print*,'Writing =',trim(qcfile),'='
       open (unit=2,file=trim(qcfile),form='binary')
       write (2) nch, ny, version
       write (2) qc(1:nch,1:ny)
       close (2)
    end if
    !Return the results
  end subroutine b1u_get_qc
!                                                         END B1U_GET_QC
!#######################################################################
!#######################################################################
!                                                        BEGIN B1U_IJ2LL
!****r* kmos_isccp/b1u_ij2ll
! NAME
! b1u_ij2ll
! SYNOPSIS
! call b1u_ij2ll( ip, jp, lat, lon, b1ugvar, vza, subsat, debug, scanangle)
!       or
! call b1u_ij2ll( ip, jp, lat, lon, nav, vza, subsat, debug, scanangle)
! FUNCTION
!    To convert line/pixel coordinate point(s) of a satellite image to
!    latitude/longitude.
! INPUTS
!  * ip - Pixel locations (default is in image coordinates):
!      * r*4 - rank2
!      * r*4 - rank1
!      * r*4 - scalar
!  * jp - Line locations (default is in image coordinates):
!      * same type & rank as ip
!  * Navigation is taken from either:
!      *  NAV - B1U Navigation array (LONARR(200)) from b1uread
!      *  b1ugvar, which is type(b1u_gvar)
! 
! OUTPUT
!  * lat (degrees)
!  * lon  (degrees)
!  * Optional outputs:
!    * vza - optional:
!      * same type and rank as ip
!    * subsat - optional - r*4(3) - [lat(deg),lon(deg),radius(km)]
! OPTIONS
!    Optional arguments:
!    * debug - i*4 - set debug=1 for optional debugging output
!    * scanangle - i*4 - set to return scan and elevation angles (deg) 
!      instead of lat/lon values.
! USES 
! nav2rec, bcd_time, imcnav_r1, julday, kep2cart, cart2earth, sincos
! 
! NOTES
! * The GVAR Navigation calls imc_nav
! * The Kepler navigation (all non-GVAR satellites) is based on 
!   Kidder and Vonder Haar, 1995: Introduction to Satellite Meteorology
! * See Enchilada for units of lat/lon
!***
!                                                  b1u_ij2ll_gvar_scalar
  subroutine b1u_ij2ll_gvar_scalar (ipoints, jpoints, outlat, outlon , b1ugvar,&
       &                            vza, subsat, debug, scanangle, center)
    implicit none
    ! External variables
    real(kind=4), intent(in)  :: ipoints
    real(kind=4), intent(in)  :: jpoints
    real(kind=4), intent(out) :: outlat
    real(kind=4), intent(out) :: outlon
    type(b1u_gvar), intent(in):: b1ugvar
    real(kind=4), optional    :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    integer(kind=4), intent(in), optional :: scanangle
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal variables
    real(kind=4), dimension(1) :: lat,lon, v

    call b1u_ij2ll( (/ipoints/), (/jpoints/), lat, lon, b1ugvar, &
                   v, subsat, debug=debug, scanangle=scanangle, center=center )
    outlat = lat(1)
    outlon = lon(1)
    if (present(vza)) then
       vza = v(1)
    end if
  end subroutine b1u_ij2ll_gvar_scalar
!                                                  b1u_ij2ll_gvar_rank2
  subroutine b1u_ij2ll_gvar_rank2 (ipoints, jpoints, outlat, outlon , b1ugvar, &
       &                           vza, subsat, debug, scanangle, center )
    ! External variables
    real(kind=4), intent(in), dimension(:,:) :: ipoints
    real(kind=4), intent(in), dimension(:,:) :: jpoints
    real(kind=4), intent(out), dimension(:,:) :: outlat
    real(kind=4), intent(out), dimension(:,:) :: outlon
    type(b1u_gvar), intent(in):: b1ugvar
    real(kind=4),dimension(:,:),optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    integer(kind=4), intent(in), optional :: scanangle
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal Variables
    integer nx, ny, n

    real(kind=4), dimension(:),allocatable :: ip, jp, lat, lon, v

    nx = size(ipoints,1)
    ny = size(ipoints,2)
    n  = size(ipoints)

    allocate(ip(n), jp(n), lat(n), lon(n))

    ip = reshape(ipoints,(/n/))
    jp = reshape(jpoints,(/n/))
    
    if (present(vza)) then
       allocate(v(n))
       call b1u_ij2ll(ip,jp,lat,lon,b1ugvar,vza=v,subsat=subsat,&
            &         debug=debug, scanangle=scanangle, center=center)
       vza = reshape(v,(/nx,ny/))
       deallocate (v)
    else
       call b1u_ij2ll(ip,jp,lat,lon,b1ugvar,subsat=subsat,debug=debug,&
            &         scanangle=scanangle, center=center)
    end if
    
    outlat = reshape(lat,(/nx,ny/))
    outlon = reshape(lon,(/nx,ny/))
    deallocate (ip, jp, lat, lon)
  end subroutine b1u_ij2ll_gvar_rank2
!                                                   b1u_ij2ll_gvar_rank1
  subroutine b1u_ij2ll_gvar_rank1 (ipoints, jpoints, outlat, outlon , b1ugvar,&
       &                           vza, subsat, debug, scanangle, center)
    implicit none
    ! External variables
    real(kind=4), intent(in), dimension(:)  :: ipoints
    real(kind=4), intent(in), dimension(:)  :: jpoints
    real(kind=4), intent(out), dimension(:) :: outlat
    real(kind=4), intent(out), dimension(:) :: outlon
    type(b1u_gvar), intent(in):: b1ugvar
    real(kind=4), dimension(:),optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    integer(kind=4), intent(in), optional :: scanangle
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal variables
    integer(kind=4) :: ew_offset, ns_offset, line_res, elem_res, sec, msec
    !interface with imcnav
    real(kind=8) ::                           se, s, glat, glon, grad
    real(kind=8), dimension(size(ipoints)) :: dlats, dlons, dlines, dpixels, satzen
    REAL(kind=8), dimension(337) :: rec
    integer(kind=4) :: key, imc, instr, nye,nde,nhe,nme, ny, nd, nh,&
         & nm, EW_NAD_CY, EW_NAD_INC, NS_NAD_CY, NS_NAD_INC

    !Convert the GVAR navigation type to the REC array
    call nav2rec( b1ugvar%b1unav, rec )

    EW_NAD_CY  = b1ugvar%b1unav%ew_cyl
    EW_NAD_INC = b1ugvar%b1unav%ew_inc
    NS_NAD_CY  = b1ugvar%b1unav%ns_cyl
    NS_NAD_INC = b1ugvar%b1unav%ns_inc

    call bcd_time(b1ugvar%b1unav%epoch_time,nye,nde,nhe,nme,sec,msec)
    se = real(sec,8) + real(msec,8)/1000.d0
    ny = b1ugvar%b1unav%img_date / 1000 + 1900
    nd = mod(b1ugvar%b1unav%img_date,1000)
    nh = b1ugvar%b1unav%img_time / 10000000
    nm = mod(b1ugvar%b1unav%img_time / 100000,100)
    s  = real(mod(b1ugvar%b1unav%img_time,100000),8) / 1000.d0
    ew_offset = b1ugvar%b1uarea%west_bound
    ns_offset = b1ugvar%b1uarea%north_bound
    elem_res =  b1ugvar%b1uarea%elem_res
    line_res =  b1ugvar%b1uarea%line_res

    imc = 0
    instr = 1
    key = 0

    dlines = real(ns_offset,8) + jpoints * real(line_res,8)
    dpixels= real(ew_offset,8) + ipoints * real(elem_res,8)
    call imcnav_r1(dlats,dlons,dlines,dpixels,key,imc,instr,  &
         nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
         ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
         REC, glat, glon, grad, satzen, scanangle)
    outlat = real(dlats,4) ; outlon = real(dlons,4)

!!$    npoints = size(ipoints)
!!$    do ipoint=1,npoints
!!$       line = jpoints(ipoint)
!!$       pixel= ipoints(ipoint)
!!$       line = real(ns_offset,8) + line * real(line_res,8)
!!$       pixel= real(ew_offset,8) + pixel* real(elem_res,8)
!!$       call imcnav (dlat,dlon,line,pixel,key,imc,instr,  &
!!$            nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
!!$            ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
!!$            REC, glat, glon, grad,satzen,scanangle=scanangle)
!!$       outlat(ipoint) = real(dlat,4)
!!$       outlon(ipoint) = real(dlon,4)
!!$       if (present(vza)) then 
!!$          vza(ipoint) = real(satzen,4)
!!$       end if
!!$    end do
    if (present(vza)) then
       vza = satzen
    endif

    if (present(subsat)) then
       subsat = (/ glat * rad, glon * rad, grad/)
    end if
  end subroutine b1u_ij2ll_gvar_rank1
!                                                b1u_ij2ll_kepler_scalar
  subroutine b1u_ij2ll_kepler_scalar (ipoints, jpoints, outlat, outlon , nav,&
       &                             vza, subsat, debug, scanangle, center )
    implicit none
    ! External variables
    real(kind=4), intent(in) :: ipoints
    real(kind=4), intent(in) :: jpoints
    real(kind=4), intent(out):: outlat
    real(kind=4), intent(out):: outlon
    integer(kind=4), intent(in), dimension(0:199) :: nav
    real(kind=4), intent(out), optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    integer(kind=4), optional :: scanangle
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal variables
    real(kind=4), dimension(1) :: lat, lon, v

    call b1u_ij2ll( (/ipoints/), (/jpoints/), lat, lon, nav, &
                    v, subsat, debug, scanangle, center)
    outlat = lat(1)
    outlon = lon(1)
    if (present(vza)) vza = v(1)
    return
  end subroutine b1u_ij2ll_kepler_scalar
!                                                 b1u_ij2ll_kepler_rank2
  subroutine b1u_ij2ll_kepler_rank2 (ipoints, jpoints, outlat, outlon , nav,&
       &                             vza, subsat, debug, scanangle, center )
    ! External variables
    real(kind=4), intent(in), dimension(:,:) :: ipoints
    real(kind=4), intent(in), dimension(:,:) :: jpoints
    real(kind=4), intent(out), dimension(:,:) :: outlat
    real(kind=4), intent(out), dimension(:,:) :: outlon
    integer(kind=4), intent(in), dimension(0:199) :: nav
    real(kind=4), intent(out), dimension(:,:), optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    integer(kind=4), optional :: scanangle
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal Variables
    integer nx, ny, n

    real(kind=4), dimension(:),allocatable :: ip, jp, lat, lon, vzaa

    nx = size(ipoints,1)
    ny = size(ipoints,2)
    n  = size(ipoints)

    allocate(ip(n), jp(n), lat(n), lon(n), vzaa(n))

    ip = reshape(ipoints,(/n/))
    jp = reshape(jpoints,(/n/))
    
    call b1u_ij2ll(ip,jp,lat,lon,nav,vzaa,subsat,debug,scanangle,center=center)
    
    outlat = reshape(lat,(/nx,ny/))
    outlon = reshape(lon,(/nx,ny/))
    if (present(vza)) then
       vza = reshape(vzaa,(/nx,ny/))
    endif
    deallocate (ip, jp, lat, lon, vzaa)
  end subroutine b1u_ij2ll_kepler_rank2
!                                                 b1u_ij2ll_kepler_rank1
  subroutine b1u_ij2ll_kepler_rank1 (ipoints, jpoints, outlat, outlon , nav, &
                                     vza, subsat, debug, scanangle, center )
    ! External variables
    real(kind=4), intent(in), dimension(:) :: ipoints
    real(kind=4), intent(in), dimension(:) :: jpoints
    real(kind=4), intent(out), dimension(:) :: outlat
    real(kind=4), intent(out), dimension(:) :: outlon
    integer(kind=4), intent(in), dimension(0:199) :: nav
    real(kind=4), dimension(:),optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    integer(kind=4), intent(in), optional :: scanangle
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal variables
    real(kind=8), dimension(6) :: kepler !Keplerian elements
    integer(kind=4) :: ep_date, ep_time, ep_msec
    real   (kind=8) :: roll, pitch, yaw
    real   (kind=8) :: cen_lin, cen_pix, linres, pixres, sline, spix
    real   (kind=8) :: linang, pixang, spper
    integer(kind=4) :: imgdate
    real   (kind=8) :: imgtime
    real   (kind=8) :: spra1, spra2, spdc1, spdc2
    integer(kind=4) :: icheb
    real   (kind=8),dimension(11) :: chebx, cheby, chebz
    integer(kind=4) :: year, jday, iyear, idoy
    real   (kind=8) :: ejulday, imgjulday
    integer(kind=4) :: spinmiss
    real   (kind=8) :: dtime_, time, glat, glon, grad
    real   (kind=8) :: spra, spdc
    real   (kind=8) :: xs1, xs2, xs3
    real   (kind=8) :: pdots,psq,s1den,twopi
    real   (kind=8), dimension(size(ipoints)) :: ip, jp, scan, elev, &
         dtime_a, julian, gra,      &
         spra_a,spdc_a, px, py, pz, q1,q2,q3,d,h,    &
         x0,x1,x2,vc0,vc1,vc2, sins, coss, sine,cose
    real   (kind=8), dimension(0:2,size(ipoints)) :: r, vc, off
    real   (kind=8), dimension(size(ipoints),0:2) :: vs
    real   (kind=8), dimension(0:2,0:2,size(ipoints)) :: s
    integer(kind=4) :: i, n, np
    real   (kind=4) :: c0, c1, c2, cen_lin_img
    real   (kind=4), dimension(size(ipoints)) :: cen_pix_img
    integer(kind=4), dimension(3) :: xq
    real :: dtime, ny
    real(kind=4), allocatable, dimension(:) :: xtemp, cpi
    integer (kind=4) :: centerflag

    twopi=2.0d0*pi
    if (present(debug)) print*,'Entering b1u_ij2ll',dtime(xq)
    np = size(ipoints)
    
    centerflag = 0
    if (present(center)) then
       if (center(1) > 1) centerflag=1
    endif

    !-----------------------------------
    !  Assign parameters based on NAV
    !-----------------------------------
    !Assign Kepler elements to the kepler array (for kep2cart subroutine)
    kepler = (/real(nav(13),8)*1.d-2, &
              real(nav(15),8)*1.d-6, &
              real(nav(14),8)*1.d-3, &
              real(nav(17),8)*1.d-3, &
              real(nav(18),8)*1.d-3, &
              real(nav(16),8)*1.d-3/)
    ep_date = nav(10) ; ep_time = nav(11) ; ep_msec = nav(12)
    !test for bad navigation
    if ((ep_date == 0) .or. (ep_date == -9999)) then
       kepler = (/real(nav(23),8)*1.d-2, &
                  real(nav(25),8)*1.d-6, &
                  real(nav(24),8)*1.d-3, &
                  real(nav(27),8)*1.d-3, &
                  real(nav(28),8)*1.d-3, &
                  real(nav(26),8)*1.d-3/)
       ep_date = nav(20) ; ep_time = nav(21) ; ep_msec = nav(22)
    endif
    if ((ep_date == 0) .or. (ep_date == -9999)) then
       kepler = (/real(nav(33),8)*1.d-2, &
                  real(nav(35),8)*1.d-6, &
                  real(nav(34),8)*1.d-3, &
                  real(nav(37),8)*1.d-3, &
                  real(nav(38),8)*1.d-3, &
                  real(nav(36),8)*1.d-3/)
       ep_date = nav(30) ; ep_time = nav(31) ; ep_msec = nav(32)
    endif
    if (present(debug)) then
       print*,'BEGIN DEBUG INFORMATION OUTPUT ----------------------'
       print*,'Kepler:',kepler
       print*,'date/time:',ep_date, ep_time, ep_msec
    end if
    !attitude parameters
    roll   =  real(nav(39),8)*1.d-6
    pitch  =  real(nav(40),8)*1.d-6
    yaw    = -real(nav(41),8)*1.d-6
    if (present(debug)) then
       print*,'R/P/Y:',roll,pitch,yaw
    end if

    !image information
    cen_lin = nav(2)/10.

    if (centerflag == 1) cen_lin = cen_lin + center(7)/10.
    ny = nav(149)

    cen_pix = nav(3)/10.
    linres = nav(4)
    pixres = nav(5)
    sline  = nav(0)
    spix   = nav(1)
    linang = nav(6)*1.d-9
    pixang = nav(7)*1.d-9

    cen_lin_img = (real(cen_lin) - real(sline)) / real(linres)
    if (nav(160) == 1) then
       c0 = real(nav(161))*1.e-7 * 10.**(nav(162))
       c1 = real(nav(163))*1.e-7 * 10.**(nav(164))
       c2 = real(nav(165))*1.e-7 * 10.**(nav(166))
    else
       c0 = (real(cen_pix) - real(spix)) / real(pixres)
       c1 = 0.0e0
       c2 = 0.0e0
    end if


    spper  = nav(8)*1.d-6    !spin period in seconds/revolution
    imgdate = nav(147)
    imgtime = nav(148)/1000. !seconds since start of day * 1000
    !spin axis attitude
    spra1 = nav(65)  / 2.d0**21
    spdc1 = nav(66)  / 2.d0**21
    spra2 = nav(125) / 2.d0**21
    spdc2 = nav(126) / 2.d0**21
    if (nav(146) > 0) icheb = 1
    chebx = nav(86:96)  /2.d0**13
    cheby = nav(97:107) /2.d0**13
    chebz = nav(108:118)/2.d0**13
    !Use JMA fit if RMS > 0
    if (nav(52) > 0) then
       spra1 = nav(45) * 1.d-6
       spra2 = nav(47) * 1.d-6
       spdc1 = nav(46) * 1.d-6
       spdc2 = nav(48) * 1.d-6
       roll  = nav(42) * 1.d-6
       pitch = nav(43) * 1.d-6
       yaw   = nav(44) * 1.d-6
    endif

    !-----------------------------------
    !  Time information
    !-----------------------------------
    ! Epoch time
    year = int(ep_date / 1000)
    year = year + 1900 * eval(year < 1900)
    jday = mod(ep_date,1000)
    ejulday = julday(1,jday,year) + &
              ftime(ep_time) / 24.d0    + &
              real(ep_msec,8) / 1000.d0 / 3600.d0 / 24.d0
    if (present(debug)) then
       print*,'Epoch TIME'
       print*,jday,year,ejulday
    endif
    !Image time
    iyear = int(imgdate / 1000)
    iyear = iyear + 1900 * eval(iyear < 1900)
    idoy = mod(imgdate,1000)
    imgjulday = julday(1,idoy,iyear) + &
                imgtime / 86400.d0
    if (present(debug)) then
       print*,'IMAGE TIME'
       print*,idoy,iyear,imgjulday
    endif
if (present(debug)) print*,'Time info',dtime(xq)
    !-----------------------------------
    !  Get SPRA & SPDC From glat/glon if missing
    !-----------------------------------
    spinmiss = 0
    if ( (spra1 == spra2) .and. (spra1 == 0.d0) .and. &
         (spdc1 == spdc2) .and. (spdc1 == 0.d0)) then
       spinmiss = 1
       !get satellite position using image time and kep2cart
       dtime_ = (imgjulday - ejulday) * 24.d0 * 3600.d0
       call kep2cart(kepler,dtime_,xs1,xs2,xs3)
       !get subsat using cart2earth
       time = ejulday + dtime_/3600.d0/24.d0
       call cart2earth(xs1,xs2,xs3,time,glat,glon,grad)
       !approximate using the subsatellite longitude
       if (spra1 == spra2 .and. spra1 == 0.d0) &
            spra = -glon / drad
       !approximate using the sub satellite latitude
       if (  spdc1 == spdc2 .and. &
            ((spdc1 == 0.) .or. (spdc1 == 90.d0)) )  &
            spdc = (-90. + glat) / drad
    endif

    ip = ipoints
    jp = jpoints

    !CONVERT i,j to scan/elevation angles
    !scan = (ip*pixres + spix - cen_pix)*pixang/rad
    !elev = (cen_lin - (jp*linres+sline))*linang/rad

    !Convert i,j to scan/elevation angles using the skew parameter
    !dline = cen_lin - (jp*linres + sline)
    !scan = (ip*pixres + spix - cen_pix + dline*skew_fact)*pixang/drad !was dpix
    !scan = dpix *pixang/rad
    !elev = dline*linang/drad

    !Convert i,j to scan/elev using the variable center pixel
    !elev = ((cen_lin_img - jp) * linres - sline) * linang/rad
!!$    cen_pix_img = c0 + c1 * jp + c2 * jp * jp
!!$    scan = (spix + (ip - cen_pix_img) * pixres) * pixang/rad
!!$    elev = (cen_lin - (jp*linres+sline))*linang/rad
    cen_pix_img = c0 + c1 * jp + c2 * jp * jp
!!$    scan = (spix + (ip - cen_pix_img) * pixres) * pixang/rad
!!$    elev = (cen_lin - (jp*linres+sline))*linang/rad
    if (centerflag == 1) then
       allocate(xtemp(ny))
       allocate(cpi(ny))
       do i=1,int(ny)
          xtemp(i) = (i-1)-ny/2.
       end do
       cpi =  center(1) + &
            & center(2) * xtemp + &
            & center(3) * xtemp*xtemp + &
            & center(4) * xtemp*xtemp*xtemp
       do i=1,np
          cen_pix_img(i) = cpi(within(int(jp(i)),1,int(ny)))
          !cen_pix_img(i) = cpi(int(jp(i)))
       end do
       deallocate(xtemp,cpi)
    endif
    scan = (ip - cen_pix_img) * pixres * pixang / drad
    elev = (cen_lin_img - jp) * linres * linang / drad
    if (present(scanangle)) then
       outlat = elev !+ pitch/rad
       outlon = scan !+  roll/rad
       return
    endif
if (present(debug)) print*,'scanelev',dtime(xq)

    !======
    !  VS
    !======
    !Get satellite viewing angle from the scan/elevation by correcting for
    !the roll/pitch/yaw angles

    call sincos(scan+roll/drad, sins,coss)
    call sincos(elev+pitch/drad,sine,cose)
    vs(:,0) = coss*cose - &
              sins*sine*sin(yaw/drad)
    vs(:,1) = sins*cose + &
              coss*sine*sin(yaw/drad)
    vs(:,2) = -COS(YAW/drad)*sine
!!$    vs(:,0) = cos(scan+roll/rad)*cos(elev+pitch/rad) - &
!!$              sin(scan+roll/rad)*sin(elev+pitch/rad)*sin(yaw/rad)
!!$    vs(:,1) = sin(scan+roll/rad)*cos(elev+pitch/rad) + &
!!$              cos(scan+roll/rad)*sin(elev+pitch/rad)*sin(yaw/rad)
!!$    vs(:,2) = -COS(YAW/rad)*sin(elev+pitch/rad)


    !;time since epoch at start of image
    dtime_a = (imgjulday - ejulday) * (24.d0 * 3600.d0) + &
            (jp+sline/(linres)) * spper
if (present(debug)) print*,'vs/dtime',dtime(xq)

!!$    if (present(debug)) print*,'imgjulday:',imgjulday
!!$    if (present(debug)) print*,'ejulday  :',  ejulday
!!$    if (present(debug)) print*,'diff     :',imgjulday-  ejulday
!!$    if (present(debug)) print*,'DTIME:',dtime_a
    if ( (icheb == 1) .and. &
         (maxval(dtime_a) <= (13.d0*3600.d0) ) .and. &
         (minval(dtime_a) >= 0.d0) .and. &
         sum(abs(chebx)) /= 0.) then
       !Use Chebyshev polynomials to determine Cartesian coordinates
!!$       print*,'Uh Oh. Need Chebyshev Polynomials'
call kep2cart(kepler,dtime_a,px,py,pz)
!!$       stop
!!$       cheb2cart,chebx,dtime,xs1
!!$       cheb2cart,cheby,dtime,xs2
!!$       cheb2cart,chebz,dtime,xs3
    else
       !Otherwise use Kepler elements
       call kep2cart(kepler,dtime_a,px,py,pz)
    end if
if (present(debug)) print*,'kep2cart',dtime(xq)
    julian = imgjulday + (jp+sline/real(linres))*(spper/3600.d0/24.d0)
    gra = gha(julian)

    r(0,1:np) = px(1:np)
    r(1,1:np) = py(1:np)
    r(2,1:np) = pz(1:np)
    r = r / ae

!!$    if (present(debug)) print*,'Satellite Position:',r*ae
    grad = sqrt(px(1)**2 + py(1)**2 + pz(1)**2)
    glat = asin(within(pz(1) / grad,&
         &             minimum=-1.0d0,maximum=1.0d0))
    glat = atan(be**2 * tan(glat) / ae**2)
    glon = atan2(py(1),px(1))
    glon = mod((glon-gra(1)),twopi)
    if (glon < (-pi)) glon = glon + twopi
    if (glon >   pi ) glon = glon - twopi
    glon = glon * drad
    glat = glat * drad

    if (spinmiss == 0) then
       spra_a = spra1 + dtime_a/3600./13. * (spra2-spra1)
       spdc_a = spdc1 + dtime_a/3600./13. * (spdc2-spdc1)
       spra_a = spra_a / drad
       spdc_a = spdc_a / drad
       if (spra2 == 0.) spra_a = spra1 / drad
       if (spdc2 == 0.) spdc_a = spdc1 / drad
    end if
          
    if (spinmiss == 0) then
       call sincos(spdc_a,sins,coss)
       call sincos(spra_a,sine,cose)
       s(2,0,1:np) = coss(1:np)*cose(1:np) !cos(spdc_a) * cos(spra_a)
       s(2,1,1:np) = coss(1:np)*sine(1:np) !cos(spdc_a) * sin(spra_a)
       s(2,2,1:np) = sins(1:np)      !sin(spdc_a)
    else
       s(2,0,:) = cos(spdc) * cos(spra)
       s(2,1,:) = cos(spdc) * sin(spra)
       s(2,2,:) = sin(spdc)
    end if
if (present(debug)) print*,'glonglat',dtime(xq)
    do i=1,np
       PDOTS = PX(i)*S(2,0,i) + PY(i)*S(2,1,i) + PZ(i)*S(2,2,i)
       PSQ = PX(i)*PX(i) + PY(i)*PY(i) + PZ(i)* PZ(i)
       S1DEN = SQRT(PSQ - PDOTS**2)
       S(0,0,i) = (-PX(i) + PDOTS*S(2,0,i))/S1DEN
       S(0,1,i) = (-PY(i) + PDOTS*S(2,1,i))/S1DEN
       S(0,2,i) = (-PZ(i) + PDOTS*S(2,2,i))/S1DEN
       S(1,0,i)=S(2,1,i)*S(0,2,i)-S(2,2,i)*S(0,1,i)
       S(1,1,i)=S(2,2,i)*S(0,0,i)-S(2,0,i)*S(0,2,i)
       S(1,2,i)=S(2,0,i)*S(0,1,i)-S(2,1,i)*S(0,0,i)
    end do
!!$       PDOTS = PX*S(2,0,:) + PY*S(2,1,:) + PZ*S(2,2,:)
!!$       PSQ = PX**2 + PY**2 + PZ**2
!!$       S1DEN = SQRT(PSQ - PDOTS**2)
!!$       S(0,0,:) = (-PX + PDOTS*S(2,0,:))/S1DEN
!!$       S(0,1,:) = (-PY + PDOTS*S(2,1,:))/S1DEN
!!$       S(0,2,:) = (-PZ + PDOTS*S(2,2,:))/S1DEN
!!$       S(1,0,:)=S(2,1,:)*S(0,2,:)-S(2,2,:)*S(0,1,:)
!!$       S(1,1,:)=S(2,2,:)*S(0,0,:)-S(2,0,:)*S(0,2,:)
!!$       S(1,2,:)=S(2,0,:)*S(0,1,:)-S(2,1,:)*S(0,0,:)
if (present(debug)) print*,'calc s  ',dtime(xq)
    !;======
    !;  VC
    !;======
    !;transform VS (pointing vector) to earth-centered fixed coordinates (VC)
    !;to do this, we need the S vector from satcrd
    !;vc = transpose(s) # vs
    do i=1,np
       vc(0,i) = vs(i,0)*s(0,0,i) + vs(i,1)*s(1,0,i) + vs(i,2)*s(2,0,i)
       vc(1,i) = vs(i,0)*s(0,1,i) + vs(i,1)*s(1,1,i) + vs(i,2)*s(2,1,i)
       vc(2,i) = vs(i,0)*s(0,2,i) + vs(i,1)*s(1,2,i) + vs(i,2)*s(2,2,i)
    end do
if (present(debug)) print*,'calc vc ',dtime(xq)

    off(0:1,1:np) = 1.0d0
    off(2  ,1:np) = 1.0d0 / omf2

    do i=1,np
       q1(i) = vc(0,i)*vc(0,i)*off(0,i) + &
            &  vc(1,i)*vc(1,i)*off(1,i) + &
            &  vc(2,i)*vc(2,i)*off(2,i)
       q2(i) = vc(0,i)*r(0,i) *off(0,i) + &
            &  vc(1,i)*r(1,i) *off(1,i) + &
            &  vc(2,i)*r(2,i) *off(2,i)
       q3(i) = r(0,i) *r(0,i) *off(0,i) + &
            &  r(1,i) *r(1,i) *off(1,i) + &
            &  r(2,i) *r(2,i) *off(2,i) - 1.0d0
       d(i) = q2(i)*q2(i) - q1(i)*q3(i)
    end do
    vc0(1:np) = vc(0,1:np)
    vc1(1:np) = vc(1,1:np)
    vc2(1:np) = vc(2,1:np)
    outlat(1:np) = -9999.
    outlon(1:np) = -9999.
    x0(1:np) = 0.d0
    x1(1:np) = 0.d0
    x2(1:np) = 0.d0
if (present(debug)) print*,'calc q  ',dtime(xq)
    where (d > 0) 
       h = (-q2 - sqrt(d)) / q1
       x0 = px/ae + h * vc0
       x1 = py/ae + h * vc1
       x2 = pz/ae + h * vc2
       !latitude
       outlat = atan( x2 / omf2 / sqrt(x0*x0+x1*x1) )
       !longitude
       outlon = mod(atan2( x1, x0 ) - gra, twopi)
    end where
    where (outlon > pi) outlon=outlon - twopi
    where ((outlon > -9999.) .and. &
           (outlon < (-pi))) outlon = outlon + twopi
if (present(debug)) print*,'onearth ',dtime(xq)

    outlat = outlat * drad
    outlon = outlon * drad

    !Get view zenith angle from vc and x0,x1,x2
    if (present(vza)) then
       n = size(vza)
       do i=1,n
          if (d(i) <= 0.) then
             vza(i) = -9999.
          else
             vza(i) = acos( within(dot( unit( (/x0(i),x1(i),x2(i) /)  ), &
                  &   -1.*unit(reshape(vc(:,i),(/3/))) ),&
                  &   minimum=-1.0d0,maximum=1.0d0) ) * drad
          end if
       end do
    endif

    if (present(subsat)) subsat = (/glat, glon, grad/)
    if (present(debug)) then
!!$       print*,'END   DEBUG INFORMATION OUTPUT ----------------------'
    end if

  end subroutine b1u_ij2ll_kepler_rank1
!                                                          END B1U_IJ2LL 
!#######################################################################
!#######################################################################
!                                                        BEGIN B1U_LL2IJ
!****r* kmod_isccp/b1u_ll2ij
! NAME
! b1u_ll2ij
! SYNOPSIS
! call b1u_ll2ij( lat, lon, ip, jp, b1ugvar, vza, subsat, debug, scanangle)
!       or
! call b1u_ll2ij( lat, lon, ip, jp, nav, vza, subsat, debug, scanangle)
! FUNCTION
!    To convert latitude/longitude  point(s) of a satellite image to
!    line/pixel coordinates.
! INPUTS
!  * lat (degrees)
!  * lon  (degrees)
!      * r*4 - rank2
!      * r*4 - rank1
!      * r*4 - scalar
!      * same type & rank as ip
!  * Navigation is taken from either:
!      *  NAV - B1U Navigation array (LONARR(200)) from b1uread
!      *  b1ugvar, which is type(b1u_gvar)
! 
! OUTPUT
!  * ip - Pixel locations (default is in image coordinates):
!  * jp - Line locations (default is in image coordinates):
!  * Optional outputs:
!    * vza - optional:
!      * same type and rank as ip
!    * subsat - optional - r*4(3) - [lat(deg),lon(deg),radius(km)]
! OPTIONS
!    Optional arguments:
!    * debug - i*4 - set debug=1 for optional debugging output
!    * scanangle - i*4 - set to return scan and elevation angles (deg) 
!      instead of lat/lon values.
! USES 
! nav2rec, bcd_time, imcnav_r1, julday, kep2cart, cart2earth, sincos
! 
! NOTES
! * The GVAR Navigation calls imc_nav
! * The Kepler navigation (all non-GVAR satellites) is based on 
!   Kidder and Vonder Haar, 1995: Introduction to Satellite Meteorology
! * While this routine could use Chebyshev polynomials, it is not yet set up 
!   to do it.
!***
!                                                  b1u_ll2ij_gvar_scalar
  subroutine b1u_ll2ij_gvar_scalar (inlat, inlon , &
       &                            ipoint, jpoint, b1ugvar, &
       &                            vza, subsat, debug, center )
    implicit none
    ! External variables
    real(kind=4), intent(in)  :: inlat
    real(kind=4), intent(in)  :: inlon
    real(kind=4), intent(out) :: ipoint
    real(kind=4), intent(out) :: jpoint
    type(b1u_gvar), intent(in):: b1ugvar
    real(kind=4), intent(inout), optional :: vza
    real(kind=4), optional, dimension(3)  :: subsat
    integer(kind=4), intent(in), optional :: debug
    real(kind=4), intent(in), dimension(13), optional :: center
    
    !Internal variables
    real(kind=4), dimension(1) :: ip, jp, vzaa

    call b1u_ll2ij( (/inlat/), (/inlon/), ip, jp, b1ugvar, vzaa, subsat, debug)
    ipoint = ip(1)
    jpoint = jp(1)
    if (present(vza)) then
       vza = vzaa(1)
    endif

  end subroutine b1u_ll2ij_gvar_scalar
!                                                   b1u_ij2ll_gvar_rank2
  subroutine b1u_ll2ij_gvar_rank2 (inlat, inlon , &
       &                           ipoints, jpoints, &
       &                           b1ugvar,vza,subsat,debug,center)
    implicit none
    ! External variables
    real(kind=4), intent(in), dimension(:,:) :: inlat
    real(kind=4), intent(in), dimension(:,:) :: inlon
    real(kind=4), intent(out), dimension(:,:):: ipoints
    real(kind=4), intent(out), dimension(:,:):: jpoints
    type(b1u_gvar), intent(in):: b1ugvar
    real(kind=4), intent(inout), dimension(:,:),optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal variables
    real(kind=4), allocatable,dimension(:) :: ip, jp, lat,lon,vzaa
    integer :: n, nx, ny

    nx = size(ipoints,1)
    ny = size(ipoints,2)
    n  = size(ipoints)
    allocate(ip(n), jp(n), lat(n), lon(n), vzaa(n))
    lat = reshape(inlat, (/n/))
    lon = reshape(inlon, (/n/))

    call b1u_ll2ij(lat,lon,ip,jp,b1ugvar,vzaa,subsat,debug)

    ipoints = reshape(ip,(/nx,ny/))
    jpoints = reshape(jp,(/nx,ny/))
    if (present(vza)) then
       vza     = reshape(vzaa,(/nx,ny/))
    endif
    deallocate (ip, jp, lat, lon, vzaa)
  end subroutine b1u_ll2ij_gvar_rank2
!                                                   b1u_ij2ll_gvar_rank1
  subroutine b1u_ll2ij_gvar_rank1 (inlat, inlon , &
       &                           ipoints, jpoints, &
       &                           b1ugvar,vza,subsat,debug,center)
    implicit none
    ! External variables
    real(kind=4), intent(in), dimension(:) :: inlat
    real(kind=4), intent(in), dimension(:) :: inlon
    real(kind=4), intent(out), dimension(:):: ipoints
    real(kind=4), intent(out), dimension(:):: jpoints
    type(b1u_gvar), intent(in):: b1ugvar
    real(kind=4), intent(inout), dimension(:),optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    real(kind=4), intent(in), dimension(13), optional :: center
    !Internal variables
    integer(kind=4) :: ew_offset, ns_offset, line_res, elem_res, sec, msec
    !interface with imcnav
    real(kind=8) :: se, s, glat, glon, grad
    real(kind=8), dimension(size(ipoints)) :: dlats, dlons, dlines, &
         &        dpixels, satzen
    REAL(kind=8), dimension(337) :: rec
    integer(kind=4) :: key, imc, instr, nye,nde,nhe,nme, ny, nd, nh,&
         & nm, EW_NAD_CY, EW_NAD_INC, NS_NAD_CY, NS_NAD_INC

    !Convert the GVAR navigation type to the REC array
    call nav2rec( b1ugvar%b1unav, rec )

    EW_NAD_CY  = b1ugvar%b1unav%ew_cyl
    EW_NAD_INC = b1ugvar%b1unav%ew_inc
    NS_NAD_CY  = b1ugvar%b1unav%ns_cyl
    NS_NAD_INC = b1ugvar%b1unav%ns_inc

    call bcd_time(b1ugvar%b1unav%epoch_time,nye,nde,nhe,nme,sec,msec)
    se = real(sec,8) + real(msec,8)/1000.d0
    ny = b1ugvar%b1unav%img_date / 1000 + 1900
    nd = mod(b1ugvar%b1unav%img_date,1000)
    nh = b1ugvar%b1unav%img_time / 10000000
    nm = mod(b1ugvar%b1unav%img_time / 100000,100)
    s  = real(mod(b1ugvar%b1unav%img_time,100000),8) / 1000.d0
    ew_offset = b1ugvar%b1uarea%west_bound
    ns_offset = b1ugvar%b1uarea%north_bound
    elem_res =  b1ugvar%b1uarea%elem_res
    line_res =  b1ugvar%b1uarea%line_res

    imc = 0
    instr = 1
    key = 1
    
    dlats = real(inlat,8)
    dlons = real(inlon,8)
    call imcnav_r1(dlats,dlons,dlines,dpixels,key,imc,instr,  &
         nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
         ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
         REC, glat, glon, grad, vza=satzen)
    ipoints = real(dpixels - ew_offset,4) / real(elem_res)
    jpoints = real(dlines -  ns_offset,4) / real(line_res)
    if (present(vza)) then
       vza = satzen
    end if
    

!!$    npoints = size(ipoints)
!!$    do ipoint=1,npoints
!!$       dlat = real(inlat(ipoint),8)
!!$       dlon = real(inlon(ipoint),8)
!!$       call imcnav (dlat,dlon,line,pixel,key,imc,instr,  &
!!$            nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
!!$            ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
!!$            REC, glat, glon, grad, satzen)
!!$       ipoints(ipoint) = real(pixel - ew_offset,4) / real(elem_res)
!!$       jpoints(ipoint) = real(line -  ns_offset,4) / real(line_res)
!!$       if (present(vza)) then 
!!$          vza(ipoint) = real(satzen,4)
!!$       end if
!!$    end do
    if (present(subsat)) then
       subsat = (/ glat*rad, glon*rad, grad /)
    end if
  end subroutine b1u_ll2ij_gvar_rank1
!                                                 b1u_ll2ij_kepler_scalar
  subroutine b1u_ll2ij_kepler_scalar (inlat, inlon, ipoints, jpoints, &
       &                              nav,vza,subsat,debug,center,height)
    ! External variables
    real(kind=4), intent(in) :: inlat
    real(kind=4), intent(in) :: inlon
    real(kind=4), intent(out):: ipoints
    real(kind=4), intent(out):: jpoints
    integer(kind=4), intent(in), dimension(0:199) :: nav
    real(kind=4), intent(inout), optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    real(kind=4), intent(in), dimension(13), optional :: center
    real(kind=4), intent(in), optional :: height
    !Internal variables
    real(kind=4), dimension(1) :: ip, jp,vzaa, hgtt

    if (present(height)) then
       hgtt(1) = height
    else
       hgtt(1) = 0.
    end if 

    call b1u_ll2ij( (/inlat/), (/inlon/), ip, jp, nav,vzaa,subsat,debug,center,hgtt )
    ipoints = ip(1)
    jpoints = jp(1)
    if (present(vza)) vza = vzaa(1)
  end subroutine b1u_ll2ij_kepler_scalar
!                                                 b1u_ll2ij_kepler_rank2
  subroutine b1u_ll2ij_kepler_rank2 (inlat, inlon, ipoints, jpoints, &
       &                             nav,vza,subsat,debug,center,height)
    ! External variables
    real(kind=4), intent(in), dimension(:,:) :: inlat
    real(kind=4), intent(in), dimension(:,:) :: inlon
    real(kind=4), intent(out), dimension(:,:):: ipoints
    real(kind=4), intent(out), dimension(:,:):: jpoints
    integer(kind=4), intent(in), dimension(0:199) :: nav
    real(kind=4), intent(inout), dimension(:,:),optional :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    real(kind=4), intent(in), dimension(13),  optional :: center
    real(kind=4), intent(in), dimension(:,:), optional :: height
    !Internal variables
    real(kind=4), dimension(:),allocatable :: ip, jp, vzaa, lat,lon,hgtt
    integer n, nx, ny

    nx = size(ipoints,1)
    ny = size(ipoints,2)
    n  = size(ipoints)
    allocate(ip(n), jp(n), lat(n), lon(n), vzaa(n), hgtt(n))
    lat = reshape(inlat, (/n/))
    lon = reshape(inlon, (/n/))
    
    if (present(height)) then
       hgtt = reshape(height,(/n/) )
    else
       hgtt(:) = 0.
    end if

    call b1u_ll2ij( lat, lon, ip, jp, nav, vzaa, subsat, debug, center,hgtt)
    ipoints = reshape(ip,(/nx,ny/))
    jpoints = reshape(jp,(/nx,ny/))
    if (present(vza)) then
       vza     = reshape(vzaa,(/nx,ny/))
    endif
    deallocate (ip, jp, lat, lon, vzaa)
  end subroutine b1u_ll2ij_kepler_rank2
!                                                 b1u_ll2ij_kepler_rank1

  subroutine b1u_ll2ij_kepler_rank1 (inlat, inlon, ipoints, jpoints, &
       &                             nav,vza,subsat,debug,center,height)
    implicit none
    ! External variables
    real(kind=4), intent(in), dimension(:) :: inlat
    real(kind=4), intent(in), dimension(:) :: inlon
    real(kind=4), intent(out), dimension(:) :: ipoints
    real(kind=4), intent(out), dimension(:) :: jpoints
    integer(kind=4), intent(in), dimension(0:199) :: nav
    real(kind=4), intent(out), optional, dimension(:) :: vza
    real(kind=4), optional,dimension(3)    :: subsat
    integer(kind=4), intent(in), optional :: debug
    real(kind=4), intent(in), dimension(13), optional :: center
    real(kind=4), intent(in), dimension(:), optional :: height
    !Internal variables
    real   (kind=4) :: ny
    real(kind=8), dimension(6) :: kepler !Keplerian elements
    integer(kind=4) :: ep_date, ep_time, ep_msec
    real   (kind=8) :: roll, pitch, yaw
    real   (kind=8) :: rollr, pitchr, yawr
    real   (kind=8) :: cen_lin, cen_pix, linres, pixres, sline, spix
    real   (kind=8) :: linang, pixang, spper
    integer(kind=4) :: imgdate
    real   (kind=8) :: imgtime
    real   (kind=8) :: spra1, spra2, spdc1, spdc2, spra, spdc
    integer(kind=4) :: icheb
    real   (kind=8),dimension(11) :: chebx, cheby, chebz
    integer(kind=4) :: year, jday, iyear, idoy
    real   (kind=8) :: ejulday, imgjulday
    real   (kind=8) :: amul, con_2_21, con_2_13
    real   (kind=8) :: c0, c1, c2, cen_lin_img
    real   (kind=8), dimension(size(inlat)) :: cen_pix_img
    !Scalar REALS
    real   (kind=8) :: p, xs1, xs2, xs3, glat, glon, grad, &
         &             dtime_s, time_s, tanyaw, cosyaw, cartlon, earthradius, philat,&
         &             pdots, psq, s1den, cspdc_a, cspra_a, sigma, xi,sphilat,cphilat
    !Rank=1 REALS
    real   (kind=8), dimension(size(inlat)) :: &
         lat, lon, phi1, dtime_1, phiold, dsline, lasttime, &
         px,py,pz, time, gra,&
         dotp, &
         spra_a, spdc_a, phinew, &
         theta

    !Rank=2 REALS
    real   (kind=8), dimension(0:2,size(inlat)) :: &
         r, x, off, w, vc, vs
    !Rank=3 REALS
    real   (kind=8), dimension(0:2,0:2,size(inlat)) :: &
         s
    !INTEGERS
    integer(kind=4) :: ndsline, try, spinmiss, i, n, np
    real dtime, temp
    integer,dimension(2) :: xq
    real,dimension(10) :: parts
    real(kind=4), allocatable, dimension(:) :: cpi, xtemp
    integer (kind=4) :: centerflag

    centerflag = 0
    if (present(center)) then
       if (center(1) > 1) centerflag=1
    endif

    np=size(inlat)

    parts(:) = 0.
    if (present(debug)) then
       print*,'Debuggin'
    endif
    if (present(debug)) print*,'Entering b1u_ll2ij_kepler_rank1',dtime(xq)
    !-----------------------------------
    !  Assign parameters based on NAV
    !-----------------------------------
    !Assign Kepler elements to the kepler array (for kep2cart subroutine)
    kepler = (/real(nav(13),8)*1.d-2, &
              real(nav(15),8)*1.d-6, &
              real(nav(14),8)*1.d-3, &
              real(nav(17),8)*1.d-3, &
              real(nav(18),8)*1.d-3, &
              real(nav(16),8)*1.d-3/)
    ep_date = nav(10) ; ep_time = nav(11) ; ep_msec = nav(12)
    !test for bad navigation
    if ((ep_date == 0) .or. (ep_date == -9999)) then
       kepler = (/real(nav(23),8)*1.d-2, &
                  real(nav(25),8)*1.d-6, &
                  real(nav(24),8)*1.d-3, &
                  real(nav(27),8)*1.d-3, &
                  real(nav(28),8)*1.d-3, &
                  real(nav(26),8)*1.d-3/)
       ep_date = nav(20) ; ep_time = nav(21) ; ep_msec = nav(22)
    endif
    if ((ep_date == 0) .or. (ep_date == -9999)) then
       kepler = (/real(nav(33),8)*1.d-2, &
                  real(nav(35),8)*1.d-6, &
                  real(nav(34),8)*1.d-3, &
                  real(nav(37),8)*1.d-3, &
                  real(nav(38),8)*1.d-3, &
                  real(nav(36),8)*1.d-3/)
       ep_date = nav(30) ; ep_time = nav(31) ; ep_msec = nav(32)
    endif
    if (sum(kepler) == 0.) then
       print*,'NO Kepler information??'
       print*,'1:',nav(13:18)
       print*,'2:',nav(23:28)
       print*,'3:',nav(33:38)
       print*,'Navigation Header'
       print*,nav
       stop
    endif


    !attitude parameters
    roll   =  real(nav(39),8)*1.d-6
    pitch  =  real(nav(40),8)*1.d-6
    yaw    = -real(nav(41),8)*1.d-6
    !image information
    cen_lin = nav(2)/10.
    cen_pix = nav(3)/10.
    linres = nav(4)
    pixres = nav(5)
    sline  = nav(0)
    spix   = nav(1)
    linang = nav(6)*1.d-9
    pixang = nav(7)*1.d-9

    if (centerflag == 1) cen_lin = cen_lin + center(7)/10.

    cen_lin_img = (real(cen_lin) - real(sline)) / real(linres)
    if (nav(160) == 1) then
       c0 = real(nav(161))*1.d-7 * 10.d0**(nav(162))
       c1 = real(nav(163))*1.d-7 * 10.d0**(nav(164))
       c2 = real(nav(165))*1.d-7 * 10.d0**(nav(166))
    else
       c0 = (real(cen_pix) - real(spix)) / real(pixres)
       c1 = 0.0e0
       c2 = 0.0e0
    end if

    ny = nav(149)

    spper  = nav(8)*1.d-6    !spin period in seconds/revolution
    imgdate = nav(147)
    imgtime = nav(148)/1000. !seconds since start of day * 1000
    !spin axis attitude
    con_2_21 = 2.0d0**21
    con_2_13 = 2.0d0**13
    spra1 = nav(65)  / con_2_21
    spdc1 = nav(66)  / con_2_21
    spra2 = nav(125) / con_2_21
    spdc2 = nav(126) / con_2_21
    if (nav(146) > 0) icheb = 1
    chebx = nav(86:96)  /con_2_13
    cheby = nav(97:107) /con_2_13
    chebz = nav(108:118)/con_2_13
    !Use JMA fit if RMS > 0
    if (nav(52) > 0) then
       spra1 = nav(45) * 1.d-6
       spra2 = nav(47) * 1.d-6
       spdc1 = nav(46) * 1.d-6
       spdc2 = nav(48) * 1.d-6
       roll  = nav(42) * 1.d-6
       pitch = nav(43) * 1.d-6
       yaw   = nav(44) * 1.d-6
    endif

    rollr = roll / drad
    pitchr = pitch / drad
    yawr = yaw / drad


    !-----------------------------------
    !  Time information
    !-----------------------------------
    ! Epoch time
    year = int(ep_date / 1000)
    year = year + 1900 * eval(year < 1900)
    jday = mod(ep_date,1000)
    ejulday = julday(1,jday,year) + &
              ftime(ep_time) / 24.d0    + &
              real(ep_msec,8) / 1000.d0 / 3600.d0 / 24.d0
    !Image time
    iyear = int(imgdate / 1000)
    iyear = iyear + 1900*eval(iyear < 1900)
    idoy = mod(imgdate,1000)
    imgjulday = julday(1,idoy,iyear) + &
                imgtime / 86400.d0
    !-----------------------------------
    !  Get SPRA & SPDC From glat/glon if missing
    !-----------------------------------
    spinmiss = 0
    if ( (spra1 == spra2) .and. (spra1 == 0.d0) .and. &
         (spdc1 == spdc2) .and. (spdc1 == 0.d0)) then
       spinmiss = 1
       !get satellite position using image time and kep2cart
       dtime_s = (imgjulday - ejulday) * 24.d0 * 3600.d0
print*,kepler,dtime_s
       call kep2cart(kepler,dtime_s,xs1,xs2,xs3)
print*,'kep2cart:',xs1,xs2,xs3
       !get subsat using cart2earth
       time_s = ejulday + dtime_s/3600.d0/24.d0
       call cart2earth(xs1,xs2,xs3,time_s,glat,glon,grad)
       !approximate using the subsatellite longitude
       if (spra1 == spra2 .and. spra1 == 0.d0) &
            spra = -glon / drad
       !approximate using the sub satellite latitude
       if (  spdc1 == spdc2 .and. &
            ((spdc1 == 0.) .or. (spdc1 == 90.d0)) )  &
            spdc = (-90. + glat) / drad
    endif

    !this is a constant that relates to # of radians per line ... but this
    !is at a different resolution that linang
    amul = 1.0986d-2 / drad

    lat = inlat / drad
    lon = inlon / drad
    !CALCULATIONS START


    !estimate the time the point was scanned
    phi1  = atan(sin(lat)/(6.61d0-cos(lat))) - pitchr
    !t_scn1 =    frtime + ((phi1/linang*rad)+cen_lin)*spper/linres
    !;time since epoch at start of image
    dtime_1 = (imgjulday - ejulday) * 24.d0 * 3600.d0 + &
            (cen_lin - (phi1/linang*drad))*spper/linres
    phiold = phi1
    dsline = 10.
    ndsline = sum(dsline)
    try = 0
if (present(debug)) print*,'Begin while',dtime(xq),spinmiss
    !iterate the i/j coordinates until the change in line is zero
    do while (ndsline /= 0)
       try = try + 1
       lasttime = dtime_1
       if ( (icheb == 1) .and. &
            (maxval(dtime_1) <= (13.d0*3600.d0) ) .and. &
            (minval(dtime_1) >= 0.d0) .and. &
            (sum(abs(chebx)) /= 0.) ) then
          !Use Chebyshev polynomials to determine Cartesian coordinates
          stop
!!$       cheb2cart,chebx,dtime,xs1
!!$       cheb2cart,cheby,dtime,xs2
!!$       cheb2cart,chebz,dtime,xs3
       else
          !Otherwise use Kepler elements
          call kep2cart(kepler,dtime_1,px,py,pz)
       end if
if (present(debug)) parts(1) = parts(1) + dtime(xq)
      
!!$       time = ejulday + dtime_1/(3600.d0*24.d0)
!!$       gra = gha(time)
       temp = (3600.d0*24.d0)
       do i=1,np
          time(i) = ejulday + dtime_1(i)/temp
       end do
       gra = gha(time)
       
       do i=1,np
          r(0,i) = px(i) / ae
          r(1,i) = py(i) / ae
          r(2,i) = pz(i) / ae
       end do

       glat = asin(within(pz(1) / sqrt(px(1)*px(1) + py(1)*py(1) + pz(1)*pz(1)),&
            &             -1.0d0,1.0d0))
       glat = atan(be*be * tan(glat) / ae*ae)
       glon = atan2(py(1),px(1))
       glon = mod((glon-gra(1)),2.d0*pi)
       if (glon < (-pi)) glon = glon + 2.*pi
       if (glon >   pi ) glon = glon - pi
       glon = glon * drad
       glat = glat * drad
       ! X = position of earth location in fixed coordinates
       !convert geographic to geocentric latitude
       p = omf2 - 1.d0
       do i=1,np
          philat = atan(omf2 * tan(lat(i)))
          !normalize longitude with greenwich rgt. asc.
          cartlon = lon(i) + gra(i)
          cphilat = cos(philat)
          sphilat = sin(philat)
          earthradius = sqrt(1.d0 + p*(sphilat*sphilat)) + height(i) / ae
          x(0,i) = earthradius * cphilat*cos(cartlon)
          x(1,i) = earthradius * cphilat*sin(cartlon)
          x(2,i) = earthradius * sphilat
!!$print*,'ae',ae
!!$print*,'be',be
!!$print*,'height',height(i)
!!$print*,10./ae,10./be,10./ae-10./be
!!$print*,earthradius
!!$print*,x(0,i),x(1,i),x(2,i)
!stop
          off(0,i) = 1.0d0 
          off(1,i) = 1.0d0 
          off(2,i) = 1.d0/omf2
       end do
if (present(debug)) parts(2) = parts(2) + dtime(xq)

       !Dotproduct to determine which pixels are visible
!!$       dotp = sum(x * r * off,dim=1)
       do i=1,np
          dotp(i) = x(0,i) * r(0,i) * off(0,i) + &
               &    x(1,i) * r(1,i) * off(1,i) + &
               &    x(2,i) * r(2,i) * off(2,i) 
       end do
if (present(debug)) parts(3) = parts(3) + dtime(xq)

       w = x - r

       if (spinmiss == 0) then
          do i=1,np
             spra_a(i) = (spra1 + dtime_1(i)/(3600.*13.) * (spra2-spra1)) / drad
             spdc_a(i) = (spdc1 + dtime_1(i)/(3600.*13.) * (spdc2-spdc1)) / drad
!!$             spra_a = spra_a / rad
!!$             spdc_a = spdc_a / rad
          end do
          if (spra2 == 0.) spra_a = spra1 / drad
          if (spdc2 == 0.) spdc_a = spdc1 / drad
       end if
       if (spinmiss == 0) then
          do i=1,np
             cspdc_a = cos(spdc_a(i))
             cspra_a = cos(spra_a(i))
             s(2,0,i) = cspdc_a *              cspra_a
             s(2,1,i) = cspdc_a * sin(spra_a(i))
             s(2,2,i) = sin(spdc_a(i))
          end do
       else
          s(2,0,:) = cos(spdc) * cos(spra)
          s(2,1,:) = cos(spdc) * sin(spra)
          s(2,2,:) = sin(spdc)
       end if
if (present(debug)) parts(4) = parts(4) + dtime(xq)
!!$       PDOTS = PX*S(2,0,:) + PY*S(2,1,:) + PZ*S(2,2,:)
!!$       PSQ = PX**2 + PY**2 + PZ**2
!!$       S1DEN = SQRT(PSQ - PDOTS**2)
!!$       S(0,0,:) = (-PX + PDOTS*S(2,0,:))/S1DEN                                     
!!$       S(0,1,:) = (-PY + PDOTS*S(2,1,:))/S1DEN
!!$       S(0,2,:) = (-PZ + PDOTS*S(2,2,:))/S1DEN
!!$       S(1,0,:)=S(2,1,:)*S(0,2,:)-S(2,2,:)*S(0,1,:)
!!$       S(1,1,:)=S(2,2,:)*S(0,0,:)-S(2,0,:)*S(0,2,:)
!!$       S(1,2,:)=S(2,0,:)*S(0,1,:)-S(2,1,:)*S(0,0,:)
       do i=1,np
          PDOTS = PX(i)*S(2,0,i) + PY(i)*S(2,1,i) + PZ(i)*S(2,2,i)
          PSQ = PX(i)*PX(i) + PY(i)*PY(i) + PZ(i)*PZ(i)
          S1DEN = SQRT(PSQ - PDOTS*PDOTS)
          S(0,0,i) = (-PX(i) + PDOTS*S(2,0,i))/S1DEN                                     
          S(0,1,i) = (-PY(i) + PDOTS*S(2,1,i))/S1DEN
          S(0,2,i) = (-PZ(i) + PDOTS*S(2,2,i))/S1DEN
          S(1,0,i)=S(2,1,i)*S(0,2,i)-S(2,2,i)*S(0,1,i)
          S(1,1,i)=S(2,2,i)*S(0,0,i)-S(2,0,i)*S(0,2,i)
          S(1,2,i)=S(2,0,i)*S(0,1,i)-S(2,1,i)*S(0,0,i)
       end do
if (present(debug)) parts(5) = parts(5) + dtime(xq)
       !=======
       !   VC
       !=======
       vc = w * ae
       !=======
       !   VS = s#vc
       !=======
       do i=1,np
          vs(0,i) = vc(0,i)*s(0,0,i) + vc(1,i)*s(0,1,i) + vc(2,i)*s(0,2,i)
          vs(1,i) = vc(0,i)*s(1,0,i) + vc(1,i)*s(1,1,i) + vc(2,i)*s(1,2,i)
          vs(2,i) = vc(0,i)*s(2,0,i) + vc(1,i)*s(2,1,i) + vc(2,i)*s(2,2,i)
       end do
if (present(debug)) parts(6) = parts(6) + dtime(xq)
       !vs1 = vs(0,:) ; vs2 = vs(1,:) ; vs3 = vs(2,:)
       TANYAW = TAN( YAWR )
       COSYAW = COS( YAWR )
       do i=1,np
          SIGMA = ATAN( VS(1,i) / VS(0,i) )
          if ((VS(0,i)*VS(0,i) + VS(1,i)*VS(1,i) - VS(2,i)*VS(2,i) * TANYAW*TANYAW ) < 0) then
             print*,i,try
             print*,lat(i),lon(i)
             print*,vs(0:2,i),tanyaw,yawr, yaw
          endif
          XI = ATAN( VS(2,i) * TANYAW / &
                     SQRT( VS(0,i)*VS(0,i) + VS(1,i)*VS(1,i) - VS(2,i)*VS(2,i) * TANYAW*TANYAW ) )
          PHInew(i) = ATAN( -VS(2,i) / ( COSYAW * COS(XI) * &
               &                SQRT(VS(0,i)*VS(0,i)+VS(1,i)*VS(1,i)))) &
               &      - PITCHr
          theta(i) = xi + sigma - rollr
       end do
if (present(debug)) parts(7) = parts(7) + dtime(xq)
       !;time since epoch at start of image
       dtime_1 = (imgjulday - ejulday) * 24.d0 * 3600.d0 + &
               (cen_lin - (phinew/linang*drad))/(linres)*spper
       
       !Check iteration of the line number
       dsline = int( ( (phiold/amul*linres) - (phinew/amul*linres) ) * 10 )
       ndsline = count(dsline /= 0)
       phiold = phinew
if (present(debug)) parts(8) = parts(8) + dtime(xq)
       if (try > 1000) then
          print*,'No Convergence'
          stop
       end if
       !if (present(debug)) print*,try,dtime(xq)
    end do
if (present(debug)) print*,'End while',dtime(xq)
do i=1,8
if (present(debug))    print*,i,parts(i)
end do

!!$    ipoints = cen_pix + theta /pixang*rad
!!$    jpoints = cen_lin - phinew/linang*rad
!!$    ipoints = (ipoints - spix)  / real(pixres)
!!$    jpoints = (jpoints - sline) / real(linres)
    !This combines the above 4 lines into 2 lines
    !ipoints = (cen_pix + theta /pixang*drad - spix) / real(pixres)
    !jpoints = (cen_lin - phinew/linang*drad -sline) / real(linres)
    !The following attempts to incorporate the SKEW into calculations
    !dline = phinew/linang*drad !dline = number of lines from the center
    !ipoints = (cen_pix + theta /pixang*drad - spix - skew_fact*dline) / real(pixres)

    !The following incorporates the variable cen_pix_img
!!$    jpoints = cen_lin_img - (phinew/linang*drad + sline)/real(linres)
!!$    cen_pix_img = c0 + c1 * jpoints + c2 * jpoints * jpoints
!!$    ipoints = cen_pix_img + (theta/pixang*drad - spix) / real(pixres)
    jpoints = cen_lin_img - (phinew/linang*drad)/real(linres,8)
    cen_pix_img = c0 + c1 * jpoints + c2 * jpoints * jpoints
    if (centerflag == 1) then
       allocate(xtemp(ny))
       allocate(cpi(ny))
       do i=1,int(ny)
          xtemp(i) = (i-1)-ny/2.
       end do
       cpi =  center(1) + &
            & center(2) * xtemp + &
            & center(3) * xtemp*xtemp + &
            & center(4) * xtemp*xtemp*xtemp
       do i=1,np
          if (within(int(jpoints(i)),1,int(ny)) < 1) then
             print*,ny
             print*,jpoints(i),int(jpoints(i))
          endif
          cen_pix_img(i) = cpi(within(int(jpoints(i)),1,int(ny)))
       end do
       deallocate(xtemp,cpi)       
    endif
    ipoints = cen_pix_img + (theta /pixang*drad)/real(pixres,8)
    where (dotp < 1.)
       ipoints = -9999.
       jpoints = -9999.
    end where

    !get vza
    if (present(vza)) then
       n = size(ipoints)
       do i=1,n
          if (ipoints(i) >= 0) then
             vza(i) =  acos(within(dot(unit(reshape(x(:,i),(/3/)) ), &
                                      -1.*unit(reshape(vc(:,i),(/3/))) ) ,&
                                 &    -1.0d0,1.0d0)) * drad
          end if
       end do
    end if

    if (present(subsat)) subsat = (/glat, glon, &
                                    sqrt(px(1)*px(1) + py(1)*py(1) + pz(1)*pz(1)) /)

    return
  end subroutine b1u_ll2ij_kepler_rank1
!                                                          END B1U_LL2IJ
!#######################################################################
!#######################################################################
!                                        BEGIN VIEW_ZENITH_CORRECT_ISCCP
!****f* kmod_isccp/view_zenith_correct_isccp
! NAME
! view_zenith_correct_isccp
! SYNOPSIS
! temp_correct = view_zenith_correct_isccp(BTemp,VZA)
! FUNCTION
! Corrects the brightness temperature of the IR Window channel
! based on theory.
! INPUTS
! * BTemp - real*4(rank2, rank1, scalar) - Brightness Temperature in Kelvin
! * VZA   - real*4(same rank) - view zenith in degrees
! OUTPUT
! temp_correct - Corrected brightness temperature in Kelvin
!***

  function vzci_rank2 &
       ( temp, vza ) result (correct)

    !External
    real(kind=4), intent(in), dimension(:,:) :: temp, vza
    real(kind=4), dimension(size(temp,1),size(temp,2)) :: correct
    !Internal
    real(kind=4), dimension(size(temp,1),size(temp,2)) :: mu !,c0, c1


    mu = cos(vza/rad)
!    c0 = - (1.930 + 2.520*mu) * (1./mu - mu) / 4.8
!    c1 =   (0.267 + 0.053*mu) * (1./mu - mu) / 4.8
    correct = temp
    correct = correct &
         - (1.930 + 2.520*mu) * (1./mu - mu) / 4.8
    correct = correct &
         + (  (0.267 + 0.053*mu) * (1./mu - mu) / 4.8) * (temp - 250.)
    where (temp < 1.)
       correct = temp
    end where
  end function vzci_rank2

  function vzci_rank1 &
       ( temp, vza ) result (view_zenith_correct_isccp)

    !External
    real(kind=4), dimension(:) :: temp, vza
    real(kind=4), dimension(size(temp)) :: view_zenith_correct_isccp
    !Internal
    real(kind=4), dimension(size(temp)) :: c0, c1, mu

    mu = cos(vza/rad)
    c0 = - (1.930 + 2.520*mu) * (1./mu - mu) / 4.8
    c1 =   (0.267 + 0.053*mu) * (1./mu - mu) / 4.8
    view_zenith_correct_isccp = &
         temp + c0 + c1 * (temp - 250.)
    where (temp < 1.)
       view_zenith_correct_isccp = temp
    end where
  end function vzci_rank1

  real(kind=4) function vzci_scalar &
       ( temp, vza )

    !External
    real(kind=4) :: temp, vza
    !Internal
    real(kind=4) :: c0, c1, mu

    mu = cos(vza/rad)
    c0 = - (1.930 + 2.520*mu) * (1./mu - mu) / 4.8
    c1 =   (0.267 + 0.053*mu) * (1./mu - mu) / 4.8
    vzci_scalar = &
         temp + c0 + c1 * (temp - 250.)
    if (temp < 1.) vzci_scalar = temp
  end function vzci_scalar
!                                          END VIEW_ZENITH_CORRECT_ISCCP
!#######################################################################


  



end module kmod_isccp
