module kmod_math
!****h* /kmod_math
! NAME
! kmod_math
! SYNOPSIS
! Useful mathematical routines
! USES
! kmod_utilities
!***

  use kmod_utilities

  implicit none

  INTERFACE dot
     MODULE PROCEDURE dot_real4
     MODULE PROCEDURE dot_real8
     MODULE PROCEDURE dot_real8_r2
     MODULE PROCEDURE dot_int4
  END INTERFACE
  INTERFACE idl_moment
     MODULE PROCEDURE idl_moment_rank2_r4
     MODULE PROCEDURE idl_moment_rank1_r4
     MODULE PROCEDURE idl_moment_rank2_i4
     MODULE PROCEDURE idl_moment_rank2_i2
  END INTERFACE
  INTERFACE median
     MODULE PROCEDURE median_rank2_r4
  END INTERFACE
  INTERFACE sincos
     MODULE PROCEDURE sincos_scalar_r4
     MODULE PROCEDURE sincos_rank1_r4
     MODULE PROCEDURE sincos_rank1_r8
     MODULE PROCEDURE sincos_rank2_r4
     MODULE PROCEDURE sincos_rank2_r8
  END INTERFACE
  INTERFACE spatstdev
     MODULE PROCEDURE spatstdev_real
     MODULE PROCEDURE spatstdev_int
  END INTERFACE
  INTERFACE stdev
     MODULE PROCEDURE stdev_rank1_r4
     MODULE PROCEDURE stdev_rank1_i4
     MODULE PROCEDURE stdev_rank1_i2
     MODULE PROCEDURE stdev_rank2_i2
     MODULE PROCEDURE stdev_rank3_i2     
  END INTERFACE
  INTERFACE unit
     MODULE PROCEDURE unit_real4
     MODULE PROCEDURE unit_real8
     MODULE PROCEDURE unit_real8_r2
  END INTERFACE

contains

!#######################################################################
!                                                           BEGIN AVEVAR
!****r* kmod_math/avevar
! NAME
! avevar
! SYNOPSIS
! call avevar(data, n, ave, var, skew)
! FUNCTION
! Calculate moments of a data set.
! INPUTS
! data - real*4(n) - Data from which to determine statistical moments
! n - int*4 - Number of points
! OUTPUT
! ave - real*4 - average of data
! var - real*4 - variance of data
! skew- real*4 - skewness of data (optional)
! NOTES
!***
  subroutine avevar(data, n, ave, var, skew)
    integer n
    real data(n), ave, var
    real, optional :: skew
    ave = 0.0
    ave = sum(data) / real(n)
    var = 0.
    var = sum((data-ave)*(data-ave))/real(n-1)
    if (present(skew)) then
       skew = sum((data-ave)*(data-ave)*(data-ave)) / (float(n-1)*sqrt(var)**3)
    end if
  end subroutine avevar
!                                                             END AVEVAR
!#######################################################################
!#######################################################################
!                                                              BEGIN DOT
!****f* kmod_utilities/dot
! NAME
! dot
! SYNOPSIS
! dot = dot(a, b)
! FUNCTION
! Computes the dot product of two arrays
! INPUTS
! * a
! * b
!   * r*4 - rank1
!   * r*8 - rank1
!   * i*4 - rank1
!   * r*8 - rank2
! 
! RESULT
! * dot_product - The dot product of a and b
! NOTES
! * If a,b are rank2, then it computes the dot product by
!   output = sum(a * b,dim=1)
!
!***
  real(kind=4) function dot_real4 ( a, b )
    !external variables
    real(kind=4),intent(in),dimension(:) :: a, b
    dot_real4 = sum(a * b)
  end function dot_real4
  real(kind=8) function dot_real8 ( a, b )
    !external variables
    real(kind=8),intent(in),dimension(:) :: a, b
    dot_real8 = sum(a * b)
  end function dot_real8

  function dot_real8_r2 ( a, b ) result ( output )
    !external variables
    real(kind=8),intent(in),dimension(:,:) :: a, b
    real(kind=8), dimension(size(a,2)) :: output
    ! a & b should be of the form(2,:) or (3,:)
    output = sum(a * b,dim=1)
  end function dot_real8_r2

  integer(kind=4) function dot_int4 ( a, b )
    !external variables
    integer(kind=4),intent(in),dimension(:) :: a, b
    dot_int4 = sum(a * b)
  end function dot_int4
!                                                                END DOT
!#######################################################################
!#######################################################################
!                                                       BEGIN IDL_MOMENT
! Based on IDL intrinsic function moment.pro
!****r* kmod_math/idl_moment
! NAME
! idl_moment
! SYNOPSIS
! result = moment(values,mask)
! FUNCTION
! Mimics the IDL routine moment.
! INPUTS
! * values - i*2(:,:), i*4(:,:), r*4(:), r*4(:,:) - input values
! * mask - logical - same dimension as value (optional)
! RESULT
! * r*4(4)
! * Result(1) = mean
! * Result(2) = standard deviation (n-1)
! * Result(3) = skewness
! * Result(4) = kurtosis
!***

  function idl_moment_rank2_i2 ( input, mask ) result (output)
    !Externals
    integer(kind=2), intent(in), dimension(:,:) :: input
    logical,         intent(in), dimension(:,:), optional :: mask
    real(kind=4), dimension(4) :: output
    !Internals
    real(kind=4), dimension(size(input,1),size(input,2)) :: newinput
    
    newinput = real(input,4)
    output = idl_moment(newinput,mask)
    return
  end function idl_moment_rank2_i2
  function idl_moment_rank2_i4 ( input, mask ) result (output)
    !Externals
    integer(kind=4), intent(in), dimension(:,:) :: input
    logical,         intent(in), dimension(:,:), optional :: mask
    real(kind=4), dimension(4) :: output
    !Internals
    real(kind=4), dimension(size(input,1),size(input,2)) :: newinput
    
    newinput = real(input,4)
    output = idl_moment(newinput,mask)
    return
  end function idl_moment_rank2_i4
  function idl_moment_rank1_r4 ( input, mask ) result (output)
    !Externals
    real(kind=4), intent(in), dimension(:) :: input
    logical,      intent(in), dimension(:), optional :: mask
    real(kind=4), dimension(4) :: output
    !Internals
    real(kind=4), dimension(1,size(input)) :: newinput
    logical,      dimension(1,size(input)) :: newmask
    newinput(1,:) = input
    if (present(mask)) then
       newmask(1,:) = mask
    else
       newmask(:,:) = .true.
    endif
    output = idl_moment(newinput,newmask)
    return
  end function idl_moment_rank1_r4
  !-----------------------------------------------------------
  function idl_moment_rank2_r4 ( input, mask ) result (output)
    !Externals
    real(kind=4), intent(in), dimension(:,:) :: input
    logical,      intent(in), dimension(:,:), optional :: mask
    real(kind=4), dimension(4) :: output
    !Internals
    real(kind=8) :: nx
    real(kind=8), dimension(size(input,1),size(input,2)) :: x, resid
    real(kind=8) :: mean, stdev, skew, kurt, var
    logical, dimension(size(input,1),size(input,2)) :: imask

    mean = -99.0d0
    stdev= -99.0d0
    skew = -99.0d0
    kurt = -99.0d0

    if (present(mask)) then
       imask = mask
    else
       imask = .true.
    endif

    nx = count(imask)
    if (nx < 2) then
       print*,'Not enough points:',size(input)
    endif
    x = real(input,8)
    mean = sum(x,mask=imask)/nx
    resid = x - mean
    ! Numerically-stable "two-pass" formula, which offers less
    ! round-off error. Page 613, Numerical Recipes in C.
    var = (sum(resid*resid,mask=imask) - &
         &(sum(resid,mask=imask)*&
         & sum(resid,mask=imask))/nx)/(nx-1.0d0)
    !var = (sum(resid*resid,mask=imask)/(nx-1.))
    stdev = sqrt(within(var,minimum=0.0d0))
    if (stdev > 0.d0) then
       skew = sum(resid**3,mask=imask) / (nx * stdev**3)
       kurt = sum(resid**4,mask=imask) / (nx * stdev**4) - 3.0d0
    endif
    
    output = real( (/mean, stdev, skew, kurt/), 4)
    return
  end function idl_moment_rank2_r4
!                                                         END IDL_MOMENT
!#######################################################################
!#######################################################################
!                                                           BEGIN MEDIAN
!****r* kmod_math/median
! NAME
! median
! SYNOPSIS
! result = median(values,mask)
! FUNCTION
! Calculates the median
! INPUTS
! * values - r*4(:,:) - input alies
! * mask - logical - same dimension as value (optional)
! RESULT
! * idl_where
! * qsort
!***
  real function median_rank2_r4(input,mask)
    !Externals
    real(kind=4), dimension(:,:) :: input
    logical, dimension(:,:), optional :: mask
    !Internals
    real(kind=4), dimension(size(input)) :: input_r1
    logical,      dimension(size(input)) :: mask_r1

    integer(kind=4) :: n
    integer(kind=4), allocatable, dimension(:) :: s, ind
    real(kind=4), dimension(:), allocatable :: input_m

    n = size(input)
    input_r1 = reshape(input, (/ n /) )
    if (present(mask)) then
       call idl_where(mask_r1,ind,n)
       allocate (input_m(n),s(n))
       input_m(1:n) = input_r1(ind(1:n))
       call qsort(input_m,s,n)
       print*,n,nint(n*0.5),s(nint(n*0.5)),count(s == 0)
       median_rank2_r4 = input_m(s(nint(n*0.5)))
    else
       allocate (s(n))
       call qsort(input_r1,s,n)
       median_rank2_r4 = input_r1(s(nint(n*0.5)))
    endif

  end function median_rank2_r4
!                                                             END MEDIAN
!#######################################################################
!#######################################################################
!                                                           BEGIN SINCOS
!****r* kmod_math/sincos
! NAME
! sincos
! SYNOPSIS
! call sincos(angle,sina,cosa)
! FUNCTION
!    Calculates sine and cosine of angle faster than doing it separately.
! INPUTS
!  * angle - degrees
!    * real*4: scalar, rank1, rank2
!    * real*8: rank1, rank2
! OUTPUT
!  Same rank and type as angle:
!  * sina - sine(angle)
!  * cosa - cosine(angle)
! NOTES
! This routine was created to speed up routines which use both
! sine and cosine of a set of angles. It is faster than performing
! the calculations separately since the cosine is used, then
! the sine is calculated from the result of the cosine. However,
! it may not be faster when using the scalar, it is much faster 
! when the array is large.
!***
  subroutine sincos_scalar_r4(ang,sinang,cosang)
    real(kind=4) :: ang, sinang, cosang
    real(kind=4) :: da
    real pi,twopi
    pi = 3.14159265358979323846e0
    twopi=2.0e0*pi
    cosang = cos(ang)
    sinang = sqrt(1.0e0 - cosang*cosang)
    da = ang - floor(ang/(twopi))*(twopi)
    if (da > pi) sinang = -sinang
  end subroutine sincos_scalar_r4

  subroutine sincos_rank1_r4(ang,sinang,cosang)
    real(kind=4), dimension(:) :: ang, sinang, cosang
    real(kind=4), dimension(size(ang)) :: da
    real pi,twopi
    pi = 3.14159265358979323846e0
    twopi=2.0e0*pi
    cosang = cos(ang)
    sinang = sqrt(1.0e0 - cosang*cosang)
    da = ang - floor(ang/(twopi))*(twopi)
    where (da > pi) sinang = -sinang
  end subroutine sincos_rank1_r4

  subroutine sincos_rank1_r8(ang,sinang,cosang)
    real(kind=8), dimension(:) :: ang, sinang, cosang
    real(kind=8), dimension(size(ang)) :: da
    real pi,twopi
    pi = 3.14159265358979323846d0
    twopi=2.0d0*pi
    cosang = cos(ang)
    sinang = sqrt(1.0d0 - cosang*cosang)
    da = ang - floor(ang/(twopi))*(twopi)
    where (da > pi) sinang = -sinang
  end subroutine sincos_rank1_r8

  subroutine sincos_rank2_r4(ang,sinang,cosang)
    real(kind=4), dimension(:,:) :: ang, sinang, cosang
    real(kind=4), dimension(size(ang,1),size(ang,2)) :: da
    real pi,twopi
    pi = 3.14159265358979323846e0
    twopi=2.0e0*pi
    cosang = cos(ang)
    sinang = sqrt(1.0e0 - cosang*cosang)
    da = ang - floor(ang/(twopi))*(twopi)
    where (da > pi) sinang = -sinang
  end subroutine sincos_rank2_r4

  subroutine sincos_rank2_r8(ang,sinang,cosang)
    real(kind=8), dimension(:,:) :: ang, sinang, cosang
    real(kind=8), dimension(size(ang,1),size(ang,2)) :: da
    real pi,twopi
    pi = 3.14159265358979323846d0
    twopi=2.0d0*pi
    cosang = cos(ang)
    sinang = sqrt(1.0d0 - cosang*cosang)
    da = ang - floor(ang/(twopi))*(twopi)
    where (da > pi) sinang = -sinang
  end subroutine sincos_rank2_r8
  
!                                                             END SINCOS
!#######################################################################
!#######################################################################
!                                                           BEGIN SMOOTH
!****f* kmod_math/smooth
! NAME
! smooth
! SYNOPSIS
! sd = smooth( input, width, flag, boxcar, binomial ) 
! FUNCTION
! This function smooths input data using either boxcar or binomial smoothing.
! INPUTS
! input - real*4 - rank1 - The input data to be smoothed.
! width - integer*4 - Width of the boxcar/polynomial smoothing operators
! Optional Inputs:
! * flag - logical - rank1 - Ignore input values where flag is false
! * For boxcar smoothing, set BOXCAR=1
! * For binomial smoothing, set BINOMIAL=1
! OUTPUT
! The returned array is the smoothed array.
! USED BY
! b1u_get_qc
!***
    function smooth ( input, width, flag, boxcar, binomial ) result (output)
      !Externals
      real(kind=4), dimension(:), intent(in) :: input
      integer(kind=4) :: width
      logical(kind=4), dimension(:), intent(in), optional :: flag
      integer(kind=4), optional :: boxcar, binomial
      real(kind=4), dimension(size(input)) :: output
      !Internals
      integer :: iw, w, k, i, n, j
      integer(kind=4), dimension(size(input)) :: f,den1
      real(kind=4), dimension(size(input))    :: temp, num

      n = size(input)
      f = 1
      if (present(flag)) f = eval(flag)

      if (present(binomial)) then
         !Binomial smoothing
         !This is simply a 2-point boxcar applied WIDTH times
         w = width
         k = 1
         output = input
         do iw=1,w
            num = 0.
            den1 = 0
            if (k < 0) then
               temp(1) = output(1)
               do i=2,n
                  do j = 0,1
                     num(i) = num(i) + output(i-j) * f(i-j)
                     den1(i)=den1(i) + f(i-j)
                  end do
               end do
            else
               temp(n) = output(n)
               do i=1,n-1
                  do j=0,1
                     num(i) = num(i) + output(i+j) * f(i+j)
                     den1(i)=den1(i) + f(i+j)
                  end do
               end do
            end if
            f(:) = 1
            do i=1,n
               output(i) = num(i) / max(float(den1(i)),1.)
               if (den1(i) == 0) f(i) = 0
            end do
            
            k = -1 * k
         end do
      else
         !Boxcar smooth by default
         !Width should be odd
         den1(1:n) = 0.
         w = width/2
         do iw=-w,w
            if (iw < 0) then
               num(1:n+iw) = num(1:n+iw) + &
                    &        input(1-iw:n)*f(1-iw:n)
               den1(1-iw:n)= den1(1-iw:n) + f(1-iw:n)
            else
               num(1+iw:n) = num(1+iw:n) + &
                    &      input(1:n-iw)*f(1:n-iw)
               den1(1:n-iw)= den1(1:n-iw) + f(1:n-iw)
            end if
         end do
         
         do i=1,n
            output(i) = num(i) / max(float(den1(i)),1.0)
         end do
         where (f == 0) output = 1.
      end if
    end function smooth
!                                                             END SMOOTH
!#######################################################################
!#######################################################################
!                                                        BEGIN SPATSTDEV
!****f* kmod_math/spatstdev
! NAME
! spatstdev
! SYNOPSIS
! sd = spatstdev(image,nin)
! FUNCTION
! Returns the spatial standard deviation at each location in image. The
! size of the box is defined by +/- nin/2 (so nin=5 is +/- 2)
! INPUTS
! image = real*4(rank2)
! nin   = int*4
! mask (OPTIONAL) - logical (same size) - only use points which are true
! OUTPUT
! sd = real*4(rank2)
! HISTORY
! 2007Oct18 - Modifying to accept REAL and INT arrays as well as a QC mask
!***
  function spatstdev_int ( array, nin, mask ) result (output)
    integer(kind=2), dimension(:,:) :: array
    integer(kind=4) :: nin
    logical, dimension(:,:), optional :: mask
    real(kind=4), dimension(size(array,1),size(array,2)) :: output

    output = spatstdev_real(real(array,4),nin,mask=mask)

  end function spatstdev_int

  function spatstdev_real ( array, nin, mask ) result (output)
    real(kind=4), dimension(:,:) :: array
    integer(kind=4) :: nin
    logical, dimension(:,:), optional :: mask
    real(kind=4), dimension(size(array,1),size(array,2)) :: output
    
    integer :: n, nx, ny, n2, ii, jj
    logical, dimension(size(array,1),size(array,2)) :: inmask
    real(kind=8), dimension(size(array,1),size(array,2)) :: sumn, sumx, sumxx
    real, dimension(:,:), allocatable :: omask, var


    if (present(mask)) then
       inmask=mask
    else
       inmask(:,:) = .true.
    endif

    n = nin
    if (mod(n,2) == 0) n = n + 1
    n2 = n/2
    nx = size(array,1)
    ny = size(array,2)
    allocate (var(nx+2*n2,ny+2*n2), omask(nx+2*n2,ny+2*n2))
    omask = 0
    var = 0.
    var(n2+1:n2+nx,n2+1:n2+ny) = array
    omask(n2+1:n2+nx,n2+1:n2+ny) = eval(inmask)
    sumn = 0.
    sumx = 0.
    sumxx = 0.
    do ii=-n2,n2
       do jj=-n2,n2
          sumn = sumn + omask(n2+1+ii:n2+nx+ii,n2+1+jj:n2+ny+jj)
          sumx = sumx + omask(n2+1+ii:n2+nx+ii,n2+1+jj:n2+ny+jj)* var(n2+1+ii:n2+nx+ii,n2+1+jj:n2+ny+jj)
          sumxx= sumxx+ omask(n2+1+ii:n2+nx+ii,n2+1+jj:n2+ny+jj)*(var(n2+1+ii:n2+nx+ii,n2+1+jj:n2+ny+jj)**2)
       end do
    end do

    output=-1.
    where(inmask) output = sqrt((sumxx*sumn - sumx**2) / (sumn-1) / sumn)
    deallocate (var, omask)
  end function spatstdev_real
!                                                          END SPATSTDEV
!#######################################################################
!#######################################################################
!                                                            BEGIN STDEV
!****f* kmod_math/stdev
! NAME
! stdev
! SYNOPSIS
! s = stdev(data)
! INPUTS
! data = real*4(rank1) or int*4(rank1) or int*2(rank1,rank2 or rank3)
! OUTPUT
! s = real*4 = standard deviation of data
!***
  real(kind=4) function stdev_rank1_r4 ( array )
    !External variables
    real(kind=4), dimension(:) :: array
    !Internal
    real(kind=8) :: sumx, sumx2, nx
    sumx = sum(real(array,8))
    sumx2 = sum(real(array,8)*real(array,8))
    nx = size(array)
    stdev_rank1_r4 = sqrt((nx*sumx2 - sumx**2) / (nx*(nx-1))  )
  end function stdev_rank1_r4

  real(kind=4) function stdev_rank1_i4 ( array )
    !External variables
    integer(kind=4), dimension(:) :: array
    !Internal
    real(kind=8) :: sumx, sumx2, nx
    sumx = sum(real(array,8))
    sumx2 = sum(real(array,8)*real(array,8))
    nx = size(array)
    stdev_rank1_i4 = sqrt((nx*sumx2 - sumx**2) / (nx*(nx-1))  )
  end function stdev_rank1_i4

  real(kind=4) function stdev_rank1_i2 ( array )
    !External variables
    integer(kind=2), dimension(:) :: array
    !Internal
    real(kind=8) :: sumx, sumx2, nx
    sumx = sum(real(array,8))
    sumx2 = sum(real(array,8)*real(array,8))
    nx = size(array)
    stdev_rank1_i2 = sqrt((nx*sumx2 - sumx**2) / (nx*(nx-1))  )
  end function stdev_rank1_i2

  real(kind=4) function stdev_rank2_i2 ( array )
    !External variables
    integer(kind=2), dimension(:,:) :: array
    !Internal
    real(kind=8) :: sumx, sumx2, nx
    sumx = sum(real(array,8))
    sumx2 = sum(real(array,8)*real(array,8))
    nx = real(size(array),8)
    stdev_rank2_i2 = sqrt((nx*sumx2 - sumx**2) / (nx*(nx-1))  )
  end function stdev_rank2_i2

  real(kind=4) function stdev_rank3_i2 ( array )
    !External variables
    integer(kind=2), dimension(:,:,:) :: array
    !Internal
    real(kind=8) :: sumx, sumx2, nx
    sumx = sum(real(array,8))
    sumx2 = sum(real(array,8)*real(array,8))
    nx = size(array)
    stdev_rank3_i2 = sqrt((nx*sumx2 - sumx**2) / (nx*(nx-1))  )
  end function stdev_rank3_i2
!                                                              END STDEV
!#######################################################################
!#######################################################################
!                                                             BEGIN UNIT
!****f* kmod_math/unit
! NAME
! unit
! SYNOPSIS
! out = unit(array)
! FUNCTION
! Creates a unit vector from an array.
! INPUTS
! * array - real*4(:), real*8(:), real*8(:,:)
! OUTPUT
! * out - same type and rank as array
!***
  function unit_real4 ( a ) result ( output )
    !External variables
    real (kind=4), dimension(:), intent(in) :: a
    real (kind=4), dimension(size(a)) :: output
    output = a / sqrt(sum(a*a))
  end function unit_real4

  function unit_real8 ( a ) result ( output )
    !External variables
    real (kind=8), dimension(:), intent(in) :: a
    real (kind=8), dimension(size(a)) :: output
    output = a / sqrt(sum(a*a))
  end function unit_real8

  function unit_real8_r2 ( a ) result ( output )
    !External variables
    real (kind=8), dimension(:,:), intent(in) :: a
    real (kind=8), dimension(size(a,1),size(a,2)) :: output
    output = a / transpose(spread(sqrt(sum(a*a,dim=1)),2,3))
  end function unit_real8_r2
    
!                                                               END UNIT
!#######################################################################

  

end module kmod_math
