module kmod_mcidas
!****h* /kmod_mcidas
! NAME
! kmod_mcidas
! SYNOPSIS
! Used for reading and navigating McIDAS Area format data
! USES
! kmod_constants
! kmod_math
!***

  use kmod_constants
  use kmod_math
  use kmod_utilities

  implicit none
  
!****s* kmod_mcidas/gvar_area
! NAME
! gvar_area
! SYNOPSIS
! Format and content for the McIDAS AREA file format file header.
! SOURCE

  TYPE gvar_area
     SEQUENCE
     integer(kind=4) :: area_status
     integer(kind=4) :: version_number
     integer(kind=4) :: satellite_id
     integer(kind=4) :: img_date
     integer(kind=4) :: img_time
     integer(kind=4) :: north_bound
     integer(kind=4) :: west_bound
     integer(kind=4) :: z_coor
     integer(kind=4) :: num_line
     integer(kind=4) :: num_elem
     integer(kind=4) :: bytes_per_pixel
     integer(kind=4) :: line_res
     integer(kind=4) :: elem_res
     integer(kind=4) :: num_chan
     integer(kind=4) :: num_byte_line_pre
     integer(kind=4) :: project_number
     integer(kind=4) :: creation_date
     integer(kind=4) :: creation_time
     integer(kind=4) :: channel_map
     integer(kind=4) :: image_id_number
     integer(kind=4), dimension(4) :: id
     integer(kind=1), dimension(32) :: comment
     integer(kind=4) :: primary_key_calib
     integer(kind=4) :: primary_key_nav
     integer(kind=4) :: secondary_key_nav
     integer(kind=4) :: val_code
     integer(kind=4), dimension(8) :: pdl
     integer(kind=4) :: band8
     integer(kind=4) :: act_img_date
     integer(kind=4) :: act_img_time
     integer(kind=4) :: act_start_scan
     integer(kind=4) :: len_prefix_doc
     integer(kind=4) :: len_prefix_calib
     integer(kind=4) :: len_prefix_lev
     integer(kind=1), dimension(4) :: src_type
     integer(kind=1), dimension(4) :: calib_type
     integer(kind=4) :: average_or_sample
     integer(kind=4) :: poes_signal
     integer(kind=4) :: poes_up_down
     integer(kind=1), dimension(4) :: original_source_type
     integer(kind=4), dimension(7) :: reserved
  END TYPE gvar_area
!***
  !Define GVAR Navigation Header
!****s* kmod_mcidas/gvar_sinusoid
! NAME
! gvar_sinusoid
! SYNOPSIS
! Format and content for sinusoid portion of the McIDAS GVAR
! Navigation Header.
! SOURCE

  TYPE gvar_sinusoid
     SEQUENCE
     integer(kind=4) :: magnitude
     integer(kind=4) :: phase_angle
  END TYPE gvar_sinusoid
!***
!****s* kmod_mcidas/gvar_monomial
! NAME
! gvar_monomial
! SYNOPSIS
! Format and content for monomial portion of the McIDAS GVAR
! Navigation Header.
! SOURCE

  TYPE gvar_monomial
     SEQUENCE
     integer(kind=4) :: order_applicable_sinusoid
     integer(kind=4) :: order_monomial_sinusoid
     integer(kind=4) :: magnitude_monomial_sinusoid
     integer(kind=4) :: phase_angle_monomial_sinusoid
     integer(kind=4) :: angle_from_epoch
  END TYPE gvar_monomial
!***
!****s* kmod_mcidas/gvar_angle_information
! NAME
! gvar_angle_information
! SYNOPSIS
! Format and content for angular information within the McIDAS GVAR
! Navigation Header. It combines the monomial and sinusoidal
! components.
! SOURCE

  TYPE gvar_angle_information
     SEQUENCE
     integer(kind=4) :: exponential_magnitude
     integer(kind=4) :: exponential_time_constant
     integer(kind=4) :: mean_attitude_angle
     integer(kind=4) :: number_of_sinusoids
     type(gvar_sinusoid), dimension(15) :: sinusoid
     integer(kind=4) :: number_of_monomials
     type(gvar_monomial), dimension(4) :: monomial
  END TYPE gvar_angle_information
!***
!****s* kmod_mcidas/gvar_nav
! NAME
! gvar_nav
! SYNOPSIS
! Format and content for the McIDAS AREA file format file header.
! navigation header for GVAR data.
! NOTES
! SOURCE

  TYPE gvar_nav
     SEQUENCE
     integer(kind=1), dimension(4) :: navigation_type
     integer(kind=1), dimension(4) :: ID
     integer(kind=4) :: imc_status
     integer(kind=4) :: flip
     integer(kind=4) :: spare1
     integer(kind=4) :: ref_longitude
     integer(kind=4) :: ref_radius
     integer(kind=4) :: ref_latitude
     integer(kind=4) :: ref_orbit_yaw
     integer(kind=4) :: ref_att_roll
     integer(kind=4) :: ref_att_pitch
     integer(kind=4) :: ref_att_yaw
     integer(kind=1), dimension(8) :: epoch_time
     integer(kind=4) :: start_time
     integer(kind=4) :: imc_corr_roll
     integer(kind=4) :: imc_corr_pitch
     integer(kind=4) :: imc_corr_yaw
     integer(kind=4), dimension(13) :: ref_longitude_change
     integer(kind=4), dimension(11) :: ref_radius_change
     integer(kind=4), dimension(9) :: sine_latitude
     integer(kind=4), dimension(9) :: sine_orbital_yaw
     integer(kind=4) :: solar_rate
     integer(kind=4) :: exponential_start_time
     type(gvar_angle_information) :: roll_att
     integer(kind=4), dimension(10) :: spare2
     integer(kind=1), dimension(4) :: more1
     integer(kind=1), dimension(4) :: gvar1
     type(gvar_angle_information) :: pitch_att
     type(gvar_angle_information) :: yaw_att
     integer(kind=4), dimension(16) :: spare3
     integer(kind=1), dimension(4) :: more2
     integer(kind=1), dimension(4) :: gvar2
     type(gvar_angle_information) :: roll_misalign
     type(gvar_angle_information) :: pitch_misalign
     integer(kind=4) :: img_date
     integer(kind=4) :: img_time
     integer(kind=4) :: instrument
     integer(kind=4), dimension(9) :: spare4
     integer(kind=4) :: ns_cyl
     integer(kind=4) :: ew_cyl
     integer(kind=4) :: ns_inc
     integer(kind=4) :: ew_inc
     integer(kind=1), dimension(4) :: more3
     integer(kind=1), dimension(4) :: gvar3
     integer(kind=4), dimension(126) :: spare5
     integer(kind=1), dimension(4) :: more4
     integer(kind=1), dimension(4) :: gvar4
     integer(kind=4), dimension(127) :: spare6
  END TYPE gvar_nav
!***
!****s* kmod_mcidas/visr_nav
! NAME
! gvar_area
! SYNOPSIS
! Format and content for the McIDAS AREA file format navigation
! header for VISR data.
! SOURCE
! 
  TYPE visr_nav
     SEQUENCE
     character(len=4) :: type
     integer(kind=4) :: iddate
     integer(kind=4) :: time
     integer(kind=4) :: orbit
     integer(kind=4) :: etimy
     integer(kind=4) :: etimh
     integer(kind=4) :: semima
     integer(kind=4) :: eccen
     integer(kind=4) :: orbinc
     integer(kind=4) :: meana
     integer(kind=4) :: perigee
     integer(kind=4) :: asnode
     integer(kind=4) :: declin
     integer(kind=4) :: rascen
     integer(kind=4) :: piclin
     integer(kind=4) :: spinp
     integer(kind=4) :: deglin
     integer(kind=4) :: lintot
     integer(kind=4) :: delele
     integer(kind=4) :: eletot
     integer(kind=4) :: pitch
     integer(kind=4) :: yaw
     integer(kind=4) :: roll
     integer(kind=4) :: reserved
     integer(kind=4) :: iajust
     integer(kind=4) :: iajtim
     integer(kind=4) :: res2
     integer(kind=4) :: iseang
     integer(kind=4) :: skew
     integer(kind=4) :: res3
     integer(kind=4) :: bt1_scan
     integer(kind=4) :: bt1_time
     integer(kind=4) :: bt1_tim2
     integer(kind=4) :: bt1_cnt
     integer(kind=4) :: bt2_scan
     integer(kind=4) :: bt2_time
     integer(kind=4) :: bt2_tim2
     integer(kind=4) :: bt2_cnt
     integer(kind=4) :: gamma
     integer(kind=4) :: gamdot
     integer(kind=4), dimension(80) :: res4
     character(len=32) :: memo
  END TYPE visr_nav
!***
  INTERFACE gvar_ll2ij
     MODULE PROCEDURE gvar_ll2ij_scalar
     MODULE PROCEDURE gvar_ll2ij_rank2
     MODULE PROCEDURE gvar_ll2ij_rank1
  END INTERFACE
  INTERFACE gvar_ij2ll
     MODULE PROCEDURE gvar_ij2ll_scalar
     MODULE PROCEDURE gvar_ij2ll_rank2
     MODULE PROCEDURE gvar_ij2ll_rank1
  END INTERFACE

contains
!#######################################################################
!                                                         BEGIN BCD_TIME
!****r* kmod_mcidas/bcd_time
! NAME
! bcd_time
! SYNOPSIS
! call bcd_time (time,year,jday,hour,minute,second,msec)
! FUNCTION 
! Converts Byte-encoded time (BCD=Binary Coded Decimal) to "normal"
! values.
! INPUTS
! time - i*1(0:7) - byte-encoded time
! OUTPUT
! year 
! jday   - i*4 - day of year
! hour   - i*4 - hour of day
! minute - i*4 - minute of hour
! second - i*4 - second of minute
! msec   - i*4 - # of milliseconds
!***
  subroutine bcd_time ( time, year, jday, hour, minute, second, msec )
    implicit none
    !External variables
    integer(kind=1), dimension(0:7) :: time
    integer(kind=4) :: year, jday, hour, minute, second, msec
    !Internal variables
    integer(kind=1) :: yy1, yy2, yy3, yy4, &
                       jd1, jd2, jd3, &
                       hh1, hh2, mm1, mm2, &
                       ss1, ss2, ms1, ms2, ms3

    yy1 = ibits(time(3),4,4)
    yy2 = ibits(time(3),0,4)
    yy3 = ibits(time(2),4,4)
    yy4 = ibits(time(2),0,4)
    jd1 = ibits(time(1),4,4)
    jd2 = ibits(time(1),0,4)
    jd3 = ibits(time(0),4,4)
    
    hh1 = ibits(time(0),0,4)
    hh2 = ibits(time(7),4,4)
    mm1 = ibits(time(7),0,4)
    mm2 = ibits(time(6),4,4)
    ss1 = ibits(time(6),0,4)
    ss2 = ibits(time(5),4,4)
    ms1 = ibits(time(5),0,4)
    ms2 = ibits(time(4),4,4)
    ms3 = ibits(time(4),0,4)

    year = mod(int(yy1)*1000 + int(yy2)*100 + int(yy3)*10 + yy4, 10000)
    jday = int(jd1)*100 + int(jd2)*10 + int(jd3)
    hour = hh1*10 + hh2
    minute = mm1*10 + mm2
    second= ss1*10 + ss2
    msec = int(ms1)*100 + int(ms2)*10 + int(ms3)

  end subroutine bcd_time
    
!                                                           END BCD_TIME
!#######################################################################
!#######################################################################
!                                                       BEGIN GVAR_LL2IJ
!****r* kmod_mcidas/gvar_ll2ij
! NAME
! gvar_ll2ij
! SYNOPSIS
! call gvar_ll2ij( lat, lon, i, j, area, nav)
! FUNCTION
! This function navigates Earth points (defined as lat/lon locations)
! to satellite image coordinates using the area and navigation headers
! from the McIDAS area file.
!
! INPUTS
! lat - real*4 - latitude 
! lon - real*4 - longitude
! * lat,lon can be of rank1, rank2 or scalar
!
! OUTPUT
! i - real*4 - Image element number
! j - real*4 - Image line number
! * i & j are the same rank as the input lat/lon
! 
! USES
! bcd_time, nav2rec, imcnav
!***
  subroutine gvar_ll2ij_scalar(inlat,inlon,ipoints,jpoints,AREA,GVARNAV)
    ! External variables
    real(kind=4), intent(in) :: inlat
    real(kind=4), intent(in) :: inlon
    real(kind=4), intent(out):: ipoints
    real(kind=4), intent(out):: jpoints
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    !Internal variables
    real(kind=4), dimension(1) :: ip, jp, lat,lon
    
    lat = inlat
    lon = inlon

    call gvar_ll2ij(lat,lon,ip,jp,AREA,GVARNAV)

    ipoints = ip(1)
    jpoints = jp(1)

  end subroutine gvar_ll2ij_scalar
  
  subroutine gvar_ll2ij_rank2(inlat,inlon,ipoints,jpoints,AREA,GVARNAV)
    ! External variables
    real(kind=4), intent(in), dimension(:,:) :: inlat
    real(kind=4), intent(in), dimension(:,:) :: inlon
    real(kind=4), intent(out), dimension(:,:):: ipoints
    real(kind=4), intent(out), dimension(:,:):: jpoints
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    !Internal variables
    real(kind=4), allocatable,dimension(:) :: ip, jp, lat,lon
    integer :: n, nx, ny
    
    nx = size(ipoints,1)
    ny = size(ipoints,2)
    n  = size(ipoints)
    allocate(ip(n), jp(n), lat(n), lon(n))
    lat = reshape(inlat, (/n/))
    lon = reshape(inlon, (/n/))
    
    call gvar_ll2ij_rank1(lat,lon,ip,jp,AREA,GVARNAV)
    ipoints = reshape(ip,(/nx,ny/))
    jpoints = reshape(jp,(/nx,ny/))
    deallocate (ip, jp, lat, lon)
  end subroutine gvar_ll2ij_rank2
  
  subroutine gvar_ll2ij_rank1(inlat,inlon,ipoints,jpoints,AREA,GVARNAV)
    ! External variables
    real(kind=4), intent(in), dimension(:) :: inlat
    real(kind=4), intent(in), dimension(:) :: inlon
    real(kind=4), intent(out), dimension(:):: ipoints
    real(kind=4), intent(out), dimension(:):: jpoints
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    !internal
    integer(kind=4) :: ew_offset, ns_offset, line_res, elem_res, sec, msec
    !interface with imcnav
    real(kind=8) :: dlat, dlon, line, pixel, se, s, glat, glon, satzen, grad
    REAL(kind=8), dimension(337) :: rec
    integer(kind=4) :: key, imc, instr, nye,nde,nhe,nme, ny, nd, nh,&
         & nm, EW_NAD_CY, EW_NAD_INC, NS_NAD_CY, NS_NAD_INC
    integer npoints, ipoint

    call nav2rec( GVARnav, rec )
    EW_NAD_CY  = GVARNAV%ew_cyl
    EW_NAD_INC = GVARNAV%ew_inc
    NS_NAD_CY  = GVARNAV%ns_cyl
    NS_NAD_INC = GVARNAV%ns_inc

    call bcd_time(GVARNAV%epoch_time,nye,nde,nhe,nme,sec,msec)
    se = real(sec,8) + real(msec,8)/1000.d0
    ny = GVARNAV%img_date / 1000 + 1900
    nd = mod(GVARNAV%img_date,1000)
    nh = GVARNAV%img_time / 10000000
    nm = mod(GVARNAV%img_time / 100000,100)
    s  = real(mod(GVARNAV%img_time,100000),8) / 1000.d0
    ew_offset = AREA%west_bound
    ns_offset = AREA%north_bound
    elem_res =  AREA%elem_res
    line_res =  AREA%line_res

    imc = 0
    instr = 1
    key = 1

    npoints = size(ipoints)
    do ipoint=1,npoints
       dlat = real(inlat(ipoint),8)
       dlon = real(inlon(ipoint),8)
       call imcnav (dlat,dlon,line,pixel,key,imc,instr,  &
            nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
            ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
            REC, glat, glon, grad, satzen)
       ipoints(ipoint) = real(pixel - ew_offset,4) / real(elem_res)
       jpoints(ipoint) = real(line -  ns_offset,4) / real(line_res)
    end do
  end subroutine gvar_ll2ij_rank1
!                                                         END GVAR_LL2IJ
!#######################################################################
!#######################################################################
!                                                       BEGIN GVAR_IJ2LL
!****r* kmod_mcidas/gvar_ij2ll
! NAME
! gvar_ij2ll
! SYNOPSIS
! call gvar_ij2ll( i, j, lat, lon, area, nav)
! FUNCTION
! This function navigates satellite image coordinates to Earth points 
! (defined as lat/lon locations) using the area and navigation headers
! from the McIDAS area file.
!
! INPUTS
! i - real*4 - Image element number
! j - real*4 - Image line number
! * i & j can be of rank1, rank2 or scalar
!
! OUTPUT
! lat - real*4 - latitude 
! lon - real*4 - longitude
! * lat and lon are the same rank as the input lat/lon
! 
! USES
! bcd_time, nav2rec, imcnav
!***
  subroutine gvar_ij2ll_scalar(ipoints,jpoints,inlat,inlon,AREA,GVARNAV)
    ! External variables
    real(kind=4), intent(in):: ipoints
    real(kind=4), intent(in):: jpoints
    real(kind=4), intent(out) :: inlat
    real(kind=4), intent(out) :: inlon
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    !Internal variables
    real(kind=4), dimension(1) :: ip, jp, lat, lon
    
    ip = ipoints
    jp = jpoints

    call gvar_ij2ll(ip,jp,lat,lon,AREA,GVARNAV)

    inlat = lat(1)
    inlon = lon(1)

  end subroutine gvar_ij2ll_scalar
  
  subroutine gvar_ij2ll_rank2(ipoints,jpoints,inlat,inlon,AREA,GVARNAV)
    ! External variables
    real(kind=4), intent(in), dimension(:,:):: ipoints
    real(kind=4), intent(in), dimension(:,:):: jpoints
    real(kind=4), intent(out), dimension(:,:) :: inlat
    real(kind=4), intent(out), dimension(:,:) :: inlon
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    !Internal variables
    real(kind=4), allocatable,dimension(:) :: ip, jp, lat,lon
    integer :: n, nx, ny
    
    nx = size(ipoints,1)
    ny = size(ipoints,2)
    n  = size(ipoints)
    allocate(ip(n), jp(n), lat(n), lon(n))
    
    ip = reshape(ipoints, (/ n /) )
    jp = reshape(jpoints, (/ n /) )

    call gvar_ij2ll(ip,jp,lat,lon,AREA,GVARNAV)
    inlat = reshape(lat,(/nx,ny/))
    inlon = reshape(lon,(/nx,ny/))
    deallocate (ip, jp, lat, lon)
  end subroutine gvar_ij2ll_rank2
  
  subroutine gvar_ij2ll_rank1(ipoints,jpoints,inlat,inlon,AREA,GVARNAV)
    ! External variables
    real(kind=4), intent(in), dimension(:):: ipoints
    real(kind=4), intent(in), dimension(:):: jpoints
    real(kind=4), intent(out), dimension(:) :: inlat
    real(kind=4), intent(out), dimension(:) :: inlon
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    !internal
    integer(kind=4) :: ew_offset, ns_offset, line_res, elem_res, sec, msec
    !interface with imcnav
    real(kind=8) :: dlat, dlon, line, pixel, se, s, glat, glon, satzen, grad
    REAL(kind=8), dimension(337) :: rec
    integer(kind=4) :: key, imc, instr, nye,nde,nhe,nme, ny, nd, nh,&
         & nm, EW_NAD_CY, EW_NAD_INC, NS_NAD_CY, NS_NAD_INC
    integer npoints, ipoint

    call nav2rec( GVARnav, rec )
    EW_NAD_CY  = GVARNAV%ew_cyl
    EW_NAD_INC = GVARNAV%ew_inc
    NS_NAD_CY  = GVARNAV%ns_cyl
    NS_NAD_INC = GVARNAV%ns_inc

    call bcd_time(GVARNAV%epoch_time,nye,nde,nhe,nme,sec,msec)
    se = real(sec,8) + real(msec,8)/1000.d0
    ny = GVARNAV%img_date / 1000 + 1900
    nd = mod(GVARNAV%img_date,1000)
    nh = GVARNAV%img_time / 10000000
    nm = mod(GVARNAV%img_time / 100000,100)
    s  = real(mod(GVARNAV%img_time,100000),8) / 1000.d0
    ew_offset = AREA%west_bound
    ns_offset = AREA%north_bound
    elem_res =  AREA%elem_res
    line_res =  AREA%line_res

    imc = 0
    instr = 1
    key = 0

    npoints = size(ipoints)
    do ipoint=1,npoints
!       dlat = real(inlat(ipoint),8)
!       dlon = real(inlon(ipoint),8)
       line = real(ns_offset,8) + jpoints(ipoint) * real(line_res,8)
       pixel= real(ew_offset,8) + ipoints(ipoint) * real(elem_res,8)
       call imcnav (dlat,dlon,line,pixel,key,imc,instr,  &
            nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
            ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
            REC, glat, glon, grad, satzen)
!       ipoints(ipoint) = real(pixel - ew_offset,4) / real(elem_res)
!       jpoints(ipoint) = real(line -  ns_offset,4) / real(line_res)
       inlat(ipoint) = real(dlat,4)
       inlon(ipoint) = real(dlon,4)
    end do
  end subroutine gvar_ij2ll_rank1
!                                                         END GVAR_IJ2LL
!#######################################################################
!#######################################################################
  subroutine gvar_ij2ll_test(ipoints,jpoints,inlat,inlon,AREA,GVARNAV,&
       &  scanangle, grad, glat, glon)
    ! External variables
    real(kind=4), intent(in), dimension(:):: ipoints
    real(kind=4), intent(in), dimension(:):: jpoints
    real(kind=4), intent(out), dimension(:) :: inlat
    real(kind=4), intent(out), dimension(:) :: inlon
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    integer, optional :: scanangle
    real, optional :: grad, glat, glon
    !internal
    integer(kind=4) :: ew_offset, ns_offset, line_res, elem_res, sec, msec
    !interface with imcnav
    real(kind=8) ::  se, s, glat8, glon8, grad8
    real(kind=8), dimension(size(ipoints)) :: dlats, dlons, dlines, dpixels, satzen
    REAL(kind=8), dimension(337) :: rec
    integer(kind=4) :: key, imc, instr, nye,nde,nhe,nme, ny, nd, nh,&
         & nm, EW_NAD_CY, EW_NAD_INC, NS_NAD_CY, NS_NAD_INC

    call nav2rec( GVARnav, rec )
    EW_NAD_CY  = GVARNAV%ew_cyl ; EW_NAD_INC = GVARNAV%ew_inc
    NS_NAD_CY  = GVARNAV%ns_cyl ; NS_NAD_INC = GVARNAV%ns_inc

    call bcd_time(GVARNAV%epoch_time,nye,nde,nhe,nme,sec,msec)
    se = real(sec,8) + real(msec,8)/1000.d0
    ny = GVARNAV%img_date / 1000 + 1900
    nd = mod(GVARNAV%img_date,1000)
    nh = GVARNAV%img_time / 10000000
    nm = mod(GVARNAV%img_time / 100000,100)
    s  = real(mod(GVARNAV%img_time,100000),8) / 1000.d0
    ew_offset = AREA%west_bound  ; elem_res =  AREA%elem_res
    ns_offset = AREA%north_bound ; line_res =  AREA%line_res

    imc = 0 ; instr = 1
    key = 0

    dlines = real(ns_offset,8) + jpoints * real(line_res,8)
    dpixels= real(ew_offset,8) + ipoints * real(elem_res,8)
    call imcnav_r1(dlats,dlons,dlines,dpixels,key,imc,instr,  &
         nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
         ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
         REC, glat8, glon8, grad8, satzen, scanangle)
    inlat = real(dlats,4) ; inlon = real(dlons,4)
    if (present(grad)) grad = grad8
    if (present(glat)) glat = glat8
    if (present(glon)) glon = glon8
  end subroutine gvar_ij2ll_test
!---------------------
  subroutine gvar_ll2ij_test(inlat,inlon,ipoints,jpoints,AREA,GVARNAV,&
       &  scanangle, grad, glat, glon)
    ! External variables
    real(kind=4), intent(out), dimension(:) :: ipoints
    real(kind=4), intent(out), dimension(:) :: jpoints
    real(kind=4), intent(in), dimension(:) :: inlat
    real(kind=4), intent(in), dimension(:) :: inlon
    type(gvar_area) :: AREA
    type(gvar_nav)  :: GVARNAV
    integer, optional :: scanangle
    real, optional :: grad, glat, glon
    !internal
    integer(kind=4) :: ew_offset, ns_offset, line_res, elem_res, sec, msec
    !interface with imcnav
    real(kind=8) :: se, s, glat8, glon8, grad8
    real(kind=8), dimension(size(ipoints)) :: dlats, dlons, dlines, dpixels, satzen
    REAL(kind=8), dimension(337) :: rec
    integer(kind=4) :: key, imc, instr, nye,nde,nhe,nme, ny, nd, nh,&
         & nm, EW_NAD_CY, EW_NAD_INC, NS_NAD_CY, NS_NAD_INC

    call nav2rec( GVARnav, rec )
    EW_NAD_CY  = GVARNAV%ew_cyl ; EW_NAD_INC = GVARNAV%ew_inc
    NS_NAD_CY  = GVARNAV%ns_cyl ; NS_NAD_INC = GVARNAV%ns_inc

    call bcd_time(GVARNAV%epoch_time,nye,nde,nhe,nme,sec,msec)
    se = real(sec,8) + real(msec,8)/1000.d0
    ny = GVARNAV%img_date / 1000 + 1900
    nd = mod(GVARNAV%img_date,1000)
    nh = GVARNAV%img_time / 10000000
    nm = mod(GVARNAV%img_time / 100000,100)
    s  = real(mod(GVARNAV%img_time,100000),8) / 1000.d0
    ew_offset = AREA%west_bound  ; elem_res =  AREA%elem_res
    ns_offset = AREA%north_bound ; line_res =  AREA%line_res

    imc = 0 ; instr = 1
    key = 1

!    inlat = real(dlats,4) ; inlon = real(dlons,4)
    dlats = real(inlat,8)
    dlons = real(inlon,8)
    call imcnav_r1(dlats,dlons,dlines,dpixels,key,imc,instr,  &
         nye,nde,nhe,nme,se,ny,nd,nh,nm,s,  &
         ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
         REC, glat8, glon8, grad8, satzen, scanangle)
!    dlines = real(ns_offset,8) + jpoints * real(line_res,8)
!    dpixels= real(ew_offset,8) + ipoints * real(elem_res,8)
    if (.not. (present(scanangle))) then
       ipoints = (real(dpixels,4) - ew_offset) / real(elem_res,4)
       jpoints = (real(dlines,4)  - ns_offset) / real(line_res,4)
    else
       ipoints = real(dpixels,4)
       jpoints = real(dlines,4)
    endif
    if (present(grad)) grad = grad8
    if (present(glat)) glat = glat8
    if (present(glon)) glon = glon8
  end subroutine gvar_ll2ij_test
!                                                         END GVAR_LL2IJ
!#######################################################################
!!#######################################################################
!                                                           BEGIN IMCNAV
!****r* kmod_mcidas/imcnav
! NAME
! imcnav
! SYNOPSIS
! call imcnav(dlat,dlon,line,pixel,key,imc,instr,
!       nye,nde,nhe,nme,se,  &
!       ny,nd,nh,nm,s,  &
!       ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
!       REC, glat, glon, grad, vza,scanangle)
! FUNCTION
! Performs conversion between lat/lon and i/j (the direction of the 
! transform depends on key).
! INPUTS
! OUTPUT
! NOTES
! * This code is the code delivered by contractors as part of the Earth Location 
!   User's Guide (ELUG) It was converted to Fortran90 code by to_f90.
!   * Code converted using TO_F90 by Alan Miller Date: 2004-12-03  Time: 08:31:57
! * This is the basis of the imcnav_r1 routine, which performs the same operation
!   except on rank 1 array.
! MODIFICATION HISTORY
! * Conversion to Fortran90
! * Knapp adjusted it to optionall return scan/elevation angles instead
!   of lat/lons.
! 
!***
! Code converted using TO_F90 by Alan Miller
! Date: 2004-12-03  Time: 08:31:57
!     Front end to IDL Call to navigate GVAR imagery
!     Calls fortran routines to perform navigation
  SUBROUTINE imcnav(dlat,dlon,line,pixel,  &
       key,imc,instr,  &
       nye,nde,nhe,nme,se,  &
       ny,nd,nh,nm,s,  &
       ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
       REC, glat, glon, grad, vza, scanangle)
    implicit none
    !External variables
    REAL(kind=8), INTENT(IN OUT)    :: dlat
    REAL(kind=8), INTENT(IN OUT)    :: dlon
    REAL(kind=8), INTENT(IN OUT) :: line
    REAL(kind=8), INTENT(IN OUT) :: pixel
    INTEGER, INTENT(IN OUT)      :: key
    INTEGER, INTENT(IN OUT)      :: imc
    INTEGER, INTENT(IN OUT)      :: instr
    INTEGER, INTENT(IN OUT)      :: nye
    INTEGER, INTENT(IN)          :: nde
    INTEGER, INTENT(IN)          :: nhe
    INTEGER, INTENT(IN)          :: nme
    REAL(kind=8), INTENT(IN OUT) :: se
    INTEGER, INTENT(IN OUT)      :: ny
    INTEGER, INTENT(IN)          :: nd
    INTEGER, INTENT(IN)          :: nh
    INTEGER, INTENT(IN)          :: nm
    REAL(kind=8), INTENT(IN OUT) :: s
    INTEGER, INTENT(IN OUT)      :: ew_nad_cy
    INTEGER, INTENT(IN OUT)      :: ew_nad_inc
    INTEGER, INTENT(IN OUT)      :: ns_nad_cy
    INTEGER, INTENT(IN OUT)      :: ns_nad_inc
    REAL(kind=8), INTENT(IN OUT) :: REC(336)
    REAL(kind=8), INTENT(OUT)    :: glat
    REAL(kind=8), INTENT(OUT)    :: glon
    REAL(kind=8), INTENT(OUT)    :: grad
    REAL(kind=8), intent(out),optional    :: vza
    integer(kind=4), intent(in), optional :: scanangle
    !Internal variables
    REAL(kind=8) t,tu,rl,rp, &
          e,rlat,rlon,rrad
    INTEGER :: ier
    !parameters ... I hope this doesn't cause a memory leak
    REAL(kind=8), PARAMETER :: svx = -21.d-6
    REAL(kind=8), PARAMETER :: svy =  14.d-6
    REAL(kind=8), PARAMETER, dimension(4,2) :: &
         doff= reshape( (/28.e-6,56.e-6,-28.e-6,-56.e-6, &
         84.e-6,112.e-6, 14.e-6,42.e-6/), (/4,2/) )
    INTEGER(kind=4), PARAMETER :: icx = 1, incx = 2715, icy = 5, incy = 2580
    !------------------------------------------------------------------------
    !      INCLUDE variables
    !  By including the information here, the variables will be available to 
    !  routines within the CONTAINS statement
    !INCLUDE 'elcons.f90'
!   REAL(kind=8), parameter :: pi=3.141592653589793D0
    REAL(kind=8), parameter :: deg=180.d0/pi
!    REAL(kind=8), PARAMETER :: rad=pi/180.d0! DEGREES TO RADIANS CONVERSION PI/180
    REAL(kind=8), PARAMETER :: nomorb=42164.365D0 ! NOM RAD DISTANCE OF SATELLITE (km)
    REAL(kind=8), PARAMETER :: ae=6378.137D0      ! EARTH EQUATORIAL RADIUS (km)
    REAL(kind=8), PARAMETER :: fer=1.d0-(6356.7533D0/ae) ! EARTH FLAT COEFFICIENT
    REAL(kind=4), PARAMETER :: aebe2=1.d0/(1.d0-fer)**2
    REAL(kind=4), PARAMETER :: aebe3=aebe2-1.
    REAL(kind=4), PARAMETER :: aebe4=(1.d0-fer)**4-1.
    !INCLUDE 'instco.f90'
    INTEGER(kind=4) incmax(2) ! NUMBER OF INCREMENTS PER CYCLE
    REAL(kind=4) elvmax(2)    ! BOUNDS IN ELEVATION (RADIANS)
    REAL(kind=4) scnmax(2)    ! BOUNDS IN SCAN ANGLE (RADIANS)
    REAL(kind=4) elvinc(2)    ! CHANGE IN ELEVATION ANGLE PER INCREMENT (RAD)
    REAL(kind=4) scninc(2)    ! CHANGE IN SCAN ANGLE PER INCREMENT (RADIANS)
    REAL(kind=4) elvln(2)     ! ELEVATION ANGLE PER DETECTOR LINE (RADIANS)
    REAL(kind=4) scnpx(2)     ! SCAN ANGLE PER PIXEL (RADIANS)
    !INCLUDE 'elcomm.f90'
    REAL(kind=8) xs(3)          ! NORMALIZED S/C POSITION IN ECEF COORDINATES
    REAL(kind=8) bt(3,3)        ! ECEF TO INSTRUMENT COORDINATES TRANSFORMATION
    REAL(kind=8)  q3            ! USED IN SUBROUTINE LPOINT
    REAL(kind=8) pitch,roll,yaw ! PITCH,ROLL,YAW ANGLES OF INSTRUMENT (RAD)
    REAL(kind=4) pma,rma        ! PITCH,ROLL MISALIGNMENTS OF INSTRUMENT (RAD)
  
    CALL imc_setcon(instr,ns_nad_cy,ns_nad_inc,ew_nad_cy,ew_nad_inc)
    !     COMPUTE EPOCH TIME
    tu=imc_timex(nye,nde,nhe,nme,se)
    !     COMPUTE ACTUAL TIME
    t=imc_timex(ny,nd,nh,nm,s)
    !     COMPUTE DATA NEEDED FOR INSTRUMENT TO EARTH COORDINATES
    !     TRANSFORMATION
    CALL imc_lmodel(t,tu,REC,imc,rlat,rlon,rrad)
    glat=rlat
    glon=rlon
    grad=rrad
    
    IF (key /= 1) then !GO TO 10
       !       TRANSFORM LINE/PIXEL TO GEOGRAPHIC COORDINATES:
       !       SET INPUT LINE/PIXEL NUMBERS
       !ken      RL=L(INSTR)
       !ken      RP=IP(INSTR)
       rp = line
       rl = pixel
       !       COMPUTE ELEVATION AND SCAN ANGLES (E,S) RELATED TO INPUT
       !       LINE AND PIXEL NUMBERS
       e=imc_evln(instr,rp)
       s=imc_scpx(instr,rl)
       if (present(scanangle)) then
          dlon = s
          dlat = e
          return
       end if

       !       TRANSFORM ELEVATION AND SCAN ANGLES TO GEOGRAPHIC COORDINATES
       CALL imc_lpoint(e,s,rlat,rlon,ier,vza)
       dlat = rlat*deg
       dlon = rlon*deg
    else
       !     IF KEY == 1
       !     GEOGRAPHIC TO LINE/PIXEL COORDINATES TRANSFORMATION:
       !       SET INPUT LATITUDE AND LONGITUDE
       rlat=dlat/rad
       rlon=dlon/rad
       !       TRANSFORM LAT/LON TO ELEVATION AND SCAN ANGLES
       CALL imc_gpoint(rlat,rlon,e,s,ier,vza)
       !       CONVERT ELEVATION AND SCAN ANGLES TO LINE/PIXEL COORDINATES
       line  = -1000.
       pixel = -1000.
       IF (ier == 0) THEN
          CALL imc_evsc2l(instr,e,s,rl,rp)
          line = rl
          pixel = rp
       END IF
    end IF
    
  CONTAINS
!***********************************************************************
!**   NOAA/NESDIS/SOCC/SOFTWARE BRANCH
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : IMC_TIME50
!**   SOURCE    : F.IMC_TIME50
!**   LOAD NAME : ANY
!**   PROGRAMMER: THOMAS I. BABICKI
!**
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A      2/17/89  TB   INITIAL CREATION
!***********************************************************************
!**   IMC_TIME50 ACCEPTS TWO WORDS CONTAINING DATE AND TIME
!**   AND RETURNS TIME EXPRESSED AS DOUBLE PRECISION MINUTES FROM
!**   1950 JAN. 1.0
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_time50(i)
      !External variables
      INTEGER(kind=4), INTENT(IN) :: i(2)
      REAL(kind=8) :: imc_time50
      !Internal variables
      INTEGER(kind=4) ny ! YEAR
      INTEGER(kind=4) nd ! DAY OF YEAR
      INTEGER(kind=4) nh ! HOUR
      INTEGER(kind=4) nm ! MINUTE
      REAL(kind=8)    s  ! SECONDS
      INTEGER :: j
      integer :: iaa, iab, nbc, iac, def
      !***********************************************************************
      !     CONVERT INPUT YEAR, DAY OF YEAR, HOUR AND MINUTE
      !     TO INTEGER VALUES.
      ny=i(1)/10000
      iaa=i(1)-(ny*10000)
      nd=(i(1)-(ny*10000))*.1
      iab=(iaa-(nd*10))*10
      nbc=i(2)/10000000.
      iac=i(2)-(nbc*10000000)
      nh=iab+nbc
      def=i(2)-iac
      nm=iac*.00001
      s=(i(2)-(def+(nm*100000)))*.001D0
      !***********************************************************************
      !     HERE WE CONVERT INTEGER YEAR AND DAY OF YEAR TO NUMBER OF
      !     DAYS FROM 0 HOUR UT, 1950 JAN. 1.0
      !     THIS CONVERTION IS BASED ON AN ALGORITHM BY FLIEGEL AND VAN
      !     FLANDERN, COMM. OF ACM, VOL.11, NO. 10, OCT. 1968 (P.657)
      j=nd+1461*(ny+4799)/4-3*((ny+4899)/100)/4-2465022
      !     COMPUTE TIME IN MINUTES FROM JANUARY 1.0, 1950
      imc_time50=j*1440.d0+nh*60.d0+nm+s/60.d0
      RETURN
    END FUNCTION imc_time50
!***********************************************************************
!**   NOAA/NESDIS/SOCC/SOFTWARE BRANCH
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : IMC_TIMEX
!**   SOURCE    : F.TIMEX
!**   LOAD NAME : ANY
!**   PROGRAMMER: THOMAS I. BABICKI
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A      4/25/89  TB   INITIAL CREATION
!***********************************************************************
    FUNCTION imc_timex(ny,nd,nh,nm,s)
      !External variables
      INTEGER(kind=4), INTENT(IN OUT):: ny
      INTEGER(kind=4), INTENT(IN)    :: nd
      INTEGER(kind=4), INTENT(IN)    :: nh
      INTEGER(kind=4), INTENT(IN)    :: nm
      REAL(kind=8), INTENT(IN)       :: s
      REAL(kind=8) imc_timex
      !Internal variables
      INTEGER :: j
      j=nd+1461*(ny+4799)/4-3*((ny+4899)/100)/4-2465022
      !     COMPUTE ACTUAL TIME IN MINUTES FROM JANUARY 1.0, 1950
      imc_timex=j*1440.d0+nh*60.d0+nm+s/60.d0
      RETURN
    END FUNCTION imc_timex
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : SETCONS
!**   SOURCE    : F.SETCONS
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     02/16/89  IL   INITIAL CREATION
!**
!**   B     05/19/94  NP   ADDED CALCULATION OF INSTRUMENT ELEVATION AND
!**                        SCAN ANGLE BIASES BASED ON USER INPUT
!***********************************************************************
!**   THIS SUBROUTINE GENERATES CONSTANTS IN COMMON  INSTCOMM
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: INSTCO
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
!***********************************************************************
    SUBROUTINE imc_setcon(instr,ns_nad_cy,ns_nad_inc,ew_nad_cy,ew_nad_inc)
      !External variables
      INTEGER, INTENT(IN OUT) :: instr
      INTEGER, INTENT(IN OUT) :: ns_nad_cy
      INTEGER, INTENT(IN OUT) :: ns_nad_inc
      INTEGER, INTENT(IN OUT) :: ew_nad_cy
      INTEGER, INTENT(IN OUT) :: ew_nad_inc
!***********************************************************************
      incmax(1)=6136
      incmax(2)=2805
      elvinc(1)=8.0D-6
      elvinc(2)=17.5D-6
      scninc(1)=16.d-6
      scninc(2)=35.d-6
      elvln(1)=28.d-6
      elvln(2)=280.d-6
      scnpx(1)=16.d-6
      scnpx(2)=280.d-6
!     ************************************************************
!     COMMENTED OUT ELEVATION AND SCAN BIAS CONSTANTS SINCE INSTRUMENT
!     EARTH NADIR POSITION IS AVAILABLE IN GVAR DATA AND PERIODICALLY
!     UPDATED
!        ELVMAX(1)=0.220896D0
!        ELVMAX(2)=0.22089375D0
!        SCNMAX(1)=0.24544D0
!        SCNMAX(2)=0.2454375D0
!     RECOMPUTE ELEVATION AND SCAN BIASES BASED ON USER INPUTS OF
!     CYCLES & INCREMENTS OBTAINED FROM GVAR
!     ELVMAX(INSTR) = (NS_NAD_CY*INCMAX(INSTR)+NS_NAD_INC)*ELVINC(INSTR)
      IF(instr == 1)THEN
         elvmax(instr)=(ns_nad_cy*incmax(instr)+ns_nad_inc)*elvinc(instr)
      ELSE
         elvmax(instr)=((9-ns_nad_cy)*incmax(instr)-ns_nad_inc) *elvinc(instr)
      END IF
      scnmax(instr) = (ew_nad_cy*incmax(instr)+ew_nad_inc)*scninc(instr)
!     ************************************************************
      RETURN
    END SUBROUTINE imc_setcon
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : LMODEL
!**   SOURCE    : F.LMODEL
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   1     01/09/89  IL   INITIAL CREATION
!**   2     06/02/89  IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL 504-01
!**   3     08/21/89  IL   CORRECTED ORBIT ANGLE COMPUTATIONS
!**   4     03/08/94  SC   S/C COMPENSATION APPLIED UNCONDITIONALLY;
!**                        REFERENCE RADIAL DISTANCE, LATITUDE AND
!**                        ORBIT YAW SET TO ZERO IF IMC DISABLED.
!**   5     03/08/94  SC   ADDED TRAP FOR SLAT=SYAW=0; CORRECTED
!**                        EXPRESSION FOR LAM.
!***********************************************************************
!**   THIS SUBROUTINE COMPUTES THE POSITION OF THE SATELLITE AND THE
!**   ATTITUDE OF THE IMAGER OR SOUNDER.  THE CALCULATIONS ARE BASED
!**   ON THE OATS ORBIT AND ATTITUDE MODEL REPRESENTED BY THE O&A
!**   PARAMETER SET IN GVAR BLOCK 0.
!**        INPUTS:
!**          TIME, EPOCH TIME, O&A PARAMETER SET, IMC STATUS.
!**        OUTPUTS:
!**          THE SPACECRAFT POSITION VECTOR IN EARTH FIXED COORDINATES;
!**          THE GEOMETRIC ROLL, PITCH, YAW ANGLES AND THE ROLL,
!**          PITCH MISALIGNMENTS FOR EITHER THE IMAGER OR THE SOUNDER;
!**          THE EARTH FIXED TO INSTRUMENT FRAME TRANSFORMATION MATRIX;
!**          GEOGRAPHIC LATITUDE AND LONGITUDE AT SUBSATELLITE POINT.
!**   DESCRIPTION
!**   LMODEL ACCEPTS AN INPUT DOUBLE PRECISION TIME IN MINUTES FROM
!**   1950, JAN.1.0 AND AN INPUT SET OF O&A PARAMETERS AND COMPUTES
!**   POSITION OF THE SATELLITE, THE ATTITUDE ANGLES AND ATTITUDE
!**   MISALIGNMENTS AND THE INSTRUMENT TO EARTH FIXED COORDINATES
!**   TRANSFORMATION MATRIX.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: /ELCOMM/ XS,Q3,PITCH,ROLL,YAW,PMA,RMA,BT
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : INST2ER,GATT
!***********************************************************************
    SUBROUTINE imc_lmodel(t,tu,REC,imc,rlat,rlon,rrad)
      !External variables
      REAL(kind=8), INTENT(IN)    :: t       ! INPUT TIME FROM 1950, JAN 1.0 (MIN)
      REAL(kind=8), INTENT(IN)    :: tu      ! EPOCH TIME FROM 1950, JAN 1.0 (MIN)
      REAL(kind=8), INTENT(IN)    :: REC(336)! INPUT O&A PARAMETER SET
      INTEGER,      INTENT(IN OUT):: imc     ! INPUT IMC STATUS: 0 - ON, 1 - OFF
      REAL(kind=8), INTENT(OUT)   :: rlat    ! SUBSATELLITE GEODETIC LATITUDE (RAD)
      REAL(kind=8), INTENT(OUT)   :: rlon,rrad ! SUBSATELLITE LONGITUDE IN RADIANS
      !Internal variables
      REAL(kind=8) r      ! NORMALIZED SATELLITE DISTANCE (IN UNITS OF KMER9)
      REAL(kind=8) ts     ! TIME FROM EPOCH IN MINUTES
      REAL(kind=8) b(3,3) ! SPACCRAFT TO EARTH FIXED COORDINATES TRANSFORMATION
      !                     MATRIX
      REAL(kind=8) te     ! EXPONENENTIAL TIME DELAY FROM EPOCH (IN MINUTES)
      REAL(kind=8) phi    ! SUBSATELLITE GEOCENTRIC LATITUDE IN RADIANS
      REAL(kind=8) dr     ! RADIAL DISTANCE FROM THE NOMINAL (KM)
      REAL(kind=8) psi    ! ORBITAL YAW (IN RADIANS)
      REAL(kind=8) lam    ! IMC LONGITUDE (IN RADIANS)
      REAL(kind=8) u      ! ARGUMENT OF LATITUDE (IN RADIANS)
      REAL(kind=8) su,cu  ! SIN(U), COS(U)
      REAL(kind=8) si,ci  ! SINE AND COSINE OF THE ORBIT INCLINATION
      REAL(kind=8) slat   ! SINE OF GEOCENTRIC LATITUDE
      REAL(kind=8) asc    ! LONGITUDE OF THE ASCENDING NODE IN RADIANS
      REAL(kind=8) sa,ca  ! SINE AND COSINE OF ASC
      REAL(kind=8) syaw   ! SINE OF THE ORBIT YAW
      REAL(kind=8) wa     ! SOLAR ORBIT ANGLE IN RADIANS
      REAL(kind=8) w      ! ORBIT ANGLE IN RADIANS
      REAL(kind=8) sw,cw  ! SIN(W),  COS(W)
      REAL(kind=8) s2w,c2w! SIN(2*W),  COS(2*W)
      REAL(kind=8) sw1,cw1! SIN(0.927*W),  COS(0.927*W)
      REAL(kind=8) sw3,cw3! SINE AND COSINE OF 1.9268*W
      REAL(kind=8) dlat   ! CHANGE IN SINE OF GEOCENTRIC LATITUDE
      REAL(kind=8) dyaw   ! CHANGE IN SINE OF ORBIT YAW
      !INCLUDE 'elcons.f90'
      !INCLUDE 'elcomm.f90'
!***********************************************************************
      !     ASSIGN REFERENCE VALUES TO THE SUBSATELLITE LONGITUDE AND
      !     LATITUDE, THE RADIAL DISTANCE AND THE ORBIT YAW.
      lam=REC(5)
      dr=REC(6)
      phi=REC(7)
      psi=REC(8)
      !     ASSIGN REFERENCE VALUES TO THE ATTITUDES AND MISALIGNMENTS
      roll=REC(9)
      pitch=REC(10)
      yaw=REC(11)
      rma=0.0D0
      pma=0.0D0
      !     IF IMC IS OFF, COMPUTE CHANGES IN THE SATELLITE ORBIT
      IF (imc /= 0) THEN
         !     SET REFERENCE RADIAL DISTANCE, LATITUDE AND ORBIT YAW TO ZERO
         dr=0.0D0
         phi=0.0D0
         psi=0.0D0
         !     COMPUTE TIME SINCE EPOCH (IN MINUTES)
         ts=t-tu
         !     COMPUTES ORBIT ANGLE AND THE RELATED TRIGONOMETRIC FUNCTIONS.
         !     EARTH ROTATIONAL RATE=.729115E-4 (RAD/S)
         w=0.729115D-4*60.0D0*ts
         sw=DSIN(w)
         cw=DCOS(w)
         sw1=DSIN(0.927D0*w)
         cw1=DCOS(0.927D0*w)
         s2w=DSIN(2.0D0*w)
         c2w=DCOS(2.0D0*w)
         sw3=DSIN(1.9268D0*w)
         cw3=DCOS(1.9268D0*w)
         !     COMPUTES CHANGE IN THE IMC LONGITUDE FROM THE REFERENCE
         lam=lam+REC(18)+(REC(19)+REC(20)*w)*w  &
                + (REC(27)*sw1+REC(28)*cw1+REC(21)*sw+REC(22)*cw  &
                + REC(23)*s2w+REC(24)*c2w + REC(25)*sw3+REC(26)*cw3  &
                + w*(REC(29)*sw+REC(30)*cw))*2.0D0
         !     COMPUTES CHANGE IN RADIAL DISTANCE FROM THE REFERENCE (KM)
         dr=dr + REC(31) + REC(32)*cw+REC(33)*sw  &
               + REC(34)*c2w+REC(35)*s2w + REC(36)*cw3+REC(37)*sw3  &
               + REC(38)*cw1+REC(39)*sw1 + w*(REC(40)*cw+REC(41)*sw)
         !     COMPUTES THE SINE OF THE CHANGE IN THE GEOCENTRIC LATITUDE
         dlat=REC(42) + REC(43)*cw+REC(44)*sw +REC(45)*c2w+REC(46)*s2w  &
              + w*(REC(47)*cw+REC(48)*sw) +REC(49)*cw1+REC(50)*sw1
         !     COMPUTES GEOCENTRIC LATITUDE BY USING AN EXPANSION FOR ARCSINE
         phi=phi+dlat*(1.0D0+dlat*dlat/6.0D0)
         !     COMPUTES SINE OF THE CHANGE IN THE ORBIT YAW
         dyaw=REC(51) + REC(52)*sw+REC(53)*cw +REC(54)*s2w+REC(55)*c2w  &
              +w*(REC(56)*sw+REC(57)*cw) +REC(58)*sw1+REC(59)*cw1
         !     COMPUTES THE ORBIT YAW BY USING AN EXPANSION FOR ARCSINE.
         psi=psi+dyaw*(1.0D0+dyaw*dyaw/6.0D0)
         !     CALCULATION OF CHANGES IN THE SATELLITE ORBIT ENDS HERE
      END IF
      !     CONVERSION OF THE IMC LONGITUDE AND ORBIT YAW TO THE SUBSATELLITE
      !     LONGITUDE AND THE ORBIT INCLINATION (REF: GOES-PCC-TM-2473, INPUTS
      !     REQUIRED FOR EARTH LOCATION AND GRIDDING BY SPS,  JUNE 6, 1988)
      slat=DSIN(phi)
      syaw=DSIN(psi)
      si=slat**2+syaw**2
      ci=DSQRT(1.0D0-si)
      si=DSQRT(si)
      IF (slat == 0.0D0.AND.syaw == 0.0D0) THEN
         u=0.0D0
      ELSE
         u=DATAN2(slat,syaw)
      END IF
      su=DSIN(u)
      cu=DCOS(u)
      !     COMPUTES LONGITUDE OF THE ASCENDING NODE
      asc=lam - u
      sa=DSIN(asc)
      ca=DCOS(asc)
      !     COMPUTES THE SUBSATELLITE GEOGRAPHIC LATITUDE
      rlat=DATAN(aebe2*DTAN(phi))
      !     COMPUTES THE SUBSATELLITE LONGITUDE
      rlon=asc+DATAN2(ci*su,cu)
      !     COMPUTE THE SUBSATELLITE DISTANCE TO EARTH'S CENTER
      rrad = nomorb+dr
      !     COMPUTES THE SPACECRAFT TO EARTH FIXED COORDINATES TRANSFORMATION
      !     MATRIX:
      !         (VECTOR IN ECEF COORDINATES) = B * (VECTOR IN S/C COORDINATES)
      
      b(1,2)=-sa*si
      b(2,2)= ca*si
      b(3,2)=-ci
      b(1,3)=-ca*cu+sa*su*ci
      b(2,3)=-sa*cu-ca*su*ci
      b(3,3)=-slat
      b(1,1)=-ca*su-sa*cu*ci
      b(2,1)=-sa*su+ca*cu*ci
      b(3,1)= cu*si
      !     COMPUTES THE NORMALIZED SPACECRAFT POSITION VECTOR IN EARTH FIXED
      !     COORDINATES - XS.
      r=(nomorb+dr)/ae
      xs(1)=-b(1,3)*r
      xs(2)=-b(2,3)*r
      xs(3)=-b(3,3)*r
      !     PRECOMPUTES Q3 (USED IN LPOINT)
      q3=xs(1)**2+xs(2)**2+aebe2*xs(3)**2-1.0D0
      !     COMPUTES THE ATTITUDES AND MISALIGNMENTS IF IMC IS OFF
      IF (imc /= 0) THEN
         !     COMPUTES THE SOLAR ORBIT ANGLE
         wa=REC(60)*ts
         !     COMPUTES THE DIFFERENCE BETWEEN CURRENT TIME, TS, AND THE
         !     EXPONENTIAL TIME, REC(61). NOTE THAT BOTH TIMES ARE SINCE EPOCH.
         te=ts-REC(61)
         !     COMPUTES ROLL + ROLL MISALIGNMENT
         roll=roll+imc_gatt(62,REC,wa,te)
         !     COMPUTES PITCH + PITCH MISALIGNMENT
         pitch=pitch+imc_gatt(117,REC,wa,te)
         !     COMPUTES YAW
         yaw=yaw+imc_gatt(172,REC,wa,te)
         !     COMPUTES ROLL MISALIGNMENT
         rma=imc_gatt(227,REC,wa,te)
         !     COMPUTES PITCH MISALIGNMENT
         pma=imc_gatt(282,REC,wa,te)
         !     APPLY THE SPCECRAFT COMPENSATION
         roll=roll+REC(15)
         pitch=pitch+REC(16)
         yaw=yaw+REC(17)
      END IF
      !     COMPUTES THE INSTRUMENT TO EARTH FIXED COORDINATES TRANSFORMATION
      !     MATRIX - BT
      
      CALL imc_inst2er(roll,pitch,yaw,b,bt)
      RETURN
    END SUBROUTINE imc_lmodel
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : GPOINT
!**   SOURCE    : F.GPOINT
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     12/10/87  IL   INITIAL CREATION
!**   A     06/10/88  IL   REPLACED ASIN WITH ATAN TO SAVE TIME
!**   A     06/02/89  IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL 504-01
!**   4     03/08/94  SC   IMPLEMENTED NEW FORMULAE FOR SCAN ANGLE
!**                        CORRECTION DUE TO MISALIGNMENTS
!***********************************************************************
!**   THIS SUBROUTINE CONVERTS GEOGRAPHIC LATITUDE AND LONGITUDE
!**   TO THE RELATED ELEVATION AND SCAN ANGLES.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_gpoint(rlat,rlon,alf,gam,ierr,vza)
      !External variables
      REAL(kind=8), INTENT(IN OUT):: rlat ! GEOGRAPHIC LATITUDE IN RADIANS (INPUT)
      REAL(kind=8), INTENT(IN OUT):: rlon ! GEOGRAPHIC LONGITUDE IN RADIANS (INPUT)
      REAL(kind=8), INTENT(OUT)   :: alf  ! ELEVATION ANGLE IN RADIANS (OUTPUT)
      REAL(kind=8), INTENT(OUT)   :: gam  ! SCAN ANGLE IN RADIANS (OUTPUT)
      INTEGER, INTENT(OUT)        :: ierr ! OUTPUT STATUS; 
      !                                  0 - SUCCESSFUL COMPLETION,
      !                                  1 - POINT WITH GIVEN LAT/LON IS INVISIBLE
      real(kind=8),intent(in out),optional :: vza !Satellite View Zenith Angle (rad)
      !Internal variables
      REAL(kind=8) f(3)  ! POINTING VECTOR IN EARTH CENTERED COORDINATES
      REAL(kind=8) ft(3) ! POINTING VECTOR IN INSTRUMENT COORDINATES
      REAL(kind=8) u(3)  ! COORDINATES OF THE EARTH POINT (KM)
      REAL(kind=8) sing,slat,w1,w2 ! WORK SPACE
      !INCLUDE 'elcons.f90'
      !INCLUDE 'elcomm.f90'
!***********************************************************************
      !     COMPUTES SINUS OF GEOGRAPHIC (GEODETIC) LATITUDE
      sing=DSIN(rlat)
      w1=aebe4*sing*sing
      !     SINUS OF THE GEOCENTRIC LATITUDE
      slat=((0.375D0*w1-0.5D0)*w1+1.0D0)*sing/aebe2
      !     COMPUTES LOCAL EARTH RADIUS AT SPECIFIED POINT
      w2=slat*slat
      w1=aebe3*w2
      w1=(0.375D0*w1-0.5D0)*w1+1.d0
      !     COMPUTES CARTESIAN COORDINATES OF THE POINT
      u(3)=slat*w1
      w2=w1*DSQRT(1.0D0-w2)
      u(1)=w2*DCOS(rlon)
      u(2)=w2*DSIN(rlon)
      !     POINTING VECTOR FROM SATELLITE TO THE EARTH POINT
      f(1)=u(1)-xs(1)
      f(2)=u(2)-xs(2)
      f(3)=u(3)-xs(3)
      w2=u(1)*SNGL(f(1))+u(2)*SNGL(f(2))+ u(3)*SNGL(f(3))*aebe2
      !     VERIFIES VISIBILITY OF THE POINT
      IF (w2 > 0.0D0) THEN ! INVISIBLE POINT ON THE EARTH
         ierr=1
         alf=99999.0D0
         gam=99999.0D0
         RETURN
      END IF
      !     CONVERTS POINTING VECTOR TO INSTRUMENT COORDINATES
      ft(1)=bt(1,1)*f(1)+bt(2,1)*f(2)+bt(3,1)*f(3)
      ft(2)=bt(1,2)*f(1)+bt(2,2)*f(2)+bt(3,2)*f(3)
      ft(3)=bt(1,3)*f(1)+bt(2,3)*f(2)+bt(3,3)*f(3)
      !     CONVERTS POINTING VECTOR TO SCAN AND ELEVATION ANGLES AND
      !     CORRECTS FOR THE ROLL AND PITCH MISALIGNMENTS
      gam=DATAN(ft(1)/SQRT(ft(2)**2+ft(3)**2))
      alf=-DATAN(ft(2)/ft(3))
      w1=DSIN(alf)
      w2=DCOS(gam)
      alf=alf+rma*(1.0D0-DCOS(alf)/w2)+pma*w1*(1.0D0/w2+DTAN(gam))
      gam=gam-rma*w1
      ierr=0

      !Inserted by Knapp
      !Calculate the view zenith angle
      !It is the angle between U and -G1
!      VZA = acos( within(dot( unit(u), unit(-1.*f) ),&
!           &             minimum=-1.0d0,maximum=1.0d0) ) * rad

      RETURN
    END SUBROUTINE imc_gpoint
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : INST2ER
!**   SOURCE    : F.INST2ER
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   1     08/16/88  IL   INITIAL CREATION
!**   2     11/11/88  IL   TRIGONOMETRIC FUNCTIONS REPLACED WITH
!**                        SMALL ANGLE APPROXIMATIONS
!**   3    06/02/89   IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL 504-01
!***********************************************************************
!**   INST2ER ACCEPTS THE SINGLE PRECISION ROLL, PITCH AND YAW ANGLES
!**   OF AN INSTRUMENT AND RETURNS THE DOUBLE PRECISION INSTRUMENT TO
!**   EARTH COORDINATES TRANSFORMATION MATRIX.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_inst2er(r,p,y,a,at)
      !External variables
      REAL(kind=8), INTENT(IN) :: r ! ROLL ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN) :: p ! PITCH ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN) :: y ! YAW ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN) :: a(3,3) ! SPACECRAFT TO ECEF COORDINATES
      REAL(kind=8), INTENT(OUT):: at(3,3)! INSTRUMENT TO ECEF COORDINATES
      !Internal variables
      REAL(kind=8) rpy(3,3) ! INSTRUMENT TO BODY COORDINATES
                            ! TRANSFORMATION MATRIX
      INTEGER(kind=4) i,j   ! INDICES
!***********************************************************************
!     WE COMPUTE INSTRUMENT TO BODY COORDINATES TRANSFORMATION
!     MATRIX BY USING A SMALL ANGLE APPROXIMATION OF TRIGONOMETRIC
!     FUNCTIONS OF THE ROLL, PITCH AND YAW.
      rpy(1,1)=1.0D0-0.50D0*(p*p+y*y)
      rpy(1,2)=-y
      rpy(1,3)=p
      rpy(2,1)=y+p*r
      rpy(2,2)=1.0D0-0.50D0*(y*y+r*r)
      rpy(2,3)=-r
      rpy(3,1)=-p+r*y
      rpy(3,2)=r+p*y
      rpy(3,3)=1.0D0-0.50D0*(p*p+r*r)
      !     MULTIPLICATION OF MATRICES A AND RPY
      DO  i=1,3
         DO  j=1,3
            at(i,j)=a(i,1)*rpy(1,j)+a(i,2)*rpy(2,j)+a(i,3)*rpy(3,j)
         END DO
      END DO
      RETURN
    END SUBROUTINE imc_inst2er
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : LPOINT
!**   SOURCE    : F.LPOINT
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     01/09/89  IL   INITIAL CREATION
!**   A     06/02/89  IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL504-01
!**   3     03/08/94  SC   IMPLEMENTED NEW FORMULAE FOR SCAN ANGLE
!**                        CORRECTIONS DUE TO MISALIGNMENTS
!***********************************************************************
!**   THIS SUBROUTINE CONVERTS THE INSTRUMENT ELEVATION AND SCAN
!**   ANGLES TO THE RELATED GEOGRAPHIC LATITUDE AND LONGITUDE.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_lpoint(alpha,zeta,rlat,rlon,ierr,vza)
      !External variables
      REAL(kind=8), INTENT(IN) :: alpha ! ELEVATION ANGLE (RAD)
      REAL(kind=8), INTENT(IN) :: zeta  ! SCAN ANGLE (RAD)
      REAL(kind=8), INTENT(OUT):: rlat  ! LATITUDE IN RADIANS (OUTPUT)
      REAL(kind=8), INTENT(OUT):: rlon  ! LONGITUDE IN RADIANS (OUTPUT)
      INTEGER,      INTENT(OUT):: ierr  ! OUTPUT STATUS; 0 - POINT ON THE EARTH
      !                                   FOUND, 1 - INSTRUMENT POINTS OFF EARTH
      real(kind=8),intent(out) :: vza   ! Satellite View Zenith Angle (rad)
      !Internal variables
      REAL(kind=8) g1(3)   ! POINTING VECTOR IN EARTH CENTERED COORDINATES
      REAL(kind=8) h       ! SLANT DISTANCE TO THE EARTH POINT (KM)
      REAL(kind=8) q1,q2,d ! WORK SPACE
      REAL(kind=8) g(3)    ! POINTING VECTOR IN INSTRUMENT COORDINATES
      REAL(kind=8) u(3)    ! COORDINATES OF THE EARTH POINT (KM)
      REAL(kind=8) sa,ca,da,dz,d1,cz ! WORK SPACE
      !INCLUDE 'elcons.f90'
      !INCLUDE 'elcomm.f90'
!***********************************************************************
      ierr=1
      !     COMPUTES TRIGONOMETRIC FUNCTIONS OF THE SCAN AND ELEVATION
      !     ANGLES CORRECTED FOR THE ROLL AND PITCH MISALIGNMENTS
      ca=DCOS(alpha)
      sa=DSIN(alpha)
      cz=DCOS(zeta)
      da=alpha-pma*sa*(1.0D0/cz+DTAN(zeta))-rma*(1.0D0-ca/cz)
      dz=zeta+rma*sa
      !                              CORRECTED SCAN ANGLE
      cz=DCOS(dz)
      !     COMPUTES POINTING VECTOR IN INSTRUMENT COORDINATES
      g(1)=    DSIN(dz)
      g(2)=-cz*DSIN(da)
      g(3)= cz*DCOS(da)
      !     TRANSFORMS THE POINTING VECTOR TO EARTH FIXED COORDINATES
      g1(1)=bt(1,1)*g(1)+bt(1,2)*g(2)+bt(1,3)*g(3)
      g1(2)=bt(2,1)*g(1)+bt(2,2)*g(2)+bt(2,3)*g(3)
      g1(3)=bt(3,1)*g(1)+bt(3,2)*g(2)+bt(3,3)*g(3)
      !     COMPUTES COEFFICIENTS AND SOLVES A QUADRATIC EQUATION TO
      !     FIND THE INTERSECT OF THE POINTING VECTOR WITH THE EARTH
      !     SURFACE
      q1=g1(1)**2+g1(2)**2+aebe2*g1(3)**2
      q2=xs(1)*g1(1)+xs(2)*g1(2)+aebe2*xs(3)*g1(3)
      d=q2*q2-q1*q3
      IF (DABS(d) < 1.d-9) d=0.0D0
      !     IF THE DISCIMINANTE OF THE EQUATION, D, IS NEGATIVE, THE
      !     INSTRUMENT POINTS OFF THE EARTH
      IF (d < 0.0D0) THEN
         rlat=999999.0D0
         rlon=999999.0D0
         vza = 999999.0D0
         RETURN
      END IF
      d=DSQRT(d)
      !     SLANT DISTANCE FROM THE SATELLITE TO THE EARTH POINT
      h=-(q2+d)/q1
      !     CARTESIAN COORDINATES OF THE EARTH POINT
      u(1)=xs(1)+h*g1(1)
      u(2)=xs(2)+h*g1(2)
      u(3)=xs(3)+h*g1(3)
      !     SINUS OF GEOCENTRIC LATITUDE
      d1=u(3)/DSQRT(u(1)**2+u(2)**2+u(3)**2)
      !     GEOGRAPHIC (GEODETIC) COORDINATES OF THE POINT
      rlat=DATAN(aebe2*d1/DSQRT(1.0D0-d1*d1))
      rlon=DATAN2(u(2),u(1))

      !Inserted by Knapp
      !Calculate the view zenith angle
      !It is the angle between U and -G1
!      VZA = acos( within(dot( unit(u), unit(-1.*g1) ),&
!           &             minimum=-1.0d0,maximum=1.0d0) ) * rad

      ierr=0
      RETURN
    END SUBROUTINE imc_lpoint
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : SNDELOC
!**   SOURCE    : F.SNDELOC
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     02/16/89  IL   INITIAL CREATION
!***********************************************************************
!**   SNDELOC ACCEPTS THE MIRROR POSITION IN CYCLES AND INCREMENTS,
!**   SERVO ERROR VALUES, AND THE POSITIONAL OFFSETS FOR FOUR DETECTORS
!**   OF A SELECTED SOUNDER CHANNEL AND COMPUTES THE DETECTOR EARTH
!**   LOCATIONS IN LATITUDE/LONGITUDE COORDINATES.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : LPOINT
!***********************************************************************
    SUBROUTINE imc_sndelo(cyew,incew,cyns,incns,svew,svns,doff,geo)
      !External variables
      INTEGER, INTENT(IN OUT)     :: cyew  ! E-W CYCLES
      INTEGER, INTENT(IN OUT)     :: incew ! E-W INCREMENTS
      INTEGER, INTENT(IN OUT)     :: cyns  ! N-S CYCLES
      INTEGER, INTENT(IN OUT)     :: incns ! N-S INCREMENTS
      REAL(kind=8), INTENT(IN)    :: svew  ! E-W SERVO ERROR IN RADIANS
      REAL(kind=8), INTENT(IN)    :: svns  ! N-S SERVO ERROR IN RADIANS
      REAL(kind=8), INTENT(IN)    :: doff(4,2) !OFFSETS FOR 4 DETECTORS (RADIANS)
      !                                         DOFF(*,1) = E-W OFFSET
      !                                         DOFF(*,2) = N-S OFFSET
      REAL(kind=8), INTENT(IN OUT):: geo(4,2) ! GEOGRAPHIC COORDINATES RELATED 
      !                                               TO 4 DETECTORS
      !                                         GEO(*,1) = LATITUDE IN RADIANS
      !                                         GEO(*,2) = LONGITUDE IN RADIANS
      !Internal variables
      REAL(kind=8) e,s,h,ev,sc,sine,cose,de,ds,vza
      INTEGER :: i,ier
      !INCLUDE 'instco.f90'
!***********************************************************************
      !     CONVERT THE MIRROR POSITION, GIVEN IN CYCLES AND INCREMENTS, TO
      !     ELEVATION AND SCAN ANGLES
      !     E=(CYNS*INCMAX(2)+INCNS)*ELVINC(2)-ELVMAX(2)
      e=((cyns-9)*incmax(2)+incns)*elvinc(2)-elvmax(2)
      s=(cyew*incmax(2)+incew)*scninc(2)-scnmax(2)
      !     CORRECT ELEVATION AND SCAN ANGLES FOR SERVO ERRORS OBTAINING THE
      !     TRUE MIRROR POINTING
      e=e+svns
      s=s+svew
      sine=DSIN(e)
      cose=DCOS(e)
      h=-2.0D0*scnpx(2)
!     COMPUTE DETECTOR ROTATION OFFSETS FOR EACH DETECTOR
!      ALPHA = 0.643501D0 + E
!      BETA = 0.244979D0 - E
!      DOFF(1,1) = -0.064976D0
!      DOFF(1,2) = 0.00042D0
!      DOFF(2,1) = 0.00056D0
!      DOFF(2,2) = 0.00014D0
!      DOFF(3,1) = -0.064976D0
!      DOFF(3,2) = -0.065396D0
!      DOFF(4,1) = 0.00056D0
!      DOFF(4,2) = -0.065116D0
!      DOFF(1,1) = - 700.0D0*DCOS(ALPHA)*1.0D-6
!      DOFF(1,2) =   700.0D0*DSIN(ALPHA)*1.0D-6
!      DOFF(2,1) =   577.23479D0*DCOS(BETA)*1.D-6
!      DOFF(2,2) =   577.23479D0*DSIN(BETA)*1.0D-6
!      DOFF(3,1) = - 577.23479D0*DCOS(BETA)*1.0D-6
!      DOFF(3,2) = - 577.23479D0*DSIN(BETA)*1.0D-6
!      DOFF(4,1) =   700.0D0*DCOS(ALPHA)*1.0D-6
!      DOFF(4,2) = - 700.0D0*DSIN(ALPHA)*1.0D-6
!     COMPUTE EARTH LOCATIONS FOR FOUR DETECTORS
      DO  i=1,4
         !     COMPUTE POSITIONAL OFFSETS OF I-TH DETECTOR
         de=(2.5-i)*elvln(2)+doff(i,2)
         ds=h+doff(i,1)
         !     COMPUTE ELEVATION AND SCAN ANGLES RELATED TO I-TH DETECTOR
         !     AND CORRECT THEM FOR THE DETECTOR POSITIONAL OFFSETS
         !           EV=E+DOFF(I,2)
         !           SC=S+DOFF(I,1)
         !     CONVERT POSITIONAL OFFSETS TO ANGULAR OFFSETS AND
         !     CORRECT ELEVATION AND SCAN ANGLES
         ev = e + de*cose - ds*sine
         sc = s + de*sine + ds*cose
         !     TRANSFORM DETECTOR'S POINTING ANGLES TO GEOGRAPHIC COORDINATES
         !     OF THE CORRESPONDING POINT ON THE EARTH SURFACE.
         !     NOTE:  IF A DETECTOR LOOKS OFF THE EARTH, THE RELATED LATITUDE
         !            AND LONGITUDE ARE SET TO 999999.
         CALL imc_lpoint(ev,sc,geo(i,1),geo(i,2),ier,vza)
         h=-h
      END DO
      RETURN
    END SUBROUTINE imc_sndelo
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : EVSC2LPF
!**   SOURCE    : F.EVSC2LPF
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     10/27/88  IL   INITIAL CREATION
!***********************************************************************
!**   THIS SUBROUTINE CONVERTS ELEVATION AND SCAN ANGLES
!**   TO THE FRACTIONAL LINE AND PIXEL NUMBERS.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_evsc2l(instr,elev,scan,rl,rp)
      !External variables
      INTEGER,      INTENT(IN OUT):: instr ! INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)
      REAL(kind=8), INTENT(IN OUT):: elev  ! ELEVATION ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN OUT):: scan  ! SCAN ANGLE IN RADIANS
      REAL(kind=8), INTENT(OUT)   :: rl    ! LINE NUMBER
      REAL(kind=8), INTENT(OUT)   :: rp    ! PIXEL NUMBER
!**************************************************************
      !     COMPUTE FRACTIONAL LINE NUMBER
      rl=(elvmax(instr)-elev)/elvln(instr)
      IF (instr == 1) THEN
         rl=rl+4.5D0
      ELSE
         rl=rl+2.5D0
      END IF
      !     COMPUTE FRACTIONAL PIXEL NUMBER
      rp=(scnmax(instr)+scan)/scnpx(instr)+1.0D0
      RETURN
    END SUBROUTINE imc_evsc2l
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : EVLN
!**   SOURCE    : F.EVLN
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     10/27/88  IL   INITIAL CREATION
!***********************************************************************
!**   THIS FUNCTION CONVERTS FRACTIONAL LINE NUMBER TO ELEVATION ANGLE
!**   IN RADIANS.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_evln(instr,rline)
      !External variables
      INTEGER,      INTENT(IN OUT):: instr  ! INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)
      REAL(kind=8), INTENT(IN OUT):: rline
      REAL(kind=8)  imc_evln                    ! FRACTIONAL LINE  NUMBER
!***********************************************************************
      IF (instr == 1) THEN
         imc_evln=elvmax(instr)*1.0D0 - (rline-4.5 )*(elvln(instr)*1.0D0)
      ELSE
         imc_evln=elvmax(instr)*1.0D0 - (rline -2.5D0)*(elvln(instr)*1.0D0)
      END IF
      RETURN
    END FUNCTION imc_evln
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : SCPX
!**   SOURCE    : F.SCPX
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     09/22/87  IL   INITIAL CREATION
!***********************************************************************
!**   THIS FUNCTION CONVERTS FRACTIONAL PIXEL NUMBER TO SCAN ANGLE
!**   IN RADIANS.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_scpx(instr,pix)
      !External variables
      INTEGER,      INTENT(IN OUT):: instr ! INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)
      REAL(kind=8), INTENT(IN OUT):: pix   ! FRACTIONAL PIXEL NUMBER
      REAL(kind=8) imc_scpx
!***********************************************************************
      imc_scpx=((pix*1.0D0)-1.0D0)*(scnpx(instr)*1.0D0) -(scnmax(instr)*1.0D0)
      RETURN
    END FUNCTION imc_scpx
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : GATT
!**   SOURCE    : F.GATT
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     12/01/88  IL   INITIAL CREATION
!***********************************************************************
!**    THIS FUNCTION COMPUTES AN ATTITUDE/MISALIGNMENT ANGLE FROM A
!**    GIVEN SUBSET OF THE O&A PARAMETERS IN GVAR BLOK 0.
!**    ARGUMENT K0 INDICATES THE FIRST WORD OF THE SUBSET.
!***********************************************************************
!**   CALLED BY       : LMODEL
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_gatt(k0,REC,wa,te)
      !External variables
      INTEGER,     INTENT(IN):: k0       ! STARTING POSITION OF A PARAMETER SUBSET IN THE
      REAL(kind=8),INTENT(IN):: REC(336) ! INPUT O&A PARAMETER SET
      REAL(kind=8),INTENT(IN):: wa       ! INPUT SOLAR ORBIT ANGLE IN RADIANS
      REAL(kind=8),INTENT(IN):: te       ! INPUT EXPONENTIAL TIME DELAY FROM EPOCH (MINUTES)
      REAL(kind=8) :: imc_gatt
      ! Internal variables 
      INTEGER(kind=4) i, j, m,l,ll,k
      REAL   (kind=8) ir,jr,mr,att
      EQUIVALENCE (j,jr),(m,mr)
      !***********************************************************************
      !     CONSTANT COMPONENT
      k=k0
      att=REC(k+2)
      !     COMPUTES THE EXPONENTIAL TERM
      IF ((te >= 0.0D0).AND.(REC(k+1) > 0))THEN
         att=att+REC(k)*DEXP(-te/REC(k+1))
      END IF
      !     EXTRACTS THE NUMBER OF SINUSOIDS
      ir=REC(k+3)
      !      I = INT4(IR)
      i = INT(ir)
      !     CALCULATION OF SINUSOIDS
      DO  l=1,i
         att=att+REC(k+2*l+2)*DCOS(wa*l+REC(k+2*l+3))
      END DO
      !     POINTER TO THE NUMBER OF MONOMIAL SINUSOIDS
      k=k+34
      !     EXTACTS NUMBER OF MONOMIAL SINUSOIDS
      ir=REC(k)
      !      I=INT4(IR)
      i=INT(ir)
      !     KKK=REC(K)
      !     COMPUTES MONOMIAL SINUSOIDS
      DO  l=1,i
         ll=k+5*l
         !          ORDER OF SINUSOID
         jr=REC(ll-4)
         !          ORDER OF MONOMIAL SINUSOID
         mr=REC(ll-3)
         att=att+REC(ll-2)*((wa-REC(ll))**m)*DCOS(j*wa+REC(ll-1))
      END DO
      imc_gatt=att
      RETURN
    END FUNCTION imc_gatt
  END SUBROUTINE imcnav
!                                                             END IMCNAV
!#######################################################################

!#######################################################################
!                                                        BEGIN IMCNAV_R1
!****r* kmod_mcidas/imcnav_r1
! NAME
! imcnav_r1
! SYNOPSIS
! call imcnav_r1(dlat,dlon,line,pixel,key,imc,instr,
!       nye,nde,nhe,nme,se,  &
!       ny,nd,nh,nm,s,  &
!       ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
!       REC, glat, glon, grad, vza,scanangle)
! FUNCTION
! Performs conversion between lat/lon and i/j (the direction of the 
! transform depends on key).
! INPUTS
! OUTPUT
! NOTES
! * This code is the code delivered by contractors as part of the Earth Location 
!   User's Guide (ELUG) It was converted to Fortran90 code by to_f90.
!   * Code converted using TO_F90 by Alan Miller Date: 2004-12-03  Time: 08:31:57
! * This code is based on the imcnav routine.
! MODIFICATION HISTORY
! * Conversion to Fortran90
! * Knapp adjusted this to use rank1 location parameters
! * Knapp adjusted it to optionall return scan/elevation angles instead
!   of lat/lons.
! 
!***
!     Front end to IDL Call to navigate GVAR imagery
!     Calls fortran routines to perform navigation
  SUBROUTINE imcnav_r1(dlat,dlon,line,pixel,  &
       key,imc,instr,  &
       nye,nde,nhe,nme,se,  &
       ny,nd,nh,nm,s,  &
       ew_nad_cy, ew_nad_inc, ns_nad_cy, ns_nad_inc,  &
       REC, glat, glon, grad, vza,scanangle)
    implicit none
    !External variables
    REAL(kind=8), INTENT(IN OUT),dimension(:) :: dlat
    REAL(kind=8), INTENT(IN OUT),dimension(:) :: dlon
    REAL(kind=8), INTENT(IN OUT),dimension(:) :: line
    REAL(kind=8), INTENT(IN OUT),dimension(:) :: pixel
    INTEGER, INTENT(IN OUT)      :: key
    INTEGER, INTENT(IN OUT)      :: imc
    INTEGER, INTENT(IN OUT)      :: instr
    INTEGER, INTENT(IN OUT)      :: nye
    INTEGER, INTENT(IN)          :: nde
    INTEGER, INTENT(IN)          :: nhe
    INTEGER, INTENT(IN)          :: nme
    REAL(kind=8), INTENT(IN OUT) :: se
    INTEGER, INTENT(IN OUT)      :: ny
    INTEGER, INTENT(IN)          :: nd
    INTEGER, INTENT(IN)          :: nh
    INTEGER, INTENT(IN)          :: nm
    REAL(kind=8), INTENT(IN OUT) :: s
    INTEGER, INTENT(IN OUT)      :: ew_nad_cy
    INTEGER, INTENT(IN OUT)      :: ew_nad_inc
    INTEGER, INTENT(IN OUT)      :: ns_nad_cy
    INTEGER, INTENT(IN OUT)      :: ns_nad_inc
    REAL(kind=8), INTENT(IN OUT) :: REC(336)
    REAL(kind=8), INTENT(OUT)    :: glat
    REAL(kind=8), INTENT(OUT)    :: glon
    REAL(kind=8), INTENT(OUT)    :: grad
    REAL(kind=8), intent(out),optional,dimension(size(dlon))    :: vza
    integer, optional :: scanangle
    !Internal variables
    real(kind=8), dimension(size(dlat)) :: rp, rl, e, ss, rlat, rlon
    integer, dimension(size(dlat)) :: ier
    REAL(kind=8) t,tu
    !parameters ... I hope this doesn't cause a memory leak
!!$    REAL(kind=8), PARAMETER :: svx = -21.d-6
!!$    REAL(kind=8), PARAMETER :: svy =  14.d-6
    REAL(kind=8), PARAMETER, dimension(4,2) :: &
         doff= reshape( (/28.e-6,56.e-6,-28.e-6,-56.e-6, &
         84.e-6,112.e-6, 14.e-6,42.e-6/), (/4,2/) )
    INTEGER(kind=4), PARAMETER :: icx = 1, incx = 2715, icy = 5, incy = 2580
    !------------------------------------------------------------------------
    !      INCLUDE variables
    !  By including the information here, the variables will be available to 
    !  routines within the CONTAINS statement
    !INCLUDE 'elcons.f90'
!   REAL(kind=8), parameter :: pi=3.141592653589793D0
    REAL(kind=8), parameter :: deg=180.d0/pi
!    REAL(kind=8), PARAMETER :: rad=pi/180.d0! DEGREES TO RADIANS CONVERSION PI/180
    REAL(kind=8), PARAMETER :: nomorb=42164.365D0 ! NOM RAD DISTANCE OF SATELLITE (km)
    REAL(kind=8), PARAMETER :: ae=6378.137D0      ! EARTH EQUATORIAL RADIUS (km)
    REAL(kind=8), PARAMETER :: fer=1.d0-(6356.7533D0/ae) ! EARTH FLAT COEFFICIENT
    REAL(kind=4), PARAMETER :: aebe2=1.d0/(1.d0-fer)**2
    REAL(kind=4), PARAMETER :: aebe3=aebe2-1.
    REAL(kind=4), PARAMETER :: aebe4=(1.d0-fer)**4-1.
    !INCLUDE 'instco.f90'
    INTEGER(kind=4) incmax(2) ! NUMBER OF INCREMENTS PER CYCLE
    REAL(kind=4) elvmax(2)    ! BOUNDS IN ELEVATION (RADIANS)
    REAL(kind=4) scnmax(2)    ! BOUNDS IN SCAN ANGLE (RADIANS)
    REAL(kind=4) elvinc(2)    ! CHANGE IN ELEVATION ANGLE PER INCREMENT (RAD)
    REAL(kind=4) scninc(2)    ! CHANGE IN SCAN ANGLE PER INCREMENT (RADIANS)
    REAL(kind=4) elvln(2)     ! ELEVATION ANGLE PER DETECTOR LINE (RADIANS)
    REAL(kind=4) scnpx(2)     ! SCAN ANGLE PER PIXEL (RADIANS)
    !INCLUDE 'elcomm.f90'
    REAL(kind=8) xs(3)          ! NORMALIZED S/C POSITION IN ECEF COORDINATES
    REAL(kind=8) bt(3,3)        ! ECEF TO INSTRUMENT COORDINATES TRANSFORMATION
    REAL(kind=8)  q3            ! USED IN SUBROUTINE LPOINT
    REAL(kind=8) pitch,roll,yaw ! PITCH,ROLL,YAW ANGLES OF INSTRUMENT (RAD)
    REAL(kind=4) pma,rma        ! PITCH,ROLL MISALIGNMENTS OF INSTRUMENT (RAD)
  
    CALL imc_setcon(instr,ns_nad_cy,ns_nad_inc,ew_nad_cy,ew_nad_inc)
    !     COMPUTE EPOCH TIME
    tu=imc_timex(nye,nde,nhe,nme,se)
    !     COMPUTE ACTUAL TIME
    t=imc_timex(ny,nd,nh,nm,s)
    !     COMPUTE DATA NEEDED FOR INSTRUMENT TO EARTH COORDINATES
    !     TRANSFORMATION
    CALL imc_lmodel(t,tu,REC,imc,glat,glon,grad)
    
    IF (key /= 1) then !GO TO 10
       !       TRANSFORM LINE/PIXEL TO GEOGRAPHIC COORDINATES:
       !       SET INPUT LINE/PIXEL NUMBERS
       rp = line
       rl = pixel
       !       COMPUTE ELEVATION AND SCAN ANGLES (E,S) RELATED TO INPUT
       !       LINE AND PIXEL NUMBERS
       e=imc_evln(instr,rp)
       ss=imc_scpx(instr,rl)
       if (present(scanangle)) then
          dlon = ss
          dlat = e
          return
       end if
       !       TRANSFORM ELEVATION AND SCAN ANGLES TO GEOGRAPHIC COORDINATES
       CALL imc_lpoint(e,ss,rlat,rlon,ier,vza)
       dlat = rlat*deg
       dlon = rlon*deg
    else
       !     IF KEY == 1
       !     GEOGRAPHIC TO LINE/PIXEL COORDINATES TRANSFORMATION:
       !       SET INPUT LATITUDE AND LONGITUDE
       rlat=dlat/rad
       rlon=dlon/rad
       !       TRANSFORM LAT/LON TO ELEVATION AND SCAN ANGLES
       CALL imc_gpoint(rlat,rlon,e,ss,ier,vza)
       if (present(scanangle)) then
          line = e
          pixel= ss
          return
       endif
       !       CONVERT ELEVATION AND SCAN ANGLES TO LINE/PIXEL COORDINATES
       CALL imc_evsc2l(instr,e,ss,rl,rp)
       line = rl
       pixel = rp
       where (ier == 1)
          line = -1000.
          pixel = -1000.
       end where
    end IF
    
  CONTAINS
!***********************************************************************
!**   NOAA/NESDIS/SOCC/SOFTWARE BRANCH
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : IMC_TIME50
!**   SOURCE    : F.IMC_TIME50
!**   LOAD NAME : ANY
!**   PROGRAMMER: THOMAS I. BABICKI
!**
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A      2/17/89  TB   INITIAL CREATION
!***********************************************************************
!**   IMC_TIME50 ACCEPTS TWO WORDS CONTAINING DATE AND TIME
!**   AND RETURNS TIME EXPRESSED AS DOUBLE PRECISION MINUTES FROM
!**   1950 JAN. 1.0
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_time50(i)
      !External variables
      INTEGER(kind=4), INTENT(IN) :: i(2)
      REAL(kind=8) :: imc_time50
      !Internal variables
      INTEGER(kind=4) ny ! YEAR
      INTEGER(kind=4) nd ! DAY OF YEAR
      INTEGER(kind=4) nh ! HOUR
      INTEGER(kind=4) nm ! MINUTE
      REAL(kind=8)    s  ! SECONDS
      INTEGER :: j
      integer :: iaa, iab, nbc, iac, def
      !***********************************************************************
      !     CONVERT INPUT YEAR, DAY OF YEAR, HOUR AND MINUTE
      !     TO INTEGER VALUES.
      ny=i(1)/10000
      iaa=i(1)-(ny*10000)
      nd=(i(1)-(ny*10000))*.1
      iab=(iaa-(nd*10))*10
      nbc=i(2)/10000000.
      iac=i(2)-(nbc*10000000)
      nh=iab+nbc
      def=i(2)-iac
      nm=iac*.00001
      s=(i(2)-(def+(nm*100000)))*.001D0
      !***********************************************************************
      !     HERE WE CONVERT INTEGER YEAR AND DAY OF YEAR TO NUMBER OF
      !     DAYS FROM 0 HOUR UT, 1950 JAN. 1.0
      !     THIS CONVERTION IS BASED ON AN ALGORITHM BY FLIEGEL AND VAN
      !     FLANDERN, COMM. OF ACM, VOL.11, NO. 10, OCT. 1968 (P.657)
      j=nd+1461*(ny+4799)/4-3*((ny+4899)/100)/4-2465022
      !     COMPUTE TIME IN MINUTES FROM JANUARY 1.0, 1950
      imc_time50=j*1440.d0+nh*60.d0+nm+s/60.d0
      RETURN
    END FUNCTION imc_time50
!***********************************************************************
!**   NOAA/NESDIS/SOCC/SOFTWARE BRANCH
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : IMC_TIMEX
!**   SOURCE    : F.TIMEX
!**   LOAD NAME : ANY
!**   PROGRAMMER: THOMAS I. BABICKI
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A      4/25/89  TB   INITIAL CREATION
!***********************************************************************
    FUNCTION imc_timex(ny,nd,nh,nm,s)
      !External variables
      INTEGER(kind=4), INTENT(IN OUT):: ny
      INTEGER(kind=4), INTENT(IN)    :: nd
      INTEGER(kind=4), INTENT(IN)    :: nh
      INTEGER(kind=4), INTENT(IN)    :: nm
      REAL(kind=8), INTENT(IN)       :: s
      REAL(kind=8) imc_timex
      !Internal variables
      INTEGER :: j
      j=nd+1461*(ny+4799)/4-3*((ny+4899)/100)/4-2465022
      !     COMPUTE ACTUAL TIME IN MINUTES FROM JANUARY 1.0, 1950
      imc_timex=j*1440.d0+nh*60.d0+nm+s/60.d0
      RETURN
    END FUNCTION imc_timex
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : SETCONS
!**   SOURCE    : F.SETCONS
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     02/16/89  IL   INITIAL CREATION
!**
!**   B     05/19/94  NP   ADDED CALCULATION OF INSTRUMENT ELEVATION AND
!**                        SCAN ANGLE BIASES BASED ON USER INPUT
!***********************************************************************
!**   THIS SUBROUTINE GENERATES CONSTANTS IN COMMON  INSTCOMM
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: INSTCO
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
!***********************************************************************
    SUBROUTINE imc_setcon(instr,ns_nad_cy,ns_nad_inc,ew_nad_cy,ew_nad_inc)
      !External variables
      INTEGER, INTENT(IN OUT) :: instr
      INTEGER, INTENT(IN OUT) :: ns_nad_cy
      INTEGER, INTENT(IN OUT) :: ns_nad_inc
      INTEGER, INTENT(IN OUT) :: ew_nad_cy
      INTEGER, INTENT(IN OUT) :: ew_nad_inc
!***********************************************************************
      incmax(1)=6136
      incmax(2)=2805
      elvinc(1)=8.0D-6
      elvinc(2)=17.5D-6
      scninc(1)=16.d-6
      scninc(2)=35.d-6
      elvln(1)=28.d-6
      elvln(2)=280.d-6
      scnpx(1)=16.d-6
      scnpx(2)=280.d-6
!     ************************************************************
!     COMMENTED OUT ELEVATION AND SCAN BIAS CONSTANTS SINCE INSTRUMENT
!     EARTH NADIR POSITION IS AVAILABLE IN GVAR DATA AND PERIODICALLY
!     UPDATED
!        ELVMAX(1)=0.220896D0
!        ELVMAX(2)=0.22089375D0
!        SCNMAX(1)=0.24544D0
!        SCNMAX(2)=0.2454375D0
!     RECOMPUTE ELEVATION AND SCAN BIASES BASED ON USER INPUTS OF
!     CYCLES & INCREMENTS OBTAINED FROM GVAR
!     ELVMAX(INSTR) = (NS_NAD_CY*INCMAX(INSTR)+NS_NAD_INC)*ELVINC(INSTR)
      IF(instr == 1)THEN
         elvmax(instr)=(ns_nad_cy*incmax(instr)+ns_nad_inc)*elvinc(instr)
      ELSE
         elvmax(instr)=((9-ns_nad_cy)*incmax(instr)-ns_nad_inc) *elvinc(instr)
      END IF
      scnmax(instr) = (ew_nad_cy*incmax(instr)+ew_nad_inc)*scninc(instr)
!     ************************************************************
      RETURN
    END SUBROUTINE imc_setcon
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : LMODEL
!**   SOURCE    : F.LMODEL
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   1     01/09/89  IL   INITIAL CREATION
!**   2     06/02/89  IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL 504-01
!**   3     08/21/89  IL   CORRECTED ORBIT ANGLE COMPUTATIONS
!**   4     03/08/94  SC   S/C COMPENSATION APPLIED UNCONDITIONALLY;
!**                        REFERENCE RADIAL DISTANCE, LATITUDE AND
!**                        ORBIT YAW SET TO ZERO IF IMC DISABLED.
!**   5     03/08/94  SC   ADDED TRAP FOR SLAT=SYAW=0; CORRECTED
!**                        EXPRESSION FOR LAM.
!***********************************************************************
!**   THIS SUBROUTINE COMPUTES THE POSITION OF THE SATELLITE AND THE
!**   ATTITUDE OF THE IMAGER OR SOUNDER.  THE CALCULATIONS ARE BASED
!**   ON THE OATS ORBIT AND ATTITUDE MODEL REPRESENTED BY THE O&A
!**   PARAMETER SET IN GVAR BLOCK 0.
!**        INPUTS:
!**          TIME, EPOCH TIME, O&A PARAMETER SET, IMC STATUS.
!**        OUTPUTS:
!**          THE SPACECRAFT POSITION VECTOR IN EARTH FIXED COORDINATES;
!**          THE GEOMETRIC ROLL, PITCH, YAW ANGLES AND THE ROLL,
!**          PITCH MISALIGNMENTS FOR EITHER THE IMAGER OR THE SOUNDER;
!**          THE EARTH FIXED TO INSTRUMENT FRAME TRANSFORMATION MATRIX;
!**          GEOGRAPHIC LATITUDE AND LONGITUDE AT SUBSATELLITE POINT.
!**   DESCRIPTION
!**   LMODEL ACCEPTS AN INPUT DOUBLE PRECISION TIME IN MINUTES FROM
!**   1950, JAN.1.0 AND AN INPUT SET OF O&A PARAMETERS AND COMPUTES
!**   POSITION OF THE SATELLITE, THE ATTITUDE ANGLES AND ATTITUDE
!**   MISALIGNMENTS AND THE INSTRUMENT TO EARTH FIXED COORDINATES
!**   TRANSFORMATION MATRIX.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: /ELCOMM/ XS,Q3,PITCH,ROLL,YAW,PMA,RMA,BT
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : INST2ER,GATT
!***********************************************************************
    SUBROUTINE imc_lmodel(t,tu,REC,imc,rlat,rlon,rrad)
      !External variables
      REAL(kind=8), INTENT(IN)    :: t       ! INPUT TIME FROM 1950, JAN 1.0 (MIN)
      REAL(kind=8), INTENT(IN)    :: tu      ! EPOCH TIME FROM 1950, JAN 1.0 (MIN)
      REAL(kind=8), INTENT(IN)    :: REC(336)! INPUT O&A PARAMETER SET
      INTEGER,      INTENT(IN OUT):: imc     ! INPUT IMC STATUS: 0 - ON, 1 - OFF
      REAL(kind=8), INTENT(OUT)   :: rlat    ! SUBSATELLITE GEODETIC LATITUDE (RAD)
      REAL(kind=8), INTENT(OUT)   :: rlon,rrad ! SUBSATELLITE LONGITUDE IN RADIANS
      !Internal variables
      REAL(kind=8) r      ! NORMALIZED SATELLITE DISTANCE (IN UNITS OF KMER9)
      REAL(kind=8) ts     ! TIME FROM EPOCH IN MINUTES
      REAL(kind=8) b(3,3) ! SPACCRAFT TO EARTH FIXED COORDINATES TRANSFORMATION
      !                     MATRIX
      REAL(kind=8) te     ! EXPONENENTIAL TIME DELAY FROM EPOCH (IN MINUTES)
      REAL(kind=8) phi    ! SUBSATELLITE GEOCENTRIC LATITUDE IN RADIANS
      REAL(kind=8) dr     ! RADIAL DISTANCE FROM THE NOMINAL (KM)
      REAL(kind=8) psi    ! ORBITAL YAW (IN RADIANS)
      REAL(kind=8) lam    ! IMC LONGITUDE (IN RADIANS)
      REAL(kind=8) u      ! ARGUMENT OF LATITUDE (IN RADIANS)
      REAL(kind=8) su,cu  ! SIN(U), COS(U)
      REAL(kind=8) si,ci  ! SINE AND COSINE OF THE ORBIT INCLINATION
      REAL(kind=8) slat   ! SINE OF GEOCENTRIC LATITUDE
      REAL(kind=8) asc    ! LONGITUDE OF THE ASCENDING NODE IN RADIANS
      REAL(kind=8) sa,ca  ! SINE AND COSINE OF ASC
      REAL(kind=8) syaw   ! SINE OF THE ORBIT YAW
      REAL(kind=8) wa     ! SOLAR ORBIT ANGLE IN RADIANS
      REAL(kind=8) w      ! ORBIT ANGLE IN RADIANS
      REAL(kind=8) sw,cw  ! SIN(W),  COS(W)
      REAL(kind=8) s2w,c2w! SIN(2*W),  COS(2*W)
      REAL(kind=8) sw1,cw1! SIN(0.927*W),  COS(0.927*W)
      REAL(kind=8) sw3,cw3! SINE AND COSINE OF 1.9268*W
      REAL(kind=8) dlat   ! CHANGE IN SINE OF GEOCENTRIC LATITUDE
      REAL(kind=8) dyaw   ! CHANGE IN SINE OF ORBIT YAW
      !INCLUDE 'elcons.f90'
      !INCLUDE 'elcomm.f90'
!***********************************************************************
      !     ASSIGN REFERENCE VALUES TO THE SUBSATELLITE LONGITUDE AND
      !     LATITUDE, THE RADIAL DISTANCE AND THE ORBIT YAW.
      lam=REC(5)
      dr=REC(6)
      phi=REC(7)
      psi=REC(8)
      !     ASSIGN REFERENCE VALUES TO THE ATTITUDES AND MISALIGNMENTS
      roll=REC(9)
      pitch=REC(10)
      yaw=REC(11)
      rma=0.0D0
      pma=0.0D0
      !     IF IMC IS OFF, COMPUTE CHANGES IN THE SATELLITE ORBIT
      IF (imc /= 0) THEN
         !     SET REFERENCE RADIAL DISTANCE, LATITUDE AND ORBIT YAW TO ZERO
         dr=0.0D0
         phi=0.0D0
         psi=0.0D0
         !     COMPUTE TIME SINCE EPOCH (IN MINUTES)
         ts=t-tu
         !     COMPUTES ORBIT ANGLE AND THE RELATED TRIGONOMETRIC FUNCTIONS.
         !     EARTH ROTATIONAL RATE=.729115E-4 (RAD/S)
         w=0.729115D-4*60.0D0*ts
         sw=DSIN(w)
         cw=DCOS(w)
         sw1=DSIN(0.927D0*w)
         cw1=DCOS(0.927D0*w)
         s2w=DSIN(2.0D0*w)
         c2w=DCOS(2.0D0*w)
         sw3=DSIN(1.9268D0*w)
         cw3=DCOS(1.9268D0*w)
         !     COMPUTES CHANGE IN THE IMC LONGITUDE FROM THE REFERENCE
         lam=lam+REC(18)+(REC(19)+REC(20)*w)*w  &
                + (REC(27)*sw1+REC(28)*cw1+REC(21)*sw+REC(22)*cw  &
                + REC(23)*s2w+REC(24)*c2w + REC(25)*sw3+REC(26)*cw3  &
                + w*(REC(29)*sw+REC(30)*cw))*2.0D0
         !     COMPUTES CHANGE IN RADIAL DISTANCE FROM THE REFERENCE (KM)
         dr=dr + REC(31) + REC(32)*cw+REC(33)*sw  &
               + REC(34)*c2w+REC(35)*s2w + REC(36)*cw3+REC(37)*sw3  &
               + REC(38)*cw1+REC(39)*sw1 + w*(REC(40)*cw+REC(41)*sw)
         !     COMPUTES THE SINE OF THE CHANGE IN THE GEOCENTRIC LATITUDE
         dlat=REC(42) + REC(43)*cw+REC(44)*sw +REC(45)*c2w+REC(46)*s2w  &
              + w*(REC(47)*cw+REC(48)*sw) +REC(49)*cw1+REC(50)*sw1
         !     COMPUTES GEOCENTRIC LATITUDE BY USING AN EXPANSION FOR ARCSINE
         phi=phi+dlat*(1.0D0+dlat*dlat/6.0D0)
         !     COMPUTES SINE OF THE CHANGE IN THE ORBIT YAW
         dyaw=REC(51) + REC(52)*sw+REC(53)*cw +REC(54)*s2w+REC(55)*c2w  &
              +w*(REC(56)*sw+REC(57)*cw) +REC(58)*sw1+REC(59)*cw1
         !     COMPUTES THE ORBIT YAW BY USING AN EXPANSION FOR ARCSINE.
         psi=psi+dyaw*(1.0D0+dyaw*dyaw/6.0D0)
         !     CALCULATION OF CHANGES IN THE SATELLITE ORBIT ENDS HERE
      END IF
      !     CONVERSION OF THE IMC LONGITUDE AND ORBIT YAW TO THE SUBSATELLITE
      !     LONGITUDE AND THE ORBIT INCLINATION (REF: GOES-PCC-TM-2473, INPUTS
      !     REQUIRED FOR EARTH LOCATION AND GRIDDING BY SPS,  JUNE 6, 1988)
      slat=DSIN(phi)
      syaw=DSIN(psi)
      si=slat**2+syaw**2
      ci=DSQRT(1.0D0-si)
      si=DSQRT(si)
      IF (slat == 0.0D0.AND.syaw == 0.0D0) THEN
         u=0.0D0
      ELSE
         u=DATAN2(slat,syaw)
      END IF
      su=DSIN(u)
      cu=DCOS(u)
      !     COMPUTES LONGITUDE OF THE ASCENDING NODE
      asc=lam - u
      sa=DSIN(asc)
      ca=DCOS(asc)
      !     COMPUTES THE SUBSATELLITE GEOGRAPHIC LATITUDE
      rlat=DATAN(aebe2*DTAN(phi))
      !     COMPUTES THE SUBSATELLITE LONGITUDE
      rlon=asc+DATAN2(ci*su,cu)
      !     COMPUTE THE SUBSATELLITE DISTANCE TO EARTH'S CENTER
      rrad = nomorb+dr
      !     COMPUTES THE SPACECRAFT TO EARTH FIXED COORDINATES TRANSFORMATION
      !     MATRIX:
      !         (VECTOR IN ECEF COORDINATES) = B * (VECTOR IN S/C COORDINATES)
      
      b(1,2)=-sa*si
      b(2,2)= ca*si
      b(3,2)=-ci
      b(1,3)=-ca*cu+sa*su*ci
      b(2,3)=-sa*cu-ca*su*ci
      b(3,3)=-slat
      b(1,1)=-ca*su-sa*cu*ci
      b(2,1)=-sa*su+ca*cu*ci
      b(3,1)= cu*si
      !     COMPUTES THE NORMALIZED SPACECRAFT POSITION VECTOR IN EARTH FIXED
      !     COORDINATES - XS.
      r=(nomorb+dr)/ae
      xs(1)=-b(1,3)*r
      xs(2)=-b(2,3)*r
      xs(3)=-b(3,3)*r
      !     PRECOMPUTES Q3 (USED IN LPOINT)
      q3=xs(1)**2+xs(2)**2+aebe2*xs(3)**2-1.0D0
      !     COMPUTES THE ATTITUDES AND MISALIGNMENTS IF IMC IS OFF
      IF (imc /= 0) THEN
         !     COMPUTES THE SOLAR ORBIT ANGLE
         wa=REC(60)*ts
         !     COMPUTES THE DIFFERENCE BETWEEN CURRENT TIME, TS, AND THE
         !     EXPONENTIAL TIME, REC(61). NOTE THAT BOTH TIMES ARE SINCE EPOCH.
         te=ts-REC(61)
         !     COMPUTES ROLL + ROLL MISALIGNMENT
         roll=roll+imc_gatt(62,REC,wa,te)
         !     COMPUTES PITCH + PITCH MISALIGNMENT
         pitch=pitch+imc_gatt(117,REC,wa,te)
         !     COMPUTES YAW
         yaw=yaw+imc_gatt(172,REC,wa,te)
         !     COMPUTES ROLL MISALIGNMENT
         rma=imc_gatt(227,REC,wa,te)
         !     COMPUTES PITCH MISALIGNMENT
         pma=imc_gatt(282,REC,wa,te)
         !     APPLY THE SPCECRAFT COMPENSATION
         roll=roll+REC(15)
         pitch=pitch+REC(16)
         yaw=yaw+REC(17)
      END IF
      !     COMPUTES THE INSTRUMENT TO EARTH FIXED COORDINATES TRANSFORMATION
      !     MATRIX - BT
      
      CALL imc_inst2er(roll,pitch,yaw,b,bt)
      RETURN
    END SUBROUTINE imc_lmodel
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : GPOINT
!**   SOURCE    : F.GPOINT
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     12/10/87  IL   INITIAL CREATION
!**   A     06/10/88  IL   REPLACED ASIN WITH ATAN TO SAVE TIME
!**   A     06/02/89  IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL 504-01
!**   4     03/08/94  SC   IMPLEMENTED NEW FORMULAE FOR SCAN ANGLE
!**                        CORRECTION DUE TO MISALIGNMENTS
!***********************************************************************
!**   THIS SUBROUTINE CONVERTS GEOGRAPHIC LATITUDE AND LONGITUDE
!**   TO THE RELATED ELEVATION AND SCAN ANGLES.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_gpoint(rlat,rlon,alf,gam,ierr,vza)
      !External variables
      REAL(kind=8), INTENT(IN OUT),dimension(:) :: rlat ! GEOGRAPHIC LATITUDE IN RADIANS (INPUT)
      REAL(kind=8), INTENT(IN OUT),dimension(:) :: rlon ! GEOGRAPHIC LONGITUDE IN RADIANS (INPUT)
      REAL(kind=8), INTENT(OUT),dimension(size(rlat)) :: alf  ! ELEVATION ANGLE IN RADIANS (OUTPUT)
      REAL(kind=8), INTENT(OUT),dimension(size(rlat)) :: gam  ! SCAN ANGLE IN RADIANS (OUTPUT)
      INTEGER, INTENT(OUT),dimension(size(rlat))      :: ierr ! OUTPUT STATUS; 
      !                                  0 - SUCCESSFUL COMPLETION,
      !                                  1 - POINT WITH GIVEN LAT/LON IS INVISIBLE
      real(kind=8),intent(out),optional,dimension(size(rlat)) :: vza !Satellite View Zenith Angle (rad)
      !Internal variables
      REAL(kind=8),dimension(3,size(rlat)):: f  ! POINTING VECTOR IN EARTH CENTERED COORDINATES
      REAL(kind=8),dimension(3,size(rlat)):: ft ! POINTING VECTOR IN INSTRUMENT COORDINATES
      REAL(kind=8),dimension(3,size(rlat)):: u  ! COORDINATES OF THE EARTH POINT (KM)
      REAL(kind=8),dimension(size(rlat))  :: sing,slat,w1,w2,w3 ! WORK SPACE
      !INCLUDE 'elcons.f90'
      !INCLUDE 'elcomm.f90'
!***********************************************************************
      !     COMPUTES SINUS OF GEOGRAPHIC (GEODETIC) LATITUDE
      sing=DSIN(rlat)
      w1=aebe4*sing*sing
      !     SINUS OF THE GEOCENTRIC LATITUDE
      slat=((0.375D0*w1-0.5D0)*w1+1.0D0)*sing/aebe2
      !     COMPUTES LOCAL EARTH RADIUS AT SPECIFIED POINT
      w2=slat*slat
      w1=aebe3*w2
      w1=(0.375D0*w1-0.5D0)*w1+1.d0
      !     COMPUTES CARTESIAN COORDINATES OF THE POINT
      u(3,:)=slat*w1
      w2=w1*DSQRT(1.0D0-w2)
      u(1,:)=w2*DCOS(rlon)
      u(2,:)=w2*DSIN(rlon)
      !     POINTING VECTOR FROM SATELLITE TO THE EARTH POINT
      f(1,:)=u(1,:)-xs(1)
      f(2,:)=u(2,:)-xs(2)
      f(3,:)=u(3,:)-xs(3)
      w2 =  u(1,:)*SNGL(f(1,:))+&
           &u(2,:)*SNGL(f(2,:))+&
           &u(3,:)*SNGL(f(3,:))*aebe2
      w3=w2
      !     VERIFIES VISIBILITY OF THE POINT
!!$      IF (w2 > 0.0D0) THEN ! INVISIBLE POINT ON THE EARTH
!!$         ierr=1
!!$         alf=99999.0D0
!!$         gam=99999.0D0
!!$         RETURN
!!$      END IF
      !     CONVERTS POINTING VECTOR TO INSTRUMENT COORDINATES
      ft(1,:)=bt(1,1)*f(1,:)+bt(2,1)*f(2,:)+bt(3,1)*f(3,:)
      ft(2,:)=bt(1,2)*f(1,:)+bt(2,2)*f(2,:)+bt(3,2)*f(3,:)
      ft(3,:)=bt(1,3)*f(1,:)+bt(2,3)*f(2,:)+bt(3,3)*f(3,:)
      !     CONVERTS POINTING VECTOR TO SCAN AND ELEVATION ANGLES AND
      !     CORRECTS FOR THE ROLL AND PITCH MISALIGNMENTS
      gam=DATAN(ft(1,:)/SQRT(ft(2,:)**2+ft(3,:)**2))
      alf=-DATAN(ft(2,:)/ft(3,:))
      w1=DSIN(alf)
      w2=DCOS(gam)
      alf=alf+rma*(1.0D0-DCOS(alf)/w2)+pma*w1*(1.0D0/w2+DTAN(gam))
      gam=gam-rma*w1
      ierr=0

      !Inserted by Knapp
      !Calculate the view zenith angle
      !It is the angle between U and -G1
      if (present(vza)) VZA = acos( dot( unit(u), unit(-1.*f) ) ) * rad
      where (w3 > 0.d0)
         ierr = 1
         alf = 99999.d0
         gam = 99999.d0
      end where

      RETURN
    END SUBROUTINE imc_gpoint
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : INST2ER
!**   SOURCE    : F.INST2ER
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   1     08/16/88  IL   INITIAL CREATION
!**   2     11/11/88  IL   TRIGONOMETRIC FUNCTIONS REPLACED WITH
!**                        SMALL ANGLE APPROXIMATIONS
!**   3    06/02/89   IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL 504-01
!***********************************************************************
!**   INST2ER ACCEPTS THE SINGLE PRECISION ROLL, PITCH AND YAW ANGLES
!**   OF AN INSTRUMENT AND RETURNS THE DOUBLE PRECISION INSTRUMENT TO
!**   EARTH COORDINATES TRANSFORMATION MATRIX.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_inst2er(r,p,y,a,at)
      !External variables
      REAL(kind=8), INTENT(IN) :: r ! ROLL ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN) :: p ! PITCH ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN) :: y ! YAW ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN) :: a(3,3) ! SPACECRAFT TO ECEF COORDINATES
      REAL(kind=8), INTENT(OUT):: at(3,3)! INSTRUMENT TO ECEF COORDINATES
      !Internal variables
      REAL(kind=8) rpy(3,3) ! INSTRUMENT TO BODY COORDINATES
                            ! TRANSFORMATION MATRIX
      INTEGER(kind=4) i,j   ! INDICES
!***********************************************************************
!     WE COMPUTE INSTRUMENT TO BODY COORDINATES TRANSFORMATION
!     MATRIX BY USING A SMALL ANGLE APPROXIMATION OF TRIGONOMETRIC
!     FUNCTIONS OF THE ROLL, PITCH AND YAW.
      rpy(1,1)=1.0D0-0.50D0*(p*p+y*y)
      rpy(1,2)=-y
      rpy(1,3)=p
      rpy(2,1)=y+p*r
      rpy(2,2)=1.0D0-0.50D0*(y*y+r*r)
      rpy(2,3)=-r
      rpy(3,1)=-p+r*y
      rpy(3,2)=r+p*y
      rpy(3,3)=1.0D0-0.50D0*(p*p+r*r)
      !     MULTIPLICATION OF MATRICES A AND RPY
      DO  i=1,3
         DO  j=1,3
            at(i,j)=a(i,1)*rpy(1,j)+a(i,2)*rpy(2,j)+a(i,3)*rpy(3,j)
         END DO
      END DO
      RETURN
    END SUBROUTINE imc_inst2er
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : LPOINT
!**   SOURCE    : F.LPOINT
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     01/09/89  IL   INITIAL CREATION
!**   A     06/02/89  IL   COORDINATE AXES CHANGED ACCORDING TO
!**                        FORD'S DEFINITION IN SDAIP, DRL504-01
!**   3     03/08/94  SC   IMPLEMENTED NEW FORMULAE FOR SCAN ANGLE
!**                        CORRECTIONS DUE TO MISALIGNMENTS
!***********************************************************************
!**   THIS SUBROUTINE CONVERTS THE INSTRUMENT ELEVATION AND SCAN
!**   ANGLES TO THE RELATED GEOGRAPHIC LATITUDE AND LONGITUDE.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_lpoint(alpha,zeta,rlat,rlon,ierr,vza)
      !External variables
      REAL(kind=8), INTENT(IN), dimension(:) :: alpha ! ELEVATION ANGLE (RAD)
      REAL(kind=8), INTENT(IN), dimension(:) :: zeta  ! SCAN ANGLE (RAD)
      REAL(kind=8), INTENT(OUT),dimension(size(zeta)):: rlat  ! LATITUDE IN RADIANS (OUTPUT)
      REAL(kind=8), INTENT(OUT),dimension(size(zeta)):: rlon  ! LONGITUDE IN RADIANS (OUTPUT)
      INTEGER,      INTENT(OUT),dimension(size(zeta)):: ierr  ! OUTPUT STATUS; 0 - POINT ON THE EARTH
      !                                   FOUND, 1 - INSTRUMENT POINTS OFF EARTH
      real(kind=8),intent(out),dimension(size(zeta)),optional :: vza   ! Satellite View Zenith Angle (rad)
      !Internal variables
      REAL(kind=8), dimension(3,size(zeta)):: g1   ! POINTING VECTOR IN EARTH CENTERED COORDINATES
      REAL(kind=8), dimension(size(zeta)):: h       ! SLANT DISTANCE TO THE EARTH POINT (KM)
      REAL(kind=8), dimension(size(zeta)):: q1,q2,d ! WORK SPACE
      REAL(kind=8), dimension(3,size(zeta)):: g    ! POINTING VECTOR IN INSTRUMENT COORDINATES
      REAL(kind=8), dimension(3,size(zeta)):: u    ! COORDINATES OF THE EARTH POINT (KM)
      REAL(kind=8),dimension(size(zeta))::  sa,ca,da,dz,d1,cz,d3 ! WORK SPACE
      !INCLUDE 'elcons.f90'
      !INCLUDE 'elcomm.f90'
!***********************************************************************
      ierr=1
      !     COMPUTES TRIGONOMETRIC FUNCTIONS OF THE SCAN AND ELEVATION
      !     ANGLES CORRECTED FOR THE ROLL AND PITCH MISALIGNMENTS
      ca=DCOS(alpha)
      sa=DSIN(alpha)
      cz=DCOS(zeta)
      da=alpha-pma*sa*(1.0D0/cz+DTAN(zeta))-rma*(1.0D0-ca/cz)
      dz=zeta+rma*sa
      !                              CORRECTED SCAN ANGLE
      cz=DCOS(dz)
      !     COMPUTES POINTING VECTOR IN INSTRUMENT COORDINATES
      g(1,:)=    DSIN(dz)
      g(2,:)=-cz*DSIN(da)
      g(3,:)= cz*DCOS(da)
      !     TRANSFORMS THE POINTING VECTOR TO EARTH FIXED COORDINATES
      g1(1,:)=bt(1,1)*g(1,:)+bt(1,2)*g(2,:)+bt(1,3)*g(3,:)
      g1(2,:)=bt(2,1)*g(1,:)+bt(2,2)*g(2,:)+bt(2,3)*g(3,:)
      g1(3,:)=bt(3,1)*g(1,:)+bt(3,2)*g(2,:)+bt(3,3)*g(3,:)
      !     COMPUTES COEFFICIENTS AND SOLVES A QUADRATIC EQUATION TO
      !     FIND THE INTERSECT OF THE POINTING VECTOR WITH THE EARTH
      !     SURFACE
      q1=g1(1,:)**2+g1(2,:)**2+aebe2*g1(3,:)**2
      q2=xs(1)*g1(1,:)+xs(2)*g1(2,:)+aebe2*xs(3)*g1(3,:)
      d=q2*q2-q1*q3
      where (dabs(d) < 1.d-9)
         d = 0.0d0
      end where
!      IF (DABS(d) < 1.d-9) d=0.0D0
      !     IF THE DISCIMINANTE OF THE EQUATION, D, IS NEGATIVE, THE
      !     INSTRUMENT POINTS OFF THE EARTH
!!$      IF (d < 0.0D0) THEN
!!$         rlat=999999.0D0
!!$         rlon=999999.0D0
!!$         vza = 999999.0D0
!!$         RETURN
!!$      END IF
      d3=0.
      where (d > 0.d0)
         d3=DSQRT(d)
      end where
      !     SLANT DISTANCE FROM THE SATELLITE TO THE EARTH POINT
      h=-(q2+d3)/q1
      !     CARTESIAN COORDINATES OF THE EARTH POINT
      u(1,:)=xs(1)+h*g1(1,:)
      u(2,:)=xs(2)+h*g1(2,:)
      u(3,:)=xs(3)+h*g1(3,:)
      !     SINUS OF GEOCENTRIC LATITUDE
      d1=u(3,:)/DSQRT(u(1,:)**2+u(2,:)**2+u(3,:)**2)
      !     GEOGRAPHIC (GEODETIC) COORDINATES OF THE POINT
      rlat=DATAN(aebe2*d1/DSQRT(1.0D0-d1*d1))
      rlon=DATAN2(u(2,:),u(1,:))

      !Inserted by Knapp
      !Calculate the view zenith angle
      !It is the angle between U and -G1
      if (present(vza)) VZA = acos( dot( unit(u), unit(-1.*g1) ) ) * rad
      ierr=0
      where (d < 0.0d0)
         rlat = 999999.d0*pi/180.
         rlon = 999999.d0*pi/180.
         vza  = 999999.d0*pi/180.
         ierr = 1
      end where

      RETURN
    END SUBROUTINE imc_lpoint
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : SNDELOC
!**   SOURCE    : F.SNDELOC
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     02/16/89  IL   INITIAL CREATION
!***********************************************************************
!**   SNDELOC ACCEPTS THE MIRROR POSITION IN CYCLES AND INCREMENTS,
!**   SERVO ERROR VALUES, AND THE POSITIONAL OFFSETS FOR FOUR DETECTORS
!**   OF A SELECTED SOUNDER CHANNEL AND COMPUTES THE DETECTOR EARTH
!**   LOCATIONS IN LATITUDE/LONGITUDE COORDINATES.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : LPOINT
!***********************************************************************
!!$    SUBROUTINE imc_sndelo(cyew,incew,cyns,incns,svew,svns,doff,geo)
!!$      !External variables
!!$      INTEGER, INTENT(IN OUT)     :: cyew  ! E-W CYCLES
!!$      INTEGER, INTENT(IN OUT)     :: incew ! E-W INCREMENTS
!!$      INTEGER, INTENT(IN OUT)     :: cyns  ! N-S CYCLES
!!$      INTEGER, INTENT(IN OUT)     :: incns ! N-S INCREMENTS
!!$      REAL(kind=8), INTENT(IN)    :: svew  ! E-W SERVO ERROR IN RADIANS
!!$      REAL(kind=8), INTENT(IN)    :: svns  ! N-S SERVO ERROR IN RADIANS
!!$      REAL(kind=8), INTENT(IN)    :: doff(4,2) !OFFSETS FOR 4 DETECTORS (RADIANS)
!!$      !                                         DOFF(*,1) = E-W OFFSET
!!$      !                                         DOFF(*,2) = N-S OFFSET
!!$      REAL(kind=8), INTENT(IN OUT):: geo(4,2) ! GEOGRAPHIC COORDINATES RELATED 
!!$      !                                               TO 4 DETECTORS
!!$      !                                         GEO(*,1) = LATITUDE IN RADIANS
!!$      !                                         GEO(*,2) = LONGITUDE IN RADIANS
!!$      !Internal variables
!!$      REAL(kind=8) e,s,h,ev,sc,alpha,beta,sine,cose,de,ds,vza
!!$      INTEGER :: i,ier
!!$      !INCLUDE 'instco.f90'
!!$!***********************************************************************
!!$      !     CONVERT THE MIRROR POSITION, GIVEN IN CYCLES AND INCREMENTS, TO
!!$      !     ELEVATION AND SCAN ANGLES
!!$      !     E=(CYNS*INCMAX(2)+INCNS)*ELVINC(2)-ELVMAX(2)
!!$      e=((cyns-9)*incmax(2)+incns)*elvinc(2)-elvmax(2)
!!$      s=(cyew*incmax(2)+incew)*scninc(2)-scnmax(2)
!!$      !     CORRECT ELEVATION AND SCAN ANGLES FOR SERVO ERRORS OBTAINING THE
!!$      !     TRUE MIRROR POINTING
!!$      e=e+svns
!!$      s=s+svew
!!$      sine=DSIN(e)
!!$      cose=DCOS(e)
!!$      h=-2.0D0*scnpx(2)
!!$!     COMPUTE DETECTOR ROTATION OFFSETS FOR EACH DETECTOR
!!$!      ALPHA = 0.643501D0 + E
!!$!      BETA = 0.244979D0 - E
!!$!      DOFF(1,1) = -0.064976D0
!!$!      DOFF(1,2) = 0.00042D0
!!$!      DOFF(2,1) = 0.00056D0
!!$!      DOFF(2,2) = 0.00014D0
!!$!      DOFF(3,1) = -0.064976D0
!!$!      DOFF(3,2) = -0.065396D0
!!$!      DOFF(4,1) = 0.00056D0
!!$!      DOFF(4,2) = -0.065116D0
!!$!      DOFF(1,1) = - 700.0D0*DCOS(ALPHA)*1.0D-6
!!$!      DOFF(1,2) =   700.0D0*DSIN(ALPHA)*1.0D-6
!!$!      DOFF(2,1) =   577.23479D0*DCOS(BETA)*1.D-6
!!$!      DOFF(2,2) =   577.23479D0*DSIN(BETA)*1.0D-6
!!$!      DOFF(3,1) = - 577.23479D0*DCOS(BETA)*1.0D-6
!!$!      DOFF(3,2) = - 577.23479D0*DSIN(BETA)*1.0D-6
!!$!      DOFF(4,1) =   700.0D0*DCOS(ALPHA)*1.0D-6
!!$!      DOFF(4,2) = - 700.0D0*DSIN(ALPHA)*1.0D-6
!!$!     COMPUTE EARTH LOCATIONS FOR FOUR DETECTORS
!!$      DO  i=1,4
!!$         !     COMPUTE POSITIONAL OFFSETS OF I-TH DETECTOR
!!$         de=(2.5-i)*elvln(2)+doff(i,2)
!!$         ds=h+doff(i,1)
!!$         !     COMPUTE ELEVATION AND SCAN ANGLES RELATED TO I-TH DETECTOR
!!$         !     AND CORRECT THEM FOR THE DETECTOR POSITIONAL OFFSETS
!!$         !           EV=E+DOFF(I,2)
!!$         !           SC=S+DOFF(I,1)
!!$         !     CONVERT POSITIONAL OFFSETS TO ANGULAR OFFSETS AND
!!$         !     CORRECT ELEVATION AND SCAN ANGLES
!!$         ev = e + de*cose - ds*sine
!!$         sc = s + de*sine + ds*cose
!!$         !     TRANSFORM DETECTOR'S POINTING ANGLES TO GEOGRAPHIC COORDINATES
!!$         !     OF THE CORRESPONDING POINT ON THE EARTH SURFACE.
!!$         !     NOTE:  IF A DETECTOR LOOKS OFF THE EARTH, THE RELATED LATITUDE
!!$         !            AND LONGITUDE ARE SET TO 999999.
!!$         CALL imc_lpoint(ev,sc,geo(i,1),geo(i,2),ier,vza)
!!$         h=-h
!!$      END DO
!!$      RETURN
!!$    END SUBROUTINE imc_sndelo
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : EVSC2LPF
!**   SOURCE    : F.EVSC2LPF
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     10/27/88  IL   INITIAL CREATION
!***********************************************************************
!**   THIS SUBROUTINE CONVERTS ELEVATION AND SCAN ANGLES
!**   TO THE FRACTIONAL LINE AND PIXEL NUMBERS.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    SUBROUTINE imc_evsc2l(instr,elev,scan,rl,rp)
      !External variables
      INTEGER,      INTENT(IN OUT):: instr ! INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)
      REAL(kind=8), INTENT(IN OUT),dimension(:):: elev  ! ELEVATION ANGLE IN RADIANS
      REAL(kind=8), INTENT(IN OUT),dimension(:):: scan  ! SCAN ANGLE IN RADIANS
      REAL(kind=8), INTENT(OUT),dimension(size(elev))   :: rl    ! LINE NUMBER
      REAL(kind=8), INTENT(OUT),dimension(size(elev))   :: rp    ! PIXEL NUMBER
!**************************************************************
      !     COMPUTE FRACTIONAL LINE NUMBER
      rl=(elvmax(instr)-elev)/elvln(instr)
      IF (instr == 1) THEN
         rl=rl+4.5D0
      ELSE
         rl=rl+2.5D0
      END IF
      !     COMPUTE FRACTIONAL PIXEL NUMBER
      rp=(scnmax(instr)+scan)/scnpx(instr)+1.0D0
      RETURN
    END SUBROUTINE imc_evsc2l
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : EVLN
!**   SOURCE    : F.EVLN
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     10/27/88  IL   INITIAL CREATION
!***********************************************************************
!**   THIS FUNCTION CONVERTS FRACTIONAL LINE NUMBER TO ELEVATION ANGLE
!**   IN RADIANS.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_evln(instr,rline) result (output)
      !External variables
      INTEGER,      INTENT(IN OUT):: instr  ! INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)
      REAL(kind=8), INTENT(IN OUT),dimension(:):: rline
!      REAL(kind=8)  imc_evln                    ! FRACTIONAL LINE  NUMBER
      real(kind=8), dimension(size(rline)) :: output
!***********************************************************************
      IF (instr == 1) THEN
         output=elvmax(instr)*1.0D0 - (rline-4.5 )*(elvln(instr)*1.0D0)
      ELSE
         output=elvmax(instr)*1.0D0 - (rline -2.5D0)*(elvln(instr)*1.0D0)
      END IF
      RETURN
    END FUNCTION imc_evln
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : SCPX
!**   SOURCE    : F.SCPX
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     09/22/87  IL   INITIAL CREATION
!***********************************************************************
!**   THIS FUNCTION CONVERTS FRACTIONAL PIXEL NUMBER TO SCAN ANGLE
!**   IN RADIANS.
!***********************************************************************
!**   CALLED BY       : ANY
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_scpx(instr,pix) result (output)
      !External variables
      INTEGER,      INTENT(IN OUT):: instr ! INSTRUMENT CODE (1-IMAGER, 2-SOUNDER)
      REAL(kind=8), INTENT(IN OUT), dimension(:) :: pix   ! FRACTIONAL PIXEL NUMBER
!      REAL(kind=8) imc_scpx
      real(kind=8), dimension(size(pix)) :: output
!***********************************************************************
      output=((pix*1.0D0)-1.0D0)*(scnpx(instr)*1.0D0) -(scnmax(instr)*1.0D0)
      RETURN
    END FUNCTION imc_scpx
!***********************************************************************
!**   INTEGRAL SYSTEMS, INC.
!***********************************************************************
!**   PROJECT   : OPERATIONS GROUND EQUIPMENT FOR GOES-NEXT
!**   SYSTEM    : EARTH LOCATION USERS GUIDE
!**   ROUTINE   : GATT
!**   SOURCE    : F.GATT
!**   LOAD NAME : ANY
!**   PROGRAMMER: IGOR LEVINE
!**   VER.    DATA    BY   COMMENT
!**   ----  --------  ---  ---------------------------------------------
!**   A     12/01/88  IL   INITIAL CREATION
!***********************************************************************
!**    THIS FUNCTION COMPUTES AN ATTITUDE/MISALIGNMENT ANGLE FROM A
!**    GIVEN SUBSET OF THE O&A PARAMETERS IN GVAR BLOK 0.
!**    ARGUMENT K0 INDICATES THE FIRST WORD OF THE SUBSET.
!***********************************************************************
!**   CALLED BY       : LMODEL
!**   COMMONS MODIFIED: NONE
!**   INPUTS          : NONE
!**   OUTPUTS         : NONE
!**   ROUTINES CALLED : NONE
!***********************************************************************
    FUNCTION imc_gatt(k0,REC,wa,te)
      !External variables
      INTEGER,     INTENT(IN):: k0       ! STARTING POSITION OF A PARAMETER SUBSET IN THE
      REAL(kind=8),INTENT(IN):: REC(336) ! INPUT O&A PARAMETER SET
      REAL(kind=8),INTENT(IN):: wa       ! INPUT SOLAR ORBIT ANGLE IN RADIANS
      REAL(kind=8),INTENT(IN):: te       ! INPUT EXPONENTIAL TIME DELAY FROM EPOCH (MINUTES)
      REAL(kind=8) :: imc_gatt
      ! Internal variables 
      INTEGER(kind=4) i, j, m,l,ll,k
      REAL   (kind=8) ir,jr,mr,att
      EQUIVALENCE (j,jr),(m,mr)
      !***********************************************************************
      !     CONSTANT COMPONENT
      k=k0
      att=REC(k+2)
      !     COMPUTES THE EXPONENTIAL TERM
      IF ((te >= 0.0D0).AND.(REC(k+1) > 0))THEN
         att=att+REC(k)*DEXP(-te/REC(k+1))
      END IF
      !     EXTRACTS THE NUMBER OF SINUSOIDS
      ir=REC(k+3)
      !      I = INT4(IR)
      i = INT(ir)
      !     CALCULATION OF SINUSOIDS
      DO  l=1,i
         att=att+REC(k+2*l+2)*DCOS(wa*l+REC(k+2*l+3))
      END DO
      !     POINTER TO THE NUMBER OF MONOMIAL SINUSOIDS
      k=k+34
      !     EXTACTS NUMBER OF MONOMIAL SINUSOIDS
      ir=REC(k)
      !      I=INT4(IR)
      i=INT(ir)
      !     KKK=REC(K)
      !     COMPUTES MONOMIAL SINUSOIDS
      DO  l=1,i
         ll=k+5*l
         !          ORDER OF SINUSOID
         jr=REC(ll-4)
         !          ORDER OF MONOMIAL SINUSOID
         mr=REC(ll-3)
         att=att+REC(ll-2)*((wa-REC(ll))**m)*DCOS(j*wa+REC(ll-1))
      END DO
      imc_gatt=att
      RETURN
    END FUNCTION imc_gatt
  END SUBROUTINE imcnav_r1
!                                                             END IMCNAV_R1
!#######################################################################



!#######################################################################
!                                                      BEGIN MCIDAS_READ
!****r* kmod_mcidas/mcidas_read
! NAME
! mcidas_read
! SYNOPSIS
! call mcidas_read (filename, AREA, GVAR, VISR, image, LINhdr, cmb, info)
! FUNCTION 
! Reads McIDAS Area format files.
! INPUTS
! filename - char(len=*) - filename of the Area file
! OUTPUT
! AREA - gvar_area - area header
! GVAR - gvar_nav  - GVAR navigation header (optional)
! VISR - visr_nav  - VISR navigation header (optional)
! image- int(nx,ny,nch)- multi-bandedsatellite image (optional)
! cmd  - char(len=120) (:) - MCIDAS command line (optional)
! info - int(4)    - information?? (optional)
! NOTES
! Uses a LSTAT: a lahey/fortran 95 specific routine which may or may not be
! available with other compilers. This is used to retrieve the file size.
!***
  subroutine mcidas_read ( file, AREA, GVAR, VISR, image, LINhdr, cmd, info)
    use kmod_utilities
    !External variables
    character(len=*), intent(in) :: file
    type(gvar_area), intent(inout) :: AREA
    type(gvar_nav),  optional, intent(inout) :: GVAR
    type(visr_nav),  optional, intent(inout) :: VISR
    integer(kind=2), optional, dimension(:,:,:), allocatable :: image
    integer(kind=2), optional, dimension(:,:), allocatable :: LINhdr
    character(len=120), optional, dimension(:), allocatable :: cmd
    integer, optional :: info
    !Internal variables
    integer(kind=1) :: iswap
    integer(kind=4) :: area_array(64)
    type(gvar_area) :: internal_area
    !equivalence (area_array, internal_area)
    integer(kind=4) :: gvar_array(640)
    !type(gvar_nav)  :: internal_gvar
    !equivalence (gvar_array, internal_gvar)
    integer(kind=4) :: visr_array(128)
    !type(visr_nav)  :: internal_visr
    !equivalence (visr_array, internal_visr)
    integer(kind=1) :: one_byte
    character(len=4) :: source
    integer(kind=4) :: prefix, nch, nx, ny, bytes, ipzero, fsize1, &
                       fsize2
    integer(kind=4) :: i, j, ii
    integer(kind=4) :: finfo(13)
    integer(kind=1) :: scan1b(40000)

    character(len=16) :: io_access, io_form

!    external functions specific to Lahey/Fujitsu Fortran 95 
!    (but also works with g95)

!### UNCOMMENT THE FOLLOWING LINE REGION IF YOU PLAN ON USING THE CMD option
    integer(kind=4) :: lstat

    iswap = 0
    
    !the following allows the code to compile using g95 and lf95
    if (is_g95()) then
       !g95 set up for BINARY I/O
       io_access = 'stream'
       io_form   = 'unformatted'
    else
       !Lahey/Fortran 95 options for BINARY I/O
       io_access = 'sequential'
       io_form   = 'binary'
    end if
    !Generic open interface
    open (unit=1, file=trim(file), status = 'old',&
        & access=trim(io_access), form=trim(io_form))

    ! Read the AREA information
!!!!!! For lf95
!!$    open ( unit=1, file=trim(file), form='binary', &
!!$           access='sequential', status='old' )
!!!!!! 
    
!!!!!! For g95
!!$    open ( unit=1, file=trim(file), &
!!$           access='STREAM', status='old' )
!!!!!!
    
    read (1) area_array

    !this is performed now because characters are not byte swapped
    internal_area = transfer(area_array,internal_area)
    do i=1,4
       source(i:i) = char(internal_area%src_type(i))
    end do

    !Should we byte-swap the file?
    if (area_array(2) /= 4) then 
       iswap = 1
       area_array = swap_endian(area_array)
    end if
    ! This old method required equivalence statements
    ! AREA = internal_area
    ! The new does not ...
    AREA = transfer(area_array,AREA)

    ! Read the GVAR information
    call point_lun(1,area%secondary_key_nav)
    if (source == 'GVAR' .and. present(gvar)) then
       read (1) gvar_array
       if (iswap == 1) gvar_array = swap_endian(gvar_array)
!       GVAR = internal_gvar
       GVAR = transfer(gvar_array,GVAR)
    else if (present(visr)) then
       read (1) visr_array
       if (iswap == 1) visr_array = swap_endian(visr_array)
       !VISR = internal_visr
       VISR = transfer(visr_array,VISR)
    end if

    !Set up the image and line header for input
    nch = AREA%num_chan
    nx =  AREA%num_elem
    ny =  AREA%num_line
    prefix = AREA%num_byte_line_pre
    ipzero = AREA%primary_key_nav
    bytes  = AREA%bytes_per_pixel
    fsize1 = ipzero + (nx*nch*bytes+prefix)*ny

    ! Need to get file size from LSTAT
    ! NOTE: USES LAHEY FORTRAN DEPENDENT ROUTINE LSTAT
    i = lstat(trim(file),finfo)
    fsize2 = finfo(8)

    !Adjust NY if needed... this tends to happen for merged files
    if (fsize2 < fsize1) then
       if (present(info)) then
          print*,'Adjusting NY since actual file is smaller than'
          print*,'what the header suggests'
          print*,'Old NY=',ny
       end if
       NY = ((fsize2-ipzero)/(nx*nch*bytes+prefix))
       if (present(info)) print*,'New NY=',ny
       AREA%num_line = ny
    endif

    if (mod(prefix,2) == 1) print*,'Odd prefix length?',prefix

    if (present(image) .and. .not. present(linhdr)) then
       print*,'Error: Keyword LINHDR missing when IMAGE is specified'
       stop
    end if
    if (present(linhdr) .and. .not. present(image)) then
       print*,'Error: Keyword IMAGE missing when LNHDR is specified'
       stop
    end if
    if (present(CMD) .and. .not. present(image)) then
       print*,'Error: Keyword IMAGE missing when CMD is specified'
       stop
    end if
    
    if (present(image)) then
       !set up line prefix and image
       if (allocated(linhdr)) deallocate(linhdr)
       if (allocated(image)) deallocate (image)
       allocate (linhdr(prefix/2,ny), image(nx,ny,nch))
       
       !Read in the line header and image
       call point_lun(1,ipzero)
       do i=1,ny
          read (1) linhdr(:,i)
          do j=1,nch
             if (bytes == 1) then
                read (1) scan1b(1:nx)
                image(1:nx,i,j) = scan1b(1:nx)
                do ii=1,nx
                   if (image(ii,i,j) < 0) image(ii,i,j) = image(ii,i,j) + 256
                end do
             else
                read (1) image(:,i,j)
             end if
          enddo
       enddo
       if (iswap == 1 .and. bytes /= 1) image = swap_endian(image)
       if (bytes /= 1) image = ishft(image,-5)
       if (present(cmd)) then
          !If not at end of file, then get the rest and put into command
          if (fsize2 > fsize1) then
             if (allocated(cmd)) deallocate(cmd)
             allocate (cmd(ceiling(real(fsize2-fsize1)/80.)))
             do i=1,fsize2-fsize1
                read (1) one_byte
                cmd(ceiling(i/80.))(mod(i,80):mod(i,80)) = achar(one_byte)
             end do
          endif
       end if
    end if

    !Close McIDAS file
    close(1)

  end subroutine mcidas_read
!                                                        END MCIDAS_READ
!#######################################################################
!#######################################################################
!                                                          BEGIN NAV2REC
!****r* kmod_mcidas/nav2rec
! NAME
! nav2rec
! SYNOPSIS
! call nav2rec(nav,rec)
! FUNCTION
!    This routine converts the navigation structure to the
!    REC array.
! INPUTS
! * nav - type(gvar_nav) - Navigation info produced from b1uread
! OUTPUT
! rec - r*8(:)
! USED BY
! b1u_ij2ll, b1u_ll2ij (rec is used by imcnav_r1)
!***
  subroutine nav2rec ( nav, rec )
    implicit none
    !External variables
    type(gvar_nav), intent(in) :: nav
    real(kind=8), dimension(:), intent(out) :: rec
    !Internal Variables
    integer(kind=4) :: st

    rec(5)   = NAV%ref_longitude       * 1.d-7
    rec(6)   = NAV%ref_radius   * 1.d-7
    rec(7)   = NAV%ref_latitude        * 1.d-7
    rec(8)   = NAV%ref_orbit_yaw    * 1.d-7
    rec(9)   = NAV%ref_att_roll   * 1.d-7
    rec(10)  = NAV%ref_att_pitch  * 1.d-7
    rec(11)  = NAV%ref_att_yaw    * 1.d-7
    !rec(12) = NAV%   epoch
    !rec(13) = NAV%   epoch
    rec(14)  = NAV%start_time     * 1.d-2
    rec(15)  = NAV%imc_corr_roll  * 1.d-7
    rec(16)  = NAV%imc_corr_pitch * 1.d-7
    rec(17)  = NAV%imc_corr_yaw   * 1.d-7
    rec(18:30)=NAV%ref_longitude_change* 1.d-7
    rec(31:41)=NAV%ref_radius_change * 1.d-7
    rec(42:50)=NAV%sine_latitude       * 1.d-7
    rec(51:59)=NAV%sine_orbital_yaw   * 1.d-7
    rec(60)  = NAV%solar_rate     * 1.d-7
    rec(61)  = NAV%exponential_start_time * 1.d-2

    st = 62
    rec(st)  = NAV%roll_att%exponential_magnitude * 1.d-7
    rec(st+1)= NAV%roll_att%exponential_time_constant* 1.d-2
    rec(st+2)= NAV%roll_att%mean_attitude_angle * 1.d-7
    rec(st+3)= NAV%roll_att%number_of_sinusoids

!    ind = st+ 4 + lindgen(15)*2
    rec(66:94:2) = NAV%roll_att%sinusoid(:)%magnitude* 1.d-7
    rec(67:95:2) = NAV%roll_att%sinusoid(:)%phase_angle* 1.d-7
    rec(st+34)=NAV%roll_att%number_of_monomials

!    ind = st + 35 + lindgen(4)*5
    rec(97:112:5)= NAV%roll_att%monomial(:)%order_applicable_sinusoid
    rec(98:113:5)= NAV%roll_att%monomial(:)%order_monomial_sinusoid
    rec(99:114:5)= NAV%roll_att%monomial(:)%magnitude_monomial_sinusoid* 1.d-7
    rec(100:115:5)=NAV%roll_att%monomial(:)%phase_angle_monomial_sinusoid* 1.d-7
    rec(101:116:5)=NAV%roll_att%monomial(:)%angle_from_epoch* 1.d-7

    st = 117
    rec(st)  = NAV%pitch_att%exponential_magnitude* 1.d-7
    rec(st+1)= NAV%pitch_att%exponential_time_constant*1.d-2
    rec(st+2)= NAV%pitch_att%mean_attitude_angle* 1.d-7
    rec(st+3)= NAV%pitch_att%number_of_sinusoids

!    ind = st+ 4 + lindgen(15)*2
    rec(121:149:2)=NAV%pitch_att%sinusoid(:)%magnitude* 1.d-7
    rec(122:150:2)=NAV%pitch_att%sinusoid(:)%phase_angle* 1.d-7
    rec(st+34)    =NAV%pitch_att%number_of_monomials

!    ind = st + 35 + lindgen(4)*5
    rec(152:167:5)=NAV%pitch_att%monomial(:)%order_applicable_sinusoid
    rec(153:168:5)=NAV%pitch_att%monomial(:)%order_monomial_sinusoid
    rec(154:169:5)=NAV%pitch_att%monomial(:)%magnitude_monomial_sinusoid* 1.d-7
    rec(155:170:5)=NAV%pitch_att%monomial(:)%phase_angle_monomial_sinusoid* 1.d-7
    rec(156:171:5)=NAV%pitch_att%monomial(:)%angle_from_epoch* 1.d-7

    st = 172
    rec(st)  = NAV%yaw_att%exponential_magnitude* 1.d-7
    rec(st+1)= NAV%yaw_att%exponential_time_constant*1.d-2
    rec(st+2)= NAV%yaw_att%mean_attitude_angle* 1.d-7
    rec(st+3)= NAV%yaw_att%number_of_sinusoids

!    ind = st+ 4 + lindgen(15)*2
    rec(176:204:2)=NAV%yaw_att%sinusoid(:)%magnitude* 1.d-7
    rec(177:205:2)=NAV%yaw_att%sinusoid(:)%phase_angle* 1.d-7
    rec(st+34)    =NAV%yaw_att%number_of_monomials

!    ind = st + 35 + lindgen(4)*5
    rec(207:222:5)=NAV%yaw_att%monomial(:)%order_applicable_sinusoid
    rec(208:223:5)=NAV%yaw_att%monomial(:)%order_monomial_sinusoid
    rec(209:224:5)=NAV%yaw_att%monomial(:)%magnitude_monomial_sinusoid* 1.d-7
    rec(210:225:5)=NAV%yaw_att%monomial(:)%phase_angle_monomial_sinusoid* 1.d-7
    rec(211:226:5)=NAV%yaw_att%monomial(:)%angle_from_epoch* 1.d-7

    st = 227 !172+55
    rec(st)  = NAV%roll_misalign%exponential_magnitude* 1.d-7
    rec(st+1)= NAV%roll_misalign%exponential_time_constant*1.d-2
    rec(st+2)= NAV%roll_misalign%mean_attitude_angle* 1.d-7
    rec(st+3)= NAV%roll_misalign%number_of_sinusoids

!    ind = st+ 4 + lindgen(15)*2
    rec(231:259:2)=NAV%roll_misalign%sinusoid(:)%magnitude* 1.d-7
    rec(232:260:2)=NAV%roll_misalign%sinusoid(:)%phase_angle* 1.d-7
    rec(st+34)    =NAV%roll_misalign%number_of_monomials

!    ind = st + 35 + lindgen(4)*5
    rec(262:277:5)=NAV%roll_misalign%monomial(:)%order_applicable_sinusoid
    rec(263:278:5)=NAV%roll_misalign%monomial(:)%order_monomial_sinusoid
    rec(264:279:5)=NAV%roll_misalign%monomial(:)%magnitude_monomial_sinusoid* 1.d-7
    rec(265:280:5)=NAV%roll_misalign%monomial(:)%phase_angle_monomial_sinusoid* 1.d-7
    rec(266:281:5)=NAV%roll_misalign%monomial(:)%angle_from_epoch* 1.d-7

    st = 282 !172+55+55
    rec(st)  = NAV%pitch_misalign%exponential_magnitude* 1.d-7
    rec(st+1)= NAV%pitch_misalign%exponential_time_constant*1.d-2
    rec(st+2)= NAV%pitch_misalign%mean_attitude_angle* 1.d-7
    rec(st+3)= NAV%pitch_misalign%number_of_sinusoids

!    ind = st+ 4 + lindgen(15)*2
    rec(286:314:2)=NAV%pitch_misalign%sinusoid(:)%magnitude* 1.d-7
    rec(287:315:2)=NAV%pitch_misalign%sinusoid(:)%phase_angle* 1.d-7
    rec(st+34)    =NAV%pitch_misalign%number_of_monomials

!    ind = st + 35 + lindgen(4)*5
    rec(317:332:5)=NAV%pitch_misalign%monomial(:)%order_applicable_sinusoid
    rec(318:333:5)=NAV%pitch_misalign%monomial(:)%order_monomial_sinusoid
    rec(319:334:5)=NAV%pitch_misalign%monomial(:)%magnitude_monomial_sinusoid* 1.d-7
    rec(320:335:5)=NAV%pitch_misalign%monomial(:)%phase_angle_monomial_sinusoid* 1.d-7
    rec(321:336:5)=NAV%pitch_misalign%monomial(:)%angle_from_epoch* 1.d-7
  end subroutine nav2rec
!                                                            END NAV2REC
!#######################################################################


end module kmod_mcidas
