module kmod_time
!****h* /kmod_time
! NAME
! kmod_time
! SYNOPSIS
! Used for modifying time values
! USES
! none!  
!***

  use kmod_constants

  implicit none

  INTERFACE caldat
     MODULE PROCEDURE caldat_rank1
     MODULE PROCEDURE caldat_scalar
  END INTERFACE
  INTERFACE ftime
     MODULE PROCEDURE ftime_rank1
     MODULE PROCEDURE ftime_scalar
  END INTERFACE
  INTERFACE julday
     MODULE PROCEDURE julday_rank1
     MODULE PROCEDURE julday_scalar
  END INTERFACE

contains
!#######################################################################
!                                                           BEGIN CALDAT
!****r* kmod_time/caldat
! NAME
! caldat
! SYNOPSIS
! call caldat(julian, mm, id, iyyy, hour, min, sec)
! FUNCTION
! Inverse of the function julday. Here JULIAN is input as a Julian 
! Day Number and the routine outputs mm, id and iyyy as the month, 
! day and year on which the specified Julian Day started at noon
! INPUTS
! * Julian - real*8(scalar, rank1) - Julian Date (e.g., output from julday)
! OUTPUT
! Outputs are same rank as julian:
! * mm    = int*4 = Month of year (1-12)
! * id    = int*4 = Day of Month
! * iyyyy = int*4 = Year
! * hour  = int*4 = hour of day (optional)
! * min   = int*4 = minute of hour (optional)
! * sec   = real*4= second of minute (optional)
! NOTES
! This routine was typed in by Ken Knapp and is based on 
! the work presented in:
!    Numerical Recipes in Fortran 77
!    Second Edition 
!    Vol. 1 
!    by Press et al.
!***
  subroutine caldat_scalar ( julian, mm, id, iyyy, hour, min, sec)
    !External variables
    real   (kind=8), intent(in) :: julian
    integer(kind=4), intent(out):: mm    !month
    integer(kind=4), intent(out):: id    !day of month
    integer(kind=4), intent(out):: iyyy  !year 
    integer(kind=4), optional, intent(out) :: hour  !hour 
    integer(kind=4), optional, intent(out) :: min   !minute
    real   (kind=4), optional, intent(out) :: sec   !second

    integer (kind=4), dimension(1) :: m1,m2,m3,m4,m5
    real    (kind=4), dimension(1) :: m6

    call caldat ( (/julian/), m1,m2,m3,m4,m5,m6)
    mm = m1(1)
    id = m2(1)
    iyyy = m3(1)
    if (present(hour)) hour = m4(1)
    if (present(min))  min  = m5(1)
    if (present(sec))  sec  = m6(1)

  end subroutine caldat_scalar
  subroutine caldat_rank1 ( julian, mm, id, iyyy, hour, min, sec)
    !External variables
    real   (kind=8), intent(in),  dimension(:) :: julian
    integer(kind=4), intent(out), dimension(:) :: mm    !month
    integer(kind=4), intent(out), dimension(:) :: id    !day of month
    integer(kind=4), intent(out), dimension(:) :: iyyy  !year 
    integer(kind=4), optional, intent(out), dimension(:) :: hour  !hour 
    integer(kind=4), optional, intent(out), dimension(:) :: min   !minute
    real   (kind=4), optional, intent(out), dimension(:) :: sec   !second
    !Internal variables
    integer(kind=4) :: igreg
    integer(kind=4), dimension(size(julian)) :: ja, jalpha, jb, jc, jd, je, jullong
    real   (kind=8), dimension(size(julian)) :: frac

    igreg = 2299161
    !
    ! Calculate Month, Day, Year
    !
    !Cross over to Gregorian calendar produces this correction
    jullong = floor(julian + 0.5d0)
    where(jullong > igreg) 
       jalpha = int(((jullong - 1867216.) - 0.25d0) / 36524.25d0)
       ja = jullong + 1 + jalpha  - int(0.25 * jalpha)
    end where
    jb = ja + 1524.
    jc = int(6680. + ((jb - 2439870) - 122.1)/365.25)
    jd = int(365.d0 * jc + int(0.25d0*jc))
    je = int((jb-jd)/30.6001d0)
    
    !day of month
    id = jb - jd - int(30.6001d0*je)
    !month
    mm = je - 1
    where(mm > 12)
       mm = mm - 12
    end where
    !year
    iyyy = jc - 4715
    where (mm > 2)
       iyyy=iyyy-1
    end where
    where (iyyy <= 0)
       iyyy = iyyy - 1
    end where

    if (present(hour)) then
       !
       ! Calculate Hour:Minute:Second
       !
       frac = julian + 0.5d0 - jullong
       hour = floor(frac * 24.d0)
       frac = frac - real(hour,8)/24.d0
       min = floor(frac*1440.d0)
       sec  = (frac - real(min,8)/1440.d0) * 86400.d0
    endif
  end subroutine caldat_rank1
!                                                             END CALDAT
!#######################################################################
!#######################################################################
!                                                            BEGIN FTIME
!                                                           ftime_scalar
!****f* kmod_time/ftime
! NAME
! ftime
! SYNOPSIS
! result = ftime(hour,min,sec)
! FUNCTION
! Computes the fractional time of day based on HH:MM:SS
! INPUTS
! * HOUR = HHMMSS
!        or
! * hour
! * minute
! * second
!
! The inputs are either scalar or rank1 int*4.
! 
! RESULT
! * result - same rank as input hour, but type is real*4
!***
! Written by Ken Knapp
  real function ftime_scalar ( hour, minute, second ) 
    implicit none
    integer(kind=4), intent(in) :: hour
    integer(kind=4), intent(in), optional :: minute
    real   (kind=4), intent(in), optional :: second

    real   (kind=4), dimension(1) :: ftime_out
    if (present(minute) .and. present(second)) then
       ftime_out = ftime_rank1((/hour/),(/minute/),(/second/))
    else
       ftime_out = ftime_rank1((/hour/))
    endif
    ftime_scalar = ftime_out(1)
    return
  end function ftime_scalar
!                                                            ftime_rank1
  function ftime_rank1 ( hour, minute, second ) result (ftime_)
    implicit none
    ! External variables
    integer(kind=4), intent(in), dimension(:) :: hour
    integer(kind=4), intent(in), dimension(:), optional:: minute
    real   (kind=4), intent(in), dimension(:), optional :: second
    real   (kind=4), dimension(size(hour)) :: ftime_
    
    integer(kind=4), dimension(size(hour)) :: min, sec, h
    
    if (present(minute) .and. present(second)) then
       ftime_ = real(hour) + real(minute)/60. + second/3600.
    else
       h   = hour / 10000
       min = mod(int(hour/100),100)
       sec = mod(hour,100)
       ftime_ = real(h) + real(min)/60. + sec/3600.
    endif

    return
  end function ftime_rank1
!                                                              END FTIME
!#######################################################################
!#######################################################################
!                                                           BEGIN JULDAY
!****f* kmod_time/julday
! NAME
! julday
! SYNOPSIS
! time = julday(mon,dom,year,hour,min,sec)
! FUNCTION
! Calculates the Julian Date based on conventional time.
! INPUTS
! * mon - i*4 - month of year (1-12)
! * dom - i*4 - day of month (1-31)
!   * if mon = 1 then this is the daoy (Day of year: 1-366)
! * year- i*4
! * hour- i*4 - hour of day (optional)
! * min - i*4 - minute of hour (optional)
! * sec - r*4 - second of minute (optional)
!
!   * If any of the optional parameters are left out, they are set to zero.
!   * Time can be input as scalar or rank1.
! RESULT
! * time (r*8: same rank as inputs) - Julian Date
! NOTES
! This routine was typed in by Ken Knapp and is  based on 
! the work presented in:
!    Numerical Recipes in Fortran 77
!    Second Edition 
!    Vol. 1 
!    by Press et al.
!
! In this routine, julday returns the Julian Day Number that begins
! at noon of the calendar date specified by month mon, day dom, and year. 
! Positive year signifies A.D and negative year B.C. Remeber, 1 BC is 
! followed by 1 AD (there is no zero year)
!***
!                                                           julday_rank1
  function julday_rank1 ( mm, id, iyyy, &
                          hour, minute, second) result (julday_)
    implicit none
    !External variables
    integer(kind=4), intent(in), dimension(:) :: mm
    integer(kind=4), intent(in), dimension(:) :: id
    integer(kind=4), intent(in), dimension(:) :: iyyy
    integer(kind=4), intent(in), optional, dimension(:) :: hour
    integer(kind=4), intent(in), optional, dimension(:) :: minute
    real   (kind=4), intent(in), optional, dimension(:) :: second
    real   (kind=8), dimension(size(mm)) :: julday_
    !Internal variables
    integer(kind=4), dimension(size(mm)) :: ja, jm, jy
    !Gregorian Calendar adopted October 15, 1582
    integer(kind=4) :: igreg 
    igreg = 15+31*(10+12*1582)
    
    jy=iyyy
    if (count(jy .eq. 0) .eq. 1) &
         print*,'JULDAY: There is no zero year'
    where (jy < 0) 
       jy = jy + 1
    end where
    where (mm > 2)
       jm = mm + 1
    elsewhere
       jy=jy-1
       jm=mm+13
    end where
    
    julday_ = (int(365.25*jy)+int(30.6001*jm)+id+1720995)
    
    !Test whether to change to gregorian calendar
    where ((id + 31*(mm+12*iyyy))>igreg)
       ja = int(0.01*jy)
       julday_ = julday_ + 2 - ja + int(0.25*ja)
    end where
    julday_ = julday_ - 0.5d0
    !
    ! Adjust for time of day if present
    if (present(hour) .and. present(minute) .and. &
        present(second)) then
       julday_ = julday_ + real(hour,8) / 24.d0 + &
                 real(minute,8) / 24.d0 / 60.d0 + &
                 real(second,8) / 24.d0 / 60.d0 / 60.d0
    endif
    return
  end function julday_rank1
!                                                          julday_scalar
  function julday_scalar ( mm, id, iyyy, &
                           hour, minute, second)
    implicit none
    ! External variables
    integer(kind=4), intent(in) :: mm
    integer(kind=4), intent(in) :: id
    integer(kind=4), intent(in) :: iyyy
    integer(kind=4), intent(in), optional :: hour
    integer(kind=4), intent(in), optional :: minute
    real   (kind=4), intent(in), optional :: second
    real   (kind=8) :: julday_scalar
    !Internal variables
    real (kind=8), dimension(1) :: julday_out
    if (present(hour) .and. present(minute) .and. present(second)) then
       julday_out = julday_rank1((/mm/),(/id/),(/iyyy/),&
                                 (/hour/),(/minute/),(/second/))
    else
       julday_out = julday_rank1((/mm/),(/id/),(/iyyy/))
    endif
    julday_scalar = julday_out(1)
    return
  end function julday_scalar
!                                                             END JULDAY
!#######################################################################

!#######################################################################
!                                                   BEGIN yymmdd_2_yyjjj
!****f* kmod_time/yymmdd_2_yyjjj
! NAME
! yymmdd_2_yyjjj
! SYNOPSIS
! time = yymmdd(input)
! FUNCTION
! Calculates the Julian Date based on conventional time.
! INPUTS
! input - date in format YYMMDD
! * If optional arguments are specified, then input is merely the 
!   dom
! RESULT
! * 
! NOTES
! * Although the title has YY, year can either be 2 or 4 digits
!***
   integer(kind=4) function yymmdd_2_yyjjj(input,mon,year)
    integer(kind=4), intent(in) :: input
    integer(kind=4), intent(in), optional :: mon, year
    
    integer(kind=4) :: kyear, kmon, kdom, leap, kdoy

    if (present(mon) .and. present(year)) then
       kdom = mod(input,100)
       kmon = mod(mon,100)
       kyear= mod(year,10000)
    else
       kdom = mod(input,100)
       kmon = mod(input/100,100)
       kyear= input/10000
    end if
    !check range on year
    if (kyear < 100) kyear=kyear+1900

    !the following is only for satellite data which does not occur before 1970
    if (kyear < 1970) kyear=kyear+100
    !check for leap year
    leap = 0
    if (mod(year,4) == 0) then
       leap=1
       if (mod(year,100) == 0) leap=0
       if (mod(year,1000) == 0) leap=1
    endif

    if (kmon == 1) then
       kdoy = kdom
    else
       if (leap == 1) then
          kdoy = sum(daysinmonth_leap(1:kmon-1)) + kdom
       else
          kdoy = sum(daysinmonth(     1:kmon-1)) + kdom
       end if
    end if

    yymmdd_2_yyjjj = kyear*1000 + kdoy
  end function yymmdd_2_yyjjj


end module kmod_time
