module kmod_utilities
!****h* /kmod_utilities
! NAME
! kmod_utilities
! SYNOPSIS
! This is the generic module which holds all generic utilities for general 
! distribution to the public.
! NOTES
!***

  ! Included in order to test for the g95 compiler
    type g95_t
     integer x
    end type g95_t

    INTERFACE ASSIGNMENT (=)
       module procedure assign_t
    END INTERFACE
  ! End Include

  INTERFACE eval
     MODULE PROCEDURE eval_scalar
     MODULE PROCEDURE eval_rank1
     MODULE PROCEDURE eval_rank2
  END INTERFACE
  INTERFACE qsort
     MODULE PROCEDURE qsortd_r8
     MODULE PROCEDURE qsortd_r4
     MODULE PROCEDURE qsortd_i4
     MODULE PROCEDURE qsortd_i2
     MODULE PROCEDURE qsortd_i1
  END INTERFACE
  INTERFACE swap_endian
     MODULE PROCEDURE swap_endian_kind2_rank1
     MODULE PROCEDURE swap_endian_kind2_rank2
     MODULE PROCEDURE swap_endian_kind2_rank3
     MODULE PROCEDURE swap_endian_kind2_scalar
     MODULE PROCEDURE swap_endian_kind4_rank1
     MODULE PROCEDURE swap_endian_kind4_scalar
     MODULE PROCEDURE swap_endian_real4_rank1
     MODULE PROCEDURE swap_endian_real4_rank2
     MODULE PROCEDURE swap_endian_real4_scalar
  END INTERFACE
  INTERFACE within
     MODULE PROCEDURE within_i2_rank2
     MODULE PROCEDURE within_i2_scalar
     MODULE PROCEDURE within_i4_scalar
     MODULE PROCEDURE within_i4_rank1
     MODULE PROCEDURE within_i4_rank2
     MODULE PROCEDURE within_r4_scalar
     MODULE PROCEDURE within_r4_rank1
     MODULE PROCEDURE within_r4_rank2
     MODULE PROCEDURE within_r8_scalar
     MODULE PROCEDURE within_r8_rank1
     MODULE PROCEDURE within_r8_rank2
  END INTERFACE

contains

  ! Included in order to test for the g95 compiler
  function is_g95()
    logical is_g95
    type(g95_t) y(3)
    
    y%x = (/1,2,3/)
    y = y((/2,3,1/))
    is_g95 = y(3)%x == 1
  end function is_g95
  
  elemental subroutine assign_t(lhs,rhs)
    type(g95_t), intent(in) :: rhs
    type(g95_t), intent(out) :: lhs
    
    lhs%x = rhs%x
  end subroutine assign_t
  ! End g95 include

!#######################################################################
!                                                             BEGIN EVAL
!****f* kmod_utilities/eval
! NAME
! eval
! SYNOPSIS
! mask = eval(logical1)
! FUNCTION
! Returns an int*4 evaluated to 1 or 0 based on logical1
! INPUTS
! logical1 - logical (scalar, rank1, rank2)
! OUTPUT
! mask - int*4(same rank as logical1)
! NOTES
! * set to 1 if logical2 is true
! * set to 0 if logical1 is false
! * This is based on the IDL usage:
!   IDL> print,indgen(6) gt 2
!     0 0 0 1 1 1
! EXAMPLE
!  mask = eval((/4, 5, 6, 7/) == 6 )
!  returns:
!     mask = (/ 0, 0, 1, 0 /)
!***
  pure function eval_scalar ( test ) result ( output)
    !External
    logical, intent(in) ::  test
    integer(kind=4) :: output
    output = 0
    if (test) output = 1

  end function eval_scalar

  pure function eval_rank1 ( test ) result (output)
    !Externals
    logical, dimension(:), intent(in) :: test
    integer(kind=4), dimension(size(test)) :: output

    output = 0
    where (test) output = 1
    
  end function eval_rank1

  pure function eval_rank2 ( test ) result (output)
    !Externals
    logical, dimension(:,:), intent(in) :: test
    integer(kind=4), dimension(size(test,1),size(test,2)) :: output

    output = 0
    where (test) output = 1
    
  end function eval_rank2
!                                                               END EVAL
!#######################################################################
!#######################################################################
!                                                        BEGIN IDL_WHERE
!****r* kmod_utilities/idl_where
! NAME
! idl_where
! SYNOPSIS
! call idl_where(test,index,num)
! FUNCTION
! This function mimics the IDL function where. It returns an array with
! values assigned where the array test is .true. 
!
! INPUTS
!
! OUTPUT
! 
! EXAMPLES
!  The foolowing source:
!      call idl_where( (/ 1,2,3,4/) >= 3,index,number)
!      print*,number
!      print*,index
!  Produces:
!      2
!      3 4
! 
! USED BY
! clavr_ir_tests.f90, clavr_comp6.f90, median
!***
  subroutine idl_where ( test, index, number )
    !Externals
    logical, dimension(:) :: test
    integer(kind=4), dimension(:), allocatable :: index
    integer(kind=4) :: number
    !Internals
    integer(kind=4) :: i, j
    number = count(test)
    if (allocated(index)) deallocate(index)
    allocate (index(number))
    j=0
    do i=1,size(test)
       if (test(i)) then
          j=j+1
          index(j) = i
       end if
    end do
  end subroutine idl_where
!                                                          END IDL_WHERE
!#######################################################################
!#######################################################################
!****f* kmod_utilities/is_zipped
! NAME
! is_zipped
! SYNOPSIS
! if (is_zipped(filename)) then ...
! FUNCTION
! This function determines whether a file is compressed.
!
! INPUTS
! * Char(len=*) - filename - Filename of the file to be tested
! RESULT
! true  - if the file ends with 'gz'
! false - otherwise
! USES
! strsplit
!***
  logical function is_zipped(infile)
    character(len=*), intent(in) :: infile
    !internal variables
    integer(kind=4) :: n
    character(len=120), allocatable, dimension(:) :: f

    !split the filename
    call strsplit(infile,'.',f,n)
    if (trim(f(n)) == 'gz') then 
       is_zipped = .true.
       return
    else
       is_zipped = .false.
       return
    endif

  end function is_zipped
!#######################################################################
!#######################################################################
!                                                        BEGIN POINT_LUN
!****r* kmod_utilities/point_lun
! NAME
! point_lun
! SYNOPSIS
! call point_lun(unit,position)
! FUNCTION
!    To place the current location of logical unit UNIT at POSITION 
!    bytes into the file.
! INPUTS
!  * unit - an open logical unit number
!  * position - position in bytes
! NOTES
! This mimics IDL point_lun routine, but it is very slow, since it
! rewinds the file and reads in POSITION bytes. There is a faster
! routine in Lahey Fortran, but that would make this not very portable.
! USED BY
! b1uread
!***

  subroutine point_lun ( unit, position)
    integer(kind=4), intent(in) :: unit
    integer(kind=4), intent(in) :: position
    
    integer(kind=1) :: one_byte
    integer(kind=4) :: i

    rewind (unit)
    do i=1,position
       read (unit) one_byte
    end do
  end subroutine point_lun
!                                                          END POINT_LUN
!#######################################################################
!#######################################################################
!                                                           BEGIN QSORTD
!****r* kmod_utitlities/qsortd
! NAME
! qsortd
! SYNOPSIS
! call qsortd(x,ind,n)
! FUNCTION
! Sorts the data in parameter x returning the sorted indices in IND
! INPUTS
! x - rank1 - real*4 or 8 or integer *1, 2 or 4 - input data to be sorted
! n - # of elements in x
! OUTPUT
! ind - indices of the sorted parameters
! 
! NOTES
!
!                                                         ROBERT RENKA
!                                                 OAK RIDGE NATL. LAB.
!   THIS SUBROUTINE USES AN ORDER N*LOG(N) QUICK SORT TO SORT A REAL (dp)
! ARRAY X INTO INCREASING ORDER.  THE ALGORITHM IS AS FOLLOWS.  IND IS
! INITIALIZED TO THE ORDERED SEQUENCE OF INDICES 1,...,N, AND ALL INTERCHANGES
! ARE APPLIED TO IND.  X IS DIVIDED INTO TWO PORTIONS BY PICKING A CENTRAL
! ELEMENT T.  THE FIRST AND LAST ELEMENTS ARE COMPARED WITH T, AND
! INTERCHANGES ARE APPLIED AS NECESSARY SO THAT THE THREE VALUES ARE IN
! ASCENDING ORDER.  INTERCHANGES ARE THEN APPLIED SO THAT ALL ELEMENTS
! GREATER THAN T ARE IN THE UPPER PORTION OF THE ARRAY AND ALL ELEMENTS
! LESS THAN T ARE IN THE LOWER PORTION.  THE UPPER AND LOWER INDICES OF ONE
! OF THE PORTIONS ARE SAVED IN LOCAL ARRAYS, AND THE PROCESS IS REPEATED
! ITERATIVELY ON THE OTHER PORTION.  WHEN A PORTION IS COMPLETELY SORTED,
! THE PROCESS BEGINS AGAIN BY RETRIEVING THE INDICES BOUNDING ANOTHER
! UNSORTED PORTION.
! INPUT PARAMETERS -   N - LENGTH OF THE ARRAY X.
!                      X - VECTOR OF LENGTH N TO BE SORTED.
!                    IND - VECTOR OF LENGTH >= N.
! N AND X ARE NOT ALTERED BY THIS ROUTINE.
! OUTPUT PARAMETER - IND - SEQUENCE OF INDICES 1,...,N PERMUTED IN THE SAME
!                          FASHION AS X WOULD BE.  THUS, THE ORDERING ON
!                          X IS DEFINED BY Y(I) = X(IND(I)).
!
! -- IU AND IL MUST BE DIMENSIONED >= LOG(N) WHERE LOG HAS BASE 2.
!
! -- Knapp modified it to allow different types of real and integers
!***

  SUBROUTINE qsortd_r8(x,ind,n)
    ! Code converted using TO_F90 by Alan Miller
    ! Date: 2002-12-18  Time: 11:55:47
    INTEGER, PARAMETER  :: dp = 8 !SELECTED_REAL_KIND(12, 60)
    REAL (dp), INTENT(IN)  :: x(:)
    INTEGER, INTENT(OUT)   :: ind(:)
    INTEGER, INTENT(IN)    :: n
    INTEGER   :: iu(21), il(21)
    INTEGER   :: m, i, j, k, l, ij, it, itt, indx
    REAL      :: r
    REAL (dp) :: t
    ! LOCAL PARAMETERS -
    ! IU,IL =  TEMPORARY STORAGE FOR THE UPPER AND LOWER
    !            INDICES OF PORTIONS OF THE ARRAY X
    ! M =      INDEX FOR IU AND IL
    ! I,J =    LOWER AND UPPER INDICES OF A PORTION OF X
    ! K,L =    INDICES IN THE RANGE I,...,J
    ! IJ =     RANDOMLY CHOSEN INDEX BETWEEN I AND J
    ! IT,ITT = TEMPORARY STORAGE FOR INTERCHANGES IN IND
    ! INDX =   TEMPORARY INDEX FOR X
    ! R =      PSEUDO RANDOM NUMBER FOR GENERATING IJ
    ! T =      CENTRAL ELEMENT OF X
    IF (n <= 0) RETURN
    ! INITIALIZE IND, M, I, J, AND R
    DO  i = 1, n
       ind(i) = i
    END DO
    m = 1
    i = 1
    j = n
    r = .375
    ! TOP OF LOOP
20  IF (i >= j) GO TO 70
    IF (r <= .5898437) THEN
       r = r + .0390625
    ELSE
       r = r - .21875
    END IF
    ! INITIALIZE K
30  k = i
    ! SELECT A CENTRAL ELEMENT OF X AND SAVE IT IN T
    ij = i + r*(j-i)
    it = ind(ij)
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) > t) THEN
       ind(ij) = indx
       ind(i) = it
       it = indx
       t = x(it)
    END IF
    ! INITIALIZE L
    l = j
    ! IF THE LAST ELEMENT OF THE ARRAY IS LESS THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(j)
    IF (x(indx) >= t) GO TO 50
    ind(ij) = indx
    ind(j) = it
    it = indx
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) <= t) GO TO 50
    ind(ij) = indx
    ind(i) = it
    it = indx
    t = x(it)
    GO TO 50
    ! INTERCHANGE ELEMENTS K AND L
40  itt = ind(l)
    ind(l) = ind(k)
    ind(k) = itt
    ! FIND AN ELEMENT IN THE UPPER PART OF THE ARRAY WHICH IS
    !   NOT LARGER THAN T
50  l = l - 1
    indx = ind(l)
    IF (x(indx) > t) GO TO 50
    ! FIND AN ELEMENT IN THE LOWER PART OF THE ARRAY WHCIH IS NOT SMALLER THAN T
60  k = k + 1
    indx = ind(k)
    IF (x(indx) < t) GO TO 60
    ! IF K <= L, INTERCHANGE ELEMENTS K AND L
    IF (k <= l) GO TO 40
    ! SAVE THE UPPER AND LOWER SUBSCRIPTS OF THE PORTION OF THE
    !   ARRAY YET TO BE SORTED
    IF (l-i > j-k) THEN
       il(m) = i
       iu(m) = l
       i = k
       m = m + 1
       GO TO 80
    END IF
    il(m) = k
    iu(m) = j
    j = l
    m = m + 1
    GO TO 80
    ! BEGIN AGAIN ON ANOTHER UNSORTED PORTION OF THE ARRAY
70  m = m - 1
    IF (m == 0) RETURN
    i = il(m)
    j = iu(m)
80  IF (j-i >= 11) GO TO 30
    IF (i == 1) GO TO 20
    i = i - 1
    ! SORT ELEMENTS I+1,...,J.  NOTE THAT 1 <= I < J AND J-I < 11.
90  i = i + 1
    IF (i == j) GO TO 70
    indx = ind(i+1)
    t = x(indx)
    it = indx
    indx = ind(i)
    IF (x(indx) <= t) GO TO 90
    k = i
100 ind(k+1) = ind(k)
    k = k - 1
    indx = ind(k)
    IF (t < x(indx)) GO TO 100
    ind(k+1) = it
    GO TO 90
  END SUBROUTINE qsortd_r8
  
  SUBROUTINE qsortd_r4(x,ind,n)
    ! Code converted using TO_F90 by Alan Miller
    ! Date: 2002-12-18  Time: 11:55:47
    INTEGER, PARAMETER  :: dp = 4 !SELECTED_REAL_KIND(12, 60)
    REAL (dp), INTENT(IN)  :: x(:)
    INTEGER, INTENT(OUT)   :: ind(:)
    INTEGER, INTENT(IN)    :: n
    INTEGER   :: iu(21), il(21)
    INTEGER   :: m, i, j, k, l, ij, it, itt, indx
    REAL      :: r
    REAL (dp) :: t
    ! LOCAL PARAMETERS -
    ! IU,IL =  TEMPORARY STORAGE FOR THE UPPER AND LOWER
    !            INDICES OF PORTIONS OF THE ARRAY X
    ! M =      INDEX FOR IU AND IL
    ! I,J =    LOWER AND UPPER INDICES OF A PORTION OF X
    ! K,L =    INDICES IN THE RANGE I,...,J
    ! IJ =     RANDOMLY CHOSEN INDEX BETWEEN I AND J
    ! IT,ITT = TEMPORARY STORAGE FOR INTERCHANGES IN IND
    ! INDX =   TEMPORARY INDEX FOR X
    ! R =      PSEUDO RANDOM NUMBER FOR GENERATING IJ
    ! T =      CENTRAL ELEMENT OF X
    IF (n <= 0) RETURN
    ! INITIALIZE IND, M, I, J, AND R
    DO  i = 1, n
       ind(i) = i
    END DO
    m = 1
    i = 1
    j = n
    r = .375
    ! TOP OF LOOP
20  IF (i >= j) GO TO 70
    IF (r <= .5898437) THEN
       r = r + .0390625
    ELSE
       r = r - .21875
    END IF
    ! INITIALIZE K
30  k = i
    ! SELECT A CENTRAL ELEMENT OF X AND SAVE IT IN T
    ij = i + r*(j-i)
    it = ind(ij)
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) > t) THEN
       ind(ij) = indx
       ind(i) = it
       it = indx
       t = x(it)
    END IF
    ! INITIALIZE L
    l = j
    ! IF THE LAST ELEMENT OF THE ARRAY IS LESS THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(j)
    IF (x(indx) >= t) GO TO 50
    ind(ij) = indx
    ind(j) = it
    it = indx
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) <= t) GO TO 50
    ind(ij) = indx
    ind(i) = it
    it = indx
    t = x(it)
    GO TO 50
    ! INTERCHANGE ELEMENTS K AND L
40  itt = ind(l)
    ind(l) = ind(k)
    ind(k) = itt
    ! FIND AN ELEMENT IN THE UPPER PART OF THE ARRAY WHICH IS
    !   NOT LARGER THAN T
50  l = l - 1
    indx = ind(l)
    IF (x(indx) > t) GO TO 50
    ! FIND AN ELEMENT IN THE LOWER PART OF THE ARRAY WHCIH IS NOT SMALLER THAN T
60  k = k + 1
    indx = ind(k)
    IF (x(indx) < t) GO TO 60
    ! IF K <= L, INTERCHANGE ELEMENTS K AND L
    IF (k <= l) GO TO 40
    ! SAVE THE UPPER AND LOWER SUBSCRIPTS OF THE PORTION OF THE
    !   ARRAY YET TO BE SORTED
    IF (l-i > j-k) THEN
       il(m) = i
       iu(m) = l
       i = k
       m = m + 1
       GO TO 80
    END IF
    il(m) = k
    iu(m) = j
    j = l
    m = m + 1
    GO TO 80
    ! BEGIN AGAIN ON ANOTHER UNSORTED PORTION OF THE ARRAY
70  m = m - 1
    IF (m == 0) RETURN
    i = il(m)
    j = iu(m)
80  IF (j-i >= 11) GO TO 30
    IF (i == 1) GO TO 20
    i = i - 1
    ! SORT ELEMENTS I+1,...,J.  NOTE THAT 1 <= I < J AND J-I < 11.
90  i = i + 1
    IF (i == j) GO TO 70
    indx = ind(i+1)
    t = x(indx)
    it = indx
    indx = ind(i)
    IF (x(indx) <= t) GO TO 90
    k = i
100 ind(k+1) = ind(k)
    k = k - 1
    indx = ind(k)
    IF (t < x(indx)) GO TO 100
    ind(k+1) = it
    GO TO 90
  END SUBROUTINE qsortd_r4
  SUBROUTINE qsortd_i4(x,ind,n)
    ! Code converted using TO_F90 by Alan Miller
    ! Date: 2002-12-18  Time: 11:55:47
    integer (kind=4), INTENT(IN)  :: x(:)
    INTEGER, INTENT(OUT)   :: ind(:)
    INTEGER, INTENT(IN)    :: n
    INTEGER   :: iu(21), il(21)
    INTEGER   :: m, i, j, k, l, ij, it, itt, indx
    REAL      :: r
    integer (kind=4) :: t
    ! LOCAL PARAMETERS -
    ! IU,IL =  TEMPORARY STORAGE FOR THE UPPER AND LOWER
    !            INDICES OF PORTIONS OF THE ARRAY X
    ! M =      INDEX FOR IU AND IL
    ! I,J =    LOWER AND UPPER INDICES OF A PORTION OF X
    ! K,L =    INDICES IN THE RANGE I,...,J
    ! IJ =     RANDOMLY CHOSEN INDEX BETWEEN I AND J
    ! IT,ITT = TEMPORARY STORAGE FOR INTERCHANGES IN IND
    ! INDX =   TEMPORARY INDEX FOR X
    ! R =      PSEUDO RANDOM NUMBER FOR GENERATING IJ
    ! T =      CENTRAL ELEMENT OF X
    IF (n <= 0) RETURN
    ! INITIALIZE IND, M, I, J, AND R
    DO  i = 1, n
       ind(i) = i
    END DO
    m = 1
    i = 1
    j = n
    r = .375
    ! TOP OF LOOP
20  IF (i >= j) GO TO 70
    IF (r <= .5898437) THEN
       r = r + .0390625
    ELSE
       r = r - .21875
    END IF
    ! INITIALIZE K
30  k = i
    ! SELECT A CENTRAL ELEMENT OF X AND SAVE IT IN T
    ij = i + r*(j-i)
    it = ind(ij)
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) > t) THEN
       ind(ij) = indx
       ind(i) = it
       it = indx
       t = x(it)
    END IF
    ! INITIALIZE L
    l = j
    ! IF THE LAST ELEMENT OF THE ARRAY IS LESS THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(j)
    IF (x(indx) >= t) GO TO 50
    ind(ij) = indx
    ind(j) = it
    it = indx
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) <= t) GO TO 50
    ind(ij) = indx
    ind(i) = it
    it = indx
    t = x(it)
    GO TO 50
    ! INTERCHANGE ELEMENTS K AND L
40  itt = ind(l)
    ind(l) = ind(k)
    ind(k) = itt
    ! FIND AN ELEMENT IN THE UPPER PART OF THE ARRAY WHICH IS
    !   NOT LARGER THAN T
50  l = l - 1
    indx = ind(l)
    IF (x(indx) > t) GO TO 50
    ! FIND AN ELEMENT IN THE LOWER PART OF THE ARRAY WHCIH IS NOT SMALLER THAN T
60  k = k + 1
    indx = ind(k)
    IF (x(indx) < t) GO TO 60
    ! IF K <= L, INTERCHANGE ELEMENTS K AND L
    IF (k <= l) GO TO 40
    ! SAVE THE UPPER AND LOWER SUBSCRIPTS OF THE PORTION OF THE
    !   ARRAY YET TO BE SORTED
    IF (l-i > j-k) THEN
       il(m) = i
       iu(m) = l
       i = k
       m = m + 1
       GO TO 80
    END IF
    il(m) = k
    iu(m) = j
    j = l
    m = m + 1
    GO TO 80
    ! BEGIN AGAIN ON ANOTHER UNSORTED PORTION OF THE ARRAY
70  m = m - 1
    IF (m == 0) RETURN
    i = il(m)
    j = iu(m)
80  IF (j-i >= 11) GO TO 30
    IF (i == 1) GO TO 20
    i = i - 1
    ! SORT ELEMENTS I+1,...,J.  NOTE THAT 1 <= I < J AND J-I < 11.
90  i = i + 1
    IF (i == j) GO TO 70
    indx = ind(i+1)
    t = x(indx)
    it = indx
    indx = ind(i)
    IF (x(indx) <= t) GO TO 90
    k = i
100 ind(k+1) = ind(k)
    k = k - 1
    indx = ind(k)
    IF (t < x(indx)) GO TO 100
    ind(k+1) = it
    GO TO 90
  END SUBROUTINE qsortd_i4
  SUBROUTINE qsortd_i2(x,ind,n)
    ! Code converted using TO_F90 by Alan Miller
    ! Date: 2002-12-18  Time: 11:55:47
    integer(kind=2), INTENT(IN)  :: x(:)
    INTEGER, INTENT(OUT)   :: ind(:)
    INTEGER, INTENT(IN)    :: n
    INTEGER   :: iu(21), il(21)
    INTEGER   :: m, i, j, k, l, ij, it, itt, indx
    REAL      :: r
    integer(kind=2) :: t
    ! LOCAL PARAMETERS -
    ! IU,IL =  TEMPORARY STORAGE FOR THE UPPER AND LOWER
    !            INDICES OF PORTIONS OF THE ARRAY X
    ! M =      INDEX FOR IU AND IL
    ! I,J =    LOWER AND UPPER INDICES OF A PORTION OF X
    ! K,L =    INDICES IN THE RANGE I,...,J
    ! IJ =     RANDOMLY CHOSEN INDEX BETWEEN I AND J
    ! IT,ITT = TEMPORARY STORAGE FOR INTERCHANGES IN IND
    ! INDX =   TEMPORARY INDEX FOR X
    ! R =      PSEUDO RANDOM NUMBER FOR GENERATING IJ
    ! T =      CENTRAL ELEMENT OF X
    IF (n <= 0) RETURN
    ! INITIALIZE IND, M, I, J, AND R
    DO  i = 1, n
       ind(i) = i
    END DO
    m = 1
    i = 1
    j = n
    r = .375
    ! TOP OF LOOP
20  IF (i >= j) GO TO 70
    IF (r <= .5898437) THEN
       r = r + .0390625
    ELSE
       r = r - .21875
    END IF
    ! INITIALIZE K
30  k = i
    ! SELECT A CENTRAL ELEMENT OF X AND SAVE IT IN T
    ij = i + r*(j-i)
    it = ind(ij)
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) > t) THEN
       ind(ij) = indx
       ind(i) = it
       it = indx
       t = x(it)
    END IF
    ! INITIALIZE L
    l = j
    ! IF THE LAST ELEMENT OF THE ARRAY IS LESS THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(j)
    IF (x(indx) >= t) GO TO 50
    ind(ij) = indx
    ind(j) = it
    it = indx
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) <= t) GO TO 50
    ind(ij) = indx
    ind(i) = it
    it = indx
    t = x(it)
    GO TO 50
    ! INTERCHANGE ELEMENTS K AND L
40  itt = ind(l)
    ind(l) = ind(k)
    ind(k) = itt
    ! FIND AN ELEMENT IN THE UPPER PART OF THE ARRAY WHICH IS
    !   NOT LARGER THAN T
50  l = l - 1
    indx = ind(l)
    IF (x(indx) > t) GO TO 50
    ! FIND AN ELEMENT IN THE LOWER PART OF THE ARRAY WHCIH IS NOT SMALLER THAN T
60  k = k + 1
    indx = ind(k)
    IF (x(indx) < t) GO TO 60
    ! IF K <= L, INTERCHANGE ELEMENTS K AND L
    IF (k <= l) GO TO 40
    ! SAVE THE UPPER AND LOWER SUBSCRIPTS OF THE PORTION OF THE
    !   ARRAY YET TO BE SORTED
    IF (l-i > j-k) THEN
       il(m) = i
       iu(m) = l
       i = k
       m = m + 1
       GO TO 80
    END IF
    il(m) = k
    iu(m) = j
    j = l
    m = m + 1
    GO TO 80
    ! BEGIN AGAIN ON ANOTHER UNSORTED PORTION OF THE ARRAY
70  m = m - 1
    IF (m == 0) RETURN
    i = il(m)
    j = iu(m)
80  IF (j-i >= 11) GO TO 30
    IF (i == 1) GO TO 20
    i = i - 1
    ! SORT ELEMENTS I+1,...,J.  NOTE THAT 1 <= I < J AND J-I < 11.
90  i = i + 1
    IF (i == j) GO TO 70
    indx = ind(i+1)
    t = x(indx)
    it = indx
    indx = ind(i)
    IF (x(indx) <= t) GO TO 90
    k = i
100 ind(k+1) = ind(k)
    k = k - 1
    indx = ind(k)
    IF (t < x(indx)) GO TO 100
    ind(k+1) = it
    GO TO 90
  END SUBROUTINE qsortd_i2
  SUBROUTINE qsortd_i1(x,ind,n)
    ! Code converted using TO_F90 by Alan Miller
    ! Date: 2002-12-18  Time: 11:55:47
    integer(kind=1), INTENT(IN)  :: x(:)
    INTEGER, INTENT(OUT)   :: ind(:)
    INTEGER, INTENT(IN)    :: n
    INTEGER   :: iu(21), il(21)
    INTEGER   :: m, i, j, k, l, ij, it, itt, indx
    REAL      :: r
    integer(kind=1) :: t
    ! LOCAL PARAMETERS -
    ! IU,IL =  TEMPORARY STORAGE FOR THE UPPER AND LOWER
    !            INDICES OF PORTIONS OF THE ARRAY X
    ! M =      INDEX FOR IU AND IL
    ! I,J =    LOWER AND UPPER INDICES OF A PORTION OF X
    ! K,L =    INDICES IN THE RANGE I,...,J
    ! IJ =     RANDOMLY CHOSEN INDEX BETWEEN I AND J
    ! IT,ITT = TEMPORARY STORAGE FOR INTERCHANGES IN IND
    ! INDX =   TEMPORARY INDEX FOR X
    ! R =      PSEUDO RANDOM NUMBER FOR GENERATING IJ
    ! T =      CENTRAL ELEMENT OF X
    IF (n <= 0) RETURN
    ! INITIALIZE IND, M, I, J, AND R
    DO  i = 1, n
       ind(i) = i
    END DO
    m = 1
    i = 1
    j = n
    r = .375
    ! TOP OF LOOP
20  IF (i >= j) GO TO 70
    IF (r <= .5898437) THEN
       r = r + .0390625
    ELSE
       r = r - .21875
    END IF
    ! INITIALIZE K
30  k = i
    ! SELECT A CENTRAL ELEMENT OF X AND SAVE IT IN T
    ij = i + r*(j-i)
    it = ind(ij)
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) > t) THEN
       ind(ij) = indx
       ind(i) = it
       it = indx
       t = x(it)
    END IF
    ! INITIALIZE L
    l = j
    ! IF THE LAST ELEMENT OF THE ARRAY IS LESS THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(j)
    IF (x(indx) >= t) GO TO 50
    ind(ij) = indx
    ind(j) = it
    it = indx
    t = x(it)
    ! IF THE FIRST ELEMENT OF THE ARRAY IS GREATER THAN T,
    !   INTERCHANGE IT WITH T
    indx = ind(i)
    IF (x(indx) <= t) GO TO 50
    ind(ij) = indx
    ind(i) = it
    it = indx
    t = x(it)
    GO TO 50
    ! INTERCHANGE ELEMENTS K AND L
40  itt = ind(l)
    ind(l) = ind(k)
    ind(k) = itt
    ! FIND AN ELEMENT IN THE UPPER PART OF THE ARRAY WHICH IS
    !   NOT LARGER THAN T
50  l = l - 1
    indx = ind(l)
    IF (x(indx) > t) GO TO 50
    ! FIND AN ELEMENT IN THE LOWER PART OF THE ARRAY WHCIH IS NOT SMALLER THAN T
60  k = k + 1
    indx = ind(k)
    IF (x(indx) < t) GO TO 60
    ! IF K <= L, INTERCHANGE ELEMENTS K AND L
    IF (k <= l) GO TO 40
    ! SAVE THE UPPER AND LOWER SUBSCRIPTS OF THE PORTION OF THE
    !   ARRAY YET TO BE SORTED
    IF (l-i > j-k) THEN
       il(m) = i
       iu(m) = l
       i = k
       m = m + 1
       GO TO 80
    END IF
    il(m) = k
    iu(m) = j
    j = l
    m = m + 1
    GO TO 80
    ! BEGIN AGAIN ON ANOTHER UNSORTED PORTION OF THE ARRAY
70  m = m - 1
    IF (m == 0) RETURN
    i = il(m)
    j = iu(m)
80  IF (j-i >= 11) GO TO 30
    IF (i == 1) GO TO 20
    i = i - 1
    ! SORT ELEMENTS I+1,...,J.  NOTE THAT 1 <= I < J AND J-I < 11.
90  i = i + 1
    IF (i == j) GO TO 70
    indx = ind(i+1)
    t = x(indx)
    it = indx
    indx = ind(i)
    IF (x(indx) <= t) GO TO 90
    k = i
100 ind(k+1) = ind(k)
    k = k - 1
    indx = ind(k)
    IF (t < x(indx)) GO TO 100
    ind(k+1) = it
    GO TO 90
  END SUBROUTINE qsortd_i1
!                                                             END QSORTD
!#######################################################################
!#######################################################################
!                                                    BEGIN RANDOM_STRING
!****f* kmod_utilities/random_string
! NAME
! random_string
! SYNOPSIS
! temp_file=random_string(N)
! FUNCTION
! Creates a random string of length N.
! INPUTS
! N - i*4 - length of the string to be generated
! RESULT
! * char(len=120) - random string
! NOTES
!***
  character(len=120) function random_string(length)
    integer, intent(in) :: length
    integer i
    integer, dimension(1) :: c
    real :: rand
    random_string=''
    call system_clock(i)
    c(1) = i
    call random_seed(put=c)
    do i=1,length
       call random_number(rand)
       random_string=trim(random_string)//char(int(65+26*rand))
    end do
  end function random_string
!                                                      END RANDOM_STRING
!#######################################################################
!#######################################################################
!****f* kmod_utilities/remove_gz
! NAME
! remove_gz
! SYNOPSIS
! temp_file=remove_gz(filename)
! FUNCTION
! Returns a character string with the extension '.gz' removed (if present)
! INPUTS
! * filename - char(len=*) - Filename
! RESULT
! * char(len=120) - string with extension '.gz' removed (if present)
! USES
! strjoin, strsplit
!***
  character(len=120) function remove_gz( infile )
    character(len=*), intent(in) :: infile
    !internal variables
    integer(kind=4) :: n
    character(len=120), allocatable, dimension(:) :: f

    !split the filename
    call strsplit(infile,'.',f,n)
    if (trim(f(n)) == 'gz') then 
       call strjoin(f(1:n-1),'.',remove_gz)
       return
    else
       remove_gz=infile
       return
    endif

  end function remove_gz
!#######################################################################
!#######################################################################
!                                                        BEGIN STRIPNAME
!****f* kmod_utilities/stripname
! NAME
! stripname
! SYNOPSIS
! file = stripname(full_filename)
! FUNCTION
! This strips the path from the filename
! INPUTS
! full_filename - char(len=*) - filename which may include a path
! OUTPUT
! file - filename with path removed
! USES
! strsplit
!***

  character(len=120) function stripname ( filename )
    !external variables
    character(len=*), intent(in) :: filename
    !internal variables
    integer(kind=4) :: n
    character(len=120), allocatable, dimension(:) :: output
    
!!$    n = scan(filename,'/',back=.true.)
!!$    if (n >= 0 .and. n <= len_trim(filename)) then
!!$       stripname = filename(n:len_trim(filename))
!!$    else
!!$       stripname = filename
!!$    endif
    ! the string 'bug/' makes sure at least one path is present
    call strsplit('bug/'//trim(filename),'/',output,n)
    stripname = output(n)
    return
  end function stripname
!                                                          END STRIPNAME
!#######################################################################
!#######################################################################
!                                                          BEGIN STRJOIN
!****r* kmod_utilities/strjoin
! NAME
! strjoin
! SYNOPSIS
! call strjoin(string,sep,output)
! FUNCTION
! Joins an array of characters using SEP
! INPUTS
! * string - char(len=*), rank1
! * sep    - char(len=1) 
! OUTPUT
! * output - char(len=*) - resulting joined string
! NOTES
! This is meant to mimic the IDL function STRJOIN (the inverse of STRSPLIT)
!***
  subroutine strjoin(array, sep, output)
    !Externals
    character(len=120), dimension(:) :: array
    character(len=*) :: sep
    character(len=*) :: output
    !Internals
    integer(kind=4) :: n, i

    n = size(Array)
    output = trim(adjustl(array(1)))
    if (n > 2) then
       do i=2,n-1
          output = trim(output)//sep//trim(adjustl(array(i)))
       end do
    end if
    output = trim(output)//sep//trim(adjustl(array(n)))
  end subroutine strjoin
!                                                            END STRJOIN
!#######################################################################
!#######################################################################
!                                                           BEGIN STRPOS
!****f* kmod_utilities/strpos
! NAME
! strpos
! SYNOPSIS
! loc = strpos(string,part)
! FUNCTION
! Find the location of part in string
! INPUTS
! * string - char(len=*)
! * part   - char(len=*) 
! OUTPUT
! * loc - int*4 - location of the first character where part = string
! NOTES
! This is meant to mimic the IDL function STRPOS
!***
 pure integer function strpos ( string, part )
    !external variables
    character(len=*), intent(in) :: string, part
    !internal
    integer :: lp !length of part
    integer :: ls !length of string to search
    integer :: i

    lp = len_trim(part)
    ls = len_trim(string)
    do i=1,ls
       if ((i+lp-1) <= ls) then
          if (string(i:i+lp-1) == part) then
             strpos = i
             return
          endif
       else
          strpos = -1
          return
       endif
    enddo
  end function strpos
!                                                             END STRPOS
!#######################################################################
!#######################################################################
!                                                         BEGIN STRSPLIT
!****r* kmod_utilities/strsplit
! NAME
! strsplit
! SYNOPSIS
! call strsplit(string,sep,output,nn)
! FUNCTION
! Separate the character STRING based on SEP into the OUTPUT variable.
! INPUTS
! * string - char(len=*)
! * sep    - char(len=1) - separator - e.g., '.', '-', ',', etc.
! OUTPUT
! * output - char(len=120), allocatable 
! * nn - number of elements in output
! NOTES
! This is meant to mimic the IDL function STRSPLIT
!***

  subroutine strsplit ( string, sep, output, nn ) !result ( output )
    !External variables
    character(len=*) :: string
    character(len=1) :: sep
    character(len=120), dimension(:), allocatable :: output
    integer(kind=4), intent(inout),optional ::  nn
    !Internal variables
    character(len=120) temp
    integer :: i, length, i0, j, k, n
    !Set up variables
    if (len_trim(string) > 120) print*,'Error in strsplit: String too long'
    temp = trim(string)
    length = len_trim(temp)
    !Find number of occurences of SEP
    j = 1
    n=0
    if (temp(1:1) == sep) j = 0
    do i=1,length
       if (temp(i:i) == sep) then
          if (j /= 0) then 
             n=n+1
             j = 0
          endif
       else
          j = 1
       endif
    enddo
    if (j == 1 .and. n > 0)  n=n+1
    if (n == 0) then
       n = 1
       if (allocated(output)) deallocate(output)
       allocate (output(1))
       output = trim(string)
       goto 1
    endif
    !Allocate the array
    if (allocated(output)) deallocate(output)
    allocate  (output(n))
    output = ' '
    i0 = 1
    do i=1,n
       !Start at first non SEP
       do k=i0,length
          if (temp(k:k) /= SEP) exit
       enddo
       i0 = k
       do j=i0+1,length
          if (temp(j:j) == sep) then
             output(i) = temp(i0:j-1)
             i0 = j
             exit
          endif
       enddo
       if ((j > length) .and. &
           (temp(length:length) /= sep)) output(i) = temp(i0:length)
    enddo
    
1   if (present(nn)) nn = n
  end subroutine strsplit
!                                                           END STRSPLIT
!#######################################################################
!#######################################################################
!                                                      BEGIN SWAP_ENDIAN
!****f* kmod_utilities/swap_endian
! NAME
! swap_endian
! SYNOPSIS
! out = swap_endian(in)
! FUNCTION
! Swaps the Endian-ness of a variable
! INPUTS
! in - a variable which could be:
! * int*2:  scalar, rank1, rank2, rank3
! * int*4:  scalar, rank1
! * real*4: scalar, rank1, rank2
! OUTPUT
! The byte-swapped output of same type and rank as IN
!***
  integer function swap_endian_kind2_scalar ( input )
    integer(kind=2), intent(in) :: input
    integer(kind=2), dimension(1) :: out
    out = swap_endian( (/input/) )
    swap_endian_kind2_scalar = out(1)
    return
  end function swap_endian_kind2_scalar

  function swap_endian_kind2_rank1 ( input ) result ( output )
    integer(kind=2), intent(in), dimension(:) :: input
    integer(kind=2), dimension(size(input))   :: output
    output = ishftc(input,-8)
    return
  end function swap_endian_kind2_rank1

  function swap_endian_kind2_rank2 ( input ) result ( output )
    integer(kind=2), intent(in), dimension(:,:) :: input
    integer(kind=2), dimension(size(input,1),size(input,2))   :: output
    output = ishftc(input,-8)
    return
  end function swap_endian_kind2_rank2

  function swap_endian_kind2_rank3 ( input ) result ( output )
    integer(kind=2), intent(in), dimension(:,:,:) :: input
    integer(kind=2), dimension(size(input,1),&
                               size(input,2),size(input,3))   :: output
    output = ishftc(input,-8)
    return
  end function swap_endian_kind2_rank3

  integer function swap_endian_kind4_scalar ( input )
    integer(kind=4), intent(in) :: input
    integer(kind=4), dimension(1) :: out
    out = swap_endian( (/input/) )
    swap_endian_kind4_scalar = out(1)
    return
  end function swap_endian_kind4_scalar

  function swap_endian_kind4_rank1 ( input ) result ( output )
    integer(kind=4), intent(in), dimension(:) :: input
    integer(kind=4), dimension(size(input))   :: output
    output = ishftc(ishftc(ishftc(input,-8),-8,24),-8,16)
    return
  end function swap_endian_kind4_rank1

  real function swap_endian_real4_scalar ( input )
    real(kind=4), intent(in) :: input
    real(kind=4), dimension(1) :: out
    out = swap_endian( (/input/) )
    swap_endian_real4_scalar = out(1)
    return
  end function swap_endian_real4_scalar

  function swap_endian_real4_rank1 ( input ) result ( output )
    real(kind=4), intent(in), dimension(:) :: input
    real(kind=4), dimension(size(input))   :: output
    integer(kind=4), dimension(size(input)) :: inta, intb
    inta = transfer(input,inta)
    intb = ishftc(ishftc(ishftc(inta,-8),-8,24),-8,16)
    output = transfer(intb,output)
    return
  end function swap_endian_real4_rank1

  function swap_endian_real4_rank2 ( input ) result ( output )
    real(kind=4), intent(in), dimension(:,:) :: input
    real(kind=4),    dimension(size(input,1),size(input,2)) :: output
    integer(kind=4), dimension(size(input)) :: inta, intb
    integer(kind=4) :: nx, ny, n
    n = size(input)
    nx = size(input,1)
    ny = size(input,2)
    inta = transfer(input,inta,n)
    intb = ishftc(ishftc(ishftc(inta,-8),-8,24),-8,16)
    output = reshape(transfer(intb,output,n),(/ nx, ny/) )
    return
  end function swap_endian_real4_rank2
!                                                        END SWAP_ENDIAN
!#######################################################################
!#######################################################################
!                                                           BEGIN WITHIN
!****f* kmod_utilities/within
! NAME
! within
! SYNOPSIS
! out = within(array,min,max)
! FUNCTION
! Forces variable array to have values within min and max.
! INPUTS
! * array - type can be any of:
!   *  int*4: scalar, rank1, rank2
!   *  int*2: rank2
!   * real*4: scalar, rank1, rank2
!   * real*8: scalar, rank1, rank2
! * min - scalar with same type as array - optional
! * max - scalar with same type as array - optional
! OUTPUT
! out - same rank and type as input, but now is within the range [min,max]
! NOTES
! * This routine mimics the following IDL function:
!       IDL> print,indgen(6)>1<4
!        1 1 2 3 4 4
!***
  pure function within_i2_scalar ( x, minimum, maximum) result (output)
    !External
    integer(kind=2), intent(in) :: x
    integer(kind=4), intent(in), optional :: minimum, maximum
    integer(kind=2) :: output
    output = x
    if (present(minimum)) then
       !output = max(x,minimum)
       if (x < minimum) output = minimum
    end if
    if (present(maximum)) then
       !output = min(x,maximum)
       if (x > maximum) output = maximum
    end if
  end function within_i2_scalar

  pure function within_i4_scalar ( x, minimum, maximum) result (output)
    !External
    integer(kind=4), intent(in) :: x
    integer(kind=4), intent(in), optional :: minimum, maximum
    integer(kind=4) :: output
    output = x
    if (present(minimum)) then
       !output = max(x,minimum)
       if (x < minimum) output = minimum
    end if
    if (present(maximum)) then
       !output = min(x,maximum)
       if (x > maximum) output = maximum
    end if
  end function within_i4_scalar

  pure function within_r4_scalar ( x, minimum, maximum) result (output)
    !External
    real(kind=4), intent(in) :: x
    real(kind=4), intent(in), optional :: minimum, maximum
    real(kind=4) :: output
    output = x
    if (present(minimum)) then
       if (x < minimum) output = minimum
    end if
    if (present(maximum)) then
       if (x > maximum) output = maximum
    end if
  end function within_r4_scalar

  pure function within_r8_scalar ( x, minimum, maximum) result (output)
    !External
    real(kind=8), intent(in) :: x
    real(kind=8), intent(in), optional :: minimum, maximum
    real(kind=8) :: output
    output = x
    if (present(minimum)) then
       if (x < minimum) output = minimum
    end if
    if (present(maximum)) then
       if (x > maximum) output = maximum
    end if
  end function within_r8_scalar

  pure function within_r4_rank1 ( x, minimum, maximum) result (output)
    !External
    real(kind=4), intent(in), dimension(:) :: x
    real(kind=4), intent(in), optional :: minimum, maximum
    real(kind=4), dimension(size(x)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_r4_rank1
  pure function within_i4_rank1 ( x, minimum, maximum) result (output)
    !External
    integer(kind=4), intent(in), dimension(:) :: x
    integer(kind=4), intent(in), optional :: minimum, maximum
    integer(kind=4), dimension(size(x)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_i4_rank1
  pure function within_r8_rank1 ( x, minimum, maximum) result (output)
    !External
    real(kind=8), intent(in), dimension(:) :: x
    real(kind=8), intent(in), optional :: minimum, maximum
    real(kind=8), dimension(size(x)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_r8_rank1
  pure function within_r4_rank2 ( x, minimum, maximum) result (output)
    !External
    real(kind=4), intent(in), dimension(:,:) :: x
    real(kind=4), intent(in), optional :: minimum, maximum
    real(kind=4), dimension(size(x,1),size(x,2)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_r4_rank2
  pure function within_r8_rank2 ( x, minimum, maximum) result (output)
    !External
    real(kind=8), intent(in), dimension(:,:) :: x
    real(kind=8), intent(in), optional :: minimum, maximum
    real(kind=8), dimension(size(x,1),size(x,2)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_r8_rank2

  pure function within_i2_rank2 ( x, minimum, maximum) result (output)
    !External
    integer(kind=2), intent(in), dimension(:,:) :: x
    integer(kind=2), intent(in), optional :: minimum, maximum
    integer(kind=2), dimension(size(x,1),size(x,2)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_i2_rank2

  pure function within_i4_rank2 ( x, minimum, maximum) result (output)
    !External
    integer(kind=4), intent(in), dimension(:,:) :: x
    integer(kind=4), intent(in), optional :: minimum, maximum
    integer(kind=4), dimension(size(x,1),size(x,2)) :: output
    output = x
    if (present(minimum)) then
       where (x < minimum)
          output = minimum
       end where
    end if
    if (present(maximum)) then
       where (x > maximum)
          output = maximum
       end where
    end if
  end function within_i4_rank2
!                                                             END WITHIN
!#######################################################################


end module kmod_utilities
