;****I* idl/idl.lmodel
; NAME
; lmodel
; 
; PURPOSE
; THIS SUBROUTINE COMPUTES THE POSITION OF THE SATELLITE AND THE            
; ATTITUDE OF THE IMAGER OR SOUNDER.  THE CALCULATIONS ARE BASED            
; ON THE OATS ORBIT AND ATTITUDE MODEL REPRESENTED BY THE O&A               
; PARAMETER SET IN GVAR BLOCK 0.
; 
; USAGE
; INPUT
;          TIME, EPOCH TIME, O&A PARAMETER SET, IMC STATUS.
; OUTPUT
;          THE SPACECRAFT POSITION VECTOR IN EARTH FIXED COORDINATES
;          THE GEOMETRIC ROLL, PITCH, YAW ANGLES AND THE ROLL,
;          PITCH MISALIGNMENTS FOR EITHER THE IMAGER OR THE SOUNDER
;          THE EARTH FIXED TO INSTRUMENT FRAME TRANSFORMATION MATRIX
;          GEOGRAPHIC LATITUDE AND LONGITUDE AT SUBSATELLITE POINT.      
; RESULT
; EXAMPLE
; USES
; * elcons.pro
; * elcomm.pro
; * atan2.pro
; * gatt.pro
; * inst2er.pro
; 
; USED BY
; * b1u_imc.pro
; 
; 
; NOTES
;          LMODEL ACCEPTS AN INPUT DOUBLE PRECISION TIME IN MINUTES FROM
;          1950, JAN.1.0 AND AN INPUT SET OF O&A PARAMETERS AND COMPUTES
;          POSITION OF THE SATELLITE, THE ATTITUDE ANGLES AND ATTITUDE
;          MISALIGNMENTS AND THE INSTRUMENT TO EARTH FIXED COORDINATES
;          TRANSFORMATION MATRIX.
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; CREATION DATE
; 
; HISTORY
; SOURCE

pro lmodel,t,tu,rec,imc,rlat,rlon


@elcons.pro
@elcomm.pro

;ASSIGN REFERENCE VALUES TO THE SUBSATELLITE LONGITUDE AND
;LATITUDE, THE RADIAL DISTANCE AND THE ORBIT YAW.

LAM=REC(5-1) 
DR=REC(6-1)
PHI=REC(7-1)
PSI=REC(8-1)

;ASSIGN REFERENCE VALUES TO THE ATTITUDES AND MISALIGNMENTS
ROLL =REC(9-1)
PITCH=REC(10-1)
YAW  =REC(11-1)
RMA  =0.0D0
PMA  =0.0D0

;IF IMC IS OFF, COMPUTE CHANGES IN THE SATELLITE ORBIT                     
IF (IMC NE 0) THEN begin
    ;SET REFERENCE RADIAL DISTANCE, LATITUDE AND ORBIT YAW TO ZERO
    DR=0.0D0
    PHI=0.0D0
    PSI=0.0D0
    ;COMPUTE TIME SINCE EPOCH (IN MINUTES)                                     
    TS=T-TU                                                                   
    ;COMPUTES ORBIT ANGLE AND THE RELATED TRIGONOMETRIC FUNCTIONS.             
    ;EARTH ROTATIONAL RATE=.729115E-4 (RAD/S)                                  
    W=0.729115D-4*60.0D0*TS                                                   
    SW=SIN(W)
    CW=COS(W)
    SW1=SIN(0.927D0*W)
    CW1=COS(0.927D0*W)
    S2W=SIN(2.0D0*W)
    C2W=COS(2.0D0*W)
    SW3=SIN(1.9268D0*W)
    CW3=COS(1.9268D0*W)
    ;COMPUTES CHANGE IN THE IMC LONGITUDE FROM THE REFERENCE
    LAM=LAM+REC(18-1)+(REC(19-1)+REC(20-1)*W)*W $
      +(REC(27-1)*SW1+REC(28-1)*CW1+REC(21-1)*SW+REC(22-1)*CW $
        +REC(23-1)*S2W+REC(24-1)*C2W + REC(25-1)*SW3+REC(26-1)*CW3 $
        +W*(REC(29-1)*SW+REC(30-1)*CW))*2.0D0
    ;COMPUTES CHANGE IN RADIAL DISTANCE FROM THE REFERENCE (KM)
    DR=DR + REC(31-1) + REC(32-1)*CW+REC(33-1)*SW $
      +REC(34-1)*C2W+REC(35-1)*S2W + REC(36-1)*CW3+REC(37-1)*SW3 $
      +REC(38-1)*CW1+REC(39-1)*SW1 + W*(REC(40-1)*CW+REC(41-1)*SW)
    ;COMPUTES THE SINE OF THE CHANGE IN THE GEOCENTRIC LATITUDE                
    DLAT=REC(42-1) + REC(43-1)*CW+REC(44-1)*SW $
      +REC(45-1)*C2W+REC(46-1)*S2W $
      +W*(REC(47-1)*CW+REC(48-1)*SW) $
      +REC(49-1)*CW1+REC(50-1)*SW1
    ;COMPUTES GEOCENTRIC LATITUDE BY USING AN EXPANSION FOR ARCSINE
    PHI=PHI+DLAT*(1.0D0+DLAT*DLAT/6.0D0)
    ;COMPUTES SINE OF THE CHANGE IN THE ORBIT YAW                              
    DYAW=REC(51-1) + REC(52-1)*SW+REC(53-1)*CW $
      +REC(54-1)*S2W+REC(55-1)*C2W $
      +W*(REC(56-1)*SW+REC(57-1)*CW) $
      +REC(58-1)*SW1+REC(59-1)*CW1                                          
    ;COMPUTES THE ORBIT YAW BY USING AN EXPANSION FOR ARCSINE.                 
    PSI=PSI+DYAW*(1.0D0+DYAW*DYAW/6.0D0)
    ;CALCULATION OF CHANGES IN THE SATELLITE ORBIT ENDS HERE                   
ENDIF                                                                    

;CONVERSION OF THE IMC LONGITUDE AND ORBIT YAW TO THE SUBSATELLITE         
;LONGITUDE AND THE ORBIT INCLINATION (REF: GOES-PCC-TM-2473, INPUTS        
;REQUIRED FOR EARTH LOCATION AND GRIDDING BY SPS,  JUNE 6, 1988)           
SLAT=SIN(PHI)
SYAW=SIN(PSI)
SI=SLAT^2+SYAW^2                                                        
CI=SQRT(1.0D0-SI)
SI=SQRT(SI)
IF (SLAT EQ 0.0D0 AND SYAW  EQ  0.0D0) THEN begin
    U=0.0D0
endif ELSE begin
    U=ATAN2(SLAT,SYAW)
ENDelse
SU=SIN(U)
CU=COS(U)
;COMPUTES LONGITUDE OF THE ASCENDING NODE
ASC=LAM - U
SA=SIN(ASC)
CA=COS(ASC)
;COMPUTES THE SUBSATELLITE GEOGRAPHIC LATITUDE                             
RLAT=ATAN(AEBE2*TAN(PHI))
;COMPUTES THE SUBSATELLITE LONGITUDE                                       
RLON=ASC+ATAN2(CI*SU,CU)
;COMPUTES THE SPACECRAFT TO EARTH FIXED COORDINATES TRANSFORMATION         
;MATRIX:                                                                   
;(VECTOR IN ECEF COORDINATES) = B * (VECTOR IN S/C COORDINATES)
b = dblarr(3,3)
B(1-1,2-1)=-SA*SI                                                             
B(2-1,2-1)= CA*SI                                                             
B(3-1,2-1)=-CI                                                                
B(1-1,3-1)=-CA*CU+SA*SU*CI                                                    
B(2-1,3-1)=-SA*CU-CA*SU*CI                                                    
B(3-1,3-1)=-SLAT                                                              
B(1-1,1-1)=-CA*SU-SA*CU*CI                                                    
B(2-1,1-1)=-SA*SU+CA*CU*CI                                                    
B(3-1,1-1)= CU*SI                                                             
;COMPUTES THE NORMALIZED SPACECRAFT POSITION VECTOR IN EARTH FIXED
;COORDINATES - XS.
R=(NOMORB+DR)/AE
xs = dblarr(3)
XS(1-1)=-B(1-1,3-1)*R
XS(2-1)=-B(2-1,3-1)*R
XS(3-1)=-B(3-1,3-1)*R

;PRECOMPUTES Q3 (USED IN LPOINT)                                           
Q3=XS(1-1)^2+XS(2-1)^2+AEBE2*XS(3-1)^2-1.0D0
;COMPUTES THE ATTITUDES AND MISALIGNMENTS IF IMC IS OFF                    
IF (IMC NE 0) THEN begin
    ;COMPUTES THE SOLAR ORBIT ANGLE                                            
    WA=REC(60-1)*TS     
    ;COMPUTES THE DIFFERENCE BETWEEN CURRENT TIME, TS, AND THE
    ;EXPONENTIAL TIME, REC(61). NOTE THAT BOTH TIMES ARE SINCE EPOCH.
    TE=TS-REC(61-1)                                                          
    ;COMPUTES ROLL + ROLL MISALIGNMENT                                         
    ROLL=ROLL+GATT(62,REC,WA,TE)                                           
    ;COMPUTES PITCH + PITCH MISALIGNMENT                                       
    PITCH=PITCH+GATT(117,REC,WA,TE)                                        
    ;COMPUTES YAW                                                              
    YAW=YAW+GATT(172,REC,WA,TE)                                            
    ;COMPUTES ROLL MISALIGNMENT                                                
    RMA=GATT(227,REC,WA,TE)                                                
    ;COMPUTES PITCH MISALIGNMENT                                               
    PMA=GATT(282,REC,WA,TE)                                                
    ;APPLY THE SPCECRAFT COMPENSATION
    ROLL =ROLL +REC(15-1)
    PITCH=PITCH+REC(16-1)
    YAW  =YAW  +REC(17-1)
ENDIF                                                                    
;COMPUTES THE INSTRUMENT TO EARTH FIXED COORDINATES TRANSFORMATION
;MATRIX - BT
INST2ER,ROLL,PITCH,YAW,B,BT

end
;***
