;****I* idl/idl.lr
; NAME
; lr
; 
; PURPOSE
; To produce linear regression statistics on a paired set as input.
; 
; USAGE
; 
; INPUT
; x (array) - dependent set of input data
; y (Array) - independent set of input data
; 
; KEYWORDS
; Keyword inputs:
; * FILTER - binary array: 0 = do not use, 1 = use
; * error  - input - estimate of error for each point
; * confidence - (0-1) confidence limit (DEFAULT=0.99)
; 
; OUTPUT
; Keyword outputs:
; * newx   - output - which values were used
; * newy   - output - which values were used
; * slopesd - std. devation of slope
; 
; 
; RESULT
; - float array having 11 elements e.g., output: array = lr(x,y)
; All values are based on simple linear least square regression (using
; the IDL routine regress)
; 
; array(0) = # of points
; array(1) = correlation coefficient
; array(2) = slope of linear regression
; array(3) = offset (intercept) of linear regression
; array(4) = Goodness of fit [stdev(yfit-y)]
; array(5) = root mean square (rms) difference
; array(6) = bias difference [mean(x-y)]
; array(7) = Chi squared parameter
; array(8) = sigma (?)
; array(9) = Slope Error (confidence interval at DEFAULT=99%)
; array(10) = Intercept error (confidence interval at DEFAULT=99%)
; 
; EXAMPLE
; IDL> st = lr(x,y)
; 
; USES
; * stdev.pro
; * mean.pro
; 
; USED BY
; * b1u_dn2cal.pro
; * cal_b1u_monthly.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; ~2000
; 
; HISTORY
; 2010-08 KRK - Added robodoc header
; 
; SOURCE

function lr,x,y, $
            filter=filter, $
            newx=newx, $
            newy=newy, $
            error=error, $
            slopesd=slopesd, $     ;std. dev. of regression slope
            confidence=confidence
;***
nx = n_elements(x)
ny = n_elements(y)
if (nx ne ny) then begin
    print,'X and Y arrays must have same length'
    lr = -1
    return,lr
endif

if keyword_set(error) then begin
    inerror = error
endif else begin
    inerror = fltarr(nx)
    inerror(*) = 1.
endelse

if not(keyword_set(filter)) then begin
    filter = fltarr(nx)
    filter(*) = 1.
endif
filteri=fix(filter)
filteri = filteri < 1
filteri=reform(filteri)

np = total(filter)
if (np lt 3) then begin
    lr1 = fltarr(9)
    lr1(*) = 0.
    return,lr1
endif
newx = fltarr(np)
newy=fltarr(np)
wgt = fltarr(np)
wgt(*) = 1.
c=0l
xi=reform(x)
yi=reform(y)
for i=0l,nx-1 do begin
    newx(c) = xi(i)
    newy(c) = yi(i)
    c = c + filteri(i)
    if (c eq np) then break
endfor

result = regress(reform(newx,1,np),newy,chisq=chisq,const=const,$
                 correlation=r, sigma=sigma, yfit=yfit, measure = error)

xbar = mean(newx)
ybar = mean(newy)

sb = stdev(yfit-newy) / sqrt(total((newx-xbar)^2))

slopesd = sb

cl = 0.99 ; Confidence limit
if keyword_set(confidence) then cl=confidence

p = (1.0-cl)/2.                 ;divide by 2 b/c a 2 tail test is required
t = t_cvf(p,np-2)
slperr = sb * t
interr = abs(const - (ybar - (result(0)+slperr)*xbar))


lr1=fltarr(11)
lr1(0) = np
lr1(1) = r(0)
lr1(2) = result(0)
lr1(3) = const
lr1(4) = stdev(yfit-newy)
lr1(5) = sqrt(mean((newx-newy)*(newx-newy)))
lr1(6) = mean(newx-newy)
lr1(7) = chisq
lr1(8) = sigma
lr1(9) = slperr
lr1(10) = interr
lr=lr1
return,lr
end
