;****I* mcidas/idl.mcidas_calibrate
; NAME
; mcidas_calibrate
; 
; PURPOSE
; Calibrate McIDAS imagery read by mcidas_read.pro
; 
; INPUTS
; All variables are input and output through keywords:
; AREAstr - McIDAS AREA file directory header
; images  - Image (as provided from mcidas_read.pro)
; LINhdr  - Scan line prefix header 
;
; OUTPUT
; radiance- Radiances determined from the IMAGES input
; temperature- Temperature determined from the IMAGES input.
; 
; OPTIONS
; The radiance output is optional.
;
; NOTES
; This routine was coded up from information freely available on the
; world wide web. References to web sites are provided in the code
; documentation.
;
; This code will need an update to work with satellites other than
; those listed below:
; * GOES Imagers (GOES-8,9,10,11,12,13)
;
; Last Updated June 23, 2006
;
; USED BY
; * b1read_aes_g8.pro
; * b1read_csu_g91012.pro
; * b1read.pro
; 
; USES
; * Nothing
; 
; TO DO
; * Sounder calibration routines
; * Mode-A calibration
; * Multi-channel imagery (i.e., images=intarr(nchan,nx,ny)
;
; SOURCE

pro mcidas_calibrate,AREAstr=AREAstr,        $
                     images=images,          $
                     LINhdr=LINhdr,          $
                     radiance=radiance,      $
                     temperature=temperature, $
                     tableonly=tableonly, $
                     channel=channel, $
                     debug=debug, $
                     goessat=goessat

;constants
;c1 & c2 from http://www.oso.noaa.gov/goes/goes-calibration/contents/page19.htm
c1 = 1.191066e-5              ;mW/(m2-sr-cm-4)
c2 = 1.438833                   ;K/cm-1
;select satellite from area header

;from
;http://www.ssec.wisc.edu/mcidas/doc/misc_doc/area2.html
;This routine can not calibrate each of the following images, but it
;can at least recoginize the satellites and tell you whether it can
;calibrate it or not.
case AREAstr.SAT_ID_NUM of
    4 : sat ='PDUS METEOSAT Visible'
    5 : sat ='PDUS METEOSAT Infrared'
    6 : sat ='PDUS METEOSAT Water Vapor'
    7 : sat ='Radar'
    8 : sat ='Miscellaneous Aircraft Data (MAMS)'
    9 : sat ='Raw METEOSAT'
    12: sat ='GMS Visible prior to GMS-5'
    13: sat ='GMS Infrared prior to GMS-5'
    14: sat ='ATS 6 Visible'
    15: sat ='ATS 6 Infrared'            
    16: sat ='SMS-1 Visible'          
    17: sat ='SMS-1 Infrared'            
    18: sat ='SMS-2 Visible'            
    19: sat ='SMS-2 Infrared'            
    20: sat ='GOES-1 Visible'            
    21: sat ='GOES-1 Infrared'           
    22: sat ='GOES-2 Visible'            
    23: sat ='GOES-2 Infrared'           
    24: sat ='GOES-3 Visible'            
    25: sat ='GOES-3 Infrared'           
    26: sat ='GOES-4 Visible-VAS'         
    27: sat ='GOES-4 Infrared-VAS' 
    28: sat ='GOES-5 Visible'            
    29: sat ='GOES-5 Infrared-VAS' 
    30: sat ='GOES-6 Visible'            
    31: sat ='GOES-6 Infrared'           
    32: sat ='GOES-7 Visible'            
    33: sat ='GOES-7 Infrared'           
    41: sat ='TIROS-N POES'               
    42: sat ='NOAA-6'                
    43: sat ='NOAA-7'                
    44: sat ='OAA-8'                
    45: sat ='NOAA-9'
    54: sat ='METEOSAT-3'           
    55: sat ='METEOSAT-4'
    56: sat ='METEOSAT-5'
    57: sat ='METEOSAT-6'
    60: sat ='NOAA-10'
    61: sat ='NOAA-11'
    62: sat ='NOAA-12'
    63: sat ='NOAA-13'
    64: sat ='NOAA-14'
    70: sat ='GOES-8 Imager'
    71: sat ='GOES-8 Sounder'
    72: sat ='GOES-9 Imager'
    73: sat ='GOES-9 Sounder'
    74: sat ='GOES-10 Imager'
    75: sat ='GOES-10 Sounder'
    76: sat ='GOES-11 Imager'
    77: sat ='GOES-11 Sounder'
    78: sat ='GOES-12 Imager'
    79: sat ='GOES-12 Sounder'
    80: sat ='ERBE'
    82: sat ='GMS-4'
    83: sat ='GMS-5'
    84: sat ='GMS-6'
    85: sat ='GMS-7'
    87: sat ='DMSP F-8'
    88: sat ='DMSP F-9'
    89: sat ='DMSP F-10'
    90: sat ='DMSP F-11'
    91: sat ='DMSP F-12'
    95: sat ='FY-1b'                
    96: sat ='FY-1c'                
    97: sat ='FY-1d'  
    180: sat = 'GOES-13 Imager'  
    181: sat = 'GOES-13 Sounder'
    182: sat = 'GOES-14 Imager'
    183: sat = 'GOES-14 Sounder'
    184: sat = 'GOES-15 Imager'
    185: sat = 'GOES-15 Sounder'
    else: sat='Unknown'
endcase
if (sat eq 'Unknown') then begin
    Print,'Calibration not available'
    Print,'Unknown satellite ID number: '+strtrim(AREAstr.sat_id_num,2)
    if keyword_set(debug) then message,'Error'
    return
endif

if (AREAstr.num_chan gt 1) then begin
    message,'This routine is not currently set up to calibrate multi-channel McIDAS files',/info
endif

nx = AREAstr.num_elem
ny = AREAstr.num_line

s=size(images)
nx = s(1)
ny = s(2)

;The following is an attempt to calibrate as many McIDAS files as
;necessary. 
;
;Initially, this will only include  Geostationary data available from CLASS.
if (strmid(sat,0,4) eq 'GOES') then begin
    g1=strsplit(sat,' ',/extr)
    g2=strsplit(g1(0),'-',/extr)
    goes_num=fix(g2(1))

    if keyword_set(goessat) then goes_num = goessat

    if (goes_num ge 8 and goes_num le 15) then begin
        if (AREAstr.bytes_per_pixel ne 2 and not(keyword_set(tableonly))) then begin
            message,'This routine is not currently set up for 1-byte GOES-GVAR McIDAS files',/info
            if keyword_set(debug) then Message,'Error'
            return
        endif
        ;Calibrate GVAR files for GOES.
        ;details from:
        ;http://www.oso.noaa.gov/goes/goes-calibration/contents/page19.htm
        
        ;determine the channel number
        if (g1(1) eq 'Imager') then begin
            ;get channel number
            if keyword_set(tableonly) then begin
                if not(keyword_set(channel)) then Message,'Error'
                chan = channel
            endif else begin
                CHAN =linhdr(0).doc.lindoc.CHAN
                if (chan lt 2 or chan gt 6) then begin
                    message,'Channel number not identified:'+strtrim(chan,2),/info
                    if keyword_set(debug) then Message,'Error'
                    return
                endif
            endelse
            ;set up Detector ID mask
            if keyword_set(tableonly) then begin
                nbin=256
                if (areastr.bytes_per_pixel eq 2) then nbin=1024
                images=indgen(nbin)
                if (areastr.bytes_per_pixel eq 0) then images=images*4
                det=intarr(nbin)

                ;set detector based on input channel
                ichan = channel
                if (goes_num le 11) then begin
                    ;set detectors based on initial series
                    case CHANNEL of
                        2: det(*) = 5
                        3: det(*) = 7
                        4: det(*) = 1
                        5: det(*) = 3
                        else: Message,'Error: Channel number does not match satellite'
                    endcase
                endif else begin
                    ;set detectors based on second series
                    case CHANNEL of
                        2: det(*) = 1
                        3: det(*) = 3
                        4: det(*) = 5
                        6: det(*) = 7
                        else: Message,'Error: Channel number does not match satellite'
                    endcase 
                endelse
            endif else begin
                det=rebin(reform(linhdr.doc.lindoc.lidet,1,ny),nx,ny)
            endelse

            ;get detector ID
            ;values range from 1-8 for VISIBLE
            ;                  1-7 for IR detectors on GOES-8-13
            ;                  1-8 for IR detectors on GOES-14-15
            ; GOES-8-11
            ;LIDET   GOES_CHANNEL DETECTOR
            ;  1         4/a         1
            ;  2         4/b         2
            ;  3         5/a         1
            ;  4         5/b         2
            ;  5         2/a         1
            ;  6         2/b         2
            ;  7         3           1
            ; GOES-12-15 (GOES-O and beyond???)
            ;LIDET   GOES_CHANNEL DETECTOR
            ;  1         2/a         1
            ;  2         2/b         2
            ;  3         3/a         1
            ;  4         3/b         2
            ;  5         4/a         1
            ;  6         4/b         2
            ;  7         6/a         1
            ;  8         6/b         2

            ;GOES-12/13 from http://www.oso.noaa.gov/goes/goes-calibration/gvar-conversion.htm

            ;Slope:  UNITS= COUNTS/(mW/[m^2 ster sm^-1])
            m = [ -1., 227.3889, 38.8383,  5.2285,  5.0273,  5.5297]
            ;Offset: UNITS= COUNTS
            b = [ -1.,  68.2167, 29.1287, 15.6854, 15.3332, 16.5892]

            RADIANCE = (IMAGES - b(chan-1)) / m(chan-1)
            ;remove the possibility of negative radiances
            ;this is only done to calculate temperatures
            Radiance = Radiance> 1.e-10

            ;loop calibration to temperature for each detector
            sort_det=sort(det)
            uniq_det = uniq(det(sort(det)))
            nuniq = n_elements(uniq_det)
            temperature = radiance*0.
            for idet=0,nuniq-1 do begin
                LIDET=det(sort_det(uniq_det(idet)))
                mask_det = where(det eq LIDET)
            if (goes_num eq 8) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_1.htm
                if            (LIDET eq 5) then begin ;2/a
                    row = [2556.71,      -0.618007,       1.001825 ,    -6.021442e-07]
                endif else if (LIDET eq 6) then begin ;2/b
                    row = [2558.62,      -0.668648,       1.002221 ,    -1.323758e-06]
                endif else if (LIDET eq 7) then begin ;3
                    row = [1481.91,      -0.656443,       1.001914 ,    -9.535221e-07]
                endif else if (LIDET eq 1) then begin ;4/a
                    row = [934.30,      -0.519333 ,      1.002834  ,   -3.005194e-06]
                endif else if (LIDET eq 2) then begin ;4/b
                    row = [935.38,      -0.553431 ,      1.002894  ,   -3.077855e-06]
                endif else if (LIDET eq 3) then begin ;5/a
                    row = [837.06,      -0.383077 ,      1.000856  ,    6.026892e-07]
                endif else if (LIDET eq 4) then begin ;5/b
                    row = [837.00,      -0.351510 ,      1.000340  ,    1.761416e-06]
                endif
            endif else if (goes_num eq 9) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_2.htm
                if            (LIDET eq 5) then begin ;2/a
                    row = [2555.18,      -0.592268,       1.001040,     -1.882973e-07]
                endif else if (LIDET eq 6) then begin ;2/b
                    row = [2555.18,      -0.592268,       1.001040,     -1.882973e-07]
                endif else if (LIDET eq 7) then begin ;3
                    row = [1481.82,      -0.559306,      1.001602,     -1.010812e-06]
                endif else if (LIDET eq 1) then begin ;4/a
                    row = [934.59,      -0.525515 ,     1.002411,     -2.148433e-06]
                endif else if (LIDET eq 2) then begin ;4/b
                    row = [934.28,      -0.532929 ,     1.002616,     -2.584012e-06]
                endif else if (LIDET eq 3) then begin ;5/a
                    row = [834.02,      -0.317704 ,     1.001058,     -2.245684e-07]
                endif else if (LIDET eq 4) then begin ;5/b
                    row = [834.09,      -0.346344 ,     1.001261,     -6.031501e-07]
                endif
            endif else if (goes_num eq 10) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_3.htm
                if            (LIDET eq 5) then begin ;2/a
                    row = [2552.9845,     -0.63343694,      1.0013206,   -4.2038547e-07]
                endif else if (LIDET eq 6) then begin ;2/b
                    row = [2552.9845,     -0.63343694,      1.0013206,   -4.2038547e-07]
                endif else if (LIDET eq 7) then begin ;3
                    row = [1486.2212,     -0.66500842,      1.0017857,   -7.3885254e-07]
                endif else if (LIDET eq 1) then begin ;4/a
                    row = [936.10260,     -0.36939128,      1.0017466,   -1.4981835e-06]
                endif else if (LIDET eq 2) then begin ;4/b
                    row = [935.98981,     -0.41013697,      1.0020766,   -2.1303556e-06]
                endif else if (LIDET eq 3) then begin ;5/a
                    row = [830.88473,     -0.32763317,      1.0014057,   -9.5563444e-07]
                endif else if (LIDET eq 4) then begin ;5/b
                    row = [830.89691,     -0.32184480,      1.0013828,   -9.3581045e-07]
                endif
            endif else if (goes_num eq 11) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_4.htm
                if            (LIDET eq 5) then begin ;2/a
                    row = [2562.07,      -0.651377,      1.000828,     -1.002675e-07]
                endif else if (LIDET eq 6) then begin ;2/b
                    row = [2562.07,      -0.651377,      1.000828,     -1.002675e-07]
                endif else if (LIDET eq 7) then begin ;3
                    row = [1481.53,      -0.620175,      1.002104,     -1.171163e-06]
                endif else if (LIDET eq 1) then begin ;4/a
                    row = [931.76,      -0.546157 ,     1.003175 ,    -3.656323e-06]
                endif else if (LIDET eq 2) then begin ;4/b
                    row = [931.76 ,     -0.546157 ,     1.003175 ,    -3.656323e-06]
                endif else if (LIDET eq 3) then begin ;5/a
                    row = [833.67,      -0.329940 ,     1.000974 ,     5.000439e-08]
                endif else if (LIDET eq 4) then begin ;5/b
                    row = [833.04,      -0.307032 ,     1.000903 ,     1.233306e-07]
                endif
            endif else if (goes_num eq 12) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_5.htm
                if            (LIDET eq 1) then begin ;2/a
                    row = [2562.45,      -0.727744 ,     1.002131,     -1.173898e-06]
                endif else if (LIDET eq 2) then begin ;2/b
                    row = [2562.45,      -0.727744 ,     1.002131,     -1.173898e-06]
                endif else if (LIDET eq 3) then begin ;3/a
                    row = [1536.43,      -5.278975 ,     1.016476,     -7.754348e-06]
                endif else if (LIDET eq 4) then begin ;3/b
                    row = [1536.95,      -5.280110 ,     1.016383,     -7.607900e-06]
                endif else if (LIDET eq 5) then begin ;4/a
                    row = [933.21,      -0.534982  ,    1.002693 ,    -2.667092e-06]
                endif else if (LIDET eq 6) then begin ;4/b
                    row = [933.21,      -0.534982  ,    1.002693 ,    -2.667092e-06]
                endif else if (LIDET eq 7) then begin ;6
                    row = [751.91,      -0.177244  ,    1.000138,      1.163496e-06]
                endif
            endif else if (goes_num eq 13) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_6.htm
                if            (LIDET eq 1) then begin ;2/a
                    row = [2561.7421,      -1.4755462,      1.0028656, -5.8203946e-07]
                endif else if (LIDET eq 2) then begin ;2/b
                    row = [2561.7421,      -1.4755462,      1.0028656, -5.8203946e-07]
                endif else if (LIDET eq 3) then begin ;3/a
                    row = [1522.5182,      -4.1556932,      1.0142082, -8.0255086e-06]
                endif else if (LIDET eq 4) then begin ;3/b
                    row = [1521.6645,      -4.1411143,      1.0142255, -8.0755893e-06]
                endif else if (LIDET eq 5) then begin ;4/a
                    row = [937.23449,     -0.52227011,      1.0023802, -2.0798856e-06]
                endif else if (LIDET eq 6) then begin ;4/b
                    row = [937.27498,     -0.51783545,      1.0023789, -2.1027609e-06]
                endif else if (LIDET eq 7) then begin ;6
                    row = [749.82589,     -0.16089410,      1.0006896, -3.9853774e-07]
                endif
            endif else if (goes_num eq 14) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/gvar-conversion.htm
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_7.htm
                if            (LIDET eq 1) then begin ;2/a
                    row = [2577.98,      -1.624786,       1.002851,      -4.220156e-07]
                endif else if (LIDET eq 2) then begin ;2/b
                    row = [2577.98,      -1.624786,      1.002851,      -4.220156e-07]
                endif else if (LIDET eq 3) then begin ;3/a
                    row = [1529.35,      -4.068539,       1.013407,      -7.391130e-06]
                endif else if (LIDET eq 4) then begin ;3/b
                    row = [1530.13,      -4.071805,       1.013250,      -7.198898e-06]
                endif else if (LIDET eq 5) then begin ;4/a
                    row = [936.20,      -0.539978,       1.003263,      -3.856398e-06]
                endif else if (LIDET eq 6) then begin ;4/b
                    row = [936.14,      -0.543131,       1.003284,      -3.884668e-06]
                endif else if (LIDET eq 7) then begin ;6/a
                    row = [753.30,      -0.165606,       1.000381,       4.307351e-07]
                endif else if (LIDET eq 8) then begin ;6/b
                    row = [753.84,      -0.170905,       1.000217,       8.971296e-07]
                endif
            endif else if (goes_num eq 15) then begin
;http://www.oso.noaa.gov/goes/goes-calibration/gvar-conversion.htm
;http://www.oso.noaa.gov/goes/goes-calibration/docs/table3_8.htm
                if            (LIDET eq 1) then begin ;2/a
                    row = [2560.75,      -1.661727      , 1.002864,      -4.322291e-07]
                endif else if (LIDET eq 2) then begin ;2/b
                    row = [2560.75 ,     -1.661727     ,  1.002864,      -4.322291e-07]
                endif else if (LIDET eq 3) then begin ;3/a
                    row = [1538.62  ,    -3.641359    ,   1.012076,      -6.791749e-06]
                endif else if (LIDET eq 4) then begin ;3/b
                    row = [1538.66   ,   -3.637214   ,    1.012033,      -6.748332e-06]
                endif else if (LIDET eq 5) then begin ;4/a
                    row = [935.09     , -0.518513   ,    1.002645 ,     -2.674370e-06]
                endif else if (LIDET eq 6) then begin ;4/b
                    row = [934.89      ,-0.515443  ,     1.002754 ,     -2.914254e-06]
                endif else if (LIDET eq 7) then begin ;6/a
                    row = [752.91      ,-0.164346 ,      1.000240  ,     7.850444e-07]
                endif else if (LIDET eq 8) then begin ;6/b
                    row = [752.76      ,-0.162255,       1.000288   ,    6.448548e-07]
                endif
            endif else begin
                MESSAGE,'Coefficients (n, a, b) for GOES-'+strtrim(goes_num,2)+ $
                        'not currently available',/info
                if keyword_set(debug) then stop
                return
            endelse
            if (lidet gt 0) then begin
                wvn = row(0)
                a   = row(1)
                b   = row(2)
                g   = row(3)
                Teff = (c2*wvn)/alog(1.0 + (c1*wvn*wvn*wvn)/RADIANCE(mask_det))
                Temperature(mask_det) = a + b*Teff + g*(Teff*Teff)
            endif
            endfor                  ;iterate this loop once per detector
        endif else if (g1(1) eq 'Sounder') then begin
            Message,'Sounder calibration not fully tested.',/info
            if keyword_set(debug) then stop
            return

            M = [528.9773, 540.0049, 485.6243, 394.5752, 357.8019, $
                 334.1747, 311.5226, 314.6032, 434.3518, 1126.224, $
                 1899.565, 2874.342, 9642.747, 14105.38, 26221.34, $
                 10720.60, 12136.11, 19358.13, -1.0]
            B = [1745.625, 1566.014, 1311.186, 887.7943, 787.1643, $
                 417.7184, 249.2180, 251.6826, 716.6805, 900.9795, $
                 1139.739, 2155.757, 626.7785, 916.8496, 1704.387, $
                 428.8239, 497.5806, 348.4463]
            RADIANCE = (IMAGES - b(chan-1)) / m(chan-1)
            ;Teff = (c2*n)/ln(1 + [c1*n^3]/R)
            ;T = b*Teff + a

        endif else begin 
            Message,'Unknown GOES sensor:'+g1(1),/info
            if keyword_set(debug) then stop
            return
        endelse 
    endif else if (goes_num lt 8) then begin
        ;Calibrate pre GOES-8 data (i.e., the mode data)
    endif else begin 
        message,'The GOES satellite number is not recognized.',/info
        message,'Satellite:'+sat,/info
        message,'If this is a new satellite, the routine should be '+ $
                'updated with new coefficients.',/info
    endelse 
endif else begin
    Message,'The following satellite:',/info
    Message,'Satellite ID: '+strtrim(AREAstr.sat_id_num,2),/info
    Message,'Satellite   : '+sat,/info
    Message,'can''t be calibrated by the routine as compiled.',/info
endelse
;if keyword_set(debug) then stop
end
;***
