;****I* mcidas/idl.mcidas_read
;
;+
; NAME
;    mcidas_read
;
; WARNING
;    THIS ROUTINE HAS NOT BEEN FULLY TESTED. THE PARAMETERS RETURNED
;    SHOULD BE CHECKED FOR ACCURACY AND CONSISTANCY. IT HAS  BEEN
;    TESTED AGAINST A SMALL SAMPLE OF FILES OBTAINED FROM CLASS
;    (http://www.class.noaa.gov or http://www.class.ncdc.noaa.gov)
; 
; PURPOSE
;    To read in the mcidas GOES area data. This routine is based on
;    the ISCCP B1 access to CSU GOES-9/10/12 data. The routine ONLY
;    reads the mcidas data. Calibration and navigation are entirely
;    different algorithms. Navigation requires converting lat/lon
;    values to line/pixels (or vice versa). Information to do this is
;    stored in the AREAstr and NAVstr structures. Calibration involves
;    converting images counts [from the IMAGES variable] to radiance
;    units. Information to do this is available online.
;
; INPUTS
; All input/output are keyword parameters.
; * file = filename of mcidas area file [if not provided, then the
; * dialog_pickfile will prompt user to select a file]
;
; OUTPUT
; * AREAstr:Mcidas area header
; * NAVstr: Mcidas Navigation header
; * IMAGES: The image data. The return is ARRAY[NCHAN,NX,NY]
; * LINhdr: McIDAS scan line prefix
; * cmd   : McIDAS command line (if stored)
; 
; OPTIONS
; * NOREAD - This reads only the AREAstr and NAVstr header and skips
;            reading the IMAGES array (useful for checking out the
;            header without the I/O of reading the entire file).
;
; USES
; * define_mcidas_area_gvar.pro
; * define_mcidas_lineprefix_class.pro
; * define_mcidas_nav_goes.pro
; * define_mcidas_nav_gvar.pro
;
; USED BY
; * b1read_noa_goe.pro
; 
; MODIFICATION HISTORY
;       $Id: mcidas_read.pro 22659 2010-09-10 19:10:28Z kknapp $
;       Created 11/16/2003 - Ken Knapp NCDC
;               03/11/2004   Imported to CVS
;               03/15/2004   Added the option to return line headers
;               03/19/2004   Converted to read mcidas AREA files
;               05/27/2004   Updating for distribution.
; TO DO
;     - Add option to output line header time (converted from BCD)
;     - Read the calibration header if available.
;
; SOURCE

pro mcidas_read,file=file,      $     
                images=images,  $  
                AREAstr=AREAstr,$ 
                NAVstr=NAVstr,  $   
                LINhdr=LINhdr,  $
                cmd=cmd,        $
                NOREAD=NOREAD,  $
                quiet=quiet
iquiet=1
if keyword_set(quiet) then iquiet = 0

; Get a file name if none provided
if not(keyword_set(file)) then begin
;    file = dialog_pickfile(path='~/data/mcidas/',$
;                           filter=filters)
    file = dialog_pickfile(path='/home/kknapp/ssec/modeaaa/data/',$
                           filter=filters)
endif
if not(file_test(file)) then begin
    print,'Non-existent file: '+file
    goto,endd
endif

;get file information
finfo = file_info(file)

;AREA Structure
define_mcidas_area_gvar,AREAstr

;Navigation structure definitions
define_mcidas_nav_gvar,NAVstr
define_mcidas_nav_goes,VISRstr

;CLASS line header
define_mcidas_lineprefix_class,class_pre

;GVAR line header
gvar_pre = {block_header_crc0:0, $     ; 2 bytes
            scan_status:0l, $          ; 4 bytes
            BCD_time:lonarr(2), $      ; 8 bytes
            blockHeader:bytarr(30), $  ;30 bytes
            linedoc:bytarr(32)}        ;32 bytes

;Unknown blocks
blkheader = {bid:0b,$           
             wsize:0b,$
             wcount:0,$
             pid:0,$
             rflag:0b,$
             vernum:0b,$
             dval:0b,$
             ascii:0b,$
             spare:0b,$
             range:0b,$
             bcount:0,$
             spare2:0,$
             time:bytarr(8),$
             spare3:0l,$
             err:0}


; This line header structure HD describes the line header as defined
; in McIdas and GVAR documentation [e.g., Table 3-5 defines the
; BlockHeader structure]. However, the structure does not seem to
; match the data. It looks like there is a time code in the data, but
; not where it is supposed to be. Not sure what to do. Will leave it
; this way for now. I may learn something from other data sets, or
; maybe should contact Garrett for help.
hd = {val_code:0,$
      scnnum:0,$
      crc:bytarr(2),$
      scan_stat:0l,$
      time:bytarr(8),$
      blkhdr:blkheader,$
      other:intarr(16)}
; CHECK to see if the file is compressed
ICOMP = 0
if (strmid(file,strlen(file)-3,3) eq '.gz') then ICOMP=1

; CHECK to see if byte swapping is necessary
iswap = 0
openr,unit,file,/get_lun,swap_end=iswap,compress=ICOMP
test = lonarr(2)
readu,unit,test
close,unit
free_lun,unit
iswap = test(1) ne 4            ;use byte swap if (test(1) ne 4)

openr,unit,file,/get_lun,swap_end=iswap,compress=ICOMP

; read area header
readu,unit,AREAstr

; read navigation header
src = strtrim(areastr.src_type)
if (src eq 'GVAR' or src eq 'RAVG') then begin
    POINT_LUN, unit, AREAstr.SEC_KEY_NAV
    ReadU, unit, NavStr
endif else if (src eq 'VISR' or $
               src eq 'AAA' or $
               src eq 'VAS') then begin
    POINT_LUN, unit, AREAstr.SEC_KEY_NAV
    ReadU,unit,VISRstr
    NAVstr = VISRstr
endif else begin
    print,'Unknown Image type'
    stop
endelse

; set up image parameters
ipzero=areastr.pri_key_nav

if (ipzero eq 0) then begin
    ;this should NOT be zero
    message,'Resetting Image Offset from 0 to 3328',/info
    ipzero = 3328l
endif


nch=areastr.num_chan
nx=areastr.num_elem
ny=areastr.num_line
res=[areastr.elem_res,areastr.line_res]
prefix=areastr.num_byte_ln_prefix
bytes=areastr.bytes_per_pixel
fsize = ipzero + (nx*nch*bytes+prefix)*ny

print,'IPZero',ipzero

finfo = file_info(file)
filesize=finfo.size
if (ICOMP) then begin
    ;can't run file_info on a compressed file
    spawn,'gzip -l '+file,out,err
    n=n_elements(out) 
    temp = strsplit(out(n-1),/extr)
    filesize=long(temp(1))
endif
if (filesize lt fsize) then begin
    if (iquiet) then print,'Adjusting NY since file size doesn''t match'
    ny = floor((finfo.size - ipzero) / (nx*nch*bytes+prefix))
    if (iquiet) then print,'New NY = ',ny
endif

if keyword_set(NOREAD) then goto,skipimage
point_lun,unit,ipzero

; set up line prefix: LNHDR
pre_doc_len = areastr.len_prefix_doc
pre_cal_len = areastr.len_prefix_calib
pre_lev_len = areastr.len_prefix_lev
pre_val_len = prefix - pre_doc_len - pre_cal_len - pre_lev_len
if (pre_val_len ne 0) then begin
    if (pre_val_len eq 4) then begin
        lnhdr = {val:0l}
    endif else begin 
        stop
    endelse 
endif
if (pre_doc_len gt 0) then begin
    ;228 is the CLASS header
    if (pre_doc_len eq 228) then begin
        if (n_tags(lnhdr) eq 0) then begin
            lnhdr = {doc:class_pre}
        endif else begin
            lnhdr = create_struct(lnhdr,['doc'],class_pre)
        endelse
    endif else if (pre_doc_len eq 128) then begin
        if (n_tags(lnhdr) eq 0) then begin
            lnhdr = {doc:bytarr(128)}
        endif else begin
            lnhdr = create_struct(lnhdr,['doc'],bytarr(128))
        endelse
    endif else if (pre_doc_len eq 512) then begin
        if (n_tags(lnhdr) eq 0) then begin
            lnhdr = {doc:bytarr(128,4)}
        endif else begin
            lnhdr = create_struct(lnhdr,['doc'],bytarr(128,4))
        endelse
    endif else if (pre_doc_len eq 76)  then begin
        if (n_tags(lnhdr) eq 0) then begin
            lnhdr = {doc:gvar_pre}
        endif else begin
            lnhdr = create_struct(lnhdr,['doc'],gvar_pre)
        endelse
    endif else begin 
        if (iquiet) then MESSAGE,'Unknown line header prefix length: '+strtrim(pre_doc_len,2),/info
        if (n_tags(lnhdr) eq 0) then begin
            lnhdr = {doc:bytarr(pre_doc_len)}
        endif else begin
            lnhdr = create_struct(lnhdr,['doc'],bytarr(pre_doc_len))
        endelse
    endelse
    ;OTHER HEADER LENGTHS
    ;   76 ???  thought this was from McIDAS??
endif
if (pre_cal_len gt 0) then begin
    if (n_tags(lnhdr) eq 0) then begin
        lnhdr = {cal:bytarr(pre_cal_len)}
    endif else begin
        lnhdr = create_struct(lnhdr,['cal'],bytarr(pre_cal_len))
    endelse
endif
if (pre_lev_len gt 0) then begin
    if (n_tags(lnhdr) gt 0) then begin
        lnhdr = create_struct(lnhdr,['level'],bytarr(pre_lev_len))
    endif else begin
        lnhdr = {level:bytarr(pre_lev_len)}
    endelse
endif

;Set up the image scan: SCAN
if (bytes eq 1) then begin
    scan=bytarr(nch,nx)
endif else if (bytes eq 2) then begin
    scan = intarr(nch,nx)
endif else begin
    if (iquiet) then MESSAGE,'Wrong number of bytes: '+strtrim(bytes,2)
endelse
scan=reform(scan)

if (prefix ne 0) then begin
    input = {header:lnhdr, $
             scan:scan}
    in = replicate(input,ny)
endif else begin
    in = rebin(reform(scan,nx,1),nx,ny)
endelse

; read image and line prefix
readu,unit,in
point_lun,-unit,here


;the last bytes in the AREA file sometimes are the command that
;created the file.
;EXAMPLE: 
;cmd = '04078 213634 IMGCOPY EASTS/ALL.-1 AGRAUMAN/KNAPP.1
;                                    SIZE=600 1600 BAN=1 LAT=35 83'
if (here ne filesize) then begin
    ;print,here,fsize,finfo.size
    len = finfo.size-fsize
len=finfo.size-here
    cmd = bytarr(len)
    readu,unit,cmd
    ;cmd = string(cmd)
endif

; separate line prefix from image
if (prefix ne 0) then begin
    images = in(*).scan 
    LINhdr = in(*).header
endif else begin
    images = in
endelse

; byte shift the image for 2-byte images
if (bytes eq 2) then images = ishft(images,-5)

skipimage:
close,unit
free_lun,unit


endd:
end
;***
