;------------------------------------------------
;****I* idl/check_kep
; NAME
; check_kep
; 
; PURPOSE
; To determine if two ses of Kepler elements are similar. Many
; parameters can be used to describe the identical orbit, thus it is
; important to determine if two sets of Kepler elements describe a
; similar orbit.
; This performs as very simple comparison, basically, the test fails
; if:
; - the relative difference in Semi-major axisor eccentricity is > 1.5%
;     or
; - the angular differences in angular parameters is > 1 degree
; 
; USAGE
; INPUT
; kep1   - First set of kepler elements
; epoch1 - Epoch data of first Kepler set
; kep2   - Second set of kepler elements
; epoch2 - Epoch data of second Kepler set
; kep3   - Third set of kepler elements
; epoch3 - Epoch data of third Kepler set
;              
; OUTPUT
; 1 - Elements are similar
; 0 - Elements are not similar
; 
; EXAMPLE
; USES
; * gha.pro
; * mean.pro
; 
; USED BY
; * navcheck.pro
; 
; SOURCE

function check_kep,kep1,epoch1,kep2,epoch2,kep3,epoch3
; RETURN 1 means all keplerian elements are similar
; RETURN 0 means they are not similar and more testing is needed
k1 = kep1 & k2 = kep2 & k3 = kep3
;get greenwich hour angle
gha1 = gha(epoch1,/deg)
gha2 = gha(epoch2,/deg)
gha3 = gha(epoch3,/deg)
;correct the separate kepler elements for the different 
; epoch times
k1(5) = k1(5) - gha1
k2(5) = k2(5) - gha2
k3(5) = k3(5) - gha3

;correct angular data
for i=5,5 do begin
    if ((k1(i)-k2(i)) gt 315) then k2(i) = k2(i) + 360.
    if ((k2(i)-k1(i)) gt 315) then k2(i) = k2(i) - 360.
    if ((k2(i)-k3(i)) gt 315) then k3(i) = k3(i) + 360.
    if ((k3(i)-k2(i)) gt 315) then k3(i) = k3(i) - 360.
endfor
;compare values
for i=0,5 do begin
    diff = (abs(k1(i)-k2(i)) > abs(k1(i)-k3(i)) > abs(k2(i)-k3(i))) / 2.
    avg = mean([k1(i),k2(i),k3(i)])
    if (i lt 3 and abs(diff/avg) gt 0.015) then return,0
    if (i ge 3 and abs(diff) gt 2) then return,0
endfor
return,1
end
;***

;------------------------------------------------
;****I* idl/is_perpendicular
; NAME
; is_perpendicular
; 
; PURPOSE
; To determine if two vectors are perpendicular. In this case, two
; vecotrs are declared perpendicular if and only if:
; - angle between the vectors is within LIMIT degrees of 90 degrees.
;
; By default, LIMIT=1 degree
; 
; USAGE
; INPUT
; array1 - Vector 1
; array2 - Vector 2
; anglelimit - optional - threshold to declare two angles as
;              perpendicular (in degrees)
;              
; OUTPUT
; 1 - Vectors are perpendicular
; 0 - Vectors are not perpendicular
; 
; EXAMPLE
; USES
; * Nothing
; 
; USED BY
; * navcheck.pro
; 
; SOURCE

function is_perpendicular,array1,array2,anglelimit
if (n_elements(anglelimit) eq 1) then limit = anglelimit else limit = 1
dot = total(array1*array2)
angle = acos(dot / sqrt(total(array1^2)) / sqrt(total(array2^2))) * 180./!pi
return,abs(angle-90) lt limit
end
;***
;------------------------------------------------
;****I* idl/is_similar
; NAME
; is_similar
; 
; PURPOSE
; To determine if two vectors are similar. In this case the tests pass
; if and only if:
; - angle between the vectors < 1 degree
; - difference in magnitude between the vectors is less than 1%
; 
; USAGE
; INPUT
; array1 - Vector 1
; array2 - Vector 2
; 
; OUTPUT
; 1 - Vectors are similar
; 0 - Vectors are not similar
; 
; EXAMPLE
; USES
; * Nothing
; 
; USED BY
; * navcheck.pro
; 
; SOURCE

function is_similar,array1,array2,anglelimit,percentlimit
if (n_elements(anglelimit) eq 1) then limita = anglelimit else limita = 1
if (n_elements(percentlimit) eq 1) then limitp = percentlimit else limitp = 1

dot = total(array1*array2)
mag1 = sqrt(total(array1^2))
mag2 = sqrt(total(array2^2))
angle = acos(dot / mag1 / mag2 ) * 180./!pi
diff = array1-array2
magd = sqrt(total(diff^2))
;print,angle,(100.*(magd/(mag1>mag2)))
return,abs(angle) lt 1 and (100.*(magd/(mag1>mag2)) ) lt 1
end
;***

;------------------------------------------------
;****I* idl/is_geostationary
; NAME
; is_geostationary
; 
; PURPOSE
; To determine if a set of kepler parameters describe a geostationary orbit.
; An orbit is declared geostationary if its range in longitude is less
; than 3 degrees.
; 
; USAGE
; INPUT
; kep - kepler parameter array
; 
; OUTPUT
; 0 - not a geostationary orbit
; 1 - is a geostionary orbit
; 
; EXAMPLE
; USES
; * kep2cart.pro
; * cart2earth.pro
; 
; USED BY
; * navcheck.pro
; 
; SOURCE
function is_geostationary,kep
dtime = findgen(24*3)*3600.
kep2cart,kep,dtime,x,y,z
cart2earth,x,y,z,julday(1,1,1990,0,0,dtime),lat,lon,radius
minlon = min(lon,max=maxlon)
if ((maxlon-minlon) gt 180) then begin
    lon = lon + 360.
    minlon = min(lon,max=maxlon)
endif
return,(maxlon-minlon) lt 3
end
;***

;------------------------------------------------
;****I* idl/idl.navcheck
; NAME:
;    NAVCHECK
;
; PURPOSE:
;    To check the navigation parameters provided in file headers for
;    errors or invalid values
;
; CATEGORY:
;    ISCCP
;    Navigation
;
; INPUTS:
;    pdate = [date1,time1]
;    date1 - Epoch date for Chebyshev parameters (YYJJJ)
;    time1 - Epoch time for Chebyshev parameters (sec * 100)
;    pos1 = [X1,Y1,Z1]    Satellite position at time1 (km)
;    vel1 = [VX1,VY1,VZ1] Satellite velocity at time1 (km/hour)
;    pos2 = [X2,Y2,Z2]    Satellite position at time1+13 hours (km)
;    vel2 = [VX2,VY2,VZ2] Satellite velocity at time1+13 hours (km/hour)
;    CX    - Chebyshev polynomials for X
;    CY    - Chebyshev polynomials for Y
;    CZ    - Chebyshev polynomials for Z
;    kdate = [epy,eph]
;    EPY   - Epoch date for Keplerian elements (YYMMDD)
;    EPH   - Epoch time for Keplerian elements (HHMMSS)
;    kep = [SMA,ECC,INC,RAN,AP,MA]
;    SMA   - semi-major axis (km)
;    ECC   - Eccentricity
;    INC   - Inclination (deg)
;    RAN   - Right ascension of ascending node (deg)
;    AP    - Argument of perigee (deg)
;    MA    - Mean Anomaly (deg)
;
; KEYWORD PARAMETERS:
; kepout  - The "best" kepler output of the input data
; kepdate - The "best" epoch date of the input data
;
;
; OUTPUTS:
;    RESULT - This is the result of the testing
;
; USES
; * cart2earth.pro
; * cart2kep.pro
; * cheb2cart.pro
; * gha.pro
; * itime.pro
; * kep2cart.pro
; * mean.pro
; * yyjjj_2_julday.pro
; * yymmdd_2_yyjjj.pro
; 
; USED BY
; * b1read_aes_g7.pro
; * b1read_noa_goe.pro
; * b1read_uws_g56.pro
; 
; MODIFICATION HISTORY:
;    $Id: navcheck.pro 22778 2010-09-16 12:36:36Z kknapp $
;
; SOURCE


pro navcheck,pdate,pos1,vel1,pos2,vel2,cx,cy,cz,kdate,kep,result, $
             kepout = kepout, $ ;use this kepler info
             kepdate = kepdate  ;use this kepler epoch date

okp1 = 1
if (total(pos1) eq 0) then okp1 = 0
if (total(vel1) eq 0) then okp1 = 0
okp2 = 1
if (total(pos2) eq 0) then okp2 = 0
if (total(vel2) eq 0) then okp2 = 0
okcheb = 1
if (total(cx) eq 0) then okcheb = 0
if (total(cy) eq 0) then okcheb = 0
if (total(cz) eq 0) then okcheb = 0
okkep = 1
if (total(kep) eq 0 or total(kdate) eq 0) then okkep = 0
if (okp1 eq 0 and okp2 eq 0 and okcheb eq 0 and okkep eq 0) then begin
    result = [-1]
    message,'Failed initial test',/info
    return
endif


;SET UP VARIABLES
x1 = pos1(0)
y1 = pos1(1)
z1 = pos1(2)
x2 = pos2(0)
y2 = pos2(1)
z2 = pos2(2)

vx1 = vel1(0)
vy1 = vel1(1)
vz1 = vel1(2)
vx2 = vel2(0)
vy2 = vel2(1)
vz2 = vel2(2)

;reset test values
tp1 = 0
tp2 = 0
tgk = 0
tgk1 = 0
tgk2 = 0
ts1  = 0 & ts2  = 0
tsc1 = 0 & tsc2 = 0
tsk2 = 0 & tsk1 = 0
tck1 = 0 & tck2 = 0
tsc1 = 0
tsc2 = 0


;begin tests
;Are positions perpendicular to velocities??
if (okp1) then tp1 = is_perpendicular(pos1,vel1)
if (okp2) then tp2 = is_perpendicular(pos2,vel2)

;Is kep geostationary?
if (okkep) then tgk = is_geostationary(kep)

;Are pos1,vel1 geostationary?
if (tp1) then begin
    cart2kep,pos1,vel1,kep1
    tgk1 = is_geostationary(kep1)
endif
;Are pos2,vel2 geostationary?
if (tp2) then begin
    cart2kep,pos2,vel2,kep2
    tgk2 = is_geostationary(kep2)
endif

kdateflag=1
pdateflag=1
if (pdate(0) lt 0 or $
    pdate(0) gt 200000l $
   ) then begin
    pdateflag=0
endif 
if (kdate(0) lt 0 or $
    kdate(0) gt 2000000l $
   ) then begin
    kdateflag=0
endif 

if (kdateflag) then $
  ejulday = yyjjj_2_julday(yymmdd_2_yyjjj(kdate(0)),time=kdate(1),/itime)
if (pdateflag) then $
  pjulday = yyjjj_2_julday(pdate(0),time=pdate(1)/100.,/seconds)

;chevk to see how similar the kepler parameters are ... if nearly
;identical, then keep kep
if (tp1 and tp2 and tgk and $
    kdateflag and pdateflag) then begin
    
    testit = check_kep(kep,ejulday, $
                       kep1,pjulday, $
                       kep2,pjulday + 13.d0/24.d0)
    if (testit) then begin
        result = [2,1]
        kepout = kep
        kepdate = ejulday
    endif
endif

if (okcheb) then begin
    c = dblarr(3)
;Is pos1 similar to that derived from Chebyshev polynomials?
    cheb2cart,cx,0,cx1
    cheb2cart,cy,0,cy1
    cheb2cart,cz,0,cz1
    tsc1 = is_similar(pos1,[cx1,cy1,cz1])
;Is pos2 similar to that derived from Chebyshev polynomials?
    cheb2cart,cx,13.*3600.,cx2
    cheb2cart,cy,13.*3600.,cy2
    cheb2cart,cz,13.*3600.,cz2
    tsc2 = is_similar(pos2,[cx2,cy2,cz2])
endif
;check positions from kep
if (tgk and pdateflag and kdateflag) then begin
    dtime = (pjulday-ejulday)*24.d0*3600.d0 + [0,13.d0]*3600.
    kep2cart,kep,dtime,xk,yk,zk
    cart2earth,xk,yk,zk,[pjulday,pjulday+13.d-/24.d0],glat,glon,grad
    ts1 = is_similar(pos1,[xk(0),yk(0),zk(0)])
    ts2 = is_similar(pos2,[xk(1),yk(1),zk(1)])
    if (okcheb) then tck1 = is_similar([cx1,cy1,cz1],[xk(0),yk(0),zk(0)])
    if (okcheb) then tck2 = is_similar([cx2,cy2,cz2],[xk(1),yk(1),zk(1)])
endif
;Is position 2 similar to that derived from pos1/vel1
if (tgk1 and kdateflag) then begin
    kep2cart,kep1,13.d0*3600.d0,x,y,z
    tsk1 = is_similar(pos2,[x,y,z])
endif
;Is position 1 similar to that derived from pos2/vel2
if (tgk2 and kdateflag) then begin
    kep2cart,kep2,-13.d0*3600.d0,x,y,z
    tsk2 = is_similar(pos1,[x,y,z])
endif

tests = [tp1,tp2,tgk,tgk1,tgk2,ts1,ts2,tsk1,tsk2,tsc1,tsc2,tck1,tck2]
tot = total(tests)
;print,byte(tests)


; RESULT returns which navigation elements to "trust"
;   0 = Use position 1 (i.e., convert pos1/vel1 to kepler and use it)
;   1 = Use position 2 (i.e., convert pos2/vel2 to kepler and use it)
;   2 = Kepler elements
;default
if (kdateflag) then kepdate = ejulday else begin
    message,'Bad Date',/info
    goto,nodate
endelse

;All tests pass
if (tot eq 13) then begin       
    result = [2,tests]          ;USE Keplerian elements
    kepout = kep
    return
endif
;Only Cheb are bad
if (total(tests(0:8)) eq 9) then begin 
    result = [2,tests]          ;Use Keplerian elements
    kepout = kep
    return
endif
;Only Pos1/Vel1 are bad
if (tot eq 7 and tp2 and tgk and tgk2 and ts2 and tsc2 and tck1 and tck2) then begin
    result = [2,tests]
    kepout = kep
    return
endif
;Only Pos2/Vel2 are bad
if (tot eq 7 and tp1 and tgk and tgk1 and ts1 and tsc1 and tck1 and tck2) then begin
    result = [2,tests]
    kepout = kep
    return
endif
;Only kepler is bad
if (tot eq 8 and tp1 and tp2 and tgk1 and tgk2 and tsk1 and tsk2 and tsc1 and tsc2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif
if (tot eq 9 and tp1 and tp2 and tgk and tgk1 and tgk2 and tsk1 and tsk2 and tsc1 and tsc2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif
;Pos1 and Pos2 are bad
if (tot eq 3 and tgk and tck1 and tck2) then begin
    result = [2,tests]
    kepout = kep
    return
endif
;pos1 and kepler are bad
if (tot eq 3 and tp2 and tgk2 and tsc2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif
;pos1 and cheb are bad
if (tot eq 4 and tp2 and tgk and tgk2 and ts2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif
;P2 and Kepler are bad
if (tot eq 3 and tp1 and tgk1 and tsc1) then begin
    result = [0,tests]
    kepout = kep1
    kepdate = pjulday
    return
endif
;P2 and Cheb are bad
if (tot eq 4 and tp1 and tgk and tgk1 and ts1) then begin
    result = [2,tests]
    kepout = kep
    return
endif
;kep and cheb are bad
if (tot eq 6 and tp1 and tp2 and tgk1 and tgk2 and tsk1 and tsk2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif
;ONLY Kepler is good
if (tot eq 1 and tgk) then begin
    result = [2,tests]
    kepout = kep
    return
endif
;only P2 is good
if (tot eq 2 and tp2 and tgk2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif
;only P1 is good
if (tot eq 2 and tp1 and tgk1)then begin
    result = [0,tests]
    kepout = kep1
    kepdate = pjulday
    return
endif

if (tp1 and tp2 and tgk1 and tgk2 and tsk1 and tsk2) then begin
    result = [1,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif

if (tp2 and tgk2 and ts2 and tsc2 and tck2) then begin
    result = [2,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif

if (tp2 and tgk2 and tsc2) then begin
    result = [2,tests]
    kepout = kep2
    kepdate = pjulday + 13.d0/24.d0
    return
endif

if (tgk) then begin
    result = [2,tests]
    kepout = kep
    return
endif

if (total(tests) eq 0) then begin
    result = [-1]
    message,'Failed all tests',/info
    return
endif

if (tp1 and tgk1 and ts1 and tsc1) then begin
    result = [0,tests]
    kepout = kep1
    kepdate = pjulday
    return
endif


print,'Error: Unknown test conditions.'
print,'Tests:',tests
print,'Total:',tot

nodate:
result = [-1,tests]
return
stop
end
;***
