;#################################################################
;****I* idl/idl.szacorr
;+
; NAME:
;    szacorr
;
; PURPOSE:
;    This routine corrects geostationary brightness temperatures for
;    view zenith angle. The correction depends on time of year,
;    latitude and brightness temperature.
;
;    The routine is based on the fortran routine developed by Bob
;    Joyce (NOAA/NCEP). The IDL routine has been tested, and returns
;    the exact same corrections that the fortran routine returns.
;
;    The code was converted directly from FORTRAN code.
;
; INPUTS:
;    tb    - Brightness temperature (K)  [array]
;    xlat  - Latitude (degrees)          [array]
;    sza   - VIEW zenith angle (degrees) [array]
;    xjday - Day of year (1-366)         [scalar]
;
; KEYWORD PARAMETERS:
;    None as of yet.
;
; OUTPUTS:
;    Returns Corrrected Brightness Temp (K) [array]
;    
; EXAMPLE:
;    Btempc = szacorr(Btemp,lat,vza,jday)
;        This returns the corrected values for all Btemp values
;
; USES
;    The correction information is located in two data files currently
;    stored in getenv('DATDIR')+'/szacorr/'
;
; USED BY
; * b1u_2_grisat.pro
; 
; MODIFICATION HISTORY:
;    $Id: szacorr.pro 22669 2010-09-10 19:35:10Z kknapp $
;    Created 3/18/2004   Ken Knapp
;
; SOURCE

function szacorr,tb,xlat,sza,xjday
; The following is from SZAMODELS
;
;c     programmer:      Bob Joyce
;c     date created:    3/2/2000
;c     purpose:         Subroutine read zenith angle corrections models
;c
;c    For itype enter "0" if using PC or LINUX, "1" if SGI or HP

; This algorithm attempts to receive input in array form:
;   ARRAYS:
;           tb
;           sza
;           xlat
;   SCALAR:
;           xjday   [images will be from same day in most cases]
;   OUTPUT: 
;           TBCORR or SZACORRARR [as an array]

J5MAX=240
ZABINMX=64
TMPBINMX=170
ISEASMX=8
difflat =fltarr(J5MAX,ISEASMX)
error   =fltarr(ZABINMX,TMPBINMX)
errorlat=fltarr(J5MAX,ISEASMX)

; Read in IR Correction Table
dir = getenv('DATDIR')+'/szacorr/'
file = 'error_regrid'
openr,1,dir+file ;,/swap_end
readu,1,error
close,1
;contour,-error,vza,temp,lev=findgen(13)*3,xtitle='VZA (deg)',ytitle='Temp(K)',/foll

; Read in latitudinal Adjustment
file = 'latitudinal-error_annual-cycle-model'
in = fltarr(j5max)
openr,1,dir+file,/swap_end
for iseas=0,iseasmx-1 do begin
    readu,1,in
    errorlat(*,iseas) = in
    readu,1,in
    difflat(*,iseas) = in
endfor
close,1
; END SZAMODELS EEEEEEEEEEEEEEEENNNNNNNNNNNNNNNNDDDDDDDDDDDDDDDDDDDD

; The following is from SZACORR
;c     programmer:      Bob Joyce
;c     date created:    3/2/2000
;c     purpose:         Subroutine takes the following inputs
;c          1. uncorrected Tb
;c          2. satellite zenith angle
;c          3. Julian day
;c          4. latitude of IR Tb pixel
;c
;c                      Subroutine outputs
;c          1. Seasonal, Latitudinal, Tb dependent
;c                            Zenith angle IR Tb correction
; PARAMETER statement
ZAMAX=87.0
ISTYR=1999
J5MAX=240
XMISSING=-9999
ZABINMX=64
TMPBINMX=170
SZAANGMN=26.
SZATMPMN=160.
YEARDYS=365
ISEASMX=8
ZERO=0
ONE=1
XLATMX=60
XLATMN=-60
; DATA statement
seasjday= [57.5,104.5,148.5,206.5,268.5,326.5,395,422.5]
XLATBININT=0.5

;c     add yearly day total if after feb 26 (when model began)
if (xjday lt seasjday(ONE-1)) then xjday = xjday + YEARDYS

;c     Find latitude bin for target
;if (xlat gt XLATMX) then begin
;    j5x = ONE 
;endif else if (xlat gt XLATMN and xlat lt XLATMX) then begin
;    j5x = (XLATMX - xlat)/XLATBININT + ONE
;endif else begin
;    j5x = J5MAX
;endelse
j5x = fix(xlat)*0 +j5max
a = where(xlat ge XLATMX,na)
if (na gt 0) then j5x(a) = one
a = where(xlat gt XLATMN and xlat lt XLATMX,na)
if (na gt 0) then j5x(a) = (XLATMX - xlat(a))/XLATBININT + ONE



; THE FOLLOWING IS A TEMPORAL CHECK< THUS IS NOT CHANGED, BECAUSE JDAY
; IS NOT AN ARRAY
for icday = 1, ISEASMX-1 do begin
;c   Check to see if current day is exactly seasonal model
    if (xjday eq seasjday(icday-1)) then begin
        xerrorlat = errorlat(j5x-1,icday-1)
        goto,fortyseven         ; 47
    endif else begin
        if (xjday gt seasjday(icday-1) and $
            xjday lt seasjday(icday+1-1))then begin
;c          Find day difference from current day and early and late
;           seasonal  tables
            diff1 = xjday - seasjday(icday-1)
            diff2 = seasjday(icday+1-1) - xjday
            xlnth = seasjday(icday+1-1) - seasjday(icday-1)
            w1    = diff2/xlnth
            w2    = diff1/xlnth
            xerrorlat = errorlat(j5x-1,icday-1)*w1 + $
              errorlat(j5x-1,icday+1-1)*w2
            goto,fortyseven     ;47
        endif
    endelse
endfor                          ; 35   continue

; 47   continue
fortyseven:
;c     Add zenith angle correction to IR
;iz = (sza-SZAANGMN) + 1
;it = (Tb-SZATMPMN) + 1
iz = fix(sza-szaangmn) + 1
it = fix(tb -szatmpmn) + 1

tbcorr = tb
a = where(iz gt 1,na)
if (na gt 0) then begin
    iz = iz < ZABINMX
    it = it > 1 < TMPBINMX
    Tbcorr(a) = Tb(a) - xerrorlat*error(iz(a)-1,it(a)-1)
endif
;if (iz ge 1)then begin
;    if (iz gt ZABINMX)then iz=ZABINMX
;    if (it lt 1)then it=1
;    if (it gt TMPBINMX)then it=TMPBINMX
;    Tbcorr = Tb - xerrorlat*error(iz-1,it-1)
;endif  else begin
;    Tbcorr = Tb
;endelse
;c     if sza > 87 of if Tb is non zero set to missing
;if (sza gt ZAMAX or $
;    sza lt ZERO or $
;    Tb  lt ZERO) then begin
;    Tb     = XMISSING
;    Tbcorr = XMISSING
;endif

a = where(sza gt ZAMAX or $
    sza lt ZERO or $
    Tb  lt ZERO,na)
if (na gt 0) then tbcorr(a) = xmissing

; END SZACORR   EEEEEEEEEEEEEEEENNNNNNNNNNNNNNNNDDDDDDDDDDDDDDDDDDDD


return,tbcorr

end

;***
