;****I* idl/idl.time50
; NAME
; time50
; 
; PURPOSE
;	Function Time50 will take the epoch time from the GVAR NAVstr and
;       convert it to minutes from January 1, 1950.
; 
; USAGE
; INPUT
; Epoch_time : the (modified) binary coded data time from GVAR
; 
; OUTPUT
; TU : Minutes from January 1, 1950
; 
; RESULT
; EXAMPLE
;	TU = Time50(NAVstr.epoch_time)
;	
; USES
; * No non-IDL routines
; 
; USED BY
; * b1u_ij2ll.pro
; 
; NOTES
; TODO
; BUGS
; AUTHOR
; Written by:	Kelly Dean, September 1994
; Ken Knapp, NOAA/NCDC/RSAD
; 
; CREATION DATE
; 
; HISTORY
; Jul2010 - KRK - Adjusted header info for robodoc
; 
; SOURCE

FUNCTION Time50, Epoch_time
NY = 0L ;YEAR
ND = 0L ;DAY OF YEAR
NH = 0L ;HOUR
NM = 0L ;MINUTE
S  = DOUBLE(0) ;SECONDS - Double precision
J  = 0L
word = Epoch_time ; Move 8-bytes to long word
;
; Extract the Binary Coded Time into separate year, Julian day, hour
; minutes, seconds. NOTE - Epoch time is not in the same format as
; other BCD times.
;
year_100  = word(3) AND 'F'XB             ; mask off bottom 4 bits
year_1000 = ISHFT( word(3), -4 )          ; shift top 4 bits down
year_1    = word(2) AND 'F'XB
year_10   = ISHFT( word(2), -4 )
day_10    = word(1) AND 'F'XB
day_100   = ISHFT( word(1), -4)
time_code = ISHFT( word(1), -1 ) AND '7'XB  ; shift bits down one, and mask off 3
hour_10   = word(0) AND 'F'XB
day_1     = ISHFT( word(0), -4 )
min_10    = word(7) AND 'F'XB
hour_1    = ISHFT( word(7), -4 )
sec_10    = word(6) AND 'F'XB
min_1     = ISHFT( word(6), -4 )
msec_100  = word(5) AND 'F'XB
sec_1     = ISHFT( word(5), -4 )
msec_1    = word(4) AND 'F'XB
msec_10   = ISHFT( word(4), -4 )
;
; Construct year, Julian day, hour, and minutes.
;
ny = ( 1000L * year_1000 ) + ( 100L * year_100 ) + ( 10L * year_10 ) + year_1
nd = ( 100L * day_100 ) + ( 10L * day_10 ) + day_1
nh = ( 10L * hour_10 ) + hour_1
nm = ( 10L * min_10 ) + min_1
s = DOUBLE( (sec_10 * 10.0 ) + sec_1 + ( msec_100 * 0.1 ) + ( msec_10 * 0.01 ) + ( msec_1 * 0.001) )
;
; Here the INTEGER year and day of year to number of days from 0000Z,
; 1950 January 1. This conversion is based on an algorithm by Fliegel
; and van Flandern COMM. OF ACM, VOL.11, NO. 10, OCT. 1968 (P.657)
;
J = ND + ( ( 1461L * ( NY + 4799L ) ) / 4L ) - ( 3L * ( ( NY + 4899L ) / 100L ) / 4L ) - 2465022L
;
; Compute time in minutes from January 1, 1950
;
T50 = ( J * DOUBLE(1440) ) + ( NH * DOUBLE(60) )  + NM + ( S / DOUBLE(60) )
;
RETURN, T50
END

;***
